package ca.ualberta.proteus.db;

import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import com.db4o.ObjectContainer;
import com.db4o.ObjectSet;
import com.db4o.query.Predicate;
import com.db4o.internal.freespace.FreespaceManagerIx;
import com.db4o.ext.DatabaseFileLockedException;
import com.db4o.config.ObjectClass;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: May 10, 2007
 * Time: 9:04:24 PM
 *
 * An implementation-neutral persistence queue
 *
 * Persistent queuing using the DB4O database
 */
public class DB4OPersistenceQueue implements AbstractPersistenceQueue
{
    private Logger logger = Logger.getLogger(DB4OPersistenceQueue.class);
    
    // private static boolean enabled = false;
    // private static final ObjectContainer db;

    static
    {
     /*   db = DB4OPool.getDB();
        if ( db != null )
        {
            // When we delete settings, cascade
            ObjectClass aClass = db.ext().configure().objectClass(ProteusSettings.class);
            aClass.cascadeOnDelete(true);

            enabled = true;
        }*/
       // enabled = true;
    }

    /**
     * Recovers a list of proteus runs after a crash
     *
     * @return the queue
     */
    public List<ProteusImpl> recover()
    {
        ObjectContainer db = null;
        try
        {
            db = DB4OPool.getDB();
        }
        catch (DatabaseFileLockedException e)
        {
            logger.debug(e.getMessage());
        }

        if (db != null)
        {
            try
            {
                ObjectSet<ProteusSettings> settings = db.get(ProteusSettings.class);

                List<ProteusImpl> rv = new ArrayList<ProteusImpl>();
                for ( ProteusSettings setting  : settings)
                {
                    rv.add(new ProteusImpl(setting));
                }

                return rv;
            }
            finally
            {
                db.close();
            }
        }

        return null;
    }

    /**
     * Adds a proteus thread to the persistent queue
     *
     * @param proteus
     */
    public void addToQueue(ProteusImpl proteus)
    {
        // if (enabled)
        {
            ObjectContainer db = null;
            while (db == null)
            {
                try
                {
                    db = DB4OPool.getDB();
                }
                catch (DatabaseFileLockedException e)
                {
                    logger.debug(e.getMessage());

                    try
                    {
                        logger.debug("Waiting for DB4O...");
                        Thread.sleep(1000);
                    }
                    catch (InterruptedException e1)
                    {
                        e1.printStackTrace();
                    }
                }
            }

            try
            {
                db.set(proteus.getSettings());
                db.commit();
            }
            finally
            {
                db.close();
            }

        }
    }

    /**
     * Adds a proteus thread to the persistent queue
     *
     * @param proteusList
     */
    public void addToQueue(List<ProteusImpl> proteusList)
    {
        // if (enabled)
        {
            ObjectContainer db = null;
            while (db == null)
            {
                try
                {
                    db = DB4OPool.getDB();
                }
                catch (DatabaseFileLockedException e)
                {
                    logger.debug(e.getMessage());

                    try
                    {
                        logger.debug("Waiting for DB4O...");
                        Thread.sleep(1000);
                    }
                    catch (InterruptedException e1)
                    {
                        e1.printStackTrace();
                    }
                }
            }

            try
            {
                for (ProteusImpl proteus : proteusList )
                {
                    db.set(proteus.getSettings());
                }
                db.commit();
            }
            finally
            {
                db.close();
            }

        }
    }

    /**
     * Remove a proteus from the queue
     *
     * @param proteus
     */
    public void remove(ProteusImpl proteus)
    {
       // if (enabled)
        {
            ObjectContainer db = null;
            while (db == null)
            {
                try
                {
                    db = DB4OPool.getDB();
                }
                catch (DatabaseFileLockedException e)
                {
                    logger.debug(e.getMessage());

                      try
                        {
                            logger.debug("Waiting for DB4O...");
                            Thread.sleep(1000);
                        }
                        catch (InterruptedException e1)
                        {
                            e1.printStackTrace();
                        }
                }
            }

            try
            {
                // We have to first get a reference to the object we want to delete
                ProteusSettings template = new ProteusSettings();
                final String queryId = proteus.getSettings().getID();
                template.setID(queryId);

                ObjectSet<ProteusSettings> objects = db.query(new Predicate<ProteusSettings>()
                {
                    public boolean match(ProteusSettings candidate)
                    {
                        return candidate.getID().equals(queryId);
                    }
                });
                
                assert objects.size() >= 1 : "Object isn't available to remove";

                Object settingsToDelete = objects.get(0);

                db.delete(settingsToDelete);
                db.commit();
            }
            finally
            {
                db.close();
            }

        }
    }

    /**
     * Clear the queue
     */
    public void clear()
    {
        ObjectContainer db = null;
        while (db == null)
        {
            try
            {
                db = DB4OPool.getDB();
            }
            catch (DatabaseFileLockedException e)
            {
                logger.debug(e.getMessage());

                  try
                    {
                        logger.debug("Waiting for DB4O...");
                        Thread.sleep(1000);
                    }
                    catch (InterruptedException e1)
                    {
                        e1.printStackTrace();
                    }
            }
        }

        if (db != null)
        {
            try
            {
                ObjectSet<ProteusSettings> settings = db.get(ProteusSettings.class);

                List<ProteusImpl> rv = new ArrayList<ProteusImpl>();
                for ( ProteusSettings setting  : settings)
                {
                    db.delete(setting);
                }
            }
            finally
            {
                db.close();
            }
        }   
    }

    /**
     * Close the database
     * 
     * @throws Throwable
     */
    protected void finalize() throws Throwable
    {
        super.finalize();
      //  db.close();
    }
}
