package ca.ualberta.proteus.db;

import ca.ualberta.proteus.db.model.ProteusSettingsEntity;
import ca.ualberta.proteus.db.model.ProteusSettingsIntf;
import ca.ualberta.proteus.program.ProteusImpl;
import net.java.ao.EntityManager;
import org.apache.log4j.Logger;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: May 10, 2007
 * Time: 9:04:24 PM
 * <p/>
 * An implementation-neutral persistence queue
 * <p/>
 * Persistent queuing using the DB4O database
 */
public class MySQLPersistenceQueue implements AbstractPersistenceQueue
{
    private Logger logger = Logger.getLogger(MySQLPersistenceQueue.class);

    /**
     * Recovers a list of proteus runs after a crash
     *
     * @return the queue
     */
    public List<ProteusImpl> recover()
    {
        try
        {
            ProteusSettingsIntf[] settings = SQLManager.getEntityManager().find(ProteusSettingsEntity.class);

            List<ProteusImpl> rv = new ArrayList<ProteusImpl>();
            for (ProteusSettingsIntf setting : settings)
            {
                rv.add(new ProteusImpl(setting));
            }

            return rv;

        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Adds a proteus thread to the persistent queue
     *
     * @param proteus
     */
    public void addToQueue(ProteusImpl proteus)
    {
        ProteusSettingsIntf proteusSettings = proteus.getSettings();

        ProteusSettingsEntity settings = SQLManager.getEntityManager().create(ProteusSettingsEntity.class, proteusSettings);

        // Replace the proteus settings with the persisted settings so we can remove it later
        proteus.setSettings(settings);
    }

    /**
     * Adds a proteus thread to the persistent queue
     *
     * @param proteusList
     */
    public void addToQueue(List<ProteusImpl> proteusList)
    {
        for (ProteusImpl p : proteusList)
        {
            addToQueue(p);
        }
    }

    /**
     * Remove a proteus from the queue
     *
     * @param proteus
     */
    public void remove(ProteusImpl proteus)
    {
        try
        {
            EntityManager manager = SQLManager.getEntityManager();
            ProteusSettingsEntity settings = manager.get(ProteusSettingsEntity.class, proteus.getSettings().getID());

            // Convert back to ProteusSettingsIntf
            manager.delete(settings);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Clear the queue
     */
    public void clear()
    {
        try
        {
            EntityManager manager = SQLManager.getEntityManager();
            ProteusSettingsEntity[] entities = manager.find(ProteusSettingsEntity.class);
            manager.delete(entities);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
    }
}