package ca.ualberta.proteus.db;

import ca.ualberta.proteus.db.model.ProteusSettingsEntity;
import ca.ualberta.proteus.db.model.Record;
import ca.ualberta.proteus.db.model.WebHitEntity;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.SimpleSequence;
import net.java.ao.schema.PluralizedNameConverter;
import net.java.ao.types.DatabaseType;
import net.java.ao.types.TypeManager;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.File;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 14, 2009
 * Time: 12:30:23 AM
 * To change this template use File | Settings | File Templates.
 */
public class SQLManager
{
    static EntityManager manager;

    // If you ever need more ActiveRecord classes, add them here
    static final Class[] classes = {Record.class, WebHitEntity.class, ProteusSettingsEntity.class};                    

    public static EntityManager getEntityManager()
    {
        if (manager == null)
        {
            String url = "jdbc:mysql://localhost/proteus";
            String user = "proteus";
            String pass = "proteus";

            Config config = Config.getConfig();
            if (config.contains("datasource.url"))
            {
                url = config.getStringValue("datasource.url");
                user = config.getStringValue("datasource.username");
                pass = config.getStringValue("datasource.password");
            }

            System.out.printf("Connecting to %s with user: %s pass: %s\n", url, user, pass);

            manager = new EntityManager(url, user, pass);
            manager.setTableNameConverter(new PluralizedNameConverter());

            addCustomTypes();
        }

        return manager;

    }

    private static void addCustomTypes()
    {

        TypeManager typeManager = TypeManager.getInstance();
        typeManager.addType(new DatabaseType<File>(Types.VARCHAR, 1024, File.class)
        {
            public String getDefaultName()
            {
                return "VARCHAR";
            }

            public File pullFromDatabase(net.java.ao.EntityManager manager, ResultSet res, Class<? extends File> type, String field) throws SQLException
            {
                return new File(res.getString(field));
            }

            public Object defaultParseValue(String value)
            {
                return new File(value);
            }
        });

        typeManager.addType(new DatabaseType<Sequence>(Types.VARCHAR, 30000, Sequence.class, SimpleSequence.class)
        {
            public String getDefaultName()
            {
                return "VARCHAR";
            }

            public Sequence pullFromDatabase(net.java.ao.EntityManager manager, ResultSet res, Class<? extends Sequence> type, String field) throws SQLException
            {
                String value = res.getString(field);
                return defaultParseValue(value);
            }

            @Override
            public void putToDatabase(int index, PreparedStatement stmt, Sequence value) throws SQLException
            {
                stmt.setString(index, String.format("%s//%s", value.getName(), value.seqString()));
            }

            public Sequence defaultParseValue(String value)
            {
                try
                {
                    String[] strings = value.split("//");
                    return new SimpleSequence(ProteinTools.createProtein(strings[1]), "", strings[0], null);
                }
                catch (IllegalSymbolException e)
                {
                    e.printStackTrace();
                }
                return null;
            }
        });
    }

    public static void handleSQLException(SQLException e)
    {
        if (e.getMessage().matches("Table.*doesn't exist"))
        {
            try
            {
                System.out.println("Database appears out of date.  Migrating.... try agian.");
                migrate();
            }
            catch (SQLException e1)
            {
                e1.printStackTrace();
            }
        }
    }

    public static void migrate()
            throws SQLException
    {
        SQLManager.getEntityManager().migrate(classes);
    }


}
