package ca.ualberta.proteus.db;

import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.util.XAlignRunner;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.utils.ChangeVetoException;

import java.util.ArrayList;
import java.util.Map;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Aug 9, 2005
 * Time: 7:56:20 PM
 * To change this template use File | Settings | File Templates.
 */
public class SeqSeeDBUpdater
{
    final int SEQUNCE_IDENTITY_THRESHOLD = 95;

    StructureDB FullDB, NewDB;

    public SeqSeeDBUpdater(String fullDBFileName, String outputFileName) {
        try
        {
            FullDB = new StructureDB(fullDBFileName, StructureDB.MODE.READ);
        }
        catch (IOException e)
        {
            e.printStackTrace();  
        }
        NewDB = FullDB.clone();
    }

    public void removeRedundantSequences() throws BioException, ChangeVetoException
    {
        for ( int i=0; i<FullDB.size(); i++) {
            Sequence sequence = FullDB.getSequence(i);

            if ( NewDB.containsSequence(sequence.getName())) {
                for ( int j = 0; j<NewDB.size(); j++)
                {
                    Sequence sequence2;
                    try {
                        sequence2 = NewDB.getSequence(j);
                    }
                    catch ( IllegalIDException e) {
                        continue;
                    }

                    if ( sequence.equals(sequence2))
                        continue;

                    ArrayList<Sequence> sequenceList = new ArrayList<Sequence>();
                    sequenceList.add(sequence);
                    sequenceList.add(sequence2);

                    if ( sequence2.length() < 3 ) {
                        removeSequence(sequence2, "Removed " + sequence2.getName() + " has less than 3 residues");
                      //  j--;
                        continue;
                    }

                    XAlignRunner runner = new XAlignRunner(sequence, sequenceList);
                    runner.setAddToDB(false);
                    runner.run();
                    Map<String, Map<String, Double>> ppshMap;
                    try {
                        ppshMap = runner.getPPSHMap();
                    }
                    catch (IllegalStateException e) {
                        continue;
                    }

                    if ( ppshMap == null )
                        continue;

                    Double percentIdentity = ppshMap.get(sequence2.getName()).get(sequence.getName());

                    if ( percentIdentity > SEQUNCE_IDENTITY_THRESHOLD) {
                        removeSequence(sequence2, "Removed " + sequence2.getName() + " " + percentIdentity + " similar to " + sequence.getName());
                      //  j--;
                    }
                }
            }
        }
    }

    private SequenceIterator removeSequence(Sequence sequence, String message) throws BioException, ChangeVetoException {
        SequenceIterator newDBIterator;
        NewDB.removeSequence(sequence.getName());
        newDBIterator = NewDB.sequenceIterator();
        System.out.println(message);
        return newDBIterator;
    }

    public static void main(String[] args) throws BioException, ChangeVetoException
    {
        SeqSeeDBUpdater updater = new SeqSeeDBUpdater("seqsee.db", "newSeqSee.db");
        updater.removeRedundantSequences();
    }
}
