package ca.ualberta.proteus.db.test;

import ca.ualberta.proteus.db.DB4OPool;
import ca.ualberta.proteus.db.PersistenceQueue;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.web.WebHitCounter;
import com.db4o.Db4o;
import com.db4o.ObjectContainer;
import com.db4o.config.ObjectClass;
import junit.framework.TestCase;
import org.biojava.bio.seq.db.IllegalIDException;

import java.util.Collection;
import java.util.List;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: May 10, 2007
 * Time: 9:48:22 PM
 */
public class DB4OTest extends TestCase
{
    /**
     * Fails
     * @throws IllegalIDException
     */
    public void test1() throws IllegalIDException
    {
        ProteusImpl proteus = getProteus();

        ObjectContainer db = getDB();
        db.set(proteus);
    }

    private ProteusImpl getProteus()
            throws IllegalIDException
    {
        final ProteusSettings settings = new ProteusSettings();

        settings.setMethod(MethodSettings.PSIPRED);
        settings.setRunHomodeller(false);

        settings.setTemplateID("2TRX");
        settings.setRunSecondaryStructure(false);

        // other settings
        settings.setTesting(true);
        settings.setSequence(SeqSeeDB.getTestSequence());
        settings.setID(String.valueOf(Math.random()));
        settings.setRunEnergyMinimization(false);
        settings.setOrganism(Organism.EUKARYOTIC);

        // Get the database from the config
        ProteusImpl proteus = new ProteusImpl(settings);
        return proteus;
    }

    /**
     * This should fail, Thread's cannot be stored
     */
    public void test2()
    {
        ObjectContainer db = getDB();
        db.set(new Thread()
        {
            public void run()
            {
                System.out.println("Hi");
            }
        });
    }

    public void test3() throws IllegalIDException
    {
        ProteusImpl proteus = getProteus();
        ProteusImpl proteus2 = getProteus();
        
        ObjectContainer db = getDB();
        long id1 = db.ext().getID(proteus);
        long id2 = db.ext().getID(proteus2);
        System.out.println("id2 = " + id2);
        System.out.println("id1 = " + id1);
    }


    private ObjectContainer getDB()
    {
        String DB_NAME = "/tmp/test.db4o";
        return Db4o.openFile(DB_NAME);
    }

    /*Just a one-off*/
    public void deleteAll()
    {
        String DB_NAME = "/tmp/proteus.db4o.nonjoey";
        Config.getConfig().setStringValue(Consts.DATABASE_FILE, DB_NAME);

        ObjectContainer db = DB4OPool.getDB();
        ObjectClass aClass = db.ext().configure().objectClass(ProteusSettings.class);
        aClass.cascadeOnDelete(true);
        
        PersistenceQueue persistenceQueue = new PersistenceQueue();
        List<ProteusImpl> list = persistenceQueue.recover();

        Collection<WebHitCounter.WebHit> hits = WebHitCounter.getInstance().getHits().values();

        for (final ProteusImpl proteus : list)
        {
            for (final WebHitCounter.WebHit hit : hits)
                {
                    if (hit.getType() == WebHitCounter.WebHit.Type.SUBMISSION
                            && hit.getID() != null
                            && hit.getID().equals(proteus.getID()))
                    {
                        String email = hit.getEmail();

                        if ( email != null && email.equals("josephcruz@ualberta.ca"))
                        {
                            db.delete(proteus.getSettings());
                            db.delete(hit);
                            break;
                        }
                    }
                }
        }

        db.commit();
    }

    /*Just a one-off*/
    public void deleteAll2()
    {
        String DB_NAME = "/tmp/proteus.db4o.joey";
        Config.getConfig().setStringValue(Consts.DATABASE_FILE, DB_NAME);

        ObjectContainer db = DB4OPool.getDB();
        ObjectClass aClass = db.ext().configure().objectClass(ProteusSettings.class);
        aClass.cascadeOnDelete(true);

        PersistenceQueue persistenceQueue = new PersistenceQueue();
        List<ProteusImpl> list = persistenceQueue.recover();

        Collection<WebHitCounter.WebHit> hits = WebHitCounter.getInstance().getHits().values();

        for (final ProteusImpl proteus : list)
        {
            for (final WebHitCounter.WebHit hit : hits)
                {
                    if (hit.getType() == WebHitCounter.WebHit.Type.SUBMISSION
                            && hit.getID() != null
                            && hit.getID().equals(proteus.getID()))
                    {
                        String email = hit.getEmail();

                        if (email != null && ! email.equals("josephcruz@ualberta.ca"))
                        {
                            db.delete(proteus.getSettings());
                            db.delete(hit);
                            break;
                        }
                    }
                }
        }

        db.commit();
    }
}
