package ca.ualberta.proteus.db.test;

import ca.ualberta.proteus.db.PersistenceQueue;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.Consts;
import junit.framework.TestCase;
import org.biojava.bio.seq.db.IllegalIDException;

import java.util.List;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: May 10, 2007
 * Time: 9:17:21 PM
 */
public class PersistentQueueTest extends TestCase
{
    public void test1() throws IllegalIDException
    {
        Config.getConfig().setStringValue(Consts.DATABASE_FILE, "test.db4o");

        PersistenceQueue persistenceQueue = new PersistenceQueue();
        /*Clear it */
        persistenceQueue.clear();

        /*Verify*/
        List<ProteusImpl> recovered = persistenceQueue.recover();
        assertEquals(0, recovered.size() );

        /*Put one in*/
        ProteusImpl proteus = getTestProteusInstance();
        persistenceQueue.addToQueue(proteus);

        /*Verify*/
        recovered = persistenceQueue.recover();
        assertEquals(1, recovered.size() );

        /* Put another one in */
        ProteusImpl proteus2 = getTestProteusInstance();
        persistenceQueue.addToQueue(proteus2);

        /*Verify*/
        recovered = persistenceQueue.recover();
        assertEquals(2, recovered.size() );

        /*Remove one */
        persistenceQueue.remove(proteus);

        /*Verify*/
        recovered = persistenceQueue.recover();
        assertEquals(1, recovered.size() );

        /*Remove one */
        persistenceQueue.remove(proteus2);

        /*Verify*/
        recovered = persistenceQueue.recover();
        assertEquals(0, recovered.size() );
    }

    private ProteusImpl getTestProteusInstance() throws IllegalIDException
    {
        final ProteusSettings settings = new ProteusSettings();

        settings.setMethod(MethodSettings.PSIPRED);
        settings.setRunHomodeller(false);

        settings.setTemplateID("2TRX");
        settings.setRunSecondaryStructure(false);

        // other settings
        settings.setTesting(true);
        settings.setSequence(SeqSeeDB.getTestSequence());
        settings.setID(String.valueOf(Thread.currentThread().getId()));
        settings.setRunEnergyMinimization(false);
        settings.setOrganism(Organism.EUKARYOTIC);

        return new ProteusImpl(settings);
    }
}
