package ca.ualberta.proteus.logging;

import ca.ualberta.proteus.util.Consts;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.io.Serializable;

/**
 * A class that stores all logs and can be outputted as a string later.
 * Also stores objects for later retrieval.
 */
public class RuntimeCatalog extends ProteusLogger implements Logger, Serializable
{
    private StringBuilder logBuffer = new StringBuilder();
    private StringBuilder debugBuffer = new StringBuilder();
    private StringBuilder warnBuffer = new StringBuilder();
    private StringBuilder errBuffer = new StringBuilder();
    private LinkedHashMap<String, Object> objectLogs = new LinkedHashMap<String,Object>();

    private List<Consts.Steps> stepsComplete = new ArrayList<Consts.Steps>();
    private boolean finished = false;

    public void log(String s) {
        if ( aboveLevel(LEVEL.LOG ) )
            synchronized (objectLogs) {
                logBuffer.append(s);
                logBuffer.append("\n");
            }
        System.out.println(s);
    }

    public void error(String s) {
        if ( aboveLevel(LEVEL.ERROR) )
        {
            errBuffer.append(s).append("\n");
            System.out.println("ERROR: " + s);
        }
    }

    public void warn(String s) {
        if ( aboveLevel(LEVEL.WARN ))
        {
            warnBuffer.append(s).append("\n");
            System.out.println("WARNING: " + s);
        }
    }

    public void debug(String s) {
        if ( aboveLevel(LEVEL.DEBUG ) )
        {
            debugBuffer.append(s).append("\n");
            System.out.println("DEBUG: " + s);
        }
    }

    public void logObject(String description, Object obj ) {
        if( aboveLevel(LEVEL.LOG ))
        {
            synchronized (objectLogs) {
                objectLogs.put( description, obj );
            }
        }
    }

    public String toString() {
        StringBuilder rv = new StringBuilder();
        synchronized(logBuffer) {
            rv.append(logBuffer.toString());
        }
        synchronized(objectLogs) {
            for (String s1 : objectLogs.keySet())
            {
                rv.append(s1);
                rv.append(":\n");
                Object o = objectLogs.get(s1);
                rv.append(o).append("\n");
            }
        }
        return rv.toString();
    }

    public void finish()
    {
        this.finished = true;
    }

    public boolean isFinished()
    {
        return finished;
    }

    public StringBuilder getLogBuffer()
    {
        return logBuffer;
    }

    public StringBuilder getDebugBuffer()
    {
        return debugBuffer;
    }

    public StringBuilder getErrorBuffer()
    {
        return errBuffer;
    }

    public Object getLogObject(String s)
    {
        return objectLogs.get(s);
    }

    public void stepComplete(Consts.Steps step) {
        stepsComplete.add(step);
    }

    public List<Consts.Steps> getStepsComplete() {
        return stepsComplete;
    }
}
