package ca.ualberta.proteus.program;

import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.db.model.ProteusSettingsIntf;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.File;
import java.io.Serializable;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Feb 1, 2006
 * Time: 7:40:16 PM
 * To change this template use File | Settings | File Templates.
 */
public class ProteusSettings implements Serializable, ProteusSettingsIntf
{
    static final int serialVersionUID = 10200;

    private Sequence sequence = null;
    /* Store string representation of sequence too for serialization (biojava doesn't serialize well)*/
    private String sequenceStr = null;
    private File structure = null;
    private boolean testing = false;
    private String ID;
    private MethodSettings method = MethodSettings.ALL;
    private boolean runEnergyMinimization;
    private boolean runHomodeller;
    private String templateID = null;
    private boolean evaluating;
    private Organism organism = null;
    private boolean runSecondaryStructure = true;
    private boolean runOnlyTM = false;
    private Double identityThreshold = null;


    public ProteusSettings()
    {
    }

    public ProteusSettings(ProteusSettings baseSettings)
    {
        this.setSequence(baseSettings.sequence);
        this.structure = baseSettings.structure;
        this.testing = baseSettings.testing;
        this.ID = baseSettings.ID;
        this.method = baseSettings.method;
        this.runEnergyMinimization = baseSettings.runEnergyMinimization;
        this.runHomodeller = baseSettings.runHomodeller;
        this.templateID = baseSettings.templateID;
        this.evaluating = baseSettings.evaluating;
        this.organism = baseSettings.organism;
		this.identityThreshold = baseSettings.identityThreshold;
        this.runSecondaryStructure = baseSettings.runSecondaryStructure;
    }

    public ProteusSettings(ProteusSettingsIntf baseSettings)
    {
        this.structure = baseSettings.getStructure();
        this.setSequence(baseSettings.getSequence());
        this.ID = baseSettings.getID();
        this.method = baseSettings.getMethod();
        this.runEnergyMinimization = baseSettings.isRunEnergyMinimization();
        this.runHomodeller = baseSettings.isRunHomodeller();
        this.templateID = baseSettings.getTemplateID();
        this.evaluating = baseSettings.isEvaluating();
        this.organism = baseSettings.getOrganism();
		this.identityThreshold = baseSettings.getIdentityThreshold();
        this.runSecondaryStructure = baseSettings.isRunSecondaryStructure();
    }

    public Sequence getSequence()
    {
        try
        {
            if (sequence != null)
            {
                // See if this string is intact
                sequence.seqString();
            }
            else
            {
                // No it's not, rebuild it from the string representation
                sequence = ProteinTools.createProteinSequence(sequenceStr, sequence.getName());
            }
        }
        catch (Throwable e)
        {
            // No it's not, rebuild it from the string representation
            try
            {
                sequence = ProteinTools.createProteinSequence(sequenceStr, sequence.getName());
            }
            catch (IllegalSymbolException e1)
            {
                e1.printStackTrace();
            }
        }
        return sequence;
    }

    public void setSequence(Sequence sequence)
    {
        /* Store string representation too for serialization (biojava doesn't serialize well)*/
        this.sequence = sequence;
        if (sequence != null)
        {
            this.sequenceStr = sequence.seqString();
        }
    }

    public String getSequenceStr()
    {
        return sequenceStr;
    }

    public void setSequenceStr(String sequenceStr)
    {
        this.sequenceStr = sequenceStr;
    }
	/*************************************************************/
	public Double getIdentityThreshold()
    {
        return identityThreshold;
    }
	
    public void setIdentityThreshold(Double identityThreshold)
    {
		this.identityThreshold = identityThreshold;
    }
    /*************************************************************/
    public File getStructure()
    {
        return structure;
    }

    public void setStructure(File structure)
    {
        this.structure = structure;
    }

    public boolean isTesting()
    {
        return testing;
    }

    public void setTesting(boolean testing)
    {
        this.testing = testing;
    }

    public String getID()
    {
        return ID;
    }

    public void setID(String ID)
    {
        this.ID = ID;
    }

    public MethodSettings getMethod()
    {
        return method;
    }

    public void setMethod(MethodSettings method)
    {
        this.method = method;
    }


    public void setRunEnergyMinimization(boolean runEnergyMinimization)
    {
        this.runEnergyMinimization = runEnergyMinimization;
    }

    public boolean isRunEnergyMinimization()
    {
        return runEnergyMinimization;
    }


    public boolean isRunHomodeller()
    {
        return runHomodeller;
    }

    public void setRunHomodeller(boolean runHomodeller)
    {
        this.runHomodeller = runHomodeller;
    }

    public void setTemplateID(String templateID)
    {
        this.templateID = templateID;
    }

    public String getTemplateID()
    {
        return templateID;
    }

    public void setEvaluating(boolean evaluating)
    {
        this.evaluating = evaluating;
    }

    public boolean isEvaluating()
    {
        return evaluating;
    }

    public void setOrganism(Organism organism)
    {
        this.organism = organism;
    }

    public Organism getOrganism()
    {
        return organism;
    }

    public void setRunSecondaryStructure(boolean runSecondaryStructure)
    {
        this.runSecondaryStructure = runSecondaryStructure;
    }

    public boolean isRunSecondaryStructure()
    {
        return runSecondaryStructure;
    }

    public boolean isRunOnlyTM()
    {
        return runOnlyTM;
    }

    public void setRunOnlyTM(boolean runOnlyTM)
    {
        this.runOnlyTM = runOnlyTM;
    }
}
