package ca.ualberta.proteus.program;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.StreamHandler;
import ca.ualberta.proteus.util.io.StructureDB;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.structure.Structure;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Nov 11, 2006
 * Time: 4:02:13 PM
 * To change this template use File | Settings | File Templates.
 */
public class VadarRunner
{
    public static Sequence getSecondaryStructure(Structure pdb)
    {
        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");

        String pathname = config.getStringValue("pdb_dir");
        String vadar = config.getStringValue("run_vadar");

        String name = pdb.getName();
        String s = pdb.toPDB();

        File f = new File(pathname, name);

        try
        {
            PrintWriter out = new PrintWriter(f);
            out.print(s);
            out.close();

            String cmd = String.format("%s -f %s -o %s", vadar, f, f);
            Process xalignProc = Runtime.getRuntime().exec(cmd);

            // any error message?
            StreamHandler errorReader = new
                    StreamHandler(xalignProc.getErrorStream(), "ERROR");

            // any output?
            StreamHandler outputReader = new
                    StreamHandler(xalignProc.getInputStream(), "OUTPUT");

            // kick them off
            errorReader.start();
            outputReader.start();

            int exitVal = xalignProc.waitFor();

            if ( exitVal != 0 )
            {
                System.out.println("XAlign ExitValue: " + exitVal);
                System.out.println("XAlign failed");
                System.out.println("Cmd: " + cmd );
                System.out.println(errorReader.getInputString());
                System.out.println(outputReader.getInputString());
            }

            // read the secondary structure
            StructureDB db = new StructureDB(f + ".main", StructureDB.MODE.READ);

            if ( db.size() > 0 )
            {
                return db.getSequence(0);
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        catch (BioException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }



        return null;
    }
}
