package ca.ualberta.proteus.program.hmmpfam;

import org.apache.log4j.Logger;
import org.biojava.bio.seq.Sequence;

import java.io.BufferedReader;
import java.io.Reader;
import java.text.ParseException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: Apr 15, 2007
 * Time: 11:45:09 PM
 */
public class HMMPFamReader
{
    private Logger logger = Logger.getLogger(HMMPFamReader.class);
                    
    public HMMPFamReader()
    {
    }

    /**
     * This method parse the output reader of PredictSPTM program
     * for signal Peptide prediction.
     * <br/>
     * Note: Only parses a signal sequence, not all of them.
     *
     * @param sequence
     *@param reader a reader @return A String containing the Signal Site in a sequence @throws java.text.ParseException If there is a parse error
     */
    public Integer parseSignalOutput(Sequence sequence, Reader reader) throws ParseException
    {
        String line;

        try
        {
            BufferedReader br = new BufferedReader(reader);
            String name;
            while ((line = br.readLine()) != null)
            {
                if ( line.contains("[no hits above thresholds]"))
                {
                    return null;
                }

                if (line.equals("Alignments of top-scoring domains:"))
                {
                    // end of sequence
                    break;
                }
            }

            // skip 3 lines
            br.readLine();
            br.readLine();
            br.readLine();
            line = br.readLine();

            if ( line != null )
            {
                Pattern pattern = Pattern.compile(".*\\s+(\\d+)\\s*$");
                Matcher matcher = pattern.matcher(line);

                if ( matcher.matches())
                {
                    // it should match
                    return Integer.valueOf(matcher.group(1));
                }
            }
            else
            {
                throw new ParseException("Error parsing HMMER signal peptide output file", 0);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new ParseException("Error parsing HMMER signal peptide output file", 0);
        }

        return null;
    }
}
