package ca.ualberta.proteus.program.hmmpfam.test;

import ca.ualberta.proteus.program.hmmpfam.HMMPFamReader;
import ca.ualberta.proteus.program.hmmpfam.HMMPFamRunner;
import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.io.StructureDB;
import junit.framework.TestCase;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.*;
import java.text.ParseException;
import java.util.List;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: Apr 15, 2007
 * Time: 11:38:00 PM
 */
public class HMMPFamRunnerTest extends TestCase
{
    public void testReader() throws IllegalSymbolException
    {
        String sequence = "VAPTLTARLYSLLFRRTSTFALTIVVGALFFERAFDQGADAIYEHINEGKLWKHIKHKYENK";
        Sequence seq = ProteinTools.createProteinSequence("TEST", sequence);

        String file = "hmmpfam - search one or more sequences against HMM database\n" +
                "HMMER 2.2g (August 2001)\n" +
                "Copyright (C) 1992-2001 HHMI/Washington University School of Medicine\n" +
                "Freely distributed under the GNU General Public License (GPL)\n" +
                "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n" +
                "HMM file:                 HMMs/GRAM+.hmm\n" +
                "Sequence file:            test2.faa\n" +
                "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n" +
                "\n" +
                "Query sequence: WHAT\n" +
                "Accession:      [none]\n" +
                "Description:    [none]\n" +
                "\n" +
                "Scores for sequence family classification (score includes all domains):\n" +
                "Model                 Description                       Score    E-value  N\n" +
                "--------              -----------                       -----    ------- ---\n" +
                "GRAM+SIG.reduc.subseq                                    -2.5       0.15   1\n" +
                "\n" +
                "Parsed for domains:\n" +
                "Model                 Domain  seq-f seq-t    hmm-f hmm-t      score  E-value\n" +
                "--------              ------- ----- -----    ----- -----      -----  -------\n" +
                "GRAM+SIG.reduc.subseq   1/1      11    43 ..     1    35 []    -2.5     0.15\n" +
                "\n" +
                "Alignments of top-scoring domains:\n" +
                "GRAM+SIG.reduc.subseq: domain 1 of 1, from 11 to 43: score -2.5, E = 0.15\n" +
                "                   *->kkklkkllalalalallllavlllaklapvaaaaa<-*\n" +
                "                         +++  + al++++++l++  +  + + a a\n" +
                "        WHAT    11    SLLFRRTSTFALTIVVGALFFERA--FDQGADAIY    43\n" +
                "\n" +
                "//";

        HMMPFamReader tmReader = new HMMPFamReader();
        Integer signalSite = null;
        try
        {
            signalSite = tmReader.parseSignalOutput(seq, new StringReader(file));
            assertNotNull(signalSite);
            assertEquals(43, (int) signalSite);
        }
        catch (ParseException e)
        {
            e.printStackTrace();
        }
        if (signalSite != null)
        {
            System.out.println("Signal Sites  : " + signalSite);
        }
    }

    public void testReaderNegative() throws IllegalSymbolException
    {
        String sequence = "VAPTLTARLYSLLFRRTSTFALTIVVGALFFERAFDQGADAIYEHINEGKLWKHIKHKYENK";
        Sequence seq = ProteinTools.createProteinSequence("TEST", sequence);

        String file = "Query sequence: 3\n" +
                "Accession:      [none]\n" +
                "Description:    [none]\n" +
                "\n" +
                "Scores for sequence family classification (score includes all domains):\n" +
                "Model    Description                                    Score    E-value  N\n" +
                "-------- -----------                                    -----    ------- ---\n" +
                "        [no hits above thresholds]\n" +
                "\n" +
                "Parsed for domains:\n" +
                "Model    Domain  seq-f seq-t    hmm-f hmm-t      score  E-value\n" +
                "-------- ------- ----- -----    ----- -----      -----  -------\n" +
                "        [no hits above thresholds]\n" +
                "\n" +
                "Alignments of top-scoring domains:\n" +
                "        [no hits above thresholds]\n" +
                "//";

        HMMPFamReader tmReader = new HMMPFamReader();
        Integer signalSite = null;
        try
        {
            signalSite = tmReader.parseSignalOutput(seq, new StringReader(file));
            assertNull(signalSite);
        }
        catch (ParseException e)
        {
            e.printStackTrace();
        }        
    }

    public void testRunner() throws BioException, IOException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);

        for (int i = 0; i < 5; i++)
        {
            Sequence sequence = db.getSequence(i);

            HMMPFamRunner runner = new HMMPFamRunner(sequence, Organism.GRAM_POSITIVE);
            StructureAnnotatedSequence signalSeq = runner.runHMMPFam();

            assertEquals(signalSeq.length(), sequence.length());

            System.out.println(signalSeq);
        }
    }

    public void testRunner2() throws BioException, IOException
    {
        /*String file = ">P31550\n" +
                "MLKKCLPLLLLCTAPVFAKPVLTVYTYDSFAADWGPGPVVKKAFEADCNCELKLVALEDG\n" +
                "VSLLNRLRMEGKNSKADVVLGLDNNLLDAASKTGLFAKSGVAADAVNVPGGWNNDTFVPF\n" +
                "DYGYFAFVYDKNKLKNPPQSLKELVESDQNWRVIYQDPRTSTPGLGLLLWMQKVYGDDAP\n" +
                "QAWQKLAKKTVTVTKGWSEAYGLFLKGESDLVLSYTTSPAYHILEEKKDNYAAANFSEGH\n" +
                "YLQVEVAARTAASKQPELAQKFLQFMVSPAFQNAIPTGNWMYPVANVTLPAGFEKLTKPA\n" +
                "TTLEFTPAEVAAQRQAWISEWQRAVSR";

        List<Sequence> list = ProteusTools.parseFasta(new StringReader(file));
        Sequence sequence = list.get(0);

        PredictSPTMRunner runner = new PredictSPTMRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runHMMPFam();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);*/
    }

    public void testRunner3() throws BioException, IOException
    {
        String file = ">TEST\n" +
                "VAPTLTARLYSLLFRRTSTFALTIVVGALFFERAFDQGADAIYEHINEGKLWKHIKHKYENK";

        List<Sequence> list = ProteusTools.parseFasta(new StringReader(file));
        Sequence sequence = list.get(0);

        HMMPFamRunner runner = new HMMPFamRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runHMMPFam();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);

        String EXPECTED = "ssssssssssssssssssssssssssssssssssssssssssc-------------------";

        assertEquals(signalSeq.getStructureAnnotation().seqString().replace(' ', '-'), EXPECTED);

        //////////////////////////////

        runner = new HMMPFamRunner(sequence, Organism.GRAM_NEGATIVE);
        signalSeq = runner.runHMMPFam();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);

        EXPECTED = "--------------------------------------------------------------";

        assertEquals(EXPECTED, signalSeq.getStructureAnnotation().seqString().replace(' ', '-'));
    }

    public void testSerialization() throws BioException, IOException, ClassNotFoundException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);
        Sequence sequence = db.getSequence(0);

        HMMPFamRunner runner = new HMMPFamRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runHMMPFam();

        ObjectOutputStream stream = new ObjectOutputStream(new FileOutputStream("tmp"));
        stream.writeObject(signalSeq);
        stream.close();

        ObjectInputStream in = new ObjectInputStream(new FileInputStream("tmp"));
        Object o = in.readObject();

        System.out.println(o);
    }
}
