/*
 * Created on Apr 21, 2008
 */
package ca.ualberta.proteus.program.homodeller;

import ca.ualberta.proteus.util.ProteusTools;

import org.biojava.bio.seq.GappedSequence;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 *
 * @author Joseph Cruz
 */
public class Gaps {

	private static enum Type { Insertion, Deletion }
	private static final int GAP_SIZE_BEFORE_INSERTION = 5;
	
    private GappedSequence querySequence;
    private GappedSequence databaseSequence;
    private int gapSizeBeforeInsertion;
    
    private List<Gap> gaps = new ArrayList<Gap>();
    
    public Gaps(GappedSequence querySequence, GappedSequence databaseSequence, int gapSizeBeforeInsertion) {
    	this.querySequence = querySequence;
    	this.databaseSequence = databaseSequence;
    	this.gapSizeBeforeInsertion = gapSizeBeforeInsertion;
    	this.findGaps();
    }
    
    public Gaps(GappedSequence querySequence, GappedSequence databaseSequence) {
    	this(querySequence, databaseSequence, GAP_SIZE_BEFORE_INSERTION);
    }
    
    public boolean isValid() {
    	for( int i = 0; i < this.gaps.size(); i++ ) {
    		Gap currentGap = this.gaps.get(i);
    		for( int j = i+1; j < this.gaps.size(); j++ ) {
    			Gap nextGap = this.gaps.get(j);
    			if( currentGap.getEnd() >= nextGap.start )
    				return false;
    		}
    	}
    	return true;
    }
    
    public void printGaps() {
    	for( Gap g: this.gaps )
    		g.print();
    }
    
    public void findGaps() {
    	this.gaps.clear();
    	
    	String query = this.querySequence.seqString();
    	String database = this.databaseSequence.seqString();

    	int queryLength = query.length();
    	int i = 0;
    	while( i < queryLength ) {
    		Gap gap = null;
            char q = query.charAt(i);
            char d = database.charAt(i);
            
            if( ProteusTools.isGap(d) && ProteusTools.isGap(q) ) {	// should never have gap on both query and database sequences
                throw new IllegalStateException("wtf??");
            }
            else if( ProteusTools.isGap(d) ) {	// insertion
            	gap = new Gap();
            	gap.start = i;
            	gap.type = Type.Insertion;
            	
            	int gapLength = 1;
                while( (i+gapLength) < database.length() && ProteusTools.isGap(database.charAt(i+gapLength))) {
                    gapLength++;
                }
                i += gapLength - 1;
                gap.length = gapLength;
            }
            else if( ProteusTools.isGap(q) ) {	// deletion
            	gap = new Gap();
            	gap.start = i;
            	gap.type = Type.Deletion;
            	
                int gapLength = 1;
                while( (i+gapLength) < database.length() && ProteusTools.isGap(query.charAt(i+gapLength)) ) {
                    gapLength++;
                }
                i += gapLength -1;
                gap.length = gapLength;
            }
            
            if( gap != null ) {
            	gap.fixedNTerminalPosition = gap.start - (int)Math.floor(this.gapSizeBeforeInsertion/2.0);
            	gap.fixedCTerminalPosition = gap.getEnd() + (int)Math.ceil(this.gapSizeBeforeInsertion/2.0);
            	this.gaps.add(gap);
            }
            
            i++;
        }
    }
    
    public void trimGaps() {
    	Set<Gap> gapsToRemove = new HashSet<Gap>();
    	for( Gap g: this.gaps ) {
    		if( g.start < 1 )
    			gapsToRemove.add(g);
    		else if( g.getEnd() > this.querySequence.length() )
    			gapsToRemove.add(g);
    	}
    	for( Gap g: gapsToRemove )
    		this.gaps.remove(g);
    }
    
    private class Gap {
    	int start;
    	int length;
    	int fixedNTerminalPosition;
    	int fixedCTerminalPosition;
    	Type type;
    	
    	int getEnd() {
    		return start + length + 1;
    	}
    	
    	public void print() {
    		System.out.println(type + ": " + start + "-" + getEnd() + " (" + fixedNTerminalPosition + "-" + fixedCTerminalPosition + ")");
    	}
    }
}
