package ca.ualberta.proteus.program.homodeller;

import ca.ualberta.proteus.program.exception.NotInitializedException;
import org.biojava.bio.structure.*;

import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Aug 31, 2006
 * Time: 10:47:44 PM
 * To change this template use File | Settings | File Templates.
 */
public class Loop extends ChainImpl
{
    private int variationNumber;
    private int loopLength;
    	// Note by David Arndt: It looks like loopLength is the number of coil residues in
    	// the loop. In the loop database, each loop entry appears to have 3 non-coil
    	// residues preceding and following the main coil part of the loop.
    private int loopNumber;
    private long filePointer;
    private boolean initialized;
    private ExtendedLoopData loopData;

    public int getVariationNumber()
    {
        return variationNumber;
    }

    public void setVariationNumber(int value)
    {
        //To change body of created methods use File | Settings | File Templates.
        this.variationNumber = value;
    }

    public int getLoopLength()
    {
        return loopLength;
    }

    public void setLoopLength(int resNumber)
    {
        this.loopLength = resNumber;
    }

    public int getLoopNumber()
    {
        return loopNumber;
    }

    public void setLoopNumber(int loopNumber)
    {
        this.loopNumber = loopNumber;
    }

    public long getFilePointer()
    {
        return filePointer;
    }

    public void setFileIndex(long filePointer)
    {
        this.filePointer = filePointer;
    }

    public String getName()
    {
        /*  String s = String.valueOf("Loop#: " + loopNumber);
        if ( variationNumber != 0)
        {
            s += " Variation#: " + variationNumber;
        }
        */
        return super.getName();
    }

    /**
     * string representation.
     */
    public String toString()
    {

        String str = "Chain >" + getName() + "< total length:" + getLength() + " residues" +
                ", loop length = " + loopLength + "\n";

        if (getInitialized())
        {
            // loop over the residues
            int i = 0;
            while (i < getGroups().size())
            {
                Group gr = (Group) getGroups().get(i);
                str += gr.toString() + "\n";
                i++;
            }
        }
        else
        {
            str += "Not Initialized";
        }

        return str;

    }

    public Group getGroup(int position)
    {
        ensureInitialized();

        return super.getGroup(position);
    }

    public ArrayList getGroups(String type)
    {
        ensureInitialized();
        return super.getGroups(type);
    }

    public ArrayList getGroups()
    {
        ensureInitialized();
        return super.getGroups();
    }

    public int getLength()
    {
        ensureInitialized();
        return super.getLength();
    }

    public int getLengthAminos()
    {
        ensureInitialized();
        return super.getLengthAminos();
    }

    public String getSequence()
    {
        ensureInitialized();
        return super.getSequence();
    }

    private void ensureInitialized()
    {
        if (! getInitialized())
        {
            throw new NotInitializedException();
        }
    }

    public boolean getInitialized()
    {
        return initialized;
    }

    public void setInitialized(boolean initialized)
    {
        this.initialized = initialized;
    }

    public void setLoopData(ExtendedLoopData data)
    {
        this.loopData = data;
    }

    public ExtendedLoopData getLoopData()
    {
        return loopData;
    }

    /**
     * Returns the CA-CA atom distance of the coil part of the loop
     *
     * @return
     * @throws StructureException
     */
    public double getCA_CADist() throws StructureException
    {
        /*
        List<LoopResidueGroup> groups = getGroups();
        int pre = 0, post = 0;
        for (int i = 0; i < groups.size(); i++)
        {
            LoopResidueGroup g = groups.get(i);
            if (g.getResidueType().equals("C"))
            {
                if (pre == 0)
                {
                    pre = i;
                    post = i;
                }
                else
                {
                    post++;
                }
            }
        }

        assert post - pre + 1 == loopLength: "Pre and post variables do not match loop length";
*/
        // we want the atoms flanking the loop, hence the +/- 1
        Atom preCA = getFlankingPreCoilGroup().getAtom("CA");
        Atom postCA = getFlankingPostCoilGroup().getAtom("CA");

        return HomodellerUtils.getDistance(preCA, postCA);
    }

    /**
     * Returns a clone of all groups that compose the inner coil of the loop
     * plus the flanking residues
     *
     * i.e. <br/>
     * <code>
     *      ABCDEF  <br/>
     *      HCCCHH  <br/>
     * <code>
     *
     * it would return ABCDE
     *
     * @return a clone of all groups that compose the inner coil of the loop
     */
    public Loop getCoilLoop()
    {
        Loop loop = new Loop();

        loop.setInitialized(getInitialized());
        // copy chain data:

        loop.setName(getName());
        loop.setSwissprotId(getSwissprotId());
        loop.setLoopNumber(getLoopNumber());
        loop.setLoopLength(getLoopLength());

        List<LoopResidueGroup> groups = getGroups();

        for (int i=0; i<groups.size(); i++)
        {
            LoopResidueGroup prev = i > 0 ? groups.get(i - 1) : null;
            LoopResidueGroup g = groups.get(i);
            LoopResidueGroup next = i < groups.size() - 1? groups.get(i + 1) : null;
            if ( (prev != null && prev.getResidueType().equals("C") )
                    || g.getResidueType().equals("C")
                    || ( next != null && next.getResidueType().equals("C") ) )
            {
                loop.addGroup((Group) g.clone());
            }
        }

        return loop;
    }

    /**
     * Returns a portion of the sequence that's a coil
     * @return a portion of the sequence that's a coil
     */
    public String getCoilSequence()
    {
        List<LoopResidueGroup> groups = getGroups();
        StringBuffer sequence = new StringBuffer();
        for (LoopResidueGroup g : groups)
        {
            if (g.getResidueType().equals("C"))
            {
                sequence.append(g.getAminoType());
            }
        }

        return sequence.toString();
    }

    /**
     * Get the distance from the first inserted residue N to the last inserted C (not CA)
     * @return distance from the first inserted residue N to the last inserted C
     */
    public double getNCDist() throws StructureException
    {
        List<LoopResidueGroup> groups = getGroups();
        int pre = 0, post = 0;
        for (int i = 0; i < groups.size(); i++)
        {
            LoopResidueGroup g = groups.get(i);
            if (g.getResidueType().equals("C"))
            {
                if (pre == 0)
                {
                    pre = i;
                    post = i;
                }
                else
                {
                    post++;
                }
            }
        }

        assert post - pre + 1 == loopLength: "Pre and post variables do not match loop length";

        Atom preN = getGroup(pre).getAtom("N");
        Atom postC = getGroup(post).getAtom("C");

        return HomodellerUtils.getDistance(preN, postC);
    }

    /**
     * Returns the residue flanking the coil  <br/>
     * i.e. <br/>
     * <code>
     *      ABCDE  <br/>
     *      HCCCH  <br/>
     * <code>
     * it would return E
     * @return the residue flanking the coil
     */
    public Group getFlankingPostCoilGroup()
    {
        List<LoopResidueGroup> groups = getGroups();
        int pre = 0, post = 0;
        for (int i = 0; i < groups.size(); i++)
        {
            LoopResidueGroup g = groups.get(i);
            if (g.getResidueType().equals("C"))
            {
                if (pre == 0)
                {
                    pre = i;
                    post = i;
                }
                else
                {
                    post++;
                }
            }
        }

        assert post - pre + 1 == loopLength: "Pre and post variables do not match loop length";

        // we want the atoms flanking the loop, hence the +/- 1
        return getGroup(post + 1);
    }

    /**
     * Returns the residue flanking the coil  <br/>
     * i.e. <br/>
     * <code>
     *      ABCDE  <br/>
     *      HCCCH  <br/>
     * <code>
     * it would return A
     * @return the residue flanking the coil
     */
    public Group getFlankingPreCoilGroup()
    {
        List<LoopResidueGroup> groups = getGroups();
        int pre = 0;
        for (int i = 0; i < groups.size(); i++)
        {
            LoopResidueGroup g = groups.get(i);
            if (g.getResidueType().equals("C"))
            {
                if (pre == 0)
                {
                    pre = i;
             //       post = i;
                }
                else
                {
            //        post++;
                }
            }
        }

      //  assert post - pre + 1 == loopLength: "Pre and post variables do not match loop length";

        // we want the atoms flanking the loop, hence the +/- 1
        return getGroup(pre - 1);
    }
}

/**
 * Class for keeping other data about a loop
 */
class ExtendedLoopData
{
    /*
        aa refers to CA -CA distance
        A = CA atom
        B = CB atom
        C = C atom
        O = O atom
        N = N atom
    */
    enum FIELD
    {
        aa,ac,an,ao,ab,ca,cc,cn,co,cb,na,nc,nn,no,nb,oa,oc,on,oo,ob,ba,bc,bn,bo,bb}

    private String loopType;
    private Map<FIELD, Double> data = new HashMap<FIELD, Double>();

    public String getLoopType()
    {
        return loopType;
    }

    public void setLoopType(String loopType)
    {
        this.loopType = loopType;
    }

    public void setData(FIELD field, double value)
    {
        data.put(field, value);
    }

    public Double getData(FIELD field)
    {
        Double value = data.get(field);
        return value != null ? value : 0;
    }
}