package ca.ualberta.proteus.program.homodeller;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.GenericProgramRunner;
import ca.ualberta.proteus.util.StreamHandler;

import java.io.*;
import java.util.TreeMap;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 9, 2006
 * Time: 10:35:43 AM
 * To change this template use File | Settings | File Templates.
 */
public class PhipsiRunner extends GenericProgramRunner
{
    String pdbText;
    private static boolean debug = Config.getConfig().getBooleanValue("debug");

    public PhipsiRunner(String pdb)
    {
        this.pdbText = pdb;
    }

    /**
     * Runs the phipsi program on a PDB file, whether an entire sequence, or one residue
     *
     * @return a map of residue indexes to angles
     */
    public TreeMap<Integer,Angle> runPhiPsi()
    {
        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");
        String phipsiPath = config.getStringValue("phipsi");
        try
        {
            // write PDB file to temp file
            File tempFile;
            if ( debug )
            {
                tempFile = new File("phipsi_input.pdb");
            }
            else
            {
                tempFile = File.createTempFile("phipsi_input", ".pdb");
                tempFile.deleteOnExit();
            }
            PrintWriter w = new PrintWriter(tempFile);
            w.print(pdbText);
            w.close();

            Process psipredProc = Runtime.getRuntime().exec(phipsiPath + " -f " + tempFile.getPath());

            // any error message?
            StreamHandler errorReader = new
                    StreamHandler(psipredProc.getErrorStream(), "ERROR");

            // any output?
            StreamHandler outputReader = new
                    StreamHandler(psipredProc.getInputStream(), "OUTPUT");

            // kick them off
            errorReader.start();
            outputReader.start();

            int exitVal = psipredProc.waitFor();

            if (exitVal != 0)
            {
                System.err.println("Phipsi failed " + errorReader.getInputString());
                return null;
            }

            /** Wait for the output reader to completely do its thing **/
            while (! outputReader.isFinished())
            {
                Thread.sleep(10);
            }

            String phipsiOutput = outputReader.getInputString();

            BufferedReader reader = new BufferedReader(new StringReader(phipsiOutput));

            TreeMap<Integer, Angle> chiAngles = new TreeMap<Integer, Angle>();
            String line = null;
            try
            {
                int index = 0;
                while((line = reader.readLine()) != null)
                {
                    if ( line.length() == 0)
                    {
                        continue;
                    }

                    if ( line.matches(".*\\d+\\s+\\w+.*"))
                    {
                        String[] strings = line.trim().split("\\s+");

                        Angle angle = new Angle();
                        angle.setPhi(Double.parseDouble(strings[2]));
                        angle.setPsi(Double.parseDouble(strings[3]));
                        angle.setOmega(Double.parseDouble(strings[4]));
                        angle.setChi(Double.parseDouble(strings[5]));

                        // chiAngles.put(Integer.parseInt(strings[0]), angle);
                        chiAngles.put(index++, angle);
                    }
                }
            }
            catch(Exception e)
            {
                System.err.println("Error parsing phipsi at line: \n" + line);
            }

            return chiAngles;
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }

        return null;
    }    

    public void run()
    {
        runPhiPsi();
    }
}
