package ca.ualberta.proteus.program.homodeller;

import org.biojava.bio.structure.AminoAcidImpl;
import org.biojava.bio.structure.StructureException;
import org.biojava.bio.structure.Atom;
import org.biojava.bio.structure.Group;
import org.biojava.bio.structure.io.PDBParseException;

import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 3, 2006
 * Time: 10:09:05 PM
 * To change this template use File | Settings | File Templates.
 */
public class SideChain extends AminoAcidImpl
{
    private double angle;

    public double getAngle()
    {
        return angle;
    }

    public void setAngle(double angle)
    {
        this.angle = angle;
    }

    /** string representation. */
    public String toString()
    {
        String str = super.toString();
        str += ", angle: " + getAngle();
        return str;
    }

    public static double[][] getReferenceCoordinates(Group sideChain) throws StructureException
    {
        Atom A = sideChain.getAtom("N");
        Atom B = sideChain.getAtom("CA");
        Atom C = sideChain.getAtom("C");

        return new double[][] { {A.getX(), A.getY(), A.getZ() },
                                {B.getX(), B.getY(), B.getZ() },
                                {C.getX(), C.getY(), C.getZ() } };
    }

    public Object clone()
    {
        SideChain n = new SideChain();
        n.setPDBFlag(has3D());
        n.setPDBCode(getPDBCode());
        try
        {
            n.setPDBName(getPDBName());
        }
        catch (PDBParseException e)
        {
            e.printStackTrace();
        }
        n.setAminoType(getAminoType());

        List<Atom> atoms = getAtoms();

        // copy the atoms
        for (Atom atom : atoms)
        {
            Atom newAtom = (Atom) atom.clone();

            // TODO Biojava's Atom.clone() method doesn't create a new array for coordinates
            double[] newCoords = new double[3];
            double[] coords = atom.getCoords();
            System.arraycopy(coords, 0, newCoords, 0, coords.length);

            newAtom.setCoords(newCoords);

            n.addAtom(newAtom);
        }
        return n;
    }
}
