package ca.ualberta.proteus.program.homodeller;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.GenericProgramRunner;
import ca.ualberta.proteus.util.StreamHandler;
import org.biojava.bio.structure.*;
import org.biojava.bio.structure.io.PDBFileParser;

import java.io.*;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 9, 2006
 * Time: 10:35:43 AM
 * To change this template use File | Settings | File Templates.
 */
public class SimplexRunner extends GenericProgramRunner
{
    private static boolean debug = Config.getConfig().getBooleanValue("debug");
    private Group preResidue;
    private Group postResidue;
    private Chain loop;

    public SimplexRunner(Group preResidue, Group postResidue, Chain loop)
    {
        this.preResidue = preResidue;
        this.postResidue = postResidue;
        this.loop = loop;
    }

    public void setLoop(Chain loop)
    {
        this.loop = loop;
    }

    /**
     * Runs the phipsi program on a PDB file, whether an entire sequence, or one residue
     *
     * @return a map of residue indexes to angles
     */
    public Chain runSimplex()
    {
        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");
        String simplexPath = config.getStringValue("simplex");

        File simplex = new File(simplexPath);
        File dir = simplex.getParentFile().getAbsoluteFile();

        try
        {
// Write out the first file, with the pre and post residues in it
            ChainImpl chain = new ChainImpl();
            chain.addGroup(preResidue);
            chain.addGroup(postResidue);

            StructureImpl tmp = new StructureImpl();
            tmp.addChain(chain);
            String pdbText = tmp.toPDB();
            File file1 = new File("debug" + File.separator + "pre_post.pdb");
            if( !debug )
            	file1.deleteOnExit();

            try
            {
                PrintWriter w = new PrintWriter(file1);
                w.print(pdbText);
                w.close();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }

            // write the loop to file, this is second simplex parameter
            tmp = new StructureImpl();

            tmp.addChain(loop);
            pdbText = tmp.toPDB();
            File file2 = new File("debug" + File.separator + "result.pdb");
            if( !debug )
            	file2.deleteOnExit();

            try
            {
                PrintWriter w = new PrintWriter(file2);
                w.print(pdbText);
                w.close();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }


            String command = simplexPath + " " + file1.getAbsolutePath() + " " + file2.getAbsolutePath();
            
            System.err.println("Running simplex: " + command); // debug -- David Arndt
            //System.err.println("Exiting before call to simplex on purpose for debugging.");
            //System.exit(0);

            Process psipredProc = Runtime.getRuntime().exec(command, null, dir);

            // any error message?
            StreamHandler errorReader = new
                    StreamHandler(psipredProc.getErrorStream(), "ERROR");

            // any output?
            StreamHandler outputReader = new
                    StreamHandler(psipredProc.getInputStream(), "OUTPUT");

            // kick them off
            errorReader.start();
            outputReader.start();

            int exitVal = psipredProc.waitFor();

            if (exitVal != 1)
            {
                // weird, simplex returns 1
                System.err.println("Simplex failed " + errorReader.getInputString() + outputReader.getInputString());
                return null;
            }

            /** Wait for the output reader to completely do its thing **/
            while (! outputReader.isFinished())
            {
                Thread.sleep(10);
            }

            PDBFileParser parser = new PDBFileParser();
            Structure structure = parser.parsePDBFile(new FileInputStream(new File(dir, "simplex_output")));

            return structure.getChain(0);
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }

        return null;
    }

    public void run()
    {
        runSimplex();
    }
}
