package ca.ualberta.proteus.program.homodeller;

import ca.smontgomerie.util.Math.MathExt;
import ca.ualberta.proteus.biojava.PDBFileParser;
import ca.ualberta.proteus.program.thrifty.AngleConverter;
import org.apache.log4j.Logger;
import org.biojava.bio.structure.*;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 4, 2006
 * Time: 1:04:15 AM
 * To change this template use File | Settings | File Templates.
 */
public class Superpose
{
    private static boolean DEBUG_SUPERPOSE = false;
    static final int X = 0, Y = 1, Z = 2;
    private static Logger logger = Logger.getLogger(Superpose.class);

    /**
     * ********************************************************************
     */
    static void vector(double pta[][], double ptb[][], double vectPtr[][])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            vectPtr[i][0] = ptb[i][0] - pta[i][0];

        }
    }

    /**
     * Creates a size 1 "2-D" array from a 1-D array.
     *
     * @param array
     * @return
     */
    public static double[][] createArray(double[] array)
    {
        double[][] rv = new double[3][];
        rv[0] = new double[] {array[0]};
        rv[1] = new double[] {array[1]};
        rv[2] = new double[] {array[2]};

        return rv;
    }


    public static double normalize(double pt[][])
    {
        return Math.sqrt(Math.pow(pt[0][0], 2) + Math.pow(pt[1][0], 2) + Math.pow(pt[2][0], 2));
    }

    public static double normalize(double pt[])
    {
        return Math.sqrt(Math.pow(pt[0], 2) + Math.pow(pt[1], 2) + Math.pow(pt[2], 2));
    }

    public static void translate(double pt[][], double tmatrix[][], double pointPtr[][])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            pointPtr[i][0] = pt[i][0] + tmatrix[i][0];

        }
    }

    /**
     *
     * @param pt    set of points
     * @param tmatrix transformation matrix
     */
    public static void translate(double pt[], double tmatrix[])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            pt[i] = pt[i] + tmatrix[i];
        }
    }

    static void ReverseTranslate(double pt[][], double anti_tmatrix[][], double pointPtr[][])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            pointPtr[i][0] = pt[i][0] + anti_tmatrix[i][0];

        }
    }

    static void formTrans(double pta[][], double ptb[][], double nPtr[][]) /* this function forms the translation matrix */
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            nPtr[i][0] = pta[i][0] - ptb[i][0];

        }
    }

    static void formReverseTrans(double pta[][], double ptb[][], double nPtr[][]) /* this function forms the translation matrix */
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            nPtr[i][0] = pta[i][0] + ptb[i][0];

        }
    }

    static double dotProduct(double u[][], double v[][])
    {
        double result;
        result = (u[0][0] * v[0][0] + u[1][0] * v[1][0] + u[2][0] * v[2][0]);
        return result;
    }


    static double costheta(double v1[][], double v2[][])
    {
        double product = dotProduct(v1, v2);
        double norm1 = normalize(v1);
        double norm2 = normalize(v2);
        return product / (norm1 * norm2);
    }


    static double sintheta(double v1[][], double v2[][])
    {
        double cos_theta = costheta(v1, v2);
        return Math.sqrt(1.00 - Math.pow(cos_theta, 2));
    }

    static double alpha(double pt[][])
    {
        if (pt[0][0] == 0.0 && pt[1][0] == 0.0)
            return (0.00);
        else
        {
            double tan_alpha = pt[1][0] / pt[0][0];
            double alpha_rad = Math.atan(tan_alpha);
            return (alpha_rad);
        }
    }

    static double beta(double pt[][])
    {
        if (pt[1][0] == 0.0 && pt[2][0] == 0.0)
            return (0.00);
        else
        {
            double tan_beta = pt[2][0] / pt[1][0];
            double beta_rad = Math.atan(tan_beta);
            return (beta_rad);
        }
    }


    static void form3by1(double pt[][], double matrixpt[][])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            matrixpt[i][0] = pt[i][0];
        }
    }

    static void form1by3(double matrixpt[][], double pt[][])
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            pt[i][0] = matrixpt[i][0];
        }
    }


    private static void printPoint(double pt[][], int number) /* this function prints a point or a vector */
    {
        int i;
        for (i = 0; i < number; i++)
        {
            System.out.printf("%f", pt[i][0]);
            System.out.printf("\t");
        }
    }


    private static void print3by1(double matrix[][])  /*  this function prints a 3x1 matrix */
    {
        int i;
        for (i = 0; i < 3; i++)
        {
            System.out.printf("%f", matrix[i][0]);
            System.out.printf("\n");
        }
    }


    private static void print3by3(double matrix[][])  /*  this function prints a 3x3 matrix */
    {
        int i, j;
        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 3; j++)
            {
                System.out.printf("%f\t", matrix[i][j]);
            }
            System.out.printf("\n");
        }
    }


    static void Multiply(double pt[][], double matrix[][], double newpt[][], int row, int column1, int column2)
    {
        int i, j, k;

        for (i = 0; i < row; i++)
        {
            for (j = 0; j < column2; j++)
            {
                newpt[i][j] = 0;
                for (k = 0; k < column1; k++)
                    newpt[i][j] = newpt[i][j] + matrix[i][k] * pt[k][j];
            }
        }

    }

    static void Rot_X_clock(double vect[][], double xt[][])
    {
        double angle = beta(vect);

        double[][] x = {{1, 0, 0},
                {0, Math.cos(angle), Math.sin(angle)},
                {0, -Math.sin(angle), Math.cos(angle)}};


        int i, j;
        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 3; j++)
            {
                xt[i][j] = x[i][j];
            }
        }

    }


    static void Rot_X_anticlock(double vect[][], double xt[][])
    {
        double angle = beta(vect);

        double[][] x = {{1, 0, 0},
                {0, Math.cos(angle), -Math.sin(angle)},
                {0, Math.sin(angle), Math.cos(angle)}};

        int i, j;
        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 3; j++)
            {
                xt[i][j] = x[i][j];
            }
        }

    }


    static void Rot_Z_clock(double vect[][], double zt[][])
    {
        double angle = alpha(vect);

        double[][] z = {{Math.cos(angle), Math.sin(angle), 0},
                {-Math.sin(angle), Math.cos(angle), 0},
                {0, 0, 1}};

        int i, j;
        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 3; j++)
            {
                zt[i][j] = z[i][j];
            }
        }

    }

    static void Rot_Z_anticlock(double vect[][], double zt[][])
    {
        double angle = alpha(vect);

        double[][] z = {{Math.cos(angle), -Math.sin(angle), 0},
                {Math.sin(angle), Math.cos(angle), 0},
                {0, 0, 1}};


        int i, j;
        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 3; j++)
            {
                zt[i][j] = z[i][j];
            }
        }

    }

    /***********************************************************************************************/
    /*****************                     MAIN FUNCTION               *****************************/
    /**
     * *******************************************************************************************
     */


    /**
     *
     * @param abc_values a 3x3 matrix of the original coordinates (3 rows, each with the xyz
     * values of the representative atoms)
     * @param pqr_values a 3x3 matrix of the new coordinates (3 rows, each with the xyz
     * values of the representative atoms)
     * @param x coordinate to move
     * @param y coordinate to move
     * @param z coordinate to move
     * @return a 3x1 matrix denoting the new transformed coordinates
     */
    public static double[] superpose(double abc_values[][], double pqr_values[][], double x, double y, double z)
    {
        assert abc_values.length == 3;
        assert pqr_values.length == 3;

        double[][] origin = {{0}, {0}, {0}};
        double[][] a = new double[3][1];
        double[][] b = new double[3][1];
        double[][] c = new double[3][1];
        double[][] at = new double[3][1];
        double[][] bt = new double[3][1];
        double[][] ct = new double[3][1];
        double[][] p = new double[3][1];
        double[][] q = new double[3][1];
        double[][] r = new double[3][1];
        double[][] point;
        double[][] pt = new double[3][1];
        double[][] qt = new double[3][1];
        double[][] rt = new double[3][1];
        double[][] pointt = new double[3][1];
        double[][] pRevt = new double[3][1];
        double[][] qRevt = new double[3][1];
        double[][] rRevt = new double[3][1];

        double[][] v1 = new double[3][1];
        double[][] v2 = new double[3][1];
        double[][] v3 = new double[3][1];
        double[][] v4 = new double[3][1];
        double[][] v1t = new double[3][1];
        double[][] v2t = new double[3][1];
        double[][] v3t = new double[3][1];
        double[][] v4t = new double[3][1];
        double[][] v1t_3x1 = new double[3][1];
        double[][] v2t_3x1 = new double[3][1];
        double[][] v3t_3x1 = new double[3][1];
        double[][] v4t_3x1 = new double[3][1];
        double[][] v1t_1x3 = new double[3][1];
        double[][] v2t_1x3 = new double[3][1];
        double[][] v3t_1x3 = new double[3][1];
        double[][] v4t_1x3 = new double[3][1];
        double[][] v1t_Rx = new double[3][1];
        double[][] v2t_Rx = new double[3][1];
        double[][] v3t_Rx = new double[3][1];
        double[][] v4t_Rx = new double[3][1];
        double[][] v3t_newantiRx = new double[3][1];
        double[][] v4t_newantiRx = new double[3][1];
        double[][] v1t_Rz = new double[3][1];
        double[][] v2t_Rz = new double[3][1];
        double[][] v3t_Rz = new double[3][1];
        double[][] v4t_Rz = new double[3][1];
        double[][] v3t_newantiRz = new double[3][1];
        double[][] v4t_newantiRz = new double[3][1];
        double[][] v2t_TwiceRx = new double[3][1];
        double[][] v3t_newantiRx_1x3 = new double[3][1];
        double[][] v4t_TwiceRx = new double[3][1];
        double[][] v2t_final = new double[3][1];
        double[][] v3t_new = new double[3][1];
        double[][] v4t_final = new double[3][1];
        double[][] v4t_newantiTwiceRx = new double[3][1];
        double[][] v4t_newantiRx_1x3 = new double[3][1];
        double[][] v4t_finalrotated_3x1 = new double[3][1];
        double[][] v5 = new double[3][1];
        double[][] v5t = new double[3][1];

        double[][] v5t_3x1 = new double[3][1];

        double[][] v5t_1x3 = new double[3][1];
        double[][] v5t_Rx = new double[3][1];
        double[][] v5t_newantiRx = new double[3][1];
        double[][] v5t_Rz = new double[3][1];
        double[][] v5t_newantiRz = new double[3][1];
        double[][] v5t_TwiceRx = new double[3][1];
        double[][] v5t_final = new double[3][1];
        double[][] v5t_newantiTwiceRx = new double[3][1];
        double[][] v5t_newantiRx_1x3 = new double[3][1];

        double[][] v5t_finalrotated_3x1 = new double[3][1];


        double[][] T_Matrix1 = new double[3][1];
        double[][] T_Matrix2 = new double[3][1];
        double[][] Reverse_T_Matrix1 = new double[3][1];

        double[][] Rx_clock1 = new double[3][3];
        double[][] Rx_clock2 = new double[3][3];
        double[][] Rx_anticlock1 = new double[3][3];
        double[][] Rz_clock1 = new double[3][3];
        double[][] Rz_clock2 = new double[3][3];
        double[][] Rz_anticlock1 = new double[3][3];
        double[][] Rx_finalclock1 = new double[3][3];
        double[][] Rx_finalclock2 = new double[3][3];
        double[][] Rx_finalanticlock1 = new double[3][3];

        double[][] Rot_Z_180clock = {{-1.00, 0.00, 0.00},
                {-0.00, -1.00, 0.00},
                {0.00, 0.00, 1.00}};


        double[][] Rot_X_180clock = {{1.00, 0.00, 0.00},
                {0.00, -1.00, 0.00},
                {0.00, 0.00, -1.00}};


        double[][] v3t_3x1_before = new double[3][1];
        double[][] v3t_3x1_after = new double[3][1];
        double[][] v3t_1x3_after = new double[3][1];
        double[][] v4t_final_before = new double[3][1];
        double[][] v5t_final_before = new double[3][1];
        double[][] v4t_final_after = new double[3][1];
        double[][] v5t_final_after = new double[3][1];
        double[][] v4t_final_after_1x3 = new double[3][1];
        double[][] v5t_final_after_1x3 = new double[3][1];
        double[][]  v4t_final_after_180 = new double[3][1];
        double[][]  v5t_final_after_180 = new double[3][1];


        int i;

        /*             COPYING THE VALUES OF POINTS A,B C, P,Q AND R   ******/
        for (i = 0; i < 3; i++)
        {
            a[i][0] = abc_values[0][i];
            b[i][0] = abc_values[1][i];
            c[i][0] = abc_values[2][i];
        }

        for (i = 0; i < 3; i++)
        {
            p[i][0] = pqr_values[0][i];
            q[i][0] = pqr_values[1][i];
            r[i][0] = pqr_values[2][i];
        }

        /************************************************************************************/




        vector(a, b, v1);
        vector(a, c, v2);     /* forming the vectors ab and ac */

        formTrans(origin, a, T_Matrix1);         /* forming the translation matrix between a and the origin */
        formReverseTrans(origin, a, Reverse_T_Matrix1);


        translate(a, T_Matrix1, at);    /*  translating the three points, a b and c */
        translate(b, T_Matrix1, bt);
        translate(c, T_Matrix1, ct);

        vector(at, bt, v1t);
        vector(at, ct, v2t);     /* forming vectors with the translated points */
        form3by1(v1t, v1t_3x1);
        form3by1(v2t, v2t_3x1);   /* changing the vectors into 3 by 1 matrices  */

        /*************************** RX FOR FIRST VECTOR*******************************************************/
        Rot_X_clock(v1t, Rx_clock1);          /* rotation matrix Rx(clockwise) for translated vector v1 */
        Rot_X_anticlock(v1t, Rx_anticlock1);  /* rotation matrix Rx(anticlockwise) for translated vector v1 */

        Multiply(v1t_3x1, Rx_clock1, v1t_Rx, 3, 3, 1);    /*multiply matrix v1t with Rx  */

        /************************** RZ FOR FIRST VECTOR*******************************************************/
        Rot_Z_clock(v1t_Rx, Rz_clock1);          /*  rotation matrix Rz(clockwise) for translated vector v1  */
        Rot_Z_anticlock(v1t_Rx, Rz_anticlock1);  /* rotation matrix Rz(anticlockwise) for translated vector v1 */

        Multiply(v1t_Rx, Rz_clock1, v1t_Rz, 3, 3, 1);   /* multipling with Rz after v1t has been multiplied with Rx */

        form1by3(v1t_Rz, v1t_1x3);
/********************PRINTING POINTS***********************************/
        /* System.out.printf("printing v1 before the if block\n");
      printPoint(v1t_1x3,3);System.out.printf("\n"); */

        /**************************************************************************************************************************/
        /****** PERFORMING ROTATIONS ON TRANSLATED VECTOR V2T   ******/

        Multiply(v2t_3x1, Rx_clock1, v2t_Rx, 3, 3, 1);  /* multiplying matrix v2t with Rx */
        Multiply(v2t_Rx, Rz_clock1, v2t_Rz, 3, 3, 1);  /* multiplying matrix v2t with Rz */
        form1by3(v2t_Rz, v2t_1x3);   /* forming a vector after both the multiplications are done */

        Rot_X_clock(v2t_1x3, Rx_finalclock1);    /* Rx_finalclock1 is the rotation matrix for vector v2 to bring it onto the xy plane*/
        Rot_X_anticlock(v2t_1x3, Rx_finalanticlock1);

        Multiply(v2t_Rz, Rx_finalclock1, v2t_TwiceRx, 3, 3, 1);
        form1by3(v2t_TwiceRx, v2t_final);      /* v2t_final lies on the xy plane and is the final form of vector v2  */
        /********************PRINTING POINTS***********************************/
        /* System.out.printf("printing v2 before the if block\n");
      printPoint(v2t_final,3);System.out.printf("\n"); */

/***************************************************************************************************************************/
/******************************************** SECOND SET OF POINTS**********************************************************/
/***********************************************THIRD VECTOR****************************************************************/

        vector(p, q, v3);
        vector(p, r, v4);     /* forming the vectors pq and pr  */


        formTrans(origin, p, T_Matrix2);   /* forming the translation matrix between p and the origin */

        translate(p, T_Matrix2, pt);    /* translating the three points, p b and c */
        translate(q, T_Matrix2, qt);
        translate(r, T_Matrix2, rt);


        vector(pt, qt, v3t);
        vector(pt, rt, v4t);         /* forming vectors with the translated points */

        form3by1(v3t, v3t_3x1);
        form3by1(v4t, v4t_3x1);   /*changing the vectors into 3 by 1 matrices */

        /*************************** RX FOR THIRD VECTOR*******************************************************/
        Rot_X_clock(v3t, Rx_clock2);              /*rotation matrix Rx(clockwise) for translated vector v3 */
        Multiply(v3t_3x1, Rx_clock2, v3t_Rx, 3, 3, 1);    /*multiply matrix v3t with Rx */

        /******************** RZ FOR THIRD VECTOR***************************/
        Rot_Z_clock(v3t_Rx, Rz_clock2);                 /* rotation matrix Rz(clockwise) for translated vector v3 */

        Multiply(v3t_Rx, Rz_clock2, v3t_Rz, 3, 3, 1);    /*multipling with Rz after v3t has been multiplied with Rx */

        form1by3(v3t_Rz, v3t_1x3);
        form3by1(v3t_1x3, v3t_3x1_before);
        /* System.out.printf("printing v3 before the if block\n");
      printPoint(v3t_1x3,3);System.out.printf("\n"); */
        /**************** FOURTH VECTOR***************************/
        /********************** PERFORMING ROTATIONS ON TRANSLATED VECTOR V4T   *******************/

        Multiply(v4t_3x1, Rx_clock2, v4t_Rx, 3, 3, 1);  /* multiplying matrix v4t with Rx */

        Multiply(v4t_Rx, Rz_clock2, v4t_Rz, 3, 3, 1);  /* multiplying Rz with v4t after it has been multiplied by Rx */
        form1by3(v4t_Rz, v4t_1x3);           /* forming a vector after both the multiplications are done*/


        Rot_X_clock(v4t_1x3, Rx_finalclock2);    /* Rx_finalclock2 is the rotation matrix for vector v4 to bring it onto the xy plane*/


        Multiply(v4t_Rz, Rx_finalclock2, v4t_TwiceRx, 3, 3, 1);


        form1by3(v4t_TwiceRx, v4t_final);      /* v4t_final lies on the xy plane and is the final form of vector v4 */
        form3by1(v4t_final, v4t_final_before);
        /* System.out.printf("printing v4 before the if block\n");
      printPoint(v4t_final,3);System.out.printf("\n");   */




        if ((v1t_1x3[0][0] * v3t_1x3[0][0]) < 0.000 && (v2t_final[0][0] * v4t_final[0][0]) < 0.000)
        {
            Multiply(v3t_3x1_before, Rot_Z_180clock, v3t_3x1_after, 3, 3, 1);
            Multiply(v4t_final_before, Rot_Z_180clock, v4t_final_after, 3, 3, 1);


            if ((v1t_1x3[0][0] * v3t_3x1_after[0][0]) > 0.00 && (v2t_final[1][0] * v4t_final_after[1][0]) < 0.00)
            {

                Multiply(v4t_final_after, Rot_X_180clock, v4t_final_after_180, 3, 3, 1);
                /*v4t_final_after_180[0][0] = v4t_final_after[0][0];
                                v4t_final_after_180[1][0] = - v4t_final_after[1][0];
                          v4t_final_after_180[2][0] = v4t_final_after[2][0]; */

                form1by3(v3t_3x1_after, v3t_1x3_after);
                form1by3(v4t_final_after_180, v4t_final_after_1x3);
                /*  System.out.printf("printing v3 and v4 in the second if block\n");
            printPoint(v3t_1x3_after,3);System.out.printf("\n");  printPoint(v4t_final_after_1x3,3);System.out.printf("\n");  */
            }
            else
            {
                form1by3(v3t_3x1_after, v3t_1x3_after);
                form1by3(v4t_final_after, v4t_final_after_1x3);
                /*    System.out.printf("printing v3 and v4 in the first else block\n");
             printPoint(v3t_1x3_after,3);System.out.printf("\n");  printPoint(v4t_final_after_1x3,3);System.out.printf("\n"); */
            }
        }
        else       /***  second else if block  ***/
            if ((v1t_1x3[0][0] * v3t_1x3[0][0]) < 0.00 && (v2t_final[1][0] * v4t_final[1][0]) < 0.00)
            {
                Multiply(v3t_3x1_before, Rot_Z_180clock, v3t_3x1_after, 3, 3, 1);
                Multiply(v4t_final_before, Rot_Z_180clock, v4t_final_after, 3, 3, 1);
                form1by3(v3t_3x1_after, v3t_1x3_after);
                form1by3(v4t_final_after, v4t_final_after_1x3);

                /*   System.out.printf(" printing v3 and v4 after the flip\n");
                 printPoint(v3t_1x3_after,3);System.out.printf("\n");
                printPoint(v4t_final_after_1x3,3);System.out.printf("\n"); */
            }
            /** third if block **/
            else if ((v1t_1x3[0][0] * v3t_1x3[0][0]) > 0.00 && (v2t_final[1][0] * v4t_final[1][0]) < 0.00)
            {

                Multiply(v4t_final_before, Rot_X_180clock, v4t_final_after, 3, 3, 1);
                /* v4t_final_after[0][0] = v4t_final[0][0];
              v4t_final_after[1][0] = - v4t_final[1][0];
              v4t_final_after[2][0] = v4t_final[2][0];  */
                form1by3(v3t_1x3, v3t_1x3_after);
                form1by3(v4t_final_after, v4t_final_after_1x3);
                /* System.out.printf("printing v3 and v4 in the third if block\n");
             printPoint(v3t_1x3_after,3);System.out.printf("\n");
             printPoint(v4t_final_after_1x3,3);System.out.printf("\n"); */
            }

            /** fourth if block **/
            else
            if ((v1t_1x3[0][0] * v3t_1x3[0][0]) > 0.00 && (v2t_final[0][0] * v4t_final[0][0]) < 0.00 && (v2t_final[1][0] * v4t_final[1][0]) < 0.00)
            {

                Multiply(v4t_final_before, Rot_X_180clock, v4t_final_after, 3, 3, 1);
                /* v4t_final_after[0][0] = v4t_final[0][0];
              v4t_final_after[1][0] = - v4t_final[1][0];
              v4t_final_after[2][0] = v4t_final[2][0]; */
                form1by3(v3t_1x3, v3t_1x3_after);
                form1by3(v4t_final_after, v4t_final_after_1x3);
                /* System.out.printf("printing v3 and v4 in the fourth if block\n");
             printPoint(v3t_1x3_after,3);System.out.printf("\n");
             printPoint(v4t_final_after_1x3,3);System.out.printf("\n"); */
            }

            /** fifth if block **/
            else if ((v1t_1x3[0][0] * v3t_1x3[0][0]) < 0.00 && (v2t_final[1][0] * v4t_final[1][0]) > 0.00)
            {
                Multiply(v3t_3x1_before, Rot_Z_180clock, v3t_3x1_after, 3, 3, 1);
                Multiply(v4t_final_before, Rot_Z_180clock, v4t_final_after, 3, 3, 1);
                form1by3(v3t_3x1_after, v3t_1x3_after);
                form1by3(v4t_final_after, v4t_final_after_1x3);
                /*    System.out.printf("printing v3 and v4 in the fifth if block\n");
           printPoint(v3t_1x3_after,3);System.out.printf("\n");  printPoint(v4t_final_after_1x3,3);System.out.printf("\n");   */
            }
            else
            {
                form1by3(v3t_3x1_before, v3t_1x3_after);
                form1by3(v4t_final_before, v4t_final_after_1x3);
                /*  System.out.printf("printing v3 and v4 in the last else block\n");
              printPoint(v3t_1x3_after,3);System.out.printf("\n");  printPoint(v4t_final_after_1x3,3);System.out.printf("\n");*/
            }

        /************************************** PROCESMath.sinG POINT  P  *******************************************************/
        ReverseTranslate(pt, Reverse_T_Matrix1, pRevt);

        /************************************** PROCESMath.sinG POINT Q    ****************************************************/
        form3by1(v3t_1x3_after, v3t_new);
        Multiply(v3t_new, Rz_anticlock1, v3t_newantiRz, 3, 3, 1);

        Multiply(v3t_newantiRz, Rx_anticlock1, v3t_newantiRx, 3, 3, 1);
        form1by3(v3t_newantiRx, v3t_newantiRx_1x3);

        ReverseTranslate(v3t_newantiRx_1x3, Reverse_T_Matrix1, qRevt);

        /******************************* PROCESMath.sinG POINT R **************************************/
        form3by1(v4t_final_after_1x3, v4t_finalrotated_3x1);
        Multiply(v4t_finalrotated_3x1, Rx_finalanticlock1, v4t_newantiTwiceRx, 3, 3, 1); /* MULTIPLYING POINT R BY THE antiRX MATRIX */
        Multiply(v4t_newantiTwiceRx, Rz_anticlock1, v4t_newantiRz, 3, 3, 1);     /* MULTIPLYING POINT R BY THE antiRZ MATRIX */

        Multiply(v4t_newantiRz, Rx_anticlock1, v4t_newantiRx, 3, 3, 1);         /*THE THIRD ROTATION IS ANTI CLOCKWISE ABOUT THE X AXIS  */
        form1by3(v4t_newantiRx, v4t_newantiRx_1x3);

        ReverseTranslate(v4t_newantiRx_1x3, Reverse_T_Matrix1, rRevt); /*THE FINAL OPERATION IS TRANSLATION BY THE TRANSLATION MATRIX */

        /************** STEPS TO MODIFY ANY ARBITRARY POINT ******************/
        /******************* CODE TO OPEN A FILE,MODIFY EACH POINT AND WRITE THE MODIFIED VALUES TO ANOTHER FILE ********************/






        point = new double[][] { {x}, {y}, {z} };

        vector(p, point, v5);

        translate(point, T_Matrix2, pointt);
        vector(pt, pointt, v5t);
        form3by1(v5t, v5t_3x1);

        Multiply(v5t_3x1, Rx_clock2, v5t_Rx, 3, 3, 1);  /* multiplying matrix v5t with Rx */

        Multiply(v5t_Rx, Rz_clock2, v5t_Rz, 3, 3, 1);  /* multiplying Rz with v5t after it has been multiplied by Rx */

        form1by3(v5t_Rz, v5t_1x3);           /* forming a vector after both the multiplications are done*/

        Multiply(v5t_Rz, Rx_finalclock2, v5t_TwiceRx, 3, 3, 1);

        form1by3(v5t_TwiceRx, v5t_final);      /* v5t_final lies on the xy plane and is the final form of vector v5t */
        form3by1(v5t_final, v5t_final_before);

        /* CHANGING THE PROCEDURE FOR V5 VALUES........ */

        if ((v3t_1x3[0][0] * v3t_1x3_after[0][0]) < 0.00 && (v4t_final[0][0] * v4t_final_after_1x3[0][0]) < 0.00 && (v4t_final[1][0] * v4t_final_after_1x3[1][0]) < 0.00)
        {
            Multiply(v5t_final_before, Rot_Z_180clock, v5t_final_after, 3, 3, 1);
            form1by3(v5t_final_after, v5t_final_after_1x3);

        }

        else
        if ((v3t_1x3[0][0] * v3t_1x3_after[0][0]) < 0.00 && (v4t_final[0][0] * v4t_final_after_1x3[0][0]) < 0.00 && (v4t_final[1][0] * v4t_final_after_1x3[1][0]) > 0.00)
        {
            Multiply(v5t_final_before, Rot_Z_180clock, v5t_final_after, 3, 3, 1);
            Multiply(v5t_final_after, Rot_X_180clock, v5t_final_after_180, 3, 3, 1);

            /*v5t_final_after_180[0][0] = -v5t_final_before[0][0];
                        v5t_final_after_180[1][0] =  v5t_final_before[1][0];
                  v5t_final_after_180[2][0] = -v5t_final_before[2][0]; */

            form1by3(v5t_final_after_180, v5t_final_after_1x3);

        }


        else
        if ((v3t_1x3[0][0] * v3t_1x3_after[0][0]) > 0.00 && (v4t_final[1][0] * v4t_final_after_1x3[1][0]) < 0.00)
        {

            Multiply(v5t_final_before, Rot_X_180clock, v5t_final_after_180, 3, 3, 1);
            /* v5t_final_after_180[0][0] =  v5t_final_before[0][0];
         v5t_final_after_180[1][0] = -v5t_final_before[1][0];
     v5t_final_after_180[2][0] = -v5t_final_before[2][0];  */

            form1by3(v5t_final_after_180, v5t_final_after_1x3);


        }

        else
        {
            form1by3(v5t_final_before, v5t_final_after_1x3);

        }


        form3by1(v5t_final_after_1x3, v5t_finalrotated_3x1);

        Multiply(v5t_finalrotated_3x1, Rx_finalanticlock1, v5t_newantiTwiceRx, 3, 3, 1);/*MULTIPLYING ARBITRARY POINT BY THE RX MATRIX */

        Multiply(v5t_newantiTwiceRx, Rz_anticlock1, v5t_newantiRz, 3, 3, 1); /*MULTIPLYING ARBITRARY POINT  BY THE RZ MATRIX  */

        Multiply(v5t_newantiRz, Rx_anticlock1, v5t_newantiRx, 3, 3, 1);  /* THE THIRD ROTATION IS ANTI CLOCKWISE ABOUT THE X AXIS */
        form1by3(v5t_newantiRx, v5t_newantiRx_1x3);


        double[][] pointRevt = new double[3][1];

        ReverseTranslate(v5t_newantiRx_1x3, Reverse_T_Matrix1, pointRevt);/*THE FINAL OPERATION IS TRANSLATION BY THE TRANSLATION MATRIX */
        /* System.out.printf("Printing the final values\n");
     printPoint(pointRevt,3);System.out.printf("\n");*/

        /*	fSystem.out.printf (outPtr,"%-3s %-3s %-3s",atom,acid,residueNum);*/

      /*  for (j = 0; j < 3; j++)
        {
            System.out.printf("%7.3f ", pointRevt[j][0]);
            if (j == 2)
            {
                System.out.printf("\n");
            }
        }*/

        return new double[] {pointRevt[0][0], pointRevt[1][0], pointRevt[2][0]};
        //filecounter = filecounter + 1;


    }

    /**
     * This version takes a list of original and target atoms, and a chain.
     * <br/>
     * It will find the rotation matrix between the original and target atoms, and then
     * apply it to the chain.
     * <br/>
     * The two lists (original and target) must contain the same atoms (i.e. both contain
     * two sets of C, CA, and N atoms).
     *
     * <br>
     * I think this came from
     * <a href="http://ccp14.sims.nrc.ca/ccp/ccp14/ftp-mirror/llnlrupp/pdbsup/pdbsup.for">here</a>
     * <br>
     * <code>
     *
     * Determines rotation matrix and translation vector for
              best fit superimposition of two pdb files by solving
                      the quaternion eigenvalue problem.
                      Code by B.Rupp and S.Parkin (1996)
               Lawrence Livermore National Laboratory, BBRP
             Method by S.K.Kearsley, Acta Cryst. A45, 208 (1989)
     * </code>
     *
     * @param original Group of atoms from the chain to be modified.  Typically these are the N, CA, C atoms
     * from a residue flanking a coil.  i.e. <br>
     * <pre>
     * ABCDE
     * HCCCH
     * <pre>
     * You would pass in the N, CA, C atoms from residues A and E
     * @param target Group of atoms from the residues flanking a gap.
     * i.e. <br>
     * A---B   <br>
     * Pass in A and B
     * @param chain the chain to be modified (typically a loop)
     */
    public static void superpose(Atom[] original, Atom[] target, Chain chain)
    {
        assert original.length == target.length : "Original and target lists must contain the same atoms";
       // assert original.length == 6 : "atoms lists must be of length 6"; // TODO for now

        /* Initialize: */
        int i, j, k;
        int n_sub_one, ns_sub_one;
        final int NUM_ATOMS = original.length;
        final int MAX_ATOMS_SUP = original.length + 1;

        //char *anchor_and_sup_file;
        //char *transformed_file,*anchor_file;
       // int nmrot;
        // FILE *target_fptr, *anchor_and_sup_fptr;
        //FILE *trans_fptr,*anchor_fptr;
        //LIST *RESD_LIST1, *RESD_LIST2; /* Expanded list to store the residues */

       // int mov_all;
        //int mov_selected, target_selected;

        /*------------------------------------------------------
        * q = Quaternion matrix                               *
        * xf stores the target file coordinates               *
        * xm stores the moving file coordinates               *
        *-----------------------------------------------------*/

        double dm[] = new double[5], vm[][] = new double[5][5];
        double t[][] = new double[4][4], q[][] = new double[5][5];  /* tr = a vector from cm ---> cf  */

        double dxp[][] = new double[MAX_ATOMS_SUP][4], dxm[][] = new double[MAX_ATOMS_SUP][4];

        //double allAtomCoordinates[][] = new double[MAX_ATOMS_SUP][4];   // list of atoms x coordinates
        //double allAtomCenterOfMass[] = new double[4];                   // coordinates for center of Mass

        double originalRepresentativeAtomCoordinates[][] = new double[MAX_ATOMS_SUP][4]; // list of atoms x coordinates
        double originalCoordinatesCenterOfMass[] = new double[4];  // coordinates for center of Mass
        double targetRepresentativeAtomCoordinates[][] = new double[MAX_ATOMS_SUP][4];  // list of atoms x coordinates
        double targetCoordinateCenterOfMass[] = new double[4];  // coordinates for center of Mass

        double sumf[] = new double[4], summ[] = new double[4], fill_value;
     //   double rot_xm_selected[][] = new double[MAX_ATOMS_SUP][4];

        double sum_x = 0, sum_y = 0, sum_z = 0;
        for (int i1 = 0; i1 < original.length; i1++)
        {
            // ALL of this is 1-based
            Atom a = original[i1];
            originalRepresentativeAtomCoordinates[i1 + 1][1] = a.getX();
            originalRepresentativeAtomCoordinates[i1 + 1][2] = a.getY();
            originalRepresentativeAtomCoordinates[i1 + 1][3] = a.getZ();

            sum_x += a.getX(); sum_y += a.getY(); sum_z += a.getZ();
        }

        originalCoordinatesCenterOfMass[1] = sum_x/original.length;
        originalCoordinatesCenterOfMass[2] = sum_y/original.length;
        originalCoordinatesCenterOfMass[3] = sum_z/original.length;

        sum_x = 0; sum_y = 0; sum_z = 0;
        for (int i1 = 0; i1 < target.length; i1++)
        {
            Atom a = target[i1];
            targetRepresentativeAtomCoordinates[i1 + 1][1] = a.getX();
            targetRepresentativeAtomCoordinates[i1 + 1][2] = a.getY();
            targetRepresentativeAtomCoordinates[i1 + 1][3] = a.getZ();

            sum_x += a.getX(); sum_y += a.getY(); sum_z += a.getZ();
        }

        targetCoordinateCenterOfMass[1] = sum_x/original.length;
        targetCoordinateCenterOfMass[2] = sum_y/original.length;
        targetCoordinateCenterOfMass[3] = sum_z/original.length;



        /*-- INITIALIZE  --*/
        n_sub_one = ns_sub_one = 4;
        fill_value = 0.00;

        sumf[0] = 0.00;
        summ[0] = 0.00;
        for (i = 1; i <= 3; i++)
        {
            sumf[i] = 0.00;
            summ[i] = 0.00;
        }

        /* Fill Matrix q with zeroes */
        /*-----------------------------------------------*/
        for (i = 0; i < 5; i++)
        {
            for (j = 0; j < 5; j++)
            {
                q[i][j] = 0.00;
            }
        }

        /* Initialize dm with zeroes  */
        /*-----------------------------------------------*/
        for (i = 0; i < 5; i++)
        {
            dm[i] = 0.00;
        }

      /*  moving_file = fname;
        target_file = sname;

        list1_beg_resdno = list1[0].atomno;
        list1_end_resdno = list1[list_size - 1].atomno;
        list2_beg_resdno = list2[0].atomno;
        list2_end_resdno = list2[list_size - 1].atomno;
*/
/* Storing all the co-ordinates and the centre of the moving file and target file *
*  This centre is based on all the points                                        *
* mov_all stores all the atoms of the pdb file that is being rotated
* target_all stores all the atoms of the pdb file that is not being moved.
*--------------------------------------------------------------------------------*/
//        mov_all = get_all_coords_and_centre(moving_file, allAtomCoordinates, allAtomCenterOfMass, line_mov, mov_end_portion);

        /* Storing the co-ordinates of the bbone atoms of the chosen residues *
       *  The centre of mass (cm/cf)selected are based on the selected atoms
       * ---------------------------------------------------------------------*/
/*
        mov_selected = get_selected_coords_and_centre(moving_file, originalRepresentativeAtomCoordinates, originalCoordinatesCenterOfMass, junk, list1, list_size);
        target_selected = get_selected_coords_and_centre(target_file, targetRepresentativeAtomCoordinates, targetCoordinateCenterOfMass, junk, list2, list_size);
        num_atoms = Math.min(mov_selected, target_selected);

*/
/*
        if (mov_selected == 0)
        {
            System.out.println("Cannot find the requested atoms in the first protein.. Exiting\n");
            return null;
        }
        if (target_selected == 0)
        {
            System.out.println("Cannot find the requested atoms in the sec protein.. Exiting\n");
            return null;
        }*/

        /*-------Creating co-ordinate differences--------*
  *-------delta x plus (dxp) and minus (dxm)------*/

        dxm[0][0] = dxm[0][1] = dxm[0][2] = dxm[0][3] = 0.00;
        dxp[0][0] = dxp[0][1] = dxp[0][2] = dxp[0][3] = 0.00;


        for (k = 1; k <= NUM_ATOMS; k++)
        {
            for (j = 1; j <= 3; j++)
            {

                dxm[k][j] = originalRepresentativeAtomCoordinates[k][j] - originalCoordinatesCenterOfMass[j] - (targetRepresentativeAtomCoordinates[k][j] - targetCoordinateCenterOfMass[j]);
                dxp[k][j] = originalRepresentativeAtomCoordinates[k][j] - originalCoordinatesCenterOfMass[j] + (targetRepresentativeAtomCoordinates[k][j] - targetCoordinateCenterOfMass[j]);

                dxm[k][0] = 0.00;
                dxp[k][0] = 0.00;
            }

        }

        /******************************************************************************************/

        //printf("NUM_ATOMS = %d\n\n",NUM_ATOMS);
        //exit(-1);
        /*-----------------------------------------------------------------*
*-----  Fill Upper Triangle of (symmetric) quaternion Matrix -----*/

        for (k = 1; k <= NUM_ATOMS; k++)
        {
            /* diags are sums of squared cyclic coordinate differences        PDBS0160 */
            /*-------------------------------------------------------------------------*/
            q[1][1] = q[1][1] + Math.pow(dxm[k][1], 2) + Math.pow(dxm[k][2], 2) + Math.pow(dxm[k][3], 2);
            q[2][2] = q[2][2] + Math.pow(dxp[k][2], 2) + Math.pow(dxp[k][3], 2) + Math.pow(dxm[k][1], 2);
            q[3][3] = q[3][3] + Math.pow(dxp[k][1], 2) + Math.pow(dxp[k][3], 2) + Math.pow(dxm[k][2], 2);
            q[4][4] = q[4][4] + Math.pow(dxp[k][1], 2) + Math.pow(dxp[k][2], 2) + Math.pow(dxm[k][3], 2);

            /* cross differences                                              PDBS0165  */
            /*--------------------------------------------------------------------------*/
            q[1][2] = q[1][2] + dxp[k][2] * dxm[k][3] - dxm[k][2] * dxp[k][3];
            q[1][3] = q[1][3] + dxm[k][1] * dxp[k][3] - dxp[k][1] * dxm[k][3];
            q[1][4] = q[1][4] + dxp[k][1] * dxm[k][2] - dxm[k][1] * dxp[k][2];

            q[2][3] = q[2][3] + dxm[k][1] * dxm[k][2] - dxp[k][1] * dxp[k][2];
            q[2][4] = q[2][4] + dxm[k][1] * dxm[k][3] - dxp[k][1] * dxp[k][3];
            q[3][4] = q[3][4] + dxm[k][2] * dxm[k][3] - dxp[k][2] * dxp[k][3];

        }

        /*--------------fill the rest by transposing it onto itself ----PDBS0173-------*/
        transpose_matrix(q, q, 5);

        /*- orthogonalization by jacobi rotation = solution of EV -problem -- PDBS0181 */
        jacobi(q, n_sub_one, ns_sub_one, dm, vm);

        /* --- sort eigenvectors after eigenvalues, descending --           PDBS0186  */
        eigsrt(dm, vm, n_sub_one, ns_sub_one);



        if (DEBUG_SUPERPOSE)
        {
            System.out.printf("\nThe eigenvalues are as follows:\n");
            System.out.printf("--------------------------------------------------------\n");
            for (i = 1; i <= 4; i++)
            {
                System.out.printf("dm[%d]= %10.3f  ", i, dm[i]);
            }
            System.out.printf("\n\n");

            System.out.printf("The eigenvectors are as follows:\n");
            System.out.printf("---------------------------------------\n");
            for (i = 1; i <= 4; i++)
            {
                System.out.printf("%7.3f,%7.3f,%7.3f,%7.3f\n", vm[i][1], vm[i][2], vm[i][3], vm[i][4]);
            }
            System.out.printf("\n\n");

        }

        /*-- fill the rotation matrix which is made of elements from 4th EV   PDBS0206 */
        /*-----------------------------------------------------------------------------*/
        t[1][1] = Math.pow(vm[1][4], 2) + Math.pow(vm[2][4], 2) - Math.pow(vm[3][4], 2) - Math.pow(vm[4][4], 2);
        t[2][1] = 2.0 * (vm[2][4] * vm[3][4] + vm[1][4] * vm[4][4]);
        t[3][1] = 2.0 * (vm[2][4] * vm[4][4] - vm[1][4] * vm[3][4]);

        t[1][2] = 2.0 * (vm[2][4] * vm[3][4] - vm[1][4] * vm[4][4]);
        t[2][2] = Math.pow(vm[1][4], 2) + Math.pow(vm[3][4], 2) - Math.pow(vm[2][4], 2) - Math.pow(vm[4][4], 2);
        t[3][2] = 2.0 * (vm[3][4] * vm[4][4] + vm[1][4] * vm[2][4]);

        t[1][3] = 2.0 * (vm[2][4] * vm[4][4] + vm[1][4] * vm[3][4]);
        t[2][3] = 2.0 * (vm[3][4] * vm[4][4] - vm[1][4] * vm[2][4]);
        t[3][3] = Math.pow(vm[1][4], 2) + Math.pow(vm[4][4], 2) - Math.pow(vm[2][4], 2) - Math.pow(vm[3][4], 2);
        /*-----------------------------------------------------------------------------*/

        /* -- reset dxm to store the individual rmsd's in it now -              PDBS0221 */
        /* fillMatrix(dxm, NUM_ATOMS, 3, fill_value);*/
        for (k = 1; k <= NUM_ATOMS; k++)
        {
            for (j = 1; j <= 3; j++)
            {
                dxm[k][j] = fill_value;
            }
        }

        if (DEBUG_SUPERPOSE)
        {
            System.out.printf("\nThe rotation matrix: \n");
            System.out.printf("---------------------------------------\n");
            print3Matrix(t, 4, 4);
        }

        ArrayList<Group> groups = chain.getGroups();
        for (Group g: groups)
        {
            List<Atom> atoms = g.getAtoms();
            for ( Atom a: atoms)
            {
                double x[] = new double[4];

                x[1] = a.getX() - originalCoordinatesCenterOfMass[1];
                x[2] = a.getY() - originalCoordinatesCenterOfMass[2];
                x[3] = a.getZ() - originalCoordinatesCenterOfMass[3];

                /*for (i = 1; i <= 3; i++)
                {
                    //x[i] = temp_x[k][i];
                    x[i] = allAtomCoordinates[k][i];
                    x[i] = x[i] - originalCoordinatesCenterOfMass[i];
                }*/
                /* --     the thing is centred in origin now, rotate it         PDBS0328 */
                rotvec(3, x, t);

                /* ---         now add cf                                      PDBS0330 */
                for (i = 1; i <= 3; i++)
                {
                    x[i] = x[i] + targetCoordinateCenterOfMass[i];
                }

                a.setX(x[1]);
                a.setY(x[2]);
                a.setZ(x[3]);

            }
        }/* END for(k=1; k<=mov_all; k++) */
        //*******************************************************************************************************
    }

//************************************************************************************
/* Rotate a vector..ie. Multiply the array v with the matrix t */
    static void rotvec(int n, double v[], double t[][]){
      int i,j;
      double s[] = new double[4];
      for(i=1; i<=n; i++){
        s[i] = v[i];
        v[i] = 0.00;
      }

      for(i=1; i<=n; i++){
        for(j=1; j<=n; j++){
          v[i] = v[i]+s[j]*t[i][j];
        }
      }
    }/* END rotvec .. */
//**************************************************************************************



//**************************************************************************************
/* The input is a 5x5 Matrix, the first column
    and the first row are not printed *
 --------------------------------------------*/
    void print4Matrix(double arr[][],int row, int col){
      int i,j;
      System.out.printf("----------------------------------\n");
      for(i=1; i<row; i++){
        for(j=1; j<col; j++){
          System.out.printf("%11.6f  ",arr[i][j]);
          if(j == col-1)
        System.out.printf("\n");
        }
      }
      System.out.printf("\n");
    }/*-- END printMatrix()  --*/
//****************************************************************************************





//*****************************************************************************************
    static void print3Matrix(double arr[][],int row, int col){
      int i,j;
      System.out.printf("----------------------------------\n");
      for(i=1; i<row; i++){
        for(j=1; j<col; j++){
          System.out.printf("%10.6f ",arr[i][j]);
          if(j == col-1)
        System.out.printf("\n");
        }
      }
      System.out.printf("\n");
    }/*-- END printMatrix()  --*/
//*****************************************************************************************





//*****************************************************************************************
    static int jacobi(double a[][], int n, int np, double d[],double v[][]){
      int i,ip,iq,j,maxrot,nmax;
      double c, g, h, s, sm, t, tau, theta, tresh;
      double b[] = new double[500], z[] = new double[500], zero;

      /*Initialize*/
      //nmax = 500;
      zero = 1.0*(Math.pow(10,-30));
       /* System.out.printf("zero = %10.6f\n", zero);*/
      maxrot = 50;

      /* Initialize v to the Identity Matrix */
      /*-------------------------------------*/
      for(ip=1;ip<=n;ip++){
        for(iq=1;iq<=n;iq++){
          if(ip==iq)
        v[ip][iq] = 1.00;
          else
        v[ip][iq] = 0.00;
        }
      }
      /*
      System.out.printf("Printing v(the identity matrix)\n");
      print4Matrix(v,4,4);
      */
      /*Initialize b and d to the diagonal of a */
      for(ip=1; ip<=n; ip++){
        b[ip] = a[ip][ip];
        d[ip] = b[ip];
        z[ip] = 0.00;
      }

      /*
        System.out.printf("b = %10.2f, %10.2f, %10.2f, %10.2f \n",b[0], b[1], b[2], b[3]);
        System.out.printf("d = %10.2f, %10.2f, %10.2f, %10.2f \n",d[0], d[1], d[2], d[3]);
        System.out.printf("z = %10.2f, %10.2f, %10.2f, %10.2f \n",z[0], z[1], z[2], z[3]);
      */

      for(i=1; i<=maxrot; i++){
        sm = 0.00;

        for(ip=1; ip<=(n-1); ip++){
          for(iq=(ip+1); iq<=n; iq++){
        /*printf("ip= %d,  iq=%d\n", ip, iq);*/
        sm = sm + Math.abs(a[ip][iq]);
        /*printf("sm = %f\n", sm);*/
          }
        }
        /*    System.out.printf("sm = %.4g\n", sm); */

        if(sm < zero){
          return(0);
        }
        if ( i < 4){
          tresh = 0.2*sm/(n*n);
        }
        else {
          tresh = 0.0;
        }
        /*  System.out.printf("tresh = %.6g\n",tresh); */

        for(ip=1; ip<=(n-1); ip++){

          for(iq=(ip+1); iq<=n; iq++){

        g = 100.0*Math.abs(a[ip][iq]);
        /*	System.out.printf("g = %f\n", g); */

        if( (i>4)&&((Math.abs(d[ip])+g) == Math.abs(d[ip]) )&&((Math.abs(d[iq])+g)== Math.abs(d[iq]))){
          a[ip][iq] = 0.00;
          /* System.out.printf("a[ip][iq] = %f\n",a[ip][iq]);*/
        }
        else if(Math.abs(a[ip][iq]) > tresh){
          h = d[iq]-d[ip];
          /* System.out.printf("h = %f\n",h);*/


          /* Problem over here....Have to check Math.abs(h) and g values..   */
          if( (Math.abs(h)+g) == Math.abs(h) ){
            t = (a[ip][iq])/h;
            /* System.out.printf("t = %f\n", t);*/
          }
          else {
            theta = 0.50*h/a[ip][iq];
            t = 1.00/(Math.abs(theta)+Math.sqrt(1.00+theta*theta));
            /* System.out.printf("theta = %f\n", theta);
               System.out.printf("in else t = %f\n", t);*/
            if(theta < 0.0)
              t = -t;
          }

          c = 1.0/Math.sqrt(1+t*t);
          s = t*c;
           tau = s/(1.0 + c);
          h = t*(a[ip][iq]);
          /* System.out.printf(" c = %f    ", c); System.out.printf(" s = %f", s);
             System.out.printf(" tau = %f    ", tau); System.out.printf(" h = %f\n", h);*/

          z[ip] = z[ip] - h;
          z[iq] = z[iq] + h;
          d[ip] = d[ip] - h;
          d[iq] = d[iq] + h;
          a[ip][iq] = 0.00;
          /* System.out.printf("ip = %d,   iq = %d\n",ip,iq);*/
          /*	  System.out.printf("z[ip] = %f\n",z[ip]);
          System.out.printf("z[iq] = %f\n",z[iq]);
              System.out.printf("d[ip] = %f\n",d[ip]);
              System.out.printf("d[iq] = %f\n",d[iq]);
              System.out.printf("a[ip][iq] = %f\n",a[ip][iq]); */

          for(j=1; j<=(ip-1); j++){
            g = a[j][ip];    h = a[j][iq];
            a[j][ip] = g - s*(h+g*tau);
            a[j][iq] = h + s*(g-h*tau);
            /*  System.out.printf("first for..  ");
            System.out.printf("g=%f,  h=%f,   a[j][ip]=%f,   a[j][iq]=%f\n",g,h,a[j][ip], a[j][iq]); */
          }

            for(j=(ip+1); j<=(iq-1); j++){
            g=a[ip][j];     h=a[j][iq];
            a[ip][j]=g-s*(h+g*tau);
            a[j][iq]=h+s*(g-h*tau);
            /* System.out.printf("second for..  ");
               System.out.printf("g=%f,  h=%f,   a[ip][j]=%f,   a[j][iq]=%f\n",g,h,a[ip][j], a[j][iq]);*/
          }

          for(j=(iq+1); j<=n; j++){
            g=a[ip][j];     h=a[iq][j];
            a[ip][j]=g-s*(h+g*tau);
            a[iq][j]=h+s*(g-h*tau);
            /* System.out.printf("third for..  ");
               System.out.printf("g=%f,  h=%f,   a[ip][j]=%f,   a[iq][j]=%f\n",g,h,a[ip][j], a[iq][j]);*/
          }

          for(j=1; j<=n; j++){
            g=v[j][ip];     h=v[j][iq];
            v[j][ip]=g-s*(h+g*tau);
            v[j][iq]=h+s*(g-h*tau);
            /* System.out.printf("fourth for..  ");
               System.out.printf("g=%f,  h=%f,   v[j][ip]=%f,   v[j][iq]=%f\n",g,h,v[j][ip], v[j][iq]);*/
          }
        }
          }
        }


          for(ip=1; ip<=n; ip++){
            b[ip] = b[ip] + z[ip];
            d[ip] = b[ip];
            z[ip] = 0.00;
          }
      }
      /* System.out.printf("Too many iterations in jacobi\n");*/
      return(1);
    }/*  End jacobi() */
//*********************************************************************************************************




//***************************************************************************************************
    static void eigsrt(double d[], double v[][], int n, int np){
      int i, j, k;
      double p;

      for(i=1; i<=(n-1); i++){
        k = i;
        p = d[i];

        for(j=i+1; j<=n; j++){
          if(d[j] >= p){
        k=j;
        p = d[j];
          }
        }
        if(k != i){
          d[k] = d[i];
          d[i] = p;
          for(j=1; j<=n; j++){
        p = v[j][i];
        v[j][i] = v[j][k];
        v[j][k] = p;
          }
        }
      }
    }/* End eigsrt.. */
//******************************************************************************************************

    static void transpose_matrix(double arr[][],double arr_transp[][], int n){
      int i,j;

      for(i=1; i<=4; i++){
        for(j=1; j<=4; j++){
          arr_transp[j][i] = arr[i][j];
        }
      }
    }//********************************* END transpose_matrix() **************************

    /**
     * Assuming the two chains are the same length (residues and atoms), finds the Root Mean Squared Distance
     * for all atoms between them
     * @param c1
     * @param c2
     */
    public static RMSD findRMSD(Chain c1, Chain c2)
    {
        if ( c1.getLength() != c2.getLength() )
        {
            throw new IllegalArgumentException("Chains must be of equal length");
        }

        // get a rudimentary secondary structure
        String structure = AngleConverter.convertToSecondaryStructureString(HomodellerUtils.getAngles(c1));

        // find the start and end of the secondary structures
        int start = structure.length(), end = 0;
        for ( char c = 'A'; c <= 'Z'; c++)
        {
            String str = new String(new char[]{c, c, c});
            int index = structure.indexOf(str);
            if ( index != -1 && index < start )
            {
                start = index;
            }
            index = structure.lastIndexOf(str);
            if ( index != -1 && index > end )
            {
                end = index + 3;
            }
        }

        System.out.println("Using the following * areas for SS-only calculation:");
        System.out.println(structure);
        for ( int j=0; j<structure.length();j++)
        {
            if ( j >= start && j < end )
            {
                System.out.print("*");
            }
            else
            {
                System.out.print(" ");
            }
        }
        System.out.println();


            /*
            for ( int i=1; i<matcher.groupCount(); i++)
            {
            }*/

            /*for ( int i=structure.length()-1; i>=0; i--)
            {
                if ( structure.charAt(i) != 'G')
                {
                    end = i;
                    break;
                }
            }
            
        }*/

        // superimpose the structures
        superImpose(c1, c2);

        // calculate the RMSD between all atoms
        ArrayList<Group> c1Groups = c1.getGroups();
        ArrayList<Group> c2Groups = c2.getGroups();

        double[] all = new double[3];
        double[] ca = new double[3];
        double[] backbone = new double[3];
        double[] sidechain = new double[3];
        double[] backbone_SS_only = new double[3];
        double[] ca_SS_only = new double[3];

        int numAtoms = 0;
        int numBackboneAtoms = 0;
        int numCA_Atoms = 0;
        int numSidechainAtoms = 0;
        int numBackboneAtoms_SS_only = 0;
        int numCA_Atoms_SS_only = 0;

        for ( int i=0; i<c1Groups.size(); i++)
        {
            Group group1 = c1Groups.get(i);
            Group group2 = c2Groups.get(i);

            List<Atom> group1Atoms = group1.getAtoms();
            List<Atom> group2Atoms = group2.getAtoms();

            if ( group1Atoms.size() != group2Atoms.size() )
            {
                logger.debug("Group " + (i + 1) + " has different numbers of residues" );
                // This is problem because they may not be in order
            }

            try
            {
                for (int j = 0; j < group1Atoms.size(); j++)
                {
                    Atom a1 = group1Atoms.get(j);
                    Atom a2 = null;

                    if ( j < group2Atoms.size())
                    {
                        a2 = group2Atoms.get(j);
                    }

                    String atomName = a1.getName();
                    if (a2 == null || ! atomName.equals(a2.getName()))
                    {
                        // the equivalent atom to a1 in group2 is probably not here, but try and find it anyways
                        if ( group2.hasAtom(atomName))
                        {
                            a2 = group2.getAtom(atomName);
                        }
                    }

                    if ( a2 == null )
                    {
                        // the equivalent atom to a1 in group2 is not here, don't compare
                        continue;
                    }

                    if ( ! atomName.equals(a2.getName()))
                    {
                        // names are still different, can't compare
                        continue;
                    }


                    double x = a1.getX() - a2.getX();
                    double y = a1.getY() - a2.getY();
                    double z = a1.getZ() - a2.getZ();


                    all[X] += MathExt.square(x);
                    all[Y] += MathExt.square(y);
                    all[Z] += MathExt.square(z);

                    if (atomName.equals("CA"))
                    {
                        assert atomName.equals(a2.getName()) : "Atom names are different " + atomName + " " + a2.getName();

                        ca[X] += MathExt.square(x);
                        ca[Y] += MathExt.square(y);
                        ca[Z] += MathExt.square(z);
                        numCA_Atoms++;

                        // CA and within the secondary structure limits
                        if ( i >= start && i < end )
                        {
                            ca_SS_only[X] += MathExt.square(x);
                            ca_SS_only[Y] += MathExt.square(y);
                            ca_SS_only[Z] += MathExt.square(z);
                            numCA_Atoms_SS_only++;
                        }
                    }

                    if (isBackboneAtoms(a1))
                    {
                        assert atomName.equals(a2.getName()) : "Atom names are different " + atomName + " " + a2.getName();

                        backbone[X] += MathExt.square(x);
                        backbone[Y] += MathExt.square(y);
                        backbone[Z] += MathExt.square(z);
                        numBackboneAtoms++;

                        // backbone and within the secondary structure limits
                        if ( i >= start && i < end )
                        {
                            backbone_SS_only[X] += MathExt.square(x);
                            backbone_SS_only[Y] += MathExt.square(y);
                            backbone_SS_only[Z] += MathExt.square(z);
                            numBackboneAtoms_SS_only++;
                        }
                    }
                    else
                    {
                        assert atomName.equals(a2.getName()) : "Atom names are different " + atomName + " " + a2.getName();

                        sidechain[X] += MathExt.square(x);
                        sidechain[Y] += MathExt.square(y);
                        sidechain[Z] += MathExt.square(z);

                        numSidechainAtoms++;
                    }

                    numAtoms++;
                }
            }
            catch (StructureException e)
            {
               e.printStackTrace();
            }
        }

        RMSD rmsd = new RMSD();
        rmsd.all = calcRMSD(all, numAtoms);
        rmsd.CA = calcRMSD(ca, numCA_Atoms);
        rmsd.backbone = calcRMSD(backbone, numBackboneAtoms);
        rmsd.sidechain = calcRMSD(sidechain, numSidechainAtoms);
        rmsd.backbone_SS_only = calcRMSD(backbone_SS_only, numBackboneAtoms_SS_only);
        rmsd.ca_SS_only = calcRMSD(ca_SS_only, numCA_Atoms_SS_only);

        return rmsd;
    }

    private static boolean isBackboneAtoms(Atom a)
    {
        return a.getName().equals("N")
                || a.getName().equals("CA")
                || a.getName().equals("C");
    }

    private static double calcRMSD(double[] values, int numAtoms)
    {
        return Math.sqrt(values[X] + values[Y] + values[Z]) / Math.sqrt(numAtoms);
    }

    public static class RMSD
    {
        public double all;
        public double CA;
        public double backbone;
        public double sidechain;
        public double backbone_SS_only;
        public double ca_SS_only;

        public boolean equals(Object o)
        {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final RMSD rmsd = (RMSD) o;

            if (Double.compare(rmsd.CA, CA) != 0) return false;
            if (Double.compare(rmsd.all, all) != 0) return false;
            if (Double.compare(rmsd.backbone, backbone) != 0) return false;
            if (Double.compare(rmsd.sidechain, sidechain) != 0) return false;

            return true;
        }

        public int hashCode()
        {
            int result;
            long temp;
            temp = all != +0.0d ? Double.doubleToLongBits(all) : 0L;
            result = (int) (temp ^ (temp >>> 32));
            temp = CA != +0.0d ? Double.doubleToLongBits(CA) : 0L;
            result = 29 * result + (int) (temp ^ (temp >>> 32));
            temp = backbone != +0.0d ? Double.doubleToLongBits(backbone) : 0L;
            result = 29 * result + (int) (temp ^ (temp >>> 32));
            temp = sidechain != +0.0d ? Double.doubleToLongBits(sidechain) : 0L;
            result = 29 * result + (int) (temp ^ (temp >>> 32));
            return result;
        }

        public String toString()
        {
            return "rmsd (all) = " + all
                    + "\nrmsd (backbone) = " + backbone
                    + "\nrmsd (CA) = " + CA
                    + "\nrmsd (sidechain) = " + sidechain
                    + "\nrmsd (CA, SS_only) = " + ca_SS_only
                    + "\nrmsd (backbone, SS_only) = " + backbone_SS_only;
        }

    }

    /**
     * Superimpose two similar structures on top of each other.  The chains passed
     * as parameters will be superimposed.
     *
     * @param c1 First chain to be superimposed
     * @param c2 Second chain to be superimposed
     */
    public static void superImpose(Chain c1, Chain c2)
    {
        Atom[] original = new Atom[1];
        Atom[] target = new Atom[1];

        List<Atom> originalList = new ArrayList<Atom>();
        List<Atom> targetList = new ArrayList<Atom>();

        ArrayList<Group> c1Groups = c1.getGroups();
        ArrayList<Group> c2Groups = c2.getGroups();

        for ( int i=0; i<c1Groups.size(); i++)
        {
            List<Atom> group1Atoms = c1Groups.get(i).getAtoms();
            List<Atom> group2Atoms = c2Groups.get(i).getAtoms();

            for ( int j=0; j < Math.min(group1Atoms.size(), group2Atoms.size()); j++)
            {
                Atom a1 = group1Atoms.get(j);
                Atom a2 = group2Atoms.get(j);

                // only use backbone atoms
                if ( isBackboneAtoms(a1) )
                {
                    originalList.add(a1);
                    targetList.add(a2);
                }
            }
        }

        original = originalList.toArray(original);
        target = targetList.toArray(target);

        Superpose.superpose(original, target, c1);
    }

    public static void main(String[] args)
    {
        if ( args.length != 2 )
        {
            printUsage();
            return;
        }

        try
        {
            FileInputStream file1 = new FileInputStream(args[0]);
            FileInputStream file2 = new FileInputStream(args[1]);

            PDBFileParser pdbReader = new PDBFileParser();
            Structure pdbfile1 = pdbReader.parsePDBFile(file1);
            Structure pdbfile2 = pdbReader.parsePDBFile(file2);

            RMSD rmsd = findRMSD(pdbfile1.getChain(0), pdbfile2.getChain(0));

            System.out.println(rmsd);
        }
        catch (FileNotFoundException e)
        {
            System.out.println(e);
            printUsage();
        }
        catch (IOException e)
        {
            e.printStackTrace();  
        }

    }

    private static void printUsage()
    {
        System.out.println("Usage: Superpose file1 file2");
    }
}
