package ca.ualberta.proteus.program.homodeller;

import org.biojava.bio.structure.Atom;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 14, 2006
 * Time: 10:41:18 PM
 * To change this template use File | Settings | File Templates.
 */
public class TorsionUtils
{
    final static int X = 0;
    final static int Y = 1;
    final static int Z = 2;

    public static double torsion(Atom atm1, Atom atm2, Atom atm3, Atom atm4)
    {
        int ichk;
        double ret, chk;

        double[] p = vecdif(atm1, atm2);
        ichk = dischk(p);
        if (ichk > 0)
            return (360.0);

        double[] q = vecdif(atm3, atm2);
        ichk = dischk(q);
        if (ichk > 0)
            return (360.0);

        double[] x = cross(p, q);

        p = vecdif(atm2, atm3);

        q = vecdif(atm4, atm3);
        ichk = dischk(q);
        if (ichk > 0)
            return (360.0);

        double[] y = cross(p, q);

        ret = angle(x, y);
        q = cross(x, y);
        chk = dot(p, q);
        if (chk > 0.0)
            ret = -ret;

        return (ret);

    }    /* torsion */


    /**
     * *********************************************************
     */
/* calculate the diffence of the vectors x and y.
*/
    static double[] vecdif(Atom vec1, Atom vec2)
    {
        double[] result = new double[3];
        result[0] = vec1.getX() - vec2.getX();
        result[1] = vec1.getY() - vec2.getY();
        result[2] = vec1.getZ() - vec2.getZ();

        return result;
    }    /* vecdif */


    /**
     * *********************************************************
     */
/* checks the vector atm
*/
    static int dischk(double[] atm)
    {
        double dssq;

        dssq = dot(atm, atm);
        if ((dssq < 0.5) || (dssq > 9.0))
            return (1);
        else return (0);

    }    /* dischk */


    /**
     * *********************************************************
     */
/* calculate the cross product of vectors x and y.
*/
    static double[] cross(double[] a, double[]b)
    {
        double[] result = new double[3];

        result[X] = (a[Y] * b[Z]) - (b[Y] * a[Z]);
        result[Y] = (a[Z] * b[X]) - (b[Z] * a[X]);
        result[Z] = (a[X] * b[Y]) - (b[X] * a[Y]);

        return result;
    }    /* cross */


    /**
     * *********************************************************
     */
/* calculate the dot product of vectors x and y.
*/
    static double dot(double[] x, double[] y)
    {
        int i;
        double dotprod;

        dotprod = 0.0;
        for (i = 0; i < 3; i++)
            dotprod = dotprod + (x[i] * y[i]);

        return (dotprod);

    }    /* dot */


    /*************************************************************/
/* calculate the angle between two vectors defined by the
 * ends of three vectors.
 */
    static double angle(double[] x, double[] y)
    {
        double ang, q;
        double magx, magy;

        magx = Math.sqrt(dot(x, x));
        magy = Math.sqrt(dot(y, y));
        q = magx * magy;

        if (q < 1.0e-7)
            q = 1.0e-7;
        ang = dot(x, y) / q;

        if (ang > 1.0)
            ang = 1.0;
        if (ang < -1.0)
            ang = -1.0;
        ang = 57.29577951 * Math.acos(ang);
        return (ang);

    }	/* angle */
}
