package ca.ualberta.proteus.program.homodeller;

import org.apache.log4j.Logger;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: Apr 19, 2007
 * Time: 11:29:09 PM
 */
public class VectorMath
{
    private static Logger logger = Logger.getLogger(VectorMath.class);

/*************************************************************/
    /* Set diff to v1 - v2                                       */
/**
 * *********************************************************
 */
    public static void difference(double[] v1, double[] v2, double[] diff)
    {
        diff[0] = v1[0] - v2[0];
        diff[1] = v1[1] - v2[1];
        diff[2] = v1[2] - v2[2];
    }

/*************************************************************/
    /* Set sum to v1 + v2                                        */
/**
 * *********************************************************
 */
    public static void summation(double[] v1, double[] v2, double[] sum)
    {
        sum[0] = v1[0] + v2[0];
        sum[1] = v1[1] + v2[1];
        sum[2] = v1[2] + v2[2];
    }

/*************************************************************/
    /* Scales the vector by a scalar value                       */
/**
 * *********************************************************
 */
    public static void scale(double[] vec, double scalar)
    {
        vec[0] = vec[0] * scalar;
        vec[1] = vec[1] * scalar;
        vec[2] = vec[2] * scalar;
    }

/*************************************************************/
    /* Turn the vector into a Unit Vector                        */
/**
 * *********************************************************
 */
    public static void unitVector(double[] vec, double[] uvec)
    {
        double mag = Math.sqrt(dotproduct(vec, vec));
        uvec[0] = vec[0] / mag;
        uvec[1] = vec[1] / mag;
        uvec[2] = vec[2] / mag;
    }

    /* calculate the cross product of vectors x and y.           */
/**
 * *********************************************************
 */
    public static void crossproduct(double[] a, double[] b, double[] result)
    {
        result[0] = (a[1] * b[2]) - (b[1] * a[2]);
        result[1] = (a[2] * b[0]) - (b[2] * a[0]);
        result[2] = (a[0] * b[1]) - (b[0] * a[1]);
    }

/*************************************************************/
    /* calculate the dot product of vectors x and y.             */
/**
 * *********************************************************
 */
    public static double dotproduct(double[] x, double[] y)
    {
        return ((x[0] * y[0]) + (x[1] * y[1]) + (x[2] * y[2]));
    }

/*************************************************************/
/* calculate the angle between two vectors defined by the    */

    /* ends of the vectors.                                    */
/**
 * *********************************************************
 */
    public static double angle(double[] X, double[] Y)
    {
        double XY = dotproduct(X, Y);
        double Xlen = Math.sqrt(dotproduct(X, X));
        double Ylen = Math.sqrt(dotproduct(Y, Y));
        double cos_theta = XY / (Xlen * Ylen);
        double theta;
        if (cos_theta >= 1.0)
        {
            theta = 0.0;
        }
        else if (cos_theta <= -1.0)
        {
            theta = 180.0;
        }
        else
        {
            theta = Math.acos(cos_theta);
            theta = theta * (180.0 / Math.PI); /* convert to degrees */
        }
        return theta;
    }

/*************************************************************/
    /* rotate r around n by theta counter-clockwise              */
/**
 * *********************************************************
 */
    public static void rotateCC(double[] n, double[] r, double theta)
    {
        double A, B, C;
        double n0, n1, n2;
        double r0, r1, r2;

        /*
      double[] q;
      if (theta == 180.0 || theta == -180.0) {
          A = -1.0;
          C = 0.0;
      } else {
          theta *= PI/180.0;
          A = cos(theta);
          C = sin(theta);
      }
      B = (n[0]*r[0]+n[1]*r[1]+n[2]*r[2])*(1.0 - A);
      q[0] = A*r[0] + B*n[0] - C*(r[1]*n[2] - r[2]*n[1]);
      q[1] = A*r[1] + B*n[1] - C*(r[2]*n[0] - r[0]*n[2]);
      q[2] = A*r[2] + B*n[2] - C*(r[0]*n[1] - r[1]*n[0]);
      r[0] = q[0]; r[1] = q[1]; r[2] = q[2];
      */

        // This function is very heavily used. A more optimized version
        // that uses around 30% as much time follows:

        n0 = n[0];
        n1 = n[1];
        n2 = n[2];
        r0 = r[0];
        r1 = r[1];
        r2 = r[2];

        if (theta == 180.0 || theta == -180.0)
        {
            //A = -1.0;
            //C = 0.0;

            B = (n0 * r0 + n1 * r1 + n2 * r2) * (2.0);
            r[0] = -r0 + B * n0;
            r[1] = -r1 + B * n1;
            r[2] = -r2 + B * n2;

        }
        else
        {
            theta *= Math.PI / 180.0;
            A = Math.cos(theta);
            C = Math.sin(theta);

            B = (n0 * r0 + n1 * r1 + n2 * r2) * (1.0 - A);
            r[0] = A * r0 + B * n0 - C * (r1 * n2 - r2 * n1);
            r[1] = A * r1 + B * n1 - C * (r2 * n0 - r0 * n2);
            r[2] = A * r2 + B * n2 - C * (r0 * n1 - r1 * n0);
        }

    }

/*************************************************************/
    /* Rotate r around the x-axis by theta                       */
/**
 * *********************************************************
 */
    public static void rotateX(double[] r, double theta)
    {
        //double[] U;
        //setVector(U, 1, 0, 0);
        //rotateCC(U, r, theta);

        // Alternate optimized version (derived by copying in rotateCC code and simplifying):

        double A, C;
        double r1, r2;

        if (theta == 180.0 || theta == -180.0)
        {
            //A = -1.0;
            //C = 0.0;

            r[1] *= -1.0;
            r[2] *= -1.0;
        }
        else
        {
            theta *= Math.PI / 180.0;
            A = Math.cos(theta);
            C = Math.sin(theta);

            r1 = r[1];
            r2 = r[2];

            r[1] = A * r1 - C * (r2);
            r[2] = A * r2 + C * (r1);
        }

    }

/*************************************************************/
    /* Rotate r around the y-axis by theta                       */
/**
 * *********************************************************
 */
    public static void rotateY(double[] r, double theta)
    {
        //double[] U;
        //setVector(U, 0, 1, 0);
        //rotateCC(U, r, theta);

        // Alternate optimized version (derived by copying in rotateCC code and simplifying):

        double A, C;
        double r0, r2;

        if (theta == 180.0 || theta == -180.0)
        {
            //A = -1.0;
            //C = 0.0;

            r[0] *= -1.0;
            r[2] *= -1.0;
        }
        else
        {
            theta *= Math.PI / 180.0;
            A = Math.cos(theta);
            C = Math.sin(theta);

            r0 = r[0];
            r2 = r[2];

            r[0] = A * r0 + C * (r2);
            r[2] = A * r2 - C * (r0);
        }

    }

/*************************************************************/
    /* Rotate r around the z-axis by theta                       */
/**
 * *********************************************************
 */
    public static void rotateZ(double[] r, double theta)
    {
        //double[] U;
        //setVector(U, 0, 0, 1);
        //rotateCC(U, r, theta);

        // Alternate optimized version (derived by copying in rotateCC code and simplifying):

        double A, C;
        double r0, r1;

        if (theta == 180.0 || theta == -180.0)
        {
            //A = -1.0;
            //C = 0.0;

            r[0] *= -1.0;
            r[1] *= -1.0;
        }
        else
        {
            theta *= Math.PI / 180.0;
            A = Math.cos(theta);
            C = Math.sin(theta);

            r0 = r[0];
            r1 = r[1];

            r[0] = A * r0 - C * (r1);
            r[1] = A * r1 + C * (r0);
        }

    }

/*************************************************************/
/* copy src into dst                                         */
/*                                                           */
/*                     +----------+                          */
/*                     |          |                          */

    /*                     |          v                          */
/**
 * *********************************************************
 */
    public static void copyVector(double[] src, double[] dst)
    {
        dst[0] = src[0];
        dst[1] = src[1];
        dst[2] = src[2];
    }

/*************************************************************/
    /* Set the points of the vector all to val                   */
/**
 * *********************************************************
 */
    public static void initVector(double[] vec, double val)
    {
        vec[0] = vec[1] = vec[2] = val;
    }

/*************************************************************/
    /* Set the points of the vector                              */
/**
 * *********************************************************
 */
    public static void setVector(double[] vec, double x, double y, double z)
    {
        vec[0] = x;
        vec[1] = y;
        vec[2] = z;
    }

/*************************************************************/
    /* Compute the distance between two points                   */
/**
 * *********************************************************
 */
    public static double distance(double[] a, double[] b)
    {
        // For some reason this "optimized" version seems to run slightly slower.
        /*
      double ab0 = a[0]-b[0];
      double ab1 = a[1]-b[1];
      double ab2 = a[2]-b[2];
      return sqrt(ab0*ab0 + ab1*ab1 + ab2*ab2);
      */

        return Math.sqrt((a[0] - b[0]) * (a[0] - b[0]) +
                (a[1] - b[1]) * (a[1] - b[1]) +
                (a[2] - b[2]) * (a[2] - b[2]));
    }


    public static void normalize(double[] a, double[] c)
    {
        double mag = Math.sqrt(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
        c[0] = a[0] / mag;
        c[1] = a[1] / mag;
        c[2] = a[2] / mag;
    }

    public static void printvector(double[] a)
    {
        logger.debug(String.format("%.3f, %.3f, %.3f", a[0], a[1], a[2]));
    }

}
