package ca.ualberta.proteus.program.homodeller;

import ca.ualberta.proteus.util.ProteusTools;
import org.biojava.bio.BioException;
import org.biojava.bio.symbol.Symbol;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 4, 2005
 * Time: 7:11:59 PM
 * To change this template use File | Settings | File Templates.
 */
public class WeightMatrix
{

    private static WeightMatrix instance = null;
    private static final int WT_MAT_DIM = 32;
    private static final String WT_MAT_FILE = "data/wt.rbo2";

    // data represents the rows, with the key being the left side's residue identifier
    private Map data = null; //<String, Map<String, Integer>>


    /**
     * Construct an m-by-n matrix of zeros.
     *
     */
    private WeightMatrix() throws IOException
    {
        loadMatrix();
    }

    public static WeightMatrix getInstance() throws IOException
    {
        if ( instance == null )
            instance = new WeightMatrix();
        return instance;
    }

    private void loadMatrix() throws IOException
    {
        data = new HashMap();

        BufferedReader reader = new BufferedReader(new FileReader(WT_MAT_FILE));

        readWeightMatrix(reader);
    }

    private void readWeightMatrix(BufferedReader reader) throws IOException
    {
        reader.readLine(); // discard first line

        String line = null;

        // get the column headers
        String headerLine = reader.readLine();
        assert headerLine != null;
        String[] headers = headerLine.split("\\s+");

        // read in the data
        int row = 1;
        while ( ( line = reader.readLine()) != null ) {
            String[] tokens = line.split("\\s+");

            // create the new map holding this row's data
            Map rowData = new HashMap();
            data.put(tokens[0], rowData);

            for ( int col=1; col<tokens.length; col++) {
                int value = Integer.parseInt(tokens[col]);

                // place into data
                rowData.put(headers[col], new Integer(value));
            }
            row++;
        }
    }

    public int get( String res1, String res2) {
        Integer rv = (Integer) ((Map) data.get(res2)).get(res1);

        // Matrix is lower triangular, so we need to be reflective
        if ( rv == null)
            return ((Integer) ((Map)data.get(res1)).get(res2)).intValue();
        else
            return rv.intValue();
    }

    public int get( char res1, char res2) {
        return get(String.valueOf(res1), String.valueOf(res2));
    }

    public static void main(String[] arg) {
        try
        {
            WeightMatrix wt = WeightMatrix.getInstance();

            System.out.println("Weight(A, A) = " + wt.get('A', 'A'));
            System.out.println("Weight(A, L) = " + wt.get('A', 'L'));
            System.out.println("Weight(L, A) = " + wt.get('L', 'A'));
            System.out.println("Weight(Y, R) = " + wt.get('Y', 'R'));
            System.out.println("Weight(W, M) = " + wt.get('W', 'M'));
            System.out.println("Weight(E, S) = " + wt.get('E', 'S'));
            System.out.println("Weight(E, W) = " + wt.get('E', 'W'));
            System.out.println("Weight(F, F) = " + wt.get('F', 'F'));
            System.out.println("Weight(M, R) = " + wt.get('M', 'R'));
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

    }

    public int get(Symbol res1, Symbol res2) throws BioException
    {
        char sRes1 = convertToChar( res1 );
        char sRes2 = convertToChar( res2 );
        return get( sRes1, sRes2 );
    }

    private char convertToChar(Symbol res ) throws BioException
    {
//        char rv = res.getName().charAt(0);
        /* this really should be the inverse of tokenToSymbol in ProteinTools */

        return ProteusTools.convertToChar( res );
    }
}

