/*
 * Created on May 9, 2008
 */
package ca.ualberta.proteus.program.homodeller.loop;

import ca.ualberta.proteus.program.homodeller.SideChain;
import ca.ualberta.proteus.program.homodeller.Superpose;

import org.apache.log4j.Logger;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.structure.Atom;
import org.biojava.bio.structure.Chain;
import org.biojava.bio.structure.Group;
import org.biojava.bio.structure.StructureException;
import org.biojava.bio.structure.StructureImpl;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 *
 *
 * @author Joseph Cruz
 */
public class LoopUtil {

	private static Logger logger = Logger.getLogger(LoopUtil.class);
	
	public static boolean isBeginning(Gap gap) {
		return gap.getStart() == 0;
	}
	
	public static boolean isEnd(Sequence sequence, Gap gap) {
		return gap.getEnd() >= sequence.length();
	}
	
	public static void swap(List<Group> groups, int pos1, int pos2) {
		Group temp = groups.get(pos1);
		groups.set(pos1, groups.get(pos2));
		groups.set(pos2, temp);
	}
	
	public static void writeChainToFile(String filename, Chain chain) {
		try {
			StructureImpl tmp = new StructureImpl();
			
			tmp.addChain(chain);
			File tempFile = new File(filename);
            tempFile.getParentFile().mkdirs();

            try {
                PrintWriter w = new PrintWriter(tempFile);
                w.print(tmp.toPDB());
                w.close();
            } catch(FileNotFoundException e) {
            	logger.error(e.getMessage(), e);
            }
        } catch (Exception e) {
            // Don't want any exceptions
        	logger.error(e.getMessage(), e);
        }
    }
	
	/**
     * Modifies the coordinates of the new side chain from the coordinates in the database
     * to the coordinates from the original residue
     *
     * @param original                 the original residue whose coordinates we wish to modify
     * @param representativeNewResidue the side chain whose coordinates need to be modified to the original's postion
     * @param chain                    The loop or chain to superimpose
     * @return a superimposed chain
     */
    public static Chain superImposeLoop(Group original, Group representativeNewResidue, Chain chain) {
        if (chain.getGroups().size() == 0)
            return chain;

        try {
            // For a superposition, we need two sets of coordinates
            // abc and pqr.  That will give us the translation and transformation of the new coordinates
            double[][] abc = SideChain.getReferenceCoordinates(original);
            double[][] pqr = SideChain.getReferenceCoordinates(representativeNewResidue);

            ArrayList<Group> groups = chain.getGroups();

            // do the translation on every atom in every residue in the chain
            for (Group g : groups)
            {
                List<Atom> atoms = g.getAtoms();

                for (Atom a : atoms)
                {
                    double[] coordinates = Superpose.superpose(abc, pqr, a.getX(), a.getY(), a.getZ());
                    a.setX(coordinates[0]);
                    a.setY(coordinates[1]);
                    a.setZ(coordinates[2]);
                }
            }
        }
        catch (StructureException e) {
            logger.error("Error getting atom", e);
        }

        return chain;
    }
}
