package ca.ualberta.proteus.program.homodeller.test;

import ca.ualberta.proteus.program.homodeller.Gap;
import ca.ualberta.proteus.program.homodeller.LoopInserter;
import ca.ualberta.proteus.program.homodeller.HomodellerImpl;
import ca.ualberta.proteus.util.ProteusTools;

import junit.framework.TestCase;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleGappedSequence;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.structure.Structure;

import java.io.IOException;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 13, 2006
 * Time: 9:43:34 PM
 * To change this template use File | Settings | File Templates.
 */
public class GapTest extends TestCase
{
	public void testGaps() throws IllegalSymbolException {
		String queryString = "AAAMMMMMMMMAAMMM";
		String homologString = "AAAAAAMMMMAAAMM";
		
		Sequence query = ProteinTools.createProteinSequence(queryString, "QUERY");
		Sequence homolog = ProteinTools.createProteinSequence(homologString, "HOMOLOG");
		
		GappedSequence gappedQuery = new SimpleGappedSequence(query);
		gappedQuery.addGapInSource(queryString.indexOf("AM")+2);
		gappedQuery.addGapInSource(queryString.indexOf("AM")+2);
		gappedQuery.addGapInSource(queryString.indexOf("AM")+2);
		gappedQuery.addGapInSource(queryString.lastIndexOf("AM")+2);
		
		GappedSequence gappedHomolog = new SimpleGappedSequence(homolog);
		gappedHomolog.addGapInSource(homologString.indexOf("MA")+2);
		gappedHomolog.addGapInSource(homologString.indexOf("MA")+2);
		gappedHomolog.addGapInSource(homologString.indexOf("MA")+2);
		gappedHomolog.addGapInSource(homologString.indexOf("MA")+2);
		gappedHomolog.addGapInSource(homologString.length()+1);
		
		System.out.println(gappedQuery.seqString() + " (" + queryString.length() + ")");
		System.out.println(gappedHomolog.seqString() + " (" + homologString.length() + ")");
		
		Gap gap = new Gap(gappedQuery, gappedHomolog);
		System.out.println(gappedQuery.length());
		gap.setStart(3);
		gap.setLength(3);
		System.out.println("PRE = " + gap.getPreOnAlignment());
		System.out.println("POST = " + gap.getPostOnAlignment());
	}
	
    /*public void testFindGaps() throws IOException
    {
        String path = "/home/scott/homodeller/test/102L/";
        LoopInserter loopInserter = new LoopInserter(path + "102L.out");
        loopInserter.readInfoFile();

        GappedSequence querySeq = loopInserter.getQuerySeq();
        GappedSequence databaseSeq = loopInserter.getDatabaseSeq();
        System.out.println(querySeq.seqString().replaceAll("(.{10})", "$1 "));
        System.out.println(databaseSeq.seqString().replaceAll("(.{10})", "$1 "));

        List<Gap> gaps = loopInserter.findGaps();
        for (Gap g : gaps)
        {
            System.out.print(g);

            // it's + 1 because this is the symbolList substring, based at 1
            System.out.println(",  " + querySeq.getSourceSymbolList().subStr(g.getAdjustedStartOnQuery() + 1, g.getAdjustedEndOnQuery()));
        }
    }
*/
	
	/*
    public void testFindGaps2() throws IOException, IllegalSymbolException
    {
        String query =    "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAA-AKSELDKA-GRNTNGVITKDEA--LFNQDVDAAVRGILRNAKLKPVYD" +
                "SLDAVRRAALINMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRVITTFRTGTWDAYKNL";
        String database = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAAELDKAIGRNTNGVITKDEAEKLFNQDVDAAV-GILRNAKLKP-YD" +
                "SLDAVRRAALINMVFQMGETGVAGFTNS--MLQQKRWDEAAVNLAKSRWY--TPNRAKRVITTFRTGTWDAYK~~";
        GappedSequence querySeq = ProteinTools.createGappedProteinSequence(query, "Query");
        GappedSequence databaseSeq = ProteinTools.createGappedProteinSequence(database, "DB");

        Structure structure = HomodellerImpl.readTemplateStructure("102L");

        LoopInserter loopInserter = new LoopInserter(structure, querySeq, databaseSeq);

        loopInserter.setQuerySeq(querySeq);
        loopInserter.setDatabaseSeq(databaseSeq);

        System.out.println(querySeq.seqString().replaceAll("(.{10})", "$1 "));
        System.out.println(databaseSeq.seqString().replaceAll("(.{10})", "$1 "));

        List<Gap> gaps = loopInserter.findGaps();

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + querySeq.getSourceSymbolList().subStr(g.getAdjustedStartOnQuery() + 1, g.getAdjustedEndOnQuery()));
        }

        Gap gap = gaps.get(0);

        // this is what we are going to replace
        assertEquals("AAK", gap.getQueryGapSequence().seqString());
        gap = gaps.get(1);
        assertEquals("AGR", gap.getQueryGapSequence().seqString());
        gap = gaps.get(2);
        assertEquals("ALF", gap.getQueryGapSequence().seqString());
        gap = gaps.get(3);
        assertEquals("VRG", gap.getQueryGapSequence().seqString());
        gap = gaps.get(4);
        assertEquals("PVY", gap.getQueryGapSequence().seqString());
        gap = gaps.get(5);
        assertEquals("SLR", gap.getQueryGapSequence().seqString());
        gap = gaps.get(6);
        assertEquals("YNQ", gap.getQueryGapSequence().seqString());
        gap = gaps.get(7);
        assertEquals("NL", gap.getQueryGapSequence().seqString());
    }

    private LoopInserter testGaps(String query, String database, String... expectedGaps)
            throws IllegalSymbolException, IOException
    {
        GappedSequence querySeq = ProteinTools.createGappedProteinSequence(query, "Query");
        GappedSequence databaseSeq = ProteinTools.createGappedProteinSequence(database, "DB");

        Structure structure = HomodellerImpl.readTemplateStructure("102L");

        LoopInserter loopInserter = new LoopInserter(structure, querySeq, databaseSeq);
        loopInserter.setQuerySeq(querySeq);
        loopInserter.setDatabaseSeq(databaseSeq);

        System.out.println(querySeq.seqString().replaceAll("(.{10})", "$1 "));
        System.out.println(databaseSeq.seqString().replaceAll("(.{10})", "$1 "));

        List<Gap> gaps = loopInserter.findGaps();

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }

        for (int i=0; i<gaps.size(); i++)
        {
            Gap gap = gaps.get(i);
            assertEquals(expectedGaps[i], gap.getQueryGapSequence().seqString());
        }

        return loopInserter;
    }

    public void testFindGaps2A() throws IOException, IllegalSymbolException
    {
        String query =    "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAA-AKSELDKANL";
        String database = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAAELDKA~~";
        testGaps(query, database, "AAK", "NL");
    }

    public void testFindGaps3() throws IOException, IllegalSymbolException
    {
        String query =    "MNIFEMLRIDEGLR---YK";
        String database = "MNI---LRIDEGLRLKIYK";
        testGaps(query, database, "IFEMLR", "RYK");
    }

    public void testFindGaps3A() throws IOException, IllegalSymbolException
    {
        String query =    "EGLR---YK";
        String database = "EGLRLKIYK";
        testGaps(query, database, "RYK");
    }


    public void testFindGaps4() throws IOException, IllegalSymbolException
    {
        String query =    "MNIFEMLRIDEGLRL--YK";
        String database = "MNI---LRIDEGLRLKIYK";
        testGaps(query, database, "IFEMLR", "LYK");
    }

    public void testFindGaps5() throws IOException, IllegalSymbolException
    {
        String query =    "MNIFEMLRIDEGLRL---K";
        String database = "MNI---LRIDEGLRLKIYK";
        testGaps(query, database, "IFEMLR", "RLK");
    }

    public void testFindGaps6() throws IOException, IllegalSymbolException
    {
        String query =    "MNI---LRIDEGLRLKIYK";
        String database = "MNIFEMLRIDEGLRL---K";
        testGaps(query, database, "ILR", "LKIYK");
    }

    public void testFindGaps6A() throws IOException, IllegalSymbolException
    {
        String query =    "LRLKIYK";
        String database = "LRL---K";
        testGaps(query, database, "LKIYK");
    }

    public void testFindGaps7() throws IOException, IllegalSymbolException
    {
        String query =    "---LRIDEGLRLKIYK";
        String database = "FEMLRIDEGLRL---K";
        testGaps(query, database, "", "LKIYK");
    }

    public void testFindGaps8() throws IOException, IllegalSymbolException
    {
        String query =    "--LRIDEGLRLKIYK";
        String database = "FELRIDEGLRL---K";
        testGaps(query, database, "", "LKIYK");
    }

    public void testFindGaps8A() throws IOException, IllegalSymbolException
    {
        String query =    "-LRIDEGLRLKIYK";
        String database = "ELRIDEGLRL---K";
        testGaps(query, database, "", "LKIYK");
    }

    public void testFindGaps9() throws IOException, IllegalSymbolException
    {
        String query =    "FEL-----LRLKIYK";
        String database = "FELRIDEGLRL---K";
        testGaps(query, database, "LLR", "LKIYK");
    }

    public void testFindGaps10() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRL---KR";
        String database =   "FEL-----LRLKIYKR";
        testGaps(query, database, "LRIDEGLR", "LKR");
    }

    public void testFindGaps11() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRL---KRERGER";
        String database =   "FEL-----LRLKIYKRERG--";
        testGaps(query, database, "LRIDEGLR", "LKR", "ER");
    }

    public void testFindGaps12() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRERR";
        String database =   "FELLRLKIYKRERG--";
        testGaps(query, database, "RR");
    }

    public void testFindGaps12A() throws IOException, IllegalSymbolException
    {
        String query =      "F-LRIDEGLRLKRERRE";
        String database =   "FELLR-KIYKRERG---";
        testGaps(query, database, "LRI", "IDE", "RRE");
    }

    public void testFindGaps12B() throws IOException, IllegalSymbolException
    {
        String query =      "F-LRIDEGLRLKRERREA";
        String database =   "FELLR-KIYKRERG---A";
        testGaps(query, database, "LRI", "IDE", "ERREA");
    }

    public void testFindGaps12C() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRERRE";
        String database =   "---LR-KIYKRERG---";
        testGaps(query, database, "FEL", "IDE", "RRE");
    }

    public void testFindGaps13() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRE--";
        String database =   "FELLRLKIYKRERGRR";
        testGaps(query, database, "");
    }

    public void testFindGaps14() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRE-";
        String database =   "FELLRLKIYKRERGR";
        testGaps(query, database, "");
    }

    public void testFindGaps15() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRE-";
        String database =   "-ELLRLKIYKRERGR";
        testGaps(query, database, "F", "");
    }

    public void testFindGaps16() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRE-";
        String database =   "--LLRLKIYKRERGR";
        testGaps(query, database, "FE", "");
    }

    public void testFindGaps17() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRE-";
        String database =   "---LRLKIYKRERGR";
        testGaps(query, database, "FEL", "");
    }

    public void testFindGaps18() throws IOException, IllegalSymbolException
    {
        String query =      "YNIDLKARCQ";
        String database =   "KNLDKKN-C~";
        testGaps(query, database, "ARC", "Q");
    }

    public void testFindGaps19() throws IOException, IllegalSymbolException
    {
        String query =      "NLYQFKNMIKCTVPS-RSWWDFADYGCYCGRGGSGTPVDDLDRCCQVHDNCYNEAEKISGCWPYFKTYSYECSQGTLTCKGDNNACAASVCDCDRLAAICFAGAPYNDNNYNIDLKARCQ";
        String database =   "ALWQFNGMIKCKIPSSEPLLDFNNYGCYCGLGGSGTPVDDLDRCCQTHDNCYKQAKKLDSCK-VLNNYSYSCSNNEITCSSENNACEAFICNCDRNAAICFSKVPYNKEHKNLDKKN-C~";
        LoopInserter loopInserter = testGaps(query, database, "SRS", "WPY", "ARC", "Q");

        List<Gap> gaps = loopInserter.findGaps();

        // Test after merging
        gaps = loopInserter.mergeGaps(gaps);

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }

        assertEquals("RCQ", gaps.get(2).getQueryGapSequence().seqString());
    }

    public void testFindGaps20() throws IOException, IllegalSymbolException
    {
        String query =      "FELRIDEGLRLKRERRE";
        String database =   "-E-LR-KIYKRERG---";
        LoopInserter loopInserter = testGaps(query, database, "F", "ELR", "IDE", "RRE");

        List<Gap> gaps = loopInserter.findGaps();

        // Test after merging
        gaps = loopInserter.mergeGaps(gaps);

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }

        assertEquals("FELRID", gaps.get(0).getQueryGapSequence().seqString());
        assertEquals("RRE", gaps.get(1).getQueryGapSequence().seqString());
    }

    public void testFindGaps21() throws IOException, IllegalSymbolException
    {
        String query =      "FE-RIDEGLRLKRERRE";
        String database =   "-EKLR-KIYKRERG---";
        LoopInserter loopInserter = testGaps(query, database, "F", "ERI", "IDE", "RRE");

        List<Gap> gaps = loopInserter.findGaps();

        // Test after merging
        gaps = loopInserter.mergeGaps(gaps);

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }


        // TODO these don't pass yet, but they are really rare pathological cases
        assertEquals("FERID", gaps.get(0).getQueryGapSequence().seqString());
        assertEquals("RRE", gaps.get(1).getQueryGapSequence().seqString());
    }

    public void testFindGaps22() throws IOException, IllegalSymbolException
    {
        String query =      "FE--IIDEGLRLKRERRE";
        String database =   "-EKLRR-KIYKRERG---";
        LoopInserter loopInserter = testGaps(query, database, "F", "EII", "IDE", "RRE");

        List<Gap> gaps = loopInserter.findGaps();

        // Test after merging
        gaps = loopInserter.mergeGaps(gaps);

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }

        // TODO these don't pass yet, but they are really rare pathological cases
        assertEquals("FEIID", gaps.get(0).getQueryGapSequence().seqString());
    }

    public void testFindGaps23() throws IOException, IllegalSymbolException
    {
        String query =      "FE--IDEGLRLKRERRE";
        String database =   "-EKLR-KIYKRERG---";
        LoopInserter loopInserter = testGaps(query, database, "F", "EID", "DEG", "RRE");

        List<Gap> gaps = loopInserter.findGaps();

        // Test after merging
        gaps = loopInserter.mergeGaps(gaps);

        for (Gap g : gaps)
        {
            System.out.print(g);
            System.out.println(",  " + g.getQueryGapSequence().seqString());
        }

        // TODO these don't pass yet, but they are really rare pathological cases
        assertEquals("FEID", gaps.get(0).getQueryGapSequence().seqString());
    }

    public void testFindGaps24() throws IOException, IllegalSymbolException
    {
        // TODO these don't pass yet, but they are really rare pathological cases
        String query =      "----DEGLRLKRERRE";
        String database =   "EKLR-KIYKRERG---";
        testGaps(query, database, "", "D", "RRE");
    }
    */
}
