package ca.ualberta.proteus.program.homodeller.test;

import ca.ualberta.proteus.program.homodeller.HomodellerImpl;
import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Created by Scott Montgomerie
 * Date: Sep 21, 2006
 */
public class HomodellerResultTest extends TestCase
{
    /**
     *
     * @param first the result that should be first after sorting
     * @param second the result that should be second after sorting
     */
    private void checkResult(HomodellerImpl.HomodellerOutputResult first, HomodellerImpl.HomodellerOutputResult second)
    {
        assertNotSame(first, second);

        List<HomodellerImpl.HomodellerOutputResult> list = new ArrayList<HomodellerImpl.HomodellerOutputResult>();
        list.add(first);
        list.add(second);

        Collections.sort(list);

        assertEquals( list.get(0), first);
        assertEquals( list.get(1), second);

        // reverse the orders of adding them to the list to make sure they don't appear equal
        list.clear();

        list.add(second);
        list.add(first);

        Collections.sort(list);

        assertEquals( list.get(0), first);
        assertEquals( list.get(1), second);

        // check sgn(x.compareTo(y)) == -sgn(y.compareTo(x))
        assertTrue( first.compareTo(second) < 0 );
        assertTrue( second.compareTo(first) > 0 );
        assertTrue( Math.signum(first.compareTo(second)) == -Math.signum(second.compareTo(first)) );
    }

    /**
     *
     * @param first the result that should be first after sorting
     * @param second the result that should be second after sorting
     * @param third the result that should be third after sorting
     */
    private void checkResult(HomodellerImpl.HomodellerOutputResult first, HomodellerImpl.HomodellerOutputResult second, HomodellerImpl.HomodellerOutputResult third)
    {
        assertNotSame(first, second);
        assertNotSame(first, third);
        assertNotSame(second, third);

        List<HomodellerImpl.HomodellerOutputResult> list = new ArrayList<HomodellerImpl.HomodellerOutputResult>();
        list.add(first);
        list.add(second);
        list.add(third);

        Collections.sort(list);

        assertEquals( list.get(0), first);
        assertEquals( list.get(1), second);
        assertEquals( list.get(2), third);

        // reverse the orders of adding them to the list to make sure they don't appear equal
        list.clear();

        list.add(second);
        list.add(third);
        list.add(first);

        Collections.sort(list);

        assertEquals( list.get(0), first);
        assertEquals( list.get(1), second);
        assertEquals( list.get(2), third);

        // check sgn(x.compareTo(y)) == -sgn(y.compareTo(x))
        assertTrue(first.compareTo(second) < 0);
        assertTrue(second.compareTo(first) > 0);
        assertTrue(Math.signum(first.compareTo(second)) == -Math.signum(second.compareTo(first)));

        // check transitive
        if (first.compareTo(second) < 0 && second.compareTo(third) < 0)
        {
            assertTrue(first.compareTo(third) < 0);
        }
    }

    public void testResultSorting1()
    {

        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.95;
        r1.hasLigand = true;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.5;
        r2.hasLigand = false;

        // under threshold, the one without the ligand should be first
        checkResult(r2, r1);
    }

    public void testResultSorting1A()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.80;
        r1.hasLigand = true;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.5;
        r2.hasLigand = false;

        // under the threshold, the one without the ligand should be first
        checkResult(r2, r1);
    }

    public void testResultSorting1B()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.95;
        r1.hasLigand = true;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.4;
        r2.hasLigand = false;

        // over the threshold, the one with the ligand should be first
        checkResult(r1, r2);
    }

    public void testResultSorting2()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.5;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;

        // the one with higher sequence id% (identity %) is first
        checkResult(r2, r1);
    }

    public void testResultSorting2A()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.90;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;

        // the one with higher sequence id% (identity %) is first
        checkResult(r2, r1);
    }

    public void testResultSorting3()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.5;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        // here technique shouldn't matter, the one with higher sequence id% (identity %) is first
        checkResult(r2, r1);
    }
    
    public void testResultSorting4()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.80;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        // here technique shouldn't matter, the one with higher sequence id% (identity %) is first
        checkResult(r2, r1);
    }

    public void testResultSorting5()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.80;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        // over the threshold of 20%, take the X-ray structure
        checkResult(r1, r2);
    }

    public void testResultSorting6()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.55;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.95;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        // below the threshold of 20%, we'll take the NMR structure
        checkResult(r2, r1);
    }

    public void testResultSorting7()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.55;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.75;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;

        // base case
        checkResult(r3, r2, r1);
    }


    public void testResultSorting8()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.55;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.80;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;
        r3.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        // xray should be first, then by seq %
        checkResult(r2, r3, r1);
    }

    public void testResultSorting9()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.55;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.60;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;
        r3.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        checkResult(r3, r2, r1);
    }

    public void testResultSorting10()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.75;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.60;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;
        r3.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        checkResult(r3, r2, r1);
    }

    public void testResultSorting11()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.75;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.60;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;
        r2.hasLigand = true;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;
        r3.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;
        r3.hasLigand = true;

        checkResult(r1, r3, r2);
    }

    public void testResultSorting11A()
    {
        HomodellerImpl.HomodellerOutputResult r1 = new HomodellerImpl.HomodellerOutputResult();
        r1.sequenceSimilarity = 0.75;
        r1.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        HomodellerImpl.HomodellerOutputResult r2 = new HomodellerImpl.HomodellerOutputResult();
        r2.sequenceSimilarity = 0.60;
        r2.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.XRAY;
        r2.hasLigand = true;

        HomodellerImpl.HomodellerOutputResult r3 = new HomodellerImpl.HomodellerOutputResult();
        r3.sequenceSimilarity = 0.95;
        r3.technique = HomodellerImpl.HomodellerOutputResult.TECHNIQUE.NMR;

        checkResult(r3, r1, r2);
    }
}
