package ca.ualberta.proteus.program.homodeller.test;

import ca.ualberta.proteus.program.homodeller.Loop;
import ca.ualberta.proteus.program.homodeller.LoopDB;
import ca.ualberta.proteus.program.homodeller.LoopDBReader;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.structure.Group;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * LoopDBReader Tester.
 *
 * @author <Authors name>
 * @since <pre>08/31/2006</pre>
 * @version 1.0
 */
public class LoopDBReaderTest extends TestCase {
    public LoopDBReaderTest(String name) {
        super(name);
    }

    public void setUp() throws Exception {
        super.setUp();
    }

    public void tearDown() throws Exception {
        super.tearDown();
    }

    public static Test suite() {
        return new TestSuite(LoopDBReaderTest.class);
    }

    public void testParseLoopDB() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db, false);

        //reader.parseLoopDB(new File("/home/scott/homodeller/newloopDB"));

        String input = ">1:3:1\n" +
                "        6       ASN     N   BBB B    2.067   -1.206    0.000\n" +
                "        6       ASN     CA  BBB B    3.519   -1.436    0.000\n" +
                "        6       ASN     C   BBB B    3.951   -1.730   -1.438\n" +
                "        6       ASN     O   BBB B    3.421   -2.641   -2.091\n" +
                "        6       ASN     CB  BBB B    3.845   -2.639    0.887\n" +
                "        7       TRP     N   BBB B    4.909   -0.938   -1.883\n" +
                "        7       TRP     CA  BBB B    5.474   -1.048   -3.236\n" +
                "        7       TRP     C   BBB B    5.946   -2.489   -3.435\n" +
                "        7       TRP     O   BBB B    6.576   -3.083   -2.548\n" +
                "        7       TRP     CB  BBB B    6.665   -0.097   -3.370\n" +
                "        8       CYS     N   BBB B    5.621   -3.003   -4.607\n" +
                "        8       CYS     CA  BBB B    5.975   -4.373   -5.006\n" +
                "        8       CYS     C   BBB B    7.165   -4.293   -5.965\n" +
                "        8       CYS     O   BBB B    7.381   -3.269   -6.631\n" +
                "        8       CYS     CB  BBB B    4.787   -5.020   -5.721\n" +
                "        9       GLY     N   CCC C    7.902   -5.388   -5.999\n" +
                "        9       GLY     CA  CCC C    9.092   -5.526   -6.851\n" +
                "        9       GLY     C   CCC C    9.485   -7.004   -6.877\n" +
                "        9       GLY     O   CCC C    8.677   -7.874   -7.233\n" +
                ">1:3:2\n" +
                "        6       ASN     N   BBB B    2.067   -1.206    0.000\n" +
                "        6       ASN     CA  BBB B    3.519   -1.436    0.000\n" +
                "        6       ASN     C   BBB B    3.951   -1.730   -1.438\n" +
                "        6       ASN     O   BBB B    3.421   -2.641   -2.091\n" +
                "        6       ASN     CB  BBB B    3.845   -2.639    0.887\n" +
                "        7       TRP     N   BBB B    4.909   -0.938   -1.883\n" +
                "        7       TRP     CA  BBB B    5.474   -1.048   -3.236\n" +
                "        7       TRP     C   BBB B    5.946   -2.489   -3.435\n" +
                "        7       TRP     O   BBB B    6.576   -3.083   -2.548\n" +
                "        7       TRP     CB  BBB B    6.665   -0.097   -3.370\n" +
                "        8       CYS     N   BBB B    5.621   -3.003   -4.607\n" +
                "        8       CYS     CA  BBB B    5.975   -4.373   -5.006\n" +
                "        8       CYS     C   BBB B    7.165   -4.293   -5.965\n" +
                "        8       CYS     O   BBB B    7.381   -3.269   -6.631\n" +
                "        8       CYS     CB  BBB B    4.787   -5.020   -5.721\n" +
                "        9       GLY     N   CCC C    7.902   -5.388   -5.999\n" +
                "        9       GLY     CA  CCC C    9.092   -5.526   -6.851\n" +
                "        9       GLY     C   CCC C    9.485   -7.004   -6.877\n" +
                "        9       GLY     O   CCC C    8.677   -7.874   -7.233\n" +
                "        10      LYS     N   CCC C   10.727   -7.235   -6.494\n" +
                "        10      LYS     CA  CCC C   11.312   -8.583   -6.444\n" +
                "        10      LYS     C   CCC C   10.444   -9.441   -5.520\n" +
                "        10      LYS     O   CCC C    9.434   -8.971   -4.976\n" +
                "        10      LYS     CB  CCC C   12.733   -8.503   -5.882\n" +
                ">1:3:3\n" +
                "        6       ASN     N   BBB B    2.067   -1.206    0.000\n" +
                "        6       ASN     CA  BBB B    3.519   -1.436    0.000\n" +
                "        6       ASN     C   BBB B    3.951   -1.730   -1.438\n" +
                "        6       ASN     O   BBB B    3.421   -2.641   -2.091\n" +
                "        6       ASN     CB  BBB B    3.845   -2.639    0.887\n" +
                "        7       TRP     N   BBB B    4.909   -0.938   -1.883\n" +
                "        7       TRP     CA  BBB B    5.474   -1.048   -3.236\n" +
                "        7       TRP     C   BBB B    5.946   -2.489   -3.435\n" +
                "        7       TRP     O   BBB B    6.576   -3.083   -2.548\n" +
                "        7       TRP     CB  BBB B    6.665   -0.097   -3.370\n" +
                "        8       CYS     N   BBB B    5.621   -3.003   -4.607\n" +
                "        8       CYS     CA  BBB B    5.975   -4.373   -5.006\n" +
                "        8       CYS     C   BBB B    7.165   -4.293   -5.965\n" +
                "        8       CYS     O   BBB B    7.381   -3.269   -6.631\n" +
                "        8       CYS     CB  BBB B    4.787   -5.020   -5.721";

        reader.parseDB(new BufferedReader(new StringReader(input)));

        assertEquals(3, db.size());

        Loop loop = db.get(0);
        ArrayList groups = loop.getGroups();

        assertEquals(4, groups.size());

        Group a1 = (Group) groups.get(0);
        Group a2 = (Group) groups.get(1);
        Group a3 = (Group) groups.get(2);
        Group a4 = (Group) groups.get(3);

        assertEquals(a1.getAtoms().size(), 5);
        assertEquals(a2.getAtoms().size(), 5);
        assertEquals(a3.getAtoms().size(), 5);
        assertEquals(a4.getAtoms().size(), 4);

        // # 2
        loop = db.get(1);
        groups = loop.getGroups();

        assertEquals(5, groups.size());

        a1 = (Group) groups.get(0);
        a2 = (Group) groups.get(1);
        a3 = (Group) groups.get(2);
        a4 = (Group) groups.get(3);

        assertEquals(a1.getAtoms().size(), 5);
        assertEquals(a2.getAtoms().size(), 5);
        assertEquals(a3.getAtoms().size(), 5);
        assertEquals(a4.getAtoms().size(), 4);

        // # 3
        loop = db.get(2);
        groups = loop.getGroups();

        assertEquals(3, groups.size());

        a1 = (Group) groups.get(0);
        a2 = (Group) groups.get(1);
        a3 = (Group) groups.get(2);

        assertEquals(a1.getAtoms().size(), 5);
        assertEquals(a2.getAtoms().size(), 5);
        assertEquals(a3.getAtoms().size(), 5);
    }

    public void testParseLoopLargeDB() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db);

        reader.parseDB(new File("/Users/darndt/Documents/workspace/proteus2/src/homodeller/newloopDB"));

        assertTrue(db.size() > 1);
    }

    public void testParseLoopDBSmall() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db);

        reader.parseDB(new File("/Users/darndt/Documents/workspace/proteus2/src/homodeller/data/looppdb.new"));

        assertTrue(db.size() > 1);
    }

    public void testParseLoopDBSmallWithoutLazyParsing() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db, false );

        reader.parseDB(new File("/Users/darndt/Documents/workspace/proteus2/src/homodeller/data/looppdb.new"));

        assertTrue(db.size() > 1);
    }

    public void testFillInLoop() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db);

        File file = new File("/Users/darndt/Documents/workspace/proteus2/src/homodeller/data/looppdb.new");
        reader.parseDB(file);

        assertTrue(db.size() > 1);

        Loop loop = db.get(0);

        assertEquals(0, loop.getGroups().size());

        reader.fillInLoopInfo(file, loop);

        assertTrue(loop.getGroups().size() > 0);

        System.out.println(loop);

        // #2
        loop = db.get(1);
        assertEquals(0, loop.getGroups().size());
        reader.fillInLoopInfo(file, loop);
        assertTrue(loop.getGroups().size() > 0);
        System.out.println(loop);

        // #3
        loop = db.get(2);
        assertEquals(0, loop.getGroups().size());
        reader.fillInLoopInfo(file, loop);
        assertTrue(loop.getGroups().size() > 0);
        System.out.println(loop);
    }

    public void testFillInLoops() throws IOException
    {
        LoopDB db = new LoopDB();
        LoopDBReader reader = new LoopDBReader(db);

        File file = new File("/Users/darndt/Documents/workspace/proteus2/src/homodeller/data/looppdb.new");
        reader.parseDB(file);

        assertTrue(db.size() > 1);


        List<Loop> loops = db.getLoopsBySize(10);

        for ( Loop loop : loops )
        {
            assertEquals(0, loop.getGroups().size());

            reader.fillInLoopInfo(file, loop);

            assertTrue(loop.getGroups().size() > 0);

            System.out.println(loop);
        }
    }
}
