package ca.ualberta.proteus.program.homodeller.test;

import ca.ualberta.proteus.program.homodeller.SideChain;
import ca.ualberta.proteus.program.homodeller.SideDB;
import ca.ualberta.proteus.program.homodeller.SideDBReader;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.structure.Atom;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;

/**
 * SideDBReader Tester.
 *
 * @author <Authors name>
 * @since <pre>09/03/2006</pre>
 * @version 1.0
 */
public class SideDBReaderTest extends TestCase {
    public SideDBReaderTest(String name) {
        super(name);
    }

    public void setUp() throws Exception {
        super.setUp();
    }

    public void tearDown() throws Exception {
        super.tearDown();
    }

    public static Test suite() {
        return new TestSuite(SideDBReaderTest.class);
    }

    public void testParse() throws IOException
    {
        SideDB db = new SideDB();
        SideDBReader reader = new SideDBReader(db);

        //reader.parseLoopDB(new File("/home/scott/homodeller/newloopDB"));

        String input = ">A -177.5 B 0.3 C 0.2 H 0.0 \n" +
                "ATOM    236  N   ALA    30      -3.403  25.827  12.109  1.00  8.26  \n" +
                "ATOM    237  CA  ALA    30      -2.289  26.744  11.966  1.00  9.10  \n" +
                "ATOM    238  C   ALA    30      -1.177  26.372  12.961  1.00  9.20  \n" +
                "ATOM    239  O   ALA    30       0.006  26.362  12.610  1.00  9.57  \n" +
                "ATOM    240  CB  ALA    30      -2.768  28.172  12.222  1.00  9.73  \n" +
                "\n" +
                ">A 61.0 B 0.0 C 0.0 H 0.0\n" +
                "ATOM    514  N   ALA    64       4.274  14.767  29.158  1.00 12.89  \n" +
                "ATOM    515  CA  ALA    64       4.423  13.358  28.861  1.00 12.23  \n" +
                "ATOM    516  C   ALA    64       5.747  12.870  29.416  1.00 12.59  \n" +
                "ATOM    517  O   ALA    64       6.638  13.667  29.744  1.00 12.56  \n" +
                "ATOM    518  CB  ALA    64       4.334  13.116  27.349  1.00 11.52  \n" +
                "\n" +
                ">A -60.4 B 0.7 C 0.8 H 1.0\n" +
                "ATOM    631  N   ALA    80       5.737   9.927  25.894  1.00 10.92  \n" +
                "ATOM    632  CA  ALA    80       5.801  10.893  24.806  1.00 10.10  \n" +
                "ATOM    633  C   ALA    80       5.709  10.222  23.439  1.00 10.86  \n" +
                "ATOM    634  O   ALA    80       5.198  10.815  22.476  1.00 11.57  \n" +
                "ATOM    635  CB  ALA    80       7.073  11.726  24.915  1.00  9.50  \n" +
                "\n" +
                ">C -177.5 B 0.3 C 0.2 H 0.0 \n" +
                "ATOM    236  N   CYS    30      -3.403  25.827  12.109  1.00  8.26  \n" +
                "ATOM    237  CA  CYS    30      -2.289  26.744  11.966  1.00  9.10  \n" +
                "ATOM    238  C   CYS    30      -1.177  26.372  12.961  1.00  9.20  \n" +
                "ATOM    239  O   CYS    30       0.006  26.362  12.610  1.00  9.57  \n" +
                "ATOM    240  CB  CYS    30      -2.768  28.172  12.222  1.00  9.73  \n" +
                "ATOM    241  SG  CYS    30      -1.481  29.428  12.004  1.00 11.14  ";

        reader.parseDB(new BufferedReader(new StringReader(input)));

        assertEquals(4, db.size());

        SideChain s0 = db.get(0);
        SideChain s1 = db.get(1);
        SideChain s2 = db.get(2);
        SideChain s3 = db.get(3);

        assertEquals(5, s0.getAtoms().size() );
        assertEquals(-177.5, s0.getAngle());

        assertEquals(5, s1.getAtoms().size() );
        assertEquals(5, s2.getAtoms().size() );
        assertEquals(6, s3.getAtoms().size() );

        for (SideChain c :db)
        {
            System.out.println(c);
            List<Atom> atoms = c.getAtoms();
            for ( Atom a: atoms)
            {
                System.out.println(a);
            }
        }
    }

    public void testGetInstance()
    {
        SideDB db = SideDB.getInstance();

        SideChain s0 = db.get(0);
        SideChain s1 = db.get(1);
        SideChain s2 = db.get(2);
        SideChain s3 = db.get(3);

        assertEquals(5, s0.getAtoms().size() );
        assertEquals(-177.5, s0.getAngle());

        assertEquals(5, s1.getAtoms().size() );
        assertEquals(5, s2.getAtoms().size() );
        assertEquals(6, s3.getAtoms().size() );
    }

    public void testGetType()
    {
        SideDB db = SideDB.getInstance();
        List<SideChain> sideChains = db.getSideChains('A');

        assertTrue(sideChains.size() > 0);
        
        for ( SideChain c: sideChains)
        {
            System.out.println(c);
        }
    }
}
