package ca.ualberta.proteus.program.homodeller.test;

import ca.ualberta.proteus.program.homodeller.Superpose;
import ca.ualberta.proteus.program.homodeller.HomodellerUtils;
import ca.ualberta.proteus.util.PDBFileLoader;
import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.structure.*;
import org.biojava.bio.structure.io.PDBFileParser;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Superpose Tester.
 *
 * @author <Authors name>
 * @version 1.0
 * @since <pre>09/04/2006</pre>
 */
public class SuperposeTest extends TestCase
{
    public SuperposeTest(String name)
    {
        super(name);
    }

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public static Test suite()
    {
        return new TestSuite(SuperposeTest.class);
    }

    public void testSuperpose()
    {
        double[][] pqr = new double[][]
                {{42.833, 64.280, 52.163},
                        {44.176, 63.734, 52.290},
                        {44.733, 63.898, 50.844}};
        double[][] abc = new double[][]
                {{-17.775, 41.398, 46.220},
                        {-16.791, 40.917, 45.201},
                        {-16.728, 39.422, 45.076}};

        // points taken from running the C version of superpose
        double[] point;
        //point = Superpose.superpose(abc, pqr, 42.833, 64.280, 52.163);

        point = Superpose.superpose(abc, pqr, 44.176, 63.734, 52.290);
        assertEqualsRounded(-16.818, point[0]);
        assertEqualsRounded(40.930, point[1]);
        assertEqualsRounded(45.229, point[2]);

        /*
        point = Superpose.superpose(abc, pqr, 44.733, 63.898 ,50.844);

        point = Superpose.superpose(abc, pqr, 44.493,  63.039,  49.987);
        */
    }


    public void testSuperposeAllAtoms() throws IOException
    {
        Atom[] original = new Atom[6];
        Atom[] target = new Atom[6];
        Chain chain = null;

        original[0] = new AtomImpl();
        original[0].setCoords(new double[] {7.131, -2.784, -4.269});
        original[1] = new AtomImpl();
        original[1].setCoords(new double[] {8.259 , -3.723 , -4.343});
        original[2] = new AtomImpl();
        original[2].setCoords(new double[] {9.503,  -3.012 , -3.805});
        original[3] = new AtomImpl();
        original[3].setCoords(new double[] {5.713,   1.954,   1.589});
        original[4] = new AtomImpl();
        original[4].setCoords(new double[] {4.356,   1.461,   1.316});
        original[5] = new AtomImpl();
        original[5].setCoords(new double[] {3.388,   2.638,   1.458});

        target[0] = new AtomImpl();
        target[0].setCoords(new double[] {-11.658,  42.209,  11.266});
        target[1] = new AtomImpl();
        target[1].setCoords(new double[] {-12.706,  42.490,  12.323});
        target[2] = new AtomImpl();
        target[2].setCoords(new double[] {-14.022,  41.888,  11.827});
        target[3] = new AtomImpl();
        target[3].setCoords(new double[] {-18.161,  34.827,  15.497});
        target[4] = new AtomImpl();
        target[4].setCoords(new double[] {-19.045,  35.322,  16.512});
        target[5] = new AtomImpl();
        target[5].setCoords(new double[] {-20.453,  34.778, 16.334});

        String pdb = "ATOM      1  N   GLY   205       7.131  -2.784  -4.269  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      2  CA  GLY   205       8.259  -3.723  -4.343  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      3  C   GLY   205       9.503  -3.012  -3.805  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      4  O   GLY   205      10.392  -3.642  -3.213  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      5  N   HIS   206       9.518  -1.712  -4.033  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      6  CA  HIS   206      10.621  -0.838  -3.608  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      7  C   HIS   206      10.025   0.511  -3.203  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      8  O   HIS   206       9.123   1.038  -3.871  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      9  CB  HIS   206      11.593  -0.638  -4.773  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     10  N   GLY   207      10.558   1.027  -2.110  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     11  CA  GLY   207      10.155   2.330  -1.562  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     12  C   GLY   207       9.173   2.077  -0.416  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     13  O   GLY   207       8.876   0.923  -0.072  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     14  N   ASN   208       8.700   3.177   0.139  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     15  CA  ASN   208       7.752   3.164   1.262  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     16  C   ASN   208       6.398   2.689   0.732  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     17  O   ASN   208       6.017   2.991  -0.408  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     18  CB  ASN   208       7.611   4.578   1.829  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     19  N   TYR   209       5.713   1.954   1.589  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     20  CA  TYR   209       4.356   1.461   1.316  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     21  C   TYR   209       3.388   2.638   1.458  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     22  O   TYR   209       3.756   3.711   1.958  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     23  CB  TYR   209       3.993   0.376   2.331  1.00  1.00 pre_fist_loop.pdb";

        String resultPDB = "ATOM      1  N   GLY   205     -13.072  40.759  13.160  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      2  CA  GLY   205     -13.138  41.991  12.362  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      3  C   GLY   205     -14.027  41.721  11.146  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      4  O   GLY   205     -14.721  42.621  10.650  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      5  N   HIS   206     -13.972  40.477  10.707  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      6  CA  HIS   206     -14.740  40.003   9.547  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      7  C   HIS   206     -15.164  38.559   9.820  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      8  O   HIS   206     -14.381  37.749  10.338  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM      9  CB  HIS   206     -13.858  40.048   8.297  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     10  N   GLY   207     -16.405  38.287   9.457  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     11  CA  GLY   207     -17.006  36.953   9.601  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     12  C   GLY   207     -17.860  36.954  10.870  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     13  O   GLY   207     -17.982  37.979  11.557  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     14  N   ASN   208     -18.427  35.792  11.138  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     15  CA  ASN   208     -19.296  35.574  12.302  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     16  C   ASN   208     -18.420  35.587  13.556  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     17  O   ASN   208     -17.270  35.125  13.535  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     18  CB  ASN   208     -19.988  34.215  12.177  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     19  N   TYR   209     -19.002  36.122  14.613  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     20  CA  TYR   209     -18.371  36.164  15.939  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     21  C   TYR   209     -18.436  34.758  16.539  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     22  O   TYR   209     -19.136  33.875  16.022  1.00  1.00 pre_fist_loop.pdb\n" +
                "ATOM     23  CB  TYR   209     -19.135  37.138  16.838  1.00  1.00 pre_fist_loop.pdb";

        PDBFileParser parser = new PDBFileParser();
        Structure structure = parser.parsePDBFile(new BufferedReader(new StringReader(pdb)));
        chain = structure.getChain(0);

        structure = parser.parsePDBFile(new BufferedReader(new StringReader(resultPDB)));
        Chain rotatedChain = structure.getChain(0);

        Superpose.superpose(original, target, chain);

        ArrayList<Group> groups = chain.getGroups();
        ArrayList<Group> rotatedGroups = rotatedChain.getGroups();
        for(int i=0; i<groups.size(); i++)
        {
            List<Atom> atoms = groups.get(i).getAtoms();
            List<Atom> rotatedAtoms = rotatedGroups.get(i).getAtoms();

            for ( int j=0; j<atoms.size(); j++)
            {
                Atom a = atoms.get(j);
                Atom rotated = rotatedAtoms.get(j);

                assertEqualsRounded(a.getX(), rotated.getX());
                assertEqualsRounded(a.getY(), rotated.getY());
                assertEqualsRounded(a.getZ(), rotated.getZ());
            }
        }
    }

    public void testFindRMSD() throws IOException
    {
        String newchainStr =
                        "ATOM      1  N   MET A   1     -17.775  41.398  46.220     1 26.26\n" +
                        "ATOM      2  CA  MET A   1     -16.791  40.917  45.201     1 30.69\n" +
                        "ATOM      3  C   MET A   1     -16.728  39.422  45.076     1  28.4\n" +
                        "ATOM      4  O   MET A   1     -17.049  38.703  45.989     1 27.93\n" +
                        "ATOM      5  CB  MET A   1     -15.361  41.348  45.496     1 36.41\n" +
                        "ATOM      6  CG  MET A   1     -14.798  41.991  44.270     1 35.76\n" +
                        "ATOM      7  SD  MET A   1     -15.511  43.625  44.098     1 39.67\n" +
                        "ATOM      8  CE  MET A   1     -14.955  44.413  45.619     1 43.27\n" +
                        "ATOM      9  N   ASN A   2     -16.264  38.989  43.924     1 27.73\n" +
                        "ATOM     10  CA  ASN A   2     -16.102  37.600  43.507     1 21.66\n" +
                        "ATOM     11  C   ASN A   2     -15.736  37.742  42.066     1 27.17\n" +
                        "ATOM     12  O   ASN A   2     -15.881  38.814  41.468     1 25.93\n" +
                        "ATOM     13  CB  ASN A   2     -17.406  36.802  43.480     1  17.2\n" +
                        "ATOM     14  CG  ASN A   2     -18.552  37.467  42.656     1  10.5\n" +
                        "ATOM     15  OD1 ASN A   2     -18.490  37.650  41.424     1 37.02\n" +
                        "ATOM     16  ND2 ASN A   2     -19.682  37.722  43.304     1 39.84\n" +
                        "ATOM     17  N   ILE A   3     -15.321  36.644  41.516     1 28.45\n" +
                        "ATOM     18  CA  ILE A   3     -14.816  36.592  40.185     1 24.01\n" +
                        "ATOM     19  C   ILE A   3     -15.471  37.455  39.118     1  26.5\n" +
                        "ATOM     20  O   ILE A   3     -14.839  38.352  38.547     1 27.42\n" +
                        "ATOM     21  CB  ILE A   3     -14.385  35.219  39.794     1  20.3\n" +
                        "ATOM     22  CG1 ILE A   3     -13.456  35.383  38.605     1 23.37\n" +
                        "ATOM     23  CG2 ILE A   3     -15.608  34.396  39.475     1 24.57\n" +
                        "ATOM     24  CD1 ILE A   3     -12.597  36.606  38.871     1 32.25\n" +
                        "ATOM     25  N   PHE A   4     -16.749  37.239  38.926     1  20.6\n" +
                        "ATOM     26  CA  PHE A   4     -17.479  38.021  37.969     1 11.51\n" +
                        "ATOM     27  C   PHE A   4     -17.371  39.469  38.268     1 16.87\n" +
                        "ATOM     28  O   PHE A   4     -17.375  40.303  37.419     1  28.4\n" +
                        "ATOM     29  CB  PHE A   4     -18.928  37.583  38.043     1  6.71\n" +
                        "ATOM     30  CG  PHE A   4     -19.007  36.150  37.657     1 30.85\n" +
                        "ATOM     31  CD1 PHE A   4     -18.681  35.190  38.607     1 28.74\n" +
                        "ATOM     32  CD2 PHE A   4     -19.282  35.741  36.355     1 28.34\n" +
                        "ATOM     33  CE1 PHE A   4     -18.681  33.831  38.306     1 26.72\n" +
                        "ATOM     34  CE2 PHE A   4     -19.250  34.387  36.033     1 27.24\n" +
                        "ATOM     35  CZ  PHE A   4     -18.967  33.432  37.005     1 25.58\n" +
                        "ATOM     36  N   GLU A   5     -17.269  39.782  39.479     1 16.35\n" +
                        "ATOM     37  CA  GLU A   5     -17.154  41.144  39.768     1 22.59\n" +
                        "ATOM     38  C   GLU A   5     -15.811  41.672  39.431     1 27.52\n" +
                        "ATOM     39  O   GLU A   5     -15.698  42.756  38.902     1 40.26\n" +
                        "ATOM     40  CB  GLU A   5     -17.548  41.361  41.201     1 31.54\n" +
                        "ATOM     41  CG  GLU A   5     -18.608  42.421  41.255     1   100\n" +
                        "ATOM     42  CD  GLU A   5     -19.910  41.884  40.815     1 40.74\n" +
                        "ATOM     43  OE1 GLU A   5     -19.893  41.189  39.696     1 99.52\n" +
                        "ATOM     44  OE2 GLU A   5     -20.917  42.137  41.432     1   100";

        String realChainStr =
                        "ATOM      1  N   MET     1     -17.775  41.398  46.220     1  1.00\n" +
                        "ATOM      2  CA  MET     1     -16.791  40.917  45.201     1  1.00\n" +
                        "ATOM      3  C   MET     1     -16.728  39.422  45.076     1  1.00\n" +
                        "ATOM      4  O   MET     1     -17.049  38.703  45.989     1  1.00\n" +
                        "ATOM      5  CB  MET     1     -15.445  41.427  45.695     1  1.00\n" +
                        "ATOM      6  CG  MET     1     -14.291  41.308  44.736     1  1.00\n" +
                        "ATOM      7  SD  MET     1     -12.911  42.375  45.237     1  1.00\n" +
                        "ATOM      8  CE  MET     1     -12.366  41.727  46.780     1  1.00\n" +
                        "ATOM      9  N   ASN     2     -16.264  38.989  43.924     1  1.00\n" +
                        "ATOM     10  CA  ASN     2     -16.102  37.600  43.507     1  1.00\n" +
                        "ATOM     11  C   ASN     2     -15.736  37.742  42.066     1  1.00\n" +
                        "ATOM     12  O   ASN     2     -15.881  38.814  41.468     1  1.00\n" +
                        "ATOM     13  CB  ASN     2     -17.367  36.791  43.800     1  1.00\n" +
                        "ATOM     14  CG  ASN     2     -18.561  37.260  43.017     1  1.00\n" +
                        "ATOM     15  OD1 ASN     2     -18.474  38.275  42.309     1  1.00\n" +
                        "ATOM     16  ND2 ASN     2     -19.618  36.620  43.108     1  1.00\n" +
                        "ATOM     17  N   ILE     3     -15.321  36.644  41.516     1  1.00\n" +
                        "ATOM     18  CA  ILE     3     -14.816  36.592  40.185     1  1.00\n" +
                        "ATOM     19  C   ILE     3     -15.471  37.455  39.118     1  1.00\n" +
                        "ATOM     20  O   ILE     3     -14.839  38.352  38.547     1  1.00\n" +
                        "ATOM     21  CB  ILE     3     -14.731  35.106  39.717     1  1.00\n" +
                        "ATOM     22  CG1 ILE     3     -13.716  34.965  38.586     1  1.00\n" +
                        "ATOM     23  CG2 ILE     3     -16.122  34.559  39.387     1  1.00\n" +
                        "ATOM     24  CD1 ILE     3     -13.188  33.574  38.421     1  1.00\n" +
                        "ATOM     25  N   PHE     4     -16.749  37.239  38.926     1  1.00\n" +
                        "ATOM     26  CA  PHE     4     -17.479  38.021  37.969     1  1.00\n" +
                        "ATOM     27  C   PHE     4     -17.371  39.469  38.268     1  1.00\n" +
                        "ATOM     28  O   PHE     4     -17.375  40.303  37.419     1  1.00\n" +
                        "ATOM     29  CB  PHE     4     -18.892  37.466  37.792     1  1.00\n" +
                        "ATOM     30  CG  PHE     4     -18.893  36.068  37.237     1  1.00\n" +
                        "ATOM     31  CD1 PHE     4     -18.180  35.779  36.065     1  1.00\n" +
                        "ATOM     32  CD2 PHE     4     -19.537  35.036  37.905     1  1.00\n" +
                        "ATOM     33  CE1 PHE     4     -18.104  34.468  35.572     1  1.00\n" +
                        "ATOM     34  CE2 PHE     4     -19.469  33.722  37.424     1  1.00\n" +
                        "ATOM     35  CZ  PHE     4     -18.751  33.439  36.252     1  1.00\n" +
                        "ATOM     36  N   GLU     5     -17.269  39.782  39.479     1  1.00\n" +
                        "ATOM     37  CA  GLU     5     -17.154  41.144  39.768     1  1.00\n" +
                        "ATOM     38  C   GLU     5     -15.811  41.672  39.431     1  1.00\n" +
                        "ATOM     39  O   GLU     5     -15.698  42.756  38.902     1  1.00\n" +
                        "ATOM     40  CB  GLU     5     -17.386  41.455  41.266     1  1.00\n" +
                        "ATOM     41  CG  GLU     5     -17.370  42.925  41.654     1  1.00\n" +
                        "ATOM     42  CD  GLU     5     -17.372  43.139  43.156     1  1.00\n" +
                        "ATOM     43  OE1 GLU     5     -17.043  42.191  43.898     1  1.00\n" +
                        "ATOM     44  OE2 GLU     5     -17.669  44.270  43.606     1  1.00";

        PDBFileParser parser = new PDBFileParser();
        Structure newchainStructure = parser.parsePDBFile(new BufferedReader(new StringReader(newchainStr)));
        Chain newchain = newchainStructure.getChain(0);

        newchainStructure = parser.parsePDBFile(new BufferedReader(new StringReader(realChainStr)));
        Chain realChain = newchainStructure.getChain(0);

        Superpose.RMSD rmsd1 = Superpose.findRMSD(realChain, newchain);

        writeChainToFile("real.pdb", realChain);
        writeChainToFile("new.pdb", newchain);

        Superpose.RMSD rmsd2 = Superpose.findRMSD(newchain, realChain);

        // test reciprocity (that order of chains doesn't matter)
        assertEquals(rmsd2, rmsd1);

        System.out.println("rmsd = " + rmsd2);
    }

    public void testFindRMSDFromFiles() throws IOException
    {
        PDBFileParser parser = new PDBFileParser();
        Structure newchainStructure = parser.parsePDBFile(new BufferedReader(new FileReader("/home/scott/proteus_web/homodeller/102L.homodeller.pdb")));
        Chain newchain = newchainStructure.getChain(0);

        InputStream in = PDBFileLoader.getPDBFile("102LA", new File[]{new File("/home/scott/homodeller/pdb")});

        newchainStructure = parser.parsePDBFile(in);
        Chain realChain = newchainStructure.getChain(0);

        HomodellerUtils.removeHetatoms(realChain);

        Superpose.RMSD rmsd1 = Superpose.findRMSD(realChain, newchain);

        writeChainToFile("real.pdb", realChain);
        writeChainToFile("new.pdb", newchain);

        Superpose.RMSD rmsd2 = Superpose.findRMSD(newchain, realChain);

        // test reflexion (that order of chains doesn't matter)
        assertEqualsRounded(rmsd2.all, rmsd1.all, 4);

        System.out.println("rmsd (all) = " + rmsd2.all);
        System.out.println("rmsd (backbone) = " + rmsd2.backbone);
        System.out.println("rmsd (CA) = " + rmsd2.CA);
        System.out.println("rmsd (sidechain) = " + rmsd2.sidechain);

        // these values are according to Swiss-PDBViewer
        assertEqualsRounded(1.52, rmsd2.all);
        assertEqualsRounded(1.20, rmsd2.backbone);
        assertEqualsRounded(1.23, rmsd2.CA);

        // for some reason this one is off (1.72 vs 1.79)
        // Swiss PDB must be excluding some side chains?  Maybe TRP/TYR rings?
        // assertEqualsRounded(1.79, rmsd2.sidechain);
    }

    public void testFindRMSDBaseCase() throws IOException
    {
        String newchainStr =
                        "ATOM      1  N   MET A   1     -17.775  41.398  46.220     1 26.26\n" +
                        "ATOM      2  CA  MET A   1     -16.791  40.917  45.201     1 30.69\n" +
                        "ATOM      3  C   MET A   1     -16.728  39.422  45.076     1  28.4\n" +
                        "ATOM      4  O   MET A   1     -17.049  38.703  45.989     1 27.93\n" +
                        "ATOM      5  CB  MET A   1     -15.361  41.348  45.496     1 36.41\n" +
                        "ATOM      6  CG  MET A   1     -14.798  41.991  44.270     1 35.76\n" +
                        "ATOM      7  SD  MET A   1     -15.511  43.625  44.098     1 39.67\n" +
                        "ATOM      8  CE  MET A   1     -14.955  44.413  45.619     1 43.27\n" +
                        "ATOM      9  N   ASN A   2     -16.264  38.989  43.924     1 27.73\n" +
                        "ATOM     10  CA  ASN A   2     -16.102  37.600  43.507     1 21.66\n" +
                        "ATOM     11  C   ASN A   2     -15.736  37.742  42.066     1 27.17\n" +
                        "ATOM     12  O   ASN A   2     -15.881  38.814  41.468     1 25.93\n" +
                        "ATOM     13  CB  ASN A   2     -17.406  36.802  43.480     1  17.2\n" +
                        "ATOM     14  CG  ASN A   2     -18.552  37.467  42.656     1  10.5\n" +
                        "ATOM     15  OD1 ASN A   2     -18.490  37.650  41.424     1 37.02\n" +
                        "ATOM     16  ND2 ASN A   2     -19.682  37.722  43.304     1 39.84\n" +
                        "ATOM     17  N   ILE A   3     -15.321  36.644  41.516     1 28.45\n" +
                        "ATOM     18  CA  ILE A   3     -14.816  36.592  40.185     1 24.01\n" +
                        "ATOM     19  C   ILE A   3     -15.471  37.455  39.118     1  26.5\n" +
                        "ATOM     20  O   ILE A   3     -14.839  38.352  38.547     1 27.42\n" +
                        "ATOM     21  CB  ILE A   3     -14.385  35.219  39.794     1  20.3\n" +
                        "ATOM     22  CG1 ILE A   3     -13.456  35.383  38.605     1 23.37\n" +
                        "ATOM     23  CG2 ILE A   3     -15.608  34.396  39.475     1 24.57\n" +
                        "ATOM     24  CD1 ILE A   3     -12.597  36.606  38.871     1 32.25\n" +
                        "ATOM     25  N   PHE A   4     -16.749  37.239  38.926     1  20.6\n" +
                        "ATOM     26  CA  PHE A   4     -17.479  38.021  37.969     1 11.51\n" +
                        "ATOM     27  C   PHE A   4     -17.371  39.469  38.268     1 16.87\n" +
                        "ATOM     28  O   PHE A   4     -17.375  40.303  37.419     1  28.4\n" +
                        "ATOM     29  CB  PHE A   4     -18.928  37.583  38.043     1  6.71\n" +
                        "ATOM     30  CG  PHE A   4     -19.007  36.150  37.657     1 30.85\n" +
                        "ATOM     31  CD1 PHE A   4     -18.681  35.190  38.607     1 28.74\n" +
                        "ATOM     32  CD2 PHE A   4     -19.282  35.741  36.355     1 28.34\n" +
                        "ATOM     33  CE1 PHE A   4     -18.681  33.831  38.306     1 26.72\n" +
                        "ATOM     34  CE2 PHE A   4     -19.250  34.387  36.033     1 27.24\n" +
                        "ATOM     35  CZ  PHE A   4     -18.967  33.432  37.005     1 25.58\n" +
                        "ATOM     36  N   GLU A   5     -17.269  39.782  39.479     1 16.35\n" +
                        "ATOM     37  CA  GLU A   5     -17.154  41.144  39.768     1 22.59\n" +
                        "ATOM     38  C   GLU A   5     -15.811  41.672  39.431     1 27.52\n" +
                        "ATOM     39  O   GLU A   5     -15.698  42.756  38.902     1 40.26\n" +
                        "ATOM     40  CB  GLU A   5     -17.548  41.361  41.201     1 31.54\n" +
                        "ATOM     41  CG  GLU A   5     -18.608  42.421  41.255     1   100\n" +
                        "ATOM     42  CD  GLU A   5     -19.910  41.884  40.815     1 40.74\n" +
                        "ATOM     43  OE1 GLU A   5     -19.893  41.189  39.696     1 99.52\n" +
                        "ATOM     44  OE2 GLU A   5     -20.917  42.137  41.432     1   100";


        PDBFileParser parser = new PDBFileParser();
        Structure newchainStructure = parser.parsePDBFile(new BufferedReader(new StringReader(newchainStr)));
        Chain chain1 = newchainStructure.getChain(0);

        Chain chain2 = (Chain) chain1.clone();

        // modify one of the chains' coordinates so they aren't exact
        ArrayList<Group> groups = chain1.getGroups();
        for (Group group : groups)
        {
            List<Atom> res = group.getAtoms();
            for (Atom a : res)
            {
                // Atom.clone() doesn't create a new array for the coordinates
                double[] coords = new double[3];
                for (int i = 0; i < a.getCoords().length; i++)
                {
                    double d = a.getCoords()[i];
                    coords[i] = d + 1;
                }

                a.setCoords(coords);
            }
        }

        Superpose.RMSD rmsd1 = Superpose.findRMSD(chain2, chain1);

        writeChainToFile("real.pdb", chain2);
        writeChainToFile("new.pdb", chain1);

        Superpose.RMSD rmsd2 = Superpose.findRMSD(chain1, chain2);

        // test reciprocity (that order of chains doesn't matter)
        assertEqualsRounded(rmsd2.all, rmsd1.all, 5);
        assertEqualsRounded(0.0, rmsd1.all, 5);

        System.out.println("rmsd (all) = " + rmsd2.all);
        System.out.println("rmsd (backbone) = " + rmsd2.backbone);
        System.out.println("rmsd (CA) = " + rmsd2.CA);
        System.out.println("rmsd (sidechain) = " + rmsd2.sidechain);
    }

    private void writeChainToFile(String filename, Chain chain)
    {
        StructureImpl tmp = new StructureImpl();

        tmp.addChain(chain);
        String pdbText = tmp.toPDB();
        File tempFile = new File(filename);
        tempFile.deleteOnExit();

        try
        {
            PrintWriter w = new PrintWriter(tempFile);
            w.print(pdbText);
            w.close();
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    private void assertEqualsRounded(double v, double v1, int decimals)
    {
        double rounder = Math.pow(10, decimals);
        double roundedV = Math.round(v * rounder) / rounder;
        double roundedV1 = Math.round(v1 * rounder) / rounder;
        assertEquals(roundedV , roundedV1 );

        rounder = Math.pow(10, 5);
        roundedV = Math.round(v * rounder) / rounder;
        roundedV1 = Math.round(v1 * rounder) / rounder;

        System.out.printf("Expected: %.5f\t Observed: %.5f\n", roundedV, roundedV1);
    }

    private void assertEqualsRounded(double v, double v1)
    {
        try
        {
            assertEqualsRounded(v, v1, 2);
        }
        catch (AssertionFailedError e)
        {
            if (Math.abs(v - v1) > 0.055)
            {
                throw e;
            }
            System.out.printf("Expected: %.5f\t Observed: %.5f\n", v, v1);
        }
    }

    public void testDifferentAtoms() throws IOException
    {
        String pdb1 = "ATOM     49  N   GLU C   7      41.438 -12.517  15.837     1 19.43\n" +
                "ATOM     50  CA  GLU C   7      42.490 -12.298  16.835     1 20.32\n" +
                "ATOM     51  C   GLU C   7      43.871 -12.237  16.147     1 19.03\n" +
                "ATOM     52  O   GLU C   7      44.698 -11.379  16.471     1 19.49\n" +
                "ATOM     53  CB AGLU C   7      42.485 -13.407  17.895   0.5  20.5\n" +
                "ATOM     54  CB BGLU C   7      42.434 -13.415  17.888   0.5 21.64\n" +
                "ATOM     55  CG AGLU C   7      43.486 -13.191  19.037   0.5 20.58\n" +
                "ATOM     56  CG BGLU C   7      41.124 -13.472  18.718   0.5 23.01\n" +
                "ATOM     57  CD AGLU C   7      43.575 -14.368  20.005   0.5 20.38\n" +
                "ATOM     58  CD BGLU C   7      39.859 -13.762  17.894   0.5 24.72\n" +
                "ATOM     59  OE1AGLU C   7      42.972 -15.439  19.741   0.5 19.67\n" +
                "ATOM     60  OE1BGLU C   7      39.736 -14.866  17.330   0.5 23.08\n" +
                "ATOM     61  OE2AGLU C   7      44.263 -14.215  21.033   0.5 19.04\n" +
                "ATOM     62  OE2BGLU C   7      38.972 -12.883  17.820   0.5 27.83";
        String pdb2 = "ATOM     49  N   GLU     7       1.024  13.537  42.118     1 12.28\n" +
                "ATOM     50  CA  GLU     7       0.594  14.915  42.279     1 10.82\n" +
                "ATOM     51  C   GLU     7       1.104  15.486  43.607     1 18.76\n" +
                "ATOM     52  O   GLU     7       1.595  16.613  43.689     1 15.52\n" +
                "ATOM     53  CB  GLU     7      -0.941  14.948  42.245     1 14.09\n" +
                "ATOM     54  CG  GLU     7      -1.524  16.362  42.325     1    19\n" +
                "ATOM     55  CD  GLU     7      -3.019  16.380  42.491     1 24.92\n" +
                "ATOM     56  OE1 GLU     7      -3.675  15.448  42.917     1 20.95\n" +
                "ATOM     57  OE2 GLU     7      -3.535  17.519  42.130     1  33.7";

        PDBFileParser parser = new PDBFileParser();
        Structure newchainStructure = parser.parsePDBFile(new BufferedReader(new StringReader(pdb1)));
        Chain newchain = newchainStructure.getChain(0);

        newchainStructure = parser.parsePDBFile(new BufferedReader(new StringReader(pdb2)));
        Chain realChain = newchainStructure.getChain(0);

        Superpose.RMSD rmsd1 = Superpose.findRMSD(realChain, newchain);

        System.out.println("rmsd = " + rmsd1);
    }
}
