package ca.ualberta.proteus.program.predictSPTM;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.Reader;
import java.text.ParseException;

/**
 * Created by Eclipse IDEA.
 * User: savita shrivastava
 * Date: Jan 19, 2007
 * Time: 9:30:00 AM
 */

public class PredictSPTMReader
{
    private Logger logger = Logger.getLogger(PredictSPTMReader.class);

    public PredictSPTMReader()
    {
    }

    /**
     * This method parse the output reader of PredictSPTM program
     * for signal Peptide prediction.
     * <br/>
     * Note: Only parses a signal sequence, not all of them.
     *
     * @param reader a reader
     * @throws ParseException If there is a parse error
     * @return A String containing the Signal Site in a sequence
     */
    public StructureAnnotatedSequence parseSignalOutput(Reader reader) throws ParseException
    {
        String line;

        try
        {
            BufferedReader br = new BufferedReader(reader);
            String name;
            while ((line = br.readLine()) != null)
            {
                if ( line.startsWith("//"))
                {
                    // end of sequence
                    break;
                }
                if (line.length() > 0 )
                {
                    if ( line.startsWith(">"))
                    {
                        name = line.substring(1);
                        StringBuilder sequence = new StringBuilder();
                        StringBuilder signal = new StringBuilder();
                        while ((line = br.readLine()) != null)
                        {
                            if ( line.startsWith("//"))
                            {
                                // end of sequence
                                break;
                            }
                            if ( line.length() > 0 )
                            {                                
                                sequence.append(line);
                                signal.append(br.readLine());
                            }
                        }



                        String signalStr = signal.toString().replaceAll("-", " ");

                        logger.info("PredictSPTM Prediction: " + signalStr.replace(" ", "-"));
                        
                        return new StructureAnnotatedSequence(sequence.toString(), signalStr, name);
                    }
                }
            }


            
            
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new ParseException("Error parsing predictSPTM signal peptide output file", 0);
        }

        return null;
    }
}
