package ca.ualberta.proteus.program.predictSPTM;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ExecUtils;
import ca.ualberta.proteus.util.GenericProgramRunner;
import ca.ualberta.proteus.util.StreamHandler;
import org.apache.log4j.Logger;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.io.SeqIOTools;

import java.io.*;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Jan 18, 2007
 * Time: 10:50:22 PM
 * To change this template use File | Settings | File Templates.
 */
public class PredictSPTMRunner extends GenericProgramRunner
{
    private Sequence sequence;
    private Organism organism = Organism.EUKARYOTIC;
    private Logger logger = Logger.getLogger(PredictSPTMRunner.class);
    private static final Map<Organism, String> organisms = new HashMap<Organism, String>();

    static
    {
        organisms.put(Organism.GRAM_NEGATIVE, "gram-");
        organisms.put(Organism.GRAM_POSITIVE, "gram+");
        organisms.put(Organism.EUKARYOTIC, "euk");
    }

    public PredictSPTMRunner(Sequence sequence, Organism organism)
    {
        this.sequence = sequence;
        this.organism = organism;
    }

    public void run()
    {
        runSPTM();
    }

    public StructureAnnotatedSequence runSPTM()
    {
        Config config = Config.getConfig();
        String path = config.getStringValue("sptm");
        try
        {
            // write PDB file to temp file
            File tempFile;
            tempFile = File.createTempFile("sptm", ".in");
            tempFile.deleteOnExit();

            File outfile = File.createTempFile("sptm", ".out");
            outfile.deleteOnExit();
          
            FileOutputStream blastfile;
            blastfile = new FileOutputStream(tempFile);
            SeqIOTools.writeFasta(blastfile, sequence );
            blastfile.close();

            String cmd = path + " -i " + tempFile.getPath();
            cmd += " -o " + outfile.getPath();
            cmd += " -c " + organisms.get(organism);

            Process proc = ExecUtils.exec(cmd, 1000, 30 * 1000); // 30 second timeout

            // any error message?
            StreamHandler errorReader = new
                    StreamHandler(proc.getErrorStream(), "ERROR");

            // any output?
            StreamHandler outputReader = new
                    StreamHandler(proc.getInputStream(), "OUTPUT");

            // kick them off
            errorReader.start();
            outputReader.start();

            int exitVal = proc.waitFor();

            if (exitVal != 0)
            {
                System.err.println(cmd);
                System.err.println("TMHMM failed " + errorReader.getInputString());
                return null;
            }

            /** Wait for the output reader to completely do its thing **/
            while (! outputReader.isFinished())
            {
                Thread.sleep(10);
            }

            String output = outputReader.getInputString();
            logger.debug(output);

            BufferedReader reader = new BufferedReader(new FileReader(outfile));

            PredictSPTMReader sptmreader = new PredictSPTMReader();

            StructureAnnotatedSequence result = sptmreader.parseSignalOutput(reader);

            if ( result != null )
            {
                return result;
            }
            else
            {
                logger.error("Problem parsing SPTM output file");
                logger.error("Cmd: " + cmd);                                
            }
            
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        catch (ParseException e)
        {
            e.printStackTrace();          
        }

        return null;
    }

}
