package ca.ualberta.proteus.program.predictSPTM.test;

import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.program.predictSPTM.PredictSPTMReader;
import ca.ualberta.proteus.program.predictSPTM.PredictSPTMRunner;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.io.StructureDB;
import junit.framework.TestCase;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;

import java.io.*;
import java.text.ParseException;
import java.util.List;

/**
 * Created by Eclipse IDEA.
 * User: savita shrivastava
 * Date: Jan 19, 2007
 * Time: 9:30:00 AM
 */

public class PredictSPTMReaderTest extends TestCase
{

    public void testReader()
    {
        String file = ">2\n" +
                "VAPTLTARLYSLLFRRTSTFALTIVVGALFFERAFDQGADAIYEHINEGKLWKHIKHKYENK\n" +
                "SSSSSSSSSSSSSSSSSSSSSSSSSSScTTTTTTT---------------------------\n" +
                "\n" +
                "//";

        PredictSPTMReader tmReader = new PredictSPTMReader();
        StructureAnnotatedSequence signalSite = null;
        try
        {
            signalSite = tmReader.parseSignalOutput(new StringReader(file));
        }
        catch (ParseException e)
        {
            e.printStackTrace();
        }
        if (signalSite != null)
        {
            System.out.println("Signal Sites  : " + signalSite);
        }
    }

    public void testRunner() throws BioException, IOException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);

        for (int i = 0; i < 5; i++)
        {
            Sequence sequence = db.getSequence(i);

            PredictSPTMRunner runner = new PredictSPTMRunner(sequence, Organism.GRAM_POSITIVE);
            StructureAnnotatedSequence signalSeq = runner.runSPTM();

            assertEquals(signalSeq.length(), sequence.length());

            System.out.println(signalSeq);
        }
    }

    public void testRunner2() throws BioException, IOException
    {
        /*String file = ">P31550\n" +
                "MLKKCLPLLLLCTAPVFAKPVLTVYTYDSFAADWGPGPVVKKAFEADCNCELKLVALEDG\n" +
                "VSLLNRLRMEGKNSKADVVLGLDNNLLDAASKTGLFAKSGVAADAVNVPGGWNNDTFVPF\n" +
                "DYGYFAFVYDKNKLKNPPQSLKELVESDQNWRVIYQDPRTSTPGLGLLLWMQKVYGDDAP\n" +
                "QAWQKLAKKTVTVTKGWSEAYGLFLKGESDLVLSYTTSPAYHILEEKKDNYAAANFSEGH\n" +
                "YLQVEVAARTAASKQPELAQKFLQFMVSPAFQNAIPTGNWMYPVANVTLPAGFEKLTKPA\n" +
                "TTLEFTPAEVAAQRQAWISEWQRAVSR";

        List<Sequence> list = ProteusTools.parseFasta(new StringReader(file));
        Sequence sequence = list.get(0);

        PredictSPTMRunner runner = new PredictSPTMRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runHMMPFam();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);*/
    }

    public void testRunner3() throws BioException, IOException
    {
        String file = ">TEST\n" +
                "VAPTLTARLYSLLFRRTSTFALTIVVGALFFERAFDQGADAIYEHINEGKLWKHIKHKYENK";

        List<Sequence> list = ProteusTools.parseFasta(new StringReader(file));
        Sequence sequence = list.get(0);

        PredictSPTMRunner runner = new PredictSPTMRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runSPTM();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);

        String EXPECTED = "ssssssssssssssssssssssssssssssssssssssssssc-------------------";

        assertEquals(signalSeq.getStructureAnnotation().seqString(), EXPECTED);

        //////////////////////////////

        runner = new PredictSPTMRunner(sequence, Organism.GRAM_NEGATIVE);
        signalSeq = runner.runSPTM();

        assertEquals(signalSeq.length(), sequence.length());

        System.out.println(signalSeq);

        EXPECTED = "---------------tttttttttttttttttttt---------------------------";

        assertEquals(EXPECTED, signalSeq.getStructureAnnotation().seqString());
    }

    public void testSerialization() throws BioException, IOException, ClassNotFoundException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);
        Sequence sequence = db.getSequence(0);

        PredictSPTMRunner runner = new PredictSPTMRunner(sequence, Organism.GRAM_POSITIVE);
        StructureAnnotatedSequence signalSeq = runner.runSPTM();

        ObjectOutputStream stream = new ObjectOutputStream(new FileOutputStream("tmp"));
        stream.writeObject(signalSeq);
        stream.close();

        ObjectInputStream in = new ObjectInputStream(new FileInputStream("tmp"));
        Object o = in.readObject();

        System.out.println(o);
    }
}
