package ca.ualberta.proteus.program.proteus;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.transsec.PredictionScore;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.util.StreamHandler;
import ca.ualberta.proteus.util.FileUtils;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: May 10, 2005
 * Time: 8:26:23 PM
 * To change this template use File | Settings | File Templates.
 */
public class PsipredRunner extends SecondaryStructurePredictorRunner
{
    //private String pssmFile = null;
    private String checkpointFile = null;
    private String blastInputFileName = null;
    private ScoreAnnotatedSequence sequence;
    public Sequence querySequence;

    public void run()
    {
        this.finished = false;

        Config config = Config.getConfig();
        String psipredCmd = config.getStringValue("psipred.command");
        String psipassCmd = config.getStringValue("psipass.command");
        String datadir = config.getStringValue("psipred.datadir");

        try
        {
            String fname = getCheckpointFile().substring(0, getCheckpointFile().indexOf('.'));
            fname = fname.replaceFirst(".*/(.*$)", "$1");

            String uniqueIdentifier = getUniqueIdentifier();
            String id = "blast" + uniqueIdentifier;

            // remove all path, get just the file name
            File checkpointFile = new File(getCheckpointFile()); // current dir

            runMakeMat(fname, checkpointFile);


            StreamHandler errorReader;
            StreamHandler outputReader;
            int exitVal;


            psipredCmd += " " + fname + ".mtx " + datadir + "/weights.dat " + datadir + "/weights.dat2 "
                             + datadir + "/weights.dat3 " + datadir + "/weights.dat4 ";

            Process psipredProc = Runtime.getRuntime().exec(psipredCmd);

            OutputStream fileout = new FileOutputStream(id + ".ss");
            // any error message?
            errorReader = new
                    StreamHandler(psipredProc.getErrorStream(), "ERROR");

            // any output?
            outputReader = new
                    StreamHandler(psipredProc.getInputStream(), "OUTPUT", fileout);

            // kick them off
            errorReader.start();
            outputReader.start();

            exitVal = psipredProc.waitFor();

            if ( exitVal != 0 ) {
                System.err.println("Psipred failed " + errorReader.getInputString());
                return;
            }

            /** Wait for the output reader to completely do its thing **/
            while ( ! outputReader.isFinished() )
            {
                Thread.sleep(10);
            }

            psipassCmd += " " + datadir + "/weights_p2.dat 1.0 1.0 1.3 " +  id + ".ss2 "  + id + ".ss ";

            Process filterProc = Runtime.getRuntime().exec(psipassCmd);

            // any error message?
            errorReader = new
                    StreamHandler(filterProc.getErrorStream(), "ERROR");

            // any output?
            outputReader = new
                    StreamHandler(filterProc.getInputStream(), "OUTPUT");

            // kick them off
            errorReader.start();
            outputReader.start();

            exitVal = filterProc.waitFor();

            if ( exitVal != 0 ) {
                System.err.println("Psipass failed " + errorReader.getInputString() + outputReader.getInputString());
            }

            /** Wait for the output reader to completely do its thing **/
            while ( ! outputReader.isFinished() )
            {
                Thread.sleep(10);
            }

            BufferedReader reader = new BufferedReader(new FileReader(id + ".ss2"));

            try {
                String line;
                StringBuffer newStructure = new StringBuffer();
                StringBuffer sequenceStr = new StringBuffer();
                StringBuilder psipredOutputFile = new StringBuilder();
                List<PredictionScore> scoresList = new ArrayList<PredictionScore>();

                reader.readLine(); reader.readLine(); // skip 2 lines
                while( ( line = reader.readLine() ) != null ) {
                    // need to use trim - psipred has spaces in beginning for padding for >1000 residues
                    final String[] strings = line.trim().split("\\s+");
                    newStructure.append( strings[2] );
                    sequenceStr.append( strings[1] );
                    if ( strings.length < 6 )
                    {
                        throw new IllegalStateException("PSIPRED bad output: " + line );
                    }
                    else                    
                    {
                        scoresList.add( new PredictionScore( Double.parseDouble(strings[3]), Double.parseDouble(strings[4]), Double.parseDouble(strings[5])) );                //unfilteredPrediction.getScores()[i++] =
                    }
                    psipredOutputFile.append(line).append("\n");
                }

                PredictionScore[] scores = new PredictionScore[scoresList.size()];
                sequence = new ScoreAnnotatedSequence(sequenceStr.toString(), newStructure.toString(), querySequence.getName(), scoresList.toArray(scores));

                if ( sequenceStr.length() != querySequence.length() ) {
                    System.out.println("PSIPRED output: ");
                    System.out.println(psipredOutputFile.toString());
                    System.out.println(sequenceStr);
                    System.out.println(querySequence.seqString());
                    System.out.flush();
                    throw new IllegalStateException("Result doesn't match sequence!");
                }
            }
            finally {
                reader.close();
                cleanup(fname, id);
            }
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();
        }
        catch (BioException e)
        {
            e.printStackTrace();
        }
        finally{
            finished = true;
        }

    }

    /**
     * Runs makemat and returns true if it succeeds
     * @param fname
     * @param checkpointFile
     * @return true on success, false otherwise
     * @throws IOException
     */
    public boolean runMakeMat(String fname, File checkpointFile)
            throws IOException
    {
        String makemat = Config.getConfig().getStringValue("makemat.command");

        File inputFile = new File(getBlastInputFileName()); // current dir

        if ( checkpointFile.getParent()!= null && ! checkpointFile.getParent().equals(""))
        {
            File oldInputFile = new File(getBlastInputFileName());
            inputFile = new File(oldInputFile.getName());
            FileUtils.copyFile(oldInputFile, inputFile);

            File oldCheckpointFile = new File(getCheckpointFile());
            checkpointFile = new File(oldCheckpointFile.getName());
            FileUtils.copyFile(oldCheckpointFile, checkpointFile);
        }

        FileOutputStream fos = new FileOutputStream( fname + ".pn");
        fos.write(checkpointFile.getName().getBytes());
        fos.close();

        fos = new FileOutputStream( fname + ".sn");

        fos.write(inputFile.getName().getBytes());
        fos.close();

        String command = makemat + " -P " + fname;
        Process makematProc = Runtime.getRuntime().exec(command);

        // any error message?
        StreamHandler errorReader = new
                StreamHandler(makematProc.getErrorStream(), "ERROR");

        // any output?
        StreamHandler outputReader = new
                StreamHandler(makematProc.getInputStream(), "OUTPUT");

        // kick them off
        errorReader.start();
        outputReader.start();

        // any error???
        int exitVal = 0;
        try
        {
            exitVal = makematProc.waitFor();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        if ( exitVal != 0 )
        {
            System.err.println("Makemat failed " + errorReader.getInputString());
            return false;
        }
        return true;
    }

    private void cleanup(String fname, String id)
    {
        File f = new File(id + ".ss");
        f.delete();
        f = new File(id + ".ss2");
        f.delete();
        f = new File(fname + ".pn");
        f.delete();
        f = new File(fname + ".sn");
        f.delete();
        f = new File(fname + ".pn");
        f.delete();
        f = new File(fname + ".sn");
        f.delete();
        f = new File(fname + ".mtx");
        f.delete();
        f = new File(fname + ".mn");
        f.delete();
        f = new File(fname + ".aux");
        f.delete();
    }

    public ScoreAnnotatedSequence getPrediction()
    {
        return sequence;
    }

    /**
     * Gets the blast position matrix file name given by the -Q option to 'blastpgp'
     */
  /*  public String getPssmFile()
    {
        if ( pssmFile == null )
            throw new IllegalStateException("Blast output file is not set - run blast and set this property before running");
        return pssmFile;
    }
    */

    /**
     * Gets the blast checkpoint file name given by the -C option to 'blastpgp'
     */
    public String getCheckpointFile()
    {
        if ( checkpointFile == null )
            throw new IllegalStateException("Blast output file is not set - run blast and set this property before running");
        return checkpointFile;
    }

    public String getBlastInputFileName()
    {
        return blastInputFileName;
    }

    /**
     * Sets the blast position matrix file name given by the -Q option to 'blastpgp'
     * and the blast checkpoint file name given by the -C option
     * You need to set this before running PSIPRED
     * @param pssmFile
     * @param checkpointFile
     */
    public void setParams(String pssmFile, String checkpointFile, String blastInputFileName)
    {
        //this.pssmFile = pssmFile;
        this.checkpointFile = checkpointFile;
        this.blastInputFileName = blastInputFileName;
    }

    public static void main(String[] args) {
        PsipredRunner psipredRunner = new PsipredRunner();
        psipredRunner.setParams("blast.pos", "blast.chk", "file.fa");
        psipredRunner.run();
        System.out.println(psipredRunner.getPrediction());
    }
}
