package ca.ualberta.proteus.program.test;

import ca.ualberta.proteus.logging.RuntimeCatalog;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.Proteus;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.FileUtils;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.File;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Aug 2, 2006
 * Time: 9:31:38 PM
 * To change this template use File | Settings | File Templates.
 */
public class ProteusTests extends TestCase
{
    public static Test suite()
    {
        return new TestSuite(ProteusTests.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void testCommandLine()
    {
        String[] args = {};

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args);

            // we want this to fail
            assertNull(jsapResult);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLine2()
    {
        String[] args = {""};

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args);

            // we want this to fail
            assertNull(jsapResult);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLine3()
    {
        String[] args = {"test/test1.fa"};

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args);
            assertNotNull(jsapResult);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLine4()
    {
        String args = "-o proteus.out test/test1.fa";

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args.split("\\s+"));

            assertNotNull(jsapResult);

            assertEquals(jsapResult.getString("output"), "proteus.out");
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLineMethod()
    {
        String args = "-m psipred -o proteus.out test/test1.fa";

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args.split("\\s+"));

            assertNotNull(jsapResult);

            final MethodSettings method = Proteus.getMethod(jsapResult);

            assertEquals(method, MethodSettings.PSIPRED);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLineMethod2()
    {
        String args = "-m jnet -o proteus.out test/test1.fa";

        try
        {
            JSAPResult jsapResult = Proteus.parseCommandLine(args.split("\\s+"));

            assertNotNull(jsapResult);

            MethodSettings method = Proteus.getMethod(jsapResult);

            assertEquals(method, MethodSettings.JNET);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLineMethod3()
    {
        String args = "-m fast -o proteus.out test/test1.fa";

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args.split("\\s+"));

            assertNotNull(jsapResult);

            final MethodSettings method = Proteus.getMethod(jsapResult);

            assertEquals(method, MethodSettings.PSIPRED_JNET);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testCommandLineMethod4()
    {
        String args = "-m asf -o proteus.out test/test1.fa";

        try
        {
            final JSAPResult jsapResult = Proteus.parseCommandLine(args.split("\\s+"));

            assertNull(jsapResult);
        }
        catch (JSAPException e)
        {
            fail();
        }
    }

    public void testProteus() throws JSAPException, BioException, IOException
    {
        String args = "-o proteus.out test/test1.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testFast() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-v -m fast -o proteus.out test/test5.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testShort() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-v -m fast -o proteus.out test/short.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }


    public void testAlternateConfig() throws JSAPException, BioException, IOException
    {
        // Copy original config file to new one
        File file = new File("proteus.config");
        File dest = new File("proteus.config.2");
        FileUtils.copyFile(file, dest);

        String args = "-v -m fast -o proteus.out -c " + dest + " test/short.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);

        dest.delete();
    }

    public void testSignalPeptide() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-v -m fast -o proteus.out test/signalpeptide.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testReallyHardStructure() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-o proteus.out test/test5.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testReallyHardStructureOnlyTM() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-o proteus.out -t test/test5.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        StructureDB db = new StructureDB("proteus.out", StructureDB.MODE.READ);
        StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) db.getSequence(0);

        String expected = "CCCCCTTTTTTTTTTTTTTTTTTTTCCCCTTTTTTTTTTTTTTTTTTTTT" +
                "TTCCCCCCCCCCCCCCCCCBBBBCCCCCCCCCCCCCCCCBBBBBBBCHHH" +
                "HHCCCCCCCBBBBBBCTTTTTTTTTTTTTTTTTTTTTTTTTTCCCHHHHC" +
                "CCCCCCCBBBBCCCHHHCCCCCCCCCCCTTTTTTTTTTTTTTTTTTTTTT" +
                "TTTTTTCCCCTTTTTTTTTTTTTTTTTCTTTTTTCCCCCCCBBBHHHHHH" +
                "HHHHHHHHHCCCCCCTTTTTTTTTTTTTTTTTTTTTTTTTTCCCCCCCCH" +
                "HHHHHCCCHHHHHHHHHHHHHCCCCCCTTTTTTTTTTTTTTTTTTTTTTT" +
                "TTCCCCCCCC";

        System.out.println("Observed: " + sequence.getStructureAnnotation().seqString());
        System.out.println("Expected: " + expected);
    }

    public void testOnlyTM() throws JSAPException, BioException, IOException
    {
        String sequenceStr = "MFINRWLFSTNHKDIGTLYLLFGAWAGMVGTALSLLIRAELGQPGTLLGDDQIYNVVVTAHAFVMIFFMVMPIMIGGFGNWLVPLMI";
        Sequence sequence = ProteinTools.createProteinSequence(sequenceStr, "TEST6");

        final ProteusSettings settings = new ProteusSettings();
        settings.setRunOnlyTM(true);
        settings.setSequence(sequence);

        ProteusImpl proteus = new ProteusImpl(settings);
        proteus.run();

        RuntimeCatalog logger = proteus.getLogger();
        ScoreAnnotatedSequence finalSequence = (ScoreAnnotatedSequence) logger.getLogObject(Consts.FINAL_WITH_HOMOLOGY);

        // Best homolog should be 1OCCA        
        assertEquals(finalSequence.getStructureAnnotation().seqString().toUpperCase(),
                "CHHHHHCCCCCTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTCCCCCCCCCTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTC");
    }

    public void testMultiSequence() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure doing transmembrane only
        String args = "-o proteus.out -t test/multi.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testOrganism() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure doing transmembrane only
        String args = "-o proteus.out -t test/multi.fa --organism 1";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void testMultiSequenceVerbose() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-v -o proteus.out test/multi.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void test7() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-o proteus.out test/test7.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void test8() throws JSAPException, BioException, IOException
    {
        // This is Dave's Structure
        String args = "-o proteus.out test/test8.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    public void test9() throws JSAPException, BioException, IOException
    {
        // This is a structure sent to us by Chai Christopher that crashes TM prediction.
        String args = "-o proteus.out test/test9.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }

    private void verify(JSAPResult options) throws BioException, IOException
    {
        String out = options.getString(Proteus.OUTPUT_FILE_FLAG);

        StructureDB db = new StructureDB(out, StructureDB.MODE.READ);

        assertTrue(db.size() > 0);
    }

    /**
     * Test to see if a sequence out of the database gives us the exact database result 
     *
     * @throws BioException
     */
    public void testHomolog() throws BioException
    {
        StructureDB db = SeqSeeDB.getInstance();

        StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) db.getSequence(0);
        System.out.println("Running " + sequence.getName());
        sequence.setName("QUERY");

        final ProteusSettings settings = new ProteusSettings();
        settings.setSequence(sequence);
        settings.setMethod(MethodSettings.PSIPRED_JNET);

        ProteusImpl proteus = new ProteusImpl(settings);
        proteus.run();

        ScoreAnnotatedSequence result = proteus.getResult();

        assertEquals(result.seqString(), sequence.seqString());
        assertEquals(result.getStructureAnnotation().seqString(), sequence.getStructureAnnotation().seqString());
    }

    public void testHomolog3() throws IllegalSymbolException
    {
        // 1I3QA
        String seq =    "VGQQYSSAPLRTVKEVQFGLFSPEEVRAISVAKIRFPETMDETQTRAKIGGLNDPRLGSIDRNLKCQTCQEGMNECPGHFGHIDLAKPVFHVGFIAKIKKVCECVCMHCGKLLLDEHNELMRQALAIKDSKKRFAAIWTLCKTKMVCETDVPSEDDPTQLVSRGGCGNTQPTIRKDGLKLVGSWKKDRATGDADEPELRVLS";
        String struct = "CCCCCCCCCCCCCEEEEEEEECHHHHHHHEEECCCCCCEEECCCCEEECCEEECCCCCCCCCCCCCCCCCCCCCCCCCEEEEEEECCEEEECCCHHHHHHHHHCCCCCCCCCCCCCCCHHHHHCCCCCCHHHHHHHHHHHHCCCEEECEEEEECCCCCEEEEECCCCEEEEEEEECCCEEEEEEEECCCCCCCCCCEEEEEC";

        StructureAnnotatedSequence sequence = new StructureAnnotatedSequence(seq.substring(40), struct.substring(40), "1I3QA");

        final ProteusSettings settings = new ProteusSettings();
        settings.setSequence(sequence);
        settings.setMethod(MethodSettings.PSIPRED_JNET);

        ProteusImpl proteus = new ProteusImpl(settings);
        proteus.run();

        ScoreAnnotatedSequence result = proteus.getResult();

        assertEquals(result.seqString(), sequence.seqString());
        assertEquals(sequence.getStructureAnnotation().seqString(), result.getStructureAnnotation().seqString());
    }

    /**
     * Test to see if a sequence out of the database gives us the exact database result
     *
     * @throws BioException
     */
    public void testHomolog2() throws BioException, IOException
    {
        StructureDB db = new StructureDB(Config.getConfig().getStringValue("TMPDB"), StructureDB.MODE.READ);

        StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) db.getSequence(0);
        sequence.setName("QUERY");

        final ProteusSettings settings = new ProteusSettings();
        settings.setSequence(sequence);

        ProteusImpl proteus = new ProteusImpl(settings);
        proteus.run();

        ScoreAnnotatedSequence result = proteus.getResult();

        assertEquals(result.getStructureAnnotation().seqString(), sequence.getStructureAnnotation().seqString());
    }

    /**
     * Test to see if a sequence out of the database gives us the exact database result
     *
     * @throws BioException
     */
    public void testHomodeller() throws BioException, IOException, JSAPException
    {
        String file = "proteus.out";
        String args = "-h -o " + file + " test/short.fa";

        File pdbFile = new File(file + ".pdb");
        pdbFile.delete();

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
        
        assertTrue(pdbFile.exists());
    }

    /**
     * Test the output for 2 models
     *
     * @throws BioException
     */
    public void testHomodellerMulti() throws BioException, IOException, JSAPException
    {
        String file = "proteus.out";
        String args = "-h -v -o " + file + " test/multi.fa";

        File pdbFile = new File(file + ".pdb");
        pdbFile.delete();

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);

        assertTrue(pdbFile.exists());
    }
    
    /**
     * Test to see if a sequence out of the database gives us the exact database result
     *
     * @throws BioException
     */
    public void testHomodeller2() throws BioException, IOException, JSAPException
    {
        String args = "-h -v -o proteus.out test/multi2.fa";

        final JSAPResult options = Proteus.parseCommandLine(args.split("\\s+"));

        Proteus.runFromCommandLine(options);

        verify(options);
    }
}
