package ca.ualberta.proteus.program.thrifty;

import ca.ualberta.proteus.program.homodeller.Angle;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;

import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 9, 2006
 * Time: 1:34:05 AM
 * To change this template use File | Settings | File Templates.
 */
public class AngleConverter
{
    /**
     * Does a mapping of ramachandran space (phi and psi angles) to 9-state secondary structure
     *
     * @param sequence
     * @param angles
     * @return a StructureAnnotatedSequence, where the sequence is the protein sequence, and the structure
     * is a 9-state structure
     */
    public static StructureAnnotatedSequence convertToSecondaryStructure(String sequence, Map<Integer, Angle> angles)
    {
        assert angles.size() == sequence.length() : "Angles and sequence don't match";

        String structure = convertToSecondaryStructureString(angles);

        try
        {
            SimpleSymbolList symbolList = ProteusTools.createDavesStructure(structure);
            Sequence protein = ProteinTools.createProteinSequence(sequence, "QUERY");

            return new StructureAnnotatedSequence(protein, symbolList);
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();
        }

        return null;
    }

    public static String convertToSecondaryStructureString(Map<Integer, Angle> angles)
    {
        StringBuilder structure = new StringBuilder();

        for (Integer resid : angles.keySet())
        {
            Angle angle = angles.get(resid);
            double phi = angle.getPhi();
            double psi = angle.getPsi();
            char res = 'G';

            if ( phi >= -80 && phi <= -20
                    && psi >= -80 && psi <= -20 )
            {
                res = 'Q';
            }
            else if ( phi >= -140 && phi <= -80
                    && psi >= -90 && psi <= -20 )
            {
                res = 'E';
            }
            else if ( phi >= -160 && phi <= -20
                    && psi >= -20 && psi <= 40 )
            {
                res = 'S';
            }
            else if ( phi >= -90 &&  phi <= 0
                    && psi >= 40 && psi <= 180 )
            {
                res = 'L';
            }
            else if ( phi >= -180 && phi <= -90
                    && psi >= 40 && psi <= 180 )
            {
                res = 'I';
            }
            else if ( phi >= 20 && phi <= 80
                    && psi >= 20 && psi <= 60 )
            {
                res = 'P';
            }
            else if ( phi >= -180 && phi <= -20
                    && psi >= -180 && psi <= -150 )
            {
                res = 'F';
            }
            else if ( phi >= 150 && phi <= 180
                    && psi >= 90 && psi <= 180 )
            {
                res = 'V';
            }

            structure.append(res);
        }
        return structure.toString();
    }
}
