package ca.ualberta.proteus.program.thrifty;

import java.io.StringReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 9, 2006
 * Time: 12:04:34 AM
 * To change this template use File | Settings | File Templates.
 */
public class MatrixEditor
{
    String[] header;
    private Map<Character, List<String>> matrix = new LinkedHashMap<Character, List<String>>();

    public MatrixEditor(StringBuilder originalMatrix)
    {
        populate(originalMatrix);
    }

    private void populate(StringBuilder matrixStr)
    {
        BufferedReader reader = new BufferedReader(new StringReader(matrixStr.toString()));

        try
        {
            String line;
            while ( (line = reader.readLine()) != null )
            {
                if ( line.length() == 0 )
                {
                    continue;
                }

                char c = line.charAt(0);
                if ( c == '#')
                {
                    continue;
                }

                if ( c == ' ')
                {
                    header = line.split("\\s+");

                    if ( header[0].length() == 0)
                    {
                        // shift left
                        System.arraycopy(header, 1, header, 0, header.length-1);
                    }
                }
                else
                {
                    ArrayList<String> list = new ArrayList<String>();
                    matrix.put(c, list);

                    String[] values = line.substring(1).split("\\s+");
                    for ( String s: values)
                    {
                        if ( s.length() > 0)
                        {
                            list.add(s);
                        }
                    }
                }
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    private static final int SAME_VALUE = 3;
    private static final int G_OTHER = -5;
    private static final int DIFFERENT = 1;
    private static final int OTHER_STAR = -11;

    public void modifyTo9StateMatrix()
    {
        fill("0");

        List<String> list;
        char[] letters = {'A', 'E', 'S', 'F', 'I', 'P', 'L', 'V', 'G'};

        // set diagonals
        for ( char c : letters)
        {
            setValue(c, c, SAME_VALUE);
        }

        // set anything and G as
        for ( char c : letters)
        {
            if ( c != 'G')
            {
                setValue(c, 'G', G_OTHER);
            }
        }

        // set anything and * as
        for ( char c : letters)
        {
            if ( c != '*')
            {
                setValue(c, '*', OTHER_STAR);
            }
        }

        for ( char a : letters)
        {
            if ( a != 'G')
            {
                for ( char b: letters )
                {
                    if ( b != 'G' && a != b )
                    {
                        setValue(a, b, DIFFERENT);
                    }
                }
            }
        }

        setValue('*', '*', 1);

    }

    private void setValue(char letter1, char letter2, int value)
    {
        List<String> list;
        list = matrix.get(letter1);
        list.set(find(letter2), String.valueOf(value));

        if ( letter1 != letter2)
        {
            list = matrix.get(letter2);
            list.set(find(letter1), String.valueOf(value));
        }
    }

    /**
     * Fill the entire matrix with this value
     * @param value
     */
    public void fill(String value)
    {
        for ( Character letter : matrix.keySet())
        {
            List<String> row = matrix.get(letter);
            for ( int i=0; i<row.size(); i++)
            {
                row.set(i, value);
            }
        }
    }

    private int find(char letter)
    {
        assert header != null :"Header is null";

        for (int i = 0; i < header.length; i++)
        {
            if ( header[i].charAt(0) == letter )
            {
                return i;
            }
        }

        assert false;

        return -1;
    }

    public String toString()
    {
        StringBuilder rv = new StringBuilder();

        rv.append(" "); // space for the leading letter

        for ( String s: header)
        {
            rv.append(String.format("%4s", s));
        }

        rv.append('\n');

        for ( Character letter : matrix.keySet())
        {
            List<String> row = matrix.get(letter);
            rv.append(letter);

            for ( String value : row )
            {
                rv.append(String.format("%4s", value));
            }

            rv.append('\n');
        }

        return rv.toString();
    }
}
