package ca.ualberta.proteus.program.thrifty.test;

import ca.ualberta.proteus.program.homodeller.Angle;
import ca.ualberta.proteus.program.homodeller.HomodellerUtils;
import ca.ualberta.proteus.program.thrifty.AngleConverter;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.structure.Chain;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.structure.io.PDBFileParser;

import java.io.*;
import java.util.TreeMap;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 9, 2006
 * Time: 2:07:12 PM
 * To change this template use File | Settings | File Templates.
 */
public class CreateDatabase
{
    public static void main(String[] args)
    {
        File pdbDir = new File("/home/scott/proteus_web/data/pdb/pdbdata/");
        File outputFile = new File("seqsee9.db");

        File[] files = pdbDir.listFiles(new FilenameFilter()
        {
            public boolean accept(File dir, String name)
            {
                return name.endsWith("pdb");
            }
        });

        PDBFileParser parser = new PDBFileParser();
        StructureDB structureDB = new StructureDB(StructureDB.MODE.WRITE);

        int index = 0;
        for (File f : files)
        {
            try
            {
                Structure structure = parser.parsePDBFile(new BufferedReader(new FileReader(f)));

                Chain ch = structure.getChain(0);
                HomodellerUtils.removeHetatoms(ch);

                TreeMap<Integer, Angle> angles = HomodellerUtils.getAngles(ch);

                StructureAnnotatedSequence sequence = AngleConverter.convertToSecondaryStructure(ch.getSequence(), angles);
                sequence.setName(structure.getPDBCode() + ch.getName());

                if (sequence.length() > 0)
                {
                    structureDB.addSequence(sequence);
                }

                if (index++ < 5)
                {
                    System.out.println(sequence);

                    // print it for the first 5
                    SeqSeeDB db = SeqSeeDB.getInstance();
                    if (db.containsSequence(sequence.getName()))
                    {
                        Sequence result = db.getSequence(sequence.getName());
                        System.out.println(result);
                    }
                }

                if (structureDB.size() % 1000 == 0)
                {
                    PrintStream writer = new PrintStream(outputFile);
                    writeDB(structureDB.sequenceIterator(), writer);
                    writer.close();
                }
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            catch (IllegalIDException e)
            {
                e.printStackTrace();
            }
            catch (BioException e)
            {
                e.printStackTrace();
            }

            //todo
            //break;
        }

        System.out.println("DB has " + structureDB.size() + " sequences");
        try
        {
            PrintStream writer = new PrintStream(outputFile);
            writeDB(structureDB.sequenceIterator(), writer);
            writer.close();
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    private static void writeDB(SequenceIterator sequenceIterator, PrintStream writer)
    {
        try
        {
            for (Sequence sequence = sequenceIterator.nextSequence(); sequenceIterator.hasNext(); sequence = sequenceIterator.nextSequence())
            {
                String seqName = sequence.getName();
                if (seqName.length() == 4)
                {
                    seqName += "_";
                }

                writer.println(">" + seqName);

                String structStr = null;
                if (sequence instanceof StructureAnnotatedSequence)
                {
                    structStr = ((StructureAnnotatedSequence) sequence).getStructureAnnotation().seqString();
                }

                for (int i = 0; i < sequence.length(); i += 50)
                {
                    assert structStr != null;

                    String line = structStr.substring(i, Math.min(i + 50, structStr.length()));
                    line = line.toUpperCase().replace("E", "B");
                    line = line.replaceAll("(\\w{10})", "$1 ");
                    writer.println(line);
                }
                writer.println();

                /*writer.println("NUM RESIDU: " + sequence.length());
                writer.println("BRKHAVN ID: " + seqName);
                writer.println();

                String seqStr = sequence.seqString();
                String structStr = null;
                if (sequence instanceof StructureAnnotatedSequence)
                {
                    structStr = ((StructureAnnotatedSequence) sequence).getStructureAnnotation().seqString();
                }

                for (int i = 0; i < sequence.length(); i += 50)
                {
                    assert structStr != null;

                    assert structStr.length() == seqStr.length();

                    String line = seqStr.substring(i, Math.min(i + 50, seqStr.length()));
                    line = line.replaceAll("(\\w{10})", "$1 ");
                    writer.println(line);

                    line = structStr.substring(i, Math.min(i + 50, seqStr.length()));
                    line = line.toUpperCase().replace("E", "B");
                    line = line.replaceAll("(\\w{10})", "$1 ");
                    writer.println(line);
                    writer.println();
                }*/
            }

        }
        catch (BioException e)
        {
            e.printStackTrace();
        }

    }
}
