package ca.ualberta.proteus.program.thrifty.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestCase;
import ca.ualberta.proteus.program.thrifty.FastXAlignRunner;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.ProteinTools;

import java.util.Map;

/**
 * FastXAlignRunner Tester.
 *
 * @author <Authors name>
 * @since <pre>10/29/2006</pre>
 * @version 1.0
 */
public class FastXAlignRunnerTest extends TestCase {

    public void testGetAlignment() throws Exception
    {
        String sequence = "SRTEYDPLKFWPITHYCPHSARKDTYPERFYANMPKLDNQGPLSTYPLSTQWPIIVDTASATLMS";
        String structure = "cchhhhhhhhhchhhhhhhccccccccccbbbbccccccccccccchhhhhhcbbbbbbcccccc";

        SimpleSymbolList symbolList = ProteusTools.createStructure(structure);
        Sequence protein = ProteinTools.createProteinSequence(sequence, Consts.QUERY );

        StructureAnnotatedSequence query = new StructureAnnotatedSequence(protein, symbolList);
        String template = "SRSELDPLKFMPLPITYCGHSAREATYPERDDANMPKLENSTGPLQTYPLLSYQCPIIVDTAKHLLNS";
        Sequence templateSequence = ProteinTools.createProteinSequence(template, "TEMPL");

        FastXAlignRunner runner = new FastXAlignRunner(query, templateSequence);

        runner.run();

        Alignment alignment = runner.getAlignment();
        Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

        for ( Object label: alignment.getLabels())
        {
            SymbolList seq = alignment.symbolListForLabel(label);

            System.out.println(label + ": " + seq.seqString());

            System.out.println(ppshMap.get(label).get(Consts.QUERY) );
        }
    }

    public static Test suite() {
        return new TestSuite(FastXAlignRunnerTest.class);
    }
}
