package ca.ualberta.proteus.program.thrifty.test;

import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.program.thrifty.FastXAlignRunner;
import ca.ualberta.proteus.program.thrifty.AngleConverter;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.homodeller.HomodellerUtils;
import ca.ualberta.proteus.program.homodeller.Angle;
import ca.ualberta.proteus.program.VadarRunner;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.biojava.PDBFileParser;
import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.BioException;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.structure.Chain;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.*;
import java.util.*;

/**
 * These are all obtained by using the Dapper
 */
public class TestStructureAlignment extends TestCase
{
    static final int CUTOFF = 30;
    static final String SUPER_DB = "superdb.db";
    StructureDB superDB;

    public TestStructureAlignment() throws IOException
    {
        superDB = new StructureDB("superdb.db", StructureDB.MODE.READWRITE);
    }

    public static Test suite()
    {
        return new TestSuite(TestStructureAlignment.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void test3TRX() throws IOException, BioException
    {
        String[] results = {
                "1FG4B", "1THO_", "1QMVE", "1EGR_", "1JHB_", "1E2YJ", "1A8Y_", "1QGVA", "1QK8A", "1FO5A", "1E2YD",
                "1KTE_", "1E2YC", "1QQ2B", "1BJX_", "1FOHB", "1E2YA", "1QMVG", "1QMVF", "1QFNA", "1EGO_", "1FB0A",
                "1QMVH", "1FB6A", "1E2YF", "1E2YH", "1G6YB", "1QMVC", "1T7PB", "1F6MG", "1B4QA", "1E2YI", "1F6MH",
                "1EEMA", "2TRCP", "1JFUB", "1G7EA", "1QMVA", "1HQOA", "1EZKA", "1I5GA", "1F6MC", "1QMVB", "2BJXA",
                "1JFUA", "1QQ2A", "1B9YC", "1F6MD", "1DBYA", "1B9XC", "1PRXB", "1G6WD", "1THX_", "1SRX_", "2TIR_",
                "1G6WA", "1GRX_", "1E2YB", "1E2YE", "1XOA_", "1G6WB", "1PRXA", "1HYUA", "1E2YG", "1HD2A", "1A8L_",
                "1EWXA", "1QMVJ", "2TRXB", "1QMVD", "1FG4A", "2TRXA", "1G6WC", "1TXXA", "1G6YA", "1MEK_", "1QMVI"
        };

        runForResults("3TRX", results);
    }
                  
    private void runForResults(String sequenceName, String[] proteinIDs)
            throws IllegalSymbolException, IllegalIDException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();
        //StructureDB superDB = new StructureDB(SUPER_DB);

        // get the query sequence
        Sequence query = get9StateSequence(sequenceName);

        Map<String, XAlignResult> results = new HashMap<String, XAlignResult>();

        // Put all of our desired proteins in the result set
        int finished = 0;
        for (String proteinID : proteinIDs)
        {
            // get the template (or subject to examine) sequence
            StructureAnnotatedSequence subject = get9StateSequence(proteinID);

            if ( subject == null )
            {
                continue;
            }

            Alignment alignment;
            try
            {
                // Run XAlign
                FastXAlignRunner runner = new FastXAlignRunner(subject, query);

                runner.run();

                alignment = runner.getAlignment();
                Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

                Double ppsh = ppshMap.get(subject.getName()).get(query.getName());

                System.out.println(ppsh);
                // put the results in a list
                results.put(proteinID, new XAlignResult(alignment, ppsh, query.getName()));
            }
            catch ( ProcessNotCompleteException e)
            {
                // if it didn't work, screw it
                continue;
            }
            // shwo the alignment
            for (Object label : alignment.getLabels())
            {
                SymbolList seq = alignment.symbolListForLabel(label);

                System.out.println(label + ": " + seq.seqString());
            }

            finished++;
        }

        // write the DB to cache it
        writeSuperDB(superDB);

        System.out.println(finished + " out of " + proteinIDs.length + " finished");

        try
        {
            // now use our seqsee DB to go through and find alignments
            List<Sequence> sequences = ProteusTools.parseFasta(new File("seqsee9.db"));

            int count = 0;
            // Go through the entire database of known sequences - like a modified BLAST
            for ( Sequence sequence9 : sequences)
            {
                // 9-state structure
                if ( ! db.containsSequence(sequence9.getName()))
                {
                    continue;
                }

                // 3-state structure
                StructureAnnotatedSequence structure = (StructureAnnotatedSequence) db.getSequence(sequence9.getName());

                /* we have the 3-state for all our templates, but not our query sequence.
                 we will use the 9-state sequence as the "sequence", and the 3-state as the structure.
                 That way, XAlign will be less likely to split the alignment in a structure.
                */
                StructureAnnotatedSequence template = new StructureAnnotatedSequence(sequence9, structure.getStructureAnnotation());


                try
                {
                    // run Xalign on this seqence
                    FastXAlignRunner runner = new FastXAlignRunner(template, query);

                    runner.run();

                    Alignment alignment = runner.getAlignment();
                    Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

                    Double ppsh = ppshMap.get(sequence9.getName()).get(query.getName());

                    // TODO debug only
                    if ( ppsh > 53 )
                    {
                        System.out.println(ppsh);

                        for ( Object label: alignment.getLabels())
                        {
                            SymbolList seq = alignment.symbolListForLabel(label);

                            System.out.println(label + ": " + seq.seqString());
                        }
                    }

                    results.put(template.getName(), new XAlignResult(alignment, ppsh, query.getName()));

                    count++;
                    // TODO debug only
                 /*   if ( count > 3000 )
                    {
                        break;
                    }*/
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (BioException e)
        {
            e.printStackTrace();
        }

        // Find out which sequences were marked the highest - hopefully our desired proteins were
        Map.Entry<String, XAlignResult>[] e = new Map.Entry[results.size()];
        results.entrySet().toArray(e);
        List<Map.Entry<String, XAlignResult>> entries = Arrays.asList(e);

        Collections.sort( entries, new Comparator<Map.Entry<String, XAlignResult>>()
        {
            public int compare(Map.Entry<String, XAlignResult> o1, Map.Entry<String, XAlignResult> o2)
            {
                XAlignResult value1 = o2.getValue();
                XAlignResult value2 = o1.getValue();
                return (int) ((value1.getAdjustedScore() - value2.getAdjustedScore()) * 100);
            }

        } );

        Map<String , XAlignResult> top20 = new HashMap<String, XAlignResult>();
        Map<String , XAlignResult> top10 = new HashMap<String, XAlignResult>();

        for ( int i=0; i<20; i++)
        {
            Map.Entry<String, XAlignResult> entry = entries.get(i);
            top20.put(entry.getKey(), entry.getValue());
        }

        for ( int i=0; i<10; i++)
        {
            Map.Entry<String, XAlignResult> entry = entries.get(i);
            top10.put(entry.getKey(), entry.getValue());

            printXAlignResult(entry.getValue());
        }

        // find out how many of our desired proteins are in the top 10
        List desired = Arrays.asList(proteinIDs);
        Collections.sort(desired);

        int found = 0;
        for ( String id: top20.keySet() )
        {
            if ( Collections.binarySearch(desired, id) >= 0 )
            {
                found++;
            }
            else
            {
                System.out.println("Not in desired set:");
                printXAlignResult(top20.get(id));
            }
        }

        System.out.println(found + " out of our desired proteins were in the top " + top20.size() );

        found = 0;
        for ( String id: top10.keySet() )
        {
            if ( Collections.binarySearch(desired, id) >= 0 )
            {
                found++;
            }
        }

        System.out.println(found + " out of our desired proteins were in the top " + top10.size() );

        found = 0;
        for ( Map.Entry<String, XAlignResult> entry: entries )
        {
            if ( Collections.binarySearch(desired, entry.getKey()) >= 0 )
            {
                found++;
            }
        }

        System.out.println(found + " out of our desired proteins were in the total set " );
    }

    private void writeSuperDB(StructureDB superDB)
    {
        try
        {
            PrintStream out = new PrintStream(SUPER_DB);
            superDB.writeDB(out);
            out.close();
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    private StructureAnnotatedSequence get9StateSequence(String sequenceName)
            throws IllegalSymbolException, IllegalIDException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();

        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");

        String pathname = config.getStringValue("pdb_dir");

        InputStream in = PDBFileLoader.getPDBFile(sequenceName, new File[]{new File(pathname)});

        PDBFileParser pdbReader = new PDBFileParser();

        Structure pdb;
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return null;
        }

        Chain chain = pdb.getChain(0);
        HomodellerUtils.removeHetatoms(chain);
        TreeMap<Integer, Angle> angles = HomodellerUtils.getAngles(chain);
        StructureAnnotatedSequence str = AngleConverter.convertToSecondaryStructure(chain.getSequence(), angles);
        Sequence _9state = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), sequenceName);

        // get the 3-state sequence from the database
        StructureAnnotatedSequence _3state = null;

        // It's in the database
        if ( db.containsSequence(sequenceName) )
        {
            _3state = (StructureAnnotatedSequence) db.getSequence(sequenceName);
        }                                                   
        else if ( superDB.containsSequence(sequenceName))
        {
            _3state = (StructureAnnotatedSequence) superDB.getSequence(sequenceName);
        }

        if ( _3state == null || _3state.length() != _9state.length() )
        {
            if ( _3state== null)
            {
                System.out.println(sequenceName + " does not exist in DB");
            }
            else
            {
                System.out.println(sequenceName + " was wrong in DB");
            }

            if ( pdb.getName().length() == 0 )
            {
                pdb.setName(sequenceName);
            }

            // Not in DB, run VADAR on it
            _3state = (StructureAnnotatedSequence) VadarRunner.getSecondaryStructure(pdb);

            if ( _3state == null || _3state.length() != _9state.length() )
            {
                System.err.println("VADAR produced incorrect structure");
                 return null;
            }

            _3state.setName(sequenceName);
            superDB.addSequence(_3state);

            /*     PrintStream out = new PrintStream(SUPER_DB);
             superDB.writeDB(out);
             out.close();*/
        }

//        assertEquals(_9state.length(), _3state.length());

        return new StructureAnnotatedSequence(_9state, _3state.getStructureAnnotation());
    }

    public void test1F21() throws IllegalSymbolException, IllegalIDException
    {
        String[] results = {
                "1HNIA", "1FKPA", "1C0TA", "1C0MA", "1JLGA", "3HVTA", "1EP4A", "1IKXA", "1EKEB", "1DLOA",
                "1C0MB", "1HVUA", "1HQUA", "1RT3A", "1JLCA", "1DTTA", "1REVA", "1JLFA", "1EKEA", "1QE1A",
                "1RDHA", "1JLQA", "1JLBA", "1C0MD", "1HYSA", "1IKYA", "1HVUG", "1BQMA", "1RT5A", "1DLOB",
                "1IKVA", "1UWBA", "1JLEA", "2HMIA", "1EETA", "1JLAA", "1TVRB", "1RT4A", "1HQUB", "1RT7A",
                "1HRHA", "1HNIB", "1C0UA", "1FKOA", "1JKHA", "1RT6A", "1DTQA", "1HNVA", "1HNVB", "1FK9A",
                "1HMVA", "1TVRA", "1HPZB", "1BQMB", "1C0MC", "1RDHB", "1RTJA", "1BQNB", "1HQEA", "1KLMA",
                "1HRHB", "1FKOB", "1BQNA", "1VRTA", "1HPZA", "1C1CA", "1VRUA", "1RT2A", "1QE1B", "1RTDC",
                "1HVUJ", "1RT1A", "1RTIA", "1HVUD", "1RTHA", "1IKWA", "1RTDA", "1UWBB", "1C1BA"
        };

        runForResults("1F21", results);
    }

    public void test4ICB() throws IllegalSymbolException, IllegalIDException
    {
        String[] results = {
                "1NCY_", "1TRF_", "1HQVA", "1I84T", "1AJ5B", "1BR1D", "1EJ3B", "1CDLC", "1BR4D", "1F4OB",
                "1ALWB", "2CTN_", "1BR4F", "1ALVB", "1J7RA", "2SAS_", "1SMG_", "1BR1B", "1AJ4_", "1NCX_",
                "1B47A", "4TNC_", "1DF0B", "1BR4B", "1F4QB", "5TNC_", "1ZAC_", "1SPY_", "1J7QA", "1DVIA",
                "1QLSA", "1B47B", "1DF0A", "1BR1F", "1NCZ_", "1CMF_", "1ALVA", "1DVIB", "2SCPA", "1C07A",
                "1JC2A", "2TN4_", "1JF2A", "1I84W", "1TCF_", "1SKT_", "2CBLA", "1FBVA", "1AP4_", "1B47C",
                "1SRA_", "2SCPB", "1F4OA", "1BR1H", "1TOP_", "1AJ5A", "1DMO_", "1F4QA", "1BR4H", "1CFC_",
                "1CFD_", "1A2XA", "1ALWA", "1F70A", "1QJTA"
        };

        runForResults("4ICB", results);
    }

    public void test1UBQ() throws IllegalSymbolException, IllegalIDException
    {
        String[] results = {
                "1E5WA", "1D4BA", "1GG3C", "1GC7A", "1F2RC", "1E8ZA", "1HE8A", "1EF1A", "1E7VA", "1GG3A", "1IP9A",
                "1LFDA", "1LFDC", "1LXDA", "1FMAD", "1VCBA", "1EF1B", "1C9FA", "2RGF_", "1RRB_", "1FM0D", "1IPGA",
                "1EO6B", "1EF5A", "1IBXA", "1LXDB", "1A5R_", "1E8XA", "1GG3B", "1E8WA", "1EO6A", "1C1YB", "1RFA_",
                "1IBXB", "1E7UA", "1EUVB", "1F2RI", "1GC6A", "1H8CA", "1RLF_", "1VCBG", "1GUAB", "1E8YA", "1JRUA",
                "1E90A", "1VCBD", "1RAXA", "1VCBJ", "1I42A"};

        runForResults("1UBQ", results);
    }

    private void testThreading(String ID, String[] results)
    {
        // TODO run threading, get a list of possible aligned files

        // then go through the results and see if they are there

        // TODO manually look at alignments

    }

    public void getStructures()
    {
        // The URLS were found by running CE, and getting the result URL
        // http://cl.sdsc.ca/ce/all-to-all/all-to-all.html

        InputStream xml = getXML("http://cl.sdsc.edu/ce_scratch/nb28752.html");
        Map<String, Double> structures = getStructures(xml);
        writeStructures("3TRX", structures);

        xml = getXML("http://cl.sdsc.edu/ce_scratch/nb4534.html");
        structures = getStructures(xml);
        writeStructures("1F21", structures);

        xml = getXML("http://cl.sdsc.edu/ce_scratch/nb6768.html");
        structures = getStructures(xml);
        writeStructures("4ICB", structures);

        xml = getXML("http://cl.sdsc.edu/ce_scratch/nb8311.html");
        structures = getStructures(xml);
        writeStructures("1UBQ", structures);
    }

    private void writeStructures(String s, Map<String, Double> structures)
    {
        System.out.println(s);
        System.out.println("{");
        for (String ID : structures.keySet())
        {
            System.out.print("\"" + ID.replace(" Neighbors", "").replace(":","") + "\", ");
        }
        System.out.println("}");
    }

    private Map<String, Double> getStructures(InputStream xml)
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try
        {
            builder = factory.newDocumentBuilder();
            Document doc = builder.parse(xml);

            Element root = doc.getDocumentElement();

            Map<String, Double> Ids = new HashMap<String, Double>();

            for (int i = 0; i < root.getChildNodes().getLength(); i++)
            {
                Node node = root.getChildNodes().item(i);

                if (node.getNodeName().equals("ID"))
                {
                    Node seqnode = root.getChildNodes().item(i + 2);
                    String ID = node.getTextContent();
                    String seqIdentity = seqnode.getTextContent();


                    try
                    {
                        double v = Double.parseDouble(seqIdentity);
                        if (v < CUTOFF)
                        {
                            Ids.put(ID, v);
                        }
                    }
                    catch (NumberFormatException e)
                    {

                    }
                }
            }

            return Ids;
        }
        catch (ParserConfigurationException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (SAXException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return null;
    }

    /**
     * Get a parse of the structrures using dapper, www.dappit.com
     * Dapper name = CE Find All 2
     * User: proteus88
     * Pass: metal
     *
     * @param url
     * @return XML stream
     */
    private InputStream getXML(String url)
    {


        HttpClient httpClient = new HttpClient();
        HttpMethod method = new GetMethod("http://www.dappit.com/RunDapp?dappName=CEFindAll2&v=1&applyToUrl=" + url);
        InputStream responseBodyAsStream = null;
        try
        {
            int httpresult = httpClient.executeMethod(method);
            if (httpresult != HttpStatus.SC_OK)
            {
                System.err.println("Method failed: " + method.getStatusLine());
            }

            if (httpresult == HttpStatus.SC_NOT_FOUND)
            {
                throw new FileNotFoundException("Error downloading file");
            }

            responseBodyAsStream = method.getResponseBodyAsStream();

            return responseBodyAsStream;
        }
        catch (NullPointerException e)
        {
            e.printStackTrace();
            System.err.println("Error was handled, couldn't download PDB file");
        }
        catch (HttpException e)
        {
            System.err.println("Fatal protocol violation " + e.getMessage());
            e.printStackTrace();
        }
        catch (IOException e)
        {
            System.err.println("Fatal transport error " + e.getMessage());
            e.printStackTrace();
        }
        return responseBodyAsStream;
    }

    /**
     * Here I'm getting the 9-state structure of a test PDB file, and then running XAlign
     * on it iterating through a bunch of 9-state structures in the database, like a slow BLAST
     *
     * @throws BioException
     */
    public void test9StateXAlign() throws BioException, IOException
    {
        List<Sequence> sequences = ProteusTools.parseFasta(new File("seqsee9.db"));

        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");

        String pathname = config.getStringValue("pdb_dir");
        InputStream in = PDBFileLoader.getPDBFile("3TRX", new File[]{new File(pathname)});

        PDBFileParser pdbReader = new PDBFileParser();

        Structure pdb = null;
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        Chain chain = pdb.getChain(0);
        TreeMap<Integer, Angle> angles = HomodellerUtils.getAngles(chain);
        StructureAnnotatedSequence str = AngleConverter.convertToSecondaryStructure(chain.getSequence(), angles);
        Sequence query = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), "QUERY");

        SeqSeeDB seqSeeDB = SeqSeeDB.getInstance();

        double maxPPSH = 0;
        Sequence bestMatch = null;

        for ( int i=0; i<1000; i++)
        {
            // 9-state structure
            Sequence sequence9 = sequences.get(i);

            if ( ! seqSeeDB.containsSequence(sequence9.getName()))
            {
                continue;
            }

            // 3-state structure
            StructureAnnotatedSequence structure = (StructureAnnotatedSequence) seqSeeDB.getSequence(sequence9.getName());

            /* we have the 3-state for all our templates, but not our query sequence.
             we will use the 9-state sequence as the "sequence", and the 3-state as the structure.
             That way, XAlign will be less likely to split the alignment in a structure.
            */
            StructureAnnotatedSequence template = new StructureAnnotatedSequence(sequence9, structure.getStructureAnnotation());


            try
            {
                FastXAlignRunner runner = new FastXAlignRunner(template, query);

                runner.run();

                Alignment alignment = runner.getAlignment();
                Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

                Double ppsh = ppshMap.get(sequence9.getName()).get(Consts.QUERY);

                if ( ppsh > 53 )
                {
                    System.out.println(ppsh);

                    for ( Object label: alignment.getLabels())
                    {
                        SymbolList seq = alignment.symbolListForLabel(label);

                        System.out.println(label + ": " + seq.seqString());
                    }
                }

                if ( ppsh > maxPPSH )
                {
                    maxPPSH = ppsh;
                    bestMatch = sequence9;
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }

        System.out.println(bestMatch.getName());
        System.out.println(maxPPSH);
    }

    /**
     * Here I'm getting the 9-state structure of 2TRX and 3TRX, and then running XAlign
     * to see if they match up OK
     *
     * @throws BioException
     */
    public void testAlignment() throws BioException, IOException
    {
        List<Sequence> sequences = ProteusTools.parseFasta(new File("seqsee9.db"));

        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");

        String pathname = config.getStringValue("pdb_dir");
        InputStream in = PDBFileLoader.getPDBFile("3TRX", new File[]{new File(pathname)});

        PDBFileParser pdbReader = new PDBFileParser();

        Structure pdb = null;
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        Chain _3TRX = pdb.getChain(0);
        TreeMap<Integer, Angle> angles = HomodellerUtils.getAngles(_3TRX);
        StructureAnnotatedSequence str = AngleConverter.convertToSecondaryStructure(_3TRX.getSequence(), angles);
        Sequence _3TRXSequence = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), "_3TRX");
        String str3State = "CCEECCCHHHHHHHHHHCCCCCEEEEEECHHHHCCCCCHHHHHCHHHHCCCHEEEEEECCCCHHHHHHHCCCCCCEEEEEECCEEEEEEECCCCHHHHHHHHHHC";

        assertEquals(_3TRXSequence.length(), str3State.length());


        in = PDBFileLoader.getPDBFile("2TRX", new File[]{new File(pathname)});
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        Chain _2TRX = pdb.getChain(0);

        HomodellerUtils.removeHetatoms(_2TRX);

        angles = HomodellerUtils.getAngles(_2TRX);
        str = AngleConverter.convertToSecondaryStructure(_2TRX.getSequence(), angles);
        Sequence _2TRXSequence = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), "_2TRX");

        str3State = "CCCEEECCCCCHHHHCCCCCCEEEEEEECCCCHHHHHHHHHHHHHHHHCCCCEEEEEEECCCCCCHHHHCCCCCCCEEEEEECCEEEEEEECCCCHHHHHHHHHHHHC";

        assertEquals(_2TRXSequence.length(), str3State.length());

        StructureAnnotatedSequence query = new StructureAnnotatedSequence(_2TRXSequence, str3State);

        FastXAlignRunner runner = new FastXAlignRunner(query, _3TRXSequence);

        runner.run();

        Alignment alignment = runner.getAlignment();
        Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

        Double ppsh = ppshMap.get(query.getName()).get(_3TRXSequence.getName());

        System.out.println(ppsh);

        for ( Object label: alignment.getLabels())
        {
            SymbolList seq = alignment.symbolListForLabel(label);

            System.out.println(label + ": " + seq.seqString());
        }
    }

    /**
     * Here I'm getting the 9-state structure of 2TRX and 3TRX, and then running XAlign
     * to see if they match up OK, but I'm doing it on the last half of the sequences.
     *
     * @throws BioException
     */
    public void testAlignment2() throws BioException, IOException
    {
        List<Sequence> sequences = ProteusTools.parseFasta(new File("seqsee9.db"));

        Config config = Config.getConfig(Consts.HOMODELLER, "config/homodeller.xml");

        String pathname = config.getStringValue("pdb_dir");
        InputStream in = PDBFileLoader.getPDBFile("3TRX", new File[]{new File(pathname)});

        PDBFileParser pdbReader = new PDBFileParser();

        Structure pdb = null;
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        Chain _3TRX = pdb.getChain(0);
        TreeMap<Integer, Angle> angles = HomodellerUtils.getAngles(_3TRX);
        StructureAnnotatedSequence str = AngleConverter.convertToSecondaryStructure(_3TRX.getSequence(), angles);
        Sequence _3TRXSequence = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), "_3TRX");
        String str3State = "CCEECCCHHHHHHHHHHCCCCCEEEEEECHHHHCCCCCHHHHHCHHHHCCCHEEEEEECCCCHHHHHHHCCCCCCEEEEEECCEEEEEEECCCCHHHHHHHHHHC";

        assertEquals(_3TRXSequence.length(), str3State.length());


        in = PDBFileLoader.getPDBFile("2TRX", new File[]{new File(pathname)});
        try
        {
            pdb = pdbReader.parsePDBFile(in);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        Chain _2TRX = pdb.getChain(0);

        HomodellerUtils.removeHetatoms(_2TRX);

        angles = HomodellerUtils.getAngles(_2TRX);
        str = AngleConverter.convertToSecondaryStructure(_2TRX.getSequence(), angles);
        Sequence _2TRXSequence = ProteinTools.createProteinSequence(str.getStructureAnnotation().seqString(), "_2TRX");

        str3State = "CCCEEECCCCCHHHHCCCCCCEEEEEEECCCCHHHHHHHHHHHHHHHHCCCCEEEEEEECCCCCCHHHHCCCCCCCEEEEEECCEEEEEEECCCCHHHHHHHHHHHHC";

        assertEquals(_2TRXSequence.length(), str3State.length());

        StructureAnnotatedSequence query = new StructureAnnotatedSequence(_2TRXSequence, str3State);

        SimpleSequence template = new SimpleSequence(_3TRXSequence.subList(25, _3TRXSequence.length() - 50 ), "", _3TRXSequence.getName(), _3TRXSequence.getAnnotation());
        FastXAlignRunner runner = new FastXAlignRunner(query.subSequence(30, query.length() - 33), template);

        runner.run();

        Alignment alignment = runner.getAlignment();
        Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

        Double ppsh = ppshMap.get(query.getName()).get(_3TRXSequence.getName());

        System.out.println(ppsh);

        for ( Object label: alignment.getLabels())
        {
            SymbolList seq = alignment.symbolListForLabel(label);

            System.out.println(label + ": " + seq.seqString());
        }
    }

    public void testXalignResult() throws IllegalSymbolException, IllegalIDException
    {
        Sequence _3TRXSequence = get9StateSequence("3TRX");
        StructureAnnotatedSequence _9state = get9StateSequence("2TRX");

        writeSuperDB(superDB);

        Alignment alignment;

        // Run XAlign
        FastXAlignRunner runner = new FastXAlignRunner(_9state, _3TRXSequence);

        runner.run();

        alignment = runner.getAlignment();
        Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

        Double ppsh = ppshMap.get(_9state.getName()).get(_3TRXSequence.getName());

        XAlignResult xAlignResult = new XAlignResult(alignment, ppsh, _3TRXSequence.getName());

        Double score = xAlignResult.getAdjustedScore();

        System.out.println(score);

    }

    public void testXalignResult2() throws IllegalSymbolException, IllegalIDException
    {
        Sequence query = get9StateSequence("1F21");
        StructureAnnotatedSequence _1PP5A = get9StateSequence("1PP5A");
        StructureAnnotatedSequence _1TBUA = get9StateSequence("1TBUA");
        StructureAnnotatedSequence _1C0MA = get9StateSequence("1C0MA");
        StructureAnnotatedSequence _1H641 = get9StateSequence("1H641");


        writeSuperDB(superDB);

        Alignment alignment;

        // Run XAlign
        XAlignResult xAlignResult1 = runOn2Sequences(_1PP5A, query);
        XAlignResult xAlignResult2 = runOn2Sequences(_1TBUA, query);
        XAlignResult xAlignResult3 = runOn2Sequences(_1C0MA, query);
        XAlignResult xAlignResult4 = runOn2Sequences(_1H641, query);

        printXAlignResult(xAlignResult1);
        printXAlignResult(xAlignResult2);
        printXAlignResult(xAlignResult3);
        printXAlignResult(xAlignResult4);

        double score = xAlignResult1.getMetric1();
        double score2 = xAlignResult2.getMetric1();
        double score3 = xAlignResult3.getMetric1();
        double score4 = xAlignResult4.getMetric1();

        System.out.println(score);
        System.out.println(score2);
        System.out.println(score3);
        System.out.println(score4);

        // 1TBUA is much better than 1PP5A
        assertTrue(score2 > score);

        score = xAlignResult1.getMetric2();
        score2 = xAlignResult2.getMetric2();
        score3 = xAlignResult3.getMetric2();
        score4 = xAlignResult4.getMetric2();

        System.out.println(score);
        System.out.println(score2);
        System.out.println(score3);
        System.out.println(score4);

        // 1TBUA is much better than 1PP5A
        assertTrue(score2 > score);
    }

    private void printXAlignResult(XAlignResult xAlignResult1)
    {
        for ( Object label: xAlignResult1.alignment.getLabels())
        {
            SymbolList seq = xAlignResult1.alignment.symbolListForLabel(label);

            System.out.println(label + ": " + seq.seqString());
        }
        System.out.println("");
    }

    private XAlignResult runOn2Sequences(StructureAnnotatedSequence _1PP5A, Sequence _3TRXSequence)
    {
        Alignment alignment;
        FastXAlignRunner runner = new FastXAlignRunner(_1PP5A, _3TRXSequence);

        runner.run();

        alignment = runner.getAlignment();
        Map<String, Map<String, Double>> ppshMap = runner.getPpshMap();

        Double ppsh = ppshMap.get(_1PP5A.getName()).get(_3TRXSequence.getName());

        XAlignResult xAlignResult = new XAlignResult(alignment, ppsh, _3TRXSequence.getName());
        return xAlignResult;
    }
}

class XAlignResult
{
    public Alignment alignment;
    public Double score;
    private String queryName;

    public Double getAdjustedScore()
    {
        return getMetric1();
    }

    /**
     *
     * @return Score minus the difference in lengths
     */
    public double getMetric1()
    {
        assert alignment.getLabels().size() == 2;

        int firstGaps[] = new int[2];
        int lastGaps[] = new int[2];
        int index = 0;
        for ( Object label: alignment.getLabels())
        {
            GappedSequence seq = (GappedSequence) alignment.symbolListForLabel(label);
            firstGaps[index] = seq.firstNonGap();
            lastGaps[index] = seq.lastNonGap();

            index++;
        }

        int lengthDiff = Math.abs(firstGaps[0] - firstGaps[1]);
        lengthDiff += Math.abs(lastGaps[0] - lastGaps[1]);

        return score - lengthDiff;
    }

    /**
     * Score + Overlapping region
     * @return Score + Overlapping region
     */
    public double getMetric2()
    {
        assert alignment.getLabels().size() == 2;

        GappedSequence query = null;
        GappedSequence other = null;

        for ( Object label: alignment.getLabels())
        {
            if ( label.equals(queryName))
            {
                query = (GappedSequence) alignment.symbolListForLabel(label);
            }
            else
            {
                other = (GappedSequence) alignment.symbolListForLabel(label);
            }
        }

        assert query != null;
        assert other != null;

        int lengthDiff = 0;
        int overlap = query.length();

        // calculate the overlapping region of the alignment
        if ( query.firstNonGap() == 1 )
        {
            lengthDiff += other.firstNonGap();

            overlap -= other.firstNonGap();
        }

        if ( query.lastNonGap() == query.length()  )
        {
            overlap -= (other.length() - other.lastNonGap());
        }


        return score + overlap;
    }

    public XAlignResult(Alignment alignment, Double score, String name)
    {
        this.alignment = alignment;
        this.score = score;
        this.queryName = name;
    }
}