package ca.ualberta.proteus.program.tmbhunt;

import ca.ualberta.proteus.logging.RuntimeCatalog;
import org.apache.log4j.Logger;
import ca.ualberta.proteus.util.Consts;
import java.io.BufferedReader;
import java.io.Reader;
import java.text.ParseException;
/**
 * Created by Eclipse IDEA.
 * User: savita shrivastava
 * Date: Feb 13, 2007
 * Time: 11:30:00 AM
 */

public class TMBHuntReader {
	 private Logger logger = Logger.getLogger(TMBHuntReader.class);
	 public RuntimeCatalog pLogger = new RuntimeCatalog();
	    
	 public TMBHuntReader() {}
	    
	 /**
	    * This method parse the output file of TMBHunt program.
	    *
	    * @param file Output file path
	    * @returns A String containing the whether a protein is beta barrel or not
	    * 
	    */
	 
	 public boolean parseTMBHuntOutput(Reader reader) throws ParseException{
		 String line;
	     double ratio = 0;
         double eValue = 0; 
	     try{
	    	 BufferedReader br = new BufferedReader(reader);
	         String name;
	         while ((line = br.readLine()) != null){
	        	 if (line.length() > 1){
	        		 name = line.substring(1);
	                 String[] str = line.split("\\s\\s");
	                    
	                 for(String str2: str){
	                	 //System.out.println(str2);		
	                    if(str2.contains("ExpDataBBTM_ratio:")){
	                    	ratio = Double.parseDouble(str2.substring(19));
	                    }
	                    if(str2.contains("ExpDataE-value:")){
	                    	eValue = Double.parseDouble(str2.substring(15));
	                    }
	                 }

	                  // print the evalues and score to the log
	                    logger.info("TMB-Hunt Prediction: Score is " + ratio +" Evalue is "+ eValue);
	                                                     
	             }
	           }      
	     }
	     catch (Exception e){
	    	 e.printStackTrace();
	         throw new ParseException("Error parsing TMB-Hunt output file", 0);
	     }
	      return isProteinBbtm(ratio,eValue);
     }   
	    
	    
     public boolean isProteinBbtm(double ratio, double eValue){
    	 boolean isbbtm = false;
    	 if(ratio >=6 && eValue<=0.1){
    		 isbbtm = true;
    		 pLogger.logObject(Consts.TMBHUNT_EVALUE, String.valueOf(eValue));
    		 pLogger.logObject(Consts.TMBHUNT_SCORE, String.valueOf(ratio));
	    }
	   	return isbbtm;
	 }

}
