package ca.ualberta.proteus.program.tmhmm;

import org.apache.log4j.Logger;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Arrays;

/**
 * Created by Eclipse IDEA.
 * User: savita shrivastava
 * Date: Jan 18, 2007
 * Time: 10:00:00 AM
 */

public class TMHMMReader
{
    private Logger logger = Logger.getLogger(TMHMMReader.class);


    public TMHMMReader()
    {
    }

    /**
     * This method parse the output file of TMHMM program.
     *
     * @param file Output file path
     * @returns A String containing the TMHMM predicted transmembrane region(s)
     * for a sequence
     */
    public SymbolList parseTMHMMOutput(Reader file) throws IOException
    {
        String regions = "";
        int length = 0;
        BufferedReader bReader = new BufferedReader(file);
        String line = null;
        
        try
        {
            line = bReader.readLine();
            if (line == null)
            {
                throw new IOException("could not parse TMHMM output file, BufferedReader returns null, File may be empty!");
            }
            if (line.length() > 0 && line.contains("Topology"))
            {
                String[] strings = line.split("\t");
                for ( String chunk : strings )
                {
                    if ( chunk.contains("len"))
                    {
                        length = Integer.parseInt(chunk.substring(4));
                    }

                    if (line.contains("Topology"))
                    {
                        String newLine[] = line.split("Topology=");
                        String str = newLine[1].replace("i", "; ");
                        str = str.replace("o", "; ");
                        int lastIndex = str.lastIndexOf(";");
                        int beginIndex = 2;
                        if ( lastIndex > 0 )
                        {
                            regions = str.substring(beginIndex, lastIndex);
                        }
                    }
                }
            }
            return parseRegions(regions, length);
        }
        catch (Exception e)
        {
            logger.error("Error parsing TMHMM output file", e);
            if ( line == null || line.length() == 0)
            {
                logger.error("Output was null");
            }
            else
            {
                logger.error(line);
            }
            throw new IOException("Error parsing TMHMM output file");
        }
	}

    private SymbolList parseRegions(String regionStr, int length)
    {
        String[] regions = regionStr.split("\\s*;\\s*");

        Symbol[] symbols = new Symbol[length];
        StructureTools tools = new StructureTools();
        Arrays.fill(symbols, tools.get_());

        for ( String s : regions)
        {
            String[] strings = s.split("-");

            if ( strings.length == 2)
            {
                int start = Integer.parseInt(strings[0]);
                int end = Integer.parseInt(strings[1]);
    
                for ( int i=start; i<=end; i++)
                {
                    symbols[i] = tools.getT();
                }
            }
        }

        return new SimpleSymbolList(symbols, length, tools.getStructure());
    }
}
