package ca.ualberta.proteus.program.tmhmm.test;

import ca.ualberta.proteus.program.tmhmm.TMHMMReader;
import ca.ualberta.proteus.program.tmhmm.TMHMMRunner;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.StructureDB;
import junit.framework.TestCase;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.SymbolList;

import java.io.*;

/**
 * Created by Eclipse IDEA.
 * User: savita
 * Date: Jan 18, 2007
 * Time: 10:52:44 PM
 */

public class TMHMMReaderTest extends TestCase
{

    public void testReader() throws IOException
    {
        String testFile = "Translated_549_residues\tlen=549\tExpAA=66.56\tFirst60=0.90\tPredHel=3\tTopology=i70-92o107-124i131-153o";

        Reader file = new StringReader(testFile);
        TMHMMReader tmReader = new TMHMMReader();
        SymbolList transmembrane = tmReader.parseTMHMMOutput(file);
        if (transmembrane != null)
        {
            System.out.println("Transmembrane Regions : " + transmembrane.seqString());
        }

    }
    public void testReader2() throws IOException
    {
        String testFile = "900136\tlen=33\tExpAA=0.15\tFirst60=0.1\tPredHel=0\tTopology=o";

        Reader file = new StringReader(testFile);
        TMHMMReader tmReader = new TMHMMReader();
        SymbolList transmembrane = tmReader.parseTMHMMOutput(file);
        if (transmembrane != null)
        {
            System.out.println("Transmembrane Regions (with space): \n" + transmembrane.seqString());
            System.out.println("Transmembrane Regions (with -):\n" + transmembrane.seqString().replace(" ", "-"));
        }

    }

    public void testRunner() throws BioException, IOException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);
        Sequence sequence = db.getSequence(0);

        TMHMMRunner runner = new TMHMMRunner(sequence);
        StructureAnnotatedSequence s = runner.runTMHMM();

        System.out.println(s);
    }

    public void testSerialization() throws BioException, IOException, ClassNotFoundException
    {
        Config config = Config.getConfig();
        String tmpdb = config.getStringValue("TMPDB");

        StructureDB db = new StructureDB(tmpdb, StructureDB.MODE.READ);
        Sequence sequence = db.getSequence(0);

        TMHMMRunner runner = new TMHMMRunner(sequence);
        StructureAnnotatedSequence s = runner.runTMHMM();

        ObjectOutputStream stream = new ObjectOutputStream(new FileOutputStream("tmp"));
        stream.writeObject(s);
        stream.close();

        ObjectInputStream in = new ObjectInputStream(new FileInputStream("tmp"));
        Object o = in.readObject();

        System.out.println(o);
    }
}
