package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.GappedStructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.*;
import org.joone.engine.DirectSynapse;
import org.joone.engine.Pattern;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;


/**
 * Created by IntelliJ IDEA.
 *
 * @author Scott Montgomerie
 *         Date: Apr 3, 2005
 *         Time: 8:33:47 PM
 *         To change this template use File | Settings | File Templates.
 */
public class ConsensusTrainingSet extends TrainingSet
{
    public static int WINDOW_SIZE = 9;
    public static final int MIDDLERES = (int) WINDOW_SIZE / 2;

    public static final int NUM_INPUTS = ProteusTools.NUM_STRUCTURES * WINDOW_SIZE;
    public static final int NUM_OUTPUTS = ProteusTools.NUM_STRUCTURES;

    public boolean storeData = false;
    private static final String DATA_FILE = "consensusTrainingSet.xml";

    private boolean useGapInTrainingSet = true; //TODO try without?
    private boolean verbose = true;

    public HashMap<String, PSSM> getPssmList()
    {
        return pssmList;
    }

    private HashMap<String, PSSM> pssmList = new HashMap<String, PSSM>();

    /**
     * Add instance to the training set with a sequence and structure
     *
     * @param sequences - The list of sequences to add
     */
    public void addInstances(List<StructureAnnotatedSequence> sequences)
    {
       /* List blastProgs = new ArrayList();
        for (Iterator it = sequences.iterator(); it.hasNext();)
        {
            StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.next();
            if (XAlignDB.getInstance().contains(sequence))
            {
                // not in the DB, run BLAST
                BlastRunner blastRunner = new BlastRunner(sequence);

                // TODO FIX THIS TO USE PSIBLAST AFTER TESTING
                blastRunner.setBlastType(BlastRunner.BLASTTYPE.PSIBLAST);
                blastRunner.start();
                blastProgs.add(blastRunner);
            }
        }

        // Go through all processes and see if they are finished
        boolean completedAllBlast = false;
        while (!completedAllBlast)
        {
            completedAllBlast = true;
            for (int i = 0; i < blastProgs.size(); i++)
            {
                BlastRunner blastRunner = (BlastRunner) blastProgs.get(i);

                if (!blastRunner.waitFor())
                    completedAllBlast = false;
            }

            sleep(100);
        }

        // get the list of alignments out of the blast results
        List<List<Sequence>> alignmentList = new ArrayList<List<Sequence>>();
        for (int i = 0; i < blastProgs.size(); i++)
        {
            BlastRunner blastRunner = (BlastRunner) blastProgs.get(i);
            alignmentList.add(blastRunner.getUnalignedResult());
        }

        for (Iterator it = sequences.iterator(); it.hasNext();)
        {
            StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.next();
            if (AlignmentDB.getInstance().contains(sequence))
            {
                SimpleAlignment alignment = (SimpleAlignment) AlignmentDB.getInstance().get(sequence);

                List<Sequence> seqList = new ArrayList<Sequence>();
                for (Iterator seqIt = alignment.symbolListIterator(); seqIt.hasNext();)
                {
                    seqList.add(GappedStructureAnnotatedSequence.removeGaps((Sequence) seqIt.next()));
                }
                alignmentList.add(seqList);
            }
        }


        // remove the DB to clear some memory
        AlignmentDB.getInstance().clearDB();

        assert(alignmentList.size() == sequences.size());

        */
        //Predictor predictor = ConsensusPredictor.getPredictor();
        //SeqSeeDB db = SeqSeeDB.getInstance();

        // create PSSM's for each sequence/BLAST result set, and pass that to the create instance method
        for (int i = 0; i < sequences.size(); i++)
        {
            try
            {
                final StructureAnnotatedSequence sequence = sequences.get(i);
                //PSSM pssm = new PSSM(ProteinTools.getAlphabet(), sequence.length());
                //pssm.addQuerySequence(sequence);

                Alignment alignment;
                Map<String, Map<String, Double>> ppshMap;
                if (!XAlignDB.getInstance().contains(sequence))
                {

                    // not in the DB, run BLAST
                    BlastSettings settings = new BlastSettings();
                    settings.setSequence(sequence);
                    
                    // TODO FIX THIS TO USE PSIBLAST AFTER TESTING
                    settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);

                    BlastRunner blastRunner = new BlastRunner(settings);
                    blastRunner.run();

                    List<Sequence> alignments = blastRunner.getUnalignedResult();

                    // eliminate some before heading to XAlign
                    String numAlignments = Config.getConfig().getStringValue("num_alignments");
                    ProteusTools.chooseAlignments(sequence, alignments, Integer.valueOf(numAlignments) + 100 - 1);// leave 100 in there to be reduced later on
                    alignments.add(sequence);


                    XAlignRunner xAlignRunner = new XAlignRunner(sequence, alignments);
                    //xAlignRunner.setUseCache(false);  // can't use the cache cuz we need the PPSH
                    //xAlignRunner.setAddToDB(false);  // can't use the cache cuz we need the PPSH
                    xAlignRunner.run();

                    alignment = xAlignRunner.getAlignment();
                    ppshMap = xAlignRunner.getPPSHMap();
                }
                else
                {
                    // Already in database no need to recreate it
                    alignment = XAlignDB.getInstance().get(sequence);
                    ppshMap = XAlignDB.getInstance().getPSSHMap(sequence);
                }

                // eliminate some alignments
                alignment = ProteusTools.chooseAlignments(sequence, alignment, ppshMap, ProteusTools.MIN_PPSH_THRESHOLD, ProteusTools.MAX_PPSH_THRESHOLD);

                PSSM pssm = new PSSM(sequence, alignment, ConsensusPredictor.getPredictor());

                if (storeData)
                    pssmList.put(sequence.getName(), pssm);

                for (int resindex = 0; resindex < pssm.length(); resindex++)
                {
                    addInputInstance(pssm, resindex);

                    addOutputInstance(pssm, resindex);
                }

                //AlignmentDB.getInstance().clearDB();
            }
            catch (Throwable e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     *
     * @param sequence - The sequence to add
     */
    public void addInstance(Sequence sequence)
    {
        throw new IllegalArgumentException("Don't call this - use AddInstances, and getPredictionResult");
    }

    public void addInstance(StructureAnnotatedSequence sequence)
    {
        throw new IllegalArgumentException("Don't call this - use AddInstances, and getPredictionResult");
    }


    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     *
     * @param sequence - The sequence to add
     * @param memInp
     * @param memOut
     */
    public ScoreAnnotatedSequence getPredictionResult(Sequence sequence, DirectSynapse memInp, DirectSynapse memOut)
    {
        return getPredictionResult(sequence, memInp, memOut, null);
    }


    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     *
     * @param sequence - The sequence to add
     * @param memInp
     * @param memOut
     */
    public ScoreAnnotatedSequence getPredictionResult(Sequence sequence, DirectSynapse memInp, DirectSynapse memOut, PSSM storedPSSM)
    {
        PSSM pssm;
        if (storedPSSM != null)
        {
            pssm = storedPSSM;
        }
        else
            pssm = createPSSM(sequence);

        /* This is done in createPSSM */
        for (int resindex = 0; resindex < pssm.length(); resindex++)
        {
            addInputInstance(pssm, resindex);

            addOutputInstance(pssm, resindex);
        }

        Symbol[] symbolList = new Symbol[numInstances()];

        int index = 1;
        ScoreAnnotatedSequence result = null;

        result = new ScoreAnnotatedSequence(pssm.getGappedQuerySequence());

        for (Iterator it = getInputInstanceIterator(); it.hasNext();)
        {

            // Prepare the next input pattern
            Pattern iPattern = new Pattern((double[]) it.next());
            iPattern.setCount(1);

            // Interrogate the net
            memInp.fwdPut(iPattern);

            // Read the output pattern and print out it
            Pattern pattern = memOut.fwdGet();


            // if ( verbose ) System.out.print("Output #" + (index) + " = ");
            double max = -1;
            int maxIndex = -1;
            final double[] outputArr = pattern.getArray();
            NumberFormat formatter = DecimalFormat.getInstance();
            formatter.setMaximumFractionDigits(3);

            for (int j = 0; j < outputArr.length; j++)
            {
                if (outputArr[j] > max)
                {
                    max = outputArr[j];
                    maxIndex = j;
                }

              //   if ( verbose )  System.out.print(" " + formatter.format(outputArr[j]) );
            }
            //if ( verbose ) System.out.println();

            // set the score of the chosen output
            result.setScore(index, outputArr[ProteusTools.COIL], outputArr[ProteusTools.HELIX], outputArr[ProteusTools.STRAND], outputArr[ProteusTools.GAP]);

            // get the symbol for this prediction
            Symbol symbol;
            try
            {
                symbol = ProteusTools.convertIntToStructure(maxIndex);

                // if it's a gap, it's really a coil
                if (symbol.equals(new StructureTools().getStructure().getGapSymbol()))
                    symbol = new StructureTools().getC();

                symbolList[index - 1] = symbol;

            }
            catch (IllegalSymbolException e)
            {
                e.printStackTrace();
            }

            index++;
        }

        SimpleSymbolList structure = new SimpleSymbolList(symbolList, symbolList.length, new StructureTools().getStructure());

        result.setStructureAnnotation(structure);

        // ungap this prediction
        result = GappedStructureAnnotatedSequence.createUngappedStructure(sequence, result);

        /* Stop training - don't know why this is necessary */
        Pattern stop = new Pattern(new double[getInputInstanceSize()]);
        stop.setCount(-1);
        memInp.fwdPut(stop);
        memOut.fwdGet();

        /*** Just see what a PSSM can do... */
        //pssm.getUnalignedResult();

        return result;

    }

    /**
     * Creates a PSSM based on the sequence provided, by doing a BLAST search and getting the structure for each
     * aligned sequence
     *
     * @param sequence
     * @return a PSSM
     */
    private PSSM createPSSM(Sequence sequence)
    {
        PSSM pssm = null;

        Predictor predictor = ConsensusPredictor.getPredictor();
        Alignment alignment;

        if (!XAlignDB.getInstance().contains(sequence))
        {
            BlastSettings settings = new BlastSettings();
            settings.setSequence(sequence);
            settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);

            BlastRunner blastRunner = new BlastRunner(settings);
            blastRunner.run();
            
            List<Sequence> alignments;

            alignments = blastRunner.getUnalignedResult();

            System.out.println("Number of alignments: " + alignments.size());

            String numAlignments = Config.getConfig().getStringValue("num_alignments");
            ProteusTools.chooseAlignments(sequence, alignments, Integer.valueOf(numAlignments) + 100 - 1); // leave 100 in there to be reduced later on
            alignments.add(sequence);

            XAlignRunner xAlignRunner = new XAlignRunner(sequence, alignments);
            //xAlignRunner.setUseCache(false);  // can't use the cache cuz we need the PPSH
            //xAlignRunner.setAddToDB(false);  // can't use the cache cuz we need the PPSH
            xAlignRunner.run();

            alignment = xAlignRunner.getAlignment();

            alignment = ProteusTools.chooseAlignments(sequence, alignment, xAlignRunner.getPPSHMap(),
                                                      ProteusTools.MIN_PPSH_THRESHOLD, ProteusTools.MAX_PPSH_THRESHOLD);
        }
        else
        {
            // It's in the DB, just get it from there

            XAlignDB db = XAlignDB.getInstance();
            alignment = db.get(sequence);
            alignment = ProteusTools.chooseAlignments(sequence, alignment, db.getPSSHMap(sequence),
                                                      ProteusTools.MIN_PPSH_THRESHOLD, ProteusTools.MAX_PPSH_THRESHOLD);
        }

        // create PSSM for sequence/BLAST result set, and pass that to the create instance method
        pssm = new PSSM(sequence, alignment, predictor);

        //pssm.update(alignment);

        return pssm;
    }


    private void sleep(int time)
    {
        try
        {
            Thread.sleep(time);
        }
        catch (InterruptedException e)
        {
        }
    }

    private void addOutputInstance(PSSM pssm, int resindex)
    {
        SymbolList predictedStructure = null;
        if (pssm.getGappedQuerySequence() != null)
            predictedStructure = pssm.getGappedQuerySequence().getStructureAnnotation();

        double[] outputs = new double[NUM_OUTPUTS];

        /* Now add the output
         If predictedStructure is null, then we will simply add zeroes */
        if (predictedStructure != null && predictedStructure.length() > 0)
        {
            Symbol structureSymbol = predictedStructure.symbolAt(resindex + 1);

            int structAsInt = ProteusTools.convertStructureToInt(structureSymbol);

            outputs[structAsInt] = 1;
        }

        outputArray.add(outputs);
    }

    private void addInputInstance(PSSM pssm, int resindex)
    {
//        Sequence sequence = pssm.getGappedQuerySequence();

        double[] inputs = new double[NUM_INPUTS];

        for (int i = resindex - MIDDLERES; i < resindex - MIDDLERES + WINDOW_SIZE; i++)
        {
            PredictionScore score = new PredictionScore();
            if ((i >= 0) && (i < pssm.length()))
            {
                score.helix = pssm.get(i, ProteusTools.HELIX);
                score.strand = pssm.get(i, ProteusTools.STRAND);
                score.coil = pssm.get(i, ProteusTools.COIL);

                if (useGapInTrainingSet)
                    score.gap = pssm.get(i, ProteusTools.GAP);
            }
            else
            {
                score.gap = 1;
            }

            int index = (i + MIDDLERES - resindex) * ProteusTools.NUM_STRUCTURES;
            inputs[index + ProteusTools.HELIX] = score.helix;
            inputs[index + ProteusTools.COIL] = score.coil;
            inputs[index + ProteusTools.STRAND] = score.strand;
            inputs[index + ProteusTools.GAP] = score.gap;


            /* put the residue information in */
    /*        Symbol residue;
            if ((i >= 0) && (i < sequence.length()))
            {
                residue = sequence.symbolAt(i + 1);
            }
            else
            {
                residue = ProteusTools.x();
            }

            /* Now we know what the residue is, add this knowledge to the training set */

   /*         int resAsInt = 0;
            try
            {
                resAsInt = ProteusTools.convertToInt(residue);
            }
            catch (IllegalAlphabetException e)
            {
                //System.out.println(e + " " + residue);
                try
                {
                    resAsInt = ProteusTools.convertToInt(ProteusTools.x());
                }
                catch (IllegalAlphabetException e1)
                {
                    //      e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (IllegalSymbolException e1)
                {
                    e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
            catch (IllegalSymbolException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

            int offset = WINDOW_SIZE * ProteusTools.NUM_STRUCTURES;

            inputs[offset + (i + MIDDLERES - resindex) * ProteusTools.NUM_RESIDUES + resAsInt] = 1;
            */
        }

        inputArray.add(inputs);

    }

    public void readFromStream(InputStream aInputStream) throws IOException
    {
        BufferedReader reader = new BufferedReader(new InputStreamReader(aInputStream));
        String line;
        while ((line = reader.readLine()) != null)
        {
            String[] values = line.split(";");
            double[] inputInstance = new double[NUM_INPUTS];
            double[] outputInstance = new double[NUM_OUTPUTS];

            if (values.length != NUM_INPUTS + NUM_OUTPUTS)
                throw new IllegalStateException("Training set not in correct format - data does not match the number of inputs and outputs (values = " + values.length + ")");

            double val;
            for (int i = 0; i < values.length; i++)
            {
                val = Double.parseDouble(values[i]);
                if (i < NUM_INPUTS)
                    inputInstance[i] = val;
                else
                    outputInstance[i - NUM_INPUTS] = val;
            }

            double max = 0;
            for (int i = 0; i < NUM_OUTPUTS; i++)
                if (outputInstance[i] > max)
                    max = outputInstance[i];
            if (max != outputInstance[ProteusTools.GAP] || Math.random() < 0.5)
            {
                inputArray.add(inputInstance);
                outputArray.add(outputInstance);
            }
        }

        assert(inputArray.size() == outputArray.size());
    }

    public void saveMetaDataToFile(String filename)
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            DOMImplementation impl = builder.getDOMImplementation();

            Document doc = impl.createDocument(null, "training_set_meta_data", null);

            Element root = doc.getDocumentElement();
            Date today = new Date();
            Attr date = doc.createAttribute("created");
            date.setValue(today.toString());
            root.getAttributes().setNamedItem(date);

            for (Iterator it = pssmList.keySet().iterator(); it.hasNext();)
            {
                String sequenceName = (String) it.next();
                PSSM pssm = pssmList.get(sequenceName);

                Element element = doc.createElement("PSSM");
                element.setAttribute("name", sequenceName);
                pssm.saveToXML(doc, element);

                root.appendChild(element);
            }

            // Prepare the DOM document for writing, and the output file
            Source source = new DOMSource(doc);
            File file = new File(filename);
            Result result = new StreamResult(file);
            // Write the DOM document to file, using a document transformer
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.transform(source, result);
        }
        catch (DOMException e)
        {
            e.printStackTrace();
        }
        catch (ParserConfigurationException e)
        {
            System.err.println("You need to install JAXP for DOM");
        }
        catch (TransformerConfigurationException e)
        {
            e.printStackTrace();  //TODO
        }
        catch (TransformerException e)
        {
            e.printStackTrace();  //TODO
        }
    }

    public void readMetaDataFromFile(String filename) throws IOException
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document doc = builder.parse(filename);

            Element root = doc.getDocumentElement();

            for (int i = 0; i < root.getChildNodes().getLength(); i++)
            {
                Node node = root.getChildNodes().item(i);
                PSSM pssm = PSSM.readFromXMLNode(node);
                pssmList.put(node.getAttributes().getNamedItem("name").getNodeValue(), pssm);
            }
        }
        catch (DOMException e)
        {
            e.printStackTrace();
        }
        catch (ParserConfigurationException e)
        {
            System.err.println("You need to install JAXP for DOM");
        }
        catch (SAXException e)
        {
            e.printStackTrace();  //TODO
        }
    }
}
