package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.util.db.PredictionDB;
import ca.ualberta.proteus.util.db.PredictionDBFactory;
import ca.ualberta.proteus.util.db.DBType;
import ca.ualberta.proteus.program.transsec.test.AveragingPredictor;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import org.biojava.bio.BioException;
import org.biojava.bio.Annotation;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.db.IllegalIDException;

import java.io.*;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Date;

/**
 * Created by IntelliJ IDEA.
 * User: Smontgomerie
 * Date: Jun 8, 2005
 * Time: 11:17:29 AM
 * To change this template use File | Settings | File Templates.
 */
public class ExternalEvaluator
{
    private String databaseName;
    public boolean keepProteus;
    private static final double MAX_EVALUE = 1e-40;

    public ExternalEvaluator(String databaseName) {
        this.databaseName = databaseName;
    }

    public void run() throws IOException
    {
        run(-1, -1);
    }

    public void run(int start, int end) throws IOException
    {
        StructureDB db = new StructureDB(databaseName, StructureDB.MODE.READ);

        if ( start < 0 )
            start = 0;
        if ( end < 0 )
            end = db.size();

        List<StructureAnnotatedSequence> list = new ArrayList<StructureAnnotatedSequence>();
        for ( int i = start; i< end; i++ ) {
            try
            {
                StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) db.getSequence(i);
                list.add(sequence);
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

        }

        run(list, MAX_EVALUE);
    }

    public void run(File listFile) throws IOException
    {
        run(listFile, -1, -1);
    }

    public void run(File listFile, int start, int end) throws IOException
    {
        List<StructureAnnotatedSequence> list = new ArrayList<StructureAnnotatedSequence>();
        StructureDB db = new StructureDB(databaseName, StructureDB.MODE.READ);

        if ( start < 0 )
            start = 0;
        if ( end < 0 )
            end = db.size();

        try
        {
            BufferedReader reader = new BufferedReader(new FileReader(listFile));

            String line;
            int i=0;
            while((line = reader.readLine()) !=null)
            {
                boolean add = false;
                if ( i >= start && i <= end )
                    add = true;

                i++;

                if ( add )
                {
                    try
                    {
                        list.add((StructureAnnotatedSequence) db.getSequence(line));
                    }
                    catch (IllegalIDException e)
                    {
                        System.out.println( line + " does not exist");
                    }
                }
            }

        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        run(list, MAX_EVALUE);
    }

    public void run(List<StructureAnnotatedSequence> sequenceList, double cutoff)
    {
        RuntimeCatalog proteusLogger = new RuntimeCatalog();
        MainPredictor mainPredictor = new MainPredictor(proteusLogger);
        mainPredictor.useStoredNN = true;
        mainPredictor.setEvaluating(true);

        PredictionDB predictionDB = PredictionDBFactory.getRunTimeDB( true );

        Score finalScore = new Score();
        Score jnet = new Score();
        Score psipred = new Score();
        Score proteus = new Score();
        Score transsec = new Score();
        Score homologScore = new Score();
        int homologsFound = 0;


        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);

        PredictionData[] scoreList = new PredictionData[sequenceList.size()];

        System.out.println("Running on " + sequenceList.size() + " sequences");

        int numSequences = 0;
        for( int i =0; i<sequenceList.size(); i++)
        {
            StructureAnnotatedSequence sequence = sequenceList.get(i);
            try
            {

                if ( sequence.length() < 10 || sequence.length() > 2000 )
                {
                    System.out.println("Skipping " + sequence.getName());
                    continue;
                }

                System.out.println("Record: " + i);
                System.out.println(sequence.getName());

                Date date = new Date();

                ScoreAnnotatedSequence finalSequence = mainPredictor.predict(sequence);
                ScoreAnnotatedSequence jnetSequence = predictionDB.get(sequence, "JNET");
                ScoreAnnotatedSequence psipredSequence = predictionDB.get(sequence, "PSIPRED");
                ScoreAnnotatedSequence proteusSequence = predictionDB.get(sequence, "PROTEUS");
                ScoreAnnotatedSequence transsecSequence = predictionDB.get(sequence, "TRANSSEC");
                //List<ScoreAnnotatedSequence> homologSequences =
                //        mainPredictor.searchForHomolog(sequence, MainPredictor.DEFAULT_EVALUE, 1e-30, false );

                /*if ( homologSequences.size() != 0 ) {
                    System.out.println("Homolog found : " + homologSequences.get(0).toString());
                    homologSequence = mainPredictor.overlayHomolog(finalSequence, homologSequences);
                }
                */

                Score score;
                scoreList[i] = new PredictionData();
                scoreList[i].sequence = sequence;
                score = getScore("PSIPRED", sequence, psipredSequence, psipred, format);
                scoreList[i].psipred = score;
                score = getScore("JNET", sequence, jnetSequence, jnet, format);
                scoreList[i].jnet = score;
                score = getScore("PROTEUS", sequence, proteusSequence, proteus, format);
                scoreList[i].proteus = score;
                if ( transsecSequence != null ) {
                    score = getScore("TRANSSEC", sequence, transsecSequence, transsec, format);
                    scoreList[i].transsec = score;
                }
                score = getScore("FINAL", sequence, finalSequence, finalScore, format);
                scoreList[i].finalScore = score;

                HomologCombiner homologCombiner = HomologCombiner.getInstance();
                homologCombiner.setLogger(proteusLogger);

                ScoreAnnotatedSequence homologSequence = homologCombiner.searchAndOverlayHomologs(finalSequence, MainPredictor.DEFAULT_EVALUE, cutoff, null,null);

                long time = new Date().getTime() - date.getTime();
                System.out.println("Sequence length: " + sequence.length());
                System.out.println("Time taken: " + time/1000 + "s");

                if ( homologSequence != null ) {
                    score = getScore("HOMOLOG", sequence, homologSequence, homologScore, format);
                    Annotation annotation = homologSequence.getAnnotation();
                    System.out.printf( "Homolog: %s Evalue: %e\n%s\n", homologSequence.getName(),
                                        (Double) annotation.getProperty("e-value"),
                                        homologSequence);
                    scoreList[i].homologScore = score;
                    if ( annotation.containsProperty("homolog"))
                    {
                        scoreList[i].homologID = (String) annotation.getProperty("homolog");
                    }
                    scoreList[i].homologEvalue = (Double) annotation.getProperty("e-value");
                    homologsFound++;
                }

                numSequences++;

                if ( i % 10 == 0 )
                    outputTable(new PrintStream("PDBAnalysis_intermediate.tab"), scoreList);
            }
            catch( Throwable t )
            {
                t.printStackTrace();
            }
        }

        System.out.println("JNET:\t"
                        + "Q3:" + format.format(jnet.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(jnet.sov / (double) numSequences) );

        System.out.println("PSIPRED:\t"
                        + "Q3: " + format.format(psipred.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(psipred.sov / (double) numSequences) );

        System.out.println("PROTEUS:\t"
                        + "Q3: " + format.format(proteus.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(proteus.sov / (double) numSequences) );

        System.out.println("TRANSSEC:\t"
                                + "Q3: " + format.format(transsec.q3 / (double) numSequences)
                                + "\tSOV: " + format.format(transsec.sov / (double) numSequences) );

        System.out.println("FINAL:\t"
                           + "Q3: " + format.format(finalScore.q3 / (double) numSequences)
                           + "\tSOV: " + format.format(finalScore.sov / (double) numSequences) );

        System.out.println("HOMOLOGS:\t"
                                   + "Q3: " + format.format(homologScore.q3 / (double) homologsFound)
                                   + "\tSOV: " + format.format(homologScore.sov / (double) homologsFound ) );
        System.out.println( homologsFound + " sequences had homologs");
        System.out.println( String.format("%d out of %d passed (%d %%)", numSequences, sequenceList.size(), (int) (numSequences * 100 / Math.max(sequenceList.size(), 1) )) );


        try
        {
            outputTable(new PrintStream("PDBAnalysis.tab"), scoreList);
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }


    private void outputTable(PrintStream out, PredictionData[] scoreList)
    {
       /* out.println("\tPSIPRED\t\t" +
                        "JNET\t\t" + // jnet
                        "PROTEUS\t\t" + // proteus
                        "TRANSSEC\t\t" + // transsec
                        "FINAL\t\t" + // final
                        "HOMOLOG"); // homolog
        out.println("\tQ3\tSOV\t" + // psipred
                "Q3\tSOV\t" + // jnet
                "Q3\tSOV\t" + // proteus
                "Q3\tSOV\t" + // transec
                "Q3\tSOV\t" + // final
                "Q3\tSOV\t"); // homolog
                */
        for (PredictionData data : scoreList)
        {
            if (data == null)
                continue;

            out.printf("%s\t", data.sequence.getName());
            if (data.psipred == null)
                out.print("\t\t");
            else
                out.printf("%1.3f\t%1.3f\t", data.psipred.q3, data.psipred.sov);
            if (data.jnet == null)
                out.print("\t\t");
            else
                out.printf("%1.3f\t%1.3f\t", data.jnet.q3, data.jnet.sov);
            if (data.proteus == null)
                out.print("\t\t");
            else
                out.printf("%1.3f\t%1.3f\t", data.proteus.q3, data.proteus.sov);
            if (data.transsec == null)
                out.print("\t\t");
            else
                out.printf("%1.3f\t%1.3f\t", data.transsec.q3, data.transsec.sov);
            if (data.finalScore == null)
                out.print("\t\t");
            else
                out.printf("%1.3f\t%1.3f\t", data.finalScore.q3, data.finalScore.sov);

            if (data.homologScore != null)
            {
                out.printf("%1.3f\t%1.3f\t",
                        data.homologScore.q3, data.homologScore.sov);
            }

            if (data.homologID != null)
            {
                out.printf("%s\t%e\t",
                        data.homologID, data.homologEvalue);
            }

            out.println();
        }

        out.flush();
    }

    private Score getScore(String title, StructureAnnotatedSequence sequence, StructureAnnotatedSequence predictedSequence, Score score, NumberFormat format)
    {
        assert( predictedSequence != null ) : "Passed null sequence";
        assert( predictedSequence.getStructureAnnotation() != null ) : "Structure annotation on predicted sequence is null!";

        double q3;
        double sov;
        q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());
        sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());
        score.q3 += q3;
        score.sov += sov;

        System.out.println(title + "\t"
                + "Q3: " + format.format(q3)
                + "\tSOV: " + format.format(sov));
        System.out.println(predictedSequence);

        return new Score(q3, sov);
    }

    public void runAveraging(List<StructureAnnotatedSequence> sequenceList, double cutoff)
    {
        AveragingPredictor averagingPredictor = new AveragingPredictor(new RuntimeCatalog());
        averagingPredictor.setEvaluating(true);

        Score finalScore = new Score();
        Score jnet = new Score();
        Score psipred = new Score();
        Score proteus = new Score();
        Score transsec = new Score();
        Score homologScore = new Score();
        int homologsFound = 0;

        PredictionDB predictionDB = PredictionDBFactory.getDB(DBType.DB4O);
        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);

        PredictionData[] scoreList = new PredictionData[sequenceList.size()];

        System.out.println("Running on " + sequenceList.size() + " sequences");

        int numSequences = 0;
        for( int i =0; i<sequenceList.size(); i++)
        {
            StructureAnnotatedSequence sequence = sequenceList.get(i);
            try
            {

                if ( sequence.length() < 10 || sequence.length() > 2000 )
                {
                    System.out.println("Skipping " + sequence.getName());
                    continue;
                }

                System.out.println("Record: " + i);
                System.out.println(sequence.getName());

                ScoreAnnotatedSequence finalSequence = averagingPredictor.predict(sequence);
                ScoreAnnotatedSequence jnetSequence = predictionDB.get(sequence, "JNET");
                ScoreAnnotatedSequence psipredSequence = predictionDB.get(sequence, "PSIPRED");
                ScoreAnnotatedSequence proteusSequence = predictionDB.get(sequence, "PROTEUS");
                ScoreAnnotatedSequence transsecSequence = predictionDB.get(sequence, "TRANSSEC");
                //List<ScoreAnnotatedSequence> homologSequences =
                //        averagingPredictor.searchForHomolog(sequence, MainPredictor.DEFAULT_EVALUE, 1e-30, false );

                /*if ( homologSequences.size() != 0 ) {
                    System.out.println("Homolog found : " + homologSequences.get(0).toString());
                    homologSequence = averagingPredictor.overlayHomolog(finalSequence, homologSequences);
                }
                */

                Score score;
                scoreList[i] = new PredictionData();
                scoreList[i].sequence = sequence;
                score = getScore("PSIPRED", sequence, psipredSequence, psipred, format);
                scoreList[i].psipred = score;
                score = getScore("JNET", sequence, jnetSequence, jnet, format);
                scoreList[i].jnet = score;
                score = getScore("PROTEUS", sequence, proteusSequence, proteus, format);
                scoreList[i].proteus = score;
                if ( transsecSequence != null ) {
                    score = getScore("TRANSSEC", sequence, transsecSequence, transsec, format);
                    scoreList[i].transsec = score;
                }
                score = getScore("FINAL", sequence, finalSequence, finalScore, format);
                scoreList[i].finalScore = score;

                ScoreAnnotatedSequence homologSequence = averagingPredictor.searchAndOverlayHomolog(finalSequence, MainPredictor.DEFAULT_EVALUE, cutoff, false);

                if ( homologSequence != null ) {
                    score = getScore("HOMOLOG", sequence, homologSequence, homologScore, format);
                    Annotation annotation = homologSequence.getAnnotation();
                    System.out.printf( "Homolog: %s Evalue: %e\n%s\n", homologSequence.getName(),
                                        (Double) annotation.getProperty("e-value"),
                                        homologSequence);
                    scoreList[i].homologScore = score;
                    if ( annotation.containsProperty("homolog"))
                    {
                        scoreList[i].homologID = (String) annotation.getProperty("homolog");
                    }
                    scoreList[i].homologEvalue = (Double) annotation.getProperty("e-value");
                    homologsFound++;
                }

                numSequences++;

                if ( i % 10 == 0 )
                    outputTable(new PrintStream("PDBAnalysis_intermediate.tab"), scoreList);
            }
            catch( Throwable t )
            {
                t.printStackTrace();
            }
        }

        /*
        System.out.println("JNET:\t"
                        + "Q3:" + format.format(jnet.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(jnet.sov / (double) numSequences) );

        System.out.println("PSIPRED:\t"
                        + "Q3: " + format.format(psipred.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(psipred.sov / (double) numSequences) );

        System.out.println("PROTEUS:\t"
                        + "Q3: " + format.format(proteus.q3 / (double) numSequences)
                        + "\tSOV: " + format.format(proteus.sov / (double) numSequences) );

        System.out.println("TRANSSEC:\t"
                                + "Q3: " + format.format(transsec.q3 / (double) numSequences)
                                + "\tSOV: " + format.format(transsec.sov / (double) numSequences) );

        System.out.println("FINAL:\t"
                           + "Q3: " + format.format(finalScore.q3 / (double) numSequences)
                           + "\tSOV: " + format.format(finalScore.sov / (double) numSequences) );

        System.out.println("HOMOLOGS:\t"
                                   + "Q3: " + format.format(homologScore.q3 / (double) homologsFound)
                                   + "\tSOV: " + format.format(homologScore.sov / (double) homologsFound ) );
        System.out.println( homologsFound + " sequences had homologs");

*/

        try
        {
            outputTable(new PrintStream("PDBAnalysis.tab"), scoreList);
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    public void runAveraging(File listFile, int start, int end) throws IOException
    {
        List<StructureAnnotatedSequence> list = new ArrayList<StructureAnnotatedSequence>();
        StructureDB db = new StructureDB(databaseName, StructureDB.MODE.READ);

        if ( start < 0 )
            start = 0;
        if ( end < 0 )
            end = db.size();

        try
        {
            BufferedReader reader = new BufferedReader(new FileReader(listFile));

            String line;
            int i=0;
            while((line = reader.readLine()) !=null)
            {
                boolean add = false;
                if ( i >= start && i <= end )
                    add = true;

                i++;

                if ( add )
                {
                    try
                    {
                        list.add((StructureAnnotatedSequence) db.getSequence(line));
                    }
                    catch (IllegalIDException e)
                    {
                        System.out.println( line + " does not exist");
                    }
                }
            }

        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        runAveraging(list, MAX_EVALUE);
    }



    class PredictionData {
        Sequence sequence;
        Score finalScore;
        Score jnet;
        Score psipred;
        Score proteus;
        Score homologScore;
        public Score transsec;
        String homologID;
        double homologEvalue;
    }
}
