package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.TrainingSet;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;

import java.io.IOException;
import java.io.InputStream;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 19, 2005
 * Time: 5:02:00 PM
 * To change this template use File | Settings | File Templates.
 */
public class NeuralFilterTrainingSet extends TrainingSet {

    private final static int WINDOW_SIZE = 19;
    private final static int INPUT_LAYER = ProteusTools.NUM_STRUCTURES * WINDOW_SIZE;
    public static final int MIDDLERES = (int) WINDOW_SIZE / 2;

    private final static int NUM_OUTPUTS = ProteusTools.NUM_STRUCTURES;

    public void addInstance(ScoreAnnotatedSequence predictedStructure, PredictionScore[] scores, SymbolList realStructure) {

        assert( scores.length == predictedStructure.length() );

        for (int resindex = 0; resindex < predictedStructure.length(); resindex++)
        {
            addInputInstance(predictedStructure.getStructureAnnotation(), scores, resindex);

            addOutputInstance(realStructure, resindex);
        }
    }

    public void addInstance(StructureAnnotatedSequence sequence) {
        new IllegalStateException("Can't call this method");
    }

    public void addInstance(Sequence sequence) {
        throw new IllegalStateException("Can't call this method");
    }

    public void addInstance(SymbolList symbolList, PredictionScore[] scores) {
        assert( scores.length == symbolList.length() );

        for (int resindex = 0; resindex < symbolList.length(); resindex++)
        {
            addInputInstance(symbolList, scores, resindex);

            addOutputInstance(null, resindex);
        }
    }

    public void readFromStream(InputStream aInputStream) throws IOException {
        throw new IllegalStateException("Not implemented");
    }

    private void addOutputInstance(SymbolList predictedStructure, int resindex)
    {
        double[] outputs = new double[NUM_OUTPUTS];

        /* Now add the output
         If predictedStructure is null, then we will simply add zeroes */
        if (predictedStructure != null && predictedStructure.length() > 0)
        {
            Symbol structureSymbol = predictedStructure.symbolAt(resindex + 1);

            int structAsInt = ProteusTools.convertStructureToInt(structureSymbol);

            outputs[structAsInt] = 1;
        }

        outputArray.add(outputs);
    }

    private void addInputInstance(SymbolList predictedStructure, PredictionScore[] scores, int resindex)
    {
        double[] inputs = new double[INPUT_LAYER];

        for (int i = resindex - MIDDLERES; i < resindex - MIDDLERES + WINDOW_SIZE ; i++)
        {
            int index = i + MIDDLERES - resindex;

            PredictionScore score;
            if ((i >= 0) && (i < predictedStructure.length()))
            {
                score = scores[i];
                score.normalize();

                inputs[ index * ProteusTools.NUM_STRUCTURES + ProteusTools.HELIX] = score.helix;
                inputs[ index * ProteusTools.NUM_STRUCTURES + ProteusTools.STRAND] = score.strand;
                inputs[ index * ProteusTools.NUM_STRUCTURES + ProteusTools.COIL] = score.coil;
            }
            else
            {

                // off the edge

                score = PredictionScore.DEFAULT_SCORE;

                inputs[ index * ProteusTools.NUM_STRUCTURES + ProteusTools.NOTHING] = 1;
            }

            //
            //PredictionScore score = scores[index];


        }

        inputArray.add(inputs);
    }
}

