package ca.ualberta.proteus.program.transsec;

import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.proteomics.StructureTools;

import java.io.Serializable;

/**
 * A class to represent a prediction score of one annotation of one residue.  In a neural net's
 * classification, each classification receives a score and usually we take the highest score
 * (winner takes all approach).  Later, for filtering, we need those scores, this class holds them, and
 * provides utilities for handling them.
 */
public class PredictionScore implements Serializable {
    double coil;
    double helix;
    double strand;
    public static final PredictionScore DEFAULT_SCORE = new PredictionScore(0,0,0);
    public double gap;

    static transient StructureTools structureTools = new StructureTools();

    public double getMax()
    {
        return Math.max( Math.max(coil, helix), strand );
    }

    public PredictionScore() {
        this.coil = 0;
        this.helix = 0;
        this.strand = 0;
        this.gap = 0;
    }

    public PredictionScore(double coil, double helix, double strand, double gap) {
        this.coil = coil;
        this.helix = helix;
        this.strand = strand;
        this.gap = gap;
    }

    public PredictionScore(double coil, double helix, double strand)
    {
        this(coil, helix, strand, 0);
    }

    public PredictionScore(Symbol symbol, double score)
    {
        assert score < 1 : "Score must be lower than 1";

        if ( symbol == structureTools.getC())
        {
            coil = score;
        }
        else if ( symbol == structureTools.getE())
        {
            strand = score;
        }
        else if ( symbol == structureTools.getH())
        {
            helix = score;
        }
        else
        {
            // default
            coil = score;
        }
    }

    public void normalize() {
        double total = coil + helix + strand;
        coil = coil / total;
        helix = helix / total;
        strand = strand / total;
    }


    public PredictionScore clone()
    {
        return new PredictionScore(coil, helix, strand, gap); 
    }
}
