package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import org.biojava.bio.seq.Sequence;
import org.joone.engine.Monitor;
import org.joone.engine.NeuralNetEvent;
import org.joone.engine.NeuralNetListener;
import org.joone.io.MemoryInputSynapse;
import org.joone.io.StreamInputSynapse;
import org.joone.net.NeuralNet;
import org.joone.util.LearningSwitch;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.GZIPOutputStream;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 13, 2005
 * Time: 10:54:47 PM
 * To change this template use File | Settings | File Templates.
 */
public abstract class Predictor implements Serializable, NeuralNetListener
{
    protected static NeuralNet nnet = null;

    protected List<PredictionFilter> filters = new ArrayList<PredictionFilter>();
    private long lastCycleTime = System.currentTimeMillis();

    public abstract void train(List sequences);

    /**
     *  For predicting the sequence with the instance's neural net
     * @param sequence
     * @return a prediction
     */
    public ScoreAnnotatedSequence predict(Sequence sequence) {
        return predict(sequence, null);
    }


    /**
     * For predicting the sequence with a copy of a neural net that is provided in a parameter.  Does filtering by default.
     * @param sequence
     * @param localNNet
     * @return a prediction
     */
    public ScoreAnnotatedSequence predict(Sequence sequence, NeuralNet localNNet)  {
        return predict(sequence, localNNet, true);
    }

    /**
     * Predict with a sequence, instance neural net, and an option to filter or not
     * @param sequence
     * @param localNN
     * @param filter
     * @return a prediction
     */
    public abstract ScoreAnnotatedSequence predict(Sequence sequence, NeuralNet localNN, boolean filter);

    public void cicleTerminated(NeuralNetEvent e)
    {
        Monitor mon = (Monitor) e.getSource();

        /* Calculate the time for the current cycle */
        long cycleTime = System.currentTimeMillis() - lastCycleTime;
        lastCycleTime = System.currentTimeMillis();
        System.out.println( "**** Cycle " + mon.getCurrentCicle() + " terminated ****  Time for cycle (s) = " + cycleTime / 1000 );

         // Cleans the old listeners
    /*    localNN.removeAllListeners();

        // Set the parameters on the validation
        NeuralNetValidator nnv = new NeuralNetValidator(localNN);
        nnv.addValidationListener(this);
        nnv.start();
        */

        if (false)
        {

            mon.Stop();
        }
       
        System.gc();

    }

    public void addFilter(PredictionFilter filter)
    {
        filters.add(filter);
    }

    public void netStarted(NeuralNetEvent e)
    {
        System.out.println("Started training... Num CPU's: " + Runtime.getRuntime().availableProcessors());
    }

    public void netStopped(NeuralNetEvent e)
    {
        System.out.println("Training stopped.");
    }

    public void errorChanged(NeuralNetEvent e)
    {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void netStoppedError(NeuralNetEvent e, String error)
    {
        System.out.println("Training stopped with errors: " + error);
    }

    /**
     * Creates a FileInputSynapse
     */
    protected MemoryInputSynapse createInput(double[][] inputArray, int firstRow, int firstCol, int lastCol)
    {
        MemoryInputSynapse input = new MemoryInputSynapse();
        input.setInputArray( inputArray );
        input.setFirstRow(firstRow);
        if (firstCol != lastCol)
            input.setAdvancedColumnSelector(firstCol + "-" + lastCol);
        else
            input.setAdvancedColumnSelector(Integer.toString(firstCol));

        return input;
    }

    protected LearningSwitch createSwitch(StreamInputSynapse IT, StreamInputSynapse IV)
    {
        LearningSwitch lsw = new LearningSwitch();
        lsw.addTrainingSet(IT);
        lsw.addValidationSet(IV);
        return lsw;
    }

    protected NeuralNet restoreNeuralNet(String fileName)
    {
        /* We need just to provide the serialized NN file name */
        System.out.print("Loading neural net (file = " + fileName + ")...");
        GzipNeuralNetLoader loader = new GzipNeuralNetLoader(fileName);
        NeuralNet myNet = loader.getNeuralNet();
        System.out.println("Done");
        return myNet;
    }

    protected void saveNeuralNet(String fileName)
    {
        try
        {
            GZIPOutputStream stream = new GZIPOutputStream( new FileOutputStream(fileName) );
            ObjectOutputStream out = new ObjectOutputStream(stream);
            out.writeObject(nnet);
            out.close();
        }
        catch (Exception excp)
        {
            excp.printStackTrace();
        }
    }
}


