package ca.ualberta.proteus.program.transsec.ReducedPredictor;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.TrainingSet;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;

import java.io.IOException;
import java.io.InputStream;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 8, 2005
 * Time: 9:32:16 PM
 * To change this template use File | Settings | File Templates.
 */

public class SingleSequenceReducedTrainingSet extends TrainingSet
{

    public static final int WINDOW_SIZE = 19;
    public static final int MIDDLERES = (int) WINDOW_SIZE / 2;
    private final static int NUM_INPUTS_PER_RESIDUE = 5; // 21 residues in 5-bit encoding
    private final static int INPUT_LAYER = NUM_INPUTS_PER_RESIDUE * WINDOW_SIZE;
    private final static int OUTPUT_LAYER = ProteusTools.NUM_STRUCTURES;

    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     * @param sequence - The sequence to add
     */
    public void addInstance(StructureAnnotatedSequence sequence)
    {
        for (int resindex = 0; resindex < sequence.length(); resindex++)
        {
            addInputInstance(sequence, resindex);

            addOutputInstance(sequence.getStructureAnnotation(), resindex);
        }
    }

    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     * @param sequence - The sequence to add
     */
    public void addInstance(Sequence sequence)
    {
        for (int resindex = 0; resindex < sequence.length(); resindex++)
        {
            addInputInstance(sequence, resindex);

            addOutputInstance(null, resindex);
        }
    }

    private void addOutputInstance(SymbolList predictedStructure, int resindex)
    {
        double[] outputs = new double[OUTPUT_LAYER];

        /* Now add the output
         If predictedStructure is null, then we will simply add zeroes */
        if (predictedStructure != null && predictedStructure.length() > 0)
        {
            Symbol structureSymbol = predictedStructure.symbolAt(resindex + 1);

            int structAsInt = ProteusTools.convertStructureToInt(structureSymbol);

            outputs[structAsInt] = 1;
        }

        outputArray.add(outputs);
    }

    private void addInputInstance(Sequence sequence, int resindex)
    {
        double[] inputs = new double[INPUT_LAYER];

        for (int i = resindex - MIDDLERES; i < resindex - MIDDLERES + WINDOW_SIZE; i++)
        {
            Symbol residue;
            if ((i >= 0) && (i < sequence.length()))
            {
                residue = sequence.symbolAt(i + 1);
            }
            else
            {
                residue = ProteusTools.x();
            }

            /* Now we know what the residue is, add this knowledge to the training set */

            int resAsInt = 0;
            try
            {
                resAsInt = ProteusTools.convertToInt(residue);
            }
            catch (BioException e)
            {
                //System.out.println(e + " " + residue);
                try
                {
                    resAsInt = ProteusTools.convertToInt(ProteusTools.x());
                }
                catch (BioException e1)
                {
              //      e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }


            // This is where the work happens to convert to a binary representation of the training set

            int offset = (i + MIDDLERES - resindex) * NUM_INPUTS_PER_RESIDUE ;

            if ( (resAsInt & 0x10) == 0x10) // 10000 in binary
                inputs[ offset + 0] = 1;
            if ( (resAsInt & 0x8) == 0x8) // 1000
                inputs[offset + 1] = 1;
            if ( (resAsInt & 0x4) == 0x4) // 100
                inputs[offset + 2] = 1;
            if ( (resAsInt & 0x2) == 0x2) // 10
                inputs[offset + 3] = 1;
            if ( (resAsInt & 0x1) == 0x1) // 1
                inputs[offset + 4] = 1;
        }

        inputArray.add(inputs);
    }

    public void readFromStream(InputStream aInputStream) throws IOException
    {
        throw new IllegalStateException("Not implemented.");
    }
}
