package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import org.biojava.bio.BioException;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Symbol;
import org.joone.engine.*;
import org.joone.engine.learning.TeachingSynapse;
import org.joone.io.MemoryInputSynapse;
import org.joone.net.NeuralNet;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 *
 * @author Scott Montgomerie
 *         Date: Mar 8, 2005
 *         Time: 9:17:08 PM
 *         To change this template use File | Settings | File Templates.
 */
public class SingleSequencePredictor2 extends Predictor
{
    //private final static int INPUT_LAYER = ProteusTools.NUM_RESIDUES * MainPredictor.N1_WINDOWSIZE;
    private static int LAYER2 = 128;
    private static int LAYER3 = 20;
    // private final static int OUTPUT_LAYER = ProteusTools.NUM_STRUCTURES;
    private final static String TRAINING_SET_FILE = "trainingSet.ser";

    private static int numToTrainOn = 100;
    private static int numToPredict = 10;
    private static boolean verbose = false;
    private static String outputFile = "testNN.model";
    public boolean useStoredNN = false; // this should only be true when using as a batch predictor
    private static NeuralNet storedNN = null;
    private double percentageSplit = 0.66;
    private List validationSequences = null;
    private final int numToValidate = 50;
    private double bestQ3ScoreSoFar = 0;
    private double bestValError = Double.MAX_VALUE;
    private double bestFilteredQ3ScoreSoFar = 0;


    public void train(List sequences)
    {
        TrainingSet trainingSet = new SingleSequenceTrainingSet();
        for (Iterator it = sequences.iterator(); it.hasNext();)
        {
            trainingSet.addInstance((StructureAnnotatedSequence) it.next());
        }

        try
        {
            OutputStream file = new FileOutputStream(SingleSequencePredictor2.TRAINING_SET_FILE);
            trainingSet.writeToStream(file);
        }
        catch (IOException ex)
        {
            System.out.println("Cannot perform output.");
        }

        /* The Layers */
        LinearLayer input = new LinearLayer();
        input.setRows( trainingSet.getNumInputs() );

        SigmoidLayer hidden = new SigmoidLayer();
        hidden.setRows(LAYER2);

        SigmoidLayer output = new SigmoidLayer();
        output.setRows( trainingSet.getNumOutputs() );

        /* The Synapses */
        FullSynapse synapse_IH = new FullSynapse(); /* Input -> Hidden conn. */
        input.addOutputSynapse(synapse_IH);
        hidden.addInputSynapse(synapse_IH);

        FullSynapse synapse_HO = new FullSynapse(); /* Hidden -> Output conn. */
        hidden.addOutputSynapse(synapse_HO);
        output.addInputSynapse(synapse_HO);

        int numInstances = trainingSet.numInstances();

        /* The I/O components */
        MemoryInputSynapse inputStream = new MemoryInputSynapse();
        inputStream.setInputArray(trainingSet.getInputArray());
        inputStream.setAdvancedColumnSelector("1-" + trainingSet.getNumInputs());
        input.addInputSynapse(inputStream);

        /* Creates all the required input data sets:
         *  IDdata  = input training data set
         *  IV Data = input validation data set
         *  DTData = desired training data set
         *  DVData = desired validation data set
         */
        int split = (int) (numInstances * percentageSplit);
        /*  MemoryInputSynapse ITData = this.createInput( trainingSet.getInputArray(), 1, 1, trainingSet.getNumInputs() );
          MemoryInputSynapse IVData = this.createInput( trainingSet.getInputArray(), split + 1, 1, trainingSet.getNumInputs() );

          MemoryInputSynapse DTData = this.createInput( trainingSet.getInputArray(), 1, trainingSet.getNumInputs() + 1, trainingSet.getNumInputs() + trainingSet.getNumOutputs() );
          MemoryInputSynapse DVData = this.createInput( trainingSet.getInputArray(), split + 1, trainingSet.getNumInputs() + 1, trainingSet.getNumInputs() + trainingSet.getNumOutputs() );
         */


        /* Creates and attach the input learning switch */
        /*LearningSwitch Ilsw = this.createSwitch(ITData, IVData);
        input.addInputSynapse(Ilsw);*/

        /* Creates and attached the desired learning switch */

        /* The Trainer and its desired file */
        /*    LearningSwitch Dlsw = this.createSwitch(DTData, DVData);
            TeachingSynapse trainer = new TeachingSynapse();
            trainer.setDesired(Dlsw);
            output.addOutputSynapse(trainer);*/

        TeachingSynapse trainer = new TeachingSynapse();
        MemoryInputSynapse samples = new MemoryInputSynapse();
        samples.setInputArray(trainingSet.getInputArray());
        trainer.setDesired(samples);

        samples.setAdvancedColumnSelector(String.valueOf(trainingSet.getNumInputs() + 1) + "-" + String.valueOf(trainingSet.getNumInputs() + trainingSet.getNumOutputs()));
        output.addOutputSynapse(trainer);

        nnet = new NeuralNet();
        nnet.addLayer(input, NeuralNet.INPUT_LAYER);
        nnet.addLayer(hidden, NeuralNet.HIDDEN_LAYER);
        //nnet.addLayer(hidden2, NeuralNet.HIDDEN_LAYER);
        nnet.addLayer(output, NeuralNet.OUTPUT_LAYER);
        nnet.setTeacher(trainer);

        Monitor monitor = nnet.getMonitor();
        monitor.setLearningRate(0.8);   // 0.8
        monitor.setMomentum(0.3);         // 0.3
        monitor.setTrainingPatterns(split); /* # of rows in the input file */
        monitor.setValidationPatterns(numInstances - split);
        monitor.setTotCicles(500); /* How many times the net must be trained*/
        monitor.setLearning(true); /* The net must be trained */
        monitor.addNeuralNetListener(this);

        nnet.start();
        nnet.getMonitor().Go();

        System.out.println("Number of instances: " + numInstances);
        PredictionScore instanceDistribution = new PredictionScore();
        for (Iterator it = trainingSet.getOutputInstanceIterator(); it.hasNext();)
        {
            double[] outputInstance = (double[]) it.next();
            instanceDistribution.helix += outputInstance[ProteusTools.HELIX];
            instanceDistribution.coil += outputInstance[ProteusTools.COIL];
            instanceDistribution.gap += outputInstance[ProteusTools.GAP];
            instanceDistribution.strand += outputInstance[ProteusTools.STRAND];
        }
        System.out.println("Instance distribution: ");
        System.out.println("Helix = " + instanceDistribution.helix);
        System.out.println("Coil = " + instanceDistribution.coil);
        System.out.println("Strand = " + instanceDistribution.strand);
        System.out.println("Gap = " + instanceDistribution.gap);
    }

    /**
     * Default predict function
     *
     * @param sequence
     * @param localNNet
     * @param doFilter
     * @return predictionsouthp
     */
    public ScoreAnnotatedSequence predict(Sequence sequence, NeuralNet localNNet, boolean doFilter)
    {
        ScoreAnnotatedSequence result = null;
        try
        {
            result = new ScoreAnnotatedSequence(sequence);

            if (localNNet == null)
            {
                if (useStoredNN && SingleSequencePredictor2.storedNN != null)
                {
                    // we have a stored NN, use it
                    localNNet = SingleSequencePredictor2.storedNN;
                }
                else
                {
                    // we either have no NN, or don't want to use it
                    localNNet = restoreNeuralNet(SingleSequencePredictor2.outputFile);
                    SingleSequencePredictor2.storedNN = localNNet;
                }
            }

            if (localNNet != null)
            {
                /* We get the first layer of the net (the input layer),
                  then remove all the input synapses attached to it and attach a DirectSynapse */
                Layer input = localNNet.getInputLayer();
                input.removeAllInputs();
                DirectSynapse memInp = new DirectSynapse();
                input.addInputSynapse(memInp);

                /* We get the last layer of the net (the output layer),
                 then remove all the output synapses attached to it and attach a DirectSynapse */
                Layer output = localNNet.getOutputLayer();
                output.removeAllOutputs();
                DirectSynapse memOut = new DirectSynapse();
                output.addOutputSynapse(memOut);

                // begin the neural net's thread
                localNNet.getMonitor().setLearning(false);
                localNNet.start();

                // Create the set that contains the instances of this sequence
                TrainingSet set = new SingleSequenceTrainingSet();
                set.addInstance(sequence);

                Symbol[] symbolList = new Symbol[set.numInstances()];

                int index = 1;
                for (Iterator it = set.getInputInstanceIterator(); it.hasNext();)
                {

                    // Prepare the next input pattern
                    Pattern iPattern = new Pattern((double[]) it.next());
                    iPattern.setCount(1);

                    // Interrogate the net
                    memInp.fwdPut(iPattern);

                    // Read the output pattern and print out it
                    Pattern pattern = memOut.fwdGet();


                    if (SingleSequencePredictor2.verbose) System.out.print("Output #" + (index) + " = ");
                    double max = -1;
                    int maxIndex = -1;
                    final double[] outputArr = pattern.getArray();
                    NumberFormat formatter = DecimalFormat.getInstance();
                    formatter.setMaximumFractionDigits(3);

                    for (int j = 0; j < outputArr.length; j++)
                    {
                        if (outputArr[j] > max)
                        {
                            max = outputArr[j];
                            maxIndex = j;
                        }

                        if (SingleSequencePredictor2.verbose) System.out.print(" " + formatter.format(outputArr[j]));
                    }

                    // set the score of the chosen output
                    result.setScore(index, outputArr[ProteusTools.COIL], outputArr[ProteusTools.HELIX], outputArr[ProteusTools.STRAND]);

                    // get the symbol for this prediction
                    final Symbol symbol = ProteusTools.convertIntToStructure(maxIndex);
                    symbolList[index - 1] = symbol;

                    if (SingleSequencePredictor2.verbose) System.out.println(" Structure = " + symbol.getName());

                    index++;
                }

                SimpleSymbolList structure = new SimpleSymbolList(symbolList, symbolList.length, new StructureTools().getStructure());

                result.setStructureAnnotation(structure);

                /* Stop training - don't know why this is necessary */
                Pattern stop = new Pattern(new double[set.getInputInstanceSize()]);
                stop.setCount(-1);
                memInp.fwdPut(stop);
                memOut.fwdGet();

                if (SingleSequencePredictor2.verbose) System.out.println("Finished");
            }
            else
            {
                throw new IllegalStateException("Cannot read in neural net!");
            }

            if (doFilter)
            {
                // filter the result with the attached filters
                for (Iterator it = filters.iterator(); it.hasNext();)
                    result = ((PredictionFilter) it.next()).filter(result);
            }
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();
        }

        if ( !useStoredNN) {
            // Cleanu up our memory
            SingleSequencePredictor2.storedNN = null;
            localNNet = null;
        }

        return result;
    }

    public void cicleTerminated(NeuralNetEvent e)
    {
        try
        {// call inherited method, it will do validation on a set of sequences
            super.cicleTerminated(e);

            Monitor mon = (Monitor) e.getSource();
            long c = mon.getCurrentCicle();
            long cycle = mon.getTotCicles() - c;
            if ((cycle < 5)  // for first 25, every 5
                    || (cycle < 25 && cycle % 5 == 0)  // for first 25, every 5
                    || (cycle < 200 && cycle % 10 == 0)  // up to 200, every 10
                    || ((cycle % 200) == 0)) // after 200, every 200
            {
                System.out.println(c + " cycles remaining - Training Error = " + mon.getGlobalError());
            }

            /*SeqSeeDB db = SeqSeeDB.getInstance();

            if (validationSequences == null)
                validationSequences = db.getEvaluationSet(numToValidate, 0, 1000);
                */
            //StructureDB db = new StructureDB("pdbstruct.fa.small");

            if (validationSequences == null)
            {
/*
                validationSequences = new ArrayList();
                for (SequenceIterator it = db.sequenceIterator(); it.hasNext();)
                    try
                    {
                        validationSequences.add(it.nextSequence());
                    }
                    catch (BioException e1)
                    {
                        e1.printStackTrace();  //TODO
                    }
                    */
                SeqSeeDB db = SeqSeeDB.getInstance();
                validationSequences = db.getEvaluationSet2(numToValidate, db.size() - 1000, db.size(), 7);
            }


            Score score = new Score();
            Score filteredScore = new Score();

            NeuralNet localNN = nnet.cloneNet();
            localNN.getMonitor().setExporting(false);
            localNN.removeAllListeners();

            NumberFormat format = NumberFormat.getNumberInstance();
            format.setMaximumFractionDigits(4);


            int i = 0;
            for (Iterator it = validationSequences.iterator(); it.hasNext();)
            {
                // every second one, do it on 10
                if ( i  > 10  && cycle % 2 != 0)
                    break;

                try
                {
                    final StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.next();

                    StructureAnnotatedSequence prediction;
                    prediction = predict(sequence, localNN, false);

                    double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                    double sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                    if (i++ < 5)
                        System.out.println("Without filter: "
                                           + " Q3: " + format.format(q3)
                                           + " SOV: " + format.format(sov)
                                           + prediction);
                    score.q3 += q3;
                    score.sov += sov;

                    prediction = predict(sequence, localNN, true);
                    if (i < 5)
                        System.out.println("Filtered: "
                                           + " Q3: " + format.format(q3)
                                           + " SOV: " + format.format(sov)
                                           + prediction);
                    q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                    sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

                    filteredScore.q3 += q3;
                    filteredScore.sov += sov;
                }
                catch( Throwable t ) {
                    t.printStackTrace();
                }
            }

            System.out.print("Avg Q3: " + score.q3 / i);
            System.out.println("\t SOV: " + score.sov / i);

            System.out.print("Filtered Q3: " + filteredScore.q3 / i);
            System.out.println("\t SOV: " + filteredScore.sov / i);

            if (score.q3 > bestQ3ScoreSoFar)
            {
                // StringFormatter format = new Formatter();
                // format.
                System.out.println("Best Q3 score so far! (" + score.q3 / validationSequences.size() + ")");
                bestQ3ScoreSoFar = score.q3;

                final Date date = new Date();
                saveNeuralNet("bestNN.model" + date.getDate() + "-" + date.getMonth());
            }

            if (filteredScore.q3 > bestFilteredQ3ScoreSoFar)
            {
                // StringFormatter format = new Formatter();
                // format.
                System.out.println("Best Filtered Q3 score so far! (" + filteredScore.q3 / validationSequences.size() + ")");
                bestFilteredQ3ScoreSoFar = filteredScore.q3;

                final Date date = new Date();
                saveNeuralNet("bestFilteredNN.model" + date.getDate() + "-" + (date.getMonth() + 1));
            }

            if (cycle != 0 && (cycle % 25) == 0)
                saveNeuralNet(SingleSequencePredictor2.outputFile);
        }
        catch ( Throwable t ) {
            t.printStackTrace();
        }
    }

    public static void main(String[] args) throws IOException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();

        boolean train = false;
        boolean predict = false;

        for (int i = 0; i < args.length; i++)
        {
            if (args[i].equals("-train"))
            {
                train = true;
                if (i < args.length - 1)
                {
                    try
                    {
                        SingleSequencePredictor2.numToTrainOn = Integer.parseInt(args[++i]);
                    }
                    catch (NumberFormatException e)
                    {
                        SingleSequencePredictor2.printUsage();
                    }
                }
                else
                {
                    SingleSequencePredictor2.printUsage();
                }
            }
            else if (args[i].equals("-predict"))
            {
                predict = true;
                if (i < args.length - 1)
                    try
                    {
                        SingleSequencePredictor2.numToPredict = Integer.parseInt(args[++i]);
                    }
                    catch (NumberFormatException e)
                    {
                        SingleSequencePredictor2.printUsage();
                    }
            }
            else if (args[i].equals("-verbose") || args[i].equals("-v"))
            {
                SingleSequencePredictor2.verbose = true;
            }
            else if (args[i].equals("-output") || args[i].equals("-o"))
            {
                if (i < args.length - 1)
                {
                    String arg = args[++i];
                    if (arg.charAt(0) == '-')
                        SingleSequencePredictor2.printUsage();
                    else
                        SingleSequencePredictor2.outputFile = arg;
                }
            }
            else if (args[i].contains("-layer"))
            {
                train = true;
                if (i < args.length - 2)
                {
                    try
                    {
                        SingleSequencePredictor2.LAYER2 = Integer.parseInt(args[++i]);
                        SingleSequencePredictor2.LAYER3 = Integer.parseInt(args[++i]);
                    }
                    catch (NumberFormatException e)
                    {
                        SingleSequencePredictor2.printUsage();
                    }
                }
            }
            else if ( args[i].equals("-validate"))
            {
                numToPredict = Integer.parseInt(args[++i]);
            }
            else if (args[i].equals("-pdbEval"))
            {

                ScoreAnnotatedSequence prediction;
                Score score = new Score();
                Score filteredScore = new Score();
                NumberFormat format = NumberFormat.getNumberInstance();
                format.setMaximumFractionDigits(4);

                SingleSequencePredictor2 predictor = new SingleSequencePredictor2();
                predictor.useStoredNN = true;
                // final NeuralFilter NFfilter = new NeuralFilter();
                // NFfilter.useStoredNN = true;
                // predictor.addFilter(NFfilter);

                final PsiPass2Filter ps2Ffilter = new PsiPass2Filter();
                // NFfilter.useStoredNN = true;
                predictor.addFilter(ps2Ffilter);

                final SimpleFilter filter = new SimpleFilter();
                predictor.addFilter(filter);

                StructureDB newdb = new StructureDB(args[++i], StructureDB.MODE.READ);
                for (SequenceIterator it = newdb.sequenceIterator(); it.hasNext();)
                {
                    StructureAnnotatedSequence sequence = null;
                    try
                    {
                        sequence = (StructureAnnotatedSequence) it.nextSequence();

                        prediction = predictor.predict(sequence, null, false);

                        System.out.println("Sequence: \n" + sequence);

                        double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                        double sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

                        score.q3 += q3;
                        score.sov += sov;

                        System.out.println("Prediction: "
                                           + " Q3: " + format.format(q3)
                                           + " SOV: " + format.format(sov)
                                           + prediction);

                        prediction = predictor.predict(sequence);

                        q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                        sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

                        filteredScore.q3 += q3;
                        filteredScore.sov += sov;

                        System.out.println("\nPrediction ** filtered ** : "
                                           + " Q3: " + format.format(q3)
                                           + " SOV: " + format.format(sov)
                                           + prediction);
                    }
                    catch (BioException e)
                    {
                        e.printStackTrace();  //TODO
                    }
                    catch (Throwable t)
                    {
                        t.printStackTrace();  //TODO
                    }
                }

                System.out.print("Avg Q3: " + format.format(score.q3 / newdb.size()));
                System.out.println("\tSOV: " + format.format(score.sov / newdb.size()));

                System.out.print("Filtered Avg Q3: " + format.format(filteredScore.q3 / newdb.size()));
                System.out.println("\tFiltered SOV: " + format.format(filteredScore.sov / newdb.size()));
                return;
            }
            else
            {
                SingleSequencePredictor2.printUsage();
            }
        }

        if (train)
        {
            // train on this set of sequences
            Predictor predictor = new SingleSequencePredictor2();

            // Have to add the filters in this order - use the NN filter, then the reg exp filter
            final PsiPass2Filter ps2Ffilter = new PsiPass2Filter();
            predictor.addFilter(ps2Ffilter);

            final SimpleFilter filter = new SimpleFilter();
            predictor.addFilter(filter);

            List randomSequences;
            //randomSequences = db.getEvaluationSet2(numToTrainOn, 0, 8000, 23);
            randomSequences = db.getRandomSequences(SingleSequencePredictor2.numToTrainOn, 0, db.size() - 1000);

            // either this, or maybe just try training on a lot more sequences
          /*  StructureDB newdb = new StructureDB("pdbstruct.fa");
            for (SequenceIterator it = newdb.sequenceIterator(); it.hasNext();)
            {
                try
                {
                    if ( Math.random() > 0.6 )
                        randomSequences.add(it.nextSequence());
                    else
                        it.nextSequence();
                }
                catch (BioException e)
                {
                    e.printStackTrace();  //TODO
                }
            }
            */

            predictor.train(randomSequences);
        }
        else if (predict)
        {
            // predict this set of sequences
            SingleSequencePredictor2 predictor = new SingleSequencePredictor2();

            // final NeuralFilter NFfilter = new NeuralFilter();
            // NFfilter.useStoredNN = true;
            // predictor.addFilter(NFfilter);

            final PsiPass2Filter ps2Ffilter = new PsiPass2Filter();
            // NFfilter.useStoredNN = true;
            predictor.addFilter(ps2Ffilter);

            final SimpleFilter filter = new SimpleFilter();
            predictor.useStoredNN = true;
            predictor.addFilter(filter);

            List randomSequences = db.getEvaluationSet(SingleSequencePredictor2.numToPredict);

            ScoreAnnotatedSequence prediction;
            Score score = new Score();
            Score filteredScore = new Score();
            NumberFormat format = NumberFormat.getNumberInstance();
            format.setMaximumFractionDigits(4);

            for (Iterator it = randomSequences.iterator(); it.hasNext();)
            {
                final StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.next();
                prediction = predictor.predict(sequence, null, false);

                System.out.println("Sequence: \n" + sequence);

                double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                double sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

                score.q3 += q3;
                score.sov += sov;

                System.out.println("Prediction: "
                                   + " Q3: " + format.format(q3)
                                   + " SOV: " + format.format(sov)
                                   + prediction);

                prediction = predictor.predict(sequence);

                q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
                sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

                filteredScore.q3 += q3;
                filteredScore.sov += sov;

                System.out.println("\nPrediction ** filtered ** : "
                                   + " Q3: " + format.format(q3)
                                   + " SOV: " + format.format(sov)
                                   + prediction);
            }

            System.out.print("Avg Q3: " + format.format(score.q3 / randomSequences.size()));
            System.out.println("\tSOV: " + format.format(score.sov / randomSequences.size()));

            System.out.print("Filtered Avg Q3: " + format.format(filteredScore.q3 / randomSequences.size()));
            System.out.println("\tFiltered SOV: " + format.format(filteredScore.sov / randomSequences.size()));
        }
        else
            SingleSequencePredictor2.printUsage();
    }

    private static void printUsage()
    {
        System.out.println("Usage: <ProgName> (-train <# seq> | -predict <#seq>) [options]");
        System.out.println("Options: ");
        System.out.println("-train <# sequences>\t\tTrain on a number of sequences");
        System.out.println("-predict <# sequences>\t\tPredict a number of sequences");
        System.out.println("-v, -verbose");
        System.out.println("-o, -output <file>\t\t\tFile to output the training model to (if applicable)");
        System.exit(-1);
    }

    public void setNeuralNetModel(String s)
    {
        SingleSequencePredictor2.outputFile = s;
    }
}
