package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.TrainingSet;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 8, 2005
 * Time: 9:32:16 PM
 * To change this template use File | Settings | File Templates.
 */

public class SingleSequenceTrainingSet extends TrainingSet
{
    public static final int WINDOW_SIZE = 19; // was 19!!!
    public static final int MIDDLERES = (int) WINDOW_SIZE / 2;
    private static final int NUM_INPUTS = ProteusTools.NUM_RESIDUES * WINDOW_SIZE;
    private static final int NUM_OUTPUTS = ProteusTools.NUM_STRUCTURES;

    /**
     * Add instance to the training set with a sequence and structure
     * If the sequence is a StructureAnnotatedSequence, it will add the output instance as well
     * @param sequence - The sequence to add
     */
    public void addInstance(StructureAnnotatedSequence sequence)
    {
        for (int resindex = 0; resindex < sequence.length(); resindex++)
        {
            addInputInstance(sequence, resindex);

            addOutputInstance(sequence.getStructureAnnotation(), resindex);
        }
    }

    /**
     * Add instance to the training set with a sequence
     * @param sequence - The sequence to add
     */
    public void addInstance(Sequence sequence)
    {
        for (int resindex = 0; resindex < sequence.length(); resindex++)
        {
            addInputInstance(sequence, resindex);

            addOutputInstance(null, resindex);
        }
    }

    private void addOutputInstance(SymbolList predictedStructure, int resindex)
    {
        double[] outputs = new double[NUM_OUTPUTS];

        /* Now add the output
         If predictedStructure is null, then we will simply add zeroes */
        if (predictedStructure != null && predictedStructure.length() > 0)
        {
            Symbol structureSymbol = predictedStructure.symbolAt(resindex + 1);

            int structAsInt = ProteusTools.convertStructureToInt(structureSymbol);

            outputs[structAsInt] = 1;
        }

        outputArray.add(outputs);
    }

    private void addInputInstance(Sequence sequence, int resindex)
    {
        double[] inputs = new double[NUM_INPUTS];

        for (int i = resindex - MIDDLERES; i < resindex - MIDDLERES + WINDOW_SIZE; i++)
        {
            Symbol residue;
            if ((i >= 0) && (i < sequence.length()))
            {
                residue = sequence.symbolAt(i + 1);
            }
            else
            {
                residue = ProteusTools.x();
            }

            /* Now we know what the residue is, add this knowledge to the training set */

            int resAsInt = 0;
            try
            {
                resAsInt = ProteusTools.convertToInt(residue);
            }
            catch (BioException e)
            {
                //System.out.println(e + " " + residue);
                try
                {
                    resAsInt = ProteusTools.convertToInt(ProteusTools.x());
                }
                catch (BioException e1)
                {
              //      e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
            inputs[(i + MIDDLERES - resindex) * ProteusTools.NUM_RESIDUES + resAsInt] = 1;
        }

        inputArray.add(inputs);
    }

    public void readFromStream(InputStream aInputStream) throws IOException
    {
        BufferedReader buf = new BufferedReader(new InputStreamReader(aInputStream));

        inputArray.clear();

        String line;
        while ((line = buf.readLine()) != null)
        {
            String[] values = line.split(";");
            assert values.length == ProteusTools.NUM_RESIDUES * WINDOW_SIZE + ProteusTools.NUM_STRUCTURES;

            double[] inputs = new double[ProteusTools.NUM_RESIDUES * WINDOW_SIZE];
            double[] outputs = new double[ProteusTools.NUM_STRUCTURES];

            for (int i = 0; i < inputs.length; i++)
            {
                inputs[i] = Double.parseDouble(values[i]);
            }

            for (int i = 0; i < outputs.length; i++)
            {
                inputs[i] = Double.parseDouble(values[ProteusTools.NUM_RESIDUES * WINDOW_SIZE + i]);
            }
        }
    }
}
