package ca.ualberta.proteus.program.transsec;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.TrainingSet;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.Symbol;

import java.io.IOException;
import java.io.InputStream;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: May 15, 2005
 * Time: 11:37:30 PM
 * To change this template use File | Settings | File Templates.
 */
public class UnifiedTrainingSet extends TrainingSet
{
    private static int WINDOW_SIZE = 15;
    private static int MIDDLERES = (int) WINDOW_SIZE / 2;
    private static int NUM_METHODS = 3;
    public static int NUM_INPUTS = WINDOW_SIZE * ProteusTools.NUM_STRUCTURES * NUM_METHODS + WINDOW_SIZE * NUM_METHODS;
    public static final int NUM_OUTPUTS = ProteusTools.NUM_STRUCTURES;

    /**
     * @deprecated
     * @param sequence
     */ 
    public void addInstance(StructureAnnotatedSequence sequence)
    {
        //TODO
    }

    /**
     * @deprecated
     * @param sequence
     */
    public void addInstance(Sequence sequence)
    {
        //TODO
    }

    public void readFromStream(InputStream aInputStream) throws IOException
    {
        //TODO
    }

    public void addInstance(StructureAnnotatedSequence sequence, ScoreAnnotatedSequence jnet, ScoreAnnotatedSequence psipred, ScoreAnnotatedSequence transsec ) {

        assert( sequence.length() == jnet.length() );
        assert( sequence.length() == psipred.length() );
        assert( sequence.length() == transsec.length() );

        for (int resindex = 0; resindex < sequence.length(); resindex++)
        {
            addInputInstance(jnet, psipred, transsec, resindex);

            addOutputInstance(sequence, resindex);
        }
    }

    public void addInstance(Sequence sequence, ScoreAnnotatedSequence jnet, ScoreAnnotatedSequence psipred, ScoreAnnotatedSequence transsec ) {

            assert( sequence.length() == jnet.length() );
            assert( sequence.length() == psipred.length() );
            assert( sequence.length() == transsec.length() );

            for (int resindex = 0; resindex < sequence.length(); resindex++)
            {
                addInputInstance(jnet, psipred, transsec, resindex);

                addOutputInstance(null, resindex);
            }
        }


    private void addOutputInstance(StructureAnnotatedSequence predictedStructure, int resindex)
    {
        double[] outputs = new double[NUM_OUTPUTS];

        /* Now add the output
         If predictedStructure is null, then we will simply add zeroes */
        if (predictedStructure != null && predictedStructure.length() > 0)
        {
            Symbol structureSymbol = predictedStructure.getStructureAnnotation().symbolAt(resindex + 1);

            int structAsInt = ProteusTools.convertStructureToInt(structureSymbol);

            outputs[structAsInt] = 1;
        }

        outputArray.add(outputs);
    }

    private void addInputInstance(ScoreAnnotatedSequence jnet, ScoreAnnotatedSequence psipred, ScoreAnnotatedSequence transsec, int resindex)
    {
        double[] inputs = new double[NUM_INPUTS];

        for (int i = resindex - MIDDLERES; i < resindex - MIDDLERES + WINDOW_SIZE; i++)
        {
            Symbol jnetSymbol, psipredSymbol, transsecSymbol;
            if ((i >= 0) && (i < jnet.length()))
            {
                jnetSymbol = jnet.getStructureAnnotation().symbolAt(i + 1);
                psipredSymbol = psipred.getStructureAnnotation().symbolAt(i + 1);
                transsecSymbol = transsec.getStructureAnnotation().symbolAt(i + 1);
            }
            else
            {
                jnetSymbol = ProteusTools.x();
                psipredSymbol = ProteusTools.x();
                transsecSymbol = ProteusTools.x();
            }

            int jnetInt = 0, psipredInt = 0, transsecInt = 0;
            jnetInt = ProteusTools.convertStructureToInt(jnetSymbol);
            transsecInt = ProteusTools.convertStructureToInt(transsecSymbol);
            psipredInt = ProteusTools.convertStructureToInt(psipredSymbol);

            inputs[(i + MIDDLERES - resindex) * ProteusTools.NUM_STRUCTURES * NUM_METHODS + NUM_METHODS * 0 + jnetInt] = 1;
            inputs[(i + MIDDLERES - resindex) * ProteusTools.NUM_STRUCTURES * NUM_METHODS + NUM_METHODS * 1 + psipredInt] = 1;
            inputs[(i + MIDDLERES - resindex) * ProteusTools.NUM_STRUCTURES * NUM_METHODS + NUM_METHODS * 2 + transsecInt] = 1;

            int offset = WINDOW_SIZE * ProteusTools.NUM_STRUCTURES * NUM_METHODS;

            if ((i >= 0) && (i < jnet.length()))
            {
                inputs[offset + (i + MIDDLERES - resindex) * NUM_METHODS + 0] = jnet.getScore(i+1).getMax();
                inputs[offset + (i + MIDDLERES - resindex) * NUM_METHODS + 1] = psipred.getScore(i+1).getMax();
                inputs[offset + (i + MIDDLERES - resindex) * NUM_METHODS + 2] = transsec.getScore(i+1).getMax();
            }

        }

        inputArray.add(inputs);
    }
}
