package ca.ualberta.proteus.program.transsec.test;

import ca.ualberta.proteus.logging.RuntimeCatalog;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.proteus.JNETRunner;
import ca.ualberta.proteus.program.proteus.PsipredRunner;
import ca.ualberta.proteus.program.transsec.*;
import ca.ualberta.proteus.sequence.GappedStructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.db.PredictionDB;
import ca.ualberta.proteus.util.db.PredictionDBFactory;
import org.apache.log4j.Logger;
import org.biojava.bio.Annotation;
import org.biojava.bio.BioException;
import org.biojava.bio.program.sax.blastxml.BlastXMLParserFacade;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.FastaAlignmentFormat;
import org.biojava.bio.seq.io.FastaFormat;
import org.biojava.bio.symbol.*;
import org.biojava.utils.ChangeVetoException;
import org.joone.engine.Monitor;
import org.joone.engine.NeuralNetEvent;
import org.joone.net.NeuralNet;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import java.io.*;
import java.text.NumberFormat;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 *
 * @author Scott Montgomerie
 *         Date: Mar 8, 2005
 *         Time: 9:32:10 AM
 *         To change this template use File | Settings | File Templates.
 */
public class AveragingPredictor extends Predictor
{
    private static final int LAYER2 = UnifiedTrainingSet.NUM_INPUTS + 2 * UnifiedTrainingSet.NUM_OUTPUTS;
    ConsensusPredictor predictor = null;
    private List validationSequences;
    private int numToValidate = 20;
    private double bestFilteredQ3ScoreSoFar = 0;
    private double bestQ3ScoreSoFar = 0;
    private boolean verbose = false;
    private static int numToEval;
    public boolean useStoredNN = false;
    private static volatile NeuralNet storedNN;
    public boolean testing = false;
    private RuntimeCatalog logger = null;
    public static final double DEFAULT_EVALUE = 1e-7;
    private MethodSettings method = MethodSettings.ALL;
    private static boolean evaluating = false;
    private static final String BLAST_DIR = "blastdata";

    Logger log4j = Logger.getLogger(AveragingPredictor.class);

    public AveragingPredictor(RuntimeCatalog logger) {
        this.logger = logger;
    }

    public AveragingPredictor(RuntimeCatalog logger, MethodSettings method) {
        this.logger = logger;
        this.method = method;
    }


    /**
     * Main running method
     *
     * @param sequence
     * @param alignment
     */
    public void run(Sequence sequence, Alignment alignment, String pssmFile, String checkpointFile, String blastOutputFileName, boolean homologySearch)
    {
        PredictionDB db = PredictionDBFactory.getRunTimeDB(AveragingPredictor.evaluating);

        ScoreAnnotatedSequence psipredResult = null;
        ScoreAnnotatedSequence jnetResult = null;
        ScoreAnnotatedSequence proteusSecResult = null;

        if ( method == MethodSettings.ALL || method == MethodSettings.PSIPRED )
        {
            psipredResult = runPSIPRED(sequence, pssmFile, checkpointFile);
            if (psipredResult != null) {
                db.add(sequence, "PSIPRED", psipredResult);

                if ( logger != null)
                {
                    logger.logObject(Consts.PSIPRED, psipredResult);
                    logger.stepComplete(Consts.Steps.PSIPRED);
                }
            }
            else {
                if ( logger != null)
                    logger.error("PSIPRED failed");
            }
        }

        if ( method == MethodSettings.ALL || method == MethodSettings.JNET )
        {
            if ( logger != null)
                logger.log("Running JNET...");

            jnetResult = runJNET(sequence, alignment, pssmFile, blastOutputFileName);
            if (jnetResult != null) {
                db.add(sequence, "JNET", jnetResult);
                if ( logger != null)
                {
                    logger.logObject(Consts.JNET, jnetResult);
                    logger.stepComplete(Consts.Steps.JNET);
                }
            }
            else {
                if ( logger != null)
                    logger.error("JNET failed");
            }
        }

        if ( method == MethodSettings.ALL || method == MethodSettings.TRANSSEC )
        {
            proteusSecResult = runProteusSec(sequence, alignment);
            if (proteusSecResult != null) {
                db.add(sequence, "PROTEUS", proteusSecResult);
                if ( logger != null)
                {
                    logger.logObject(Consts.PROTEUS, proteusSecResult);
                    logger.stepComplete(Consts.Steps.PROTEUS);
                }
            }
            else {
                if ( logger != null)
                    logger.error("Proteus failed");
            }
        }

/*        logger.log("Running MainPredictor...");
        ScoreAnnotatedSequence transsecResult = runTransSec(sequence, alignment);
        if (transsecResult!= null) {
            db.add(sequence, "TRANSSEC", transsecResult);
            logger.logObject("transsec", transsecResult);
        }
        else {
            logger.error("Transec failed");
        }
  */
        System.out.println("\nPSIPRED:" + psipredResult);
        System.out.println("\nJNET:" + jnetResult);
        System.out.println("\nPROTEUS:" + proteusSecResult);
        //System.out.println("\nTRANSSEC:" + transsecResult);

        cleanup(pssmFile, checkpointFile, blastOutputFileName);

        if ( method == MethodSettings.ALL )
        {
            UnifiedTrainingSet ts = new UnifiedTrainingSet();

            ts.addInstance(sequence, jnetResult, psipredResult, proteusSecResult);
            /*if ( transsecResult != null)
                ts.addInstance(sequence, jnetResult, psipredResult, new ScoreAnnotatedSequence(transsecResult));
              */
            try
            {
                ts.writeToStream(new FileOutputStream("unified.ts"));
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    private ScoreAnnotatedSequence runTransSec(Sequence sequence, Alignment alignment)
    {
        try {
            TranssecRunner runner = new TranssecRunner(sequence, alignment);
            runner.run();
            return new ScoreAnnotatedSequence(runner.getResult());
        }
        catch (Throwable t) {
            t.printStackTrace();
            StringWriter out = new StringWriter();
            PrintWriter writer = new PrintWriter( out );
            t.printStackTrace(writer);
            logger.error(out.toString());
            return null;
        }
    }

    public List<ScoreAnnotatedSequence> searchForHomolog(Sequence sequence)
    {
        return searchForHomolog(sequence, AveragingPredictor.DEFAULT_EVALUE, false);
    }

    public List<ScoreAnnotatedSequence> searchForHomolog(Sequence sequence, boolean isTM)
    {
        return searchForHomolog(sequence, AveragingPredictor.DEFAULT_EVALUE, isTM);
    }

    public List<ScoreAnnotatedSequence> searchForHomolog(Sequence sequence, double evalue)
    {
        return searchForHomolog(sequence, evalue);
    }

    /**
     *
     * @param sequence
     * @param searchEvalue
     * @param isTM
     * @return list of sequences that could be used to overlay
     */
    public List<ScoreAnnotatedSequence> searchForHomolog(Sequence sequence, double searchEvalue, boolean isTM)
    {
        return searchForHomolog(sequence, searchEvalue, -1 , isTM);
    }

    /**
     * Returns a structure annotated sequence that is similar to the query sequence
     * @param sequence
     * @param searchEvalue - The evalue limit to give to BLAST
     * @param minEvalue - The evalue we want to limit it to to avoid finding exact homologs (for testing puposes).  Set to -1 for no limit.
     * @param isTM - set to true if the sequence is a transmembrane sequence
     * @return list of sequences that could be used to overlay
     */
    public List<ScoreAnnotatedSequence> searchForHomolog(Sequence sequence, double searchEvalue, double minEvalue, boolean isTM)
    {
        List<ScoreAnnotatedSequence> result = new ArrayList<ScoreAnnotatedSequence>();
        HashMap<ScoreAnnotatedSequence, Double> homologEvalues = new HashMap<ScoreAnnotatedSequence, Double>();
        boolean finished = false;

        while  ( ! finished )
        {
            BlastSettings settings = new BlastSettings();
            settings.setSequence(sequence);
            BlastRunner blastRunner = new BlastRunner(settings);
            settings.setDatabase( Config.getConfig().getStringValue("PDB"));
            settings.setBlastType(BlastSettings.BLASTTYPE.BLASTP);
            settings.setEvalue(searchEvalue);
            blastRunner.run();

            List<Sequence> homologs = blastRunner.getUnalignedResult();
            logger.logObject("homologs", homologs);

            StructureAnnotatedSequence newSequence = null;

            double smallestEvalue = Double.MAX_VALUE;
            String seqName = null;
            HashMap<Double, Sequence> evalueHomologMap = new HashMap<Double, Sequence>();
            for (Sequence seq : homologs) {
                Annotation annotation = seq.getAnnotation();
                Double homologEvalue = (Double) annotation.getProperty("e-value");
                boolean addToSet = (minEvalue == -1 || homologEvalue > minEvalue);
                if ( AveragingPredictor.evaluating )
                {
                    if ( seq.getName().equalsIgnoreCase(sequence.getName()))
                    {
                        addToSet= false;
                    }
                }
                if ( addToSet)
                {
                    evalueHomologMap.put(homologEvalue, seq);

                    if (homologEvalue < smallestEvalue) {
                        smallestEvalue = homologEvalue;
                        seqName = seq.getName();
                    }
                }
            }

            Object[] evalues = evalueHomologMap.keySet().toArray();
            Arrays.sort(evalues);

            SeqSeeDB db = SeqSeeDB.getInstance();

            // Create ScoreAnnotatedSequences from the structures in the DB
            for(Object ev : evalues) {
                Double evalue = (Double) ev;
                String name = evalueHomologMap.get(ev).getName();
                System.out.println( "Sequence: " + name + " evalue: " + evalue);

                try
                {
                    newSequence = (StructureAnnotatedSequence) db.getSequence(name);
                    //logger.logObject(Consts.HOMOLOG, newSequence);
                }
                catch (BioException e)
                {
                    e.printStackTrace();
                }
                ScoreAnnotatedSequence homolog = new ScoreAnnotatedSequence(newSequence);
                try
                {
                    homolog.getAnnotation().setProperty("e-value", evalue);
                }
                catch (ChangeVetoException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }

                if ( evalue <= 10e-20)
                {
                    homolog.fillScores(9);
                }
                else if ( evalue <= 10e-13)
                {
                    homolog.fillScores(8);
                }
                else if ( evalue <= 10e-7) // not that this will ever happen
                {
                    homolog.fillScores(7);
                }

                result.add(homolog);
                homologEvalues.put(homolog, evalue);
            }

            finished = true;
        }

        if ( result.size() > 0 )
        {
            logger.logObject(Consts.HOMOLOG_LIST, result);
            logger.logObject(Consts.HOMOLOG_EVALUES, homologEvalues);
        }

        return result;
    }

    private void cleanup(String pssmFile, String checkpointFile, String blastOutputFileName)
    {
        File f = new File(pssmFile);
        f.delete();
        f = new File(checkpointFile);
        f.delete();
        f = new File(blastOutputFileName);
        f.delete();
    }

    private ScoreAnnotatedSequence runProteusSec(Sequence sequence, Alignment alignment) {
        return runProteusSec(sequence,  alignment, true);
    }

    private ScoreAnnotatedSequence runProteusSec(Sequence sequence, Alignment alignment, boolean doFilter)
    {
        if ( logger != null )
            logger.log("Running Internal Predictor...");
        if (predictor == null)
        {
            predictor = new ConsensusPredictor();

            final PsiPass2Filter ps2Ffilter = new PsiPass2Filter();
            predictor.addFilter(ps2Ffilter);

            final SimpleFilter filter = new SimpleFilter();
            predictor.addFilter(filter);
        }

        ScoreAnnotatedSequence prediction = predictor.predict(sequence, alignment);

        return prediction;
    }

     private ScoreAnnotatedSequence runPSIPRED(Sequence sequence, String pssmFile, String checkpointFile)
    {
        if ( logger != null)
            logger.log("Running PSIPRED...");
        FastaFormat fastaWriter = new FastaFormat();
        String inputFile = getUniqueIdentifier() + ".in";
        try
        {
            PrintStream os = new PrintStream(inputFile);
            fastaWriter.writeSequence(sequence, os);
            os.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        PsipredRunner psipredRunner = new PsipredRunner();
        psipredRunner.querySequence = sequence;
        psipredRunner.setParams(pssmFile, checkpointFile, inputFile);
        psipredRunner.run();
        ScoreAnnotatedSequence psipredResult = psipredRunner.getPrediction();

        File f = new File(inputFile);
        if ( f != null )
            f.delete();

        assert( psipredResult!= null && sequence.length() == psipredResult.length());
        return psipredResult;
    }

    private ScoreAnnotatedSequence runJNET(Sequence sequence, Alignment alignment, String pssmFile, String blastOutputFileName)
    {
        //TODO HACK THIS OUT
        // TODO JNET needs a -m6 blast output, the blast runner by default outputs XML (-m7)
        // do BLAST
        {
            BlastSettings settings = new BlastSettings();
            settings.setSequence(sequence);
            if ( testing )
                settings.setDatabase( Config.getConfig().getStringValue("PDB"));
            settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);
            settings.setAlignmentView(BlastSettings.ALIGNMENT_VIEW.BLUNT_ENDS); // TODO THIS IS THE HACK
            settings.setParse(false);


            BlastRunner blastRunner = new BlastRunner(settings);
            blastRunner.run();
            Map<BlastRunner.OUTPUT_FILE_TYPE, String> outputFileMap = blastRunner.getOutputFileMap();
            blastOutputFileName = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.OUTPUT);

            // cleanup unused files
            File f = new File( outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.CHECKPOINT));
            f.delete();
            f = new File( outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.PSSM));
            f.delete();
           // blastRunner.cleanupTempFiles();
        }

        // TODO END HACKING

        // write out alignment
        String alignmentFile = getUniqueIdentifier() + ".align";
        try
        {
            // write out query sequence first
            FileOutputStream os = new FileOutputStream(alignmentFile);
            FastaFormat fastaWriter = new FastaFormat();
            fastaWriter.writeSequence(sequence, new PrintStream(os));

            // then alignment
            FastaAlignmentFormat writer = new FastaAlignmentFormat();
            writer.writeProtein(os, alignment);
            os.close();
        }
        catch (BioException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        JNETRunner jnetRunner = new JNETRunner();
        jnetRunner.querySequence = sequence;
        jnetRunner.alignment = alignment;
        jnetRunner.setParams(pssmFile, alignmentFile, blastOutputFileName);
        jnetRunner.run();
        ScoreAnnotatedSequence jnetResult = jnetRunner.getPrediction();

        File f = new File(alignmentFile);
        if (!f.delete())
            System.err.println("Unable to delete " + alignmentFile);
        f = new File(alignmentFile + ".hmmprof");
        if ( f.exists() )
            f.delete();

        f = new File(pssmFile);
        f.delete();
        f = new File(blastOutputFileName);
        f.delete();

        assert(jnetResult.length() == 0 || sequence.length() == jnetResult.length());

        return jnetResult;
    }

    /**
     * Method you can call without an alignment - it will perform a BLAST search to obtain the alignment
     * and continue on
     *
     * @param sequence
     */
    public void run(Sequence sequence, boolean homologySearch)
    {
        Map<BlastRunner.OUTPUT_FILE_TYPE, String> outputFileMap = new HashMap<BlastRunner.OUTPUT_FILE_TYPE, String>();
        List<Sequence> blastSequences = new ArrayList<Sequence>();

        // can we find a stored version of the BLAST file?
        //boolean haveStoredBlastResult = findBlast(sequence.getName(), outputFileMap, blastSequences);
        boolean haveStoredBlastResult = false;
        if ( ! haveStoredBlastResult )
        // TODO Put this back in later
        {
            // do BLAST
            BlastSettings settings = new BlastSettings();
            settings.setSequence(sequence);
            if ( testing ) {
                settings.setDatabase( Config.getConfig().getStringValue("PDB"));
                settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);
            }
            else
                settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);

            if ( logger != null)
                logger.log("Running PSI-BLAST...");

            BlastRunner blastRunner = new BlastRunner(settings);
            blastRunner.run();
            blastSequences = blastRunner.getUnalignedResult();

            // get all we need out of Blastrunner, then kill it
            outputFileMap = blastRunner.getOutputFileMap();
            blastRunner = null;

            if ( logger != null)
            {
                logger.logObject(Consts.BLAST_SEQUENCES, blastSequences);
                logger.stepComplete(Consts.Steps.BLAST);
            }

        }

        // do the alignment
        blastSequences.add(sequence);

        String numAlignments = Config.getConfig().getStringValue("num_alignments");
        ProteusTools.chooseAlignments(sequence, blastSequences, Integer.valueOf(numAlignments) + 100 - 1); // leave 100 in there to be reduced later on
        if ( logger != null)
            logger.logObject(Consts.REDUCED_SEQUENCES, blastSequences);

        assert blastSequences.contains(sequence) : "Alignment sequences does not contain query sequence";


        if ( !haveStoredBlastResult)
            saveBlast(sequence, outputFileMap);


        Alignment alignment = null;

        // Only JNET and TRANSSEC need an alignment
        if ( method == MethodSettings.JNET || method == MethodSettings.TRANSSEC || method == MethodSettings.ALL )
        {
            if ( logger != null)
                logger.log("Running XAlign...");
            XAlignRunner xalignRunner = new XAlignRunner(sequence, blastSequences);
            xalignRunner.setUseCache(false);
            xalignRunner.setAddToDB(false);
            xalignRunner.run();

            alignment = xalignRunner.getAlignment();
            if ( logger != null)
            {
                logger.log("XAlign completed successfully.");
                logger.stepComplete(Consts.Steps.XALIGN);
            }

            alignment = ProteusTools.chooseAlignments(sequence, alignment, xalignRunner.getPPSHMap(),
                                ProteusTools.MIN_PPSH_THRESHOLD, ProteusTools.MAX_PPSH_THRESHOLD);

            if ( logger != null)
                logger.logObject(Consts.ALIGNMENT, alignment);

            assert alignment.getLabels().contains(sequence.getName()) : "Alignment does not contain query sequence";

            // kill the programs to free memory
            xalignRunner = null;
        }

        run(sequence, alignment, outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.PSSM),
            outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.CHECKPOINT),
            outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.OUTPUT),
            true);
    }

    public void train(List sequences)
    {
        throw new RuntimeException("");
    }

    public static void main(String[] arg) throws IOException
    {
        ExternalEvaluator evaluator = new ExternalEvaluator("vadarEVA.db");

        evaluator.runAveraging(new File("sequenceList.txt"), 0, 200);
    }

    
    /**
     * Takes a sequence with a prediction on it already, and then overlays the structure of several homologs on
     * top of it to obtain a more accurate prediction
     *
     * @param predictedSequence - the sequence that already has a prediction on it
     * @param homologSequences
     * @return a sequence with its structure annotation composed of the predicted structure + the homolog structure
     */
    public ScoreAnnotatedSequence overlayHomolog(ScoreAnnotatedSequence predictedSequence, List<ScoreAnnotatedSequence> homologSequences)
    {
        logger.debug("Overlaying list (size = " + homologSequences.size() + ") of sequences...");

        if ( homologSequences.size() == 1)
        {
            return overlayHomolog(predictedSequence, homologSequences.get(0));
        }

        List<Sequence> seqList = new ArrayList<Sequence>(homologSequences.size() + 1);
        seqList.add(predictedSequence);
        seqList.addAll(homologSequences);

        Alignment alignment;
   /*     try
        {
            XAlignRunner runner = new XAlignRunner(predictedSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        catch ( ProcessNotCompleteException e )
     */
        {
            // Try ClustalW ?
            ClustalWRunner runner = new ClustalWRunner(predictedSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        SymbolList alignedQuery = alignment.symbolListForLabel(predictedSequence.getName());

        Symbol[] newStructure = new Symbol[ alignedQuery.length() ];
        PredictionScore[] newScore = new PredictionScore[ alignedQuery.length() ];

        List<GappedStructureAnnotatedSequence> alignedHomologues = new ArrayList<GappedStructureAnnotatedSequence>();

        // Create a gappedstructure for the query sequence
        GappedStructureAnnotatedSequence gappedQuery = new GappedStructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", "QUERY", null));
        gappedQuery.createGappedStructure(predictedSequence.getStructureAnnotation());

        // make a list of all the aligned homologues
        for( ScoreAnnotatedSequence homologSequence : homologSequences)
        {
            SymbolList homolog = alignment.symbolListForLabel(homologSequence.getName());

            assert alignedQuery.length() == homolog.length() : "Aligned query does not match aligned homolog";

            // Create a gappedstructure object for the homolog
            GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(homolog, "", homologSequence.getName(), homologSequence.getAnnotation()));
            gappedHomolog.createGappedStructure(homologSequence.getStructureAnnotation());

            alignedHomologues.add(gappedHomolog);
        }

        List<GappedStructureAnnotatedSequence> alignedHomologuesClone = new ArrayList<GappedStructureAnnotatedSequence>();
        alignedHomologuesClone.addAll(alignedHomologues);

        logger.logObject(Consts.ALIGNED_HOMOLOG_LIST, alignedHomologues);
        // clone it so we can remove stuff from it and not change it in the logger
        logger.logObject(Consts.ALL_ALIGNED_HOMOLOG_LIST, alignedHomologuesClone);
        logger.logObject(Consts.ALIGNED_QUERY, gappedQuery);
        logger.logObject(Consts.ALIGNED_HOMOLOG, alignedHomologues.get(0));

        // Weed out the ones that are overlapping
        List<Sequence> removals = new ArrayList<Sequence>();
        for( int i=0; i<homologSequences.size(); i++)
        {
            GappedStructureAnnotatedSequence homologSequence = alignedHomologues.get(i);
            for ( int j=i+1; j<homologSequences.size(); j++)
            {
                GappedStructureAnnotatedSequence comparedHomoSequence = alignedHomologues.get(j);
                if ( homologSequence.overlaps(comparedHomoSequence, 20))
                {
                    removals.add(comparedHomoSequence);
                }
               /*
                if ( homologSequence.residuesNotOverlapping(comparedHomoSequence) < 20)
                {
                    removals.add(comparedHomoSequence);
                }
                */
            }
        }
        // remove overlapping sequences
        for(Sequence s : removals)
        {
            alignedHomologues.remove(s);
        }

        logger.debug("AlignedHomologs has " + alignedHomologues.size() + " sequences.");

        for( GappedStructureAnnotatedSequence gappedHomolog : alignedHomologues)
        {
            log4j.debug(gappedHomolog.seqString());

          //  logger.logObject(Consts.ALIGNED_QUERY, gappedQuery);
          //  logger.logObject(Consts.ALIGNED_HOMOLOG, gappedHomolog);

            Symbol gapSymbol = gappedHomolog.getStructureAnnotation().getAlphabet().getGapSymbol();
            String homologString = gappedHomolog.getStructureAnnotation().seqString();
            String queryString = gappedQuery.getStructureAnnotation().seqString();

            for ( int i=0; i < gappedHomolog.length() && i < gappedQuery.length(); i++)
            {
                Symbol homologSymbol = gappedHomolog.getStructureAnnotation().symbolAt(i+1);
                Symbol querySymbol = gappedQuery.getStructureAnnotation().symbolAt(i+1);
                if ( ! homologSymbol.equals(AlphabetManager.getGapSymbol()) && homologString.charAt(i) != '-')
                {
                    // homolog is not showing a gap, use homolog structure
                    newStructure[i] = homologSymbol;
                    newScore[i] = homologSequences.get(0).getScore(1);  // they're all uniform anyway
                }
                else if ( !querySymbol.equals(gapSymbol) && queryString.charAt(i) != '-')
                {
                    if ( newScore[i] == null )
                    {
                        // if the gapped query sequence isn't gapped here, use the predicted structure
                        newStructure[i] = querySymbol;
                        ///newScore[i] = null;
                    }
                }
            }
        }

        ScoreAnnotatedSequence result = null;
        result = new ScoreAnnotatedSequence(new SimpleSequence(alignedQuery, "", predictedSequence.getName(), null));
        result.setStructureAnnotation(new SimpleSymbolList(newStructure, newStructure.length, new StructureTools().getStructure()));
        result.setScores(newScore);

        // go back to a ungapped structure
        result = GappedStructureAnnotatedSequence.createUngappedStructure(predictedSequence, result);

        for ( int i=1; i<=result.length(); i++)
        {
            if ( result.getScore(i) == null || result.getScore(i).getMax() == 0)
            {
                result.setScore(i, predictedSequence.getScore(i));
            }
        }

        SimpleFilter filter = new SimpleFilter();
        result = filter.filter(result);

        /// copy the annotation
        if ( alignedHomologues.size() > 0)
        {
            Annotation annotation = alignedHomologues.get(0).getAnnotation();
            for (Object key : annotation.keys() )
            {
                try
                {
                    result.getAnnotation().setProperty(key, annotation.getProperty(key));
                }
                catch (ChangeVetoException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }

        return result;
    }


    /**
     * Takes a sequence with a prediction on it already, and then overlays the structure of several homologs on
     * top of it to obtain a more accurate prediction
     *
     * @param sequence the sequence that already has a prediction on it
     * @return a sequence with its structure annotation composed of the predicted structure + the homolog structure
     */
    public ScoreAnnotatedSequence searchAndOverlayHomolog(ScoreAnnotatedSequence sequence, double searchEvalue, double minEvalue, boolean isTM)
    {
        List<GappedSequence> homologs = new ArrayList<GappedSequence>();
        Sequence blastedSequence = sequence;
        while ( true )
        {
            BlastSettings settings = new BlastSettings();
            settings.setSequence(blastedSequence);
            BlastRunner blastRunner = new BlastRunner(settings);
            settings.setDatabase(Config.getConfig().getStringValue("PDB"));
            settings.setBlastType(BlastSettings.BLASTTYPE.PROTEUS_BLASTPARSER_BLASTP);
            settings.setEvalue(searchEvalue);
            blastRunner.run();

            List<GappedSequence> blastSequences = blastRunner.getAlignedResult();

            if ( blastSequences.size() == 0)
            {
                break;
            }

            homologs.addAll( blastSequences );
            blastedSequence = blockOutAlignedRegions(blastedSequence, homologs);
        }
        logger.logObject("homologs", homologs);

        if ( homologs.size() == 0)
        {
            System.out.println("No homologs found.");
            return null;
        }

        double smallestEvalue = Double.MAX_VALUE;
        String seqName = null;
        HashMap<Double, GappedSequence> evalueHomologMap = new HashMap<Double, GappedSequence>();

        GappedSequence alignedQuery = null;

        /** Remove some homologs **/
        for (GappedSequence seq : homologs)
        {
            Annotation annotation = seq.getAnnotation();
            Double homologEvalue = (Double) annotation.getProperty("e-value");
            boolean addToSet = homologEvalue != null && (minEvalue == -1 || homologEvalue > minEvalue);

            if ( seq.getName().equals("QUERY"))
            {
                alignedQuery = seq;
            }

            if (AveragingPredictor.evaluating)
            {
                if (seq.getName().equalsIgnoreCase(sequence.getName()))
                {
                    addToSet = false;
                }
            }
            if (addToSet)
            {
                evalueHomologMap.put(homologEvalue, seq);

                if (homologEvalue < smallestEvalue)
                {
                    smallestEvalue = homologEvalue;
                    seqName = seq.getName();
                }

                System.out.println(seq.getName() + ": " + seq.seqString());
            }
        }

        assert alignedQuery != null : "CAN'T FIND ALIGNED QUERY";

        System.out.println(alignedQuery.getName() + ": " + alignedQuery.seqString());

        Object[] evalues = evalueHomologMap.keySet().toArray();
        Arrays.sort(evalues);

        SeqSeeDB db = SeqSeeDB.getInstance();

        List<GappedStructureAnnotatedSequence> alignedSequences = new ArrayList<GappedStructureAnnotatedSequence>();
        HashMap<GappedStructureAnnotatedSequence, Double> homologEvalues = new HashMap<GappedStructureAnnotatedSequence, Double>();

        /**
         * Create ScoreAnnotatedSequences from the structures in the DB
         */
        for (Object ev : evalues)
        {
            Double evalue = (Double) ev;
            GappedSequence gappedSequence = evalueHomologMap.get(ev);
            String name = gappedSequence.getName();
            System.out.println("Sequence: " + name + " evalue: " + evalue);

            StructureAnnotatedSequence newSequence = null;
            try
            {
                newSequence = (StructureAnnotatedSequence) db.getSequence(name);
            }
            catch (BioException e)
            {
                e.printStackTrace();
            }
            GappedStructureAnnotatedSequence homolog = new GappedStructureAnnotatedSequence(gappedSequence);
            homolog.createGappedStructureToIncompleteSequence(newSequence);

            System.out.println(homolog);

            try
            {
                homolog.getAnnotation().setProperty("e-value", evalue);
            }
            catch (ChangeVetoException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

            /*if (evalue <= 10e-20)
            {
                homolog.fillScores(9);
            }
            else if (evalue <= 10e-13)
            {
                homolog.fillScores(8);
            }
            else if (evalue <= 10e-7) // not that this will ever happen
            {
                homolog.fillScores(7);
            }
            */

            alignedSequences.add(homolog);
            homologEvalues.put(homolog, evalue);
        }

        logger.logObject(Consts.HOMOLOG_EVALUES, homologEvalues);

        if ( alignedSequences.size() == 0)
        {
            System.out.println("No homologs found.");
            return null;
        }
       // List<GappedStructureAnnotatedSequence> alignedHomologues = new ArrayList<GappedStructureAnnotatedSequence>();

        // Create a gappedstructure for the query sequence
        GappedStructureAnnotatedSequence gappedQuery = new GappedStructureAnnotatedSequence(alignedQuery);
        gappedQuery.createGappedStructure(sequence.getStructureAnnotation());

        // make a list of all the aligned homologues
       /* for (ScoreAnnotatedSequence homologSequence : alignedSequences)
        {
            // Create a gappedstructure object for the homolog
            GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(homolog, "", homologSequence.getName(), homologSequence.getAnnotation()));
            gappedHomolog.createGappedStructure(homologSequence.getStructureAnnotation());

            alignedHomologues.add(gappedHomolog);
        }
        */


        List<GappedStructureAnnotatedSequence> alignedHomologuesClone = new ArrayList<GappedStructureAnnotatedSequence>();
        alignedHomologuesClone.addAll(alignedSequences);

        // clone it so we can remove stuff from it and not change it in the logger
        logger.logObject(Consts.ALL_ALIGNED_HOMOLOG_LIST, alignedHomologuesClone);


        logger.logObject(Consts.ALIGNED_QUERY, gappedQuery);

        // Weed out the ones that are overlapping
        /*
        List<Sequence> removals = new ArrayList<Sequence>();
        for (int i = 0; i < alignedSequences.size(); i++)
        {
            GappedStructureAnnotatedSequence homologSequence = alignedSequences.get(i);
            for (int j = i + 1; j < alignedSequences.size(); j++)
            {
                GappedStructureAnnotatedSequence comparedHomoSequence = alignedSequences.get(j);
                /*if (homologSequence.overlaps(comparedHomoSequence))
                {
                    removals.add(comparedHomoSequence);
                } */
                /*
                if ( homologSequence.residuesNotOverlapping(comparedHomoSequence) < 20)
                {
                    removals.add(comparedHomoSequence);
                }

            }
        }
        // remove overlapping sequences
       /* for (Sequence s : removals)
        {
            alignedSequences.remove(s);
        }
        */

        Symbol[] newStructure = new Symbol[ alignedQuery.length() ];
        PredictionScore[] newScore = new PredictionScore[ alignedQuery.length() ];

        /** Assign the structure from the homologs **/
        final int WINDOW = 7;
        final int MIDDLERES = WINDOW / 2;
        String gappedQueryString = gappedQuery.seqString();

        boolean[] alignmentUsed = new boolean[alignedSequences.size()];

        for (int resIndex = 0; resIndex < gappedQuery.length(); resIndex++)
        {
            int[] scores = new int[alignedSequences.size()];
            for (int seqIndex=0; seqIndex  < alignedSequences.size(); seqIndex++)
            {
                GappedStructureAnnotatedSequence gappedHomolog = alignedSequences.get(seqIndex);
                String gappedHomologString = gappedHomolog.seqString();

                // find each sequence's similarity score based on a window size
                for (int i = resIndex - MIDDLERES; i < resIndex - MIDDLERES + WINDOW; i++)
                {
                    if ((i >= 0) && (i < sequence.length()))
                    {
                        char homologRes = gappedHomologString.charAt(i);
                        char queryRes = gappedQueryString.charAt(i);

                        if (homologRes == queryRes)
                        {
                            // can't map to a gap
                            if ( gappedHomologString.charAt(resIndex) != '-'
                                    && gappedHomologString.charAt(resIndex) != '~' )
                            {
                                scores[seqIndex]++;
                            }
                        }
                    }
                }
            }

            // find the top score
            int topScore = -1, topScoreIndex = -1;
            for(int seqIndex=0; seqIndex< scores.length; seqIndex++)
            {
                if ( scores[seqIndex] > topScore)
                {
                    topScore = scores[seqIndex];
                    topScoreIndex = seqIndex;
                }
            }

            /* if there is a homolog at this residue that scores, and that score is > 2 /7
              then assign its structure.  Otherwise, assign the predicted structure
               */
            if ( topScoreIndex != -1
                    && topScore >= 2 )
            {
                Symbol homologSymbol = alignedSequences.get(topScoreIndex).getStructureAnnotation().symbolAt(resIndex + 1);
                newStructure[resIndex] = homologSymbol;
                alignmentUsed[topScoreIndex] = true;
                if ( topScore >= 4)
                {
                    newScore[resIndex] = new PredictionScore(homologSymbol, 0.9);

                }
                else if ( topScore >= 3 )
                {
                    newScore[resIndex] = new PredictionScore(homologSymbol, 0.8);
                }
                else if ( topScore >= 2 )
                {
                    newScore[resIndex] = new PredictionScore(homologSymbol, 0.7);
                }
            }
            else
            {
                Symbol querySymbol = gappedQuery.getStructureAnnotation().symbolAt(resIndex + 1);
                newStructure[resIndex] = querySymbol;
            }
        }

        /*
            Eliminate all aligned sequences that weren't used in the mapping, for output purposes
         */

        List<Sequence> removals = new ArrayList<Sequence>();
        for (int i = 0; i < alignedSequences.size(); i++)
        {
            GappedStructureAnnotatedSequence homologSequence = alignedSequences.get(i);
            if ( ! alignmentUsed[i])
            {
                removals.add(homologSequence);
            }
        }

        // remove sequences
        for (Sequence s : removals)
        {
            alignedSequences.remove(s);
        }

        logger.logObject(Consts.ALIGNED_HOMOLOG_LIST, alignedSequences);
        logger.debug("AlignedHomologs has " + alignedSequences.size() + " sequences.");

        /* End elimination */


        ScoreAnnotatedSequence result = null;
        result = new ScoreAnnotatedSequence(new SimpleSequence(alignedQuery, "", alignedSequences.get(0).getName(), null));
        result.setStructureAnnotation(new SimpleSymbolList(newStructure, newStructure.length, new StructureTools().getStructure()));
        result.setScores(newScore);

        // go back to a ungapped structure
        result = GappedStructureAnnotatedSequence.createUngappedStructure(sequence, result);

        for (int i = 1; i <= result.length(); i++)
        {
            if (result.getScore(i) == null || result.getScore(i).getMax() == 0)
            {
                result.setScore(i, sequence.getScore(i));
            }
        }

        SimpleFilter filter = new SimpleFilter();
        result = filter.filter(result);

        /// copy the annotation
        if (alignedSequences.size() > 0)
        {
            Annotation annotation = alignedSequences.get(0).getAnnotation();
            for (Object key : annotation.keys())
            {
                try
                {
                    result.getAnnotation().setProperty(key, annotation.getProperty(key));
                }
                catch (ChangeVetoException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }

        try
        {
            result.getAnnotation().setProperty("homolog", alignedSequences.get(0).getName());
        }
        catch (ChangeVetoException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return result;
    }

    private Sequence blockOutAlignedRegions(Sequence blastedSequence, List<GappedSequence> homologs)
    {
        char[] seqString = null;
        List<String> homologStrings = new ArrayList<String>(homologs.size());

        for(GappedSequence homolog: homologs)
        {
            if ( ! homolog.getName().equals("QUERY"))
            {
                homologStrings.add(homolog.seqString());
            }
            else
            {
                seqString = homolog.seqString().toCharArray();
            }
        }

        assert(seqString != null);

        for (int i=0; i<seqString.length; i++)
        {
            for ( String homolog: homologStrings)
            {
                if ( ! ProteusTools.isGap(homolog.charAt(i)) && seqString[i] != '-')
                {
                    seqString[i] = 'X';
                }
            }
        }

        try
        {
            blastedSequence = ProteinTools.createProteinSequence(new String(seqString), blastedSequence.getName());
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return blastedSequence;
    }

    /**
     * Takes a sequence with a prediction on it already, and then overlays the structure of a homolog on
     * top of it to obtain a more accurate prediction
     *
     * @param predictedSequence - the sequence that already has a prediction on it
     * @param homologSequence
     * @return a sequence with its structure annotation composed of the predicted structure + the homolog structure
     */
    public ScoreAnnotatedSequence overlayHomolog(ScoreAnnotatedSequence predictedSequence, ScoreAnnotatedSequence homologSequence)
    {
        List<Sequence> seqList = new ArrayList<Sequence>(2);
        seqList.add(predictedSequence);
        seqList.add(homologSequence);

        Alignment alignment;
        try
        {
            XAlignRunner runner = new XAlignRunner(predictedSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        catch ( ProcessNotCompleteException e )
        {
            // Try ClustalW ?
            ClustalWRunner runner = new ClustalWRunner(predictedSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        SymbolList alignedQuery = alignment.symbolListForLabel(predictedSequence.getName());
        SymbolList homolog = alignment.symbolListForLabel(homologSequence.getName());

        assert alignedQuery.length() == homolog.length() : "Aligned query does not match aligned homolog";

        Symbol[] newStructure = new Symbol[ alignedQuery.length() ];
        PredictionScore[] newScore = new PredictionScore[ alignedQuery.length() ];

        // Create a gappedstructure object for the homolog
        GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(homolog, "", homologSequence.getName(), null));
        gappedHomolog.createGappedStructure(homologSequence.getStructureAnnotation());

        // And for the query sequence
        GappedStructureAnnotatedSequence gappedQuery = new GappedStructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", "QUERY", null));
        gappedQuery.createGappedStructure(predictedSequence.getStructureAnnotation());

        List<GappedStructureAnnotatedSequence> alignedHomologues = new ArrayList<GappedStructureAnnotatedSequence>();
        alignedHomologues.add(gappedHomolog);

        logger.logObject(Consts.ALIGNED_HOMOLOG_LIST, alignedHomologues);
        logger.logObject(Consts.ALIGNED_QUERY, gappedQuery);
        logger.logObject(Consts.ALIGNED_HOMOLOG, gappedHomolog);

        Symbol gapSymbol = gappedHomolog.getStructureAnnotation().getAlphabet().getGapSymbol();

        String homologString = gappedHomolog.getStructureAnnotation().seqString();
        String queryString = gappedQuery.getStructureAnnotation().seqString();

        for ( int i=0; i < gappedHomolog.length() && i < gappedQuery.length(); i++)
        {
            Symbol homologSymbol = gappedHomolog.getStructureAnnotation().symbolAt(i+1);
            Symbol querySymbol = gappedQuery.getStructureAnnotation().symbolAt(i+1);
            if ( ! homologSymbol.equals(AlphabetManager.getGapSymbol()) && homologString.charAt(i) != '-')
            {
                // homolog is not showing a gap, use homolog structure
                newStructure[i] = homologSymbol;
                newScore[i] = homologSequence.getScore(1);  // they're all uniform anyway
            }
            else if ( !querySymbol.equals(gapSymbol) && queryString.charAt(i) != '-')
            {
                if ( newScore[i] == null )
                {
                    // if the gapped query sequence isn't gapped here, use the predicted structure
                    newStructure[i] = querySymbol;
                    ///newScore[i] = null;
                }
            }
        }


        ScoreAnnotatedSequence result;
        result = new ScoreAnnotatedSequence(new SimpleSequence(alignedQuery, "", predictedSequence.getName(), null));
        result.setStructureAnnotation(new SimpleSymbolList(newStructure, newStructure.length, new StructureTools().getStructure()));
        result.setScores(newScore);

        // go back to a ungapped structure
        result = GappedStructureAnnotatedSequence.createUngappedStructure(predictedSequence, result);

        for ( int i=1; i<=result.length(); i++)
        {
            if ( result.getScore(i) == null || result.getScore(i).getMax() == 0)
                result.setScore(i, predictedSequence.getScore(i));
        }

        SimpleFilter filter = new SimpleFilter();
        result = filter.filter(result);

        try
        {
            result.getAnnotation().setProperty("e-value", homologSequence.getAnnotation().getProperty("e-value"));
        }
        catch (ChangeVetoException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return result;
    }

    private static void printUsage()
    {
        System.out.println("Usage: <ProgName> (-train <# seq> | -eval <#seq> | -load <#seq> [#startAt]) [options]" );
        System.out.println("Options: ");
        System.out.println("-train <# sequences>\t\tTrain on a number of sequences");
        System.out.println("-eval <# sequences>\t\tEvaluate a number of sequences");
        System.out.println("-load <# sequences> [ID# to start at]");
        System.out.println("-ext <# Name of sequence database>");
        System.exit(-1);
    }

    public void run()
    {
        throw new IllegalArgumentException("Must pass a sequence - use other overloaded method");
    }

    public ScoreAnnotatedSequence predict(Sequence sequence, NeuralNet localNNet, boolean doFilter)
    {

        assert localNNet == null;

        PredictionDB db = PredictionDBFactory.getRunTimeDB(AveragingPredictor.evaluating);
        
        ScoreAnnotatedSequence jnetResult = db.get(sequence, "JNET");
        ScoreAnnotatedSequence psipredResult = db.get(sequence, "PSIPRED");
        ScoreAnnotatedSequence proteusSecResult = db.get(sequence, "PROTEUS");
        ScoreAnnotatedSequence transsecResult = db.get(sequence, "PROTEUS");

        if (!db.contains(sequence)
                || jnetResult == null
                || psipredResult == null
                || proteusSecResult == null)
            run(sequence, false);

        // If we aren't predicting all methods, we can't do a consensus!
        if ( method != MethodSettings.ALL )
            return null;

        if (jnetResult == null )
        {
            jnetResult = db.get(sequence, "JNET");
        }
        if (psipredResult == null )
        {
            psipredResult = db.get(sequence, "PSIPRED");
        }
        if ( proteusSecResult == null )
        {
            proteusSecResult = db.get(sequence, "PROTEUS");
        }
        if ( transsecResult == null )
        {
            transsecResult = db.get(sequence, "PROTEUS");
        }

        // verify that they are in fact in the database
        if (jnetResult == null || psipredResult == null || transsecResult == null)
        {
            run(sequence, false);

            if (jnetResult == null )
            {
                jnetResult = db.get(sequence, "JNET");
            }
            if (psipredResult == null )
            {
                psipredResult = db.get(sequence, "PSIPRED");
            }
            if ( proteusSecResult == null )
            {
                proteusSecResult = db.get(sequence, "PROTEUS");
            }
            if ( transsecResult == null )
            {
                transsecResult = db.get(sequence, "PROTEUS");
            }
        }

        // TODO hack, this shouldn't ever happen
        if ( transsecResult == null ) {
            transsecResult = psipredResult;
            System.err.println("TransSec Failed");
            logger.logObject("transsec", transsecResult);
        }

        ScoreAnnotatedSequence result = null;

        Symbol[] symbolList = new Symbol[sequence.length()];

        try
        {
            result = new ScoreAnnotatedSequence(sequence);

            String jnet = jnetResult.getStructureAnnotation().seqString().toUpperCase();
            String transSec = transsecResult.getStructureAnnotation().seqString().toUpperCase();
            String psirped = psipredResult.getStructureAnnotation().seqString().toUpperCase();


            for (int index = 0; index<sequence.length(); index++)
            {
                final int[] outputArr = new int[4];
                switch(jnet.charAt(index))
                {
                    case 'H':
                        outputArr[ProteusTools.HELIX]++;
                        break;
                    case 'E':
                        outputArr[ProteusTools.STRAND]++;
                        break;
                    case 'C':
                        outputArr[ProteusTools.COIL]++;
                        break;
                }
                switch(psirped.charAt(index))
                {
                    case 'H':
                        outputArr[ProteusTools.HELIX]++;
                        break;
                    case 'E':
                        outputArr[ProteusTools.STRAND]++;
                        break;
                    case 'C':
                        outputArr[ProteusTools.COIL]++;
                        break;
                }
                switch(transSec.charAt(index))
                {
                    case 'H':
                        outputArr[ProteusTools.HELIX]++;
                        break;
                    case 'E':
                        outputArr[ProteusTools.STRAND]++;
                        break;
                    case 'C':
                        outputArr[ProteusTools.COIL]++;
                        break;
                }

                double max = -1;
                int maxIndex = -1;
                for (int j = 0; j < outputArr.length; j++)
                {
                    if (outputArr[j] > max)
                    {
                        max = outputArr[j];
                        maxIndex = j;
                    }
                }

                final Symbol symbol = ProteusTools.convertIntToStructure(maxIndex);
                symbolList[index] = symbol;
            }

            SimpleSymbolList structure = new SimpleSymbolList(symbolList, symbolList.length, new StructureTools().getStructure());

            result.setStructureAnnotation(structure);
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();
        }

        if (verbose) System.out.println("Finished");

        if (doFilter)
        {
            // filter the result with the attached filters
            for (PredictionFilter filter : filters)
                result = filter.filter(result);

            // default filtering
            if ( filters.size() == 0 )
            {
                result = new SimpleFilter().filter(result);
            }
        }


        if ( !useStoredNN ) {
            // Clean up our memory
            AveragingPredictor.storedNN = null;
            localNNet = null;
        }

        return result;
    }

    /**
     * Returns a string that represents a unique ID for a file, based on the current time in milliseconds
     * and thread ID, so you can have many threads concurrently creating the same type of file with unique ID's
     * Note - you should append a suffix to this file name if it is a file name you're using
     *
     * @return a string that represents a unique ID for a file
     */
    protected String getUniqueIdentifier()
    {
        return String.valueOf(System.currentTimeMillis()) + String.valueOf(Thread.currentThread().getId());
    }

    public void cicleTerminated(NeuralNetEvent e)
    {
        // call inherited method, it will do validation on a set of sequences
        super.cicleTerminated(e);

        Monitor mon = (Monitor) e.getSource();
//        long c = mon.getCurrentCicle();
//        long c1 = mon.getTotCicles() - c;

        System.out.println("Training Error = " + mon.getGlobalError());

        SeqSeeDB db = SeqSeeDB.getInstance();

        if (validationSequences == null) {
            //validationSequences = db.getEvaluationSet(numToValidate , 2000, 3000);
            validationSequences = db.getEvaluationSet2(numToValidate , 1000, 6000, 77);
            //validationSequences.addAll( db.getEvaluationSet(numToValidate /2 , 5000, 6000) );
        }

        Score score = new Score();
        Score filteredScore = new Score();

        NeuralNet localNN = nnet.cloneNet();
        localNN.getMonitor().setExporting(false);
        localNN.removeAllListeners();

        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);

        int i = 0;
        for (Iterator it = validationSequences.iterator(); it.hasNext();)
        {
            final StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.next();

            StructureAnnotatedSequence prediction;
            prediction = predict(sequence, localNN, false);

            double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
            double sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
            if (i < 5)
                System.out.println("Without filter: "
                                   + " Q3: " + format.format(q3)
                                   + " SOV: " + format.format(sov)
                                   + prediction);
            score.q3 += q3;
            score.sov += sov;

            prediction = predict(sequence, localNN, true);
            if (i++ < 5)
                System.out.println("Filtered: "
                                   + " Q3: " + format.format(q3)
                                   + " SOV: " + format.format(sov)
                                   + prediction);
            q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());
            sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), prediction.getStructureAnnotation());

            filteredScore.q3 += q3;
            filteredScore.sov += sov;
        }

        System.out.print("Avg Q3: " + score.q3 / validationSequences.size());
        System.out.println("\t SOV: " + score.sov / validationSequences.size());

        System.out.print("Filtered Q3: " + filteredScore.q3 / validationSequences.size());
        System.out.println("\t SOV: " + filteredScore.sov / validationSequences.size());

        if (score.q3 > bestQ3ScoreSoFar)
        {
            // StringFormatter format = new Formatter();
            // format.
            System.out.println("Best Q3 score so far! (" + score.q3 / validationSequences.size() + ")");
            bestQ3ScoreSoFar = score.q3;

            final Date date = new Date();
            saveNeuralNet("bestFinalNN.model" + date.getDate() + "-" + ( date.getMonth() +1));
        }

        if (filteredScore.q3 > bestFilteredQ3ScoreSoFar)
        {
            // StringFormatter format = new Formatter();
            // format.
            System.out.println("Best Filtered Q3 score so far! (" + filteredScore.q3 / validationSequences.size() + ")");
            bestFilteredQ3ScoreSoFar = filteredScore.q3;

            final Date date = new Date();
            saveNeuralNet("bestFinalFilteredNN.model" + date.getDate() + "-" + (date.getMonth() + 1));
        }
    }

    public void setLogger(RuntimeCatalog logger)
    {
        this.logger = logger;
    }

    /**
     * Saves intermediate blast files in a directory in case we need to use them again
     * @param sequence
     * @param outputFileMap
     */
    private void saveBlast(Sequence sequence, Map<BlastRunner.OUTPUT_FILE_TYPE, String> outputFileMap)
    {
        String seqName = sequence.getName();

        File dir = new File(AveragingPredictor.BLAST_DIR);
        if ( !dir.exists())
            dir.mkdir();

        try
        {
            String oldFile = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.PSSM);
            File f = new File(oldFile);
            String pathname = oldFile.replaceFirst(".*(\\..*)", AveragingPredictor.BLAST_DIR + "/" + seqName + "$1");
            FileUtils.copyFile(f, new File( pathname ));

            oldFile = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.FREQUENCY);
            if ( oldFile != null )
            {
                f = new File(oldFile);
                pathname = oldFile.replaceFirst(".*(\\..*)", AveragingPredictor.BLAST_DIR + "/" + seqName + "$1");
                FileUtils.copyFile(f, new File( pathname ));
            }

            oldFile = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.OUTPUT);
            f = new File(oldFile);
            pathname = oldFile.replaceFirst(".*(\\..*)", AveragingPredictor.BLAST_DIR + "/" + seqName + "$1");
            FileUtils.copyFile(f, new File( pathname ));

            oldFile = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.CHECKPOINT);
            f = new File(oldFile);
            pathname = oldFile.replaceFirst(".*(\\..*)", AveragingPredictor.BLAST_DIR + "/" + seqName + "$1");
            FileUtils.copyFile(f, new File( pathname ));
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Finds intermediate blast files when we want them again
     * @param seqName
     * @param outputFileMap - not null
     * @param blastSequences - not null
     * @return true if it finds a set of blast files
     */
    private boolean findBlast(String seqName, Map<BlastRunner.OUTPUT_FILE_TYPE, String> outputFileMap, List<Sequence> blastSequences)
    {
        File dir = new File(AveragingPredictor.BLAST_DIR);
        if ( !dir.exists() )
            return false;

        File[] files = dir.listFiles();
        for (File f: files)
        {
            // lose everything after "."
            String fname = f.getName().replaceFirst("\\..*", "");

            if ( fname.equals(seqName) || fname.regionMatches(true, 0, seqName, 0 , 4))
            {
                try
                {
                    String uniqueIdentifier = getUniqueIdentifier();
                    File pssm = new File(AveragingPredictor.BLAST_DIR + "/" + fname + BlastRunner.PSSM_SUFFIX);
                    if ( pssm.exists() )
                    {
                        String newFile = uniqueIdentifier + BlastRunner.PSSM_SUFFIX;
                        FileUtils.copyFile(pssm, new File(newFile));
                        outputFileMap.put(BlastRunner.OUTPUT_FILE_TYPE.PSSM, newFile );
                    }
                    //else
                    //    return false;

                    File chk = new File(AveragingPredictor.BLAST_DIR + "/" + fname + BlastRunner.CHKPOINT_SUFFIX);
                    if ( chk.exists() )
                    {
                        String newFile = uniqueIdentifier + BlastRunner.CHKPOINT_SUFFIX;
                        FileUtils.copyFile(chk, new File(newFile));
                        outputFileMap.put(BlastRunner.OUTPUT_FILE_TYPE.CHECKPOINT, newFile );
                    }
                    //else
                    //    return false;

                    File out = new File(AveragingPredictor.BLAST_DIR + "/" + fname + BlastRunner.OUTPUT_SUFFIX);
                    if ( out.exists() )
                    {
                        String newFile = uniqueIdentifier + BlastRunner.OUTPUT_SUFFIX;
                        FileUtils.copyFile(out, new File(newFile));
                        outputFileMap.put(BlastRunner.OUTPUT_FILE_TYPE.OUTPUT, newFile );

                        Reader fileReader = new FileReader(out);
                        BlastParser parser = new BlastParser();

                        XMLReader oParser = new BlastXMLParserFacade();

                        parser.parseBlastResult(oParser, fileReader, blastSequences);

                        if ( method == MethodSettings.ALL || method == MethodSettings.PSIPRED )
                        {
                            if ( pssm.exists() && chk.exists() )
                            {
                                // We have to test Makemat too, sometimes it doesn't work

                                String chkptFile = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.CHECKPOINT);
                                String blastfileName = outputFileMap.get(BlastRunner.OUTPUT_FILE_TYPE.OUTPUT);
                                fname = chkptFile.substring(0, chkptFile.indexOf('.'));
                                fname = fname.replaceFirst(".*/(.*$)", "$1");

                                PsipredRunner psipredRunner = new PsipredRunner();
                                psipredRunner.setParams(pssm.getAbsolutePath(), chk.getAbsolutePath(), blastfileName);

                                if (!psipredRunner.runMakeMat(fname, chk) )
                                    continue;

                                return true;
                            }
                        }

                        return true;
                    }
                    //else
                    //    return false;
                }
                catch (FileNotFoundException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (IllegalSymbolException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (BioException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (IOException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (ChangeVetoException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                catch (SAXException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }
        return false;
    }

    public void setEvaluating(boolean b)
    {
        AveragingPredictor.evaluating = b;
    }

    public ScoreAnnotatedSequence searchAndOverlayHomolog(ScoreAnnotatedSequence sequence, boolean isTM)
    {
        return searchAndOverlayHomolog(sequence, AveragingPredictor.DEFAULT_EVALUE,  -1, isTM);
    }
}
