package ca.ualberta.proteus.program.transsec.test;

import ca.ualberta.proteus.program.transsec.ConsensusPredictor;
import ca.ualberta.proteus.program.transsec.ConsensusTrainingSet;
import ca.ualberta.proteus.program.transsec.PSSM;
import ca.ualberta.proteus.program.transsec.PsiPass2Filter;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.SimpleFilter;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

/**
 * ConsensusPredictorTest Tester.
 *
 * @author <Authors name>
 * @since <pre>05/09/2005</pre>
 * @version 1.0
 */
public class ConsensusPredictorTest extends TestCase
{
    public ConsensusPredictorTest(String name)
    {
        super(name);
    }

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public static Test suite()
    {
        return new TestSuite(ConsensusPredictorTest.class);
    }

    public static void main(String[] args) {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void testSaveLoadTrainingSets() throws Exception {

        /* Build a test set */
        ConsensusPredictor predictor = new ConsensusPredictor();

        // Have to add the filters in this order - use the NN filter, then the reg exp filter
        final PsiPass2Filter ps2Ffilter = new PsiPass2Filter();
        predictor.addFilter(ps2Ffilter);

        final SimpleFilter filter = new SimpleFilter();
        predictor.addFilter(filter);

        SeqSeeDB db = SeqSeeDB.getInstance();

        List<StructureAnnotatedSequence> randomSequences;
        randomSequences = db.getEvaluationSet(2, 2000, 3000);

        ConsensusTrainingSet savedTrainingSet = predictor.buildTrainingSet(randomSequences);

        /** Now load it back up **/
        ConsensusTrainingSet trainingSet = new ConsensusTrainingSet();

        try
        {
            FileInputStream fis = new FileInputStream(ConsensusPredictor.TRAINING_SET_FILE);
            trainingSet.readFromStream(fis);
            trainingSet.readMetaDataFromFile(ConsensusPredictor.TRAINING_SET_META_FILE);
        }
        catch (IOException ex)
        {
            System.out.println("Cannot perform output.");
        }


        assertEquals(trainingSet.getPssmList(), savedTrainingSet.getPssmList());

        // Compare them, ensure they come out in the same order
        for (Iterator it = trainingSet.getPssmList().keySet().iterator(),
                it2 = savedTrainingSet.getPssmList().keySet().iterator();
            it.hasNext() &&
                it2.hasNext(); ) {
            String key = (String) it.next();
            final PSSM pssm = (PSSM) trainingSet.getPssmList().get( key );

            String key2 = (String) it2.next();
            final PSSM pssm2 = (PSSM) savedTrainingSet.getPssmList().get( key2 );

            assertEquals(key, key2);
            assertEquals(pssm, pssm2);
        }

        assertEquals(trainingSet.getNumInputs(), savedTrainingSet.getNumInputs());
        assertEquals(trainingSet.numInstances(), savedTrainingSet.numInstances());
    }

    public void readTrainingSet() {
        ConsensusTrainingSet trainingSet = new ConsensusTrainingSet();
        try
        {
            FileInputStream fis = new FileInputStream(ConsensusPredictor.TRAINING_SET_FILE);
            trainingSet.readFromStream(fis);
            trainingSet.readMetaDataFromFile(ConsensusPredictor.TRAINING_SET_META_FILE);
        }
        catch (IOException ex)
        {
            System.out.println("Cannot perform output.");
        }

        assertTrue(trainingSet.numInstances() != 0);
    }
}
