package ca.ualberta.proteus.program.transsec.test;

import ca.ualberta.proteus.program.transsec.PredictionScore;
import ca.ualberta.proteus.program.transsec.MainPredictor;
import ca.ualberta.proteus.sequence.GappedStructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.BioException;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.symbol.*;

import java.io.PrintWriter;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

public class TransSecTest extends TestCase
{
    public static Test suite()
    {
        return new TestSuite(TransSecTest.class);
    }

    public static void main(String[] args) throws BioException, IOException
    {
        try
        {
            new junit.textui.TestRunner().doRun(suite());
        }
        catch (Exception e)
        {
        }

        if (args[0].equals("homolog"))
            new TransSecTest().findOptimalHomologSetting();
    }

    public void newtest() throws Exception
    {
        StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) SeqSeeDB.getTestSequence();
        MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
        StructureAnnotatedSequence result = mainPredictor.searchForHomolog(sequence).get(0);

        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);

        double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), result.getStructureAnnotation());
        double sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), result.getStructureAnnotation());

        System.out.println("Q3: " + format.format(q3)
                + " SOV: " + format.format(sov) + "\n"
                + result);
    }

    public void testHomologOverlay() throws Exception
    {
        StructureDB db = new StructureDB("pdbstruct.fa", StructureDB.MODE.READ);
        for (SequenceIterator it = db; it.hasNext();)
        {
            ScoreAnnotatedSequence sequence = new ScoreAnnotatedSequence((StructureAnnotatedSequence) it.nextSequence());
            MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
            ScoreAnnotatedSequence homologSequence = mainPredictor.searchForHomolog(sequence).get(0);

            if (homologSequence != null)
            {
                String s = "Homolog found : " + homologSequence.toString();

                homologSequence = mainPredictor.overlayHomolog(sequence, homologSequence);
                System.out.println(s);
            }
            break;
        }
    }

    public void testMulitpleHomologOverlay() throws Exception
    {
        String _2TRXA = "SDKIIHLTDDSFDTDVLKADGAILVDFWAEWCGPCKMIAPILDEIADEYQGKLTVAKLNIDQNPGTAPKYGIRGIPTLLLFKNGEVAATKVGALSKGQLKEFLDANLA";
        String _2TRXA_Struct = "CCCBBBBCHHHHHHHHCCCCCBBBBBBBBCCHHHHHHHHHHHHHHHHHHCCCBBBBBBBBCCCCCCHHHCCCCCBBBBBBBBCCCCBBBBBCCCCHHHHHHHHHHHHC";
        String _1AM9A = "QSRGEKRTAHNAIEKRYRSSINDKIIELKDLVVGTEAKLNKSAVLRKAIDYIRFLQHSNQKLKQENLSLRTAVHKSKSLK";
        String _1AM9A_Struct = "CCHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHCCCCCCCCHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHCCCC";


        ScoreAnnotatedSequence sequence;
        //String seqStr =     "QSRGEKRTAHNAIEKRYRSSINDKIIELKDLVVGTEAKLNKSAVLRKAIDYQVMRAQMNQIPDYTSFNTVDEWLEAIKMGQYKESFANAGFTSFDVVSQMMMEDILRVGVTL";
        //String structStr =  "ccccccchhhhhhhhhhccchhhhhhhhhhhhhcccccccccccccccccccccccccchhhhhhhhcccchhhhhhhhcccchhhhcccchhhhhhhccccCCCCCCCCCC";
        String seqStr =     _2TRXA + _1AM9A;
        String structStr =  _2TRXA_Struct + _1AM9A_Struct;
        sequence = new ScoreAnnotatedSequence(seqStr, structStr, "TEST", new PredictionScore[seqStr.length()]);
        MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
        List<ScoreAnnotatedSequence> homologSequences = mainPredictor.searchForHomolog(sequence);

        if (homologSequences.size() > 0)
        {
            StringBuilder sb = new StringBuilder("Homologs found : ");
            for (ScoreAnnotatedSequence seq : homologSequences)
                sb.append(seq.toString() + "\n");
            System.out.println(sb);

            assert(homologSequences.size() > 1);

            ScoreAnnotatedSequence homologSequence = mainPredictor.overlayHomolog(sequence, homologSequences);
            System.out.println(homologSequence);
        }
    }

    public void findHomologyPercent() throws Exception
    {
        StructureDB db = new StructureDB("pdbstruct.fa", StructureDB.MODE.READ);
        int totalResidues = 0, totalMatches = 0, sequences = 0;
        for (SequenceIterator it = db; it.hasNext();)
        {
            StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) it.nextSequence();
//sequence = (StructureAnnotatedSequence) db.get("1YIY");
            MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
            StructureAnnotatedSequence homologSequence = mainPredictor.searchForHomolog(sequence).get(0);

            if (homologSequence != null)
            {
                try
                {
                    String s = "Homolog found : " + homologSequence.toString();

                    List<Sequence> seqList = new ArrayList(2);
                    seqList.add(sequence);
                    seqList.add(homologSequence);

                    ClustalWRunner runner = new ClustalWRunner(sequence, seqList);
                    runner.setAddToDB(false);
                    runner.setUseCache(false);
                    runner.run();
                    Alignment alignment = runner.getAlignment();
                    SymbolList alignedQuery = alignment.symbolListForLabel(sequence.getName());
                    SymbolList homolog = alignment.symbolListForLabel(homologSequence.getName());

                    assert alignedQuery.length() == homolog.length() : "Aligned query does not match aligned homolog";

                    Symbol[] newStructure = new Symbol[ alignedQuery.length() ];

// Create a gappedstructure object for the homolog
                    GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(homolog, "", "HOMOLOG", null));
                    gappedHomolog.createGappedStructure(homologSequence.getStructureAnnotation());

// And for the query sequence
                    GappedStructureAnnotatedSequence gappedQuery = new GappedStructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", "QUERY", null));
                    gappedQuery.createGappedStructure(sequence.getStructureAnnotation());

                    Symbol gapSymbol = gappedHomolog.getStructureAnnotation().getAlphabet().getGapSymbol();

                    int homologousResidues = 0;
                    for (int i = 0; i < gappedHomolog.length() && i < gappedQuery.length(); i++)
                    {
                        Symbol homologSymbol = gappedHomolog.getStructureAnnotation().symbolAt(i + 1);
                        Symbol querySymbol = gappedQuery.getStructureAnnotation().symbolAt(i + 1);
                        if (! homologSymbol.equals(AlphabetManager.getGapSymbol()))
                        {
                            // homolog is not showing a gap, use homolog structure
                            newStructure[i] = homologSymbol;
                            if (!homologSymbol.equals(gapSymbol) && !querySymbol.equals(gapSymbol))
                                homologousResidues++;
                        }
                        else if (!querySymbol.equals(gapSymbol))
                        {
                            // if the gapped query sequence isn't gapped here, use the predicted structure
                            newStructure[i] = querySymbol;
                        }
                    }
                    System.out.println(s);

                    StructureAnnotatedSequence result = null;
                    result = new StructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", sequence.getName(), null));
                    result.setStructureAnnotation(new SimpleSymbolList(newStructure, newStructure.length, new StructureTools().getStructure()));

// Go back to ungapped structure
//result = GappedStructureAnnotatedSequence.createUngappedStructure(sequence, result);
                    System.out.println(result);

                    System.out.println("Gapped Query: \n" + gappedQuery);
                    System.out.println("Gapped Homolog: \n" + gappedHomolog);
                    System.out.println(homologousResidues + " out of " + newStructure.length + " residues matched");

                    totalResidues += result.length();
                    totalMatches += homologousResidues;
                    sequences++;
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            //break;

        }

        System.out.println(totalMatches + " out of " + totalResidues + " residues matched = " + (double) totalMatches * 100 / totalResidues + "%");
        System.out.println(sequences + " out of " + db.size() + " sequences");
    }

    /**
     * Returns a list of gapped sequences by doing a BLAST search and alignment on a single sequence
     * @param sequence - the query sequence
     * @return a list of gapped sequences
     */
    public static List<GappedSequence> getHomologousSequences(Sequence sequence)
    {
        MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
        List<ScoreAnnotatedSequence> homologSequences = mainPredictor.searchForHomolog(sequence);


        Alignment alignment;
        try
        {
            XAlignRunner runner = new XAlignRunner(sequence, homologSequences);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        catch ( ProcessNotCompleteException e )
        {
            // Try ClustalW ?
            ClustalWRunner runner = new ClustalWRunner(sequence, homologSequences);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }

        List<GappedSequence> gappedHomologs = new ArrayList<GappedSequence>();
        for (Object o: alignment.getLabels())
        {
            String seqName = (String) o;

            SymbolList alignedhomolog = alignment.symbolListForLabel(seqName);

            GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(alignedhomolog, "", seqName, null));

            StructureAnnotatedSequence homolog = null;
            for(StructureAnnotatedSequence seq : homologSequences)
            {
                if ( seq.getName().equals(seqName))
                {
                    homolog = seq;
                    break;
                }
            }

            if ( homolog == null )
            {
                throw new IllegalStateException("Can't find homolog in list.");
            }

            gappedHomolog.createGappedStructure(homolog.getStructureAnnotation());

            gappedHomologs.add(gappedHomolog);
        }

        return gappedHomologs;
    }

    public void testGetHomologousSequences()
    {
        SeqSeeDB db = SeqSeeDB.getInstance();

        try
        {
            List<GappedSequence> homologousSequences = getHomologousSequences(db.getSequence(0));

            for(GappedSequence s : homologousSequences)
            {
                System.out.println(s);
            }
        }
        catch (BioException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }


    public void findHomologScores() throws Exception
    {

        class HomologScore
        {
            double score;
            double percentHomology;

            public HomologScore(double score, double percentHomology)
            {
                this.score = score;
                this.percentHomology = percentHomology;
            }

            public String toString()
            {
                return "score=" + score +
                        ", percentHomology=" + percentHomology;
            }
        }
        StructureDB db = new StructureDB("pdbstruct.fa", StructureDB.MODE.READ);
        HashMap<String, HomologScore> scoreMap = new HashMap<String, HomologScore>();
        for (SequenceIterator it = db; it.hasNext();)
        {
            ScoreAnnotatedSequence sequence = new ScoreAnnotatedSequence((StructureAnnotatedSequence) it.nextSequence());
//            sequence = new ScoreAnnotatedSequence( (StructureAnnotatedSequence) db.get("1TRW") );
            MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
            ScoreAnnotatedSequence homologSequence = mainPredictor.searchForHomolog(sequence).get(0);

            if (homologSequence != null)
            {
                try
                {
                    String s = "Homolog found : " + homologSequence.getName() + homologSequence.toString();

                    List<Sequence> seqList = new ArrayList(2);
                    seqList.add(sequence);
                    seqList.add(homologSequence);

                    Alignment alignment;
                    try
                    {
                        XAlignRunner runner = new XAlignRunner(sequence, seqList);
                        runner.setAddToDB(false);
                        runner.setUseCache(false);
                        runner.run();
                        alignment = runner.getAlignment();
                    }
                    catch ( ProcessNotCompleteException e )
                    {
                        // Try ClustalW ?
                        ClustalWRunner runner = new ClustalWRunner(sequence, seqList);
                        runner.setAddToDB(false);
                        runner.setUseCache(false);
                        runner.run();
                        alignment = runner.getAlignment();
                    }

                    SymbolList alignedQuery = alignment.symbolListForLabel(sequence.getName());
                    SymbolList homolog = alignment.symbolListForLabel(homologSequence.getName());

                    assert alignedQuery.length() == homolog.length() : "Aligned query does not match aligned homolog";

                    Symbol[] newStructure = new Symbol[ alignedQuery.length() ];

// Create a gappedstructure object for the homolog
                    GappedStructureAnnotatedSequence gappedHomolog = new GappedStructureAnnotatedSequence(new SimpleSequence(homolog, "", "HOMOLOG", null));
                    gappedHomolog.createGappedStructure(homologSequence.getStructureAnnotation());

// And for the query sequence
                    GappedStructureAnnotatedSequence gappedQuery = new GappedStructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", "QUERY", null));
                    gappedQuery.createGappedStructure(sequence.getStructureAnnotation());

                    Symbol gapSymbol = gappedHomolog.getStructureAnnotation().getAlphabet().getGapSymbol();

                    int homologousResidues = 0;
                    for (int i = 0; i < gappedHomolog.length() && i < gappedQuery.length(); i++)
                    {
                        Symbol homologSymbol = gappedHomolog.getStructureAnnotation().symbolAt(i + 1);
                        Symbol querySymbol = gappedQuery.getStructureAnnotation().symbolAt(i + 1);
                        if (! homologSymbol.equals(AlphabetManager.getGapSymbol()))
                        {
                            // homolog is not showing a gap, use homolog structure
                            newStructure[i] = homologSymbol;
                            if (!homologSymbol.equals(gapSymbol) && !querySymbol.equals(gapSymbol))
                            {
                                homologousResidues++;
                            }
                        }
                        else if (!querySymbol.equals(gapSymbol))
                        {
                            // if the gapped query sequence isn't gapped here, use the predicted structure
                            newStructure[i] = querySymbol;
                        }
                    }
                    System.out.println(s);

                    StructureAnnotatedSequence result = null;
                    result = new StructureAnnotatedSequence(new SimpleSequence(alignedQuery, "", sequence.getName(), null));
                    result.setStructureAnnotation(new SimpleSymbolList(newStructure, newStructure.length, new StructureTools().getStructure()));

// Go back to ungapped structure
//result = GappedStructureAnnotatedSequence.createUngappedStructure(sequence, result);
//System.out.println(result);

//System.out.println("Gapped Query: \n" + gappedQuery);
//System.out.println("Gapped Homolog: \n" + gappedHomolog);
                    System.out.println(homologousResidues + " out of " + newStructure.length + " residues matched");

                    result = GappedStructureAnnotatedSequence.createUngappedStructure(sequence, result);

                    System.out.println("Result: \n" + result);
                    System.out.println("Query Sequence: \n" + sequence);

                    double q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), result.getStructureAnnotation());
                    scoreMap.put(sequence.getName(), new HomologScore(q3, homologousResidues * 100 / (double) newStructure.length));
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        PrintWriter p = new PrintWriter("homologScores.out");
        for (Iterator<String> it = scoreMap.keySet().iterator(); it.hasNext();)
        {
            String s = it.next();
            p.println(s + " " + scoreMap.get(s));
        }
        p.close();

        System.out.println("Sequences done: " + db.size());
    }

    public void findOptimalHomologSetting() throws BioException, IOException
    {
        StructureDB db = new StructureDB("pdbstruct.fa", StructureDB.MODE.READ);
        double evalue = 1;
        Score optimalScore = new Score();
        double optimalEvalue = evalue, optimalHomologEvalue = evalue;
        int numDone = 0, numHadHomolog = 0, optimalHadHomolog = 0;

        while (evalue > 1e-20)
        {
            double sov = 0;
            double q3 = 0;
            numDone = 0;
            numHadHomolog = 0;
            db.reset();
            for (SequenceIterator it = db; it.hasNext();)
            {
                ScoreAnnotatedSequence sequence = new ScoreAnnotatedSequence(it.nextSequence());
                MainPredictor mainPredictor = new MainPredictor(new RuntimeCatalog());
                mainPredictor.useStoredNN = true;
                try
                {
                    ScoreAnnotatedSequence predictedSequence = mainPredictor.predict(sequence);
                    ScoreAnnotatedSequence homologSequence = mainPredictor.searchForHomolog(sequence, evalue).get(0);

                    if (homologSequence != null)
                    {
                        try
                        {
                            homologSequence = mainPredictor.overlayHomolog(sequence, homologSequence);

                            sov += ProteusTools.sovScore(sequence.getStructureAnnotation(), homologSequence.getStructureAnnotation());
                            q3 += ProteusTools.q3Score(sequence.getStructureAnnotation(), homologSequence.getStructureAnnotation());
                            numHadHomolog++;
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                    }
                    else
                    {
                        sov += ProteusTools.sovScore(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());
                        q3 += ProteusTools.q3Score(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());
                    }
                    numDone++;
//if ( numDone > 10 ) break;
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }

            if (numHadHomolog > optimalHadHomolog)
            {
                optimalHadHomolog = numHadHomolog;
                optimalHomologEvalue = evalue;
            }

            System.out.println("Evalue (" + evalue + ") with q3 = " + q3 + " sov = " + sov);
            System.out.println(numHadHomolog + " had homologs");

            if (q3 > optimalScore.q3)
            {
                optimalEvalue = evalue;
                optimalScore.q3 = q3;
                optimalScore.sov = sov;
                Score tmpScore = optimalScore.clone();
                tmpScore.q3 /= (double) numDone;
                tmpScore.sov /= (double) numDone;
                System.out.println("Best Evalue (" + evalue + ") with " + tmpScore);
                System.out.println(optimalHadHomolog + " had homologs");
            }
            evalue /= 5;
        }
        System.out.println(optimalScore + " with " + numDone + " sequences");
        System.out.println(optimalHadHomolog + " had homologs");
        optimalScore.q3 /= (double) numDone;
        optimalScore.sov /= (double) numDone;
        System.out.println("Evalue of " + optimalEvalue + " had a score of " + optimalScore);
    }

    public void doPredictions()
    {
        // This is a one-off

        /*
        TODO Only out becuase of DB changes and it was too much work for one-off 

        MainPredictor mainPredictor = new MainPredictor(new ProteusLogger());
        mainPredictor.useStoredNN = true;

        ResultSet resultSet = DBUtils.executeGenericQuery("SELECT SEQUENCE_ID FROM FLEXIBILITY GROUP BY SEQUENCE_ID");

        try
        {
            while(resultSet.next())
            {
                int ID = resultSet.getInt(1);

                Sequence sequence = DBUtils.getSequenceForID(ID);

                if ( PredictionDBFactory.getDB().get(sequence, "FINAL") == null )
                {
                    ScoreAnnotatedSequence prediction = mainPredictor.predict(sequence);

                    PredictionDBFactory.getDB().add(sequence, "FINAL", prediction);
                }
            }
        }
        catch (SQLException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        */
    }
}
