package ca.ualberta.proteus.sequence.test;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.BioException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.SimpleGappedSymbolList;
import org.biojava.bio.symbol.GappedSymbolList;
import org.biojava.bio.proteomics.StructureTools;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Apr 1, 2006
 * Time: 10:47:35 PM
 * To change this template use File | Settings | File Templates.
 */
public class StructureTest extends TestCase
{

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public static Test suite()
    {
        return new TestSuite(StructureTest.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void testStructureSetSymbol() throws BioException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();
        StructureAnnotatedSequence sequence = (StructureAnnotatedSequence) db.getSequence(0);

        sequence.setStructureAt(1, new StructureTools().getC());
        assertEquals(sequence.getStructureAnnotation().symbolAt(1), new StructureTools().getC());

        sequence.setStructureAt(5, new StructureTools().getE());
        assertEquals(sequence.getStructureAnnotation().symbolAt(5), new StructureTools().getE());

        sequence.setStructureAt(10, new StructureTools().getH());
        assertEquals(sequence.getStructureAnnotation().symbolAt(10), new StructureTools().getH());
    }

    public void testGappedStructureAnnotatedSequence()
    {
        String query =
                "SAWGPAATIAARQSATGTKTDTPIQ--KVPQSISVVTAE--EMALHQPKSVKEALSYTPGVSVGTRGASNTYDHLIIRGFA" +
                        "AEGQSQNNY-LNGLK-LQ--------GNFYNDAVIDPY---MLERAEIMRGPVSVLYGKSSPGGLLNMVSKRP" +
                        "TTEPLKEVQFKAGTDSLFQTGFDFSDSLDDDGVYSYRLTGLARSANAQQKGSEEQRYAI---APAFTWRPDDK" +
                        "TNFTF-LSYFQNEPET-----GYYGWLPKEGTVEPLPNGKRLPTDFNEGAKNNTYSRNEKMVGYSFDHEFNDT" +
                        "FTVRQNLRFAENKTSQNSVYGYGVCSDPANAYSKQCAALAP--ADKGHYLARKYVVDDEKLQNFSVDTQLQSK" +
                        "FATGDIDHTLLTGVDFMRMRNDINAWFGYDDSVPLLNLYN-P-----------VNT-DFDFNAKDPANSGP--" +
                        "----YRI--LNKQKQTGV-----YVQDQAQWDKVL-VTLGGR-YDWADQESLNRVAGTTDK-RDDKQFTWRGG" +
                        "VNYLFDNGVTPYFSYSESFEPS--SQVGK---DGN-IFAPSKGKQYEVGVKYVPEDRPIVVTGAVYNLTKTNN" +
                        "LMADP--------E-----GSFFSVEGGEIRARGVEIEAKRPLSA------SVNVVGSYTYTDAEYTTDTTYK" +
                        "GNT--PAQVPKHMASLWADYTFFDGPLSGLTLGTGG---RYTGSS---YGDPAN---SFK---------VGSY" +
                        "TVVDALVRYDLARVGMAGSNVALHVNNLFDREYVASCF--NTYGCFWGAERQVVATATFRF";
        String structure =
                "ccchhhhhhhhceeeccccceeecc--ccceeeeeeehh--hhhhhcccccccccccccceeeccccccccccceeeccce" +
                        "eecccceee-eccce-ee--------cccccccccccc---ceeeeeeeecccccccceeeecceeeeeeecc" +
                        "cccbbbbbbbbbbbccbbbbbbbbbbbcccccbbbbbbbbbbbbbbbccccbbbbbbbb---bbbbbbbcccb" +
                        "bbbbb-bbbbbbbbbb-----cccceeeeccccccccccceeecccccccccbbbbbbbbbbbbbbbbbcccb" +
                        "bbbbbbbbbbbbbbbbbbbbbbbbccccccccchhhhcccc--ccccbbbbbbbbbbbbbbbbbbbbbbbbbb" +
                        "bbcccbbbbbbbbbbbbbbbbbbbbbbbbccccceeechh-hhchhhhhccccccc-ccccccccccbbbb--" +
                        "----bbb--bbbbbbbbb-----bbbbbbbbccbb-bbbbbb-bbbbbbbbbbccccbbbb-bbbbbbbbbbb" +
                        "bbbbbcccbbbbbbbbbbbbbc--ccccc---ccc-cccbbbbbbbbbbbbbbccccbbbbbbbbbbbbbbbb" +
                        "bbbcc--------c-----cccbbbbbbbbbbbbbbbbbbbbccc------cbbbbbbbbbbbbbbbbccccc" +
                        "cce--eecccbbbbbbbbbbbbcccccccbbbbbbb---bbbeee---eecccc---cee---------eebb" +
                        "bbbbbbbbbbccccccccbbbbbbbbccccceeeeecc--ccceeeeecccbbbbbbbbbb";

        String ungappedStruct =
                "ccchhhhhhhhceeeccccceeeccccceeeeeeehhhhhhhcccccccccccccceeeccccccccccceeeccceeecc" +
                        "cceeeeccceeeccccccccccccceeeeeeeecccccccceeeecceeeeeeecccccbbbbbbbbbbbccb" +
                        "bbbbbbbbbbcccccbbbbbbbbbbbbbbbccccbbbbbbbbbbbbbbbcccbbbbbbbbbbbbbbbbcccce" +
                        "eeeccccccccccceeecccccccccbbbbbbbbbbbbbbbbbcccbbbbbbbbbbbbbbbbbbbbbbbbbcc" +
                        "ccccccchhhhccccccccbbbbbbbbbbbbbbbbbbbbbbbbbbbbcccbbbbbbbbbbbbbbbbbbbbbbb" +
                        "bccccceeechhhhchhhhhcccccccccccccccccbbbbbbbbbbbbbbbbbbbbbbbbccbbbbbbbbbb" +
                        "bbbbbbbbccccbbbbbbbbbbbbbbbbbbbbcccbbbbbbbbbbbbbccccccccccccbbbbbbbbbbbbb" +
                        "bccccbbbbbbbbbbbbbbbbbbbccccccbbbbbbbbbbbbbbbbbbbbccccbbbbbbbbbbbbbbbbccc" +
                        "cccceeecccbbbbbbbbbbbbcccccccbbbbbbbbbbeeeeeccccceeeebbbbbbbbbbbbcccccccc" +
                        "bbbbbbbbccccceeeeeccccceeeeecccbbbbbbbbbb";

        String ungappedQuery =
                "SAWGPAATIAARQSATGTKTDTPIQKVPQSISVVTAEEMALHQPKSVKEALSYTPGVSVGTRGASNTYDHLIIRGFAAEGQ" +
                        "SQNNYLNGLKLQGNFYNDAVIDPYMLERAEIMRGPVSVLYGKSSPGGLLNMVSKRPTTEPLKEVQFKAGTDSL" +
                        "FQTGFDFSDSLDDDGVYSYRLTGLARSANAQQKGSEEQRYAIAPAFTWRPDDKTNFTFLSYFQNEPETGYYGW" +
                        "LPKEGTVEPLPNGKRLPTDFNEGAKNNTYSRNEKMVGYSFDHEFNDTFTVRQNLRFAENKTSQNSVYGYGVCS" +
                        "DPANAYSKQCAALAPADKGHYLARKYVVDDEKLQNFSVDTQLQSKFATGDIDHTLLTGVDFMRMRNDINAWFG" +
                        "YDDSVPLLNLYNPVNTDFDFNAKDPANSGPYRILNKQKQTGVYVQDQAQWDKVLVTLGGRYDWADQESLNRVA" +
                        "GTTDKRDDKQFTWRGGVNYLFDNGVTPYFSYSESFEPSSQVGKDGNIFAPSKGKQYEVGVKYVPEDRPIVVTG" +
                        "AVYNLTKTNNLMADPEGSFFSVEGGEIRARGVEIEAKRPLSASVNVVGSYTYTDAEYTTDTTYKGNTPAQVPK" +
                        "HMASLWADYTFFDGPLSGLTLGTGGRYTGSSYGDPANSFKVGSYTVVDALVRYDLARVGMAGSNVALHVNNLF" +
                        "DREYVASCFNTYGCFWGAERQVVATATFRF";
    }

    public void test()
    {
        String STRUCTURE = "CHE---CHE";

        SimpleSymbolList list = ProteusTools.createStructure(STRUCTURE);
        System.out.println("list = " + list.seqString());

        SimpleGappedSymbolList simpleGappedSymbolList = new SimpleGappedSymbolList(list);
        System.out.println("simpleGappedSymbolList = " + simpleGappedSymbolList.seqString());

        GappedSymbolList gappedSymbolList = ProteusTools.createGappedStructure(STRUCTURE);
        System.out.println("gappedSymbolList = " + gappedSymbolList.seqString());

        assertEquals(STRUCTURE.toLowerCase(), gappedSymbolList.seqString().toLowerCase());
    }
}