package ca.ualberta.proteus.util;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.program.ssbind.SeqSimilarityAdapter;
import org.biojava.bio.program.ssbind.BlastLikeSearchBuilder;
import org.biojava.bio.search.SearchContentHandler;
import org.biojava.bio.search.SeqSimilaritySearchResult;
import org.biojava.bio.search.SeqSimilaritySearchHit;
import org.biojava.bio.search.SeqSimilaritySearchSubHit;
import org.biojava.bio.seq.db.DummySequenceDB;
import org.biojava.bio.seq.db.DummySequenceDBInstallation;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.Annotation;
import org.biojava.bio.SimpleAnnotation;
import org.biojava.utils.ChangeVetoException;

import java.io.IOException;
import java.io.Reader;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Feb 22, 2006
 * Time: 8:26:11 PM
 * To change this template use File | Settings | File Templates.
 */
public class BlastParser
{
    /**
     * Does the grunt work of parsing the BLAST file, whether it's a text output, or XML output,
     * it should be able to parse it with the appropriate XMLReader
     * @param oParser
     * @param outputReader - a reader to read the output from BLAST
     * @param seqList - a list of sequences that will be filled when BLAST is parsed
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws org.biojava.bio.symbol.IllegalSymbolException
     * @throws org.biojava.utils.ChangeVetoException
     */
    public void parseBlastResult(XMLReader oParser, Reader outputReader, List<Sequence> seqList)
            throws IOException, SAXException, IllegalSymbolException, ChangeVetoException
    {
        //make the SAX event adapter that will pass events to a Handler.
        SeqSimilarityAdapter adapter = new SeqSimilarityAdapter();

        //set the parsers SAX event adapter
        oParser.setContentHandler(adapter);

        //The list to hold the SeqSimilaritySearchResults
        List results = new ArrayList();

        //create the SearchContentHandler that will build SeqSimilaritySearchResults
        //in the results List
        SearchContentHandler builder = new BlastLikeSearchBuilder(results,
                new DummySequenceDB("queries"), new DummySequenceDBInstallation());

        //register builder with adapter
        adapter.setSearchContentHandler(builder);

        // parse it
        oParser.parse(new InputSource(outputReader));

        //output some blast details
        for (Iterator i = results.iterator(); i.hasNext();)
        {
            SeqSimilaritySearchResult result =
                    (SeqSimilaritySearchResult) i.next();

//            Annotation anno = result.getAnnotation();

            //list the hits
            for (Iterator k = result.getHits().iterator(); k.hasNext();)
            {
                SeqSimilaritySearchHit hit =
                        (SeqSimilaritySearchHit) k.next();

                // list the sequences
                for (Iterator it = hit.getSubHits().iterator(); it.hasNext();)
                {
                    SeqSimilaritySearchSubHit subHit = (SeqSimilaritySearchSubHit) it.next();

                    Alignment alignment = subHit.getAlignment();

                    for (Iterator alignmentIt = alignment.getLabels().iterator(); alignmentIt.hasNext();)
                    {

                        Object label = alignmentIt.next();

                        // ignore query sequences
                        if (label.equals("Query"))
                            continue;

                        // add the sequence with its annotation to the list
                        SymbolList symList = subHit.getAlignment().symbolListForLabel(label);

                        // Get rid of the gaps that BLAST produces
                        String symStr = symList.seqString();
                        symStr = symStr.replace("-", "");
                        symList = ProteinTools.createProtein(symStr);

                        Annotation annotation = result.getAnnotation();

                        // Description should be like gi|107890|pir|S4918 twitchin-like protein
                        // we don't want that
                        // but it could also be 1K0BB
                        String ID = (String) hit.getAnnotation().getProperty("subjectDescription");
                        if ( ID.length() > 9 || ID.length() == 0 ) {
                            // this should be like gnl|BL_ORD_ID|801283
                            ID = hit.getSubjectID();
                        }

                        if ( ID.contains("gi|") ) {
                             String[] tmp = ID.split("\\|");
                            ID = tmp[1];
                        }
                        else if ( ID.indexOf('|') != -1 ) {
                            String tmp = ID.substring(ID.lastIndexOf('|')+1, ID.length());
                            if ( tmp.length() > 0 )
                                ID = tmp;
                        }

                        if ( ID.length() > 8 )
                            ID = ID.substring(0, 7);

                        Sequence seq = new SimpleSequence(symList, "", ID, new SimpleAnnotation(annotation));
                        seq.getAnnotation().setProperty("Score", subHit.getScore());
                        seq.getAnnotation().setProperty("e-value", subHit.getEValue());
                        seq.getAnnotation().setProperty("subjectDescription", hit.getAnnotation().getProperty("subjectDescription") );
                        seqList.add(seq);
                    }

                    //     return alignment;
                }
            }

      //      System.out.println("\n");
        }
    }
}
