package ca.ualberta.proteus.util;

import org.biojava.bio.seq.Sequence;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Feb 10, 2006
 * Time: 10:43:30 PM
 * To change this template use File | Settings | File Templates.
 */
public class BlastSettings
{
    public static final int MAX_PENALTY = 32767;

    /**
     * Type for running psiblast or BlastP
     */
    public enum BLASTTYPE
    {
        NONE, PSIBLAST, BLASTP, PROTEUS_BLASTPARSER_BLASTP;
    }

    /**
     *  Choose which type of output you want
     * BLUNT_ENDS = m6 option
     * XML = m7 option
     */
    public enum ALIGNMENT_VIEW
    {
        BLUNT_ENDS, XML
    }

    private BLASTTYPE blasttype;
    private ALIGNMENT_VIEW alignmentView = ALIGNMENT_VIEW.XML;
    private boolean keepOutput = false;
    private String database;
    private double overriddenEvalue = -1;
    private boolean parse = true;
    private Sequence sequence;
    private String matrix = "BLOSUM62";
    private int gapPenalty = -1;
    private int gapExtensionPenalty = -1;
    private int maxHits = -1;


    public void setBlasttype(BLASTTYPE blasttype)
    {
        this.blasttype = blasttype;
    }

    public void setOverriddenEvalue(double overriddenEvalue)
    {
        this.overriddenEvalue = overriddenEvalue;
    }

    public void setSequence(Sequence sequence)
    {
        this.sequence = sequence;
    }

    public boolean isKeepOutput()
    {
        return keepOutput;
    }

    public String getDatabase()
    {
        return database;
    }

    public double getOverriddenEvalue()
    {
        return overriddenEvalue;
    }

    public boolean isParse()
    {
        return parse;
    }

    public Sequence getSequence()
    {
        return sequence;
    }

    public void setDatabase(String database)
    {
        this.database = database;
    }

    public ALIGNMENT_VIEW getAlignmentView()
    {
        return alignmentView;
    }

    public void setAlignmentView(ALIGNMENT_VIEW alignmentView)
    {
        this.alignmentView = alignmentView;
    }


    /**
     * Set this to override the evalue stored in the params file
     * @param evalue
     */
    public void setEvalue(double evalue)
    {
        this.overriddenEvalue = evalue;
    }

    public Sequence getQuerySequence()
    {
        return this.sequence;
    }


    public boolean getKeepOutput()
    {
        return keepOutput;
    }

    public void setKeepOutput(boolean b)
    {
        throw new IllegalStateException("Not supported since we took out the memory retention of BLAST files");

        //this.keepOutput = b;
    }

    public BLASTTYPE getBlasttype()
    {
        return blasttype;
    }

    public void setBlastType(BLASTTYPE blasttype)
    {
        this.blasttype = blasttype;
    }

     /**
     * Set whether you want to have this BlastRunner parse the result into
     * a sequence list
     * @param b = true if it is to parse it
     */
    public void setParse(boolean b)
    {
        this.parse = b;
    }

    /**
     * The matrix to run with
     * default: BLOSUM62
     * @return the matrix
     */
    public String getMatrix()
    {
        return matrix;
    }

    public void setMatrix(String matrix)
    {
        this.matrix = matrix;
    }

    public int getGapPenalty()
    {
        return gapPenalty;
    }

    /**
     * Sets the gap  penalty.
     *
     * supported values are:
     * <pre>
     *  Gap, Extension
        32767, 32767
        7, 2
        6, 2
        5, 2
        10, 1
        9, 1
        8, 1
        </pre>
     * @param gapPenalty Must be <=10 or MAX_PENALTY
     */
    public void setGapPenalty(int gapPenalty)
    {
        if ( gapPenalty != MAX_PENALTY && gapPenalty > 10 )
        {
            throw new IllegalArgumentException("Invalid gap penalty.  Run \"blastall\" for more information");
        }
        this.gapPenalty = gapPenalty;
    }

    public int getGapExtensionPenalty()
    {
        return gapExtensionPenalty;
    }

    /**
     * Sets the gap extension penalty.
     *
     * supported values are:
     * <pre>
     *  Gap, Extension
        32767, 32767
        7, 2
        6, 2
        5, 2
        10, 1
        9, 1
        8, 1
        </pre>
     * @param gapExtensionPenalty Must be 1, 2, or MAX_PENALTY
     */
    public void setGapExtensionPenalty(int gapExtensionPenalty)
    {
        if ( gapExtensionPenalty != MAX_PENALTY && gapExtensionPenalty > 2 )
        {
            throw new IllegalArgumentException("Invalid gap extension penalty.  Run \"blastall\" for more information");
        }
        this.gapExtensionPenalty = gapExtensionPenalty;
    }

    public int getMaxHits()
    {
        return maxHits;
    }

    public void setMaxHits(int maxHits)
    {
        this.maxHits = maxHits;
    }
}
