package ca.ualberta.proteus.util;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.biojava.bio.program.sax.ClustalWAlignmentSAXParser;
import org.biojava.bio.symbol.Alignment;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Jan 29, 2008
 * Time: 11:55:52 PM
 * To change this template use File | Settings | File Templates.
 */
public class ClustalWParser
{
    private Alignment alignment = null;

    public void parse(String alignmentFile) throws IOException, SAXException
    {
        parse(new FileInputStream(alignmentFile));
    }

    public void parse(InputStream in) throws IOException, SAXException
    {

        /**
         * Create a SAX Parser that takes the native output
         * from blast-like bioinformatics software.
         */
        XMLReader oParser = new ClustalWAlignmentSAXParser();

        /**
         * Dynamically change configuration of the parser
         * in regard of Namespace support. Here,
         * the xml.org/sax/features/namespaces feature is simply "reset"
         * to its default value for SAX2.
         * The xml.org/sax/features/namespaces-prefixes feature is
         * also set to true.  This is to ensure that xmlns attributes
         * are reported by the parser. These are required because we want
         * to configure the XMLEmitter to output qualified names (see below).
         */
        try
        {
            oParser.setFeature("http://xml.org/sax/features/namespaces", true);
            oParser.setFeature("http://xml.org/sax/features/namespace-prefixes",
                    true);

        }
        catch (Exception e)
        {
            //If an illegal conmbination of features is chosen,
            //roll back to default settings. Output a warning,
            //even though this might mess up the output...
            System.out.println("WARNING: ignoring attempt to set illegal " +
                    "combination of parser features");
            System.out.println(e);
        }

        //set the parsers SAX event adapter
        ClustalWAdapter adapter = new ClustalWAdapter();
        oParser.setContentHandler(adapter);

        // parse it
        oParser.parse(new InputSource(in));

        this.alignment = adapter.getAlignment();
    }

    public Alignment getAlignment()
    {
        return alignment;
    }
}
