package ca.ualberta.proteus.util;

import java.io.IOException;
import java.io.File;
import java.util.Map;

/*
 * Copyright 2001-2007 Geert Bevin <gbevin[remove] at uwyn dot com>
 * Distributed under the terms of either:
 * - the common development and distribution license (CDDL), v1.0; or
 * - the GNU Lesser General Public License, v2.1 or later
 * $Id: ExecUtils.java,v 1.2 2007/02/01 03:02:33 montgome Exp $
 */

public abstract class ExecUtils
{
	public static Process exec(String command)
	throws IOException, InterruptedException
	{
		return exec(command, null);
	}

	public static Process exec(String command, String[] envp)
	throws IOException, InterruptedException
	{
		Process process = null;
		if (null == envp)
		{
			process = Runtime.getRuntime().exec(command);
		}
		else
		{
			process = Runtime.getRuntime().exec(command, envp);
		}

		try
		{
			process.waitFor();
		}
		catch (InterruptedException e)
		{
			throw e;
		}

		return process;
	}

	public static Process exec(String commands[], String[] envp)
	throws IOException, InterruptedException
	{
		Process process = null;
		if (null == envp)
		{
			process = Runtime.getRuntime().exec(commands);
		}
		else
		{
			process = Runtime.getRuntime().exec(commands, envp);
		}

		try
		{
			process.waitFor();
		}
		catch (InterruptedException e)
		{
			throw e;
		}

		return process;
	}

	public static Process exec(String command, File dir, long interval, long timeout)
	throws IOException, InterruptedException
	{
		return exec(command, null, dir, interval, timeout);
	}

    public static Process exec(String command, long interval, long timeout)
	throws IOException, InterruptedException
	{
		return exec(command, null, null, interval, timeout);
	}

    public static Process exec(String[] commands, long interval, long timeout)
	throws IOException, InterruptedException
	{
		return exec(commands, null, interval, timeout);
	}

	public static synchronized Process exec(String command, String[] envp, File dir, long interval, long timeout)
	throws IOException, InterruptedException
	{
		Process	process = null;
		if (null == envp && dir == null )
		{
			process = Runtime.getRuntime().exec(command);
		}
		else if ( dir == null )
		{
			process = Runtime.getRuntime().exec(command, envp);
		}
        else
        {
            process = Runtime.getRuntime().exec(command, envp, dir);
        }

		return processTimeout(process, interval,timeout);
	}

	public static synchronized Process exec(String[] commands, String[] envp, long interval, long timeout)
	throws IOException, InterruptedException
	{
		Process	process = null;
		if (null == envp)
		{
			process = Runtime.getRuntime().exec(commands);
		}
		else
		{
			process = Runtime.getRuntime().exec(commands, envp);
		}

		return processTimeout(process, interval,timeout);
	}

	private static synchronized Process processTimeout(Process process, long interval, long timeout)
	throws InterruptedException
	{
		long	time_waiting = 0;
		boolean	process_finished = false;

		while (time_waiting < timeout &&
			   !process_finished)
		{
			process_finished = true;
			try
			{
				Thread.sleep(interval);
			}
			catch (InterruptedException e)
			{
				e.fillInStackTrace();
				throw e;
			}

			try
			{
				process.exitValue();
			}
			catch (IllegalThreadStateException e)
			{
				// process hasn't finished yet
				process_finished = false;
			}
			time_waiting += interval;
		}

		if (process_finished)
		{
			return process;
		}
		else
		{
			process.destroy();
			return null;
		}
	}

	public static void outputSystemProperties()
	{
		Map.Entry property = null;
		for (Object element : System.getProperties().entrySet())
		{
			property = (Map.Entry)element;
			System.out.println(property.getKey()+"="+property.getValue());
		}
	}
}