package ca.ualberta.proteus.util;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.io.StructureDB;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.utils.ChangeVetoException;
import org.biojava.utils.io.CountedBufferedReader;
import org.biojava.utils.io.RandomAccessReader;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Mar 30, 2006
 * Time: 10:32:31 PM
 * To change this template use File | Settings | File Templates.
 */
public class MyBlastParser
{
    enum State { HEADER, SUMMARY, ALIGNMENT, DONE }

    //private static HashMap<State, State> stateTransitions = new HashMap<State, State>();
    private int alignmentStart = -1;
    private int alignmentEnd = -1;
    private final static int STATIC_READ_AHEAD_LIMIT = 20000000;

    /*
    static
    {
        stateTransitions.put(State.HEADER, State.SUMMARY);
        stateTransitions.put(State.HEADER, State.SUMMARY);
        stateTransitions.put(State.HEADER, State.SUMMARY);
        stateTransitions.put(State.HEADER, State.SUMMARY);
        stateTransitions.put(State.HEADER, State.SUMMARY);
    }
    */

    /**
     * Parses either BlastP or PSI_BLAST in the -m6 format
     *
     * CAUTION: Ensure that the -v value on PSI-BLAST equals or exceeds the -b value.
     *   That is the summary lines vs. alignments.  Built on BLAST 2.2.13
     *
     * <p>
     *  How it works:
     *  This parser only parses the last iteration of the PSI-BLAST search, where supposedly all the
     *  sequences found are output.
     *  In the first pass, we mark the reader where the last iteration output begins.
     *  Then we go into the state machine, starting in HEADER.  When we see the declaration for the
     *  summary of the one-liners ("Sequences producing significant alignments"), we enter the SUMMARY state.
     *  Then we run until a blank line, parsing the summary lines along the way. <br>
     *   IF it's PSI-BLAST, we need to be careful to include the "Sequences not found previously or not previously below threshold"
     *  sequences. <br>
     *   When QUERY is found, we revert to the last line (it is marked along the way), and move into the ALIGNMENT state.
     *
     *   We then parse each alignment.
     * </p>
     *
     *
     *
     * @param querySequence The query sequence
     * @param reader The reader used to read in the BLAST output file
     * @param seqList Out param, where the aligned sequences will be placed
     * @throws IllegalSymbolException
     * @throws IOException
     * @throws ChangeVetoException
     * @throws SAXException
     */
/*
    public void parseBlastResult(Sequence querySequence, Reader reader, List<GappedSequence> seqList) throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        if ( seqList == null )
        {
            throw new IllegalArgumentException("Can't pass in null sequence list - this needs to be populated with the results");
        }


        String line;
        Map<String, String> sequences = new HashMap<String, String>();
        Map<String, String> blockSequences = new HashMap<String, String>();
        Map<String, String> blastIDTOPDBID = new HashMap<String, String>();
        Map<String, Double> evalues = new HashMap<String, Double>();
        Map<String, Integer> scores = new HashMap<String, Integer>();

        List<String> IDs = new ArrayList<String>();
        IDs.add("QUERY");

        State state = State.HEADER;
        int sequenceIndex = 0;
        int lastIteration = -1;
        int READ_AHEAD_LIMIT = (int) (Runtime.getRuntime().totalMemory() / 4);

        if ( READ_AHEAD_LIMIT > STATIC_READ_AHEAD_LIMIT )
        {
            READ_AHEAD_LIMIT = STATIC_READ_AHEAD_LIMIT ;
        }

        {
            BufferedReader bf = new BufferedReader(reader);
            bf.mark(READ_AHEAD_LIMIT);

            /** First pass, find the last iteration
            while( (line = bf.readLine()) != null )
            {
                if ( line.contains("Results from round") )
                {
                    String tmp = line.replaceFirst(".*Results from round (\\d+)", "$1");
                    lastIteration = Integer.parseInt(tmp);

                    // mark to come back to it
                    bf.mark(READ_AHEAD_LIMIT);
                }
                else if ( line.contains("Matrix:") )
                {
                   // end of the data
                    break;
                }
            }

            try
            {
                bf.reset();
            }
            catch (IOException e)
            {
                System.err.println("The BLAST output file READ_AHEAD_LIMIT was bigger than the alotted size! (" + READ_AHEAD_LIMIT / 1024 / 1024 + " M)");
                e.printStackTrace();
            }

            //BufferedReader bf = new BufferedReader(reader);

            READFILE_LOOP:
            while( (line = bf.readLine()) != null )
            {
                switch(state)
                {
                    case HEADER:
                        if ( line.contains("No hits found"))
                        {
                            state = State.DONE;
                            break READFILE_LOOP;
                        }

                        if ( lastIteration == -1 )
                        {
                            // BLASTP

                            if ( line.contains("Sequences producing significant alignments"))
                            {
                                state = State.SUMMARY;
                            }
                        }
                        else
                        {
                            /*
                             * PSI-BLAST
                             * We have marked the reader at the above level, right after the last iteration

                            if ( line.contains("Sequences used in model and found again:") )
                            {
                                state = State.SUMMARY;
                                break;
                            }

                            /* PSI-BLAST
                             * We want to skip to the last iteration
                             */
                          /*  if ( line.contains("Results from round " + lastIteration))
                            {
                                state = State.SUMMARY;

                                // skip to summary
                                while ( (line = bf.readLine()) != null && ! line.contains("Sequences used in model and found again:") )
                                {
                                    // noop;
                                }
                            }


                        }
                        break;
                    case SUMMARY:
                        if ( line.contains("Sequences not found previously or not previously below threshold:")
                            || line.contains("CONVERGED!") )
                        {
                            // skip this line
                            // but mark it so we can come back to it on the next pass if need be
                            bf.mark(READ_AHEAD_LIMIT);
                            continue;
                        }
                        else if ( line.trim().length() > 0 )
                        {
                            // not a blank line

                            if ( lastIteration != -1 )
                            {
                                if ( line.contains("QUERY"))
                                {
                                    // we are done SUMMARY if the QUERY line comes up (QUERY is only in the alignments, not in summary)
                                    bf.reset();

                                    state = State.ALIGNMENT;

                                    for(String ID: IDs)
                                    {
                                        sequences.put(ID, "");
                                    }

                                    continue;
                                }
                            }

                            // Parse the one-line summary
                            String[] split = line.split("\\s{2,}");
                            String ID = line.substring(0, 60).trim();
                            String bits = split[split.length - 2];    // the second-last column
                            String evalue = split[split.length - 1];  // the last column

                            if ( evalue.startsWith("e"))
                            {
                                evalue = "1" + evalue;
                            }

                            IDs.add(ID);
                            evalues.put(ID, Double.valueOf(evalue));
                            scores.put(ID, Integer.valueOf(bits));
                        }
                        else if ( IDs.size() > 1 )
                        {
                            // Blank line after sequence summary list means it must be moving into the alignment

                            if ( lastIteration != -1 )
                            {
                                // need to see QUERY so we can ensure that we are done SUMMARY
                                // Mark this blank line so we can come back to it on the next pass if need be
                                bf.mark(READ_AHEAD_LIMIT);

                                continue;
                            }

                            // done parsing
                            state = State.ALIGNMENT;

                            for(String ID: IDs)
                            {
                                sequences.put(ID, "");
                            }
                        }
                        break;
                    case ALIGNMENT:
                        if ( line.contains("Database:"))
                        {
                            // Done the aligment
                            state = State.DONE;

                            // Do post-processing
                            fillBlocks(IDs, sequences, blockSequences);
                            break;
                        }
                        else if ( line.trim().length() > 0 )
                        {
                            String ID = line.substring(0, 5).trim();

                            String[] split = line.split("\\s+");
                            String sequencefrag;
                            if ( split.length == 4)
                            {
                                sequencefrag = split[2];

                                // figure out where it starts
                                if ( ID.equals("QUERY" ) )
                                {
                                    if ( alignmentStart == -1 )
                                    {
                                        alignmentStart = Integer.parseInt(split[1]);
                                    }
                                    alignmentEnd = Integer.parseInt(split[3]);
                                }
                            }
                            else if (split.length >= 2)
                            {
                                sequencefrag = split[1];
                            }
                            else
                            {
                                throw new IllegalStateException("Error\n" + line);
                            }

                            /*if the map doesn't contain it, then add it to the mapping
                            (seeing it the first time = associating it)

                            String sequenceID;
                            if ( ! blastIDTOPDBID.containsKey(ID))
                            {
                                sequenceID = IDs.get(sequenceIndex++);
                                blastIDTOPDBID.put(ID, sequenceID);
                            }
                            else
                            {
                                sequenceID = blastIDTOPDBID.get(ID);
                            }

                            assert sequences.containsKey(sequenceID);

                            blockSequences.put(sequenceID, sequencefrag);
                        }
                        else
                        {
                            // after a block
                            fillBlocks(IDs, sequences, blockSequences);
                        }
                        break;
                }
            }
            bf.close();
        }

        assert state == State.DONE : "Did not finish parsing: left in state: " + state;

        // Don't need these anymore
        blockSequences.clear();
        System.gc();


        // nothing found, finish
        if ( sequences.size() == 0 )
        {
            return;
        }

        createAlignment(sequences, seqList);

        ensureQueryLength(querySequence, seqList);

        // fill in the annotation
        for(GappedSequence seq: seqList)
        {
            seq.getAnnotation().setProperty("e-value", evalues.get(seq.getName()));
            seq.getAnnotation().setProperty("score", scores.get(seq.getName()));
        }
    }
*/
    /**
     * Parses either BlastP or PSI_BLAST in the -m6 format
     *
     * CAUTION: Ensure that the -v value on PSI-BLAST equals or exceeds the -b value.
     *   That is the summary lines vs. alignments.  Built on BLAST 2.2.13
     *
     * <p>
     *  How it works:
     *  This parser only parses the last iteration of the PSI-BLAST search, where supposedly all the
     *  sequences found are output.
     *  In the first pass, we mark the reader where the last iteration output begins.
     *  Then we go into the state machine, starting in HEADER.  When we see the declaration for the
     *  summary of the one-liners ("Sequences producing significant alignments"), we enter the SUMMARY state.
     *  Then we run until a blank line, parsing the summary lines along the way. <br>
     *   IF it's PSI-BLAST, we need to be careful to include the "Sequences not found previously or not previously below threshold"
     *  sequences. <br>
     *   When QUERY is found, we revert to the last line (it is marked along the way), and move into the ALIGNMENT state.
     *
     *   We then parse each alignment.
     * </p>
     *
     *
     *
     * @param querySequence The query sequence
     //* @param reader The reader used to read in the BLAST output file
     * @param seqList Out param, where the aligned sequences will be placed
     * @throws IllegalSymbolException
     * @throws IOException
     * @throws ChangeVetoException
     * @throws SAXException
     *
     * NOTE: This one uses random access files
     */
    public void parseBlastResult(Sequence querySequence, File file, List<GappedSequence> seqList) throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        if ( seqList == null )
        {
            throw new IllegalArgumentException("Can't pass in null sequence list - this needs to be populated with the results");
        }


        String line;
        Map<String, StringBuilder> sequences = new HashMap<String, StringBuilder>();
        Map<String, String> blockSequences = new HashMap<String, String>();
        Map<String, String> blastIDTOPDBID = new HashMap<String, String>();
        Map<String, Double> evalues = new HashMap<String, Double>();
        Map<String, Integer> scores = new HashMap<String, Integer>();

        // contains the blast ID's we see with each block.  Sometimes there are duplicates.  
        Set<String> seenThisRound = new HashSet<String>();

        // Mapping ofr sequenceIndex to ID
        Map<Integer, String> IDs = new TreeMap<Integer, String>();
        List<Integer> toSkip = new ArrayList<Integer>();

        //  keeps track of which line we're on in the block
        IDs.put(0, "QUERY");

        State state = State.HEADER;
        int sequenceIndex = 0;
        int lastIteration = -1;
        int READ_AHEAD_LIMIT = (int) (Runtime.getRuntime().totalMemory() / 4);

        if ( READ_AHEAD_LIMIT > STATIC_READ_AHEAD_LIMIT )
        {
            READ_AHEAD_LIMIT = STATIC_READ_AHEAD_LIMIT ;
        }

        {
            long offset = 0;
            RandomAccessReader reader = new RandomAccessReader(new RandomAccessFile(file.getAbsolutePath(), "r"));
            CountedBufferedReader bf = new CountedBufferedReader(reader);
            bf.mark(READ_AHEAD_LIMIT);

            /** First pass, find the last iteration */
            while( (line = bf.readLine()) != null )
            {
                if ( line.contains("Results from round") )
                {
                    String tmp = line.replaceFirst(".*Results from round (\\d+)", "$1");
                    lastIteration = Integer.parseInt(tmp);

                    // mark to come back to it
                //    bf.mark(READ_AHEAD_LIMIT);
                    offset  = bf.getFilePointer();
                }
                else if ( line.contains("Matrix:") )
                {
                   // end of the data
                    break;
                }
            }
            reader.close();

            reader = new RandomAccessReader(new RandomAccessFile(file.getAbsolutePath(), "r"));
            bf = new CountedBufferedReader(reader);
            try
            {
                reader.seek(offset);
                //bf.reset();
                //bf.seek(offset);
            }
            catch (IOException e)
            {
               // System.err.println("The BLAST output file READ_AHEAD_LIMIT was bigger than the alotted size! (" + READ_AHEAD_LIMIT / 1024 / 1024 + " M)");
                e.printStackTrace();
            }

            //BufferedReader bf = new BufferedReader(reader);

            READFILE_LOOP:
            while( (line = bf.readLine()) != null )
            {
                switch(state)
                {
                    case HEADER:
                        if ( line.contains("No hits found"))
                        {
                            state = State.DONE;
                            break READFILE_LOOP;
                        }

                        if ( lastIteration == -1 )
                        {
                            // BLASTP

                            if ( line.contains("Sequences producing significant alignments"))
                            {
                                state = State.SUMMARY;
                            }
                        }
                        else
                        {
                            /*
                             * PSI-BLAST
                             * We have marked the reader at the above level, right after the last iteration
                             */
                            if ( line.contains("Sequences used in model and found again:") )
                            {
                                state = State.SUMMARY;
                                break;
                            }

                            /* PSI-BLAST
                             * We want to skip to the last iteration
                             */
                          /*  if ( line.contains("Results from round " + lastIteration))
                            {
                                state = State.SUMMARY;

                                // skip to summary
                                while ( (line = bf.readLine()) != null && ! line.contains("Sequences used in model and found again:") )
                                {
                                    // noop;
                                }
                            }
                            */

                        }
                        break;
                    case SUMMARY:
                         if ( line.contains("Sequences not found previously or not previously below threshold:")
                            || line.contains("CONVERGED!") )
                        {
                            // skip this line
                            // but mark it so we can come back to it on the next pass if need be
                            bf.mark(READ_AHEAD_LIMIT);
                            continue;
                        }
                        else if ( line.trim().length() > 0 )
                        {
                            // not a blank line

                            if ( lastIteration != -1 )
                            {
                                if ( line.contains("QUERY"))
                                {
                                    // we are done SUMMARY if the QUERY line comes up (QUERY is only in the alignments, not in summary)
                                    bf.reset();

                                    state = State.ALIGNMENT;
                                    sequenceIndex = 0;

                                    for(String ID: IDs.values())
                                    {
                                        sequences.put(ID, new StringBuilder());
                                    }

                                    continue;
                                }
                            }

                            // Parse the one-line summary
                            String[] split = line.split("\\s{2,}");
                            String ID = line.substring(0, 60).trim();
                            String bits = split[split.length - 2];    // the second-last column
                            String evalue = split[split.length - 1];  // the last column

                            if ( evalue.startsWith("e"))
                            {
                                evalue = "1" + evalue;
                            }

                            if ( IDs.containsValue(ID))
                            {
                                // Sometimes there's duplicates in the BLAST DB, we can't parse those cuz it'll mess us up
                                toSkip.add(++sequenceIndex);
                                // IDs.add("DUPLICATE");
                            }
                            else
                            {
                                IDs.put(++sequenceIndex, ID);
                                evalues.put(ID, Double.valueOf(evalue));
                                scores.put(ID, Integer.valueOf(bits));
                            }
                        }
                        else if ( IDs.size() > 1 )
                        {
                            // Blank line after sequence summary list means it must be moving into the alignment

                            if ( lastIteration != -1 )
                            {
                                // need to see QUERY so we can ensure that we are done SUMMARY
                                // Mark this blank line so we can come back to it on the next pass if need be
                                bf.mark(READ_AHEAD_LIMIT);

                                continue;
                            }

                            // done parsing
                            state = State.ALIGNMENT;
                            sequenceIndex = 0;

                            for(String ID: IDs.values())
                            {
                                sequences.put(ID, new StringBuilder());
                            }
                        }
                        break;
                    case ALIGNMENT:
                        if ( line.contains("Database:")
                             || line.matches("Searching.*done")
                             || line.contains("No hits found"))
                        {
                            // Done the aligment
                            state = State.DONE;

                            // Do post-processing
                            fillBlocks(IDs, sequences, blockSequences);
                            break;
                        }
                        else if ( line.trim().length() > 0 )
                        {
                            String ID;
                            //ID = line.substring(0, 4).trim();

                            String[] split = line.split("\\s+");

                            ID = split[0];
                            String sequencefrag;
                            if ( split.length == 4)
                            {
                                sequencefrag = split[2];

                                // figure out where it starts
                                if ( isQuery(ID) )
                                {
                                    if ( alignmentStart == -1 )
                                    {
                                        alignmentStart = Integer.parseInt(split[1]);
                                    }
                                    alignmentEnd = Integer.parseInt(split[3]);
                                }
                            }
                            else if (split.length >= 2)
                            {
                                sequencefrag = split[1];
                            }
                            else
                            {
                                throw new IllegalStateException("Error parsing line (state = " + state + ")\n" + line);
                            }


                            // If we're not skipping this sequence because of a duplicate entry                            
                            if ( ! toSkip.contains(sequenceIndex))
                            {
                                /*if the map doesn't contain it, then add it to the mapping
                                (seeing it the first time = associating it)
                                 */
                                String sequenceID;
                                if ( ! blastIDTOPDBID.containsKey(ID))
                                {
                                    sequenceID = IDs.get(sequenceIndex);
                                    blastIDTOPDBID.put(ID, sequenceID);
                                }
                                else
                                {
                                    sequenceID = blastIDTOPDBID.get(ID);
                                }

                                // TODO sequenceIndex needs to be ++
                                
                                assert sequenceID != null;
                                assert sequences.containsKey(sequenceID);

                                blockSequences.put(sequenceID, sequencefrag);
                            }

                            if ( ! seenThisRound.contains(ID))
                            {
                                // stop duplicate sequences from increasing the sequenceIndex
                                sequenceIndex++;
                                seenThisRound.add(ID);
                            }
                        }
                        else
                        {
                            // after a block
                            fillBlocks(IDs, sequences, blockSequences);
                            sequenceIndex = 0;
                            seenThisRound.clear();
                        }
                        break;
                }
            }
            bf.close();
        }

        assert state == State.DONE : "Did not finish parsing: left in state: " + state   + "\n line = " + line;

        // Don't need these anymore
        blockSequences.clear();
        
        // nothing found, finish
        if ( sequences.size() == 0 )
        {
            return;
        }

        createAlignment(sequences, seqList);

        ensureQueryLength(querySequence, seqList);

        // fill in the annotation
        for(GappedSequence seq: seqList)
        {
            seq.getAnnotation().setProperty("e-value", evalues.get(seq.getName()));
            seq.getAnnotation().setProperty("score", scores.get(seq.getName()));
        }
    }

    private boolean isQuery(String ID)
    {
        return ID.equals("QUERY" ) || ID.equals("1_0" );
    }

    private void ensureQueryLength(Sequence querySequence, List<GappedSequence> seqList) throws IllegalSymbolException
    {
        GappedSequence gappedQuery = null;
        int gapPosition = -1, i=0;
        for(GappedSequence gs: seqList)
        {
            if ( isQuery(gs.getName()) )
            {
                gappedQuery = gs;
                gapPosition = i;
                break;
            }
            i++;
        }

        assert gappedQuery != null : "Can't find query sequence in BLAST result list";

        if ( gappedQuery.getSourceSymbolList().length() != querySequence.length() )
        {
        /*    GappedSequence newGappedQuery = ProteinTools.createGappedProteinSequence(querySequence.seqString(), "QUERY");

            /* Insert gaps */
          /*  String seqString = gappedQuery.seqString();
            int pos = seqString.indexOf('-', 0);
            while(pos!=-1)
            {
                newGappedQuery.addGapInView(alignmentStart + pos);
                pos = seqString.indexOf('-', alignmentStart + pos + 1);
            }

            assert newGappedQuery.getSourceSymbolList().length() == querySequence.length();

            */

            assert alignmentStart != -1 : "AlignmentStart never got assigned";

            String prefix = querySequence.seqString().substring(0, alignmentStart -1);
            String suffix = querySequence.seqString().substring(alignmentEnd);
            String newQuery = prefix + gappedQuery.seqString() + suffix;

            assert newQuery.replace("-", "").replace("~", "").equals(querySequence.seqString());

            GappedSequence newGappedQuery = ProteinTools.createGappedProteinSequence(newQuery, "QUERY");

            int addedLength = querySequence.length() - gappedQuery.getSourceSymbolList().length() ;

            /* Figure out the leading gap  for the rest of the sequences */
            String gappedQueryString = newGappedQuery.seqString();

            /* offset counts how many characters (including gaps) we get till we see
              alignmentStart residues (non-gaps) */
            int offset = 0, numResidues = 0;
            for (int seqIndex=0; seqIndex<gappedQueryString.length(); seqIndex++)
            {
                if ( gappedQueryString.charAt(seqIndex) != '-')
                {
                    numResidues++;
                }
                if ( numResidues > alignmentStart )
                {
                    break;
                }

                offset++;
            }


            // make sure the rest of the sequences are the same length as the gapped query
            for(GappedSequence gs: seqList)
            {
                // insert beginning gaps
                gs.addGapsInView(1, offset - 1);

                // insert end gaps
                gs.addGapsInView(gs.length() +1, addedLength - (offset - 1));

                // TODO hack
                while (gs.length() > newGappedQuery.length())
                {
                    gs.removeGap(gs.length() );
                }
              
                assert gs.length() == newGappedQuery.length();
            }

            // replace the query sequence in the list
            seqList.set(gapPosition, newGappedQuery);
        }
    }

    private void fillBlocks(Map<Integer, String> IDs, Map<String, StringBuilder> sequences, Map<String, String> blockSequences)
    {
        for(String ID: IDs.values())
        {
            // String value = sequences.get(ID);
            String seqFrag = blockSequences.get(ID);
            // append all the blocks
            sequences.get(ID).append(seqFrag);
        }
    }

    private void createAlignment(Map<String, StringBuilder> sequences, List<GappedSequence> seqList) throws IllegalSymbolException
    {
        int length = 0;
        for(String ID: sequences.keySet())
        {
            length = Math.max(length, sequences.get(ID).length() );
        }

        for(String ID: sequences.keySet())
        {
            /* For some reason BLAST doesn't include all the sequences it said it would in
             the summary, so some come out as "nullnullnull"
             */
            String seqStr = sequences.get(ID).toString();
            if ( seqStr.length() >= length )
            {
                GappedSequence gapped = ProteinTools.createGappedProteinSequence(seqStr, ID);
                seqList.add(gapped);
            }
        }
    }

    /**
     * Does the grunt work of parsing the BLAST file, whether it's a text output, or XML output,
     * it should be able to parse it with the appropriate XMLReader
     * @param oParser
     * @param outputReader - a reader to read the output from BLAST
     * @param seqList - a list of sequences that will be filled when BLAST is parsed
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws org.biojava.bio.symbol.IllegalSymbolException
     * @throws org.biojava.utils.ChangeVetoException
     */
    /*public List<Alignment> parseBlastResult(XMLReader oParser, Reader outputReader, List<GappedSequence> seqList)
            throws IOException, SAXException, IllegalSymbolException, ChangeVetoException
    {
        //make the SAX event adapter that will pass events to a Handler.
        SeqSimilarityAdapter adapter = new SeqSimilarityAdapter();

        //set the parsers SAX event adapter
        oParser.setContentHandler(adapter);

        //The list to hold the SeqSimilaritySearchResults
        List<SeqSimilaritySearchResult> results = new ArrayList<SeqSimilaritySearchResult>();

        //create the SearchContentHandler that will build SeqSimilaritySearchResults
        //in the results List
        SearchContentHandler builder = new BlastLikeSearchBuilder(results,
                new DummySequenceDB("queries"), new DummySequenceDBInstallation());

        //register builder with adapter
        adapter.setSearchContentHandler(builder);

        // parse it
        oParser.parse(new InputSource(outputReader));

        List<Alignment> alignments = new ArrayList<Alignment>();

        //output some blast details
        for (SeqSimilaritySearchResult result : results)
        {
            //list the hits
            for (Object o : result.getHits())
            {
                SeqSimilaritySearchHit hit =
                        (SeqSimilaritySearchHit) o;

                // list the sequences
                for (Iterator it = hit.getSubHits().iterator(); it.hasNext();)
                {
                    SeqSimilaritySearchSubHit subHit = (SeqSimilaritySearchSubHit) it.next();

                    Alignment alignment = subHit.getAlignment();

                    for (Object label1 : alignment.getLabels())
                    {
                        String label = (String) label1;

                        // ignore query sequences
                        if (label.equals("Query"))
                            continue;

                        // add the sequence with its annotation to the list
                        SymbolList symList = subHit.getAlignment().symbolListForLabel(label);

                        alignments.add(subHit.getAlignment());

                        // Get rid of the gaps that BLAST produces
                        String symStr = symList.seqString();
                        //symStr = symStr.replace("-", "");
                        symList = ProteinTools.createGappedProteinSequence(symStr, label);

                        Annotation annotation = result.getAnnotation();

                        // Description should be like gi|107890|pir|S4918 twitchin-like protein
                        // we don't want that
                        // but it could also be 1K0BB
                        String ID = (String) hit.getAnnotation().getProperty("subjectDescription");
                        if (ID.length() > 9 || ID.length() == 0)
                        {
                            // this should be like gnl|BL_ORD_ID|801283
                            ID = hit.getSubjectID();
                        }

                        if (ID.contains("gi|"))
                        {
                            String[] tmp = ID.split("\\|");
                            ID = tmp[1];
                        }
                        else if (ID.indexOf('|') != -1)
                        {
                            String tmp = ID.substring(ID.lastIndexOf('|') + 1, ID.length());
                            if (tmp.length() > 0)
                                ID = tmp;
                        }

                        if (ID.length() > 8)
                            ID = ID.substring(0, 7);

                        GappedSequence seq = ProteinTools.createGappedProteinSequence(symStr, ID);
                        seq.getAnnotation().setProperty("Score", subHit.getScore());
                        seq.getAnnotation().setProperty("e-value", subHit.getEValue());
                        seq.getAnnotation().setProperty("subjectDescription", hit.getAnnotation().getProperty("subjectDescription"));

                        for(Object key : subHit.getAnnotation().keys())
                        {
                            seq.getAnnotation().setProperty(key, subHit.getAnnotation().getProperty(key));
                        }
                        seqList.add(seq);
                    }

                    //     return alignment;
                }
            }

            //      System.out.println("\n");
        }
        return alignments;
    }*/

    /**
     * Main method - tests BLAST
     * @param args
     */
    public static void main(String[] args) throws IOException
    {
        StructureDB db = new StructureDB("vadarEVA.db", StructureDB.MODE.READ);
        List<GappedSequence> blastSequences;
        try
        {
            Sequence seq = db.getSequence("1a47");
            BlastSettings settings = new BlastSettings();
            settings.setSequence(seq);
            settings.setBlasttype(BlastSettings.BLASTTYPE.PSIBLAST);
            settings.setEvalue(0.001);
            settings.setDatabase(Config.getConfig().getStringValue("PDB"));
            settings.setAlignmentView(BlastSettings.ALIGNMENT_VIEW.BLUNT_ENDS);

            BlastRunner blastRunner = new BlastRunner(settings);
            blastRunner.run();

            blastSequences = blastRunner.getAlignedResult();

            System.out.println("BlastP results: ");
            for (GappedSequence sequence : blastSequences)
            {
                System.out.println(sequence.getName() + ": " + sequence.seqString());
            }

            /*settings.setBlasttype(BlastSettings.BLASTTYPE.PSIBLAST);
            settings.setDatabase(Config.getConfig().getStringValue("PDB"));
            blastRunner.run();

            blastSequences = blastRunner.getAlignedResult();

            System.out.println("PSIBLAST results: ");
            for ( Iterator<GappedSequence> iterator = blastSequences.iterator(); iterator.hasNext(); ) {
                Sequence sequence = iterator.next();
                System.out.println(sequence.getName() + ": " + sequence.seqString());
            }
            */

/*            ClustalWRunner runner = new ClustalWRunner(seq, blastSequences);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            Alignment alignment = runner.getAlignment();

            // make a list of all the aligned homologues
            for( Sequence homologSequence : blastSequences)
            {
                SymbolList homolog = alignment.symbolListForLabel(homologSequence.getName());

               // Create a gappedstructure object for the homolog
                GappedSequence gappedHomolog = ProteinTools.createGappedProteinSequence(homolog.seqString(), homologSequence.getName());

                System.out.println(gappedHomolog.seqString());
            }
            */
        }
        catch (BioException e)
        {
            e.printStackTrace();
        }
    }
}
