package ca.ualberta.proteus.util;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.log4j.Logger;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;

import com.sun.jmx.interceptor.DefaultMBeanServerInterceptor;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Feb 1, 2006
 * Time: 8:56:29 PM
 * To change this template use File | Settings | File Templates.
 */
public class PDBFileDownloader
{

    final static String PDBFTPUrl = "ftp://ftp.rcsb.org/pub/pdb/data/structures/divided/pdb/%s/pdb%s.ent.Z";
    final static String PDBHTTPUrl = "http://www.rcsb.org/pdb/cgi/export.cgi/%s.pdb?job=download;pdbId=%s;format=PDB;pre=1&compression=None";
    final static String RCSB_PDB_Link = "http://www.rcsb.org/pdb/navbarsearch.do?newSearch=yes&isAuthorSearch=no&radioset=All&inputQuickSearch=%s&image.x=0&image.y=0&image=Search";
    final static String PDBHEADER = "http://www.rcsb.org/pdb/files/%s.pdb?headerOnly=YES";
    
    private static Logger logger = Logger.getLogger(PDBFileDownloader.class);

    /**
     * Downloads a PDB file from the PDB database using HTTP
     * @param pdbFileName = 4-letter PDB ID
     * @return InputStream containing the PDB file
     * @throws IllegalArgumentException if the pdbFileName isn't 4 characters long
     */
    public static InputStream downloadPDBFile(String pdbFileName) throws IOException
    {
        if ( pdbFileName.length() != 4 )
        {
            throw new IllegalArgumentException("PDB Identifier must be 4-letter format");
        }

        final String url = String.format(PDBFTPUrl, pdbFileName.toLowerCase().substring(1, 3), pdbFileName.toLowerCase());

        // turn off default debug output
        /*System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
        System.setProperty("org.apache.commons.logging.simplelog.showdatetime", "false");
        System.setProperty("org.apache.commons.logging.simplelog.log.httpclient.wire", "info");
        System.setProperty("org.apache.commons.logging.simplelog.log.org.apache.commons.httpclient", "info");
*/
        // This is broken now

        URL _url = new URL(url);
        URLConnection urlc = _url.openConnection();
        InputStream is = urlc.getInputStream(); // To download

        UncompressInputStream gzip = new UncompressInputStream(is);
        return gzip;
/*

        HttpClient httpClient = new HttpClient();
        HttpMethod method = new GetMethod(url);
        InputStream responseBodyAsStream = null;
        try
        {
            int httpresult = httpClient.executeMethod(method);
            if (httpresult != HttpStatus.SC_OK)
            {
                System.err.println("Method failed: " + method.getStatusLine());
            }

            if ( httpresult == HttpStatus.SC_NOT_FOUND )
            {
                throw new FileNotFoundException("Error downloading file from PDB " + pdbFileName);
            }

            responseBodyAsStream = method.getResponseBodyAsStream();
        }
        catch(NullPointerException e)
        {
            e.printStackTrace();
            System.err.println("Error was handled, couldn't download PDB file");
        }
        catch (HttpException e)
        {
            System.err.println("Fatal protocol violation: " + e.getMessage());
            e.printStackTrace();
        }
        catch (IOException e)
        {
            System.err.println("Fatal transport error: " + e.getMessage());
            e.printStackTrace();
        }
        finally
        {
            // Release the connection.
            //method.releaseConnection();
        }
*/


        //System.out.println(method.getResponseBody());
        //return responseBodyAsStream;
    }

    public static String getSequenceNameFromID(String ID)
    {
        if ( ID.length() != 4 )
        {
            throw new IllegalArgumentException("PDB Identifier must be 4-letter format");
        }
        final String url = String.format(PDBHEADER, ID);

        HttpClient httpClient = new HttpClient();
        HttpMethod method = new GetMethod(url);
        InputStream responseBodyAsStream = null;
        try
        {
            logger.debug("Sending request for " + url);

            int httpresult = httpClient.executeMethod(method);
            if (httpresult != HttpStatus.SC_OK)
            {
                System.err.println("Method failed: " + method.getStatusLine());
            }

            if ( httpresult == HttpStatus.SC_NOT_FOUND )
                throw new FileNotFoundException("Error downloading file from PDB " + ID);

            responseBodyAsStream = method.getResponseBodyAsStream();
        }
        catch(NullPointerException e)
        {
            e.printStackTrace();
            System.err.println("Error was handled, couldn't download PDB file");
        }
        catch (HttpException e)
        {
            System.err.println("Fatal protocol violation: " + e.getMessage());
            e.printStackTrace();
        }
        catch (IOException e)
        {
            System.err.println("Fatal transport error: " + e.getMessage());
            //e.printStackTrace();
        }

        try
        {
            if (responseBodyAsStream != null)
            {
                // parse it and find the name

                BufferedReader reader = new BufferedReader(new InputStreamReader(responseBodyAsStream) );

                String line;

                // this is a really bad hack for parsing...
                String title = null;
                while ((line = reader.readLine()) != null)
                {
                    if (line.startsWith("TITLE"))
                    {
                        reader.close();
                        return line.substring(5, 70).trim();
                    }
                    else if (line.startsWith("COMPND"))
                    {
                        title =  line.substring(5, 70).trim();
                    }
                    else if ( line.startsWith("SEQRES"))
                    {
                        if (title != null)
                        {
                            reader.close();
                            return title;
                        }
                    }
                }
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch( NullPointerException e)
        {
            // Since it's so bad, we're likely to get a NPE at some point
            e.printStackTrace();
        }

        return "";
    }
}
