package ca.ualberta.proteus.util;

import org.apache.log4j.Logger;

import java.io.*;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 3, 2006
 * Time: 1:21:08 PM
 * To change this template use File | Settings | File Templates.
 */
public class PDBFileLoader
{
    private static Logger logger = Logger.getLogger(PDBFileLoader.class);

    /**
     *
     * @param PDBFileName
     * @param searchDirectories
     * @return Null if the file can't be found or downloaded
     */
    public static InputStream getPDBFile(String PDBFileName, File[] searchDirectories)
    {
        if ( PDBFileName.length() != 4 && PDBFileName.length() != 5 )
            throw new IllegalArgumentException("PDB file name must be 4 or 5 characters");

        InputStream in = null;
        for ( File f : searchDirectories) {
            try
            {
                String path = f.getPath();
                if ( ! path.endsWith("/"))
                    path += "/";
                File theFile = new File(path + PDBFileName);
                if ( ! theFile.exists())
                {
                    theFile = new File(path + PDBFileName + ".pdb");
                }
                if ( ! theFile.exists())
                {
                    theFile = new File(path + PDBFileName.substring(0,4) + ".pdb");
                }
                if ( theFile.exists() )
                {
                    in = new FileInputStream(theFile);
                    break;
                }
            }
            catch (FileNotFoundException e)
            {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            if ( in != null )
                break;
        }

        // not in any search directories... try and find it in the classpath
        if ( in == null )
            in = PDBFileLoader.class.getResourceAsStream("/" + PDBFileName);

        if ( in == null )
            in = PDBFileLoader.class.getResourceAsStream(PDBFileName);

        if ( in == null )
            in = Thread.currentThread().getContextClassLoader().getResourceAsStream(PDBFileName);

        if ( in == null )
            in = Thread.currentThread().getContextClassLoader().getResourceAsStream("/" + PDBFileName);

        /*
    if ( in == null )
        throw new FileNotFoundException("Cannot find file " + PDBFileName + " in classpath");
        */

        // not in classpath, download it
        if ( in == null )
            try
            {
                String PDBID = PDBFileName.substring(0, 4);
                logger.debug("Downloading file " + PDBID);

                try
                {
                    in = PDBFileDownloader.downloadPDBFile(PDBID);
                }
                catch (IOException e)
                {
                    // If there's an error downloading we'll get a FileNotFoundException
                    logger.error(e);
                    return null;
                }

                if ( in != null)
                {
                    // cache the downloaded file
                    if ( searchDirectories.length > 0 )
                    {
                        BufferedReader reader = new BufferedReader( new InputStreamReader(in) );
                        if ( ! searchDirectories[0].exists() )
                        {
                            searchDirectories[0].mkdirs();
                        }

                        String fileName = searchDirectories[0].getAbsolutePath() + "/" + PDBFileName.substring(0, 4) + ".pdb";
                        PrintWriter writer = new PrintWriter(fileName);
                        try {
                            String line;
                            while ((line = reader.readLine()) != null)
                                writer.println(line);
                        }
                        finally{
                            reader.close();
                            writer.close();
                        }

                        in = new FileInputStream(fileName);
                    }
                }
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }

        if ( in == null )
            logger.debug("Can't download PDB File " + PDBFileName);

        return in;
    }
}
