package ca.ualberta.proteus.util;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.GappedStructureAnnotatedSequence;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import ca.ualberta.cs.homodeller.Homodeller;
import org.biojava.bio.Annotation;
import org.biojava.bio.BioError;
import org.biojava.bio.BioException;
import org.biojava.bio.SimpleAnnotation;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.proteomics.StructureTools;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleGappedSequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.CharacterTokenization;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.*;
import org.biojava.utils.ChangeVetoException;
import org.biojava.utils.ClassTools;
import org.xml.sax.InputSource;

import java.io.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: Smontgomerie
 * Date: Mar 1, 2005
 * Time: 11:03:15 AM
 * To change this template use File | Settings | File Templates.
 */
public class ProteusTools
{

    public static final int NUM_RESIDUES = 21;
    public static final int NUM_STRUCTURES = 4;

    private static Map<Symbol, Character> symbolToChar = null; //<Symbol, Character>
    private static Map<Symbol, Integer> symbolToInt = null; //<Symbol, Integer>
    private static Map<Symbol, Integer> structureToInt = null; //<Symbol, Integer>
    private static Map<Integer, Symbol> intToStructure = null; //<Integer, Symbol>
    private static Map<Integer, Symbol> intToSymbol = null; //<Integer, Symbol>

    private static AtomicSymbol x = AlphabetManager.createSymbol("X");
    public static final int HELIX = 1;
    public static final int STRAND = 2;
    public static final int COIL = 3;
    public static final int NOTHING = 0;
    public static final int GAP = 0;

    public static final char HELIX_CHAR = 'H';
    public static final char STRAND_CHAR = 'E';
    public static final char COIL_CHAR = 'C';
    public static final char GAP_CHAR = '-';

    private static Map<Symbol, Character> structureToChar = null;
    private static boolean verbose = false;

    public static final float MAX_PPSH_THRESHOLD = 90;
    public static final float MIN_PPSH_THRESHOLD = 30;

    static
    {
      //   TODO put back in later for THRIFTY
       //    loadDavesAlphabet();
    }

    public static char convertToChar(Symbol res)
    {
        if (symbolToChar == null)
            loadSymbolToChar();
        return symbolToChar.get(res);
    }

    private static void loadSymbolToChar()
    {
        symbolToChar = new HashMap<Symbol, Character>();
        symbolToChar.put(ProteinTools.ala(), 'A');
        symbolToChar.put(ProteinTools.arg(), 'R');
        symbolToChar.put(ProteinTools.asn(), 'N');
        symbolToChar.put(ProteinTools.asp(), 'D');
        symbolToChar.put(ProteinTools.cys(), 'C');
        symbolToChar.put(ProteinTools.gln(), 'Q');
        symbolToChar.put(ProteinTools.glu(), 'E');
        symbolToChar.put(ProteinTools.gly(), 'G');
        symbolToChar.put(ProteinTools.his(), 'H');
        symbolToChar.put(ProteinTools.ile(), 'I');
        symbolToChar.put(ProteinTools.leu(), 'L');
        symbolToChar.put(ProteinTools.lys(), 'K');
        symbolToChar.put(ProteinTools.met(), 'M');
        symbolToChar.put(ProteinTools.phe(), 'F');
        symbolToChar.put(ProteinTools.pro(), 'P');
        symbolToChar.put(ProteinTools.sec(), 'U');
        symbolToChar.put(ProteinTools.ser(), 'S');
        symbolToChar.put(ProteinTools.ter(), '*');
        symbolToChar.put(ProteinTools.thr(), 'T');
        symbolToChar.put(ProteinTools.trp(), 'W');
        symbolToChar.put(ProteinTools.tyr(), 'Y');
        symbolToChar.put(ProteinTools.val(), 'V');

        symbolToChar.put(x(), 'X');
    }

    public static int convertToInt(Symbol res) throws IllegalAlphabetException, IllegalSymbolException
    {
        if (symbolToInt == null)
            loadSymbolToInt();
        final Integer rv = (Integer) symbolToInt.get(res);
        if ( rv == null) {
            try {
                String s = new CharacterTokenization(ProteinTools.getAlphabet(), false).tokenizeSymbol(res);
                if ( s.equals("B"))  {
                    System.err.println("Substituting \"B\" for asparagine!");
                    return (Integer) symbolToInt.get(ProteinTools.asp());
                }
            }
            catch ( IllegalSymbolException e ) {
                return (Integer) symbolToInt.get(x);
            }

            //throw new IllegalAlphabetException("Illegal symbol: " + res.getName());
        }
        assert rv != null;

        return rv;
    }

    private static void loadSymbolToInt()
    {
        symbolToInt = new HashMap<Symbol, Integer>();
        symbolToInt.put(ProteinTools.ala(), new Integer(0));
        symbolToInt.put(ProteinTools.arg(), new Integer(1));
        symbolToInt.put(ProteinTools.asn(), new Integer(2));
        symbolToInt.put(ProteinTools.asp(), new Integer(3));
        symbolToInt.put(ProteinTools.cys(), new Integer(4));
        symbolToInt.put(ProteinTools.gln(), new Integer(5));
        symbolToInt.put(ProteinTools.glu(), new Integer(6));
        symbolToInt.put(ProteinTools.gly(), new Integer(7));
        symbolToInt.put(ProteinTools.his(), new Integer(8));
        symbolToInt.put(ProteinTools.ile(), new Integer(9));
        symbolToInt.put(ProteinTools.leu(), new Integer(10));
        symbolToInt.put(ProteinTools.lys(), new Integer(11));
        symbolToInt.put(ProteinTools.met(), new Integer(12));
        symbolToInt.put(ProteinTools.phe(), new Integer(13));
        symbolToInt.put(ProteinTools.pro(), new Integer(14));
        //  symbolToInt.put( ProteinTools.sec(), 15);        // we won't ever see these...
        symbolToInt.put(ProteinTools.ser(), new Integer(15));
        // symbolToInt.put( ProteinTools.ter(), 16);
        symbolToInt.put(ProteinTools.thr(), new Integer(16));
        symbolToInt.put(ProteinTools.trp(), new Integer(17));
         symbolToInt.put(ProteinTools.tyr(), new Integer(18));
        symbolToInt.put(ProteinTools.val(), new Integer(19));
        symbolToInt.put(x(), new Integer(20));
        symbolToInt.put(ProteinTools.getAlphabet().getGapSymbol(), new Integer(20));
        symbolToInt.put(AlphabetManager.getGapSymbol(), new Integer(20));

       // assert symbolToInt.size() == NUM_RESIDUES;
    }

    public static AtomicSymbol x()
    {
        return x;
    }

    /**
     * Returns the int value of a structure symbol
     */
    public static int convertStructureToInt(Symbol structure)
    {
        if (structureToInt == null)
        {
            structureToInt = new HashMap<Symbol, Integer>(); //<Symbol, Integer>
            StructureTools structureTools = new StructureTools();
            structureToInt.put(structureTools.getH(), new Integer(HELIX));
            structureToInt.put(structureTools.getE(), new Integer(STRAND));
            structureToInt.put(structureTools.getC(), new Integer(COIL));
            structureToInt.put(structureTools.getB(), new Integer(STRAND));
            structureToInt.put(structureTools.getStructure().getGapSymbol(), new Integer(0));
            structureToInt.put(AlphabetManager.getGapSymbol(), new Integer(0));
            structureToInt.put(ProteusTools.x(), new Integer(0));
        }

        final Object o = structureToInt.get(structure);
        if ( o == null ) {
            System.out.println("Invalid structure conversion to integer " + ((BasisSymbol) structure).getName());
            return 0;
        }
        return ((Integer) o).intValue();
    }

    public static Symbol convertIntToStructure(int structure) throws IllegalSymbolException {
        if (intToStructure == null)
        {
            intToStructure = new HashMap<Integer, Symbol>();
            StructureTools structureTools = new StructureTools();
            intToStructure.put(new Integer(0), structureTools.getStructure().getGapSymbol());
            intToStructure.put(new Integer(HELIX), structureTools.getH());
            intToStructure.put(new Integer(STRAND), structureTools.getE());
            intToStructure.put(new Integer(COIL), structureTools.getC());
        }
        Symbol sym = (Symbol) intToStructure.get(new Integer(structure));
        if ( sym == null)
            throw new IllegalSymbolException("Passed illegal symbol " + structure);
        return  sym;
    }

    public static double q3Score(SymbolList sequence, SymbolList predicted)
    {

        assert ( sequence.length() == predicted.length() ) : " Predicted sequence is not the same length as query sequence";

        int i, n;
        double out;
        Symbol s;
        Symbol sse[] = new Symbol[3];
        StructureTools structureTools = new StructureTools();

        sse[0] = structureTools.getH();
        sse[1] = structureTools.getE();
        sse[2] = structureTools.getC();

        n = 0;
        out = 0.0;
        for (i = 0; i < sequence.length(); i++)
        {
            s = sequence.symbolAt(i+1);
            if (s == sse[0] || s == sse[1] || s == sse[2])
            {
                n++;
                if (sequence.symbolAt(i+1) == predicted.symbolAt(i+1))
                {
                    out = out + 1.0;
                }
            }
            else
            {
                System.out.println("Evaluation contains incorrect structure assignment");
            }
        }
        if (n > 0)
        {
            out = out / (double) n;
        }
        else
        {
            out = 1.0;
        }

        return out;
    }

    public static double sovScore(SymbolList sequence, SymbolList predicted)
    {
        int i, k, length1, length2, beg_s1, end_s1, beg_s2, end_s2;
        int j1;
        int j2;
        int k1;
        int k2;
        int minov;
        int maxov;
        double d = 0;
        double d1;
        double d2;
        int n;
        int multiple;
        Symbol s1;
        Symbol s2;
        Symbol sse[] = new Symbol[3];
        double out;
        double s, x;
        StructureTools structureTools = new StructureTools();

        sse[0] = structureTools.getH();
        sse[1] = structureTools.getE();
        sse[2] = structureTools.getC();

        n = 0;
        int n_aa = sequence.length();
        for (i = 0; i < n_aa; i++)
        {
            s1 = sequence.symbolAt(i + 1);
            if (s1 == sse[0] || s1 == sse[1] || s1 == sse[2])
            {
                n++;
            }
        }
        out = 0.0;
        s = 0.0;
        length1 = 0;
        length2 = 0;
        i = 0;
        while (i < n_aa)
        {
            beg_s1 = i;
            s1 = sequence.symbolAt(i + 1);
            while (sequence.symbolAt(i + 1) == s1 && i < n_aa - 1)
            {
                i++;
            }
            end_s1 = i++;
            length1 = end_s1 - beg_s1 + 1;
            multiple = 0;
            k = 0;
            while (k < n_aa)
            {
                beg_s2 = k;
                s2 = predicted.symbolAt(k + 1);
                while (k < n_aa - 1 && predicted.symbolAt(k + 1) == s2)
                {
                    k++;
                }
                end_s2 = k++;
                length2 = end_s2 - beg_s2 + 1;
                if (s1 == sse[0] || s1 == sse[1] || s1 == sse[2])
                {
                    if (s1 == s2 && end_s2 >= beg_s1 && beg_s2 <= end_s1)
                    {
                        if (multiple > 0)
                        {
                            n = n + length1;
                        }
                        multiple++;
                        if (beg_s1 > beg_s2)
                        {
                            j1 = beg_s1;
                            j2 = beg_s2;
                        }
                        else
                        {
                            j1 = beg_s2;
                            j2 = beg_s1;
                        }
                        if (end_s1 < end_s2)
                        {
                            k1 = end_s1;
                            k2 = end_s2;
                        }
                        else
                        {
                            k1 = end_s2;
                            k2 = end_s1;
                        }
                        minov = k1 - j1 + 1;
                        maxov = k2 - j2 + 1;
                        d1 = Math.floor(length1 * 1.0);
                        d2 = Math.floor(length2 * 0.5);
                        if (d1 > d2) d = d2;
                        if (d1 <= d2) d = d1;
                        if (d > minov)
                        {
                            d = minov;
                        }
                        if (d > maxov - minov)
                        {
                            d = maxov - minov;
                        }
                        x = 1.0 * d;
                        x = (minov + x) * length1;
                        if (maxov > 0)
                        {
                            s = s + x / maxov;
                        }
                        else
                        {
                            System.out.print("\n ERROR! minov = %-4d maxov = %-4d length = %-4d d = %-4f   %4d %4d  %4d %4d");
                        }
                    }
                }
            }
        }

        if (n > 0)
        {
            out = s / n;
        }
        else
        {
            out = 1.0;
        }
        return out;
    }

    public static SimpleSymbolList createStructure(String annotation) {
        try
        {
            SymbolTokenization a = AlphabetManager.alphabetForName("STRUCTURE").getTokenization("token");
            return new SimpleSymbolList(a, annotation);
        }
        catch (BioException se)
        {
            throw new IllegalArgumentException("Something has gone badly wrong with Structure\n" + annotation, se);
        }
    }

    public static SimpleSymbolList createDavesStructure(String annotation) {
        try
        {
            SymbolTokenization a = AlphabetManager.alphabetForName("DAVES_STRUCTURE").getTokenization("token");
            return new SimpleSymbolList(a, annotation.toLowerCase());
        }
        catch (BioException se)
        {
            throw new IllegalArgumentException("Something has gone badly wrong with Structure\n" + annotation, se);
        }
    }

    public static Symbol convertIntToSymbol(int res) {
        if ( intToSymbol == null ) {
            intToSymbol = new HashMap();

        intToSymbol.put(new Integer(0), ProteinTools.ala());
        intToSymbol.put(new Integer(1), ProteinTools.arg());
        intToSymbol.put(new Integer(2), ProteinTools.asn());
        intToSymbol.put(new Integer(3), ProteinTools.asp());
        intToSymbol.put(new Integer(4), ProteinTools.cys());
        intToSymbol.put(new Integer(5), ProteinTools.gln());
        intToSymbol.put(new Integer(6), ProteinTools.glu());
        intToSymbol.put(new Integer(7), ProteinTools.gly());
        intToSymbol.put(new Integer(8), ProteinTools.his());
        intToSymbol.put(new Integer(9), ProteinTools.ile());
        intToSymbol.put(new Integer(10), ProteinTools.leu());
        intToSymbol.put(new Integer(11), ProteinTools.lys());
        intToSymbol.put(new Integer(12), ProteinTools.met());
        intToSymbol.put(new Integer(13), ProteinTools.phe());
        intToSymbol.put(new Integer(14), ProteinTools.pro());
        //  symbolToInt.put( ProteinTools.sec(), 15);        // we won't ever see these...
        intToSymbol.put(new Integer(15), ProteinTools.ser());
        // symbolToInt.put( ProteinTools.ter(), 16);
        intToSymbol.put(new Integer(16), ProteinTools.thr());
        intToSymbol.put(new Integer(17), ProteinTools.trp());
        intToSymbol.put(new Integer(18), ProteinTools.tyr());
        intToSymbol.put(new Integer(19), ProteinTools.val());
        intToSymbol.put(new Integer(20), x());
        }
        return (Symbol) intToSymbol.get(new Integer(res));
    }

    public static void main(String[] args) {
        if ( args[0].equals("-q3") ) {
            try {
                //  open up standard input
                BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
                System.out.print("Assignment: ");
                String sequence = br.readLine();
                sequence = sequence.replace('-', 'C');

                System.out.print("Predicted:  ");
                String predicted = br.readLine();
                predicted = predicted.replace('-', 'C');

                double q3 = q3Score(ProteusTools.createStructure(sequence), ProteusTools.createStructure(predicted));
                double sov = sovScore(ProteusTools.createStructure(sequence), ProteusTools.createStructure(predicted));

                System.out.println("Q3 Score: " + q3);
                System.out.println("SOV Score: " + sov);
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Returns the int value of a structure symbol
     */
    public static char convertStructureToChar(Symbol structure)
    {
        if (structureToChar == null)
        {
            structureToChar = new HashMap(); //<Symbol, Character>
            StructureTools structureTools = new StructureTools();
            structureToChar.put(structureTools.getH(), HELIX_CHAR);
            structureToChar.put(structureTools.getE(), STRAND_CHAR);
            structureToChar.put(structureTools.getC(), COIL_CHAR );
            structureToChar.put(structureTools.getStructure().getGapSymbol(), GAP_CHAR);
            structureToChar.put(AlphabetManager.getGapSymbol(), GAP_CHAR );
            structureToChar.put(ProteusTools.x(), 'X');
        }

        final Object o = structureToChar.get(structure);
        if ( o == null ) {
            System.out.println("Invalid structure conversion to character" + ((BasisSymbol) structure).getName());
            new Exception().printStackTrace();
            return 0;
        }
        return (Character) o;
    }

    public static Sequence ungapSequence(Sequence seq)
    {
        assert !( seq instanceof StructureAnnotatedSequence ) : "Can't pass in a structure annotated sequence!";
        Symbol gapSymbol = seq.getAlphabet().getGapSymbol();
        Symbol[] symbols = new Symbol[seq.length()];

        int index = 0;
        for ( Iterator it = seq.iterator(); it.hasNext(); ) {
            Symbol s = (Symbol) it.next();
            if ( !s.equals(AlphabetManager.getGapSymbol()) && !s.equals(gapSymbol) ) {
                symbols[index++] = s;
            }
        }

        return new SimpleSequence(
                new SimpleSymbolList(symbols, index, ProteinTools.getAlphabet()),
                "", seq.getName(), seq.getAnnotation() );
    }

    /** Create a gapped sequence from a sequence that has gapped characters in it but isn't a gapped
     * sequence object
     *
     * @param gappedSequence A sequence that has gaps in it (as characters), but isn't a gapped sequence
     * @return a properly constructed gapped sequence object
     */
    public static GappedSequence createGappedSequence(Sequence gappedSequence)
    {
        SimpleGappedSequence newGappedSequence = new SimpleGappedSequence(ungapSequence(gappedSequence));

        int j = 1;
        for (int i=1; i<= gappedSequence.length(); i++) {
            if ( gappedSequence.symbolAt(i).equals(gappedSequence.getAlphabet().getGapSymbol()) ) {
                try {
                    newGappedSequence.addGapInSource(j);
                }
                catch (IndexOutOfBoundsException e) {
                    e.printStackTrace();
                }
            }
            else
                j++;
        }

        if ( gappedSequence.length() != newGappedSequence.length() ) {
            throw new IllegalStateException("New gapped sequence doesn't match (Old = " + gappedSequence.length() + ", new = " + newGappedSequence.length());
        }
        return newGappedSequence;
    }

    /**
     * Removes some sequences from the alignment
     * Main method for removing sequences
     *
     * @param alignments
     * @param ppshMap
     */
    public static void chooseAlignments(Sequence querySequence, List<? extends Sequence> alignments, Map<String, Map<String, Double>> ppshMap, float minThreshold, float maxThreshold)
    {
        assert ppshMap != null : "PPSH Map is null!";

        String numAlignments = Config.getConfig().getStringValue("num_alignments");

        ArrayList sequencesForDeletion = new ArrayList();

        int alignmentCount = alignments.size();

        /* Start deleting the sequences with the highest identity */
        for (int i=0; i<alignments.size(); i++)
        {
            Sequence sequence1 = alignments.get(i);

            if ( sequencesForDeletion.contains(sequence1))
                continue;

            for (int j=i+1; j< alignments.size(); j++)
            {
                Sequence sequence2 = alignments.get(j);
                if (sequence1 == sequence2)
                    continue; // Don't compare against itself, obviously its sequence will be 100 identical

                if (sequence1.getName().equals(sequence2.getName()))
                    continue; // Don't compare against itself, obviously its sequence will be 100 identical

                if ( sequence2 == querySequence || sequence2.getName().equals(querySequence.getName()) )
                    continue; // don't remove query sequence!

                Map<String, Double> ppshRow = ppshMap.get(sequence1.getName());
                if ( ppshRow != null ) {
                    if ( ppshRow.get(sequence2.getName()) != null ) {
                        double ppsh = ppshRow.get(sequence2.getName());
                        if ((ppsh > maxThreshold ) || (ppsh < minThreshold ))
                        {
                            sequencesForDeletion.add(sequence2);       // mark a sequence for deletion
                            if (verbose) System.out.println("Deleting " + sequence2.getName() + ", similarity to " + sequence1.getName() + " = " + ppsh);
                        }
                    }
                }
                // otherwise that sequence isn't there anyways...
            }
        }

        // now remove them all
        for (Iterator it = sequencesForDeletion.iterator(); it.hasNext();) {
            Sequence s =(Sequence) it.next();
            // alignments.remove(s) didn't seem to work
            for (Iterator j = alignments.iterator(); j.hasNext(); ) {
                Object o = j.next();
                if ( o == s ) {
                    j.remove();
                    if (verbose) System.out.println("Deleting " + s.getName() );
                    break;
                }
            }
        }

        // If there's still too many, remove more
        chooseAlignments(querySequence,  alignments, Integer.valueOf(numAlignments));

        int numRemoved = alignmentCount - alignments.size();
        if ( verbose ) System.out.println("Removed " + numRemoved + " alignments");
    }

    /**
     * Removes a bunch of sequences, keeping only the specified number
    *
    */
    public static void chooseAlignments(Sequence querySequence, List<? extends Sequence> alignments, int numToKeep)
    {
        double cutoff = 0;
        if ( alignments.size() != 0 )
        {
            cutoff = 1 / (double) alignments.size();
        }

        while (alignments.size() > numToKeep)
        {
            int i = 0;
            for (Iterator<? extends Sequence> it = alignments.iterator(); it.hasNext(); i++)
            {
                Sequence sequence = it.next();

                if ( /*  Taken out 07/23/06, why would we want to cmopare to the query sequence?
                        "sequence" is gapped!
                            sequence.equals(querySequence)
                        || */
                        sequence.getName().equals(querySequence.getName())
                        || sequence.getName().equals("QUERY") )
                {
                    continue; // don't remove query sequence!
                }

                //Annotation annotation = sequence.getAnnotation();
                //Double evalue = (Double) annotation.getProperty("e-value");
                if ( Math.random() < cutoff)
                    it.remove();
                if (alignments.size() <= numToKeep)
                    return;
            }
        }

    }

    /**
     * Removes some sequences from the alignment according to the percent sequence similarity matrix
     *
     * @param alignment
     * @param ppshMap
     */
    public static Alignment chooseAlignments(Sequence sequence, Alignment alignment, Map<String, Map<String, Double>> ppshMap, float minThreshold, float maxThreshold)
    {
        // Build a list
        List<Sequence> alignments = new ArrayList<Sequence>();
        for ( Iterator it = alignment.symbolListIterator(); it.hasNext(); ) {
            alignments.add((Sequence) it.next());
        }

        // Remove from that list
        chooseAlignments(sequence, alignments, ppshMap, minThreshold, maxThreshold);

        // Put them back in
        HashMap<String, Sequence> sequenceCollection = new HashMap<String, Sequence>();
        for (Sequence key: alignments)
        {
            sequenceCollection.put(key.getName(), key);
        }
        // Build new Alignment
        return new SimpleAlignment(sequenceCollection);
    }

    /**
     * Creates an alignment object from a set of GappedSequence
     * @param sequences
     * @return an alignment
     */
    public static Alignment createAlignment(List<GappedSequence> sequences)
    {
        HashMap<String, Sequence> sequenceCollection = new HashMap<String, Sequence>();
        for (GappedSequence key : sequences)
        {
            sequenceCollection.put(key.getName(), key);
        }

        // Build new Alignment
        return new SimpleAlignment(sequenceCollection);
    }

    /**
     * Removes a bunch of sequences from an alignment, keeping only the specified number
     *
    */
    private static Alignment chooseAlignments(Sequence sequence, Alignment alignment, Integer numToKeep)
    {
        List<Sequence> alignments = alignment.toList();
        chooseAlignments(sequence, alignments, numToKeep);

        HashMap<String, Sequence> sequenceCollection = new HashMap<String, Sequence>();
        for (Sequence key : alignments)
        {
            sequenceCollection.put(key.getName(), key);
        }
        return new SimpleAlignment(sequenceCollection);
    }

    public static Sequence cleanSequence(StructureAnnotatedSequence sequence)
    {
        //Symbol[] sList = new Symbol[sequence.length()];
        String seqStr = sequence.seqString();
        char[] structString = sequence.getStructureAnnotation().seqString().toCharArray();
        for ( int i=0; i<seqStr.length(); i++) {
            if ( seqStr.charAt(i) == 'X') {
                structString[i] = '-';
            }
        }
        sequence.setStructureAnnotation( new SimpleGappedSymbolList(createStructure(new String(structString))) );

        return sequence;
    }

    public static String cleanSequence(String seqStr) {
        if ( seqStr == null )
            return null;

        seqStr = seqStr.toUpperCase();
        seqStr = seqStr.replaceAll("\\s","" );
        seqStr = seqStr.replaceAll(" ","" );

        if ( seqStr.matches("\\W|\\d|[JZOBUXa-z]"))
            return null;

        return seqStr;
    }

    public static SymbolList parsePHD(String s) {
        String[] lines = s.split("\\n");
        String structure = "";
        int j=0;
        for (String line : lines)
        {
            if (line.contains("AA  |"))
                break;
            j++;
        }
        for( int i=j; i<lines.length; i++)
        {
            if ( lines[i].contains("  PHD")) {
                lines[i] = lines[i].replaceAll("\\s*PHD\\s*\\|", "");
                lines[i] = lines[i].replaceAll("\\|\\s*", "");
                lines[i] = lines[i].replaceAll("\\s", "C");
                structure += lines[i];
            }
            if ( lines[i].equals("END")) {
                break;
            }
        }
        return ProteusTools.createStructure(ProteusTools.cleanSequence(structure));
    }

    public static boolean isGap(char c)
    {
        return c == '-' || c == '~';
    }

    public static boolean isGap(Symbol symbol)
    {
        return symbol.equals(AlphabetManager.getGapSymbol())
                || symbol.getName().toUpperCase().equals("GAP")
                || symbol.getName().toUpperCase().equals("[]")
                || symbol.getName().toLowerCase().equals("nothing");
    }

    public static List<Sequence> parseFasta(File f) throws IOException, BioException
    {
        return parseFasta(new FileReader(f));
    }

    public static List<Sequence> parseFasta(Reader f) throws IOException, BioException
    {
        BufferedReader bufferedReader = new BufferedReader(f);
        SymbolTokenization tokenization = ProteinTools.getAlphabet().getTokenization("token");

        List<Sequence> sequences = new ArrayList<Sequence>();
        final int READ_AHEAD_LIMIT = 10000;

        StringBuilder buffer = null;
        String header = getRandomID();
        String sequenceName = "";
        String line;
        while( (line = bufferedReader.readLine()) != null )
        {
            buffer = new StringBuilder();

            //line = bufferedReader.readLine().replaceAll("\\s+", "");

            // skip blank lines
            while (line.equals("") && (line = bufferedReader.readLine()) != null) {}

            if (line != null && line.startsWith(">"))
            {
                if ( line.length() > 7 )
                {
                    sequenceName = line.substring(1);
                    header = line.substring(0, 7); // Sequence ID must only be 8 chars or will crash in XAlign
                }
                else
                {
                    sequenceName = line.substring(1);
                    header = line;
                }
            }
            else
            {
                buffer.append(line);
            }

            // loop to read the sequence body
            while ((line = bufferedReader.readLine()) != null)
            {

                if ( line.startsWith(">"))
                {
                    // this is the next sequence, reset to end of last sequence
                    bufferedReader.reset();

                    sequences.add(buildSequence(buffer.toString(), tokenization, sequenceName, header));

                    // go on to next sequence
                    break;
                }

                buffer.append(line);

                // mark, since this might be the last line of the sequence
                bufferedReader.mark(READ_AHEAD_LIMIT);
            }
        }

        // add the final sequence
        assert buffer != null;
        sequences.add(buildSequence(buffer.toString(), tokenization, sequenceName, header));

        return sequences;
    }

    private static SimpleSequence buildSequence(String buffer, SymbolTokenization tokenization, String sequenceName, String header)
            throws IllegalSymbolException
    {
        String seqStr = ProteusTools.cleanSequence(buffer);

        SimpleSymbolList newList = new SimpleSymbolList(tokenization, seqStr);
        Annotation annotation = new SimpleAnnotation();

        // Keep the real sequence name for later
        try
        {
            annotation.setProperty(Consts.SEQUENCE_NAME, sequenceName);
        }
        catch (ChangeVetoException e)
        {
            e.printStackTrace();  
        }

        return new SimpleSequence(newList, "", header.substring(1), annotation);
    }

    private static String getRandomString()
    {
        return String.valueOf(Math.random()).replaceAll("\\.", "").substring(0, 6);
    }

    private static String getRandomID()
    {
        return getRandomString();
    }

    public static void loadDavesAlphabet()
    {
        try
        {
            InputStream alphabetStream = ClassTools.getClassLoader(AlphabetManager.class).getResourceAsStream(
                    "ca/ualberta/proteus/util/DavesAlphabet.xml"
            );
            if (alphabetStream == null)
            {
                throw new BioError("Couldn't locate DavesAlphabet.xml.");
            }
            InputSource is = new InputSource(alphabetStream);
            AlphabetManager.loadAlphabets(is);
        }
        catch (Exception t)
        {
            throw new BioError("Unable to initialize AlphabetManager", t);
        }
    }

    /**
     * Takes an alignment with a lot of gaps, and removes any gaps that are not common across all sequences
     * @param alignment
     * @return a reduced alignment
     */
    public static Alignment reduceAlignment(Alignment alignment)
    {
      /*  for ( Iterator iterator = alignment.symbolListIterator(); iterator.hasNext(); )
        {
            Sequence s = (Sequence) iterator.next();
            System.out.println(s.getName() + ": " + s.seqString());
        }*/

        // clone the alignment
        alignment = ProteusTools.cloneAlignment(alignment);

/*
        for ( Iterator iterator = alignment.symbolListIterator(); iterator.hasNext(); )
        {
            Sequence s = (Sequence) iterator.next();
            System.out.println(s.getName() + ": " + s.seqString());
        }
*/

        int index = 1;
        for ( int i=1; i<alignment.length(); i++)
        {
            boolean commonGap = true;
            for (Iterator it = alignment.symbolListIterator(); it.hasNext();)
            {
                GappedSequence sequence = (GappedSequence) it.next();
                if (! ProteusTools.isGap(sequence.subStr(index, index).charAt(0)) )
                {
                    commonGap = false;
                    break;
                }
            }

            try
            {
                if (commonGap)
                {
                    for (Iterator it = alignment.symbolListIterator(); it.hasNext();)
                    {
                        GappedSequence sequence = (GappedSequence) it.next();
                        sequence.removeGap(index);
                    }
                }
                else
                {
                    // not a common gap, won't remove anything, can increase the index
                    index++;
                }
            }
            catch (IllegalSymbolException e)
            {
                e.printStackTrace();
            }
        }

        return alignment;
    }

    public static Alignment cloneAlignment(Alignment alignment)
    {
        Map<String, Sequence> map = new HashMap<String, Sequence>();

        for (Iterator it = alignment.symbolListIterator(); it.hasNext();)
        {
            GappedSequence sequence = (GappedSequence) it.next();
            try
            {
                String protein = sequence.seqString();

                // stupid biojava is very finecky... it'll print out a sequence, but won't read it in correctly
                protein = protein.replace('~', '-');

                GappedSequence gappedSeq = ProteinTools.createGappedProteinSequence(protein, sequence.getName());
             //   gappedSeq = new SimpleGappedSequence(gappedSeq);
                map.put(sequence.getName(), gappedSeq);
            }
            catch (IllegalSymbolException e)
            {
                e.printStackTrace();
            }
        }

        return new SimpleAlignment(map);
    }

    /**
     * Creates a gapped structure
     *
     * Note: use this for serialization issues
     * 
     * @param structureStr
     * @return
     */
    public static GappedSymbolList createGappedStructure(String structureStr)
    {
        SimpleSymbolList struct = createStructure(structureStr.replace("-", ""));
        GappedSymbolList newGappedSymbolList = new SimpleGappedSymbolList(struct);

        int j = 1;
        // for (int i=1; i<= struct.length(); i++) {
        int i=1;
        while ( newGappedSymbolList.length() < structureStr.length() )
        {
            if ( isGap(structureStr.charAt(i-1)) ) {
                try {
                    newGappedSymbolList.addGapInSource(j);
                }
                catch (IndexOutOfBoundsException e) {
                    e.printStackTrace();
                }
            }
            else
                j++;
            i++;
        }

        assert structureStr.equals(newGappedSymbolList.seqString().replace('~', '-'));

        assert structureStr.length() == newGappedSymbolList.length();

        return newGappedSymbolList;
    }

    /**
     * This method exists to stop duplication of code.  We were using the same variation of code in
     * many different places to get the "correct" homology model from the RuntimeCatalog.
     *
     * Basically all this method does is look to see if minimization for a particular structure was successful,
     * and if so add it to the models, and if not, add the unminimized counterpart.
     *
     * @param logger RuntimeCatalog populated with data
     * @return a non-null (but possibly empty) list of models
     */
    public static List<Structure> getFinishedModels(RuntimeCatalog logger)
    {
        List<Structure> results = (List<Structure>) logger.getLogObject(Consts.HOMODELLER_RESULTS);
        List<Structure> models = new ArrayList<Structure>();
        if (results != null)
        {
            for (int i = 0; i < results.size(); i++)
            {
                Homodeller.Result result = (Homodeller.Result) results.get(i);

                List<GappedStructureAnnotatedSequence> allAlignedHomologs = (List<GappedStructureAnnotatedSequence>) logger.getLogObject(Consts.ALL_ALIGNED_HOMOLOG_LIST);
                List<GappedStructureAnnotatedSequence> alignedTMSequences = (List<GappedStructureAnnotatedSequence>) logger.getLogObject(Consts.TM_HOMOLOGS);

                boolean hasNoHomolog = allAlignedHomologs == null || (allAlignedHomologs.size() == 0);
                hasNoHomolog &= alignedTMSequences == null || alignedTMSequences.size() == 0;

                if (result != null && result.model != null && !hasNoHomolog)
                {
                    List minimizedModels = (List) logger.getLogObject(Consts.MINIMIZED_STRUCTURE);
                    Structure model = (Structure) minimizedModels.get(i);
                    if (model == null)
                    {
                        model = result.model;
                    }

                    models.add(model);
                }
            }
        }
        return models;
    }
}
