package ca.ualberta.proteus.util;

import java.io.*;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 8, 2005
 * Time: 11:35:13 AM
 * To change this template use File | Settings | File Templates.
 */
public class StreamHandler extends Thread
{
    InputStream is;
    String type;
    OutputStream os = null;
    StringBuilder inputString = new StringBuilder();
    boolean showedMsg = false;
    boolean finished = false;

    public StreamHandler(InputStream is, String type)
    {
        this(is, type, null);
    }

    public StreamHandler(InputStream is, String type, OutputStream redirect)
    {
        this.is = is;
        this.type = type;
        this.os = redirect;
    }

    public synchronized void run()
    {
        try
        {
            PrintWriter pw = null;
            if (usingExternalBuffer())
            {
                pw = new PrintWriter(os);
            }

            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line;
            while ((line = br.readLine()) != null)
            {
                if (usingExternalBuffer())
                {
                    assert pw != null;
                    pw.println(line);
                    pw.flush();
                }
                else
                {
                    inputString.append(line).append("\n");
                }

                // HACK: JVM does not support getBytes() on strings longer than this number
                if (!usingExternalBuffer() && inputString.length() > 16777217 && !showedMsg)
                {
                    System.err.println("JVM will not support string this long!");
                    System.err.flush();
                    showedMsg = true;
                }
            }
            if (usingExternalBuffer())
            {
                assert pw != null;
                pw.flush();
            }
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }

        finished = true;
    }

    private boolean usingExternalBuffer()
    {
        return os != null;
    }

    public synchronized String getInputString()
    {
        if ( usingExternalBuffer() )
        {
            throw new IllegalArgumentException("Assigned a print writer instead, input not kept in string!");
        }
        return inputString.toString();
    }

    /**
     * Returns true if finished
     * @return true if finished
     */
    public boolean isFinished()
    {
        return finished;
    }
}
