package ca.ualberta.proteus.util;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import org.biojava.bio.SimpleAnnotation;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.structure.Chain;
import org.biojava.bio.structure.ChainImpl;
import org.biojava.bio.structure.Group;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.structure.io.PDBFileParser;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Aug 11, 2005
 * Time: 10:03:02 PM
 * To change this template use File | Settings | File Templates.
 */
public class Threading2D
{
    private static String pdbDir = "pdb";

    public Structure getResult() throws FileNotFoundException
    {
        return getResult("2CAVA.pdb", 0, 0);
    }

    public Structure getResult(String PDBFileName, File searchDirectory, int start, int end )
    {
        return getResult(PDBFileName, new File[] {searchDirectory},  start, end);
    }

    public Structure getResult(String PDBFileName, int start, int end)
    {
        return getResult(PDBFileName, new File[] {}, start, end);
    }

    public Structure getResult(String PDBFileName, File[] searchDirectories, int start, int end)
    {
        PDBFileName = PDBFileName.toUpperCase();

        InputStream in = PDBFileLoader.getPDBFile(PDBFileName, searchDirectories);

        PDBFileParser pdbReader = new PDBFileParser();
        
        Structure structure = null;
        try
        {
            structure = pdbReader.parsePDBFile(in);
            int chainIdx = 0;
            char chainID = PDBFileName.charAt(4);
            if ( PDBFileName.length() == 5 )
            {
                List model = structure.getModel(0);
                for ( int i=0; i<model.size(); i++)
                {
                    Chain chain = (Chain) model.get(i);
                    String name = chain.getName();
                    if ( name.equals(chainID))
                    {
                        chainIdx = i;
                        System.out.println("Processing chain " + PDBFileName.charAt(4));
                        break;
                    }
                }
            }
            else
                chainIdx = 0;

            Chain chain = structure.getChain(chainIdx);

            ArrayList<Group> groups = chain.getGroups("amino");

            Chain newChain = new ChainImpl();
            end = end == 0 ? groups.size() : Math.min( end, groups.size() - 1);
            for( int i= start; i<end; i++ ) {
                newChain.addGroup( groups.get(i) );
            }

            // creat new structure with only selected chain
            Structure newStructure = new ProteusStructureImpl();
            newStructure.setConnections(structure.getConnections());
            newStructure.setHeader(structure.getHeader());
            newStructure.setName(structure.getName());
            newStructure.setNmr(structure.isNmr());
            newStructure.setPDBCode(structure.getPDBCode());
            newStructure.addModel( new ArrayList() );
            newStructure.addChain( newChain );

            structure = newStructure;

            return structure;
        }
        catch (IOException e)
        {
            e.printStackTrace();
            try
            {
                in.reset();
                BufferedReader reader = new BufferedReader(new InputStreamReader(in));
                String line;
                while ( (line = reader.readLine()) != null )
                    System.out.println(line);
            }
            catch (IOException e1)
            {
                e1.printStackTrace();
            }
        }
        return structure;
    }

    /**
     * Returns a PDB file of a structural homolog
     * @param sequence A sequence with a predicted structure
     * @return a structural homolog PDB representation
     */
    public Structure findStructuralHomolog(StructureAnnotatedSequence sequence)
    {
        Config config = Config.getConfig();

        BlastSettings settings = new BlastSettings();

        // the matrix has to be the specified one that's hacked
        settings.setMatrix( config.getStringValue("structureMatrix") );
        // sequence here must be a structure
        settings.setSequence( new SimpleSequence( sequence.getStructureAnnotation(), "STRUCTURE", "STRUCTURE", new SimpleAnnotation()));
        // BLAST against the structure database
        settings.setDatabase( config.getStringValue("structureDB") );

        /*
        settings.setBlastType(BlastSettings.BLASTTYPE.BLASTP);
        settings.setEvalue(1e-10);
        settings.setKeepOutput(true);
        settings.setGapExtensionPenalty(2);
        settings.setGapPenalty(7);
        settings.setMaxHits(20);
        */

        settings.setBlastType(BlastSettings.BLASTTYPE.PSIBLAST);
        settings.setKeepOutput(true);
        settings.setMaxHits(20);
        settings.setMatrix("BLOSUM62");
        settings.setSequence( sequence );
        settings.setAlignmentView(BlastSettings.ALIGNMENT_VIEW.BLUNT_ENDS);

        settings.setParse(false);

        BlastRunner runner = new BlastRunner(settings);
        runner.run();

        int start = -1, end = -1;
        String name = "";
        try
        {
            String output = runner.getOutput();

            System.out.println(output);

            BufferedReader reader = new BufferedReader( new StringReader(output));
            String line;
            while ( (line = reader.readLine() ) != null )
            {
                if ( line.startsWith(">"))
                    name = line.substring(1);
                else if ( start == -1 && line.contains("Sbjct:"))
                {
                    line = line.replaceFirst("Sbjct:\\s*(\\d+).*", "$1");
                    start = Integer.parseInt(line);
                }
                else if ( line.contains("Sbjct:"))
                {
                    line = line.replaceFirst("Sbjct:\\s*\\d+.*\\s+(\\d+)", "$1");
                    end = Integer.parseInt(line);
                }
                else if ( line.contains("Score") && start != -1 )
                    break;
            }

        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }


        /*List<Sequence> result = runner.getUnalignedResult();

        String name = result.get(0).getName();
        System.out.println("result = " + name);
        System.out.println(result.get(0).seqString());
          */
        return getResult(name, new File(pdbDir), start, end);

        //return threading2D.getResultFromStructuralHomologID(sequence, name);
    }

    public static void main(String[] arg) throws IOException, IllegalIDException
    {
        //Threading2D threading2D = new Threading2D(SeqSeeDB.getTestSequence());

        StructureAnnotatedSequence testSequence = (StructureAnnotatedSequence) SeqSeeDB.getTestSequence();

        Threading2D threading2D = new Threading2D();
        //Structure structuralHomolog = threading2D.findStructuralHomolog(testSequence);
        List<String> strings = threading2D.runThreadIt(testSequence);
        
    }

    private Structure getResultFromStructuralHomologID(StructureAnnotatedSequence testSequence, String structuralHomolog)

    {
        SeqSeeDB db = SeqSeeDB.getInstance();
        StructureAnnotatedSequence homologSequence = null;
        try
        {
            homologSequence = (StructureAnnotatedSequence) db.getSequence(structuralHomolog);
        }
        catch (IllegalIDException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        // get an alignment
        List<Sequence> seqList = new ArrayList<Sequence>(2);
        seqList.add(new SimpleSequence( testSequence.getStructureAnnotation(), testSequence.getName(), testSequence.getName(), new SimpleAnnotation() ));
        seqList.add(new SimpleSequence( homologSequence.getStructureAnnotation(),  homologSequence.getName(), homologSequence.getName(), new SimpleAnnotation() ));

        Alignment alignment;
        try {
            XAlignRunner runner = new XAlignRunner(testSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }
        catch ( ProcessNotCompleteException e ) {
            // Try ClustalW ?
            ClustalWRunner runner = new ClustalWRunner(testSequence, seqList);
            runner.setAddToDB(false);
            runner.setUseCache(false);
            runner.run();
            alignment = runner.getAlignment();
        }


        SymbolList alignedQuery = alignment.symbolListForLabel(testSequence.getName());
        SymbolList alignedHomolog = alignment.symbolListForLabel(homologSequence.getName());

        Symbol gapSymbol = alignedQuery.getAlphabet().getGapSymbol();

        System.out.println("alignedQuery   = " + alignedQuery.seqString());
        System.out.println("alignedHomolog = " + alignedHomolog.seqString());

        List<Symbol> alignedQueryList = new ArrayList<Symbol>();
        alignedQueryList.addAll(alignedQuery.toList());
        List<Symbol> alignedHomologList = new ArrayList<Symbol>();
        alignedHomologList.addAll(alignedHomolog.toList());

        // clean the list of mutual gaps
        for ( int i=1; i<alignedHomologList.size(); i++)
        {
            Symbol querySymbol = alignedQueryList.get(i);
            Symbol homologSymbol = alignedHomologList.get(i);

            if ( querySymbol.equals(gapSymbol) && homologSymbol.equals(gapSymbol))
            {
                alignedHomologList.remove(i);
                alignedQueryList.remove(i);
            }
        }

        int start = -1, end = -1;

        // find the start
        int i=0;
        while( i < alignedHomologList.size() )
        {
            Symbol querySymbol = alignedQueryList.get(i);
            Symbol homologSymbol = alignedHomologList.get(i);

            if ( start == -1 && ! querySymbol.equals(gapSymbol))
            {
                start = i;
                break;
            }

            i++;
        }

        // find the end
        i=alignedHomologList.size()-1;
        while( i >= 0 )
        {
            Symbol querySymbol = alignedQueryList.get(i);
            Symbol homologSymbol = alignedHomologList.get(i);

            if ( end == -1 && ! querySymbol.equals(gapSymbol))
            {
                end = i;
                break;
            }

            i--;
        }

        return getResult(structuralHomolog, new File(pdbDir), start, end);
    }

    public List<String> runThreadIt(StructureAnnotatedSequence sequence)
    {
        Config config = Config.getConfig();

        File inputFile = null;
        try
        {
            inputFile = File.createTempFile("structure", ".seqsee");

            PrintWriter writer = new PrintWriter(inputFile);
            String seqName = sequence.getName();
            if (seqName.length() == 4)
                seqName += "_";

            writer.println(">" + seqName);
            writer.println("#NUM RESIDU: " + sequence.length());

            String seqStr = sequence.seqString();
            String structStr = null;
            structStr = sequence.getStructureAnnotation().seqString();

            for (int i = 0; i < sequence.length(); i += 50)
            {
                String line = seqStr.substring(i, Math.min(i + 50, seqStr.length()));
                line = line.replaceAll("(\\w{10})", "$1 ");
                writer.println(line);

                line = structStr.substring(i, Math.min(i + 50, seqStr.length()));
                line = line.toUpperCase().replace("E", "B");
                line = line.replaceAll("(\\w{10})", "$1 ");
                writer.println(line);
                writer.println();
            }
            writer.close();

            String threadingCmd = config.getStringValue("threading.command");
            File runDir = new File(threadingCmd).getParentFile();

            threadingCmd = String.format(threadingCmd, inputFile.getAbsolutePath());


            Process threadingProc = Runtime.getRuntime().exec(threadingCmd, null, runDir.getAbsoluteFile() );

            int exitVal = threadingProc.waitFor();

            List<String> rv = new ArrayList<String>();
            BufferedReader reader = new BufferedReader( new FileReader(runDir.getAbsolutePath() + "/xiaoli"));
            String line;
            while ( (line = reader.readLine()) != null )
            {
                System.out.println(line);

                if ( line.contains("Title....") )
                {
                    line = line.replaceFirst("Title....: (\\w+)", "$1");
                    rv.add(line);
                }
            }
            return rv;
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        //return getUnalignedResult("1EIWA", new File(pdbDir), 0, 0);
        return null;
    }
}
