package ca.ualberta.proteus.util;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import org.biojava.bio.seq.Sequence;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Created by IntelliJ IDEA.
 * @author Scott Montgomerie
 * Date: Mar 8, 2005
 * Time: 9:25:24 PM
 * To change this template use File | Settings | File Templates.
 */
public abstract class TrainingSet implements Serializable
{
    protected ArrayList<double[]> inputArray = new ArrayList<double[]>(); //<double[]>
    protected ArrayList<double[]> outputArray = new ArrayList<double[]>();

    public abstract void addInstance(StructureAnnotatedSequence sequence);

    public abstract void addInstance(Sequence sequence);

    public Iterator getInputInstanceIterator() {
        return inputArray.iterator();
    }

    public Iterator getOutputInstanceIterator() {
        return outputArray.iterator();
    }

    /**
     * Write the training set into the Joone format, where each instance is doubles, separated by a ;
     * Each instance is on its own line
     * @param aOutputStream
     * @throws IOException
     */
    public void writeToStream(OutputStream aOutputStream) throws IOException
    {
        BufferedOutputStream buf = new BufferedOutputStream(aOutputStream);

        assert inputArray.size() == outputArray.size() ;

        for (int j=0; j<inputArray.size(); j++ ) {

            double[] inputs = (double[]) inputArray.get(j);
            double[] outputs = (double[]) outputArray.get(j);

            // write inputs
            for (int i=0; i<inputs.length; i++) {
                buf.write(String.valueOf(inputs[i]).getBytes());

                buf.write(';');
            }

            // followed by the outputs
            for (int i=0; i<outputs.length; i++) {
                buf.write(String.valueOf(outputs[i]).getBytes());

                // write all but the last ;
                if ( i != outputs.length - 1)
                    buf.write(';');
            }

            // end of instance
            buf.write('\n');
        }

        buf.close();
    }

    public abstract void readFromStream(InputStream aInputStream) throws IOException;

    public int numInstances() {
        return inputArray.size();
    }

    public int getInputInstanceSize()
    {
        if (inputArray.size() > 0)
            return ((double[]) inputArray.get(0)).length;
        else
            return 0;
    }

    public double[][] getInputArray() {
        int numInputs = getNumInputs();
        int numOutputs = getNumOutputs();

        double[][] rv = new double[inputArray.size()][numInputs + numOutputs];


        for (int i=0; i< inputArray.size(); i++ )  {
            double[] inputs = (double[]) inputArray.get(i);
            double[] outputs = (double[]) outputArray.get(i);
            System.arraycopy(inputs, 0, rv[i], 0, numInputs);
            System.arraycopy(outputs, 0, rv[i], numInputs, numOutputs);
        }
        return rv;
    }

    public int getNumInputs() {
        return ((double[]) inputArray.get(0)).length;
    }

    public int getNumOutputs() {
        return ((double[]) outputArray.get(0)).length;
    }    
}
