package ca.ualberta.proteus.util.db;

import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import com.db4o.Db4o;
import com.db4o.ObjectContainer;
import com.db4o.ObjectSet;
import com.db4o.query.Predicate;
//import com.db4o.tools.Defragment;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Jun 8, 2006
 * Time: 10:09:22 PM
 * To change this template use File | Settings | File Templates.
 */
class PredictionDB4O implements PredictionDB
{
    static ObjectContainer db;
    static int predictionCount = 0;

    private static final String DB_NAME = "DB4O.dat";

    private static final boolean enableLogging = false;

    public static PredictionDB4O getInstance()
    {
        Db4o.configure().optimizeNativeQueries(true);
        Db4o.configure().lockDatabaseFile(false);

        // This was causing problems .  If you add a new item to the database,  then try
        // to change it later it wasn't working.
        // When creating a new database, don't disable this.

        if ( new File(DB_NAME).exists() )
        {
            Db4o.configure().detectSchemaChanges(false);
        }

        Db4o.configure().objectClass(PredictionDB4ORecord.class).cascadeOnUpdate(true);
        Db4o.configure().objectClass(SequenceRecord.class).cascadeOnUpdate(true);

        return new PredictionDB4O();
    }

    private static void openDB()
    {
        if ( db == null ) {
            db = Db4o.openFile(DB_NAME);

            if ( enableLogging )
            {
             /*   (db).getNativeQueryHandler().addListener(new Db4oQueryExecutionListener() {
                     public void notifyQueryExecuted(NQOptimizationInfo info) {
                        System.err.println("DB4O: " + info);
                      }
                });*/
            }
        }

        /*
        ((YapStream)db).getNativeQueryHandler().addListener(new Db4oQueryExecutionListener() {
          public void notifyQueryExecuted(NQOptimizationInfo info) {
            System.err.println(info);
          }
        });
        */
    }

    public boolean contains(Sequence sequence)
    {
        return get(sequence) != null;
    }

    public boolean contains(Sequence sequence, String method)
    {
        PredictionDB4ORecord record = get(sequence);

        if ( record != null )
            return record.get(method) != null;
        else
            return false;
    }

    public PredictionDB4ORecord get(final Sequence sequence)
    {
        openDB();
        try
        {
        /*    PredictionDB4ORecord proto = new PredictionDB4ORecord(null);
            ObjectSet result = db.get(proto);
            while(result.hasNext()) {
                PredictionDB4ORecord predictionRecord = (PredictionDB4ORecord) result.next();
                if ( predictionRecord.getSequenceRecord().get().equals(sequence.seqString()))
                    return predictionRecord;
            }
            */

            final String s = sequence.seqString();

            // Sometimes DB4O doesn't work cuz the open didn't open
            int tryNum =0;
            while ( tryNum++ < 10 )
            {
                try
                {
                    List<PredictionDB4ORecord> list = db.query(new Predicate<PredictionDB4ORecord>()
                    {
                        public boolean match(PredictionDB4ORecord predictionDB4ORecord)
                        {
                            /*SequenceRecord sequenceRecord = predictionDB4ORecord.getSequenceRecord();
                            if ( sequenceRecord != null )
                            {
                                return s.equals(sequenceRecord.get());
                            }
                            return false;
                            */
                            /*
                            NOTE: If you are getting NullPointerExceptions here, the database is BAD.
                             predictionDB4ORecord.getSequenceRecord() should only be null if the DB did
                             not get instantiated properly (i.e. detectSchemaChanges wasn't true).

                             We cannot use the above code because it can't be optimized in DB4O.
                            */
                            return s.equals(predictionDB4ORecord.getSequenceRecord().getSequence());
                        }
                    });

                    if ( list.size() != 0 )
                    {
                        return list.get(0);
                    }
                    else
                    {
                        return null;
                    }
                }
                catch(Exception e)
                {
                    // guess it didn't work??
                    try
                    {
                        // maybe DB is having timing issues...
                        Thread.sleep(50);
                    }
                    catch (InterruptedException e1)
                    {
                        e1.printStackTrace();
                    }

                    openDB();
                }
            }
        }
        finally
        {
            closeDB();
        }
        return null;
    }

    public ScoreAnnotatedSequence get(Sequence sequence, String methodName)
    {
        PredictionDB4ORecord record = get(sequence);

        if ( record != null )
            return record.get(methodName);
        else
            return null;
    }

    public synchronized void add(Sequence querySequence, PredictionDB4ORecord predictionRecord)
    {
        openDB();
        try
        {
            db.set(predictionRecord);
            predictionCount++;
        }
        finally
        {
            closeDB();
        }
    }

    private static void closeDB()
    {
        if ( db != null)
            db.close();
        db = null;
    }

    public void remove(Sequence sequence, String methodName)
    {
        PredictionDB4ORecord record = get(sequence);
        if ( record != null)
        {
            record.remove(methodName);
        }
        openDB();
        try
        {
            db.set(record);
        }
        finally{
            closeDB();
        }

    }

    public synchronized void add(final Sequence querySequence, String methodName, ScoreAnnotatedSequence prediction)
    {
        final String s = querySequence.seqString();

        openDB();
        try
        {
            List<PredictionDB4ORecord> list = db.query(new Predicate<PredictionDB4ORecord>()
            {
                public boolean match(PredictionDB4ORecord predictionDB4ORecord)
                {
                    /*SequenceRecord sequenceRecord = predictionDB4ORecord.getSequenceRecord();
                    if ( sequenceRecord != null )
                    {
                        return s.equals(sequenceRecord.get());
                    }
                    return false;
                    */
                    /*
                    NOTE: If you are getting NullPointerExceptions here, the database is BAD.
                     predictionDB4ORecord.getSequenceRecord() should only be null if the DB did
                     not get instantiated properly (i.e. detectSchemaChanges wasn't true).

                     We cannot use the above code because it can't be optimized in DB4O.
                    */

                    return s.equals(predictionDB4ORecord.getSequenceRecord().getSequence());
                }
            });

            PredictionDB4ORecord record;
            if ( list.size() == 0 )
                record = new PredictionDB4ORecord(querySequence);
            else
                record = list.get(0);

            record.add(methodName, prediction);

            db.set(record);

           // PredictionDB4ORecord predictionDB4ORecord = get(querySequence);
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            closeDB();
        }


    }

    public void remove(StructureAnnotatedSequence sequence, String methodName)
    {
        throw new IllegalArgumentException("Not implemented");
    }

    public static void main(String[] args)
    {

      //  PredictionDB4O db = PredictionDB4O.getInstance();
      //  SeqSeeDB seqSeeDB = SeqSeeDB.getInstance();

        openDB();
        try
        {
            PredictionDB4ORecord proto = new PredictionDB4ORecord(null);
            ObjectSet result = PredictionDB4O.db.get(proto);
            System.out.println("DB4O size: " + result.size());
        }
        finally{
            closeDB();
        }


        try
        {
            // will open the DB
            fillDBfromOld();

            openDB();

            PredictionDB4ORecord proto = new PredictionDB4ORecord(null);
            ObjectSet result = PredictionDB4O.db.get(proto);
            while(result.hasNext()) {
                PredictionDB4ORecord x = (PredictionDB4ORecord) result.next();
                SequenceRecord sequence = x.getSequenceRecord();

                System.out.println( sequence.getName() );
                System.out.println( sequence );

                for ( Iterator i = x.iterator(); i.hasNext(); )
                {
                    String method = (String) i.next();
                    System.out.println( method );
               //     ScoreAnnotatedSequence prediction = x.get(method);
                    //System.out.println( prediction.seqString() );
                    //System.out.println( prediction.getStructureAnnotation().seqString() );
                }
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            closeDB();
        }
    }

    /// HACK
    public void exportToExternalDB()
    {
        try
        {
            openDB();

            PredictionDB4ORecord proto = new PredictionDB4ORecord(null);
            ObjectSet result = PredictionDB4O.db.get(proto);
            while(result.hasNext()) {
                PredictionDB4ORecord x = (PredictionDB4ORecord) result.next();
                SequenceRecord sequenceRec = x.getSequenceRecord();
                Sequence sequence = ProteinTools.createProteinSequence(sequenceRec.getSequence(), sequenceRec.getName());

                for ( Iterator i = x.iterator(); i.hasNext(); )
                {
                    String method = (String) i.next();
                    ScoreAnnotatedSequence prediction = x.get(method);

                    SQLPredictionDB.getInstance().add(sequence, method, prediction);
                }
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            closeDB();
        }
    }

    private static void fillDBfromOld()
    {
        XMLPredictionDB db = XMLPredictionDB.getInstance();
      //  PredictionDB4O newDB = PredictionDB4O.getInstance();

        openDB();
        try
        {
            System.out.println("DB size = " + db.size());
            int i=0;
            for (Iterator<Sequence> it = db.iterator(); it.hasNext();)
            {
                Sequence seq = it.next();

                PredictionRecord predictionRecord = db.get(seq);

                final String s = seq.seqString();
                List<PredictionDB4ORecord> list = PredictionDB4O.db.query(new Predicate<PredictionDB4ORecord>()
                {
                    public boolean match(PredictionDB4ORecord predictionDB4ORecord)
                    {
                        return s.equals(predictionDB4ORecord.getSequenceRecord().getSequence());
                    }
                });

                PredictionDB4ORecord record;
                if ( list.size() == 0 )
                    record = new PredictionDB4ORecord(seq);
                else
                    record = list.get(0);

                for ( Iterator predictions = predictionRecord.iterator(); predictions.hasNext();)
                {
                    String method = (String) predictions.next();
                    ScoreAnnotatedSequence scoreAnnotatedSequence = predictionRecord.get(method);

                    record.add(method, scoreAnnotatedSequence);

                    PredictionDB4O.db.set(record);
                }

                if ( i < 10 || i % 20 == 0 )
                {
                    System.out.println("Record " + i);
                }
                i++;
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        finally{
            closeDB();
        }
    }

    @Override
    protected void finalize() throws Throwable
    {
        closeDB();
        super.finalize();
    }

    public void defragment() {
        Db4o.configure().messageLevel(-1);

        //new Defragment().run(DB_NAME, true);
    }
}

class PredictionDB4ORecord
{
    /* Schema
        Seq Name
        Seq String
        Seq Struct
        Pred *
            <Name>
            <Struct>
            </>
        */
    SequenceRecord sequence;
    private Map<String, SequenceRecord> predictions = new HashMap<String, SequenceRecord>();

    public PredictionDB4ORecord(Sequence sequence)
    {
        this.sequence = new SequenceRecord(sequence);
    }

    public void add(String methodName, ScoreAnnotatedSequence prediction)
    {
        predictions.put(methodName, new SequenceRecord(prediction));
    }

    public void add(String methodName, SequenceRecord prediction)
    {
        predictions.put(methodName, prediction);
    }

    public void remove(String methodName)
    {
        predictions.remove(methodName);
    }

    public boolean contains(String methodName)
    {
        return predictions.containsKey(methodName);
    }

    public ScoreAnnotatedSequence get(String methodName)
    {
        SequenceRecord rec = predictions.get(methodName);
        if ( rec != null )
        {
            try
            {
                return new ScoreAnnotatedSequence(rec.sequence, rec.structure, rec.name, rec.score);
            }
            catch (IllegalSymbolException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        return null;
    }

    public void setSequence(StructureAnnotatedSequence sequence)
    {
        this.sequence = new SequenceRecord(sequence);
    }

    public SequenceRecord getSequenceRecord()
    {
        return sequence;
    }

    public Iterator iterator()
    {
        return predictions.keySet().iterator();
    }
}
