package ca.ualberta.proteus.util.db;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.transsec.PredictionScore;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.SimpleSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: Smontgomerie
 * Date: May 12, 2005
 * Time: 2:55:34 PM
 * To change this template use File | Settings | File Templates.
 */
class XMLPredictionDB implements PredictionDB, Serializable
{
    /* Schema
        Seq Name
        Seq String
        Seq Struct
        Pred *
            <Name>
            <Struct>
            </>
        */
    protected static final String DB_FILE = "predictionDB.xml";
    protected static final String DB_NAME = "CustalW results";
    private static Map<Sequence, PredictionRecord> predictionList = new HashMap<Sequence,PredictionRecord>();
    private static XMLPredictionDB db = null;

    public static XMLPredictionDB getInstance()
    {
        if (db == null)
        {
            db = loadDB();
        }
        return db;
    }

    /**
     * Reads in the blast database file
     *
     * @return    a filled DB
     */
    private synchronized static XMLPredictionDB loadDB()
    {
        XMLPredictionDB rv = new XMLPredictionDB();
        InputStream in = rv.getInputStream();
        if ( in == null )
        {
           // System.out.println("Prediction database does not exist... creating a new one.");
            return rv;
        }

        System.out.println("Loading Prediction DB...");
        try
        {
            synchronized (predictionList)
            {

//                SymbolTokenization tokenization = ProteinTools.getAlphabet().getTokenization("token");
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = null;
                try
                {
                    builder = factory.newDocumentBuilder();
                    Document doc = builder.parse(in);

                    Element root = doc.getDocumentElement();

                    for (int i = 0; i < root.getChildNodes().getLength(); i++)
                    {
                        Node node = root.getChildNodes().item(i);

                        if ( node.getNodeName().equals("record") ) {
                            String seqStr = null;
                            String structStr = null;
                            for ( int j=0; j < node.getChildNodes().getLength(); j++ ) {
                                Node subNode = node.getChildNodes().item(j);

                                if ( subNode.getNodeName().equals("sequence")) {
                                    seqStr = subNode.getTextContent();
                                }
                                else if ( subNode.getNodeName().equals("structure")) {
                                    structStr = subNode.getTextContent();
                                }
                                else if ( subNode.getNodeName().equals("predictions")) {
                                    String id = node.getAttributes().getNamedItem("ID").getNodeValue();
                                    Sequence sequence;
                                    if ( structStr != null )
                                        sequence = new StructureAnnotatedSequence(seqStr, structStr, id );
                                    else
                                        sequence = new SimpleSequence(ProteinTools.createProtein(seqStr), "", id, null);

                                    PredictionRecord record = new PredictionRecord(sequence);
                                    predictionList.put(sequence, record);

                                    for (int k = 0; k < subNode.getChildNodes().getLength(); k++)
                                    {
                                        Node predictionNode = subNode.getChildNodes().item(k);
                                        if ( !predictionNode.hasChildNodes() )
                                            continue;

                                        String annotation = null;
                                        String score = null;
                                        for ( int l=0; l < predictionNode.getChildNodes().getLength(); l++ ) {
                                            if ( predictionNode.getChildNodes().item(l).getNodeName().equals("annotation"))
                                                annotation = predictionNode.getChildNodes().item(l).getTextContent();
                                            else if ( predictionNode.getChildNodes().item(l).getNodeName().equals("score"))
                                                score = predictionNode.getChildNodes().item(l).getTextContent();
                                        }

                                        ScoreAnnotatedSequence prediction = new ScoreAnnotatedSequence(seqStr, annotation, id, score);

                                        record.add(predictionNode.getAttributes().getNamedItem("method").getNodeValue(),
                                                   prediction);
                                    }
                                }
                            }
                        }
                    }
                }
                catch (ParserConfigurationException e)
                {
                    e.printStackTrace();  //TODO
                }
                catch (SAXException e)
                {
                    e.printStackTrace();  //TODO
                }

            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (BioException e)
        {
            e.printStackTrace();  //TODO
        }
        System.out.println("Done");
        return new XMLPredictionDB();
    }

    private InputStream getInputStream()
    {
        InputStream in = getClass().getResourceAsStream(DB_FILE);

        if ( in == null )
            in = getClass().getResourceAsStream("/" + DB_FILE);

        if ( in == null ) {
            try
            {
                in = new FileInputStream(DB_FILE);
            }
            catch (FileNotFoundException e)
            {
            }
        }
        return in;
    }

    public boolean contains(Sequence sequence)
    {
        for (Iterator it = predictionList.keySet().iterator(); it.hasNext();)
        {
            Sequence seq = (Sequence) it.next();
            if (seq.getName().equals(sequence.getName()) || seq.seqString().equals(sequence.seqString()))
                return true;
        }
        return false;
    }

    public boolean contains(Sequence sequence, String method)
    {
        for (Sequence seq: predictionList.keySet())
        {
            if (seq.getName().equals(sequence.getName()) || seq.seqString().equals(sequence.seqString()))
            {
                PredictionRecord predictionRecord = predictionList.get(seq);
                return predictionRecord.contains(method);
            }
        }
        return false;
    }

    public PredictionRecord get(Sequence sequence)
    {
        for (Iterator it = predictionList.keySet().iterator(); it.hasNext();)
        {
            Sequence seq = (Sequence) it.next();
            if (seq.getName().equals(sequence.getName()) || seq.seqString().equals(sequence.seqString()))
                return predictionList.get(seq);
        }
        return null;
    }

    public ScoreAnnotatedSequence get(Sequence sequence, String methodName)
    {
        for (Iterator it = predictionList.keySet().iterator(); it.hasNext();)
        {
            Sequence seq = (Sequence) it.next();
            if (seq.getName().equals(sequence.getName()) || seq.seqString().equals(sequence.seqString()))
                return predictionList.get(seq).get(methodName);
        }
        return null;
    }

    /**
     * Writes out the contents of the database file
     */
    private synchronized void writeDB()
    {
        if ( true ) return;

        if ( ! Config.getConfig().getBooleanValue("storeResults") ) {
            if ( Config.getConfig().getBooleanValue("testing")  )
                System.out.print("Not writing Prediction DB... change \"storeResults\" to true in config file");
            return;
        }

        System.err.print("Writing Prediction DB...");
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            DOMImplementation impl = builder.getDOMImplementation();

            Document doc = impl.createDocument(null, "prediction_database", null);

            Element root = doc.getDocumentElement();
            Date today = new Date();
            Attr date = doc.createAttribute("updated");
            date.setValue(today.toString());
            root.getAttributes().setNamedItem(date);

            synchronized (predictionList)
            {
                for (Sequence sequence : predictionList.keySet())
                {
                    Sequence querySequence = (Sequence) sequence;
                    PredictionRecord record = predictionList.get(querySequence);

                    Element recordElement = doc.createElement("record");
                    recordElement.setAttribute("ID", querySequence.getName());
                    root.appendChild(recordElement);

                    Element seqElement = doc.createElement("sequence");
                    seqElement.setTextContent(querySequence.seqString());
                    recordElement.appendChild(seqElement);

                    if (querySequence instanceof StructureAnnotatedSequence)
                    {
                        Element structElement = doc.createElement("structure");
                        structElement.setTextContent(((StructureAnnotatedSequence) querySequence).getStructureAnnotation().seqString());
                        recordElement.appendChild(structElement);
                    }

                    Element predictions = doc.createElement("predictions");
                    recordElement.appendChild(predictions);

                    /* for each prediction add it to the tree */
                    for (Iterator predIt = record.iterator(); predIt.hasNext();)
                    {
                        Element prediction = doc.createElement("prediction");
                        predictions.appendChild(prediction);
                        String methodName = (String) predIt.next();
                        prediction.setAttribute("method", methodName);

                        Element annotation = doc.createElement("annotation");
                        prediction.appendChild(annotation);

                        ScoreAnnotatedSequence scoreAnnotatedSequence = record.get(methodName);
                        assert scoreAnnotatedSequence != null : "Prediction for " + methodName + " is null!";
                        assert scoreAnnotatedSequence.getStructureAnnotation() != null : "Prediction for " + methodName + " is null!";

                        annotation.setTextContent(scoreAnnotatedSequence.getStructureAnnotation().seqString());

                        Element score = doc.createElement("score");
                        prediction.appendChild(score);
                        score.setTextContent(scoreAnnotatedSequence.getScoresAsStr());
                    }
                }

                // Prepare the DOM document for writing, and the output file
                Source source = new DOMSource(doc);
                File file = new File(DB_FILE);
                Result result = new StreamResult(file);
                // Write the DOM document to file, using a document transformer
                Transformer transformer = TransformerFactory.newInstance().newTransformer();
                transformer.transform(source, result);
                System.err.println("Done");
            }
        }
        catch (ParserConfigurationException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (TransformerConfigurationException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (TransformerException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Add a sequence to the database
     *
     * @param querySequence    - for the query sequence
     * @param predictionRecord - for the query sequence
     */
    public synchronized void add(Sequence querySequence, PredictionRecord predictionRecord)
    {
        predictionList.put(querySequence, predictionRecord);
        writeDB();
    }

    /**
     * Add a sequence to the database
     *
     * @param querySequence - for the query sequence
     * @param methodName    - for the query sequence
     * @param prediction
     */
    public synchronized void add(Sequence querySequence, String methodName, ScoreAnnotatedSequence prediction)
    {

        PredictionRecord record;
        if (contains(querySequence))
        {
            record = get(querySequence);
        }
        else
        {
            record = new PredictionRecord(querySequence);
            add(querySequence, record);
        }

        record.add(methodName, prediction);

        writeDB();
    }

    public void remove(StructureAnnotatedSequence sequence, String methodName)
    {
        throw new IllegalArgumentException("Not implemented");
    }

    public void clearDB()
    {
        predictionList.clear();
        db = null;
    }

    public static void main(String[] args) throws BioException
    {
        XMLPredictionDB db = XMLPredictionDB.getInstance();
        SeqSeeDB seqSeeDB = SeqSeeDB.getInstance();
                             /*
        StructureAnnotatedSequence seq = (StructureAnnotatedSequence) seqSeeDB.getRandomSequences(1).get(0);
        try
        {
            db.add(seq, "PSIPRED", new ScoreAnnotatedSequence(seq));

            db.add(seq, "JNET", new ScoreAnnotatedSequence(seq));
            db.add(seq, "JNET2", new ScoreAnnotatedSequence(seq));
        }
        catch (IllegalSymbolException e)
        {
            e.printStackTrace();  //TODO
        }
        */
        /*Sequence sequence = seqSeeDB.get("1TL2A");
            if ( db.contains(sequence) ) {
                System.out.println("Contains");
                System.out.println(db.get(sequence, "PSIPRED"));
            } */
        System.out.print("DB has " + db.size() + " sequences");

        db.writeDB();
    }

    public int size()
    {
        return predictionList.size();
    }

    public Iterator<Sequence> iterator()
    {
        return predictionList.keySet().iterator();
    }
}

class PredictionRecord
{
    /* Schema
        Seq Name
        Seq String
        Seq Struct
        Pred *
            <Name>
            <Struct>
            </>
        */
    Sequence sequence;
    private Map<String, ScoreAnnotatedSequence> predictions = new HashMap<String,ScoreAnnotatedSequence>();

    public PredictionRecord(Sequence sequence)
    {
        this.sequence = sequence;
    }

    public void add(String methodName, ScoreAnnotatedSequence prediction)
    {
        predictions.put(methodName, prediction);
    }

    public boolean contains(String methodName)
    {
        return predictions.containsKey(methodName);
    }

    public ScoreAnnotatedSequence get(String methodName)
    {
        return predictions.get(methodName);
    }

    public void setSequence(StructureAnnotatedSequence sequence)
    {
        this.sequence = sequence;
    }

    public Sequence getSequence()
    {
        return sequence;
    }

    public Iterator iterator()
    {
        return predictions.keySet().iterator();
    }
}

class SequenceRecord
{
    String name;
    String sequence;
    String structure;
    PredictionScore[] score;

    public SequenceRecord(Sequence sequence)
    {
        if ( sequence != null )
        {
            this.name = sequence.getName();

            this.sequence = sequence.seqString();
            if ( sequence instanceof StructureAnnotatedSequence )
                this.structure = ((StructureAnnotatedSequence) sequence).getStructureAnnotation().seqString();
            if ( sequence instanceof ScoreAnnotatedSequence )
                this.score = ((ScoreAnnotatedSequence) sequence).getScores();
        }
    }

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public String getSequence()
    {
        return sequence;
    }

    public void setSequence(String sequence)
    {
        this.sequence = sequence;
    }

    public String getStructure()
    {
        return structure;
    }

    public void setStructure(String structure)
    {
        this.structure = structure;
    }

    public PredictionScore[] getScore()
    {
        return score;
    }

    public void setScore(PredictionScore[] score)
    {
        this.score = score;
    }
}
