package ca.ualberta.proteus.util.io;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.Consts;
import org.apache.log4j.Logger;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.db.IllegalIDException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: Smontgomerie
 * Date: Feb 28, 2005
 * Time: 12:46:17 PM
 * To change this template use File | Settings | File Templates.
 */

public class SeqSeeDB extends StructureDB implements Runnable
{
    private static final String DB_FILE = "seqsee.db.gz";
    private static SeqSeeDB db = null;
    private static final String DEFAULT_SEQUENCE = "1EIWA";

    public static SeqSeeDB getInstance()
    {
        if (db == null)
        {
            try
            {
                db = new SeqSeeDB();
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
        }
        return db;
    }

    public static void releaseDB() {
        db = null;
    }

    private SeqSeeDB() throws IOException
    {
        super( Config.getConfig().getStringValue(Consts.SEQSEE_DB, DB_FILE), StructureDB.MODE.READ );
        this.setDaemon(true);
        this.start();
    }

    public static void main(String[] argc)
    {
        if ( false ) {
            SeqSeeDB instance = SeqSeeDB.getInstance();
            System.out.print("DB has " + instance.size() + " sequences");

            SequenceIterator it = instance;
            while (it.hasNext())
            {
                try
                {
                    System.out.println( it.nextSequence().toString() );
                }
                catch (BioException e)
                {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }
        else if (argc[0].equals("-num")) {
            try
            {
                SeqSeeDB instance = SeqSeeDB.getInstance();
                System.out.println( "Size = " + instance.size() );
                System.out.println( instance.getStats() );
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        else if ( argc[0].equals("-makePDB") )
        {
            try
            {
                SeqSeeDB instance = SeqSeeDB.getInstance();
                String fileName = "nrPDB";
                PrintWriter writer = new PrintWriter( fileName );
                System.out.print("DB has " + instance.size() + " sequences");

                SequenceIterator it = instance;
                while (it.hasNext())
                {
                    StructureAnnotatedSequence seq = (StructureAnnotatedSequence) it.nextSequence();
                    if ( seq.length() > 0 )
                    {
                        writer.println(">" + seq.getName());
                        //writer.println(seq.getStructureAnnotation().seqString());
                        writer.println(seq.seqString());
                    }
                }
                writer.close();
                SeqSeeDB.releaseDB();

                System.out.println("DB is in " + fileName);

            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            catch (IOException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        else if ( argc[0].equals("-makeStructureDB") )
        {
            try
            {
                SeqSeeDB instance = SeqSeeDB.getInstance();
                PrintWriter writer = new PrintWriter( "structures.db" );

                SequenceIterator it = instance;
                while (it.hasNext())
                {
                    StructureAnnotatedSequence seq = (StructureAnnotatedSequence) it.nextSequence();
                    writer.println(">" + seq.getName());
                    writer.println(seq.getStructureAnnotation().seqString());
                    //writer.println(seq.seqString());
                }
                writer.close();
                SeqSeeDB.releaseDB();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            catch (IOException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        else
            System.out.println("Options: -makePDB | -makeStructureDB | -num");
    }

    public static Sequence getTestSequence() throws IllegalIDException
    {
        if ( db == null )
            try
            {
                db = new SeqSeeDB();
            }
            catch (IOException e)
            {
                e.printStackTrace();  
            }
        return db.getSequence(DEFAULT_SEQUENCE);
    }

    /**
     * Returns a set of random sequences in the ID'DEFAULT_SEQUENCE of the database specified by the range low-high
     *
     * @param numToGet - The number of sequences to return
     * @param low - the lowest sequence number
     * @param high - the highest sequence number
     * @deprecated
     * *
     */
    public List getRandomSequences(int numToGet, int low, int high)
    {
        List seqList = new ArrayList();

        for (int i=0; i<numToGet; i++) {

            int random = (int) Math.round( Math.random() * ( high - low ) ) + low;            

            try
            {
                seqList.add( (StructureAnnotatedSequence) getSequence(random) );
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }

        return seqList;
    }

    /**
     * Returns a set of random sequences throughout the entire database
     *
     * @param numToGet - The number of sequences to return
     * @deprecated
     */
    public List getRandomSequences(int numToGet)
    {
        return getRandomSequences(numToGet, 0, sequences.size() - 1);
    }

    /**
     * Gets a non-random set of sequences starting at sequence 0
     * @param numToGet - number of sequences to return
     * @return an evaluation set
     */
    public List getEvaluationSet(int numToGet)
    {
        return getEvaluationSet(numToGet, 0, 1000);
    }

    /**
     * Gets a non-random set of sequences from the sequence database in the range of 1000 + numToGet
     * @param numToGet - number of sequences to return
     * @return an evaluation set
     */
    public List<StructureAnnotatedSequence> getTrainingSet(int numToGet)
    {
        List<StructureAnnotatedSequence> seqList = new ArrayList<StructureAnnotatedSequence>();

        for (int i=0; i<numToGet*2; i+=2) {
            try
            {
                seqList.add( (StructureAnnotatedSequence) getSequence( i + 1000 ) );               
            }
            catch (BioException e)
            {
                e.printStackTrace();
            }
        }

        return seqList;
    }

    public List getEvaluationSet(int numToGet, int low, int high)
    {
        List<StructureAnnotatedSequence> seqList = new ArrayList<StructureAnnotatedSequence>();

        for (int i=0; i<numToGet*3; i+=3 ) {
            try
            {
                seqList.add( (StructureAnnotatedSequence) getSequence( i + low ) );
            }
            catch (BioException e)
            {
                e.printStackTrace();
            }
        }

        return seqList;
    }

    /**
     *   Gets a random evaluation set, based on a random interval with the maximum of the 'interval' param
     * @param numToGet
     * @param low
     * @param high
     * @param interval
     * @return
     */
    public List getEvaluationSet2(int numToGet, int low, int high, int interval)
    {
        List<StructureAnnotatedSequence> seqList = new ArrayList<StructureAnnotatedSequence>();

        interval = (int) (Math.random() * interval);
        for (int i=0; i < Math.min(numToGet*interval, high); i+=interval) {
            try
            {
                seqList.add( (StructureAnnotatedSequence) getSequence( i + low ) );
            }
            catch (BioException e)
            {
                e.printStackTrace();
            }
        }

        return seqList;
    }

    /**
     * Ensures that the database is updated continually
     */
    public void run()
    {

        Config config = Config.getConfig();

        final int INTERVAL_MINUTES = config.getIntValue(Consts.DB_RELOAD_INTERVAL, 60);
        final int INTERVAL_MILLIS = INTERVAL_MINUTES * 60 * 1000;

        boolean reloadDB = Config.getConfig().getBooleanValue("reload_seqsee_db");

        File f = new File(Config.getConfig().getStringValue(Consts.SEQSEE_DB, DB_FILE));

        long time = f.lastModified();

        // check date
        if ( reloadDB )
        {
            org.apache.log4j.Logger logger = Logger.getLogger(SeqSeeDB.class);

            while( true)
            {
                try
                {
                    Thread.sleep(INTERVAL_MILLIS);
                }
                catch (InterruptedException e)
                {
                    // Nada
                }
                finally
                {
                    f = new File(Config.getConfig().getStringValue(Consts.SEQSEE_DB, DB_FILE));

                    // if the file time has changed, reload
                    if ( f.lastModified() != time )
                    {
                        logger.debug("DB File: " + f.toString());
                        try
                        {
                            reloadDB();
                        }
                        catch (IOException e)
                        {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }
                    }

                    time = f.lastModified();
                }
            }
        }
    }
}
