package ca.ualberta.proteus.util.test;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.transsec.ExternalEvaluator;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.BlastRunner;
import ca.ualberta.proteus.util.BlastSettings;
import ca.ualberta.proteus.util.MyBlastParser;
import ca.ualberta.proteus.util.io.StructureDB;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.GappedSequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.utils.ChangeVetoException;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Mar 31, 2006
 * Time: 8:05:07 PM
 * To change this template use File | Settings | File Templates.
 */
public class BlastParserTest extends TestCase
{
    private static final String VAV1 = "MELWRQCTHWLIQCRVLPPSHRVTWDGAQVCELAQALRDGVLLCQLLNNLLPHAINLREV" +
            "NLRPQMSQFLCLKNIRTFLSTCCEKFGLKRSELFEAFDLFDVQDFGKVIYTLSALSWTPI" +
            "AQNRGIMPFPTEEESVGDEDIYSGLSDQIDDTVEEDEDLYDCVENEEAEGDEIYEDLMRS" +
            "EPVSMPPKMTEYDKRCCCLREIQQTEEKYTDTLGSIQQHFLKPLQRFLKPQDIEIIFINI" +
            "EDLLRVHTHFLKEMKEALGTPGAANLYQVFIKYKERFLVYGRYCSQVESASKHLDRVAAA" +
            "REDVQMKLEECSQRANNGRFTLRDLLMVPMQRVLKYHLLLQELVKHTQEAMEKENLRLAL" +
            "DAMRDLAQCVNEVKRDNETLRQITNFQLSIENLDQSLAHYGRPKIDGELKITSVERRSKM" +
            "DRYAFLLDKALLICKRRGDSYDLKDFVNLHSFQVRDDSSGDRDNKKWSHMFLLIEDQGAQ" +
            "GYELFFKTRELKKKWMEQFEMAISNIYPENATANGHDFQMFSFEETTSCKACQMLLRGTF" +
            "YQGYRCHRCRASAHKECLGRVPPCGRHGQDFPGTMKKDKLHRRAQDKKRNELGLPKMEVF" +
            "QEYYGLPPPPGAIGPFLRLNPGDIVELTKAEAEQNWWEGRNTSTNEIGWFPCNRVKPYVH" +
            "GPPQDLSVHLWYAGPMERAGAESILANRSDGTFLVRQRVKDAAEFAISIKYNVEVKHIKI" +
            "MTAEGLYRITEKKAFRGLTELVEFYQQNSLKDCFKSLDTTLQFPFKEPEKRTISRPAVGS" +
            "TKYFGTAKARYDFCARDRSELSLKEGDIIKILNKKGQQGWWRGEIYGRVGWFPANYVEED" +
            "YSEYC";

    public static Test suite()
    {
        return new TestSuite(BlastParserTest.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void testParse() throws IllegalSymbolException
    {
        Sequence seq = ProteinTools.createProteinSequence(VAV1, "QUERY");

        BlastSettings settings = new BlastSettings();
        settings.setSequence(seq);
        settings.setBlasttype(BlastSettings.BLASTTYPE.PROTEUS_BLASTPARSER_BLASTP);
        settings.setEvalue(1e-7);
        settings.setDatabase(Config.getConfig().getStringValue("PDB"));

        BlastRunner blastRunner = new BlastRunner(settings);
        blastRunner.run();

        List<GappedSequence> blastSequences = blastRunner.getAlignedResult();

        System.out.println("BlastP results: ");
        for ( Iterator<GappedSequence> iterator = blastSequences.iterator(); iterator.hasNext(); ) {
            Sequence sequence = iterator.next();
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }

        assertEquals( 941, blastSequences.get(0).length());
    }

    public void testOverlay() throws IllegalSymbolException
    {
        //Sequence seq = ProteinTools.createProteinSequence(VAV1, "QUERY");
        char[] c = new char[VAV1.length()];
        Arrays.fill(c, 'C');

        StructureAnnotatedSequence struct = new StructureAnnotatedSequence(VAV1, new String(c), "QUERY");
        ExternalEvaluator evaluator = new ExternalEvaluator("vadarEVA.db");
        ArrayList<StructureAnnotatedSequence> sequenceList = new ArrayList<StructureAnnotatedSequence>();
        sequenceList.add(struct );
        evaluator.run( sequenceList, -1);
    }

    public void testParsePSIBLAST() throws IllegalSymbolException
    {
        Sequence seq = ProteinTools.createProteinSequence(VAV1, "QUERY");

        BlastSettings settings = new BlastSettings();
        settings.setSequence(seq);
        settings.setBlasttype(BlastSettings.BLASTTYPE.PSIBLAST);
        settings.setAlignmentView(BlastSettings.ALIGNMENT_VIEW.BLUNT_ENDS);
        settings.setEvalue(1e-7);
        settings.setDatabase(Config.getConfig().getStringValue("blast.dbname"));

        BlastRunner blastRunner = new BlastRunner(settings);
        blastRunner.run();

        List<GappedSequence> blastSequences = blastRunner.getAlignedResult();

        System.out.println("PSI-Blast results: ");
        for ( Iterator<GappedSequence> iterator = blastSequences.iterator(); iterator.hasNext(); ) {
            Sequence sequence = iterator.next();
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }

      //  assertEquals( 937, blastSequences.get(0).length());
    }

    public void anotherTest() throws IOException
    {
        StructureDB db = new StructureDB("vadarEVA.db", StructureDB.MODE.READ);
        List<GappedSequence> blastSequences;
        try
        {
            Sequence seq = db.getSequence("1a47");
            BlastSettings settings = new BlastSettings();
            settings.setSequence(seq);
            settings.setBlasttype(BlastSettings.BLASTTYPE.PSIBLAST);
            settings.setEvalue(0.001);
            settings.setDatabase(Config.getConfig().getStringValue("PDB"));
            settings.setAlignmentView(BlastSettings.ALIGNMENT_VIEW.BLUNT_ENDS);

            BlastRunner blastRunner = new BlastRunner(settings);
            blastRunner.run();

            blastSequences = blastRunner.getAlignedResult();

            System.out.println("BlastP results: ");
            for ( Iterator<GappedSequence> iterator = blastSequences.iterator(); iterator.hasNext(); ) {
                Sequence sequence = iterator.next();
                System.out.println(sequence.getName() + ": " + sequence.seqString());
            }
        }
        catch (BioException e)
        {
            e.printStackTrace();
        }

    }

    public void testParsePSIBLASTWithTestData() throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        Sequence seq = ProteinTools.createProteinSequence(VAV1, "QUERY");

        List<GappedSequence> gappedSeqList = new ArrayList<GappedSequence>();

        MyBlastParser parser = new MyBlastParser();
        parser.parseBlastResult(seq, new File("test/test.psiblast"), gappedSeqList);

        System.out.println("PSI-Blast results: ");
        for (GappedSequence sequence : gappedSeqList)
        {
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }

      //  assertEquals( 937, blastSequences.get(0).length());
    }

    public void testParsePSIBLASTRandomAccessReader() throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        Sequence seq = ProteinTools.createProteinSequence(VAV1, "QUERY");

        List<GappedSequence> gappedSeqList = new ArrayList<GappedSequence>();

        MyBlastParser parser = new MyBlastParser();
        parser.parseBlastResult(seq,  new File("test/test.psiblast"), gappedSeqList);

        System.out.println("PSI-Blast results: ");
        for (GappedSequence sequence : gappedSeqList)
        {
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }

      //  assertEquals( 937, blastSequences.get(0).length());
    }

    public void testParse2() throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        fail("This test has changed");

        Sequence seq = ProteinTools.createProteinSequence("GKITFYEDRGFQGRHYECSSDHSNLQPYFSRCNSIRVDSGCWMLYE" +
                                                            "QPNFQGPQYFLRRGDYPDYQQWMGLNDSIRSCRLIPHTGSHRLRIYE" +
                "REDYRGQMVEITEDCSSLHDRFHFSEIHSFNVLEGWWVLYEMTNYRGRQYLLRPGDYRRYHDWGATNARVGSLRRAVDFY", "QUERY");

        List<GappedSequence> gappedSeqList = new ArrayList<GappedSequence>();

        MyBlastParser parser = new MyBlastParser();
        parser.parseBlastResult(seq,  new File("test/test2.blast"), gappedSeqList);

        System.out.println("Blast results: ");
        for (GappedSequence sequence : gappedSeqList)
        {
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }
    }

    public void testParse3() throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        Sequence seq = ProteinTools.createProteinSequence("MLSFLLTLKRMLRACLRAWKDKEFQVLFVLTILTLISGTIFYSTVEGLRPI" +
                "DALYFSVVTLTTVGDGNFSPQTDFGKIFTILYIFIGIGLVFGFIHKLAVNV", "QUERY");

        List<GappedSequence> gappedSeqList = new ArrayList<GappedSequence>();

        MyBlastParser parser = new MyBlastParser();
        //parser.parseBlastResult1(seq, new RandomAccessReader(new RandomAccessFile("test/test.psiblast", "r")), gappedSeqList);
        parser.parseBlastResult(seq,  new File("test/test3.blast"), gappedSeqList);

        System.out.println("Blast results: ");
        for (GappedSequence sequence : gappedSeqList)
        {
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }
    }


    public void testParse4() throws IllegalSymbolException, IOException, ChangeVetoException, SAXException
    {
        Sequence seq = ProteinTools.createProteinSequence(
                "MLARAARGTGALLLRGSLLASGRAPRRASSGLPRNTVVLFVPQQEAWVVERMGRFHRILEPGLNILI" +
                "PVLDRIRYVQSLKEIVINVPEQSAVTLDNVTLQIDGVLYLRIMDPYKASYGVEDPEYAVTQLAQTTM" +
                "RSELGKLSLDKVFRERESLNASIVDAINQAADCWGIRCLRYEIKDIHVPPRVKESMQMQVEAERRKR" +
                "ATVLESEGTRESAINVAEGKKQAQILASEAEKAEQINQAAGEASAVLAKAKAKAEAIRILAAALTQH" +
                "NGDAAASLTVAEQYVSAFSKLAKDSNTILLPSNPGDVTSMVAQAMGVYGALTKAPVPGTPDSLSSGS" +
                "SRDVQGTDASLDEELDRVKMS", "QUERY");

        List<GappedSequence> gappedSeqList = new ArrayList<GappedSequence>();

        MyBlastParser parser = new MyBlastParser();
        parser.parseBlastResult(seq,  new File("test/test4.blast"), gappedSeqList);

        System.out.println("Blast results: ");
        for (GappedSequence sequence : gappedSeqList)
        {
            System.out.println(sequence.getName() + ": " + sequence.seqString());
        }
    }
}
