package ca.ualberta.proteus.util.test;

import ca.ualberta.proteus.util.ClustalWRunner;
import ca.ualberta.proteus.util.ClustalWParser;
import junit.framework.TestCase;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.SymbolList;
import org.xml.sax.SAXException;

import java.util.ArrayList;
import java.util.List;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.FileInputStream;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 18, 2006
 * Time: 10:26:10 PM
 * To change this template use File | Settings | File Templates.
 */
public class ClustalWTest extends TestCase
{
    public void test1() throws IllegalSymbolException
    {
        String seq1 = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKS--ELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVR";
        String seq2 = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAAELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVR";

        run(seq1, seq2);
    }

    public void test2() throws IllegalSymbolException
    {
        String seq1 = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKS--ELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPV~~~~~~~~";
        String seq2 = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKSAAELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVR";

        run(seq1, seq2);
    }

    public void test3() throws IllegalSymbolException
    {
        String seq1 = "MNIFE--RIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAKS--ELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVR";
        String seq2 = "MNIFEMLRIDEGLRLKIYKDTEGYYTIGIG----KSPSLNAAKSAAELDKAIGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAK---VYDSLDAVR";

        run(seq1, seq2);
    }

    public void testParse() throws IllegalSymbolException, IOException, SAXException
    {
        String in = "CLUSTAL 2.0.1 multiple sequence alignment\n" +
                "\n" +
                "\n" +
                "\n" +
                "THIOREDOXIN_         SDKIIHLTDDSFDTDVLKADGAILVDFWAEWCGPCKMIAPILDEIADEYQGKLTVAKLNI 60\n" +
                "Other                SDKIIHLTDD-----VLKADGAILV--------PCKMIAPILDEIADEYQGKLTVAKLNI 47\n" +
                "                     **********     **********        ***************************\n" +
                "\n" +
                "\n" +
                "THIOREDOXIN_         DQNPGTAPKYGIRGIPTLLLFKNGEVAATKVGALSKGQLKEFLDAN 106\n" +
                "Other                DQNPGTAPKYGIRGIP-----------ATKVGALSKGQLKEFLDAN 82\n" +
                "                     ****************           *******************";

        ClustalWParser parser = new ClustalWParser();
        parser.parse(new ByteArrayInputStream(in.getBytes()));

        Alignment alignment = parser.getAlignment();
        for ( Object s: alignment.getLabels())
        {
            SymbolList symbolList = alignment.symbolListForLabel(s);

            System.out.println(symbolList.seqString());
        }
    }

    public void testParseFile() throws IllegalSymbolException, IOException, SAXException
    {
        ClustalWParser parser = new ClustalWParser();
        parser.parse(new FileInputStream("test/homodeller_aln.aln"));

        Alignment alignment = parser.getAlignment();
        for ( Object s: alignment.getLabels())
        {
            SymbolList symbolList = alignment.symbolListForLabel(s);

            System.out.println(symbolList.seqString());
        }
    }


    private void run(String seq1, String seq2)
            throws IllegalSymbolException
    {
        Sequence  query = ProteinTools.createProteinSequence(seq1.replace("-", "").replace("~", ""), "QUERY");
        Sequence sequence2 = ProteinTools.createProteinSequence(seq2.replace("-", "").replace("~", ""), "OTHER");

        System.out.println(query.seqString());
        System.out.println(sequence2.seqString());

        List<Sequence> sequences = new ArrayList<Sequence>(2);
        sequences.add(0, query);
        sequences.add(1, sequence2);

        ClustalWRunner runner = new ClustalWRunner(query, sequences);
        runner.run();
        Alignment alignment = runner.getAlignment();

        for ( Object s: alignment.getLabels())
        {
            SymbolList symbolList = alignment.symbolListForLabel(s);

            System.out.println(symbolList.seqString());
        }

        assertEquals(seq1, alignment.symbolListForLabel("QUERY").seqString() );
        assertEquals(seq2, alignment.symbolListForLabel("OTHER").seqString() );
    }
}
