package ca.ualberta.proteus.util.test;

import ca.ualberta.proteus.program.transsec.ExternalEvaluator;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.Score;
import ca.ualberta.proteus.util.io.StructureDB;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.BioException;
import org.biojava.bio.symbol.SymbolList;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Mar 31, 2006
 * Time: 11:51:22 PM
 * To change this template use File | Settings | File Templates.
 */
public class EVAQ3Tests extends TestCase
{
    public static Test suite()
    {
        return new TestSuite(EVAQ3Tests.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void runSAM() throws IOException
    {
        runTests("test/SAM_T99.txt");
    }

    public void runPROFSEC() throws IOException
    {
        runTests("test/PROFSEC.txt");
    }

    public void runPROF_KING() throws IOException
    {
        runTests("test/PROF_KING.txt");
    }

    public void runPORTER() throws IOException
    {
        runTests("test/PORTER.txt");
    }

    public void runYaspin() throws IOException
    {
        runTests("test/Yaspin.txt");
    }

    public void runProteus() throws IOException
    {
        StructureDB testDB = new StructureDB("test/PORTER.txt", StructureDB.MODE.READ);
        StructureDB vadarDB = new StructureDB("vadarEVA.db", StructureDB.MODE.READ);
        List<StructureAnnotatedSequence> list = new ArrayList<StructureAnnotatedSequence>();

        for ( int i = 0; i< testDB.size(); i++ )
        {
            try
            {
                StructureAnnotatedSequence seq = (StructureAnnotatedSequence) testDB.getSequence(i);
                String name = seq.getName().replace(":", "");
                StructureAnnotatedSequence vadarSequence = (StructureAnnotatedSequence) vadarDB.getSequence(name);
                list.add(vadarSequence);

                if (seq.length() != vadarSequence.length())
                {
                    System.out.println(name + " does not match DB");
                    System.out.println("EVA: " + seq.seqString());
                    System.out.println("VAD: " + vadarSequence.seqString());
                    continue;
                }
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }

        ExternalEvaluator evaluator = new ExternalEvaluator("vadarEVA.db");
        evaluator.run(list, 1e-40);
    }

    public void runTests(String dbFile) throws IOException
    {
        StructureDB testDB = new StructureDB(dbFile, StructureDB.MODE.READ);
        StructureDB vadarDB = new StructureDB("vadarEVA.db", StructureDB.MODE.READ);
        Map<String, Score> scores = new HashMap<String, Score>();

        for ( int i = 0; i< testDB.size(); i++ ) {
            try
            {
                StructureAnnotatedSequence seq = (StructureAnnotatedSequence) testDB.getSequence(i);
                String name = seq.getName().replace(":", "");
                StructureAnnotatedSequence vadarSequence = (StructureAnnotatedSequence) vadarDB.getSequence(name);

                if ( seq.length() != vadarSequence.length())
                {
                    System.out.println(name + " does not match DB");
                    System.out.println("EVA: " + seq.seqString());
                    System.out.println("VAD: " + vadarSequence.seqString());
                    continue;
                }

                assert seq.length() == vadarSequence.length() : name + " does not match DB";

                double q3;
                double sov;
                SymbolList vadar = vadarSequence.getStructureAnnotation();
                SymbolList eva = seq.getStructureAnnotation();

                assert eva.length() == vadar.length();

                if ( eva.length() != seq.length())
                {
                    continue;
                }

                q3 = ProteusTools.q3Score(eva, vadar);
                sov = ProteusTools.sovScore(seq.getStructureAnnotation(), vadarSequence.getStructureAnnotation());

                scores.put(name, new Score(q3, sov));
            }
            catch (BioException e)
            {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }

        PrintWriter writer = new PrintWriter(dbFile + ".csv");
        for(String name: scores.keySet())
        {
            Score score = scores.get(name);
            System.out.printf("%s\t%1.2f\t%1.2f\n", name, score.q3, score.sov);
            writer.printf("%s\t%1.2f\t%1.2f\n", name, score.q3, score.sov);
        }
        writer.close();
    }
}
