package ca.ualberta.proteus.util.test;

import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.transsec.ExternalEvaluator;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.util.db.PredictionDB;
import ca.ualberta.proteus.util.db.PredictionDBFactory;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SymbolList;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Mar 31, 2006
 * Time: 11:20:13 PM
 * To change this template use File | Settings | File Templates.
 */
public class SpotTest extends TestCase
{
    public static Test suite()
    {
        return new TestSuite(SpotTest.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    private void runSequence(String seqID)
            throws IllegalIDException, IOException
    {
        StructureDB db = new StructureDB("vadarEVA.db", StructureDB.MODE.READ);

        StructureAnnotatedSequence seq = (StructureAnnotatedSequence) db.getSequence(seqID);

        ExternalEvaluator evaluator = new ExternalEvaluator("vadarEVA.db");
        ArrayList<StructureAnnotatedSequence> sequenceList = new ArrayList<StructureAnnotatedSequence>();
        sequenceList.add(seq);
        evaluator.run(sequenceList, 1e-40);
    }

    public void test1A32() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1a32");
    }

    public void test1AML() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1aml");
    }

    public void test3() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1bcsB");
    }

    public void test1CIRA() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1cirA");
    }

    public void test1CL4A() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1cl4A");
    }

    public void test4() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1a4tB");
    }

    public void test5() throws IllegalSymbolException, IllegalIDException, IOException
    {
        runSequence("1a61");
    }

    public void test6() throws IOException
    {
        int start = 0, end = 300;
        List<StructureAnnotatedSequence> list = new ArrayList<StructureAnnotatedSequence>();
        StructureDB db = new StructureDB("/home/scott/proteus_web/EVA_sequences_final_set.db", StructureDB.MODE.READ);

        if (start < 0)
            start = 0;
        if (end < 0)
            end = db.size();

        BufferedReader reader = new BufferedReader(new FileReader("/home/scott/proteus/WebContent/EVA/EVA_sequences_final_set.txt"));

        String line;
        int i = 0;
        while ((line = reader.readLine()) != null)
        {
            boolean add = false;
            if (i >= start && i <= end)
                add = true;

            i++;

            if (add)
            {
                try
                {
                    list.add((StructureAnnotatedSequence) db.getSequence(line));
                }
                catch (IllegalIDException e)
                {
                    System.out.println(line + " does not exist");
                }
            }
        }

        final List<ScoreAnnotatedSequence> predictions = new ArrayList<ScoreAnnotatedSequence>();

        final ProteusSettings settings = new ProteusSettings();
        settings.setMethod(MethodSettings.PSIPRED_JNET);
        settings.setEvaluating(true);

        final Score[] finalScore = new Score[list.size()];
        Score[] jnet = new Score[list.size()];
        Score[] psipred = new Score[list.size()];

        PredictionDB predictionDB = PredictionDBFactory.getRunTimeDB(true);


        System.out.println("Running on " + list.size() + " sequences");

        // run proteus on each sequence
        int index = 0;
        int numSequences = 0;
        for (final StructureAnnotatedSequence sequence : list)
        {
            settings.setSequence(sequence);

            final ProteusImpl proteus = new ProteusImpl(settings);

            final int index1 = index;

            proteus.addCompletionListener(new CompletionListener()
            {
                public void onComplete(boolean successful, String message)
                {
                    RuntimeCatalog logger = proteus.getLogger();

                    ScoreAnnotatedSequence predictedSequence = (ScoreAnnotatedSequence) logger.getLogObject(Consts.FINAL);

                    switch (settings.getMethod())
                    {
                        case PSIPRED:
                            predictedSequence = (ScoreAnnotatedSequence) logger.getLogObject(Consts.PSIPRED);
                            break;
                        case JNET:
                            predictedSequence = (ScoreAnnotatedSequence) logger.getLogObject(Consts.JNET);
                            break;
                        case TRANSSEC:
                            predictedSequence = (ScoreAnnotatedSequence) logger.getLogObject(Consts.TRANSSEC);
                            break;
                        case ALL:
                        case PSIPRED_JNET:
                        default:
                            break;
                    }

                    System.out.println("Final sequence:");
                    System.out.println(predictedSequence);

                    PredictionDB predictionDB = PredictionDBFactory.getRunTimeDB(true);

                    ScoreAnnotatedSequence jnetSequence = predictionDB.get(predictedSequence, "JNET");
                    ScoreAnnotatedSequence psipredSequence = predictionDB.get(predictedSequence, "PSIPRED");
                    ScoreAnnotatedSequence proteusSequence = predictionDB.get(predictedSequence, "PROTEUS");
                    ScoreAnnotatedSequence transsecSequence = predictionDB.get(predictedSequence, "TRANSSEC");
                    //List<ScoreAnnotatedSequence> homologSequences =
                    //        mainPredictor.searchForHomolog(predictedSequence, MainPredictor.DEFAULT_EVALUE, 1e-30, false );

                    /*if ( homologSequences.size() != 0 ) {
                        System.out.println("Homolog found : " + homologSequences.get(0).toString());
                        homologSequence = mainPredictor.overlayHomolog(finalSequence, homologSequences);
                    }
                    */

                    /* Do a replacement B->E */
                    SymbolList structure = sequence.getStructureAnnotation();
                    String structStr = structure.seqString().toUpperCase().replace("B", "E");
                    sequence.setStructureAnnotation(ProteusTools.createStructure(structStr));

                    finalScore[index1] = new Score();
                    finalScore[index1].q3 = ProteusTools.q3Score(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());
                    finalScore[index1].sov = ProteusTools.sovScore(sequence.getStructureAnnotation(), predictedSequence.getStructureAnnotation());

                }

            });

            try
            {
                proteus.run();

                numSequences++;
            }
            catch (Throwable t)
            {

            }

            index++;
        }

        Score avgScore = new Score();
        for (Score s : finalScore)
        {
            if (s != null)
            {
                avgScore.q3 += s.q3;
                avgScore.sov += s.sov;
            }
        }

        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);
        System.out.println("FINAL:\t"
                + "Q3: " + format.format(avgScore.q3 / (double) numSequences)
                + "\tSOV: " + format.format(avgScore.sov / (double) numSequences));
    }
}

