package ca.ualberta.proteus.util.test;

import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.program.transsec.Config;
import junit.framework.TestCase;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.db.IllegalIDException;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

/**
 * (C) Zigtag Inc. 2007
 * <p/>
 * User: scott
 * Date: May 17, 2007
 * Time: 11:28:12 AM
 */
public class StructureDBTest extends TestCase
{
    public void testRead() throws BioException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();
        assertTrue(db.size() > 0 );

        Sequence sequence = db.getSequence(0);
        System.out.println(sequence);        
        sequence = db.getSequence(1);
        System.out.println(sequence);
    }

    public void testSpeed() throws BioException, IOException
    {
        System.out.println("Read only:");
        StructureDB db = SeqSeeDB.getInstance();
        assertTrue(db.size() > 0 );

        System.out.println("Read-write (should be a lot slower):");
        db = new StructureDB(Config.getConfig().getStringValue(Consts.SEQSEE_DB), StructureDB.MODE.READWRITE);
    }

    public void testWrite() throws IOException, BioException
    {
        String SEQ1 = "ABCABCABC";
        String SEQ2 = "EFGEFGEFG";
        String STRUCTURE = "CCCCCCCCC";
        
        Sequence seq1 = new StructureAnnotatedSequence(SEQ1, STRUCTURE, "SEQ1");
        Sequence seq2 = new StructureAnnotatedSequence(SEQ2, STRUCTURE, "SEQ2");

        File testDB = new File("test.db");

        // Start fresh
        testDB.delete();

        StructureDB db = new StructureDB(StructureDB.MODE.WRITE);
        db.addSequence(seq1);
        db.addSequence(seq2);

        db.writeDB(new PrintStream(testDB));


        db = new StructureDB(testDB, StructureDB.MODE.READ);

        assertEquals(seq1.seqString(), db.getSequence(0).seqString());
        assertEquals(seq2.seqString(), db.getSequence(1).seqString());

        testDB.delete();
    }

     public void testReadWrite() throws IOException, BioException
    {
        String SEQ1 = "ABCABCABC";
        String SEQ2 = "EFGEFGEFG";
        String STRUCTURE = "CCCCCCCCC";

        Sequence seq1 = new StructureAnnotatedSequence(SEQ1, STRUCTURE, "SEQ1");
        Sequence seq2 = new StructureAnnotatedSequence(SEQ2, STRUCTURE, "SEQ2");

        File testDB = new File("test.db");

        // Start fresh
        testDB.delete();

        // Make a new Db
        StructureDB db = new StructureDB(StructureDB.MODE.WRITE);
        db.addSequence(seq1);
        db.addSequence(seq2);

        db.writeDB(new PrintStream(testDB));

        // Load it up
        db = new StructureDB(testDB, StructureDB.MODE.READWRITE);

        assertEquals(seq1.seqString(), db.getSequence(0).seqString());
        assertEquals(seq2.seqString(), db.getSequence(1).seqString());

        // write it out
        db.writeDB(testDB);

        // Read it in again, check structures
        db = new StructureDB(testDB, StructureDB.MODE.READ);

        assertEquals(2, db.size());
        assertEquals(seq1.seqString(), db.getSequence(0).seqString());
        assertEquals(seq2.seqString(), db.getSequence(1).seqString());

        testDB.delete();
    }

    public void testStructureDB() throws IllegalIDException
    {
        // 1UBQ should ALWAYS be there
        SeqSeeDB seeDB = SeqSeeDB.getInstance();
        Sequence sequence = seeDB.getSequence("1UBQ");

        System.out.println(sequence);
    }

    public void testSeqSeeDBReload() throws InterruptedException, BioException
    {
        // During this 20 seconds, make a modification to the database to see if it crashes
        Config.getConfig().setStringValue(Consts.DB_RELOAD_INTERVAL, "1");
        Config.getConfig().setStringValue("reload_seqsee_db", "true");
        SeqSeeDB db = SeqSeeDB.getInstance();
        Thread.sleep(2 * 1000 * 60);


        db.getSequence(1);
        db.getSequence(10);
    }
}
