package ca.ualberta.proteus.util.test;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.ScoreAnnotatedSequence;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.Threading2D;
import ca.ualberta.proteus.util.db.PredictionDB;
import ca.ualberta.proteus.util.db.PredictionDBFactory;
import ca.ualberta.proteus.util.db.DBType;
import ca.ualberta.proteus.program.transsec.MainPredictor;
import ca.ualberta.proteus.program.MethodSettings;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.seq.db.IllegalIDException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.SimpleAnnotation;

import java.io.PrintWriter;
import java.io.FileNotFoundException;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Feb 12, 2006
 * Time: 1:13:59 AM
 * To change this template use File | Settings | File Templates.
 */
public class Threading2DTest extends TestCase
{
    public static Test suite()
    {
        return new TestSuite(Threading2DTest.class);
    }

    public static void main(String[] args)
    {
        new junit.textui.TestRunner().doRun(suite());
    }

    public void testSimpleThreading()
    {
        StructureAnnotatedSequence testSequence = null;
        try
        {
            testSequence = (StructureAnnotatedSequence) SeqSeeDB.getTestSequence();
        }
        catch (IllegalIDException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        Threading2D threading2D = new Threading2D();
        Structure structuralHomolog = threading2D.findStructuralHomolog(testSequence);
        System.out.println(structuralHomolog.toPDB());
    }

    public void test1BFG() throws IllegalSymbolException, FileNotFoundException
    {
        String _1BFG = "PALPEDGGSGAFPPGHFKDPKRLYCKNGGFFLRIHPDGRVDGVREKSDPHIKLQLQAEERGVVSIKGVSANRYLAMKEDG" +
                "RLLASKSVTDECFFFERLESNNYNTYRSRKYTSWYVALKRTGQYKLGSKTGPGQKAILFLPMSAKS";
        testSequence(_1BFG, "1BFG");
    }

    public void test2FAL() throws IllegalSymbolException, FileNotFoundException
    {
        String _2FAL = "XSLSAAEADLAGKSWAPVFANKNANGLDFLVALFEKFPDSANFFADFKGKSVADIKASPKLRDVSSRIFTRLNEFVNNAA" +
                "NAGKMSAMLSQFAKEHVGFGVGSAQFENVRSMFPGFVASVAAPPAGADAAWTKLFGLIIDALKAAGA";
        testSequence(_2FAL, "2FAL");
    }

    public void test1HNF() throws IllegalSymbolException, FileNotFoundException
    {
        String _1HNF = "KEITNALETWGALGQDINLDIPSFQMSDDIDDIKWEKTSDKKKIAQFRKEKETFKEKDTYKLFKNGTLKIKHLKTDDQDI" +
                "YKVSIYDTKGKNVLEKIFDLKIQERVSKPKISWTCINTTLTCEVMNGTDPELNLYQDGKHLKLSQRVITHKWTTSLSAKF" +
                "KCTAGNKVSKESSVEPVSCPEK";
        testSequence(_1HNF, "1HNF");
    }

    public void test1LKKA() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "LEPEPWFFKNLSRKDAERQLLAPGNTHGSFLIRESESTAGSFSLSVRDFDQNQGEVVKHYKIRNLDNGGFYISPRITFPG" +
                "LHELVRHYTNASDGLCTRLSRPCQT";
        testSequence(sequence, "1LKKA");
    }

    public void test2MHR() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "GWEIPEPYVWDESFRVFYEQLDEEHKKIFKGIFDCIRDNSAPNLATLVKVTTNHFTHEEAMMDAAKYSEVVPHKKMHKDF" +
                "LEKIGGLSAPVDAKNVDYCKEWLVNHIKGTDFKYKGKL";
        testSequence(sequence, "2MHR");
    }

    public void test2PGD() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "AQADIALIGLAVMGQNLILNMNDHGFVVCAFNRTVSKVDDFLANEAKGTKVLGAHSLEEMVSKLKKPRRIILLVKAGQAV" +
                "DNFIEKLVPLLDIGDIIIDGGNSEYRDTMRRCRDLKDKGILFVGSGVSGGEDGARYGPSLMPGGNKEAWPHIKAIFQGIA" +
                "AKVGTGEPCCDWVGDDGAGHFVKMVHNGIEYGDMQLICEAYHLMKDVLGLGHKEMAKAFEEWNKTELDSFLIEITASILK" +
                "FQDADGKHLLPKIRDSAGQKGTGKWTAISALEYGVPVTLIGEAVFARCLSSLKDERIQASKKLKGPQNIPFEGDKKSFLE" +
                "DIRKALYASKIISYAQGFMLLRQAATEFGWTLNYGGIALMWRGGCIIRSVFLGKIKDAFDRNPGLQNLLLDDFFKSAVEN" +
                "CQDSWRRAISTGVQAGIPMPCFTTALSFYDGYRHAMLPANLIQAQRDYFGAHTYELLAKPGQFIHTNWTGHGGSVSSSSY" +
                "NA";
        testSequence(sequence, "2PGD");
    }

    public void test1PLC() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "IDVLLGADDGSLAFVPSEFSISPGEKIVFKNNAGFPHNIVFDEDSIPSGVDASKISMSEEDLLNAKGETFEVALSNKGEY" +
                "SFYCSPHQGAGMVGKVTVN";
        testSequence(sequence, "1PLC");
    }

    public void test1RCB() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "HKCDITLQEIIKTLNSLTEQKTLCTELTVTDIFAASKNTTEKETFCRAATVLRQFYSHHEKDTRCLGATAQQFHRHKQLI" +
                "RFLKRLDRNLWGLAGLNSCPVKEANQSTLENFLERLKTIMREKYSKCSS";
        testSequence(sequence, "1RCB");
    }

    public void test1THX() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "METAMSKGVITITDAEFESEVLKAEQPVLVYFWASWCGPCQLMSPLINLAANTYSDRLKVVKLEIDPNPTTVKKYKVEGV" +
                "PALRLVKGEQILDSTEGVISKDKLLSFLDTHLNNN";
        testSequence(sequence, "1THX");
    }

    public void test1UBI() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "MQIFVKTLTGKTITLEVEPSDTIENVKAKIQDKEGIPPDQQRLIFAGKQLEDGRTLSDYNIQKESTLHLVLRLRGG";
        testSequence(sequence, "1UBI");
    }

    public void test1UBSA() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence = "MERYENLFAQLNDRREGAFVPFVTLGDPGIEQSLKIIDTLIDAGADALELGVPFSDPLADGPTIQNANLRAFAAGVTPAQ" +
                "CFEMLALIREKHPTIPIGLLMYANLVFNNGIDAFYARCEQVGVDSVLVADVPVEESAPFRQAALRHNIAPIFICPPNADD" +
                "DLLRQVASYGRGYTYLLSRSGVTGAENRGALPLHHLIEKLKEYHAAPALQGFGISSPEQVSAAVRAGAAGAISGSAIVKI" +
                "IEKNLASPKQMLAELRSFVSAMKAASRA";
        testSequence(sequence, "1UBSA");
    }

    public void testCodwormHemoglobin() throws IllegalSymbolException, FileNotFoundException
    {
        String sequence =   "mhssivlatvlfvaiasasktrelcmkslehakvgtskeakqdgidlykhmfehypamkk"+
            "yfkhrenytpadvqkdpffikqgqnillachvlcatyddretfdayvgelmarherdhvk"+
            "vpndvwnhfwehfieflgskttldeptkhawqeigkefsheishhgrhsvrdhcmnsley" +
            "iaigdkehqkqngidlykhmfehyphmrkafkgrenftkedvqkdaffvnkdtrfcwpfv" +
            "ccdssyddeptfdyfvdalmdrhikddihlpqeqwhefwklfaeylnekshqhlteaekh" +
            "awstigedfaheadkhakaekdhhegehkeehh";
       testSequence(sequence, "HEMOGLOBIN");
    }

    public void test1K9JA() throws IllegalSymbolException, FileNotFoundException
    {
       String sequence =   "SGFVCNTCPEKWINFQRKCYYFGKGTKQWVHARYACDDMEGQLVSIHSPEEQDFLTKHAS" +
               "HTGSWIGLRNLDLKGEFIWVDGSHVDYSNWAPGEPTSRSQGEDCVMMRGSGRWNDAFCDR";
       testSequence(sequence, "19KJA");
    }

    public void test2TRXA() throws IllegalSymbolException, FileNotFoundException
    {
       String sequence =   "SDKIIHLTDDSFDTDVLKADGAILVDFWAEWCGPCKMIAPILDEIADEYQGKLTVAKLNIDQNPGTAPKYGIRGIPTLLL" +
               "FKNGEVAATKVGALSKGQLKEFLDANLA";
       testSequence(sequence, "2TRXA");
    }

    public void test1BIN() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   "VAFTEKQDALV SSSFEAFKAN IPQYSVVFYT SILEKAPAAK DLFSFLANGV" +
               "    DPTNPKLTGHA EKLFALVRDS AGQLKASGTV VADAALGSVH AQKAVTDPQF" +
               "    VVVKEALLKTI KAAVGDKWSD ELSRAWEVAY DELAAAIKKA";
       testSequence(sequence, "1BIN");
    }

    public void test1ODB() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   "DCPSDWSSYEG HCYKPFNEPK NWADAENFCT QQHTGSHLVS FQSTEEADFV\n" +
               "VKLAFQTFDYG IFWMGLSKIW NQCNWQWSNA AMLKYTDWAE ESYCVYFKST\n" +
               "NNKWRSITCRM IANFVCEFQA";
       testSequence(sequence, "1ODB");
    }

    public void test1AA1S() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   "MQVWPILNLKK YETLSYLPPL TTDQLARQVD YLLNNKWVPC LEFETDHGFV\n" +
               "YREHHNSPGYY DGRYWTMWKL PMFGCTDPAQ VLNELEECKK EYPNAFIRII\n" +
               "GFDSNREVQCI SFIAYKPAGY";
       testSequence(sequence, "1AA1S");
    }

    public void test1F6RA() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   " EQLTKCEVFRE LKDLKGYGGV SLPEWVCTTF HTSGYDTQAI VQNNDSTEYG\n" +
               "LFQINNKIWCK DDQNPHSSNI CNISCDKFLD DDLTDDIMCV KKILDKVGIN\n" +
               "YWLAHKALCSE KLDQWLCEKL\n";
       testSequence(sequence, "1F6RA");
    }

    public void test1EO1A() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   " MKIAIASSGTD LGSEVSRFFG RAPYFMIVEM KKGNIESSEV IENPSASASG\n" +
               "GAGIRTAQIIA NNGVKAVIAS SPGPNAFEVL NELGIKIYRA TGTSVEENLK\n" +
               "LFTEGNLEEIR SPGSGRGRRR R";
       testSequence(sequence, "1EO1A");
    }

    public void test1BJ3() throws IllegalSymbolException, FileNotFoundException
    {
        // leghemoglobin
       String sequence =   " DCPSGWSSYEG HCYKPFKLYK TWDDAERFCT EQAKGGHLVS IESAGEADFV\n" +
               "    AQLVTENIQNT KSYVWIGLRV QGKEKQCSSE WSDGSSVSYE NWIEAESKTC\n" +
               "    LGLEKETGFRK WVNIYCGQQN PFVCEA";
       testSequence(sequence, "1BJ3");
    }









    private void testSequence(String sequenceString, String sequenceName)
            throws IllegalSymbolException, FileNotFoundException
    {
        sequenceString = sequenceString.replaceAll("\\s", "");
        SymbolList protein = ProteinTools.createProtein(sequenceString);
        Sequence testSequence = new SimpleSequence(protein, sequenceName, sequenceName, new SimpleAnnotation());

        // predict the structure
        MethodSettings method= MethodSettings.ALL;
        MainPredictor mainPredictor = new MainPredictor(null, method);
        mainPredictor.useStoredNN = true;
        mainPredictor.testing = false;


        ScoreAnnotatedSequence predictedSequence = mainPredictor.predict(testSequence);

        PredictionDB predictionDB = PredictionDBFactory.getDB(DBType.DB4O);

        if ( method == MethodSettings.PSIPRED )
            predictedSequence = predictionDB.get(testSequence, "PSIPRED");

        System.out.println("Predicted Sequence: \n" + predictedSequence.toString());

        Threading2D threading2D = new Threading2D();

      /*
        For threading program
        List<String> strings = threading2D.runThreadIt(predictedSequence);
        for( String s : strings)
            System.out.println(s);   
            */

        /* For modified BLAST */
        Structure structuralHomolog = threading2D.findStructuralHomolog(predictedSequence );
        System.out.println(structuralHomolog.toPDB());

        PrintWriter pw = new PrintWriter("pdb/" + sequenceName + "_threading.pdb");
        pw.println(structuralHomolog.toPDB());
        pw.close();
    }
}
