package ca.ualberta.proteus.web;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.BlastRunner;
import ca.ualberta.proteus.util.ProteusTools;
import ca.ualberta.proteus.util.BlastSettings;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.biojava.bio.Annotation;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 11, 2005
 * Time: 5:09:22 PM
 * To change this template use File | Settings | File Templates.
 */
public class BlastAction extends Action
{

    public ActionForward execute(ActionMapping actionMapping, ActionForm actionForm,
                                 HttpServletRequest request, HttpServletResponse response) throws IOException, ClassNotFoundException, BioException
    {
        BlastForm blastAction = (BlastForm) actionForm;
        String sequence = blastAction.getSequence();

        System.out.println("Running BLAST Web Interface...");

        SymbolTokenization tokenization = ProteinTools.getAlphabet().getTokenization("token");
        BlastSettings settings = new BlastSettings();
        settings.setSequence(parseFasta(new StringReader(sequence), tokenization));
        settings.setBlastType(BlastSettings.BLASTTYPE.BLASTP);
        settings.setDatabase(Config.getConfig().getStringValue("PDB"));
        settings.setEvalue(0.01);
        settings.setKeepOutput( true );

        BlastRunner runner = new BlastRunner(settings);
        runner.run();

        List<Sequence> alignment = runner.getUnalignedResult();

        for (Iterator<Sequence> it = alignment.iterator(); it.hasNext();)
        {
            Sequence seq = it.next();
            Annotation annotation = seq.getAnnotation();
            Double evalue = (Double) annotation.getProperty("e-value");
            System.out.println(seq.getName() + ": " + evalue + " : " + seq.seqString());
        }
        request.setAttribute("blast", runner);
        return actionMapping.findForward("success");
    }

    private Sequence parseFasta(Reader reader, SymbolTokenization tokenization) throws IOException, IllegalSymbolException
    {
        BufferedReader bufferedReader = new BufferedReader(reader);
        StringBuilder buffer = new StringBuilder();
        String line = bufferedReader.readLine().replaceAll("\\s+", "");

        // skip blank lines
        while (line.equals("") && (line = bufferedReader.readLine()) != null) ;

        String header = "";
        if (line != null && line.startsWith(">"))
            header = line.substring(1); // Sequence ID must only be 8 chars or will crash in XAlign
        else
            buffer.append(line);

        while ((line = bufferedReader.readLine()) != null)
        {
            buffer.append(line);
        }

        String seqStr = ProteusTools.cleanSequence(buffer.toString());

        SimpleSymbolList newList = new SimpleSymbolList(tokenization, seqStr);
        return new SimpleSequence(newList, "", header, null);
    }

}
