package ca.ualberta.proteus.web;

import ca.ualberta.proteus.db.DB4OPool;
import com.db4o.ObjectContainer;
import com.db4o.ObjectSet;
import com.db4o.ext.DatabaseFileLockedException;
import org.biojava.bio.seq.Sequence;
import org.apache.log4j.Logger;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 11, 2005
 * Time: 8:31:43 PM
 * To change this template use File | Settings | File Templates.
 */
public class DB4OWebHitCounter extends WebHitCounter
{
    private Logger logger = Logger.getLogger(DB4OWebHitCounter.class);

    static class Record
    {

        public Record()
        {
            // Needed for DB4O persistence
        }

        private HashMap<Date, WebHit> hits = new HashMap<Date, WebHit>();
        private int numHits;
        private int submissions;
    }

    //  private static ObjectContainer db;
    private Record record;

    // private final static SimpleDateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");

    static
    {
        //      db = DB4OPool.getDB();
    }


    public DB4OWebHitCounter()
    {
        load();
    }

    public void logHit(HttpServletRequest request)
    {
        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        InetAddress addr = null;
        try
        {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        if (addr != null)
        {
            webHit.setRemoteAddr(addr.getHostName());
        }

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setType(WebHit.Type.HIT);
        webHit.setAddress(request.getRequestURI());

        addHit(webHit);

        writeLog();
    }

    public void logHit(WebHit hit)
    {
        addHit(hit);

        writeLog();
    }

    private void addHit(WebHit webHit)
    {
        record.hits.put(new Date(), webHit);

        record.numHits++;

        System.out.println("Total hits: " + record.numHits);
    }

    public void flush()
    {
        writeLog();
    }

    public void logSubmission(HttpServletRequest request, Sequence sequence, String email, String ID, boolean flush)
    {
        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        // Do reverse lookup?
        InetAddress addr = null;
        try
        {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        if (addr != null)
        {
            webHit.setRemoteAddr(addr.getHostName());
        }

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setSequence(sequence.seqString());
        webHit.setType(WebHit.Type.SUBMISSION);
        webHit.setAddress(request.getRequestURI());
        webHit.setEmail(email);
        webHit.setID(ID);

        Date key = new Date();
        if ( record.hits.containsKey(key) )
        {
            try
            {
                Thread.sleep(50);
            }
            catch (InterruptedException e)
            {
                e.printStackTrace();
            }
            System.err.println("WARN: Attempted to log to the same date key " + key);
        }
        record.hits.put(key, webHit);

        record.submissions++;

        if (flush)
        {
            writeLog();
        }
    }

    private void writeLog()
    {
        // Remove first
        // db.delete(record);

        ObjectContainer db = null;
        while (db == null)
        {
            try
            {
                db = DB4OPool.getDB();
            }
            catch (DatabaseFileLockedException e)
            {
                logger.debug(e.getMessage());

                try
                {
                    logger.debug("Waiting for DB4O to be ready...");
                    Thread.sleep(1000);
                }
                catch (InterruptedException e1)
                {
                    e1.printStackTrace();
                }
            }
        }

        try
        {
            // Get the Record object out of the database
            ObjectSet<Object> records = db.get(Record.class);

            assert records.size() <= 1 : "There should be 0 or 1 records in the database, " + records.size() + " found";

            Record record = null;
            if (records.size() > 0)
            {
                record = (Record) records.get(0);
            }

            if (record == null)
            {
                record = new Record();
            }

            if (record != null && record.hits == null)
            {
                // This happens after a DB4O crash where it can't write the DB
                record.hits = new HashMap<Date, WebHit>();
            }

            // Copy the properties of our global record to the DB version
            record.hits = this.record.hits;
            record.submissions = this.record.submissions;
            record.numHits = this.record.numHits;

            // Re-add it
            db.set(record);
            db.set(record.hits);
            db.commit();
        }
        finally
        {
            db.close();
        }

    }

    private void load()
    {
        ObjectContainer db = null;
        while (db == null)
        {
            try
            {
                db = DB4OPool.getDB();
            }
            catch (DatabaseFileLockedException e)
            {
                logger.debug(e.getMessage());


                try
                {
                    logger.debug("Waiting for DB4O...");
                    Thread.sleep(1000);
                }
                catch (InterruptedException e1)
                {
                    e1.printStackTrace();
                }
            }
        }

        if (db != null)
        {
            try
            {
                ObjectSet<Object> records = db.get(Record.class);

                assert records.size() <= 1 : "There should be 0 or 1 records in the database, " + records.size() + " found";

                if (records.size() > 0)
                {
                    record = (Record) records.get(0);

                    if (record != null && record.hits == null)
                    {
                        // This happens after a DB4O crash where it can't write the DB
                        record.hits = new HashMap<Date, WebHit>();
                    }
                }

                if (record == null)
                {
                    // must be a new database
                    record = new Record();
                }

            }
            finally
            {
                db.close();
            }
        }
    }

    public int getNumHits()
    {
        return record.numHits;
    }

    public int getSubmissions()
    {
        return record.submissions;
    }

    public HashMap<Date, WebHit> getHits()
    {
        return record.hits;
    }

    public void clear()
    {

        ObjectContainer db = DB4OPool.getDB();

        try
        {
            // Get the Record object out of the database
            ObjectSet<Object> records = db.get(Record.class);

            assert records.size() <= 1 : "There should be 0 or 1 records in the database, " + records.size() + " found";

            Record record = null;
            if (records.size() > 0)
            {
                record = (Record) records.get(0);
            }

            db.delete(record);
        }
        finally
        {
            db.close();
        }

        record = new Record();
    }

}
