package ca.ualberta.proteus.web;

import ca.ualberta.proteus.db.model.ProteusSettingsIntf;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.CompletionListener;
import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.EmailUtils;
import ca.ualberta.proteus.util.ProteusTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.SymbolList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 9, 2005
 * Time: 2:55:41 PM
 * To change this template use File | Settings | File Templates.
 */
public class MultipleSubmissionHandler extends ProteusSubmitHandler
{

    private static boolean testing = Config.getConfig().getBooleanValue("testing");

    // counts how many submissions this run
    private static int submissionCount = 0;

    public static void handleSubmission(final HttpServletRequest request, final SubmitForm submitForm, final List<Sequence> sequences, final File PDBFile)
    {
        System.out.printf("Submissions: %d, Queue Length: %d\n", ++submissionCount, SubmissionQueue.length());

        // request.setAttribute(Consts.PROTEUS_ID, "<Multiple>");
        RuntimeCatalog logger = new RuntimeCatalog();

        logger.logObject(Consts.SEQUENCE_NAME, "<Multiple>");
        logger.logObject(Consts.SUBMIT_TIME, new Date());

        request.setAttribute(Consts.LOGGER, logger);
        request.setAttribute("multiple", true);

        request.setAttribute("numSequences", sequences.size());
        // savita
        final HttpSession session = request.getSession();

        // email
        final String toEmail = submitForm != null ? submitForm.getEmailInput() : null;
        session.setAttribute("email", toEmail);
        request.setAttribute("email", toEmail);

        handleStats(request);

        // handle the request in a new thread
        new Thread()
        {
            public void run()
            {
                handleSubmissionInternal(request, submitForm, PDBFile, sequences);
            }
        }.start();
    }

    private static void handleSubmissionInternal(HttpServletRequest request, SubmitForm submitForm, File PDBFile, List<Sequence> sequences)
    {
        final String toEmail = submitForm != null ? submitForm.getEmailInput() : null;
        final boolean sendEmail = toEmail != null && !toEmail.equals("");
        final Date startTime = new Date();

        try
        {
            final ProteusSettings baseSettings = new ProteusSettings();


            if (submitForm != null)
            {
                String method = submitForm.getMethod();
                if (method.equals("psipred"))
                {
                    baseSettings.setMethod(MethodSettings.PSIPRED);
                }
                else if (method.equals("jnet"))
                {
                    baseSettings.setMethod(MethodSettings.JNET);
                }
                else if (method.equals("transsec"))
                {
                    baseSettings.setMethod(MethodSettings.TRANSSEC);
                }
                else
                {
                    baseSettings.setMethod(MethodSettings.ALL);
                }

                baseSettings.setRunHomodeller(submitForm.getRunHomodeller());

                baseSettings.setTemplateID(submitForm.getTemplateID());
                baseSettings.setRunSecondaryStructure(submitForm.getRunSecondaryStructure());
            }
            else
            {
                baseSettings.setMethod(MethodSettings.ALL);
            }

            // other baseSettings
            baseSettings.setTesting(testing);

            baseSettings.setStructure(PDBFile);
            baseSettings.setRunEnergyMinimization(submitForm == null || submitForm.isRunEnergyMinimization());
            baseSettings.setOrganism(getOrganism(submitForm.getOrganism()));

            final Map<String, StructureAnnotatedSequence> proteusIDs = new HashMap<String, StructureAnnotatedSequence>();
            final List<ProteusImpl> threads = new ArrayList<ProteusImpl>(sequences.size());

            int i = 0;
            for (Sequence sequence : sequences)
            {
                if (sequence.length() > 2000)
                {
                    continue;
                }

                ProteusSettings settings = new ProteusSettings(baseSettings);

                settings.setSequence(sequence);
                settings.setID(getUniqueSeqID());

                final ProteusImpl proteus = new ProteusImpl(settings);

                // add proteus to the queue
                // SubmissionQueue.addToQueue(proteus);
                threads.add(proteus);

                // logging the submission could take a long time because of reverse DNS lookup, so do it after submission
                // WebHitCounter.getInstance().logSubmission(request, sequence, toEmail, proteus.getID());

                proteus.addCompletionListener(new CompletionListener()
                {
                    public void onComplete(boolean successful, String message)
                    {
                        RuntimeCatalog logger = proteus.getLogger();

                        StructureAnnotatedSequence finalSequence = (StructureAnnotatedSequence) logger.getLogObject(Consts.FINAL_WITH_HOMOLOGY);
                        if (finalSequence == null)
                        {
                            finalSequence = (StructureAnnotatedSequence) logger.getLogObject(Consts.FINAL);
                        }

                        Boolean isBBTM = (Boolean) logger.getLogObject(Consts.IS_BBTM);
                        if (isBBTM)
                        {
                            // Do replacement on structure
                            SymbolList structure = finalSequence.getStructureAnnotation();
                            String structStr = structure.seqString().replace("E", "B");
                            finalSequence.setStructureAnnotation(ProteusTools.createStructure(structStr));
                        }

                        proteusIDs.put(proteus.getID(), finalSequence);

                        // store session
                        try
                        {
                            File storageDirectory = new File(Config.getConfig().getStringValue("storageDir"));
                            if (!storageDirectory.exists())
                            {
                                storageDirectory.mkdirs();
                            }

                            ProteusSettingsIntf settings = proteus.getSettings();
                            // HACK : The settings used by the ActiveObjects class aren't serializable.  So build
                            // a new serializable version
                            proteus.getLogger().logObject(Consts.SETTINGS, new ProteusSettings(settings));

                            PersistentProteusResultStorage.store(proteus.getID(), proteus.getLogger(), storageDirectory);                            
                        }
                        catch (IOException e)
                        {
                            e.printStackTrace();
                        }
                        catch (ClassNotFoundException e)
                        {
                            e.printStackTrace();
                        }
                    }
                });
            }

            // add all proteus to the queue
            SubmissionQueue.addToQueue(threads);

            // Log all the submissions
            WebHitCounter counter = WebHitCounter.getInstance();
            for (ProteusImpl proteus : threads)
            {
                Sequence sequence = proteus.getSettings().getSequence();
                counter.logSubmission(request, sequence, toEmail, proteus.getID(), false);

                // We need to sleep for a bit because the entries are keyed on date, so they can't be down to the same millisecond
                Thread.sleep(50);
            }

            counter.flush();

            // wait till all the processes are finished
            for (Thread t : threads)
            {
                // If it hasn't started running join will return
                while (t.getState() != Thread.State.TERMINATED)
                {
                    Thread.sleep(1000);
                }

                t.join();
            }

            Date endTime = new Date();

            // send success email?
            if (sendEmail)
            {
                String returnEmail = Config.getConfig().getStringValue("email");
                String message = buildMessage(threads, proteusIDs, sequences.size(), startTime, endTime);

                EmailUtils.sendMessage(returnEmail, toEmail, Consts.APPNAME + " done", message);
                System.out.println("Sending email to : " + toEmail);
            }
            else
            {
                System.out.println("Not sending email... why not");
            }

        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
    }


    /**
     * Build a email message based on a map of sequences
     *
     * @param threads
     * @param proteusIDs
     * @param initialSize Pass in 0 to not say anything about an initial size or start/end time
     * @param startTime   nullable
     * @param endTime     nullable @return
     */
    public static String buildMessage(List<ProteusImpl> threads, Map<String, StructureAnnotatedSequence> proteusIDs, int initialSize, Date startTime, Date endTime)
    {
        StringBuilder sb = new StringBuilder();

        sb.append("<pre>");

        // Check if it's > 0, if it's not, don't print the message
        if (initialSize > 0)
        {
            sb.append(ProteusSubmitHandler.buildMessageHeader(startTime, endTime, initialSize, proteusIDs.size()));
        }
        else
        {
            sb.append(Consts.APPNAME + " has completed its prediction on your sequence. <br><br>\n");
        }

        sb.append(ProteusSubmitHandler.buildPDBMessage(threads));
        for (ProteusImpl proteus : threads)
        {
            String id = proteus.getID();
            RuntimeCatalog catalog = proteus.getLogger();

            try
            {
                StructureAnnotatedSequence finalSequence = proteusIDs.get(id);

                if (finalSequence != null)
                {
                    sb.append(ProteusSubmitHandler.buildMessageSequence(finalSequence, id, catalog));
                }

                /*sb.append("You can obtain your detailed results from ");

                sb.append("http://").append(serverAddress).append("/Result.do?ID=").append(id);*/
            }
            catch (Throwable e)
            {
                // catch it so we send an email and don't crash
                e.printStackTrace();
            }
        }

        sb.append("</pre>");
        /*if ( finalSequence != null )
        {
            sb.append("Sequence: ").append(finalSequence.seqString()).append("\n");
            sb.append("Structure: ").append(finalSequence.getStructureAnnotation().seqString()).append("\n");

        }
        */
        return sb.toString();
    }
}
