package ca.ualberta.proteus.web;

import ca.ualberta.proteus.biojava.PDBFileParser;
import ca.ualberta.proteus.db.model.ProteusSettingsIntf;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import ca.ualberta.proteus.program.MethodSettings;
import ca.ualberta.proteus.program.ProteusImpl;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.program.predictSPTM.Organism;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.util.*;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.io.StructureDB;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.biojava.bio.Annotation;
import org.biojava.bio.BioException;
import org.biojava.bio.SimpleAnnotation;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.*;
import org.biojava.bio.structure.Structure;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.ChangeVetoException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 9, 2005
 * Time: 2:55:41 PM
 * To change this template use File | Settings | File Templates.
 */
public class ProteusSubmitHandler
{
    private static final String STARS = "************************************************************";
    private static boolean testing = Config.getConfig().getBooleanValue("testing");

    // counts how many submissions this run
    private static int submissionCount = 0;

    public static void handleSubmission(HttpServletRequest request, SubmitForm submitForm) throws BioException, IOException
    {
        List<Sequence> sequences = getSequenceList(submitForm);
        File PDBFIle = submitForm != null ? getPDBFile(submitForm) : null;

        if (sequences.size() == 1)
        {
            handleSubmission(request, submitForm, sequences.get(0), PDBFIle);
        }
        else
        {
            MultipleSubmissionHandler.handleSubmission(request, submitForm, sequences, PDBFIle);
        }
    }

    public static void handleSubmission(HttpServletRequest request, Sequence sequence)
    {
        handleSubmission(request, null, sequence, null);
    }

    public static void handleSubmission(HttpServletRequest request, Sequence sequence, File PDBFile)
    {
        handleSubmission(request, null, sequence, PDBFile);
    }

    public static void handleSubmission(HttpServletRequest request, SubmitForm submitForm, Sequence sequence, File PDBFile)
    {
        System.out.printf("Submissions: %d, Queue Length: %d\n", ++submissionCount, SubmissionQueue.length());

        if (sequence.length() > 2000)
        {
            throw new IllegalArgumentException("Sequence is > 2000 residues - too long!");
        }

        try
        {
            final HttpSession session = request.getSession();
            final ProteusSettings settings = new ProteusSettings();

            // email
            final String toEmail = submitForm != null ? submitForm.getEmailInput() : null;
            session.setAttribute("email", toEmail);
            final boolean sendEmail = toEmail != null && !toEmail.equals("");

            if (submitForm != null)
            {
                String method = submitForm.getMethod();
                if (method.equals("psipred"))
                {
                    settings.setMethod(MethodSettings.PSIPRED);
                }
                else if (method.equals("jnet"))
                {
                    settings.setMethod(MethodSettings.JNET);
                }
                else if (method.equals("transsec"))
                {
                    settings.setMethod(MethodSettings.TRANSSEC);
                }
                else
                {
                    settings.setMethod(MethodSettings.ALL);
                }

                settings.setRunHomodeller(submitForm.getRunHomodeller());

                settings.setTemplateID(submitForm.getTemplateID());
                settings.setRunSecondaryStructure(submitForm.getRunSecondaryStructure());
            }
            else
            {
                settings.setMethod(MethodSettings.ALL);
            }

            // other settings
            settings.setTesting(testing);
            settings.setSequence(sequence);
            settings.setStructure(PDBFile);
            settings.setID(getUniqueSeqID());
            settings.setRunEnergyMinimization(submitForm == null || submitForm.isRunEnergyMinimization());
            settings.setOrganism(getOrganism(submitForm.getOrganism()));

            final ProteusImpl proteus = new ProteusImpl(settings);
            request.setAttribute(Consts.PROTEUS_ID, proteus.getID());
            session.setAttribute(Consts.PROTEUS_OBJECT + proteus.getID(), proteus);
            RuntimeCatalog logger = proteus.getLogger();
            request.setAttribute(Consts.LOGGER, logger);

            Annotation annotation = sequence.getAnnotation();
            String seqName;
            if (annotation != null && annotation.containsProperty(Consts.SEQUENCE_NAME))
            {
                seqName = (String) annotation.getProperty(Consts.SEQUENCE_NAME);
            }
            else
            {
                // must have used BioJava's parsing??
                seqName = sequence.getName();
            }

            logger.logObject(Consts.SEQUENCE_NAME, seqName);
            logger.logObject(Consts.SUBMIT_TIME, new Date());

            handleStats(request);

            // add proteus to the queue
            SubmissionQueue.addToQueue(proteus);

            // logging the submission could take a long time because of reverse DNS lookup, so do it after submission
            WebHitCounter.getInstance().logSubmission(request, sequence, toEmail, proteus.getID());

            proteus.addCompletionListener(
                    new CompletionListener()
                    {
                        public void onComplete(boolean successful, String msg)
                        {

                            // send success email?
                            if (sendEmail)
                            {
                                String returnEmail = Config.getConfig().getStringValue("email");
                                String message = buildMessage(proteus.getLogger(), successful);
                                //TODO
                                EmailUtils.sendMessage(returnEmail, toEmail, Consts.APPNAME + " done", message);
                                System.out.println("Sending email to : " + toEmail);
                            }

                            // Send error email?
                            if (!successful)
                            {
                                String returnEmail = Config.getConfig().getStringValue("email");
                                RuntimeCatalog logger = proteus.getLogger();
                                EmailUtils.sendMessage(returnEmail, returnEmail, Consts.APPNAME + " error", "Check the server log, there was an error. \n" + logger.getErrorBuffer().toString() + "\n" + msg + "\n ID = " + proteus.getID());
                                System.out.println("ERROR! Sending error email to : " + returnEmail);
                            }

                            // store session
                            try
                            {
                                File storageDirectory = new File(Config.getConfig().getStringValue("storageDir"));
                                if (!storageDirectory.exists())
                                {
                                    storageDirectory.mkdir();
                                }
                                System.out.println(storageDirectory);

                                ProteusSettingsIntf settings = proteus.getSettings();
                                // HACK : The settings used by the ActiveObjects class aren't serializable.  So build
                                // a new serializable version
                                proteus.getLogger().logObject(Consts.SETTINGS, new ProteusSettings(settings));

                                PersistentProteusResultStorage.store(proteus.getID(), proteus.getLogger(), storageDirectory);
                                //                                session.invalidate();
                            }
                            catch (IOException e)
                            {
                                e.printStackTrace();
                            }
                            catch (ClassNotFoundException e)
                            {
                                e.printStackTrace();
                            }
                        }

                        private String buildMessage(RuntimeCatalog logger, boolean successful)
                        {
                            StringBuilder sb = new StringBuilder();
                            sb.append("<div style=\"font-family: courier, helvetica, arial, verdana\">\n");
                            sb.append("<pre>\n");

                            if (successful)
                            {
                                sb.append(ProteusSubmitHandler.buildMessageHeader());

                                StructureAnnotatedSequence finalSequence = (StructureAnnotatedSequence) logger.getLogObject(Consts.FINAL_WITH_HOMOLOGY);
                                if (finalSequence == null)
                                {
                                    finalSequence = (StructureAnnotatedSequence) logger.getLogObject(Consts.FINAL);
                                }

                                if (finalSequence != null)
                                {
                                    sb.append(ProteusSubmitHandler.buildMessageSequence(finalSequence, proteus.getID(), logger));
                                }
                            }
                            else
                            {
                                sb.append(Consts.APPNAME + " encountered an error during the prediction of your sequence.  Please try resubmitting.  The server maintainers have been notified of the error.");
                            }

                            sb.append("</pre>\n");
                            sb.append("</div>");
                            return sb.toString();
                        }
                    }
            );

        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
    }

    protected static Organism getOrganism(String organism)
    {
        if (organism == null)
        {
            return null;
        }
        if (organism.equals("euk"))
        {
            return Organism.EUKARYOTIC;
        }
        if (organism.equals("gram-"))
        {
            return Organism.GRAM_NEGATIVE;
        }
        if (organism.equals("gram+"))
        {
            return Organism.GRAM_POSITIVE;
        }
        return null;
    }

    protected static void handleStats(HttpServletRequest request)
    {

        String remoteAddr = request.getRemoteAddr();
        System.out.println("remoteAddr = " + remoteAddr);

        String header = request.getHeader("user-agent");
        System.out.println("header = " + header);
    }

    /**
     * Gets a PDB file from the submission form
     *
     * @param submitForm
     * @return a File with the PDB file data in it, null if the user did not upload a file
     * @throws IOException
     */
    private static File getPDBFile(SubmitForm submitForm) throws IOException, BioException
    {
        InputStream inputStream = null;
        if (submitForm != null && submitForm.getTemplate3d() != null)
        {
            inputStream = submitForm.getTemplate3d().getInputStream();
        }

        if (submitForm != null && (inputStream == null || submitForm.getTemplate3d() == null || submitForm.getTemplate3d().getFileSize() == 0))
        {
            if (submitForm.getTemplateID() != null)
            {
                String templateID = submitForm.getTemplateID();
                if (templateID.length() > 0)
                {
                    String id = templateID.substring(0, 4);
                    inputStream = PDBFileDownloader.downloadPDBFile(id);
                }
            }
        }

        if (inputStream == null)
        {
            return null;
        }

        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));

        File rv = File.createTempFile("download", ".pdb");
        PrintWriter writer = new PrintWriter(rv);
        String line;
        int bytesRead = 0;
        while ((line = reader.readLine()) != null)
        {
            writer.println(line);
            bytesRead += line.length();
        }
        reader.close();
        writer.close();

        if (bytesRead != 0)
        {
            // Test the integrity of the PDB file

            if (submitForm.getTemplate3d() != null)
            {
                PDBFileParser parser = new PDBFileParser();
                try
                {
                    parser.parsePDBFile(new FileInputStream(rv));
                }
                catch (IOException e)
                {
                    throw new BioException("Error parsing uploaded PDB file");
                }
            }

            return rv;
        }
        else
        {
            return null;
        }
    }

    private static List<Sequence> getSequenceList(SubmitForm submitForm) throws IOException, BioException
    {
        InputStream inputStream = null;
        if (submitForm != null && submitForm.getFile() != null)
        {
            inputStream = submitForm.getFile().getInputStream();
        }

        if (inputStream != null && submitForm.getFile().getFileSize() > 0)
        {
            Reader reader = new BufferedReader(new InputStreamReader(inputStream));

            return ProteusTools.parseFasta(reader);
        }

        else
        {
            //String seqStr = request.getParameter("sequence");
            assert submitForm != null;

            String seqStr = submitForm.getSequence();

            if (seqStr == null || seqStr.equals(""))
            {
                //return actionMapping.findForward("inputFailed");
                throw new IllegalArgumentException("inputFailed");
            }

            return ProteusTools.parseFasta(new StringReader(seqStr));
        }
    }

    private static Sequence getSequence(SubmitForm submitForm) throws BioException, IOException
    {
        Sequence sequence;

        if (testing)
        {
            SeqSeeDB db = SeqSeeDB.getInstance();
            sequence = db.getSequence(0);
        }
        else
        {
            SymbolTokenization tokenization = ProteinTools.getAlphabet().getTokenization("token");

            InputStream inputStream = null;
            if (submitForm != null && submitForm.getFile() != null)
            {
                inputStream = submitForm.getFile().getInputStream();
            }

            if (inputStream != null && submitForm.getFile().getFileSize() > 0)
            {
                Reader reader = new BufferedReader(new InputStreamReader(inputStream));

                FastaFormat fastaFormat = new FastaFormat();

                SequenceBuilderFactory factory = SeqIOTools.getFastaBuilderFactory();
                SequenceBuilder parser = factory.makeSequenceBuilder();

                try
                {
                    fastaFormat.readSequence((BufferedReader) reader, tokenization, parser);

                    sequence = parser.makeSequence();

                }
                catch (IllegalSymbolException e)
                {
                    submitForm.getFile().getInputStream().reset();
                    reader = new InputStreamReader(submitForm.getFile().getInputStream());

                    sequence = parseFasta(reader, tokenization);
                }
            }

            else
            {
                //String seqStr = request.getParameter("sequence");
                assert submitForm != null;

                String seqStr = submitForm.getSequence();

                if (seqStr == null || seqStr.equals(""))
                {
                    //return actionMapping.findForward("inputFailed");
                    throw new IllegalArgumentException("inputFailed");
                }

                sequence = parseFasta(new StringReader(seqStr), tokenization);
            }
        }

        if (sequence == null || sequence.length() < 5)
        {
            throw new IllegalSymbolException("Sequence is too small (<5 residues)");
        }

        return sequence;
    }

    private static ActionForward handleInputError(HttpServletRequest request, IllegalSymbolException e, ActionMapping actionMapping)
    {
        request.setAttribute("message", e.getMessage());

        StringWriter out = new StringWriter();
        PrintWriter writer = new PrintWriter(out);
        e.printStackTrace(writer);
        request.setAttribute("trace", out.toString());

        return actionMapping.findForward("inputFailed");
    }

    protected static String getUniqueSeqID()
    {
        // get a unique ID - can only be 8 characters long since XAlign will cack without it
        String seqID = String.valueOf(System.currentTimeMillis()) + String.valueOf(Thread.currentThread().getId() % Math.random() * 100);
        seqID = seqID.substring(seqID.length() - 7);  // get last digits, since they will be the ones to change more often
        return seqID;
    }

    private static Sequence parseFasta(Reader reader, SymbolTokenization tokenization) throws IOException, IllegalSymbolException
    {
        BufferedReader bufferedReader = new BufferedReader(reader);
        StringBuilder buffer = new StringBuilder();
        String line = bufferedReader.readLine().replaceAll("\\s+", "");

        // skip blank lines
        while (line.equals("") && (line = bufferedReader.readLine()) != null)
        {
            ;
        }

        String header = getUniqueSeqID();
        String sequenceName = "";
        if (line != null && line.startsWith(">"))
        {
            if (line.length() > 7)
            {
                sequenceName = line.substring(1);
                header = line.substring(0, 7); // Sequence ID must only be 8 chars or will crash in XAlign
            }
            else
            {
                sequenceName = line.substring(1);
                header = line;
            }
        }
        else
        {
            buffer.append(line);
        }

        while ((line = bufferedReader.readLine()) != null)
        {
            buffer.append(line);
        }

        String seqStr = ProteusTools.cleanSequence(buffer.toString());

        SimpleSymbolList newList = new SimpleSymbolList(tokenization, seqStr);
        Annotation annotation = new SimpleAnnotation();

        // Keep the real sequence name for later
        try
        {
            annotation.setProperty(Consts.SEQUENCE_NAME, sequenceName);
        }
        catch (ChangeVetoException e)
        {
            e.printStackTrace();
        }
        return new SimpleSequence(newList, "", header.substring(1), annotation);
    }

    private String cleanSequence(String seqStr)
    {
        if (seqStr == null)
        {
            return null;
        }

        seqStr = seqStr.toUpperCase();
        seqStr = seqStr.replaceAll("\\s", "");
        seqStr = seqStr.replaceAll(" ", "");

        seqStr = seqStr.replace("\\w+", "");

        if (seqStr.matches("\\W|\\d|[JZOBUXa-z]"))
        {
            return null;
        }

        return seqStr;
    }

    protected static String buildMessageHeader()
    {
        return buildMessageHeader(null, null, -1, -1);
    }

    protected static String buildMessageHeader(Date startTime, Date endTime, int numQuerySeqs, int numResultSeqs)
    {
        DateFormat df = new SimpleDateFormat("hh:mm:ss MMM dd, yyyy");

        StringBuilder sb = new StringBuilder();
        sb.append("THANK YOU FOR USING PROTEUS2\n");
        sb.append(STARS + "\n");
        sb.append("PROTEUS2 is operated and maintained by the Genome Canada \n");
        sb.append("Bioinformatics Help Desk (http://gchelpdesk.ualberta.ca) at \n");
        sb.append("the University of Alberta, Edmonton, AB, Canada. If you have \n");
        sb.append("any questions or queries about the server or its output, \n");
        sb.append("please contact: info@gchelpdesk.ualberta.ca\n");

        if (numQuerySeqs > 0 || numResultSeqs > 0 || startTime != null || endTime != null)
        {
            sb.append(STARS + "\n");
        }

        if (numQuerySeqs > 0)
        {
            sb.append(String.format("Number of sequences submitted: %d\n", numQuerySeqs));
        }
        if (numResultSeqs > 0)
        {
            sb.append(String.format("Number of sequences successfully processed: %d\n", numResultSeqs));
        }
        if (startTime != null)
        {
            sb.append("Time of submission: " + df.format(startTime) + "\n");
        }
        if (endTime != null)
        {
            sb.append("Time of completion: " + df.format(endTime) + "\n");
        }

        sb.append(STARS + "\n");
        sb.append("Below are the structure predictions for your proteins \n");
        sb.append("obtained from Proteus2. Please click on the hyperlinked \n");
        sb.append("protein name to obtain more information about the sequence \n");
        sb.append("analysis and structure prediction results.  The hyperlinks \n");
        sb.append("will remain active for 30 days from the time/date of \n");
        sb.append("completion.  After this period, the data will be deleted\n");
        sb.append("from the Proteus2 server. \n");
        sb.append("\n");
        sb.append("H = Helix, E = Beta Strand, C = Coil, T = Memebrane helix, \n");
        sb.append("B = Membrane strand, S = Signal peptide, c = Cleavage site \n");
        sb.append(STARS + "\n");

        return sb.toString();
    }

    protected static String buildMessageSequence(StructureAnnotatedSequence finalSequence, String defaultName, RuntimeCatalog logger)
    {
        String name = finalSequence.getName();
        if (name == null || name.length() == 0)
        {
            name = defaultName;
        }

        StructureDB db = new StructureDB(StructureDB.MODE.WRITE);
        db.setWriteComments(false);

        // set the name to the HTTP address
        String serverAddress = Config.getConfig().getStringValue("serverAddress");
        serverAddress = serverAddress.replace("http://", "");
        if (!serverAddress.endsWith("/"))
        {
            serverAddress += "/";
        }
        name = "<a href=\"http://" + serverAddress + "Result.do?ID=" + defaultName + "\">" + name + "</a>";


        Boolean isBBTM = (Boolean) logger.getLogObject(Consts.IS_BBTM);
        if (isBBTM)
        {
            // Do replacement on structure 
            SymbolList structure = finalSequence.getStructureAnnotation();
            String structStr = structure.seqString().replace("E", "B");
            finalSequence.setStructureAnnotation(ProteusTools.createStructure(structStr));
        }

        finalSequence.setName(name);
        db.addSequence(finalSequence);

        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        db.writeDB(new PrintStream(stream), true);

        stream.toString();

        return stream.toString();
    }

    protected static String buildMessagePDB(RuntimeCatalog logger)
    {
        List<Structure> models = ProteusTools.getFinishedModels(logger);

        if (models.size() > 0)
        {
            return "\n" + models.get(0).toPDB() + "\n";
        }
        return "";
    }

    protected static String buildPDBMessage(List<ProteusImpl> threads)
    {
        try
        {
            writeStructureFile(threads);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        String serverAddress = Config.getConfig().getStringValue("serverAddress");
        serverAddress = serverAddress.replace("http://", "");
        if (!serverAddress.endsWith("/"))
        {
            serverAddress += "/";
        }
        String link = "<a href=\"http://" + serverAddress + "view_pdb.jsp?job_id=" + threads.get(0).getID() + "\">link</a>";

        StringBuilder sb = new StringBuilder();
        sb.append("Please click this " + link + " to see the PDB coordinates for the \n");
        sb.append("proteins for which homology models could be generated. This \n");
        sb.append("link will remain active for 30 days from the time of \n");
        sb.append("completion.\n");
        sb.append(STARS + "\n");

        return sb.toString();
    }

    private static void writeStructureFile(List<ProteusImpl> threads) throws IOException
    {
        String dir = Config.getConfig().getStringValue("jobs.list");
        String filename = dir + File.separator + threads.get(0).getID();
        BufferedWriter writer = new BufferedWriter(new FileWriter(filename));
        try
        {
            for (ProteusImpl proteus : threads)
            {
                RuntimeCatalog logger = proteus.getLogger();
                List<Structure> models = ProteusTools.getFinishedModels(logger);

                for (Structure model : models)
                {
                    if (model != null)
                    {
                        writer.write(proteus.getID() + "\n");
                    }
                }
            }
        }
        finally
        {
            writer.close();
        }
    }

}

