package ca.ualberta.proteus.web;

import ca.ualberta.proteus.util.Consts;
import ca.ualberta.proteus.util.domainImage.ConservedDomainImageGenerator;
import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.program.ProteusSettings;
import ca.ualberta.proteus.sequence.GappedStructureAnnotatedSequence;
import ca.ualberta.proteus.sequence.StructureAnnotatedSequence;
import ca.ualberta.proteus.logging.RuntimeCatalog;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.biojava.bio.seq.GappedSequence;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 11, 2005
 * Time: 5:09:22 PM
 * To change this template use File | Settings | File Templates.
 */
public class ResultAction extends Action
{

    public ActionForward execute(ActionMapping actionMapping, ActionForm actionForm,
                                 HttpServletRequest request, HttpServletResponse response) throws IOException, ClassNotFoundException
    {
        String ID = request.getParameter("ID");
        if ( ID.contains(";") ) // not sure why it's doing this but...
            ID = ID.substring(0, ID.indexOf(';'));
        System.out.println("Getting result for " + ID);

        final HttpSession session = request.getSession();

        File storageDirectory = new File( Config.getConfig().getStringValue("storageDir"));
            if ( !storageDirectory.exists())
                storageDirectory.mkdir();
        try
        {
            RuntimeCatalog logger = PersistentProteusResultStorage.restore(ID, storageDirectory);

            if ( logger != null ) {
                request.setAttribute(Consts.LOGGER, logger);
                request.setAttribute(Consts.PROTEUS_ID, ID);
                session.setAttribute(Consts.LOGGER, logger);

                createConservedDomainImage(logger, ID, request);

                StructureAnnotatedSequence forStats = ProteusHelperUtils.getStructureForStats(logger);
                ProteusSettings settings = (ProteusSettings) logger.getLogObject(Consts.SETTINGS);

                if ( settings.isRunSecondaryStructure() && forStats == null)
                {
                    // it may not have been created yet
                    try
                    {
                        Thread.sleep(2000);
                    }
                    catch (InterruptedException e)
                    {/*Do nothing*/
                    }
                    // try getting it again
                    forStats = ProteusHelperUtils.getStructureForStats(logger);
                }
                if ( forStats != null )
                {
                    return actionMapping.findForward("success");
                }
                else if ( ! settings.isRunSecondaryStructure() )
                {
                    // we will only get the stats if we actually run secondary structure
                    return actionMapping.findForward("success");
                }
                else
                {
                    return actionMapping.findForward("error");
                }
            }
            else
                return actionMapping.findForward("fail");
        }
        catch ( IOException e)
        {
            System.out.println(e);
            return actionMapping.findForward("fail");
        }
    }

    private void createConservedDomainImage(RuntimeCatalog logger, String ID, HttpServletRequest request)
    {
        // Create the conserved domain image and tell the jsp where to send it
        List<GappedStructureAnnotatedSequence> alignedHomologs = (List<GappedStructureAnnotatedSequence>) logger.getLogObject(Consts.ALIGNED_HOMOLOG_LIST);
        List<GappedSequence> alignedTransmembraneHomologs = (List<GappedSequence>) logger.getLogObject(Consts.TM_HOMOLOGS);

        if ( alignedHomologs != null )
        {
            List<GappedSequence> alignedHomologsClone = new ArrayList<GappedSequence>();
            alignedHomologsClone.addAll(alignedHomologs);

            for ( int i=alignedHomologsClone.size() - 1; i>=0; i--)
            {
                GappedSequence seq = alignedHomologsClone.get(i);
                if (seq.getAnnotation().containsProperty("isTM"))
                {
                    alignedHomologsClone.remove(seq);
                }

                if ( alignedTransmembraneHomologs != null )
                {
                    // remove any from the list the are supposed to be transmembrane
                    for ( GappedSequence s : alignedTransmembraneHomologs)
                    {
                        if ( s.getName().equals(seq.getName()))
                        {
                            alignedHomologsClone.remove(seq);
                            break;
                        }
                    }
                }
            }

            // only create the image if there's actually homologs
            if ( alignedHomologsClone.size() > 0 )
            {
                ConservedDomainImageGenerator generator = new ConservedDomainImageGenerator(alignedHomologsClone );

                String name = "public_" + ID + ".png";
                File file = new File(System.getProperty("java.io.tmpdir"), name);
                try
                {
                    generator.createImage(new FileOutputStream(file), 800);
                }
                catch (FileNotFoundException e)
                {
                    e.printStackTrace();
                }
                request.setAttribute(Consts.IMAGE_NAME, name );
            }
        }

                // Create the conserved domain image and tell the jsp where to send it

/*        if ( alignedHomologs == null )
        {
            alignedHomologs = (List<GappedStructureAnnotatedSequence>) logger.getLogObject(Consts.ALIGNED_HOMOLOG_LIST);
        }
        */

        if ( alignedTransmembraneHomologs != null )
        {
            ConservedDomainImageGenerator generator = new ConservedDomainImageGenerator(alignedTransmembraneHomologs );

            String name = "public_tm_" + ID + ".png";
            File file = new File(System.getProperty("java.io.tmpdir"), name);
            try
            {
                generator.createImage(new FileOutputStream(file), 800);
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }
            request.setAttribute(Consts.TM_IMAGE_NAME, name );
        }
    }

}
