package ca.ualberta.proteus.web;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.ProteusTools;
import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.impl.SimpleSequence;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.BioException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;

/**
 * Created by IntelliJ IDEA.
 * User: Smontgomerie
 * Date: Jun 19, 2005
 * Time: 10:07:38 PM
 * To change this template use File | Settings | File Templates.
 */
public class SubmitAction extends Action
{
    private static final Logger logger = Logger.getLogger(InfoAction.class);
    private static boolean testing = Config.getConfig().getBooleanValue("testing");


    public ActionForward execute(ActionMapping actionMapping, ActionForm actionForm,
                                 HttpServletRequest request, HttpServletResponse response)
            throws Exception
    {
//        final HttpSession session = request.getSession();
        final ActionForward successForward = actionMapping.findForward("success");
        final SubmitForm submitForm = (SubmitForm) actionForm;

        try
        {
            submitForm.reset(actionMapping, request);
            submitForm.setRunHomodeller( request.getParameter("runHomodeller") != null );
            submitForm.setRunEnergyMinimization( request.getParameter("runEnergyMinimization") != null );
            submitForm.setRunSecondaryStructure( request.getParameter("runSecondaryStructure") != null );

            ProteusSubmitHandler.handleSubmission(request, submitForm);
        }
        catch (IllegalSymbolException e) {
            return handleInputError(request, e, actionMapping);
        }
        catch (BioException e)
        {
            return handleInputError(request, e, actionMapping);
        }
        catch( IllegalArgumentException e ) {
            return handleInputError(request, e, actionMapping);
        }

        return successForward;
    }

    private ActionForward handleInputError(HttpServletRequest request, Exception e, ActionMapping actionMapping)
    {
        request.setAttribute("message", e.getMessage());

        StringWriter out = new StringWriter();
        PrintWriter writer = new PrintWriter(out);
        e.printStackTrace(writer);
        request.setAttribute("trace", out.toString());

        return actionMapping.findForward("inputFailed");
    }

    private String getUniqueSeqID()
    {
        // get a unique ID - can only be 8 characters long since XAlign will cack without it
        String seqID = String.valueOf(System.currentTimeMillis()) + String.valueOf(Thread.currentThread().getId());
        seqID = seqID.substring(seqID.length() - 7);  // get last digits, since they will be the ones to change more often
        return seqID;
    }

    private Sequence parseFasta(Reader reader, SymbolTokenization tokenization) throws IOException, IllegalSymbolException
    {
        BufferedReader bufferedReader = new BufferedReader(reader);
        StringBuilder buffer = new StringBuilder();
        String line = bufferedReader.readLine().replaceAll("\\s+", "");

        // skip blank lines
        while (line.equals("") && (line = bufferedReader.readLine()) != null) ;

        String header = getUniqueSeqID();
        if (line != null && line.startsWith(">")) {
            if ( line.length() > 7 )
                header = line.substring(0, 7); // Sequence ID must only be 8 chars or will crash in XAlign
            else
                header = line;
        }
        else
            buffer.append(line);

        while ((line = bufferedReader.readLine()) != null)
        {
            buffer.append(line);
        }

        String seqStr = ProteusTools.cleanSequence(buffer.toString());

        SimpleSymbolList newList = new SimpleSymbolList(tokenization, seqStr);
        return new SimpleSequence(newList, "", header.substring(1), null);
    }

    private String cleanSequence(String seqStr)
    {
        if (seqStr == null)
            return null;

        seqStr = seqStr.toUpperCase();
        seqStr = seqStr.replaceAll("\\s", "");
        seqStr = seqStr.replaceAll(" ", "");

        seqStr = seqStr.replace("\\w+", "");

        if (seqStr.matches("\\W|\\d|[JZOBUXa-z]"))
            return null;

        return seqStr;
    }
}