package ca.ualberta.proteus.web;

import ca.ualberta.proteus.program.transsec.Config;
import ca.ualberta.proteus.util.BlastRunner;
import ca.ualberta.proteus.util.io.SeqSeeDB;
import ca.ualberta.proteus.util.BlastSettings;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.ProteinTools;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Tests a few sequences on proteus to make sure it still works
 *
 * Note: Will not test UI
 */
public class TestAction extends Action
{

    public ActionForward execute(ActionMapping actionMapping, ActionForm actionForm,
                                 HttpServletRequest request, HttpServletResponse response) throws IOException, ClassNotFoundException, BioException
    {
        SeqSeeDB db = SeqSeeDB.getInstance();

        String sequenceID = request.getParameter("sequenceID");
        String sequence0 = request.getParameter("sequence0");
        String sequence1 = request.getParameter("sequence1");
        String sequence2 = request.getParameter("sequence2");
        String sequence3 = request.getParameter("sequence3");
        String sequence4 = request.getParameter("sequence4");

        String id0 = request.getParameter("id0");
        String id1 = request.getParameter("id1");
        String id2 = request.getParameter("id2");
        String id3 = request.getParameter("id3");

        List<String> errors = new ArrayList<String>();
        request.setAttribute("errors", errors);
        List<String> success = new ArrayList<String>();
        request.setAttribute("success", success);

        if ( ! sequenceID.equals(""))
        {

            Sequence seq = db.getSequence(sequenceID);
            BlastSettings settings = new BlastSettings();
            settings.setSequence(seq);
            settings.setBlastType(BlastSettings.BLASTTYPE.BLASTP);
            settings.setDatabase(Config.getConfig().getStringValue("PDB"));
            settings.setEvalue(0.01);
            //settings.setKeepOutput( true );

            BlastRunner runner = new BlastRunner(settings);
            runner.run();

            List<Sequence> alignment = runner.getUnalignedResult();

            boolean found = false;
            for (Iterator<Sequence> it = alignment.iterator(); it.hasNext();)
            {
                Sequence s = it.next();
                if ( sequenceID.equalsIgnoreCase(s.getName()))
                {
                    found = true;
                    break;
                }
            }
            if ( !found )
                errors.add("Sequence " + sequenceID +" not found in BLAST result - reduced PDB database and SeqSeeDB must differ");
            else
                success.add("Sequence " + sequenceID + " found");
        }

        if ( sequence0 != null && sequence0.equals("on"))
        {
            try
            {
                ProteusSubmitHandler.handleSubmission(request,  db.getSequence(Integer.parseInt(id0)) );
            }
            catch(Exception e)
            {
                errors.add(e.getMessage());
            }
        }

        if ( sequence1 != null && sequence1.equals("on"))
        {
            try
            {
                ProteusSubmitHandler.handleSubmission(request,  db.getSequence(Integer.parseInt(id1)) );
            }
            catch(Exception e)
            {
                errors.add(e.getMessage());
            }
        }

        if ( sequence2 != null && sequence2.equals("on"))
        {
            try
            {
                ProteusSubmitHandler.handleSubmission(request,  db.getSequence(Integer.parseInt(id2)) );
            }
            catch(Exception e)
            {
                errors.add(e.getMessage());
            }
        }

        if ( sequence3 != null && sequence3.equals("on"))
        {
            try
            {
                ProteusSubmitHandler.handleSubmission(request,  db.getSequence(Integer.parseInt(id3)) );
            }
            catch(Exception e)
            {
                errors.add(e.getMessage());
            }
        }

        if ( sequence3 != null && sequence3.equals("on"))
        {
            try
            {
                    String tmSequence = "LQFYRNLGKSGLRVSCLGLGTWVTFGGQITDEMAEHLMTLAYDNGINLFDTAEVYAAGKAEVVLGNIIKKKGWRRSSLVITTKIFWG"
              +"GKAETERGLSRKHIIEGLKASLERLQLEYVDVVFANRPDPNTPMEETVRAMTHVINQGMAMYWGTSRWSSMEIMEAYSVARQFNLIPPICEQAEYHMFQREKVEVQL"
              +"PELFHKIGVGAMTWSPLACGIVSGKYDSGIPPYSRASLKGYQWLKDKILSEEGRRQQAKLKELQAIAERLGCTLPQLAIAWCLRNEGVSSVLLGASNAEQLMENIGA"+
              "IQVLPKLSSSIVHEIDSILGNKPYS";

                ProteusSubmitHandler.handleSubmission(request,  ProteinTools.createProteinSequence(tmSequence, "Transmembrane"));
            }
            catch(Exception e)
            {
                errors.add(e.getMessage());
            }
        }



        return actionMapping.findForward("success");
    }
}
