package ca.ualberta.proteus.web;

import ca.ualberta.proteus.program.transsec.Config;
import org.biojava.bio.seq.Sequence;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Oct 11, 2005
 * Time: 8:31:43 PM
 * To change this template use File | Settings | File Templates.
 */
public class XMLWebHitCounter extends WebHitCounter
{
    private HashMap<Date, WebHit> hits = new HashMap<Date, WebHit>();
    private File storageFile;
    private int numHits;
    private int submissions;
    private static SimpleDateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");

    public void ensureLoaded()
    {
        if ( storageFile == null )
        {
            File storageDirectory = new File( Config.getConfig().getStringValue("storageDir"));
            if ( !storageDirectory.exists())
                storageDirectory.mkdir();
            storageFile = new File(storageDirectory.getAbsolutePath() + "/webCounter");
            System.out.println("Using webcounter file " + storageFile.getAbsolutePath());
            load();
        }
    }

    public void logHit(HttpServletRequest request) {

        ensureLoaded();
        
        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        InetAddress addr = null;
        try {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e) {
            e.printStackTrace();
        }
        if ( addr != null )
            webHit.setRemoteAddr(addr.getHostName());

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setType(WebHit.Type.HIT);
        webHit.setAddress(request.getRequestURI());

        addHit(webHit);

        writeLog();
    }

    private void addHit(WebHit webHit)
    {
        hits.put(new Date(), webHit);

        numHits++;
    }

    public void logSubmission(HttpServletRequest request, Sequence sequence, String email, String ID, boolean flush) {
        ensureLoaded();

        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        // Do reverse lookup?
        InetAddress addr = null;
        try {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        if ( addr != null )
            webHit.setRemoteAddr(addr.getHostName());

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setSequence(sequence.seqString());
        webHit.setType(WebHit.Type.SUBMISSION);
        webHit.setAddress(request.getRequestURI());
        webHit.setEmail(email);
        webHit.setID(ID);
        hits.put(new Date(), webHit);

        submissions++;                               
            
        writeLog();
    }

    public void flush()
    {
        writeLog();
    }

    public void writeLog() {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            DOMImplementation impl = builder.getDOMImplementation();

            Document doc = impl.createDocument(null, "web_access_log", null);

            Element root = doc.getDocumentElement();
            Date today = new Date();
            Attr date = doc.createAttribute("updated");
            date.setValue(today.toString());
            root.getAttributes().setNamedItem(date);

            synchronized (hits)
            {
                for (Date key : hits.keySet())
                {
                    WebHit record = hits.get(key);

                    try {
                        Element recordElement = doc.createElement("hit");
                        recordElement.setAttribute("type", record.getType().toString());
                        recordElement.setAttribute("time", dateFormat.format(key));
                        recordElement.setAttribute("userAgent", record.getUserAgent());
                        recordElement.setAttribute("remoteAddr", record.getRemoteAddr());
                        recordElement.setAttribute("address", record.getAddress());
                        recordElement.setAttribute("email", record.getEmail());
                        recordElement.setAttribute("ID", String.valueOf(record.getID()));

                        root.appendChild(recordElement);

                        if (record.getType() == WebHit.Type.SUBMISSION)
                        {
                            Element seqElement = doc.createElement("sequence");
                            seqElement.setTextContent(record.getSequence());
                            recordElement.appendChild(seqElement);
                        }
                    }
                    catch ( Exception e) {
                        e.printStackTrace();
                    }
                }

                // Prepare the DOM document for writing, and the output file
                Source source = new DOMSource(doc);
                Result result = new StreamResult(storageFile);
                // Write the DOM document to file, using a document transformer
                Transformer transformer = TransformerFactory.newInstance().newTransformer();
                transformer.transform(source, result);
                //System.err.println("Done");
            }
        }
        catch (ParserConfigurationException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (TransformerConfigurationException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        catch (TransformerException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public void load() {
        try
        {
            InputStream in = null;
            try
            {
                in = new FileInputStream(storageFile);
                System.out.println("Loading web hits... using storage file " + storageFile);
            }
            catch( FileNotFoundException e) {
                System.out.println("Can't find storage file " + storageFile);
                System.out.println("Making new one...");
            }
            if (in==null)
                return;
            synchronized (hits)
            {

//                SymbolTokenization tokenization = ProteinTools.getAlphabet().getTokenization("token");
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = null;
                hits.clear();

                try
                {
                    builder = factory.newDocumentBuilder();
                    Document doc = builder.parse(in);

                    Element root = doc.getDocumentElement();

                    for (int i = 0; i < root.getChildNodes().getLength(); i++)
                    {
                        Node node = root.getChildNodes().item(i);

                        if (node.getNodeName().equals("hit"))
                        {
                            WebHit webHit = new WebHit();
                            Date date = null;
                            for (int j = 0; j < node.getAttributes().getLength(); j++)
                            {
                                Attr attr = (Attr) node.getAttributes().item(j);

                                if (attr.getName().equals("type"))
                                    webHit.setType(WebHit.Type.valueOf(attr.getValue()));
                                if (attr.getName().equals("time"))
                                {
                                    try
                                    {
                                        date = dateFormat.parse(attr.getValue());
                                    }
                                    catch (ParseException e)
                                    {
                                        e.printStackTrace();  
                                    }
                                }
                                if (attr.getName().equals("userAgent"))
                                    webHit.setUserAgent(attr.getValue());
                                if (attr.getName().equals("remoteAddr"))
                                    webHit.setRemoteAddr(attr.getValue());
                                if (attr.getName().equals("email"))
                                    webHit.setEmail(attr.getValue());
                                if (attr.getName().equals("ID"))
                                    webHit.setID(attr.getValue());
                                if (attr.getName().equals("address"))
                                    webHit.setAddress(attr.getValue());
                            }
                            if (webHit.getType() == WebHit.Type.HIT)
                                numHits++;
                            else
                            {
                                submissions++;
                                Node seqNode = node.getChildNodes().item(0);
                                webHit.setSequence(seqNode.getTextContent());
                            }
                            hits.put(date, webHit);
                        }
                    }
                }
                catch (ParserConfigurationException e)
                {
                    e.printStackTrace();  //TODO
                }
                catch (SAXException e)
                {
                    e.printStackTrace();  //TODO
                }

            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    public int getNumHits() {
        return numHits;
    }

    public int getSubmissions() {
        return submissions;
    }

    public HashMap<Date, WebHit> getHits() {
        return hits;
    }


}
