package ca.ualberta.proteus.web.db;

import ca.ualberta.proteus.db.SQLManager;
import ca.ualberta.proteus.db.model.Record;
import ca.ualberta.proteus.db.model.WebHitEntity;
import ca.ualberta.proteus.db.model.WebHitIntf;
import ca.ualberta.proteus.web.WebHitCounter;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.Logger;
import org.biojava.bio.seq.Sequence;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: Sep 14, 2009
 * Time: 12:10:23 AM
 * To change this template use File | Settings | File Templates.
 */
public class SQLWebHitCounter extends WebHitCounter
{
    private Logger logger = Logger.getLogger(SQLWebHitCounter.class);


    private Record record;

    public SQLWebHitCounter()
    {
        load();
    }

    private void load()
    {
        try
        {
            Record[] records = SQLManager.getEntityManager().find(Record.class);

            assert records.length <= 1 : "There should be 0 or 1 records in the database, " + records.length + " found";

            if (records.length > 0)
            {
                record = records[0];
            }

            if (record == null)
            {
                record = SQLManager.getEntityManager().create(Record.class);
            }
        }
        catch (SQLException e)
        {
            SQLManager.handleSQLException(e);
            e.printStackTrace();
        }
    }


    public void logHit(HttpServletRequest request)
    {
        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        InetAddress addr = null;
        try
        {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        if (addr != null)
        {
            webHit.setRemoteAddr(addr.getHostName());
        }

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setType(WebHitEntity.Type.HIT);
        webHit.setAddress(request.getRequestURI());

        addHit(webHit);
    }

    public WebHitEntity logHit(WebHit hit)
    {
        return addHit(hit);
    }

    private WebHitEntity addHit(WebHit webHit)
    {
        WebHitEntity webHitEntity = SQLManager.getEntityManager().create(WebHitEntity.class, webHit);

        record.setNumHits(record.getNumHits() + 1);
        record.save();

        return webHitEntity;
    }

    public void flush()
    {
    }

    public void logSubmission(HttpServletRequest request, Sequence sequence, String email, String ID, boolean flush)
    {
        WebHit webHit = new WebHit();
        webHit.setRemoteAddr(request.getRemoteAddr());

        // Do reverse lookup?
        InetAddress addr = null;
        try
        {
            addr = InetAddress.getByName(webHit.getRemoteAddr());
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        if (addr != null)
        {
            webHit.setRemoteAddr(addr.getHostName());
        }

        webHit.setUserAgent(request.getHeader("user-agent"));
        webHit.setSequence(sequence.seqString());
        webHit.setType(WebHit.Type.SUBMISSION);
        webHit.setAddress(request.getRequestURI());
        webHit.setEmail(email);
        // webHit.setID(ID);
        webHit.setProteusID(ID);
        webHit.setDate(new Date());

        addHit(webHit);

        record.setNumSubmissions(record.getNumSubmissions() + 1);
        record.save();
    }

    public int getNumHits()
    {
        return record.getNumHits();
    }

    public int getSubmissions()
    {
        return record.getNumSubmissions();
    }

    /**
     * Gets a hit by Proteus ID
     * @param proteusID
     * @return
     */
    public WebHit getHit(int proteusID)
    {
        try
        {
            WebHitEntity[] entities = SQLManager.getEntityManager().find(WebHitEntity.class, "proteusID = ?", proteusID);

            if ( entities.length == 1 )
            {
                return convertHit(entities[0]);
            }
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }

        return null;
    }


    public HashMap<Date, WebHit> getHits()
    {
        return getHits(null);
    }

    public HashMap<Date, WebHit> getHits(WebHitIntf.Type type)
    {
        try
        {
            WebHitEntity[] entities;
            if ( type == null )
            {
                entities = SQLManager.getEntityManager().find(WebHitEntity.class);
            }
            else
            {
                entities = SQLManager.getEntityManager().find(WebHitEntity.class, "type = ?" , type);
            }

            HashMap<Date, WebHit> map = new HashMap<Date, WebHit>();
            for (WebHitEntity e : entities)
            {
                WebHit h = convertHit(e);
                map.put(e.getDate(), h);
            }

            return map;
        }
        catch (SQLException e)
        {
            SQLManager.handleSQLException(e);
            e.printStackTrace();
        }

        return null;
    }

    private WebHit convertHit(WebHitEntity e)
    {
        WebHit h = new WebHit();
        try
                {
                    BeanUtils.copyProperties(h, e);
        }
        catch (IllegalAccessException e1)
        {
            e1.printStackTrace();
        }
        catch (InvocationTargetException e1)
        {
            e1.printStackTrace();
        }
        return h;
    }

    public void clear()
    {
        try
        {
            SQLManager.getEntityManager().deleteAll(WebHitEntity.class);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
    }

    public HashMap<Date, WebHit> getSubmissionsByEmail(String email)
    {
        try
        {
            WebHitEntity[] entities;
            entities = SQLManager.getEntityManager().find(WebHitEntity.class, "type = ? and email = ?" , WebHitIntf.Type.SUBMISSION, email);

            HashMap<Date, WebHit> map = new HashMap<Date, WebHit>();
            for (WebHitEntity e : entities)
            {
                WebHit h = convertHit(e);
                map.put(e.getDate(), h);
            }

            return map;
        }
        catch (SQLException e)
        {
            SQLManager.handleSQLException(e);
            e.printStackTrace();
        }

        return null;
    }
}

