#!/usr/bin/perl
# Written by Haiyan Zhang to convert bmrb file format to shifty file format
# the residue numbers may not be continuous, no * as residue name
# The output format has been modified by Raj Maiti.
# The parser was modifed by Peter Tang to accomodate for the 10th column in Chemical shift.

if(@ARGV<2)
{
  print "Usage:perl bmrb2shifty.pl <bmrbfile> <output>\n";
  exit(0);
}

%TAA=('GLY'=>'G','ALA'=>'A', 'CYS'=>'C','ASP'=>'D','GLU'=>'E','PHE'=>'F',
       'HIS'=>'H', 'ILE'=>'I', 'LEU'=>'L', 'MET'=>'M', 'ASN'=>'N', 'PRO'=>'P',
       'LYS'=>'K','GLN'=>'Q', 'ARG'=>'R','SER'=>'S','THR'=>'T','VAL'=>'V',
       'TYR'=>'Y','TRP'=>'W');

open(in,"$ARGV[0]");
@data=<in>;
close(in);
#print @data;

open(out, ">$ARGV[1]");
 
$min=10000;
$max=0;

#####
# Constants
#####
$change_flag = 1; # To indent if needed 

$resnum = 2;
$resname = 3;
$atomsym = 4;
$csvalue = 6;

if (@items > 9) {

    $resnum += 1;
    $resname += 1;
    $atomsym += 1;
    $csvalue += 1;
}

foreach $line(@data) {
 
    if($line=~/_Chem_shift_ambiguity_code/i) { 

	$start=1; $stop=0;
    }

    if ($line=~/stop_/i) { 
	$stop=1;
    } 

    if ($start==1 && $stop==1) {
	last;
    }
    
    if ($start==1) { 

	#1  2  2 ALA H    H   8.61 0.00 1
        #print "Line : " . $line . "\n";
	
	@items=split(/\s+/,$line); 

	if(@items > 5) {

	    if ($change_flag == 1) {

		if (@items > 9) {

		    $resnum += 1;
		    $resname += 1;
		    $atomsym += 1;
		    $csvalue += 1;
		}

		$change_flag = 0;
	    }

	    if ($min > $items[$resnum]) {
		$min = $items[$resnum];
	    }

	    if ($max < $items[$resnum]) {
		$max = $items[$resnum];
	    }

	    ## Amino Acid Abbreviation
	    $aa[$items[$resnum]][0] = $TAA{$items[$resname]};

	    #print $items[$atomsym] . " " . $aa[$items[$resnum]][0] . "\n";
	  
	    ## Case Specific
	    if($items[$atomsym] eq 'HA') {
		$aa[$items[$resnum]][1] = $items[$csvalue];
	    }

	    if($items[$atomsym] eq 'H')  {
		$aa[$items[$resnum]][6] = $items[$csvalue];
	    }
	    
	    if($items[$atomsym] eq 'N')  {
		$aa[$items[$resnum]][5]=$items[$csvalue];
	    }
	    
	    if($items[$atomsym] eq 'CA') {
		$aa[$items[$resnum]][2]=$items[$csvalue];
	    }

	    if($items[$atomsym] eq 'CB') {
		$aa[$items[$resnum]][3]=$items[$csvalue];
	    }

	    if($items[$atomsym] eq 'C')  {
		$aa[$items[$resnum]][4]=$items[$csvalue];
	    }

	    #print $resnum . " " . $items[$resnum] . " " . $aa[$items[$resnum]][0] . " " . $aa[$items[$resnum]][1] . " " . $aa[$items[$resnum]][2] . " " . $aa[$items[$resnum]][3] . " " . $aa[$items[$resnum]][4] . "\n";
	}
    }
}
    

printf(out  "#NUM    AA     HA     CA     CB     CO     N     HN\n");
#printf(out  " --- --- ------ ------ -------- ------- ------- --------\n");
    
#print "$max\n";

for($i=1;$i<=$max;$i++)  {
    
    #print $i . " " . $aa[$i][0] . "\n";

    #printf(out "%-4d",$i);
    
    if($aa[$i][0] eq '') {
	
	$aa[$i][0]='*';
    }

    if($aa[$i][0] ne '*') {
    
	printf(out "% -4d",$i);
	
	for $j(0..6) {
	    
	    if($aa[$i][$j] eq '') {
	    
		$aa[$i][$j]='0.00'; 
	    } 
	    
	    printf( out "%6s ",$aa[$i][$j]);
	}    
	 
	print out "\n";
    }
}

close(out);

