#ifndef __GA_POTENTIAL_SCORES_H__
#define __GA_POTENTIAL_SCORES_H__

#define ANGSTROMS_PER_HELIX_RESIDUE 1.5 // Length (Angstroms) per residue in a helix.

/* Global variables */
extern int use_Prosa;
	// Whether to make an external system call to Prosa (prosa2003) when evaluating a peptide.
	// The Prosa score is used in place of the normal threading score.
extern int computeSecondaryScore; // Whether to compute a secondary structure score.
extern int use_Proteus;
	// Whether to call Proteus to predict the secondary structure for use with the secondary structure score.
extern char *proteus_dir; // Directory in which to call Proteus (run_proteus).
extern char secondary_structure_filename[];
	// The name of the secondary structure file, either given by the user or computed by Proteus.
extern char *target_sec_structure;
		// "target_sec_structure" is the secondary structure assignment read in from a file, e.g.
		// HHHHCCCBBBCCCHHHBBBCCC and so on, where H = Helix, B = Beta strand, - or C = coil.
		// It is the secondary structure against which the secondary structure score is computed.


/* Function prototypes */
float get_radGyr(ResData *res, int residue_number, double *delta);
float get_rama_value(ResData *residues, int numRes);
float getOmegaScore(ResData *residues, int numRes);
void printRamaAndOmegaScoreBreakdown(ResData *residues, int numRes);
float getChiScore(ResData *residues, int totalRes);
float get_hcount(ResData *res, int number, double *sumEnergy, double *linPerc, double *hperc, int *disulphide_bond_count);
void calculate_hbonds(ResData *res, int number, double *sumEnergy, int *disulphide_bond_count);
float calc_energy_score(float energy);
int getHbondIndex_GA_Pot(double hodist, Hbonds *hbonds);
float get_hbond_energy(ResData res1, ResData res2);
float get_bump_score(ResData *res, int n);
float getProsaScore(char *PDB_filename, int thread_id);
void print_C_alpha_distances(ResData *residues, int numRes);


#endif
