#ifndef __ROTAMER_H__
#define __ROTAMER_H__

/*
 * Rotamer library data for one residue. This data is loaded from the file bbrotamer.lib.new.
 * The residue name is given in res_name. In avg_chi and prob, the first two indices are the
 * phi index and psi index, respectively, such each index value represents 10 degrees. For
 * each combination of phi and psi, there are 3 main possible chi angles recorded in avg_chi.
 * The corresponding probabilities of each of these chi angles is given in prob.
 *
 * A continuum of chi angles are possible for any given phi-psi pair, but the chi angles tend
 * to cluster around 3 chi angle values. The average chi angle for each of these clusters is
 * what is recorded in avg_chi.
 */
typedef struct RotamerLibData {
	char res_name[4]; // Null-terminated 3-letter amino acid abbreviation.
	float avg_chi[36][36][3];
	float prob[36][36][3];
} RotamerLibData;

void loadRotamerLibrary();
float getChiProbability(char *res_name, double phi, double psi, double chi);
double getMostProbableChi(char *resName, double phi, double psi);

#endif
