#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#ifndef _WIN32
#include <unistd.h>
#endif

#include "util.h"
#include "secstruct.h"
#include "hetatm.h"
#include "residue.h"
#include "main_shiftx.h"
#include "vector.h"
#include "torison.h"
#include "ssbond.h"
#include "hydrogens.h"

#ifdef _WIN32        /* SJN */
 #include "direct.h" /* SJN */
#else                /* SJN */
 #include <unistd.h> /* SJN */
#endif


/* Function prototypes */

// Functions in Dss.c:
void do_ss(RESIDUE rz[],int rno);
void define1(RESIDUE rz[], int num_res);
void get_ca_dist(RESIDUE rz[], int num_res, double ca_matrix[]);
//static double get_dist(RESIDUE rz[],int indxi,int indxj);
void post_check(RESIDUE rz[], int num_res);
//static double get_rmsd(double ca_matrix[], double mask[], int maskLen, int idx);
void check_beta(RESIDUE rz[], int num_res, double ca_matrix[]);
void check_helix(RESIDUE rz[], int num_res, double ca_matrix[]);
void hbond(RESIDUE *res, int resIdx, RESIDUE resArray[], int num_res);
int getHbondIndex(float newDist,BOND_REC bondrec[MAX_HBONDS]);
float v_get_dist(float onex,float oney,float onez,float twox,float twoy,float twoz);
void calc_hcoords(RESIDUE resArray[], int num_res);
float get_energy(RESIDUE *res1,RESIDUE *res2);
void check_mainsidebond(int didx, int dneighbour, int donor, int aidx, int acceptor, int aneighbour,RESIDUE resArray[]);
void getall_sbond(int idx, RESIDUE resArray[], int num_res);
void check_sidebond(int didx, int dneighbour, int donor, int aidx,int acceptor, int aneighbour,RESIDUE resArray[]);
void calc_bturn(RESIDUE resArray[], int num_res);
void check_ho_bond(int didx,int donor,int dneighbour,void (*bond_check)(),RESIDUE resArray[], int num_res);
void pickType(int val1_1, int val2_1, int val1_3, int val2_3, int index, char *type1, char *type3,RESIDUE resArray[]);
int checkType(int val1,int val2,int val3,int val4,int val5,int indx,char *bturn_type,int bounds[5],RESIDUE resArray[]);
void store_dist(float dist, int didx, int aidx, int donor, int neighbour,int acceptor,int aneighbour,RESIDUE resArray[]);
float get_angle(RESIDUE rec1,int vec1a,int vec1b, RESIDUE rec2,int vec2a,int vec2b);
void define3(RESIDUE resArray[], int num_res);
void assign_define3(RESIDUE resArray[], int num_res);
int get_beta(int residx,float phi,RESIDUE resArray[]);
int old_get_beta(int i, float phi,float psi,float lastphi,RESIDUE resArray[]);
void smooth_define3(RESIDUE resArray[], int num_res);
void smooth_helix(RESIDUE resArray[], int num_res);
void smooth_beta(RESIDUE resArray[], int num_res);
void filter_define3(RESIDUE resArray[], int num_res);
void do_consensus(RESIDUE resArray[], int num_res);
void bturn_ss(RESIDUE resArray[], int num_res);
void check_turn(int i,char *bturn,RESIDUE resArray[]);
void smooth_ss(RESIDUE resArray[], int num_res);
void do_consens_caps(int i,RESIDUE resArray[], int num_res);
void do_consens_bridge(int i,RESIDUE resArray[], int num_res);
int hbonded(int i,RESIDUE resArray[]);
void assignSS(int residx,int type,char *value,RESIDUE resArray[]);
int legitBond(RESIDUE residue,RESIDUE resArray[], int bondType);
int getType(RESIDUE resx,int type);
void do_define2(RESIDUE resArray[], int num_res);

// Local static functions:
static long compute_secondary_score(RESIDUE *Rz, int rno, char *target,
		long *inStruct, int *ssErrors, int *resCount);
static void print_secondary_structure(RESIDUE *Rz, int rno);
static void print_secondary_structure_list(RESIDUE *Rz, int rno);
static void output_PyMOL_secondary_structure_files(RESIDUE *Rz, int rno, char *base_PDB_filename);
static void print_PyMOL_SS_line(FILE *fp, RESIDUE *Rz, int start, int end, char secStr);
static void print_PyMOL_SS_color_line(FILE *fp, RESIDUE *Rz, int start, int end, char secStr);
static void print_PyMOL_hbonds(FILE *fp, RESIDUE *Rz, int index, char *object_name);
static int ReadPDB_2(FILE *in,RESIDUE rez[],HETATM **h);



/*
 * This function calculates a secondary structure score for a peptide. A consensus secondary structure
 * assignment is determined by the do_ss() function, which assigns secondary structure based on various
 * analyses of the 3-D peptide structure. (No external secondary structure prediction program is used).
 * This consensus secondary structure is then compared with the given 'target' secondary structure
 * (which may be the "known" secondary structure). A helix matched with a Beta strand is given a score
 * of 8, a helix or Beta strand matched with a coil is given a score of 4, and perfect matches are given
 * a score of 0. Thus, a higher score indicates greater disagreement between the secondary structure
 * computed based on the 3-D structure and the given 'target' secondary structure.
 *
 * Consider using the function secondary_score() instead, since this function requires the PDB file to
 * be written to a file first, which may be inefficient. Also, repeated calls to Read_PDB2() resulting
 * from repeated calls to this function may result in a memory leak. 
 *
 * Note: If target is NULL, the secondary structure of the given PDB file will still be computed and
 * printed out if print_SecStr_list is TRUE.
 *
 * infile: Name of the PDB file to load.
 * target: char array with the "known" secondary structure, or whatever secondary structure we are using
 *    for comparison. e.g. HHHHCCCBBBCCCHHHBBBCCC and so on, where H = Helix, B or E = Beta strand, C = coil,
 *    - = unknown.
 * inStruct: This function sets this to the number of residues with secondary structure (helix or Beta
 *    strand) in the consensus secondary structure computed in this function.
 * ssErrors: int[9] that is set by this function to hold the counts for various types of mismatches between
 *    the given 'target' secondary structure and the computed consensus secondary structure.
 * print_SecStr: If TRUE, the secondary structure consensus will be printed. If FALSE, it will not be
 *    printed unless print_SecStr_list is TRUE. Added by David Arndt, Feb 20, 2008.
 * print_SecStr_list: If TRUE, the secondary structure consensus and predicted beta turn locations will be printed.
 *    If FALSE, they will not be printed (except that if print_SecStr is TRUE then the secondary structure
 *    will still be printed). Added by David Arndt, May 2, 2007.
 * output_PyMOL_files: If TRUE, files with PyMOL commands will be output, which can then be run in
 *    PyMOL to view the secondary structure assigned here. Added by David Arndt, Mar 12, 2008.
 * base_PDB_filename: The base name of the PDB file supplied by the user. This is used as the base name
 *    for outputting PyMOL command files, and is only required if output_PyMOL_files is TRUE. Note that
 *    the parameter 'infile' is used to specify the PDB file from which the secondary structure is calculated.
 * resCount: This function will set this value to the number of residues for which a secondary structure
 *    score value is assignable (i.e. number of residues without '-' as target secondary structure, and
 *    excluding the last residue because it is excluded in the code for some reason). Intended for use when
 *    later normalizing the score. Added by David Arndt, Jan 7, 2008.
 * returns: The secondary structure score.
 */
long secondary_score_from_PDB_file(char *infile, char *target, long *inStruct, int *ssErrors, int print_SecStr,
		int print_SecStr_list, int output_PyMOL_files, char *base_PDB_filename, int *resCount) {
	RESIDUE      Rz[MAX_RES_NUM];  /* SJN */
	long rno = 0; // rno is the number of residues in Rz minus 1.
	HETATM *water = NULL;
	FILE *ff;
	int i, j, k;
	long sScore = 0;

	ff = fopen(infile,"rt");
	if (ff == NULL) {
		fprintf(stderr, "ERROR: Could not open file in secondary_score() in secstruct.c\n");
		exit(1);
	}


	/* Initialize RESIDUE */

	memset( &Rz, 0,MAX_RES_NUM * sizeof(RESIDUE) );
	for (i=0; i < MAX_RES_NUM; i++)
	{
		for (j=0; j < NUM_COORDS; j++) 
		{
			Rz[i].coords[j].x = NA;
			Rz[i].coords[j].y = NA;
			Rz[i].coords[j].z = NA;
		}
		for (k=0; k < MAX_HBONDS; k++) 
		{
			Rz[i].hbonds[k].partner = NA;
			Rz[i].sidebond[k].partner = NA;
		}
	}
	
  	rno = ReadPDB_2(ff, Rz, &water);
	fclose(ff);
	
	
	for (i=0; i <= rno; i++)   // SJN added = 
	{
		if (i > 0) 
		{
			if (Rz[i-1].num == Rz[i].num - 1) 
			{
				atm_vector(&Rz[i-1],"C",&(Rz[i].lastc));
				atm_vector(&Rz[i-1],"CA",&(Rz[i].lastca));
			}  			
		}
		
		torison_angles(&Rz[i]);
	}
	
	
	// Compute the secondary structure of the given structure.
	do_ss(Rz, rno);
	
	// Print secondary structure info depending on options supplied by the user.
	print_secondary_structure_info(&Rz[0], rno, print_SecStr, print_SecStr_list, output_PyMOL_files,
		base_PDB_filename);
	
	
	// Compute the secondary structure score (if target is not NULL).
	
	if (target != NULL) {
		sScore = compute_secondary_score(Rz, rno, target, inStruct, ssErrors, resCount);
	}
	
	return(sScore);
}


/*
 * This function calculates a secondary structure score for a peptide. A consensus secondary structure
 * assignment is determined by the do_ss() function, which assigns secondary structure based on various
 * analyses of the 3-D peptide structure. (No external secondary structure prediction program is used).
 * This consensus secondary structure is then compared with the given 'target' secondary structure
 * (which may be the "known" secondary structure). A helix matched with a Beta strand is given a score
 * of 8, a helix or Beta strand matched with a coil is given a score of 4, and perfect matches are given
 * a score of 0. Thus, a higher score indicates greater disagreement between the secondary structure
 * computed based on the 3-D structure and the given 'target' secondary structure.
 *
 * This handles the data internally without the need to write the structure to a PDB file and read it
 * back in (as is done with the older secondary_score_from_PDB_file() function). If sec_str_is_assigned
 * is TRUE, we can assume that the secondary structure has already been assigned to the given RESIDUE
 * array 'Rz', and simply compute the secondary structure score and return it. If sec_str_is_assigned
 * is FALSE, then the function copies the coordinates from the given ResData array 'resDatas' to the
 * given RESIDUE array 'Rz', which is required by do_ss(), which assigns the secondary structure. (This
 * internal format transfer is similar to what is done by the calcChemShifts() function in Main_shiftx.c).
 * 
 * This function assumes that the peptide represented by resDatas has already been built
 * (such as by buildPeptide()).
 *
 * This function also assumes that 'Rz' points to an array of numRes RESIDUE structs that have already
 * been allocated, presumably by a call to create_RESIDUE_array_from_ResData_array(), and that the
 * variables in each RESIDUE struct may or may not have been modified by a previous call to
 * calcChemShifts() or secondary_score(). If sec_str_is_assigned is TRUE, each RESIDUE struct's 'atom'
 * member variable must point to a doubly-linked list of ATOM structs that have already been allocated
 * with the correct number of non-hydrogen atoms, and that the variables in each of these ATOM structs
 * (such as atom name or 'type') have already been set. HOWEVER, there may or may not be ATOM structs
 * for hydrogen atoms at the beginning of the list (as added by a previous call to calcChemShifts() or
 * secondary_score()); if there are hydrogen atoms, they will be removed.
 *
 * Note that hydrogen atoms are not copied from the ResData array to the RESIDUE array, and new hydrogen
 * atoms are added to the RESIDUE array structure. This could affect the assigned secondary structure,
 * but will hopefully not make a big difference. (This problem is not present when using
 * secondary_score_from_PDB_file()).
 *
 * resDatas: Array of numRes ResData structs representing the peptide.
 * numRes: Number of residues in the peptide.
 * Rz: Array of RESIDUE structs initialized as described above.
 * target: char array with the "known" secondary structure, or whatever secondary structure we are using
 *    for comparison. e.g. HHHHCCCBBBCCCHHHBBBCCC and so on, where H = Helix, B or E = Beta strand, C = coil,
 *    - = unknown.
 * sec_str_is_assigned: Should be TRUE if the secondary structure has already been assigned to the given
 *    RESIDUE array 'Rz'; FALSE otherwise.
 * inStruct: This function sets this to the number of residues with secondary structure (helix or Beta
 *    strand) in the consensus secondary structure computed in this function.
 * ssErrors: int[9] that is set by this function to hold the counts for various types of mismatches between
 *    the given 'target' secondary structure and the computed consensus secondary structure.
 * resCount: This function will set this value to the number of residues for which a secondary structure
 *    score value is assignable (i.e. number of residues without '-' as target secondary structure, and
 *    excluding the last residue because it is excluded in the code for some reason). Intended for use when
 *    later normalizing the score. Added by David Arndt, Jan 7, 2008.
 * returns: The secondary structure score.
 */
long secondary_score(ResData *resDatas, int numRes, RESIDUE *Rz, char *target, int sec_str_is_assigned,
	long *inStruct, int *ssErrors, int *resCount) {
	
	SSBOND       *ssbonds=NULL, *sspt;
	long int     seed=-1;
	int i;
	long rno = (long) (numRes-1); // rno is the number of residues in Rz minus 1.
	
	// Assign the secondary structure of the RESIDUE array:
	
	if (!sec_str_is_assigned) {
		
		copy_coords_from_ResData_array_to_RESIDUE_array(resDatas, numRes, Rz);
		
		for (i=0; i <= rno; i++)   // SJN added = 
		{
			if (i > 0) 
			{
				if (Rz[i-1].num == Rz[i].num - 1) 
				{
					atm_vector(&Rz[i-1],"C",&(Rz[i].lastc));
					atm_vector(&Rz[i-1],"CA",&(Rz[i].lastca));
				}  			
			}
			
			torison_angles(&Rz[i]);
		}
		
		for (i = 0; i <= rno; i++)
		{
			add_hydrogens(&Rz[i],&seed);
		}
		
		
		for (i = 0; i <= rno; i++)
		{
			if (!strcmp(Rz[i].type,"CYS"))
			{
				for (sspt = ssbonds; sspt != NULL; sspt = sspt->next)
				{
					if (sspt->resA == Rz[i].numLabel && sspt->insCodeA == Rz[i].insCode)
					{
						Rz[i].ssPartner = sspt->resB;
						Rz[i].ssPartnerInsCode = sspt->insCodeB;
					}
					else if (sspt->resB == Rz[i].numLabel && sspt->insCodeB == Rz[i].insCode)
					{
						Rz[i].ssPartner = sspt->resA;
						Rz[i].ssPartnerInsCode = sspt->insCodeA;
					}
				}
			}
		}
		
		// Compute the secondary structure of the given structure.
		do_ss(Rz, rno);
	}
	
	// Compute the secondary structure score:
	
	if (target != NULL) {
		return compute_secondary_score(Rz, rno, target, inStruct, ssErrors, resCount);
	}
	else {
		// Not target secondary structure, so cannot compute the secondary structure score.
		return 0L;
	}
}


/*
 * Returns the secondary structure score for the given structure. This function assumes
 * that the secondary structure has already been assigned to the structure.
 *
 * Rz: Array of RESIDUE structs representing the peptide.
 * rno: Number of residues in the peptide, minus 1.
 *
 * See description of secondary_score() for more info.
 */
static long compute_secondary_score(RESIDUE *Rz, int rno, char *target,
		long *inStruct, int *ssErrors, int *resCount) {
	
	int i;
	int structCount = 0;
	long sScore = 0;
	
	// Determine resCount.
	*resCount = 0;
	for (i = 0; i < rno; i++) {
		if (target[i] != '-') {
			*resCount += 1;
		}
	}
	
	// Compute secondary structure score.
	
	for (i = 0; i < rno; i++)
	{
		//printf("Rz[%d].consens[0] = %c\n",i, Rz[i].consens[0]);
		switch (Rz[i].consens[0])
		{
			case 'B':
				structCount++;
				//if (target[i] == 'C' || target[i] == '-')
				if (target[i] == 'C')
				{
				  sScore += 4;
				  ssErrors[1]++;
				}
				else if (target[i] == 'H')
				{
					sScore += 8;
					ssErrors[2]++;
				}
				else
				{ 
					ssErrors[0]++;
				}
				break;
				
			case 'H':
				structCount++;
				if (target[i] == 'B')
				{
					sScore += 8;
					ssErrors[6]++;
				}
				//else if (target[i] == '-' || target[i] == 'C')
				else if (target[i] == 'C')
				{
					sScore += 4;
					ssErrors[7]++;
				}
				else
				{
					ssErrors[8]++;
				}
				break;
				
			case 'C':
				if (target[i] != '-' && target[i] != 'C')
				{
					if (target[i] == 'B')
					{
						ssErrors[3]++;
					}
					else
					{
						ssErrors[5]++;
					}
					sScore += 4;
				}
				else
				{
					ssErrors[4]++;
				}
				break;
		}
	}
	
	*inStruct = structCount;
	
	return(sScore);
}

/*
 * Prints out various info about the assigned secondary structure depending on the options
 * specified by the user. This function assumes that the secondary structure has already
 * been assigned to the structure.
 *
 * Rz: Array of RESIDUE structs representing the peptide.
 * rno: Number of residues in the peptide, minus 1.
 * print_SecStr: If TRUE, the secondary structure consensus will be printed. If FALSE, it will not be
 *    printed unless print_SecStr_list is TRUE. Added by David Arndt, Feb 20, 2008.
 * print_SecStr_list: If TRUE, the secondary structure consensus and predicted beta turn locations will be printed.
 *    If FALSE, they will not be printed (except that if print_SecStr is TRUE then the secondary structure
 *    will still be printed). Added by David Arndt, May 2, 2007.
 * output_PyMOL_files: If TRUE, files with PyMOL commands will be output, which can then be run in
 *    PyMOL to view the secondary structure assigned here. Added by David Arndt, Mar 12, 2008.
 * base_PDB_filename: The base name of the PDB file supplied by the user. This is used as the base name
 *    for outputting PyMOL command files, and is only required if output_PyMOL_files is TRUE. Note that
 *    the parameter 'infile' is used to specify the PDB file from which the secondary structure is calculated.
 */
void print_secondary_structure_info(RESIDUE *Rz, int rno, int print_SecStr,
		int print_SecStr_list, int output_PyMOL_files, char *base_PDB_filename) {
	
	if (print_SecStr || print_SecStr_list) {
		print_secondary_structure(Rz, rno);
	}
	
	if (print_SecStr_list) {
		print_secondary_structure_list(Rz, rno);
	}
	
	if (output_PyMOL_files) {
		output_PyMOL_secondary_structure_files(Rz, rno, base_PDB_filename);
	}
}


/*
 * Prints the sequence (single-letter residue codes) and the assigned secondary
 * structure horizontally on two lines.
 */
static void print_secondary_structure(RESIDUE *Rz, int rno) {
	
	int i;
	
	// Print the sequence.
	for (i=0; i <= rno; i++) {
		printf("%c", res_str_to_char(Rz[i].type));
	}
	printf("\n");

	// Print the secondary structure.
	for (i=0; i <= rno; i++) {
		printf("%c", Rz[i].consens[0]);
	}
	printf("\n");
	
}


/*
 * Prints the assigned secondary structure and other info as a detailed vertical list in
 * a VADAR-like format. Prints residue IDs and names, the 3 secondary structure
 * assignments and the consensus assignment, the hydrogen bonds and their energies, and
 * assigned Beta turns.
 */
static void print_secondary_structure_list(RESIDUE *Rz, int rno) {
	int i;
	char hbond_string[10];
	char hbond_energy_string[16];
	
	printf("\n");
	printf("RES. RES. SCND  HBOND   HBOND       BTURN\n");
	printf("NUM. NAME STRUC HBOND   ENERGY      BTURN\n");
	
	for (i=0; i <= rno; i++) {
		
		if (	Rz[i].hbonds[0].partner != NA &&
				Rz[i].hbonds[1].partner != NA &&
				Rz[i].hbonds[0].energy < WEAK_HBOND_THRESHOLD &&
				Rz[i].hbonds[1].energy < WEAK_HBOND_THRESHOLD) {
			sprintf(hbond_string, "%d,%d",\
				Rz[Rz[i].hbonds[0].partner].numLabel, Rz[Rz[i].hbonds[1].partner].numLabel);
			sprintf(hbond_energy_string, "%5.2f,%5.2f",
				Rz[i].hbonds[0].energy, Rz[i].hbonds[1].energy);
		}
		else if (Rz[i].hbonds[0].partner != NA &&
				Rz[i].hbonds[0].energy < WEAK_HBOND_THRESHOLD) {
			sprintf(hbond_string, "%d", Rz[Rz[i].hbonds[0].partner].numLabel);
			sprintf(hbond_energy_string, "%5.2f", Rz[i].hbonds[0].energy);
		}
		else if (Rz[i].hbonds[1].partner != NA &&
				Rz[i].hbonds[1].energy < WEAK_HBOND_THRESHOLD) {
			sprintf(hbond_string, "%d", Rz[Rz[i].hbonds[1].partner].numLabel);
			sprintf(hbond_energy_string, "%5.2f", Rz[i].hbonds[1].energy);
		}
		else {
			hbond_string[0] = '\0';
			hbond_energy_string[0] = '\0';
		}
		
		printf("%-5d%3s  %c%c%c %c %-7s %-11s %s\n", Rz[i].numLabel, Rz[i].type,
			Rz[i].dstruct1[0], Rz[i].dstruct2[0], Rz[i].dstruct3[0],
			Rz[i].consens[0], hbond_string, hbond_energy_string, Rz[i].bturn);
	}
	printf("\n");
}


/*
 * Prints out PyMOL command files that can be run from PyMOL to display the
 * secondary structure assigned by do_ss().
 */
static void output_PyMOL_secondary_structure_files(RESIDUE *Rz, int rno, char *base_PDB_filename) {
	
	int i;
	char prev_ss = '\0'; // Secondary structure of previous residue.
	char curr_ss = '\0'; // Secondary structure of current residue.
	int start = -1; // Index of first residue in the secondary structure segment.
	int end = -1; // Index of last residue in the secondary structure segment.
	char current_dir[512];
	char object_name[512];
	
	FILE *PyMOL_ss_fp;
	FILE *PyMOL_ss_color_fp;
	FILE *PyMOL_hbonds_fp;
	
	char PyMOL_ss_filename[512];
	char PyMOL_ss_color_filename[512];
	char PyMOL_hbonds_filename[512];
	
	getcwd(current_dir, 512);
	
	if (base_PDB_filename[0] == '/') {
		// Assume that base_PDB_filename contains the full path to the input PDB file.
		sprintf(PyMOL_ss_filename, "%s.pymol_ss", base_PDB_filename);
		sprintf(PyMOL_ss_color_filename, "%s.pymol_ss_colors", base_PDB_filename);
		sprintf(PyMOL_hbonds_filename, "%s.pymol_hbonds", base_PDB_filename);
	}
	else {
		sprintf(PyMOL_ss_filename, "%s/%s.pymol_ss", current_dir, base_PDB_filename);
		sprintf(PyMOL_ss_color_filename, "%s/%s.pymol_ss_colors", current_dir, base_PDB_filename);
		sprintf(PyMOL_hbonds_filename, "%s/%s.pymol_hbonds", current_dir, base_PDB_filename);
	}
	
	
	// Print secondary structure assignments and colors:
	
	PyMOL_ss_fp = fopen(PyMOL_ss_filename, "w");
	PyMOL_ss_color_fp = fopen(PyMOL_ss_color_filename, "w");
	
	for (i=0; i <= rno; i++) {
		curr_ss = Rz[i].consens[0];
		
		if (curr_ss != prev_ss) {
			// We are starting a new secondary structure segment.
			
			end = i - 1;
			
			// Print the info for the previous segment first.
			if (i > 0) {
				print_PyMOL_SS_line(PyMOL_ss_fp, Rz, start, end, prev_ss);
				print_PyMOL_SS_color_line(PyMOL_ss_color_fp, Rz, start, end, prev_ss);
			}
			
			// Info for new segment.
			start = i;
		}
		
		prev_ss = curr_ss;
	}
	
	end = rno;
	print_PyMOL_SS_line(PyMOL_ss_fp, Rz, start, end, prev_ss);
	fprintf(PyMOL_ss_fp, "rebuild\n");
	print_PyMOL_SS_color_line(PyMOL_ss_color_fp, Rz, start, end, prev_ss);
	
	fclose(PyMOL_ss_fp);
	fclose(PyMOL_ss_color_fp);
	
	
	// Print hydrogen bonds:
	
	get_PDB_name_from_path(base_PDB_filename, object_name);
	strcat(object_name, "_hbonds");
	
	PyMOL_hbonds_fp = fopen(PyMOL_hbonds_filename, "w");
	
	for (i=0; i <= rno; i++) {
		print_PyMOL_hbonds(PyMOL_hbonds_fp, Rz, i, object_name);
	}
	
	fclose(PyMOL_hbonds_fp);
	
	// Print info about PyMOL commands to use the files:
	
	printf("Run these commands in PyMOL to set the current visible structure's appearance.\n");
	printf("To set GA Potential secondary structure:\n");
	printf("@%s\n", PyMOL_ss_filename);
	printf("To set colors according to GA Potential secondary structure:\n");
	printf("@%s\n", PyMOL_ss_color_filename);
	printf("To create object \'%s\' with GA Potential hydrogen bonds:\n", object_name);
	printf("@%s\n", PyMOL_hbonds_filename);
	
	printf("\n");
}


/*
 * Prints a single PyMOL script line for setting the secondary structure of a residue.
 */
static void print_PyMOL_SS_line(FILE *fp, RESIDUE *Rz, int start, int end, char secStr) {
	
	char pymol_ss_char;
	
	if (secStr == 'H') { pymol_ss_char = 'H'; }
	else if (secStr == 'B') { pymol_ss_char = 'S'; }
	else if (secStr == 'C') { pymol_ss_char = 'L'; }
	else { return; } // Should not happen.
	
	fprintf(fp, "alter (resi %d:%d) & (visible), ss=\'%c\'\n", Rz[start].numLabel, Rz[end].numLabel, pymol_ss_char);
}


/*
 * Prints a single PyMOL script line for coloring a residue based on its secondary structure.
 */
static void print_PyMOL_SS_color_line(FILE *fp, RESIDUE *Rz, int start, int end, char secStr) {
	
	char color[8];
	
	if (secStr == 'H') { strcpy(color, PYMOL_SS_COLOR_H); }
	else if (secStr == 'B') { strcpy(color, PYMOL_SS_COLOR_B); }
	else if (secStr == 'C') { strcpy(color, PYMOL_SS_COLOR_C); }
	else { return; } // Should not happen.
	
	fprintf(fp, "color %s, (resi %d:%d) & (visible) & (name c*)\n", color, Rz[start].numLabel, Rz[end].numLabel);
}


/*
 * Prints the PyMOL script lines that will display all the hydrogen bonds in which the
 * residue with index 'index' participates. The hydrogen bonds will be put into the
 * object with the given object name. (Note that duplicate hydrogen bonds will be included
 * for both the acceptor and the donor).
 */
static void print_PyMOL_hbonds(FILE *fp, RESIDUE *Rz, int index, char *object_name) {
	
	int k;
	int partner;
	
	for (k = 0; k < MAX_HBONDS; k++) {
		partner = Rz[index].hbonds[k].partner;
		if (partner != NA && Rz[index].hbonds[k].energy < WEAK_HBOND_THRESHOLD) {
			
			if (Rz[index].hbonds[k].DA_FLAG == ACCEPTOR) {
				fprintf(fp, "dist %s = (resi %d) & (name o) & (visible), (resi %d) & (name n) & (visible)\n", object_name,
					Rz[index].numLabel, Rz[partner].numLabel);
			}
			else {
				fprintf(fp, "dist %s = (resi %d) & (name n) & (visible), (resi %d) & (name o) & (visible)\n", object_name,
					Rz[index].numLabel, Rz[partner].numLabel);
			}
		}
	}
}


static int ReadPDB_2(FILE *in,RESIDUE rez[],HETATM **h)
{
	char inn[1000],iupacCode,insCode;
	int /* args, */ rs=-1, i /*, j*/;
	char atom[15], res[15], /*resndx[5],*/ label[2], /*sublabel,*/ atom2[15],*pt;
	char tmp[30];
	int atom_num, res_num,idx,num = -1;
	float x, y, z, a, b;
	ATOM *q;
	int ignore = 0;  // Flag: the PDB has more than one structure, ignore all but the first
	int xplorIndex, bmrbIndex, hIndex;

	fgets(inn,sizeof(inn),in);
	while (!feof(in)) 
	{
		if (!strncmp(inn,"HETATM",6))
		{
			if (UseChain == 0 || (inn[21] == UseChain || inn[21] == ' '))
			{
				*h = AddWaterOxy(inn,*h);
			}
		}
		else if (!strncmp(inn,"ATOM",4) && !ignore) 
		{
			// The following code for parsing the PDB file was added by David Arndt
			// on Mar 10, 2007. It replaces some of the code further below. However,
			// this new code supplies default values for occupancy and B-factor if
			// they are not present in the file, but these values may affect the
			// predicted chemical shifts (but I'm not sure).
			
			//printf("\n");
			//printf("%s\n", inn);
			
			// Read atom_num.
			if (sscanf(&inn[6], "%5d", &atom_num) != 1) {
				fprintf( stderr, "File format error...\n" );
				fprintf( stderr, "%s\n", inn );
				exit(1);
			}
			//printf("atom_num: %d\n", atom_num);
			
			// Read atom2 (atom name).
			for (i=0; i<5; i++) {
				atom2[i] = inn[12+i];
			}
			atom2[4] = (char)NULL;
			//printf("atom2: >%s<\n", atom2);
			
			// Read residue name.
			for (i=0; i<4; i++) {
				res[i] = inn[17+i];
			}
			res[3] = (char)NULL;
			//printf("res: >%s<\n", res);
			
			// Read label (chain identifier code).
			label[0] = inn[21];
			label[1] = (char)NULL;
			if (label[0] == ' ') {
				label[0] = 'A'; // If no label is given in the file, provide the label 'A'.
			}
			//printf("label: >%s<\n", label);
			
			// Read residue number.
			if (sscanf(&inn[22], " %d ", &res_num) != 1) {
				fprintf( stderr, "File format error...\n" );				
				fprintf( stderr, "%s\n", inn );
				exit(1);
			}
			//printf("res_num: %d\n", res_num);
			
			// Read insertion code.
			insCode = inn[26];
			
			// Read coordinates.
			strncpy(tmp, &inn[30], 24);
			tmp[24] = (char)NULL;
			sscanf(tmp,"%f %f %f", &x, &y, &z);
			//printf("coords: %f %f %f\n", x, y, z);
			
			// Read a (occupancy value for atom).
			strncpy(tmp, &inn[54], 6);
			tmp[6] = (char)NULL;
			if (sscanf(tmp, "%f", &a) != 1) {
				a = 1.00; // Supply default if none is given in the file.
			}
			//printf("a: %f\n", a);
			
			// Read b (B-value (thermal factor)).
			strncpy(tmp, &inn[60], 6);
			tmp[6] = (char)NULL;
			if (sscanf(tmp, "%f", &b) != 1) {
				b = 0.00; // Supply default if none is given in the file.
			}
			//printf("b: %f\n", b);
			
			//debug
			/*
			printf("atom_num: %d\n", atom_num);
			printf("atom2: >%s<\n", atom2);
			printf("res: >%s<\n", res);
			printf("label: >%s<\n", label);
			printf("res_num: %d\n", res_num);
			printf("coords: %f %f %f\n", x, y, z);
			printf("a: %f\n", a);
			printf("b: %f\n", b);
			*/
			
			// End additions by David Arndt, Mar 10, 2007.
			
			/*
			args = sscanf(inn + 6,"%5d%*c%4[A-Za-z0-9* ]%*c%3s%*c%c%4d%c%8f%8f%8f%6f%6f",
			 &atom_num, atom2, res,&label,&res_num, &insCode,&x, &y, &z, &a, &b);
			*/
			
			/*
			// debug
			printf("atom_num: %d\n", atom_num);
			printf("atom2: >%s<\n", atom2);
			printf("res: >%s<\n", res);
			printf("label: >%s<\n", label);
			printf("res_num: %d\n", res_num);
			printf("coords: %f %f %f\n", x, y, z);
			printf("a: %f\n", a);
			printf("b: %f\n", b);
			*/
			
			/*
			args = sscanf( inn+6, "%5d%6[A-Z 0-9]%3s%4d %f %f %f %f %f",
			 &atom_num, atom2, res,&res_num, &x, &y, &z, &a, &b );
					//    ATOM atom_num atom residue res_num x y z junk junk
					// ie ATOM 1 N SER 145 1.00 1.00 1.00 etc
			*/
			/*
			if (args != 9) {
			 args=-sscanf( inn+6, "%5d%6[A-Z 0-9]%3s%2s%4s %f %f %f %f %f", 
			 &atom_num, atom2, res, label,resndx, &x, &y, &z, &a, &b );
			 */
 					/*    ATOM atom_num atom residue chain res_num x y z junk junk */
					/* ie ATOM 1 N SER A 145 1.00 1.00 1.00 etc */
			/*
			}
			*/
			//if (args != 9 && args != -10) 
			/*
			if (args != 11)
			{
				fprintf( stderr, "File format error...\n" );				
				fprintf( stderr, "%s\n", inn );
				exit(1);
			}
			*/
			
			if (UseChain != 0 && label[0] != UseChain)
			{
				fgets(inn,sizeof(inn),in);
				continue;
			}

			/*
			if (insCode != ' ')
			{
				fgets(inn,sizeof(inn),in);
				continue;
			}
			*/
			/*
			if (args == -10) 
			{
				sublabel = resndx[strlen(resndx)-1];
				if (!isdigit(sublabel))
				{
					fgets(inn,sizeof(inn),in);
					continue;
				}
				res_num = atoi(resndx);  
			
				if (UseChain != 0 && label[0] != UseChain) 
				{
					fgets(inn,sizeof(inn),in);
					continue;
				}
				*/
		
	/*
      if (strlen(res) > 3) 
	  {
		memmove(&res[0],&res[strlen(res)-3],4);
      }
	*/
			if (iupac_code_safe(res) == 'X')   // skip over weird residues
			{
				fgets(inn,sizeof(inn),in);
				continue;
			}

			if (rs == -1 || rez[rs].numLabel != res_num || rez[rs].insCode != insCode)
			{
				if (rs != -1 && res_num < rez[rs].numLabel)  // likely indicating that there's more than one structure here
				{
#ifdef _DEBUG
					printf("Ha!\n");
#endif
					ignore = 1;
					fgets(inn,sizeof(inn),in);
					continue;
				}
				else if (rs != -1 && res_num > rez[rs].numLabel + 1)  // Deal with gaps in the PDB data
				{
					num += (res_num - rez[rs].num - 1);
				}
				rs++;
				num++;
				rez[rs].numLabel = res_num;
				rez[rs].insCode = insCode;
				rez[rs].num = num;
				strcpy( rez[rs].type, res );
			}
			else if (strcmp(rez[rs].type, res ))
			{
				fprintf( stderr, "Unexpected change in residue type!\n" );
				fprintf( stderr, "%s\n", inn );
				fprintf( stderr, "Aborting...\n" );
				exit(1);
			}


			atom[0] = '\0';
			pt = strtok(atom2," ");
			strcat(atom,pt);

			iupacCode = iupac_code(res);
			xplorIndex = -1;
			bmrbIndex = -1;
			hIndex = -1;

			if (!strcmp(atom,"CA")) /* Separate record of backbone coords, for convenience */
			{ 
				rez[rs].x = x;
				rez[rs].y = y;
				rez[rs].z = z;
			}

			idx = -1;
			if (!strcmp(atom, "H") || !strcmp(atom, "HN"))
			{
				idx = H_COORD;
			}
			else if (0 == strcmp(atom, "N"))
			{
				idx = N_COORD;
				if (rs != 0)
				{
					rez[rs-1].nextn.x = x;
					rez[rs-1].nextn.y = y;
					rez[rs-1].nextn.z = z;
				}
			}
			else if (0 == strcmp(atom, "C"))
				idx = C_COORD;
			else if (0 == strcmp(atom, "O"))
				idx = O_COORD;
			else if (0 == strcmp(atom, "NH1"))
				idx = NH1_COORD;
			else if (0 == strcmp(atom, "NH2"))
				idx = NH2_COORD;
			else if (0 == strcmp(atom, "NZ"))
				idx = NZ_COORD;
			else if (0 == strcmp(atom, "NE"))
				idx = NE_COORD;
			else if (0 == strcmp(atom, "NE1"))
				idx = NE1_COORD;
			else if (0 == strcmp(atom, "NE2"))
				idx = NE2_COORD;
			else if (0 == strcmp(atom, "ND1"))
				idx = ND1_COORD;
			else if (0 == strcmp(atom, "ND2"))
				idx = ND2_COORD;
			else if (0 == strcmp(atom, "OD1"))
				idx = OD1_COORD;
			else if (0 == strcmp(atom, "OD2"))
				idx = OD2_COORD;
			else if (0 == strcmp(atom, "OE1"))
				idx = OE1_COORD;
			else if (0 == strcmp(atom, "OE2"))
				idx = OE2_COORD;
			else if (0 == strcmp(atom, "OG"))
				idx = OG_COORD;
			else if (0 == strcmp(atom, "OG1"))
				idx = OG1_COORD;
			else if (0 == strcmp(atom, "OH"))
				idx = OH_COORD;
			else if (0 == strcmp(atom, "CG"))
				idx = CG_COORD;
			else if (0 == strcmp(atom, "CD"))
				idx = CD_COORD;
			else if (0 == strcmp(atom, "SG"))
				idx = SG_COORD;
			else if (0 == strcmp(atom, "CZ"))
				idx = CZ_COORD;
			else if (0 == strcmp(atom, "CE"))
				idx = CE_COORD;
			else if (0 == strcmp(atom, "CB"))
				idx = CB_COORD;
			
			if (idx != -1) {
				rez[rs].coords[idx].x = x;
				rez[rs].coords[idx].y = y;
				rez[rs].coords[idx].z = z;
			}
			
			if ((q=FindAtom(rez[rs].atom,atom)) == NULL)
			{
				CreateAtom(&rez[rs] );
				strcpy(rez[rs].atom->type, atom );
				rez[rs].atom->p.x = x;
				rez[rs].atom->p.y = y;
				rez[rs].atom->p.z = z;
				rez[rs].atom->occupancy = a;
				rez[rs].atom->BFactor = b;
				rez[rs].atom->hIndex = hIndex;
			}

		}
		fgets(inn,sizeof(inn),in);
  }
  return(rs);
}
