//******************************************************************
// Program: create_constraint_util.c
// Purpose: utility files for creating the constraint files..
// 
//
//******************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "memwatch.h"
#include "create_constraint.h"
#include "create_constraint_util.h"
#include "preditor_parser.h"


//********************************
int is_N_atom(char *atom){
  
  if(strcmp(atom,"N") == 0){
    return(1);
  }else{
    return(0);
  } 
  
}// END is_N_atom **************




//********************************
int is_C_atom(char *atom){
  
  if(strcmp(atom,"C") == 0){
    return(1);
  }else{
    return(0);
  } 
  
}// END is_C_atom **************


 


//********************************
int is_CA_atom(char *atom){
  
  if(strcmp(atom,"CA") == 0){
    return(1);
  }else{ 
    return(0);
  } 
  
}// END is_CA_atom **************




//******************************************************************************************************
int is_bb_atom(char *atom){
  //printf("atom = %s and strlen(atom)=%d\n",atom,strlen(atom));

  if( (strcmp(atom,"N")==0) || (strcmp(atom,"CA")==0) || (strcmp(atom,"C")==0) || (strcmp(atom,"O")==0)){
    //printf("atom = %s and strlen(atom)=%d\n",atom,strlen(atom));
    return(1);
  }else{
    return(-1);
  }
}//*******************************************************************************************************






//**************************************************************************************
//Returns TRUE if the atom is not Hydrogen..
int is_not_Hydrogen(char *atom){
  //printf("atom = %s ",atom);
  //if(atoi(&atom[0])  ) {printf("int = %d  ",atoi(&atom[0]) );
  //if( strncmp(&atom[1],"H",1)==0 || strncmp(&atom[0],"H",1)==0  || strncmp(&atom[0],"Q",1)==0 ){
  //if( strcmp(atom,"HA")==0 || strcmp(atom,"HB")==0
    //printf("%s\t",atom);
  if( (atoi(&atom[0]) && strncmp(&atom[1],"H",1)==0)  || (strncmp(&atom[0],"H",1)==0) || (strncmp(&atom[0],"Q",1)==0) ){
    //printf("%s\t",atom);

    return(0);
  }else{
    return(1);
  }
}//********************* END is_not_hydrogen *********************************************




//**********************************
int is_Q_atom(char *atom){
  if(strncmp(&atom[0],"Q",1)==0) {
    return(1);
  }else{
    return(0);
  }
}// END ****************************

 



//******************************************************
//Stores a pdbline into a PDB struct
void store_pdbline(char *buffer, PDB* pdb){
  int j;
  sscanf(&buffer[5],"%6d",  &(pdb->pdb_atomno) );
  sscanf(&buffer[11],"%5s",   pdb->pdb_atom_name); 
  sscanf(&buffer[17],"%4s",   pdb->pdb_resd_name); 
  //sscanf(&buffer[22],"%4d", &(pdb->pdb_resdno) );
  sscanf(&buffer[22],"%7s",   pdb->pdb_resdno); 
  sscanf(&buffer[31],"%f%f%f",&(pdb->x), &(pdb->y), &(pdb->z) );

  pdb->pt.x = pdb->x; pdb->pt.y = pdb->y; pdb->pt.z = pdb->z; 
  strcpy(pdb->whole_line,buffer); 
  
  for(j=0; j<=26; j++){ pdb->portion_before_coord[j] = buffer[j];}

  //printf("pdb->atomname=%s and strlen(atomname)=%d\n",&pdb->pdb_atom_name, strlen(&pdb->pdb_atom_name[0]));

}
//*********************************************************




//********************************************************88
char* get_pathname(char* full_pathname){

  char*ptr,*path_name,*start_of_pathname;
  int diff_betwn_ptrs;

  start_of_pathname = &full_pathname[0];
  ptr = strrchr(full_pathname, '/');
  if(ptr == NULL){
    path_name = (char*)malloc(1*sizeof(char));
    path_name[0]='\0';
  }else{
    diff_betwn_ptrs = (ptr - start_of_pathname);
    path_name=(char*)malloc( (diff_betwn_ptrs+2+1)*sizeof(char) );
    strncpy(path_name,full_pathname,diff_betwn_ptrs+2);
    path_name[diff_betwn_ptrs+1]='\0';
  }
  return(path_name);
}//*******************************************************************



//Checks to see whether we have a valid phipsi line
// A phipsi line will always start with a digit
// If a line starts with a digit, then we return 1,
//  else we return -1.
//*****************************************************
int is_phipsi_line(char *line){
  
  /* char c;
     c = line[0];
     if(c >= '0' && c <= '9'){
     //printf("char c = %c\n",c);
     return(1);
     }else{
     return(-1);
     }*/
  
  char first_5_chars[6];
  strncpy(first_5_chars,line,5);
  first_5_chars[5]='\0';
  //printf("first_5 = %s, atoi = %d\n",first_5_chars,atoi(first_5_chars) );

  // atoi returns zero if the first chars are not a digit.Usually
  //  a residue number will not start with a zero so this will work most of the time.
  //  However, this technique is not bulletproof...
  if(atoi(first_5_chars)==0){
    return(-1);
  }else{
    return(1);
  }
  
}// END ***********************************************






//**********************************************************************************************
// STORES THE CO-ORDS OF THE INPUT FILE INTO THE CHAIN.
//  DEPENDING ON THE FLAG, EITHER BACKBONE(BB), ALPHA CARBONS(CA), ALL ATOMS(ALL) or HEAVY ATOMS ARE STORED
//**********************************************************************************************
//int store_pdb(char *file,CHAIN *ch_all, CHAIN* ch_hv, CHAIN *ch_bb, CHAIN *ch_ca, int num_of_chains){

char** store_pdb(char *file,CHAIN *ch_all, CHAIN* ch_hv, CHAIN *ch_bb, CHAIN *ch_ca, int num_of_chains, int* num_resds){
  char buffer[MAX_LINE_LEN_CONS];
  FILE *fpr;
  int /**resd_arr, resdnum, */ resd_count = 1;
  int bb_count, ca_count, all_count, hv_count;
  char atom_name[6];  

  int i,all, hv, bb, ca,k,j,l;
  int ALL_CHUNK = 1000, BB_CHUNK = 500, HV_CHUNK=1000, CA_CHUNK=100;
  //int NUM_OF_RESDS = 200;
  int *num_atoms_in_each_resd,atom_count = 1;
  char **resd_arr, resd_string[7];
  
 fpr = fopen(file,"r");if(!fpr){printf("Error cannot open %s for reading\n",file);exit(-1);}

  /*resd_arr =(int*)malloc(1*sizeof(int));
  if(!resd_arr){printf("Cannot malloc for resd_arr..Exiting\n");exit(-1);}
  resd_arr[0] = -1000;*/

  resd_arr =(char**)malloc(1*sizeof(char*));
  if(!resd_arr){printf("Cannot malloc for resd_arr..Exiting\n");exit(-1);}
  resd_arr[0]=(char*)malloc(7*sizeof(char) );
  strcpy(resd_arr[0],"-1000");

  num_atoms_in_each_resd = (int*)malloc(1*sizeof(int));
  if(!num_atoms_in_each_resd){printf("Cannot malloc for num_atoms_in_each_resdr..Exiting\n");exit(-1);}
  num_atoms_in_each_resd[0] = 0;

  
  for(i=0; i<num_of_chains; i++){
    ch_all[i].pdb =  (PDB*)malloc(ALL_CHUNK*sizeof(PDB));   assert(ch_all[i].pdb != NULL);
    ch_hv[i].pdb =  (PDB*)malloc(HV_CHUNK*sizeof(PDB));     assert(ch_hv[i].pdb != NULL);
    ch_bb[i].pdb =  (PDB*)malloc(BB_CHUNK*sizeof(PDB));     assert(ch_bb[i].pdb != NULL);
    ch_ca[i].pdb =  (PDB*)malloc(CA_CHUNK*sizeof(PDB));     assert(ch_ca[i].pdb != NULL);
  }
  
  //printf("\n\n------------------------------------------------------------------------\n");
  //printf("Printing the pointer and the address values in store_pdb just after allocing\n");
  //for(i=0;i<num_of_chains;i++){
  //  printf("&(ch_all[%d].pdb) = %p and ch_all.pdb = %p  ",i,&(ch_all[i].pdb), ch_all[i].pdb ); 
  //  printf("&(ch_bb[%d].pdb)  = %p and ch_bb.pdb = %p\n",i,&(ch_bb[i].pdb), ch_bb[i].pdb ); 
  //  printf("&(ch_hv[%d].pdb)  = %p and ch_hv.pdb = %p  ",i,&(ch_hv[i].pdb), ch_hv[i].pdb ); 
  //  printf("&(ch_ca[%d].pdb)  = %p and ch_ca.pdb = %p\n",i,&(ch_ca[i].pdb), ch_ca[i].pdb ); 
  //  }
 

 i=0;
 all = hv = bb = ca = 0;
 all_count = bb_count = hv_count = ca_count = 0; 

  while(fgets(buffer, MAX_LINE_LEN_CONS, fpr)!= NULL){
    if(strncmp(buffer, "ATOM", 4) == 0){
      //printf("buffer = %s",buffer);
      //sscanf(&buffer[22],"%4d",&resdnum);
      sscanf(&buffer[11],"%5s",atom_name);
      sscanf(&buffer[22],"%7s",resd_string);

      //printf("atom name = %s",atom_name);

      //ch_all[i].pdb = (PDBLINE*)realloc( (PDBLINE*)ch_all[i].pdb,(all+1)*sizeof(PDBLINE)); 
      //if(is_Q_atom(atom_name)==0){
	all_count++;
	if(all_count == ALL_CHUNK){
	  //printf("all count = %d\n",all_count);
	  ch_all[i].pdb = (PDB*)realloc( (PDB*)ch_all[i].pdb,(all_count+ALL_CHUNK)*sizeof(PDB));
	  ALL_CHUNK = all_count+ALL_CHUNK; 
	}
	store_pdbline(buffer, &(ch_all[i].pdb[all]) );
	//printf("ALL = %s",buffer);
	all++;
	//}
      
      if(is_bb_atom(atom_name)==1){
	bb_count++;
	if(bb_count == BB_CHUNK){
	  //printf("bb count = %d\n",bb_count);
	  ch_bb[i].pdb = (PDB*)realloc( (PDB*)ch_bb[i].pdb,(bb_count+BB_CHUNK)*sizeof(PDB));
	  BB_CHUNK = bb_count+BB_CHUNK; 
	}
	//ch_bb[i].pdb = (PDBLINE*)realloc( (PDBLINE*)ch_bb[i].pdb,(bb+1)*sizeof(PDBLINE));
	store_pdbline(buffer, &(ch_bb[i].pdb[bb]) );
        //printf("BB = %s",buffer);
	bb++;
      }
      
      if(is_CA_atom(atom_name)==1){
	ca_count++;
	if(ca_count == CA_CHUNK){
	  //printf("ca count = %d\n",ca_count);
	  ch_ca[i].pdb = (PDB*)realloc( (PDB*)ch_ca[i].pdb,(ca_count+CA_CHUNK)*sizeof(PDB));
	  CA_CHUNK = ca_count+CA_CHUNK; 
	}
	//ch_ca[i].pdb = (PDBLINE*)realloc( (PDBLINE*)ch_ca[i].pdb,(ca+1)*sizeof(PDBLINE));
	store_pdbline(buffer, &(ch_ca[i].pdb[ca]) );
        //printf("CA = %s",buffer);
	ca++;
      }
      
      if(is_not_Hydrogen(atom_name)==1 ){
	hv_count++;
	if(hv_count == HV_CHUNK){
	  //printf("hv count = %d\n",hv_count);
	  ch_hv[i].pdb = (PDB*)realloc( (PDB*)ch_hv[i].pdb,(hv_count+HV_CHUNK)*sizeof(PDB));
	  HV_CHUNK = hv_count+HV_CHUNK; 
	}
	//ch_hv[i].pdb = (PDBLINE*)realloc( (PDBLINE*)ch_hv[i].pdb,(hv+1)*sizeof(PDBLINE));
	store_pdbline(buffer, &(ch_hv[i].pdb[hv]) );
	//printf("HV = %s",buffer);
	hv++;
      }
      
      
      /*if(resdnum != resd_arr[resd_count-1]){
	resd_arr = (int*)realloc( (int*)resd_arr,(resd_count+1)*sizeof(int) );
	if(!resd_arr){printf("Cannot realloc resd_arr..exiting\n");exit(-1);}
	resd_arr[resd_count] = resdnum;*/
	
      if(strcmp(resd_string,resd_arr[resd_count-1])!=0){
	resd_arr = (char**)realloc( (char**)resd_arr,(resd_count+1)*sizeof(char*) );
	if(!resd_arr){printf("Cannot realloc resd_arr..exiting\n");exit(-1);}
	resd_arr[resd_count]=(char*)malloc(7*sizeof(char) );
	strcpy(resd_arr[resd_count],resd_string);


	atom_count=0;
	num_atoms_in_each_resd = (int*)realloc( (int*)num_atoms_in_each_resd,(resd_count+1)*sizeof(int) );
	if(!num_atoms_in_each_resd){printf("Cannot realloc num_atoms_in_each_resd..exiting\n");exit(-1);}
        atom_count++;
	num_atoms_in_each_resd[resd_count] = atom_count;

	
	//printf("resdnum = %d and resd_arr[%d]=%d,num-atoms[%d]=%d\n",resdnum,resd_count,resd_arr[resd_count],resd_count,atom_count);
	resd_count++;
	//atom_count = 0;
      }else{
	
	atom_count++;
	num_atoms_in_each_resd[resd_count-1] = atom_count;
	//printf("num_atoms_eacresd[%d] = %d\n",resd_count-1,num_atoms_in_each_resd[resd_count-1]);
      }
    }
    
    if(strncmp(buffer,"TER",3)==0){
      ch_all[i].num_of_atoms = all;  
      ch_bb[i].num_of_atoms = bb; ch_bb[i].num_of_bbatoms = bb;
      ch_hv[i].num_of_atoms = hv; ch_hv[i].num_of_heavyatoms = hv;
      ch_ca[i].num_of_atoms = ca; ch_ca[i].num_of_cas = ca;
      
      ch_all[i].num_of_resds = resd_count-1;   ch_bb[i].num_of_resds = resd_count-1;
      ch_hv[i].num_of_resds = resd_count-1;   ch_ca[i].num_of_resds = resd_count-1;
      
      //printf("TER [%d], all=%d, hv=%d, bb=%d, ca=%d  \n",i,all,hv,bb,ca);
      //printf("all_cnt=%d  hv_cnt=%d  bb_cnt=%d  cacnt=%d\n",all_count, hv_count, bb_count, ca_count);
      
      i++;all=0;bb=0;ca=0;hv=0;
      all_count=0; bb_count =0; hv_count =0; ca_count = 0;

      //printf("resd_count = %d\n",resd_count);

      //resd_count=1; 
      //resd_arr[0] = -1000;
      //strcpy(resd_arr[0],"-1000");
      ALL_CHUNK = 1000, BB_CHUNK = 500, HV_CHUNK=1000, CA_CHUNK=100;
      
    }
  }//******** END while(fgets(buffer,MAX_LINE_LEN_CONS,fpr)... *****************
    
  for(i=0; i<num_of_chains; i++){
    ch_all[i].rsd = (RESD*)malloc(ch_all[i].num_of_resds*sizeof(RESD));
    
    k=1;
    for(j=0; j<ch_all[i].num_of_resds; j++){
      ch_all[i].rsd[j].num_of_atoms = num_atoms_in_each_resd[k++];
      ch_all[i].rsd[j].atom_line = (ATOM_LINE*)malloc(ch_all[i].rsd[j].num_of_atoms*sizeof(ATOM_LINE));
      if(!ch_all[i].rsd[j].atom_line){printf("Cannot malloc ch_all[i].rsd[j].atom_line.\n");exit(-1);}
      //k++;
    }
  }

  //for(j=0; j<ch_all[0].num_of_resds; j++){
  //printf("ch_all[0].rsd[%d].num_of_atoms = %d\n",j,ch_all[0].rsd[j].num_of_atoms);
  //}

  //exit(-1);
  l=0;
  for(i=0; i<num_of_chains; i++){
    for(j=0; j<ch_all[i].num_of_resds; j++){
      for(k=0; k<ch_all[i].rsd[j].num_of_atoms; k++){
	
	ch_all[i].rsd[j].atom_line[k].atom_name = (char*)malloc(8*sizeof(char));
	ch_all[i].rsd[j].atom_line[k].whole_line = (char*)malloc(MAX_LINE_LEN_CONS*sizeof(char));
	strcpy(ch_all[i].rsd[j].atom_line[k].atom_name, ch_all[i].pdb[l].pdb_atom_name);
	strcpy(ch_all[i].rsd[j].atom_line[k].whole_line, ch_all[i].pdb[l++].whole_line);
	//printf("[%d].atoms[%d]=%s ",j,k,ch_all[i].rsd[j].atom_line[k].atom_name);
      } 
      //printf("\n");
    }
  }
  

//for(j=0;j<ch_all[0].num_of_resds;j++){
// for(k=0;k<ch_all[0].rsd[j].num_of_atoms;k++){
//   printf("ch_all[0].rsd[%d].atoms[%d] = %s\n",j,k,ch_all[0].rsd[j].atoms[k]);
// }
//}
  
  
  fclose(fpr);
  //printf("resd_count = %d\n",resd_count);
  //for(i=0;i<resd_count;i++){
  // free(resd_arr[i]);
  //}
  //free(resd_arr);
  free(num_atoms_in_each_resd);
  //printf("------------------------------------------------------------------------\n");
  // printf("Printing the pointer and the address values in store_pdb just before exiting\n");
  //for(i=0;i<num_of_chains;i++){
  //printf("&(ch_all[%d].pdb) = %p and ch_all.pdb = %p  ",i,&(ch_all[i].pdb), ch_all[i].pdb ); 
  //printf("&(ch_bb[%d].pdb)  = %p and ch_bb.pdb = %p\n",i,&(ch_bb[i].pdb), ch_bb[i].pdb ); 
  //printf("&(ch_hv[%d].pdb)  = %p and ch_hv.pdb = %p  ",i,&(ch_hv[i].pdb), ch_hv[i].pdb ); 
  //printf("&(ch_ca[%d].pdb)  = %p and ch_ca.pdb = %p\n\n",i,&(ch_ca[i].pdb), ch_ca[i].pdb ); 
  //}
  //printf("exiting store pdb..\n");
  
  //return(ch_all[0].num_of_resds);
  *num_resds = ch_all[0].num_of_resds;
  return(resd_arr);

}//END store_pdb.. 
//**************************************************************************************************




//********************************
void print_chain(CHAIN* ch){
  int i;
  for(i=0; i<ch->num_of_atoms; i++){
    printf("%s",ch->pdb[i].whole_line);
  }
}//********************************

 




//***************************************************************************************************
void create_angle_constraint_file(char* input, char* ac_file, int num_resds){

  char *call_phipsi, *phipsi_output;
  PHIPSI *phipsi;
  int i = 0;
  FILE *fpr, *fpw;
  char buffer[MAX_LINE_LEN_CONS];
  float phi_float[num_resds], psi_float[num_resds], omega_float[num_resds], chi_float[num_resds];

  float THREE_SIXTY = 360.00;
  float ONE_EIGHTY = 180.00;
  float ZERO = 0.00;


  phipsi_output = (char*)malloc( (strlen(input)+strlen(".phipsi")+1)*sizeof(char) );
  assert(phipsi_output != NULL);
  sprintf(phipsi_output,"%s.phipsi",input);
  //printf("phipsioutput = %s\n",phipsi_output);

  // Call phipsi to create the .phipsi file
  //-----------------------------------------------------------
  call_phipsi = (char*)malloc( (strlen("/usr/local/bin/phipsi -f ")+strlen(input)+strlen(" -o ")+strlen(phipsi_output)+1)*sizeof(char) );
  if(!call_phipsi){printf("Cannot open phipsi file for writing to.\n");exit(-1);}
  sprintf(call_phipsi, "/usr/local/bin/phipsi -f %s -o %s",input,phipsi_output);
  //printf("call_phipsi = %s\n",call_phipsi);
  if (system(call_phipsi) != 0) {
	fprintf(stderr, "ERROR: Call to phipsi failed.\n");
	exit(-1);
  }
  free(call_phipsi);
  //------------------------------------------------------------
  
  
  //Store the phipsi values in a phipsi struct
  //---------------------------------------------------------------------
  phipsi = (PHIPSI*)malloc(num_resds*sizeof(PHIPSI) );
  if(!phipsi){printf("Cannot malloc phipsi.\n");exit(-1);}
  
  fpr = fopen(phipsi_output,"r");
  if(!fpr){printf("Cannot open %s for reading\n",phipsi_output);exit(-1);}
  
  while(fgets(buffer, MAX_LINE_LEN_CONS, fpr)!= NULL){
    if(is_phipsi_line(buffer) == TRUE ){
      //printf("pp line = %s",buffer);
      store_phipsi_line(buffer,&phipsi[i++]);
    }
  }

  fclose(fpr);
  //----------------------------------------------------------------------

  //Create the angle constraint file
  //resd_num is stored as an int.
  // the angles are stored as strings. Hence need to convert the chars
  //  to float and then modify them.. 
  //-----------------------------------------------------------
  //printf("acfile = %s\n",ac_file);
  fpw = fopen(ac_file,"w");
  if(!fpw){printf("Cannot open %s for writing to.\n",ac_file);exit(-1);}

  for(i=0; i<num_resds; i++){
    phi_float[i] = atof(phipsi[i].phi);
    psi_float[i] = atof(phipsi[i].psi);
    omega_float[i] = atof(phipsi[i].omega);
   
    if(strcmp(phipsi[i].chi,"NA")==0){
      //printf("chi = %s\n",phipsi[i].chi);
      chi_float[i] = ONE_EIGHTY;
    }else{
      chi_float[i] = atof(phipsi[i].chi);
    }
    
    if(phi_float[i] == THREE_SIXTY || phi_float[i] == ZERO){
      phi_float[i] = ZERO;   
      fprintf(fpw, AC_WRITE_FORMAT, phipsi[i].resd_name, phipsi[i].resd_num, phi_float[i], phi_float[i], 
	      psi_float[i] - ANGLE_DIFF, psi_float[i] + ANGLE_DIFF, omega_float[i], omega_float[i], chi_float[i], chi_float[i]); 
      //printf("1111\n");
    }
    else if(psi_float[i] == THREE_SIXTY || psi_float[i] == ZERO){
      psi_float[i] = ZERO;
      fprintf(fpw, AC_WRITE_FORMAT, phipsi[i].resd_name, phipsi[i].resd_num, phi_float[i]- ANGLE_DIFF, phi_float[i]+ANGLE_DIFF, 
	      psi_float[i], psi_float[i], omega_float[i], omega_float[i], chi_float[i], chi_float[i]); 
      //printf("22222\n");   
    }else{
      fprintf(fpw, AC_WRITE_FORMAT, phipsi[i].resd_name, phipsi[i].resd_num, phi_float[i]- ANGLE_DIFF, phi_float[i]+ANGLE_DIFF, 
	      psi_float[i]-ANGLE_DIFF, psi_float[i]+ANGLE_DIFF, omega_float[i], omega_float[i], chi_float[i], chi_float[i]); 
      //printf("33333\n"); 
    }
 
 }

  fflush(fpw);fclose(fpw);
  //-----------------------------------------------------------

  free(phipsi_output);
  free(phipsi);

}// END create_angle... 
//**********************************************************************************************************





//**********************************************************************************************************
// Create distances between one CA atom and all other (n-1)  CA atoms ..
void create_distance_constraint_file(CHAIN *ch, char *dc_file, int num_resds){

  int i,j,sec_last_atom,last_atom;
  float distance;
  FILE *fpw;

  fpw = fopen(dc_file,"w");
  if(!fpw){printf("Cannot open %s for writing to.\n",dc_file);exit(-1);}
  
  /*for(i=0; i<ch->num_of_atoms-1; i++){  
    //if(is_CA_atom(ch->pdb[i].pdb_atom_name)==1)
    printf("%s",ch->pdb[i].whole_line);
    //printf("%s\n",&ch->pdb[i].pdb_atom_name[0]);
    }*/

  //exit(-1);


  //printf("num of resds = %d\n", ch->num_of_resds);
  //for(i=0; i<ch->num_of_resds; i++){
  //for(j=1; j<ch->num_of_resds-1; j++){
  sec_last_atom = ch->num_of_atoms-1;
  last_atom = ch->num_of_atoms;

  for(i=0; i<sec_last_atom; i++){  
    for(j=i+1; j<last_atom; j++){

      /*if( (is_N_atom(ch->pdb[i].pdb_atom_name)==1) &&(is_CA_atom(ch->pdb[j].pdb_atom_name)==1) ){
	distance = get_distance(&ch->pdb[i].pt,&ch->pdb[j].pt);
	fprintf(fpw,"N %5s  CA %5s %10.3f %10.3f\n",ch->pdb[i].pdb_resdno,ch->pdb[j].pdb_resdno, distance,distance);
      }
      if( (is_N_atom(ch->pdb[i].pdb_atom_name)==1) &&(is_C_atom(ch->pdb[j].pdb_atom_name)==1) ){
	distance = get_distance(&ch->pdb[i].pt,&ch->pdb[j].pt);
	fprintf(fpw,"N %5s  C %5s %10.3f %10.3f\n",ch->pdb[i].pdb_resdno,ch->pdb[j].pdb_resdno, distance,distance);
      }
      if( (is_CA_atom(ch->pdb[i].pdb_atom_name)==1) &&(is_C_atom(ch->pdb[j].pdb_atom_name)==1) ){
	distance = get_distance(&ch->pdb[i].pt,&ch->pdb[j].pt);
	fprintf(fpw,"CA %5s  C %5s %10.3f %10.3f\n",ch->pdb[i].pdb_resdno,ch->pdb[j].pdb_resdno, distance,distance);
	}*/
      if( (is_CA_atom(ch->pdb[i].pdb_atom_name)==1) &&(is_CA_atom(ch->pdb[j].pdb_atom_name)==1) ){
	distance = get_distance(&ch->pdb[i].pt,&ch->pdb[j].pt);
	//printf("[%s].CA -- [%s].CA  distance = %4.3f\n",ch->pdb[i].pdb_resdno, ch->pdb[j].pdb_resdno, distance);
	fprintf(fpw,"CA %5s  CA %5s %10.3f %10.3f\n",ch->pdb[i].pdb_resdno,ch->pdb[j].pdb_resdno, distance,distance);
      }
    }
  }
  //printf("[%d].numatoms = %d\n",j,ch->rsd[j].num_of_atoms);

  fflush(fpw);fclose(fpw);

}// END create_distance_constraint_files
//************************************************************************************************************




//*************************************************************
float get_distance(POINT *pt1, POINT *pt2){

  float dist;
  dist = sqrt( pow((pt1->x - pt2->x),2.00) + pow((pt1->y - pt2->y),2.00) + pow((pt1->z - pt2->z),2.00) );
  return(dist);

}  //************************************************


//*********************************************************************************
void store_phipsi_line(char *line, PHIPSI *pp){
  
  //printf("line = %s",line);
  
  sscanf(line, PHIPSI_LINE_READ_FORMAT, &pp->resd_num, pp->resd_name, pp->phi, pp->psi, pp->omega, pp->chi);
  //printf("resd_num = %d, resd_name= %s, phi=%s, psi=%s, omega=%s, chi=%s\n",pp->resd_num, pp->resd_name,pp->phi,pp->psi,pp->omega,pp->chi); 
}//********************************************************************************

