/**********************************************************
 * Program     : preditor_parser	 
 * Purpose     : Parses the preditor output and stores the angles in a linked
 *               list 
 * Author      : Rajarshi Maiti
 * Contact     : rajarshi@cs.ualberta.ca
 * Created     : November 1, 2006
 * Compile     : make all 	 
 * Modified    :	
 * Modified by :	
 ***********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>
 
#include "preditor_parser.h"
#include "util.h"
//#include "memwatch.h"


/**********************************************************
 * Prints a single node
 **********************************************************/
 /*
void print_node(ANGLE_INFO *node){
  if(node != NULL){
    printf("num=%d,name=%c,phi=%4.3f, psi=%4.3f, omega=%4.3f, chi=%4.3f\n",
	   node->res_no, node->res, node->phi, node->psi, node->omega, 
	   node->chi);
  }
  
}
*/
//END print_node *****************************************



/***********************************************************
 * Prints a linked list
 ***********************************************************/
 /*
void print_list(ANGLE_INFO* head){
  ANGLE_INFO *curr = head;

  while(curr != NULL){
    print_node(curr);
    curr = curr->next;
  }
}
*/
//END print_list ******************************************




/***********************************************************
 * Deletes a linked list
 ************************************************************/
 /*
void free_list(ANGLE_INFO *head){
  ANGLE_INFO *curr, *prev;
  curr = prev = head;
  
  while(curr != NULL){
    curr = curr->next; 
    free(prev);
    prev = curr;
  }
  head = curr;
}
*/
//END free_list *******************************************





/***********************************************************
 * Allocate space for a node and returns the  node
 ***********************************************************/
 /*
ANGLE_INFO* create_node(int res_no, char res, float phi, float psi, float omega, float chi){

  ANGLE_INFO *new = NULL;  
  
  new = malloc(sizeof(ANGLE_INFO));
  if(!new){
    return(NULL);
  }else{
    new->next = NULL; 
    new->res_no = res_no;
    new->res = res;
    new->phi = phi;
    new->psi = psi;
    new->omega = omega;
    new->chi = chi;
    return(new);
  }
}
*/
//END create_node *****************************************





/***********************************************************
 * Parses an output file of predicted torsion angles from
 * Preditor and stores the torsion angles in an array of
 * ANGLE_INFO structs.
 *
 * This function also determines and returns the number of
 * residues represented in the Preditor file and allocates
 * an array of the ANGLE_INFO structs at the location
 * pointed to by angles_param.
 ************************************************************/
int parse_preditor_output(char *infile, ANGLE_INFO **angles_param){
	int num_resds = 0;
	int i;
	int start = FALSE; // Whether or not we are at the start of the list of residues in the Preditor output file.
	//int res_no;
	FILE *fpr;
	ANGLE_INFO *angles;

	char buffer[MAX_LINE_LEN];
	char phi_string[12], psi_string[12], chi_string[12];

	//float phi, psi, omega, chi; 
	//char res;
	//ANGLE_INFO *curr, *new_node;
	//curr = new_node = NULL;  

	fpr = fopen(infile, "r");
	if(!fpr){
		printf(" Cannot open %s for reading.\n", infile);
		exit(-1);
	}

	// Count number of residues in file
	while(fgets(buffer, MAX_LINE_LEN, fpr)!= NULL){
		
		if(strncmp(buffer, "Res#", 4)==0){
			start = TRUE;
		}
		else if (start) {
			if (isdigit(buffer[0])) {
				num_resds++;
			}
		}
	}
	
	rewind(fpr);
	start = FALSE;
	
	*angles_param = (ANGLE_INFO *) malloc(num_resds * sizeof(ANGLE_INFO));
	angles = *angles_param;
	
	i = 0;
	while(fgets(buffer, MAX_LINE_LEN, fpr)!= NULL){  

		if(strncmp(buffer, "Res#", 4)==0){
			start = TRUE;
		}
		else if (start) {

			if (sscanf(buffer, "%d %c %12s %lf %12s %lf %12s %lf %lf", &angles[i].res_no, &angles[i].res,
				phi_string, &angles[i].phi_err,
				psi_string, &angles[i].psi_err,
				chi_string, &angles[i].chi_confidence,
				&angles[i].omega) == 9) {
				
				angles[i].phi = atof(phi_string);
				angles[i].psi = atof(psi_string);
				angles[i].chi = atof(chi_string);
				
				if (angles[i].phi == -666.0 || angles[i].phi == 360.0) {
					angles[i].phi = PREDITOR_ANGLE_NA;
				}
				
				if (angles[i].psi == -666.0 || angles[i].psi == 360.0) {
					angles[i].psi = PREDITOR_ANGLE_NA;
				}
				
				if (angles[i].chi == -666.0 || angles[i].chi == 360.0) {
					angles[i].chi = PREDITOR_ANGLE_NA;
				}
				
				i++;
			}
			
				
			/*
			  if(num_resds == 0){
			*angles = create_node(res_no, res, phi, psi, omega, chi);
			curr = *angles;
			  }else{
			new_node = create_node(res_no, res, phi, psi, omega, chi);
			curr->next = new_node;
			curr = new_node;
			curr->next = new_node->next; // Is this line needed?
			  }
			  */
			  
			
		}
	}
	fclose(fpr);
	
	if (i != num_resds) {
		fprintf(stderr, "ERROR parsing Preditor output file.\n");
		exit(1);
	}
	
	return(num_resds);
}//END parse_preditor_output ********************************





/***********************************************************
 * Allocate space for an error node and returns the  node
 ***********************************************************/
ERR_NODE* create_err_node(int res_no, char res, float phi_err, float psi_err, float omega_err){

  ERR_NODE *new = NULL;  
  
  new = malloc(sizeof(ERR_NODE));
  if(!new){
    return(NULL);
  }else{
    new->next = NULL; 
    new->res_no = res_no;
    new->res = res;
    new->phi_err = phi_err;
    new->psi_err = psi_err;
    new->omega_err = omega_err;
    return(new);
  }
}//END create_node *****************************************





/************************************************************
 * Parses an error file and stores the error for the phi psi 
 * and omega angles in a linked list.
 *
 *
 *
 ************************************************************/
int parse_error_file(char *infile, ERR_NODE **head){
  int num_resds = 0;
  int start = FALSE;
  int res_no;
  FILE *fpr;
  
  char buffer[MAX_LINE_LEN];
  //char phi_string[12], psi_string[12], omega_string[12];

  float phi_err, psi_err, omega_err; 
  char res;
  ERR_NODE *curr, *new_node;
  curr = new_node = NULL;  
  fpr = fopen(infile, "r");
  if(!fpr){
    printf(" Cannot open %s for reading.\n", infile);
    exit(-1);
  }
  
  while(fgets(buffer, MAX_LINE_LEN, fpr)!= NULL){  
    
    if(strncmp(buffer, "#AA_No", 6)==0){
      start = TRUE;
    }
    if(start == TRUE && strncmp(buffer, "#AA_No", 6) != 0){
      
      sscanf(buffer, "%d %c %f %f %f", &res_no, &res, &phi_err, &psi_err, &omega_err);
      //printf("phi_err = %f, psi_err = %f, and omega_err = %f\n",phi_err, psi_err, omega_err);
      
      if(num_resds == 0){
	*head = create_err_node(res_no, res, phi_err, psi_err, omega_err);
	curr = *head;
      }else{
	new_node = create_err_node(res_no, res, phi_err, psi_err, omega_err);
	curr->next = new_node;
	curr = new_node;
	curr->next = new_node->next;
      }
      num_resds++;
    }
  }
  fclose(fpr);
  return(num_resds);
  
}//END parse_error_file *************************************





//***********************************************************
/*int main(int argc, char *argv[]){

  int num_resds = 0;

  ANGLE_INFO *head = NULL;
  num_resds = parse_preditor_output(argv[1], &head); 
  //print_list(head);
  //printf("Number of residues in preditor_output = %d\n", num_resds);
  free_list(head);
  
  
  return(0);
  }*///END main *************************************************

