#ifndef __RAMA_H__
#define __RAMA_H__

#include "pepmake.h"
#include "util.h"

#define RAMA_LEN 36
#define ANGLE_TYPE_PHI 0
#define ANGLE_TYPE_PSI 1
#define ANGLE_TYPE_OMEGA 2
#define ANGLE_SNAP_MARGIN 1.0
	// When we set an angle to the value at a boundary (e.g. change phi from 142.42 to 170 because
	// 170 degrees in the closest acceptable angle), set the angle to ANGLE_SNAP_MARGIN degrees within the
	// boundary so that rounding error will not put the angle back outside of the boundary. So, e.g.
	// if ANGLE_SNAP_MARGIN == 1.0 then we set phi to 171.0.
	//
	// Different values of ANGLE_SNAP_MARGIN were tested emprically on a set of proteins. A value of 1.0
	// was found to be a good compromise between improving Rama and Omega scores while minimizing damage to
	// the structure. As ANGLE_SNAP_MARGIN goes from 0 to 1.0, Rama and Omega scores improve significantly,
	// whereas for higher ANGLE_SNAP_MARGIN Rama and Omega scores improve only slightly and sometimes get
	// worse. As ANGLE_SNAP_MARGIN increases, there is a general tendency for the other GA Potential scores
	// to get worse.
#define RAD(x)  x*PI/180

extern int ramaTableGeneral[RAMA_LEN][RAMA_LEN];
extern int ramaTableGlycine[RAMA_LEN][RAMA_LEN];
extern int ramaTableProline[RAMA_LEN][RAMA_LEN];
extern int ramaTablePreProline[RAMA_LEN][RAMA_LEN];


void read_rama_tables();
int getPhiIndex(double phi);
int getPsiIndex(double psi);
int lookupRamaScore(ResData *residues, int resIndex, int numRes);
int lookupRamaScoreByIndices(ResData *residues, int resIndex, int numRes, int phiindex, int psiindex);
int lookupRamaScoreAbstract(char *resName, double phi, double psi, int isPrePro);
int lookupRamaScoreAbstractByIndices(char *resName, int phiindex, int psiindex, int isPrePro);
double angleDist(double a1, double a2);
double getClosestAllowedAngle(ResData *residues, int resIndex, int numRes, double desiredAngle, int angleType,
	double original_phi, double original_psi, int maxRamaQualityDecrease);
void findClosestAllowedPhiPsiPair(ResData *residues, int resIndex, int numRes, int min_accpetable_rama_score,
	double *phi_ptr, double *psi_ptr);
double getRandomAngleInInterval(double angle_min, double angle_max);

#endif
