
/*
	file: align.c
	Basic routines for doing alignment algorithm
	These are pretty complex algorithms IMHO
*/

#include "csipdb.h"
#include "misc.h"

void sec_struct_scores();
void get_path_start(), fill_score_mat(), s1_sum_score_mat();
int get_align_score(), s1_save_path(), get_align_strings();

/*-----------------------------------------------------
 * Calculates penalty scores for secondary structure.
 *-----------------------------------------------------
 */
void sec_struct_scores(char *sStruct, int len, int *sScores)
/* *sStruct secondary structure array, len length of sequence,
	*sScores penalty scores returned */
{
  int	i;

  for (i=0; i<len; i++)
    {
      sScores[i] = 0;
      if ((sStruct[i] == CH_HELX) || (sStruct[i] == CH_BETA))
        {
          if ((sStruct[i-1] != CH_COIL) && (sStruct[i+1] != CH_COIL))
            sScores[i] += penMiddle;
          else
            sScores[i] += penEnd;
        }
      else
        sScores[i] += penCoil;
    }

} /* sec_struct_scores */



/*--------------------------------------------------------
 * Determine initial scoring matrix for Needleman-Wunsch
 * algorithm (alignment between 2 sequences).
 *--------------------------------------------------------
 */
void fill_score_mat(wtMatrix, sm, qLen, dLen, qArr, dArr)
int	*wtMatrix;			/* weighting matrix */
int 	*sm;					/* scoring matrix */
int 	qLen, dLen;			/* lengths of both sequences */
int	*qArr, *dArr;		/* amino acid indices */
{
  int	m, k;
  int	*matRow = NULL, *dix = NULL;

  for (k=0; k<qLen; k++)
    {		/* calc each row in sm */
      m = dLen;						/* last entry */
      dix = dArr;						/* column index */

      /* What this means? */
      matRow = wtMatrix + qArr[k];	/* row index for wtMat */

      do
        {
          *sm = matRow[*dix++];		/* calc each entry in matrix row */
          sm++;
        }
      while (--m);
    }

} /* fill_score_mat */


/*----------------------------------------------------------------------
 * Finds summation of Needleman-Wunsch matrix given a penalty
 * for having a gap, size of that gap and gaps in secondary structure. 
 * "s1" means secondary structure known for database protein.
 *----------------------------------------------------------------------
 */
void s1_sum_score_mat(int *sm, int qLen, int dLen, int *dsScores, int *columns, int *gapPen)
{
  int	*colMaxPtr;
  int	diagVal, rowMax, colMax;
  int	x1, m, qssGapPen, *p1, *p2;
  int	k;
  int	queryPenGap;

  for (m=0; m<dLen; m++)
    columns[m] = -100000;			/* cannot use MIN_INT */

  for (m=dLen-1; m>0; m--)
    gapPen[m] = MIN(dsScores[m-1], dsScores[m]);

  colMaxPtr = &columns[dLen-1];
  p1 = &sm[(qLen*dLen)-1];
  p2 = p1 - dLen - 1;

  /* critical part of algorithm, coded as fast as possible */
  for (k=qLen-1; k>0; k--)
    {			/* summation of each row */
      m = dLen-1;						/* process dLen-1 columns */
      rowMax = -100000;
      queryPenGap = 0;
      do
        {							/* compute each matrix entry */
          diagVal = *p1--;
          colMax = *colMaxPtr;

          /* determine new matrix entry (p2) */
          if ((diagVal >= rowMax) && (diagVal >= colMax))
            *p2 += diagVal;
          else if (rowMax >= colMax)
            *p2 += rowMax;
          else
            *p2 += colMax;
          p2--;

          /* find maximum value in current row, adjust for penalties */
          qssGapPen = dsScores[m-1];
          x1 = diagVal - (qssGapPen + gapPenalty);
          rowMax -= MAX(qssGapPen-queryPenGap, 0);
          if (x1 >= rowMax)
            {
              rowMax = x1 - gapSizePenalty;
              queryPenGap = qssGapPen;
            }
          else
            {
              rowMax -= gapSizePenalty;
              queryPenGap = MAX(queryPenGap, qssGapPen);
            }

          /* find maximum value in current column, adjust for penalties */
          x1 = diagVal - (gapPen[m] + gapPenalty);
          if (x1 >= colMax)
            *colMaxPtr = x1 - gapSizePenalty;
          else
            *colMaxPtr = colMax - gapSizePenalty;
          colMaxPtr--;

        }
      while (--m);					/* while m != 0 */
      p1--;
      p2--;
      colMaxPtr += dLen-1;
    }

} /* s1_sum_score_mat */


/*************************************************************/
/* Return maximum score for Needleman-Wunsch scoring matrix
*/
int get_align_score(sm, qLen, dLen)
register int	*sm;
register int	qLen, dLen;
{
  register int 	j, smMax;
  register int 	*p1;

  smMax = 0;

  /* find maximum value of first row*/
  p1 = &sm[0];
  for (j=0; j<dLen; j++)
    {
      if (*p1 > smMax)
        {
          smMax = *p1;
        }
      p1++;
    }

  /* check to see if there is larger value in the first column */
  p1 = &sm[dLen];
  for (j=1; j<qLen; j++)
    {
      if (*p1 > smMax)
        {
          smMax = *p1;
        }
      p1 += dLen;
    }

  return(smMax);

}	/* get_align_score */


/*************************************************************/
/* Traverse the Needleman-Wunsch scoring matrix starting at the
** highest score. Save the indices of the traversal for these
** indicate the optimal way to align the sequences.
** Note that secondary structure is known for one of the sequences.
*/
int s1_save_path(sm, qLen, dLen, icArr, irArr, dsScores)
int	*sm;
register int qLen, dLen;
int	*icArr, *irArr;
int	*dsScores;
{
  register int 	k, maxVal, testVal;
  register int 	*tPtr;
  register int 	penalty, queryPenGap;
  int 			ir, ic, tmpir, tmpic, num;
  int 			*diagPtr, *maxPtr;
  int				colGapPen;

  get_path_start(sm, qLen, dLen, icArr, irArr);
  ic = icArr[1];			/* index of first match */
  ir = irArr[1];
  num = 1;
  /* follow path of highest scores in matrix */
  diagPtr = &sm[(ir)*dLen+(ic)];
  while ((ir < qLen) && (ic < dLen))
    {
      maxVal = *diagPtr;
      maxPtr = diagPtr;
      tmpir = ir+1;
      tmpic = ic+1;
      /* check active column for highest value */
      tPtr = diagPtr;
      colGapPen = MIN(dsScores[ic], dsScores[ic-1]);
      penalty = gapPenalty + gapSizePenalty + colGapPen;
      for (k=ir+2; k<=qLen; k++)
        {
          tPtr += dLen;
          testVal = *tPtr - penalty;
          if (testVal > maxVal)
            {
              tmpir = k;
              tmpic = ic+1;
              maxVal = testVal;
              maxPtr = tPtr;
            }
          penalty += gapSizePenalty;
        }
      /* check active row for highest value */
      tPtr = diagPtr;
      queryPenGap = dsScores[ic];
      penalty = gapPenalty + gapSizePenalty + queryPenGap;
      for (k=ic+2; k<=dLen; k++)
        {
          tPtr++;
          testVal = *tPtr - penalty;
          if (testVal > maxVal)
            {
              tmpir = ir+1;
              tmpic = k;
              maxVal = testVal;
              maxPtr = tPtr;
            }
          penalty += gapSizePenalty + MAX(dsScores[k-1] - queryPenGap, 0);
          if (dsScores[k-1] > queryPenGap)
            queryPenGap = dsScores[k-1];
        }
      /* if new element not diagonal, we have a gap */
      ir = tmpir;
      ic = tmpic;
      num++;
      irArr[num] = ir;
      icArr[num] = ic;
      diagPtr = maxPtr + dLen + 1;
    }

  return(num);

}	/* s1_save_path */


/*************************************************************/
/* Returns indices of best align score.
*/
void get_path_start(register int *sm, register int qLen, int dLen, int *icArr, int *irArr)
{
  register int j, smMax;
  register int *p1;

  irArr[0] = 0;
  icArr[0] = 0;

  /* find maximum value of first row*/
  smMax = 0;
  p1 = &sm[0];
  for (j=0; j<dLen; j++)
    {
      if (*p1 > smMax)
        {
          smMax = *p1;
          irArr[1] = 1;
          icArr[1] = j+1;
        }
      p1++;
    }
  /* check to see if there is larger value in the first column */
  p1 = &sm[dLen];
  for (j=1; j<qLen; j++)
    {
      if (*p1 > smMax)
        {
          smMax = *p1;
          irArr[1] = j+1;
          icArr[1] = 1;
        }
      p1 += dLen;
    }

}	/* get_path_start */


/*************************************************************/
/* Take the indices from "save_path" and create the desired
** character sequence string (ie, with blanks for gaps included).
*/
int get_align_strings(int indNum, int *indir, int *indic, SEQ_REC *mSeq, SB_REC	*dSeq, char *mStr, char *dStr,  char *sStr )
{
  register int	k, m;
  register int	cur, prev;
  int				len1, len2;

  len1 = mSeq->len;
  len2 = dSeq->len;
  for (k=1; k<=indNum; k++)
  {
    prev = indic[k-1]+1;
    cur  = indic[k];
    if (cur != prev)
      for (m=prev; m<=cur-1; m++)
      {
        *dStr++ = dSeq->pRes[m-1];		/* changed back */
        *sStr++ = dSeq->sRes[m-1];
        *mStr++ = ' ';
        len1++;
        if (len1 == MAX_ALIGN_LEN)
          return(len1);
      }
    prev = indir[k-1]+1;
    cur  = indir[k];
    if (cur != prev)
      for (m=prev; m<=cur-1; m++)
        {
          *mStr++ = mSeq->pRes[m-1];	/* changed back */
          *dStr++ = ' ';
          *sStr++ = ' ';
          len2++;
          if (len2 == MAX_ALIGN_LEN)
            return(len2);
        }
    *mStr++ = mSeq->pRes[indir[k]-1];	/* changed back */
    *dStr++ = dSeq->pRes[indic[k]-1];
    *sStr++ = dSeq->sRes[indic[k]-1];
  }

  for (k=indir[indNum]+1; k<=mSeq->len; k++)
  {
    *mStr++ = mSeq->pRes[k-1];/* changed back */
    *dStr++ = ' ';
    *sStr++ = ' ';
    len2++;
  }
  for (k=indic[indNum]+1; k<=dSeq->len; k++)
  {
    *dStr++ = dSeq->pRes[k-1];			/* changed back */
    *sStr++ = dSeq->sRes[k-1];
    *mStr++ = ' ';
    len1++;
  }

  *dStr = '\0';
  *mStr = '\0';
  *sStr = '\0';

  return(MAX(len1, len2));

}	/* get_align_strings */
