/*---------------------------------------------------------
 *  This file contains routines required to perform csipdb 
 *  alignments with seqbank.
 *---------------------------------------------------------
 */

#include <stdlib.h>
#include <stdio.h>
#include "csipdb.h"
#include "loop.h"
#include "btimem.h"
#include "misc.h"

ALIGN_REC  *topAligns = NULL;
int      startSeed = 0;

static int    *mSeqNums, *dSeqNums, *jSeqNums;
static int    *sm, *dsScore;
static int    *dColumns, *dGapPen;
static int    protCount, alignNum;
static double  topScoreGroup;

void init_sb_align(), clean_up(), proc_seqs();
void eval_sb_align(), save_sb_align(), sort_sb_align();
double jumble_test(), calc_test_stat();

void sec_struct_scores(), fill_score_mat(), s1_sum_score_mat();
void print_aligns();

void mask(char *pRes, char *sRes);
void show_msg(),  show_err_mem();
int check_ident(),read_seq_sb(), csthread(),jumble_array();
void print_stats(),chars_to_nums();
void do_side_chain_stuff();
extern char* emalloc(), ecalloc();

// in align.c
extern int init_seqs(), get_align_score(), s1_save_path(), get_align_strings();

int string_match_num();
/**************************************************************************\
| Precondition: Don't know if there are aa sequence in database that match |
|               with query.                                                |
| Postcondition: Found out whethere there are aa sequence in database that |
|                match with query.                                         |
| Purpose: To match query aa sequence with database.                       |
| Return Value: 1 - a match was found                                      |
|               0 - couldn't find a match                                  |
\**************************************************************************/

int aa_sequence (PROGRAM_PARAMS params)
{
  int    rc,  check;
  SB_REC  dbSeq;
  char    sVal[MAX_LINE_LEN];

  init_sb_align(&mySeq);

  /**** begin to search database *******************/
  sprintf(sVal, "\n Reading shifts database.\n");
  show_msg(sVal);
  dbSeq.pRes = bti_malloc (1);
  if (!dbSeq.pRes)
    {
      printf ("Not enough memmory to allocate to dbSeq.pRes\n");
      exit(-1);
    }

  dbSeq.sRes = bti_malloc (1);
  if (!dbSeq.sRes)
    {
      printf ("Not enough memmory to allocate to dbSeq.sRes\n");
      exit(-1);
    }
  dbSeq.aaRes = bti_malloc (1);
  if (!dbSeq.aaRes)
    {
      printf ("Not enough memmory to allocate to dbSeq.sRes\n");
      exit(-1);
    }



  while (TRUE)
    {

      rc = read_seq_sb(sbfp, &dbSeq, 0);
      if (rc == EOF)
        break;
      if ( rc == 0 )
        continue;
      if ( strlen(params.sequenceName) > 0 ) 
      {
        if ( strncmp(params.sequenceName, dbSeq.title, 4) ==  0 )
        {
          // found the sequence

          // just save this one
          proc_seqs(&mySeq, &dbSeq);

          printf("Found sequence %s in seqsee.db\n", params.sequenceName );
          fflush(stdout);
          break;
        }
      }
      else
        proc_seqs(&mySeq, &dbSeq);
    }

  free (dbSeq.sRes);
  free (dbSeq.pRes);
  free (dbSeq.aaRes);
  fclose (sbfp);

  if ( strlen(params.sequenceName) > 0 )
  {
      // we had input a template sequence, but didn't find it in the seqSee DB file
      if ( topAligns[0].dpRes == NULL )
      {
        printf("Did not find sequence %s in database.  Exiting...\n", params.sequenceName);
        exit(-1);
      }
  }
  /*****end of Searching database *******/

  check=check_ident(&topAligns[0]);

  if (check == 1)
    {
#ifdef DEBUG
      print_stats(stdout, protCount, topScoreGroup);
#endif

      print_aligns(protCount, alignNum);
#ifdef DEBUG

      printf("The SEQUENCE is: %s\n", mySeq.pRes);
      printf("The accession number is: %s\n", mySeq.id);
#endif

#ifdef STOP_AFTER_ALIGNMENTS
	exit(0);
#endif
	printf("Stopping (special homodeller build)");
	exit(0);

      csthread (mySeq.pRes,1, params.phipsiFilename, params.pdbtmp, params.pdbdir);
    }

  else
    {
      return (0);
    }

  /* FIL LOOK! This is the loop/sidechain stuff */
  /* CASE: Greater than 35% homology. eg. calm */
  do_gap_findin_and_fillin(topAligns, 1, params);

  do_side_chain_stuff( 1, params );

  /* END loop/sidechain stuff */

  //GA stuff

  //do_GA_stuff(pdbtmp);
  return (1);
}

/*------------------------------
 * initialize data structures. 
 *------------------------------
 */
void init_sb_align(SEQ_REC *mSeq)
{
  int  i, size;

  startSeed = seed;
  protCount = 0;

  alignNum = 0;
  topScoreGroup = (double)MIN_INT;

  /* init structure containing list of best alignments */

  size = NUMBER_OF_ALIGNMENTS * sizeof(ALIGN_REC);
  topAligns = (ALIGN_REC*)bti_malloc(size);
  for (i=0; i<NUMBER_OF_ALIGNMENTS; i++)
    {
      topAligns[i].len = 0;
      topAligns[i].oScor = MIN_INT;
      topAligns[i].ident = MIN_INT;
      topAligns[i].query_offset = 0;
      topAligns[i].perid = 0.0;
      topAligns[i].zVal = (double)MIN_INT;
      topAligns[i].qpRes = NULL;
      topAligns[i].dpRes = NULL;
      topAligns[i].dsRes = NULL;
      topAligns[i].gapCount = 0;
      memset(topAligns[i].gaps, 0, sizeof(GAP_INFO_REC) * MAX_NUM_GAPS);
    }

  mSeqNums = (int *)bti_malloc(sizeof(int)*mSeq->len);
  chars_to_nums(mSeq->len, mSeq->pRes, mSeqNums);   /* changed back */
  mSeq->aaRes=(char *)bti_malloc((mSeq->len+1)*sizeof(char));
  strncpy(mSeq->aaRes, mSeq->pRes, mSeq->len);
  for (i=0; i<mSeq->len; i++)
    mSeqNums[i] *= WT_MAT_DIM;    /* index into proper row */

} /* init_sb_align */


int check_ident( ALIGN_REC *align)
{

  int j, i,ix, qix, pNum, len, q_len, db_len;
  int numAminoData;
  double q_percent, db_percent, cutoff;

  numAminoData = 0;
  ix = 0;
  len = align->len;

  while (len > 0)
    {
      pNum = MIN(len, RES_PER_LINE);
      len -= pNum;
      qix = ix * RES_PER_LINE;
      for (j=0; j<pNum; j++)
        {
          if (align->dpRes[qix + j] != ' ')
            {
              numAminoData++;
            }
        }
      ix++;
    }

  q_len = mySeq.len;


  //find the length for database sequence
  db_len = 0;
  for(i = 0; i < strlen(align->dpRes); i++)
    {
      if(align->dpRes[i] != ' ')
        db_len++;
    }


  q_percent = (double)align->ident/q_len;
  db_percent = (double)align->ident/db_len;

  q_percent = q_percent * 100;
  db_percent = db_percent * 100;

  cutoff = (double) PERCENT_HOMOLOGY_CUTOFF;

  #ifdef DEBUG
    printf ("q_len=%d, db_len = %d,align->ident=%d, db_percent=%g, q_percent=%g\n",q_len,db_len,align->ident,db_percent, q_percent );
  #endif
  
#if 0
  if (q_percent > cutoff || db_percent > cutoff)
    {
      return 1;
    }
  else
    {
      printf ("Didn't find any alignment identity is greater than %d\n", PERCENT_HOMOLOGY_CUTOFF);
      return -1;
    }
#endif	
	return 1;

}    /* check_ident */

/*------------------------------------------
 * perform alignment of the two sequences. 
 *------------------------------------------
 */
void proc_seqs(SEQ_REC *mSeq, SB_REC *dSeq)
{
  int  rc, scor;

  rc = init_seqs(mSeq, dSeq);
  if (rc == FALSE)
    return;

  protCount++;
  sec_struct_scores(dSeq->sRes, dSeq->len, dsScore);
  fill_score_mat(wtMat, sm, mSeq->len, dSeq->len, mSeqNums, dSeqNums);

  // Do Needleman-Wunsch
  s1_sum_score_mat(sm, mSeq->len, dSeq->len, dsScore, dColumns, dGapPen);
  scor = get_align_score(sm, mSeq->len, dSeq->len);
  eval_sb_align(mSeq, dSeq, scor);
  clean_up(dSeq);

} /* proc_seqs */


/*-----------------------------------------------------
 * create data structures before performing alignment. 
 *-----------------------------------------------------
 */
int init_seqs(SEQ_REC *mSeq, SB_REC *dSeq)
{
  char  sVal[MAX_LINE_LEN];

  if (dSeq->len < MIN_RES)    /* sequence is too short */
    return(FALSE);

  sm = (int *)bti_malloc((mSeq->len * dSeq->len)*sizeof(int));
  dsScore = (int *)bti_malloc(sizeof(int) * dSeq->len);
  dColumns = (int *)bti_malloc(sizeof(int) * dSeq->len);
  dGapPen = (int *)bti_malloc(sizeof(int) * dSeq->len);
  dSeqNums = (int *)bti_malloc(sizeof(int) * dSeq->len);
  jSeqNums = (int *)bti_malloc(sizeof(int) * dSeq->len);

  if ((sm == NULL) || (dsScore == NULL) || (dSeqNums == NULL) ||
      (dColumns == NULL) || (dGapPen == NULL) || (jSeqNums == NULL))
    {
      sprintf(sVal, "sb_align:  Protein size = %d", dSeq->len);
      show_err_mem(FALSE, MODULE, sVal, dSeq->title);
      return(FALSE);
    }

  chars_to_nums(dSeq->len, dSeq->pRes, dSeqNums);  /* changed back */
  return(TRUE);

} /* init_seqs */


/*---------------------------------------------------------
 * determine how good our alignment is and possibly save. 
 *---------------------------------------------------------
 */
void eval_sb_align(SEQ_REC *mSeq, SB_REC *dSeq, int alignScor)
{
  double  testStat;

  if (sortAlgorithm == 0)
    {    /* use best score from scoring mat */
      testStat = (double)alignScor;
      if (alignScor > topAligns[NUMBER_OF_ALIGNMENTS-1].oScor)
        save_sb_align(mSeq, dSeq, alignScor, testStat);
    }

  else if (sortAlgorithm == 1)
    {    /* use best score / seq len */
      /*  testStat = alignScor / (double)(MIN(mSeq->len, dSeq->len) + 8); */
      testStat = alignScor / (double)(mSeq->len + 8);
      if (testStat > topAligns[NUMBER_OF_ALIGNMENTS-1].zVal && testStat>0)
        save_sb_align(mSeq, dSeq, alignScor, testStat);
    }


  else
    {              /* use jumbling method */
      testStat = jumble_test(mSeq, dSeq->pRes, dSeq->len, alignScor);
      if (testStat > topAligns[NUMBER_OF_ALIGNMENTS-1].zVal)
        {
          fill_score_mat(wtMat, sm, mSeq->len, dSeq->len,
                         mSeqNums, dSeqNums);
          s1_sum_score_mat(sm, mSeq->len, dSeq->len, dsScore,
                           dColumns, dGapPen);
          save_sb_align(mSeq, dSeq, alignScor, testStat);
        }
    }

  topScoreGroup = MAX((double)testStat, topScoreGroup);

} /* eval_sb_align */


/*-------------------------------------------------------
 * Performs jumbling algorithm, returns test statistic. 
 *-------------------------------------------------------
 */
double jumble_test(SEQ_REC *mSeq, char *jpRes, int jLen, int scor)
{
  int   i, count, maxJumbles;
  double testStat=0;
  int  jumScors[MAX_JUMBLES];

  chars_to_nums(jLen, jpRes, jSeqNums);

  maxJumbles = jumNum[JUM_TESTS - 1];
  for (count=1; count<=maxJumbles; count++)
    {
      seed = jumble_array(jSeqNums, jLen, seed);
      fill_score_mat(wtMat, sm, mSeq->len, jLen, mSeqNums, jSeqNums);
      s1_sum_score_mat(sm, mSeq->len, jLen, dsScore, dColumns, dGapPen);
      jumScors[count-1] = get_align_score(sm, mSeq->len, jLen);

      for (i=0; i<JUM_TESTS; i++)
        {
          if (count == jumNum[i])
            {
              testStat = calc_test_stat(jumScors, count, (double)scor);
              if (testStat < jumStat[i])
                return(testStat);
            }
        }
    }

  return(testStat);

}  /* jumble_test */


/*------------------------------------------------------------------
 * create an alignment record and save in our best alignment list.
 *------------------------------------------------------------------ 
 */
void save_sb_align(SEQ_REC *mSeq, SB_REC *dSeq, int rawScor, double testStat)
{
  int indNum, size, len;
  int  *indic, *indir;
  char  mStr[MAX_ALIGN_LEN], dStr[MAX_ALIGN_LEN];
  char  sStr[MAX_ALIGN_LEN];
  ALIGN_REC p;

  if ( mSeq->len > MAX_ALIGN_LEN )
    show_err(TRUE, "SAVE SB ALIGN", "Line is too long");

  size = MAX(mSeq->len, dSeq->len) + 2;
  indic = (int *)bti_malloc(sizeof(int) * size);
  indir = (int *)bti_malloc(sizeof(int) * size);
  if ((indic == NULL) || (indir == NULL))
    {
      show_err_mem(FALSE, MODULE, "save_sb_align", dSeq->title);
      return;
    }

  indNum = s1_save_path(sm, mSeq->len, dSeq->len, indic, indir, dsScore);

#ifdef DEBUG
  initDebug();
#endif

  len = get_align_strings(indNum, indir, indic, mSeq, dSeq,
                          mStr, dStr, sStr);

  p.qpRes = (char *)bti_malloc(sizeof(char)*(len + 1));
  p.dpRes = (char *)bti_malloc(sizeof(char)*(len + 1));
  p.dsRes = (char *)bti_malloc(sizeof(char)*(len + 1));

  p.len = len;
  p.ident = string_match_num(mStr, dStr, len, len);
  /*  p.perid = (double) p.ident/MIN(mSeq->len, dSeq->len);    ID% of alignment */
  p.perid = (double) p.ident/mSeq->len;
  p.oScor = rawScor;
  p.zVal = testStat;
  strncpy(p.title, dSeq->title, MAX_TITLE_LEN);

  strncpy(p.id, dSeq->bhid, MAX_ID_LEN);
  strncpy(p.qpRes, mStr, len);
  strncpy(p.dpRes, dStr, len);
  strncpy(p.dsRes, sStr, len);
  sort_sb_align(p);

  free(indic);
  free(indir);
} /* save_sb_align */


/*-------------------------------------------------------
 * Insert alignment record into list of best alignments 
 *-------------------------------------------------------
 */
void sort_sb_align(ALIGN_REC p)
{
  int i;
  ALIGN_REC temp;

  i = 0;
  if (sortAlgorithm == 0)
    {
      while ((i < NUMBER_OF_ALIGNMENTS) && (p.oScor < topAligns[i].oScor))
        i++;
    }
  else
    {
      while ((i < NUMBER_OF_ALIGNMENTS) && (p.zVal < topAligns[i].zVal))
        i++;
    }

  if (i < NUMBER_OF_ALIGNMENTS)
    {
      if (alignNum < NUMBER_OF_ALIGNMENTS)
        alignNum++;
      while (i < NUMBER_OF_ALIGNMENTS)
        {
          temp = topAligns[i];
          topAligns[i] = p;
          p = temp;
          i++;
        }
    }

// SCOTT Added back in the deletion to avoid memory leak
   free(p.qpRes);     /*drop last record */
   free(p.dpRes);
   free(p.dsRes);
} /* sort_sb_align */


/*-----------------------------------------------
 * things to do after performing one alignment. 
 *-----------------------------------------------
 */
void clean_up(SB_REC *dSeq)
{
  free(sm);
  free(dSeqNums);
  free(jSeqNums);
  free(dsScore);
  free(dColumns);
  free(dGapPen);

  if ((protCount % updateFreq) == 0)
    {
      print_stats(stdout, protCount, topScoreGroup);
      print_aligns(protCount, alignNum);
      topScoreGroup = (double)MIN_INT;
    }

}

/* Generates a new PDB file based on the template
  Note: newFile is never actually opened, just used for naming purposes */
int generate_chi_angle(char *template, char *newfile,int counter,
                       char *phipsiFilename)
{
  char syscomd[MAX_COMMAND_LEN];
  char chifile[MAX_COMMAND_LEN];
  char line[MAX_LINE_LEN];
  FILE *fp;
  int i=0,pos=0,num=0;
  CHI_REC *chi_angle;

  int len = topAligns[order[counter]].len;
  chi_angle=(CHI_REC*)bti_malloc(len*sizeof(CHI_REC));
  
  // Run phipsi 
  sprintf(syscomd,"%s -f %s -o %s.phipsi",phipsiFilename, template, newfile);
#ifdef DEBUG
  printf("Running: %s\n",syscomd);
#endif
  system(syscomd);

  sprintf(chifile,"%s.phipsi",newfile);
  fp=open_file(chifile,"r");
  if ( fp == NULL  ) {
    printf("Running phipsi failed, can't open file \"%s\"", chifile);
    fflush(stdout);
  }


  // Read in the angles from phipsi output file, put them in the chi_angle array
  i=0;
  int rv;
  while( i<topAligns[order[counter]].len )
  {
    if( fgets(line,MAX_LINE_LEN,fp) != NULL )
    {
      if(strlen(line)>5)
        {
          rv = sscanf(line,"%*d %s %*f %*f %*f %f",
                            chi_angle[i].res, 
                            &chi_angle[i].chi);
          if ( rv != 2 )
            show_err(FALSE, "generate_chi_angle", "Error reading phipsi file - sscanf failed");
          i++;
        }
    }
    else
      break;
  }

  chi_struc[counter]=(CHI_REC*)bti_malloc( topAligns[order[counter]].len*sizeof(CHI_REC) );

  // modify this array to produce the angles for the aligned sequence, not the template sequence
  for(i=0;i<topAligns[order[counter]].len;i++)
  {
    if(topAligns[order[counter]].qpRes[i] != ' ')
    {
      if(topAligns[order[counter]].dpRes[i]!= ' ')
      {
        sprintf(chi_struc[counter][num].res,"%s",chi_angle[i-pos].res);
        chi_struc[counter][num].chi=chi_angle[i-pos].chi;
        num++;
      }
      else
      {
        chi_struc[counter][num].chi=999;
        chi_struc[counter][num].res[0]='X';
        chi_struc[counter][num].res[1]='\0';
        num++;
      }
    }
    if(topAligns[order[counter]].dpRes[i]==' ')
      pos++;

  }
  fclose(fp);

  fp=open_file(chifile,"w");
  for(i=0;i<num;i++)
    fprintf(fp,"%4d   %s  %8.2f\n",i+1,chi_struc[counter][i].res,chi_struc[counter][i].chi);
  fclose(fp);
  free(chi_angle);
  return 1;
}

void do_GA_stuff (char *pdbtmp)
{
  /***********************************************************
   * Attaches side chains for thrifty.
   ***********************************************************/

  FILE * tmpfilep;
  char   usrfile [MAX_COMMAND_LEN];
  char   sidefile[MAX_COMMAND_LEN];
  char   gafile [MAX_COMMAND_LEN];
  char   command1[MAX_COMMAND_LEN];
  char   command2[MAX_COMMAND_LEN];

  /* Kludge.. need to figure out what pdb file name user chose */
  tmpfilep = open_file(pdbtmp, "r");
  fscanf (tmpfilep, "%s", usrfile);
  fclose (tmpfilep);
  /* END kludge */

  /* Put in the side chains for the alignments */

  sprintf (sidefile,"%s1.pdb.side",usrfile);
  sprintf (gafile,"%s1.side.ga",sidefile);

  sprintf(command1, "perl gen_dis_angle.pl %s %s", sidefile,gafile);
  system(command1);

  sprintf(command2,"./gafolder --ac %s.ac --dc %s_caon.dc > %s", gafile, gafile, gafile );
  system(command2);

  rename(gafile, sidefile);

}
