/*
	file...: main.c
	purpose: main program for shifty program 
 
	author.: Robert Boyko / M. Scott Watson
	date...: APRIL 23, 1997	
	place..: University of Alberta
	  	 Department of Pharmacy and Pharmaceutical Sciences
*/

#include "btimem.h"
#include "csipdb.h"
#include "misc.h"

SEQ_REC	mySeq;

void show_version(), setup_shifty();
void do_shifty(), init_parms();
static void read_fasta_seq();
//extern int aa_sequence();
extern void set_lib_path(), get_output_file();

/*----------------
 * main program
 * 
 * 
 *----------------
 */
int csipdb(char *File, PROGRAM_PARAMS params)
{

  setup_shifty(File, "shifty.parms", params.shifty_home, params.seqseedb);
  read_fasta_seq(File);

#ifdef DEBUG
  printf ("\nHomology pairing by primary sequence.\n\n");
#endif

  if (aa_sequence(params)==0)
  {
    if ( strlen(params.sequenceName) > 0 )
      printf ("\n This sequence did not have > %d%c homology.\n",PERCENT_HOMOLOGY_CUTOFF, '%');
    else
      printf ("\n Sorry, No query was found that had >%d%c homology.\n Exit HOMODELLER.\n",PERCENT_HOMOLOGY_CUTOFF, '%');
    return 0;
  }

#ifdef DEBUG
  printf("After aa_sequence\nalign.len=%d, ident=%d, query length=%d, db length=%d, struct length=%d\n",
         topAligns[0].len,
         topAligns[0].ident,
         (int) strlen(topAligns[0].qpRes),
         (int) strlen(topAligns[0].dpRes),
         (int)strlen(topAligns[0].dsRes));
#endif
  return(1);

} /* main */

static void read_fasta_seq(char *file)
{
  FILE *fp;
  int len,max=256;
  char line[256];

  fp=open_file(file,"r");
  mySeq.pRes=(char*)bti_malloc(max*sizeof(char));
  while((fgets(line,256,fp)!=NULL))
  {
    len=strlen(line)-1;
    if(line[0]!='\n' && line[0]!=' ')
    {
      if(line[0]=='>')
      {
        strncpy(mySeq.id, &line[1],len-1);
        mySeq.id[len-1]='\0';
      }
      else
      {
        if ( strpbrk( line, ".1234567890") != NULL )
        {
          show_err(TRUE, "Read FASTA", "Invalid FASTA sequence");
        }
        mySeq.pRes=(char *)realloc((char*)mySeq.pRes,(len+max)*sizeof(char));
        max+=len;
        strncat(mySeq.pRes,line,len);
      }
    }
  }
  mySeq.len=strlen(mySeq.pRes);
  fclose(fp);

  // sanity check on if the sequence is valid
  if ( strpbrk( mySeq.pRes , ".1234567890BJOUZ") )
    show_err(TRUE, "Read FASTA", "Invalid FASTA sequence");
}

void setup_shifty(char *file, char *parms, char *shifty_home, char *seqseedb)
{
  char    parmsFname[MAX_FNAME_LEN];

  set_lib_path(shifty_home); //shifty_home == "data"

  strcpy(parmsFname, parms);
	

  init_parms(parmsFname, shifty_home, seqseedb);
  strcpy (resultsFname, file);
  
  
  strcat (resultsFname, ".out");
  get_output_file(resultsFname);
}       /* setup_shifty */

