#ifndef __CSIPDB_H
#define __CSIPDB_H

/*
	shifty.h file
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "vector.h"

#define MAX_ATOM_NUM    30
#define TRUE		1
#define FALSE		0
#define MAX_LINE_LEN	256
#define MAX_FNAME_LEN	256
#define MAX_TITLE_LEN	256
#define MIN_INT		(1 << (32 - 1))
#define MAX_INP_LEN     256
#define MAX_ATOM_NUM    30
#define MAX_NAME_LEN    5
#define MAX_OTHER_LEN   20
#define MAX_ALN_LEN     2000
#define FILE_READ	   "r"		/* fopen read macro*/
#define FILE_WRITE	   "w"		/* fopen write macro       */
#define MAX_NUM_GAPS    50

/*Update definitions July 18, 2006 TODO: Add to util.h*/
#define PDB_READ_FORMAT  "%4s %6d %3s%4s %5d %7f %7f %7f %f %f %8s\n"
#define PDB_WRITE_FORMAT "%4s %6d  %-3s%4s %5d %11.3f %7.3f %7.3f %5.2f %5.2f %13s\n"
#define N_ATOM           "N"
#define C_ATOM           "C"
#define CB_ATOM          "CB"
#define O_ATOM           "O"
#define GLYCINE          "GLY"
#define REF_ATOM         "CA"
#define helix            "HELIX"
#define coil             "COIL"

/*TODO: add to error.h*/
#define ERR_MEM_ALLOC    "memory allocation failed"
#define ERR_MEM_REALLOC  "memory re-allocation failed"

/* MAX_NAME_LEN is the length of atom names and residue names */
/*#define SHIFTY_LIB	"/usr/local/lib/shifty"*/

#define SHIFTY_LIB	"."    /* for temporay use */
#define SSLIB          	"seqsee.db"


#define VERSION	        "1.1"
#define MODULE		"SHIFTY"
#define PARMS_FLAG      "-i"
#define RESULTS_FLAG    "-o"
#define USER_PARMS	"./shifty.parms"
#define KEYBD_PROMPT	"\n>> "
#define PARMS_SCH	'>'
#define PIR_SCH		'>'
#define SB_SCH		'>'
#define END_OF_SEQ	'$'
#define CH_HELX		'H'
#define CH_BETA		'B'
#define CH_COIL		'C'
#define CSI_SCH		'#'

/* may need adjustifaction */
#define WT_MAT_DIM	32
#define WT_MAT_SIZE	(WT_MAT_DIM * WT_MAT_DIM)

#define JUM_TESTS	6
#define MAX_ID_LEN	20
#define MAX_RECORDS	20
#define RES_PER_LINE	50

#define CODES		26
#define AA_NUM		20
#define AA_MATCH	'='
#define AA_NEAR_MATCH	'*'
#define AA_NOT_MATCH	' '

#define MIN_RES		3
#define MAX_RES		3000
#define MAX_ALIGN_LEN	(MAX_RES + MAX_RES/10)
#define MAX_JUMBLES	1000

#define BAD_CHAR_ERR	-20
#define MAX_RES_ERR	-30
#define MIN_RES_ERR	-40	

#define FIELD_LEN_SB	12
#define SB_REF		"#REFERENCE :"
#define SB_ID		"#SEQBANK ID:"
#define SB_BHID         "#BRKHAVN ID:"
#define SB_PIRID        "#PIR_NBR ID:"
#define SB_SPID         "#SWISPRO ID:"
#define SB_CLASS        "#FOLD CLASS:"
#define SB_RESOL        "#RESOLUTION:"
#define SB_RFACT        "#R FACTOR  :"
#define SB_RESNUM       "#NUM RESIDU:"
#define FLD_NONE        0
#define FLD_REF         1
#define FLD_SBID        2
#define FLD_BHID        3
#define FLD_PIRID       4
#define FLD_SPID        5
#define FLD_CLASS       6
#define FLD_RESOL       7
#define FLD_RFACT       8
#define FLD_RESNUM      9
#define FLD_AA          10
#define FLD_INVALID     99

#define CSI_ID		"#  Input.....:"
#define CSI_AA		6
#define CSI_SS		55	

#define A_CFORM		"A"
#define B_CFORM		"B"
#define AB_CFORM	"AB"
#define M_CFORM		"M"
#define CA_CFORM	"CA"
#define CB_CFORM	"CB"
#define CM_CFORM	"CM"
#define A_IFORM		1
#define B_IFORM		2
#define AB_IFORM	3
#define M_IFORM		4
#define CA_IFORM	5
#define CB_IFORM	6
#define CM_IFORM	7

#define MAX(x,y)	(x>y? x: y)
#define MIN(x,y)	(x<y? x: y)
#define RAND(c)		(((c * 32719) + 3) % 32749)
#define AA_IX(c)	(c - 'A')
#define AA_UC(c)	(((c>='a') && (c<='z'))? c+'A'-'a': c)
#define AA_CVALID(c)	(((c >= 'A') && (c <= 'Z'))? TRUE: FALSE) 
#define SS_CVALID(c)	( ((c=='B') || (c=='C') || (c =='H')) ? TRUE : FALSE )


#define SHIFT_PARM_NUM  3

/* for GAP_INFO_REC */
#define INSERTION 1
#define DELETION 0
#define HCH 0
#define BCB 1
#define HCB 2
#define BCH 3
#define HHH 4
#define BBB 5
/* end for GAP_INFO_REC */

#define NUMBER_OF_ALIGNMENTS 10

#define PERCENT_HOMOLOGY_CUTOFF   35

/* define data structures */

typedef enum {
    false, true
  } boolean;

/* A structure to represent a sequence in SeqSeeDB database */
typedef struct {
  int	len;
  char	id[MAX_TITLE_LEN];
  char	*pRes;    /* query's 2ndary sequence modified*/
  char	*sRes;    /* query's 2ndary sequence */
  char  *aaRes;   /* query's primary amino acid sequence */
} SEQ_REC;

/* A structure to represent a sequence in SeqSeeDB database, with additional params */
typedef struct {
  char    	title[MAX_TITLE_LEN];
  char    	sbid[MAX_ID_LEN];
  char    	bhid[MAX_ID_LEN];
  char    	pirid[MAX_ID_LEN];
  char    	spid[MAX_ID_LEN];
  int	  	fold;
  double  	resol;
  double  	rfact;
  int	  	len;
  char          *aaRes; /* database's aa */
  char    	*pRes;  /* database's 2ndary sequence modified*/
  char    	*sRes;  /* database's 2ndary sequence */
} SB_REC;

typedef struct {
  int type;             /* 0 for deletion; 1 for insertion */
  int gap_class;            /* 0:HCH, 1:BCB, 2:HCB, 3:BCH, 4:HHH, 5:BBB */
  int size;             /* Size of the gap */
  int beg;              /* Start index of the gap */
  int end;              /* Index where the gap ends */
  int loop_len;         /* length of loop needed to fill this gap */
  int isBeg;            /* 0 if gap is at beg of sequence */ 
  int isEnd;            /* 0 if gap is at end of sequence */
  int prev_eInd;        /* Closest secondary struct prior to gap for elbow dist */
  int post_eInd;        /* Closest secondary struct following the gap for elbow dist */
  double elbowdist;     /* Elbow to Elbow distance */
  int prev_fInd;        /* Closest secondary struct prior to gap for fist dist */
  int post_fInd;        /* Closest secondary struct following the gap for fist dist */
  double fistdist;      /* Fist to Fist distance */
  double dists[25];      /* Added by xiaoli to store the distance measurement in order to find best loop*/
  int candidate;	/* Loop num from LOOPPDB data base to fill hole */
  double first;
  double second;
  double third;
  boolean dontFill;
} GAP_INFO_REC;

typedef struct {
  int candidate;	/* Loop num from LOOPPDB data base to fill hole */
  float score;
  float first;
  float second;
  float third;
  float average;
  int number;
} CANDIDATE_REC;

typedef struct {
  int	len;
  long	oScor;
  int  	ident;          /* number of identical matches in 2 strings - assigned from string_match_num (calc.c) */
  int   query_offset;	/* Index of first query seq residue with respect to db seq */
  double	perid;  /* percent identity of an alignment */
  double	zVal;   /* Test stat (I assume z-Value of normal distribution?) - assigned from calc_test_stat (calc.c) */
  char  id[MAX_ID_LEN];
  char  title[MAX_TITLE_LEN];
  char  *qpRes;  /* contains the query sequence */
  char	*dpRes;  /* contains the database sequence */
  char	*dsRes;  /* contains the secondary structure information */
  int gapCount;  /* the total number of gaps */
  GAP_INFO_REC gaps[MAX_NUM_GAPS]; /* contains the info about gaps in query and database sequence */
} ALIGN_REC;

typedef struct {
	char atomName[MAX_NAME_LEN];
  /*
	int		atomNum;
  */
	VECTOR	vec;
}	ATOM_REC;


struct resRec {
  ATOM_REC	*atom;
  int		atomNum;
  char          res;    /* contain is the aa */
  char          resName[5];
};	

struct loopRec {
  ATOM_REC      atom[3];            /* contains the x y z coordinates */
  int           atomNum[3];       /* residue number */
  char          res[3];
  char          sec_struct[3];
};

struct chemshiftRec {
	int	num;
	char	res;
	float	ha;
	float hn;
	float	n15;
	float	ca;
	float	cb;
	float	co;
	float	phi;
	float psi;
}; /* for shiftyx */

struct top {
    char ddex[100];
    char qdex[100];
    int  ident;
    char name[MAX_ID_LEN];
};

struct chi {
	char res[MAX_NAME_LEN];  // name of the residue
	float chi;    // chi angle
};

/*TODO: Add the following type defnitions to Util.h*/
typedef struct {
  //double x; double y; double z;
  float x,y,z;
} POINT;	

typedef struct{
  char   flag[5];
  int    atom_no;
  char   atom[5];
  char   res[5];
  int    res_no;
  POINT  pt;
  float num1;
  float num2;
  char   extra[10];
  char whole_line[MAX_LINE_LEN];
  }PDB_LINE;


typedef struct {
  POINT first_CA, sec_CA, third_CA;
  POINT third_last_CA, sec_last_CA, last_CA;
}SIX_CA_POINTS;	  

typedef struct chi CHI_REC;
typedef struct top TOPO;
typedef struct resRec RES_REC;
typedef struct chemshiftRec CS_REC;
typedef struct loopRec LOOP_REC;

/* global variables */

extern FILE    *sbfp;
extern char    sMatFname[MAX_FNAME_LEN];
extern int     wtMat[WT_MAT_SIZE];
extern int     nearMatchScore;
extern int     seed, startSeed;
extern int     sortAlgorithm;
extern double  jumStat[JUM_TESTS];
extern int     jumNum[JUM_TESTS];
extern int     updateFreq;
extern int     gapPenalty;
extern int     gapSizePenalty;
extern int     penCoil;
extern int     penMiddle;
extern int     penEnd;
//extern int     topAlignNum;
extern SEQ_REC mySeq;
extern ALIGN_REC *topAligns;
TOPO *top;
extern char resultsFname[MAX_FNAME_LEN];
extern char shifty_lib[MAX_FNAME_LEN];
CHI_REC **chi_struc;
int order[100];

struct programParams
{
  char phipsiFilename[MAX_FNAME_LEN];   /* phipsi executable filename */
  char pdbtmp[MAX_FNAME_LEN];           /* "pdb.tmp" file */
  char pdbdir[MAX_FNAME_LEN];           /* directory where to find PDB files */
  char shifty_home[MAX_FNAME_LEN];      /* directory to find homodeller */
  char tmpdir[MAX_FNAME_LEN];           /* temporary directory that can be written to */
  char simplexFilename[MAX_FNAME_LEN];  /*simplex excetuable filename */
  char seqseedb[MAX_FNAME_LEN];         /* seqsee.db file location */
  char sequenceName[MAX_FNAME_LEN];         /*Sequence Name - for debugging, we can choose to align
                                             against a certain sequence found in the seqsee DB.*/
};

typedef struct programParams PROGRAM_PARAMS;


int aa_sequence ( PROGRAM_PARAMS params );
int csipdb(char *File, PROGRAM_PARAMS params);

#endif
