#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "btimem.h"
#include "csipdb.h"
#include "misc.h"

extern int generate_chi_angle();

int copyPDBFileFromAlignment(char* templatePDBFileName, char* newPDBFileName, int counter);
static void quicksort();
static void partition();
int lefth,leftt,righth,rightt;


/*************************************************************\
| Precondition: conversion hasn't occurred                    |
| Postcondition: the conversion has occurred                  |
| Purpose: To convert the single letter aa code to the 3      |
|          letter code.                                       |
\*************************************************************/
void convert (char aa_name[], char aa)
{
  switch (toupper(aa))
  {
  case 'A':
    {
      strcpy (aa_name,"ALA");
      break;
    }
  case 'R':
    {
      strcpy (aa_name,"ARG");
      break;
    }
  case 'D':
    {
      strcpy (aa_name,"ASP");
      break;
    }
  case 'N':
    {
      strcpy (aa_name,"ASN");
      break;
    }
  case 'C':
    {
      strcpy (aa_name,"CYS");
      break;
    }
  case 'E':
    {
      strcpy (aa_name,"GLU");
      break;
    }
  case 'Q':
    {
      strcpy (aa_name,"GLN");
      break;
    }
  case 'G':
    {
      strcpy (aa_name,"GLY");
      break;
    }
  case 'H':
    {
      strcpy (aa_name,"HIS");
      break;
    }
  case 'I':
    {
      strcpy (aa_name,"ILE");
      break;
    }
  case 'L':
    {
      strcpy (aa_name,"LEU");
      break;
    }
  case 'K':
    {
      strcpy (aa_name,"LYS");
      break;
    }
  case 'M':
    {
      strcpy (aa_name,"MET");
      break;
    }
  case 'F':
    {
      strcpy (aa_name,"PHE");
      break;
    }
  case 'P':
    {
      strcpy (aa_name,"PRO");
      break;
    }
  case 'S':
    {
      strcpy (aa_name,"SER");
      break;
    }
  case 'T':
    {
      strcpy (aa_name,"THR");
      break;
    }
  case 'W':
    {
      strcpy (aa_name,"TRP");
      break;
    }
  case 'Y':
    {
      strcpy (aa_name,"TYR");
      break;
    }
  case 'V':
    {
      strcpy (aa_name,"VAL");
      break;
    }
  }
}

/*************************************************************\
| Precondition: conversion hasn't occurred                    |
| Postcondition: the conversion has occurred                  |
| Purpose: To convert the current ALIGN_REC and SEQ_REC to    |
|          ALN_REC and resRec                                 |
\*************************************************************/
void convert_rec(RES_REC *query)
{
  int counter=0;

  switch (query->res)
  {
  case 'A':
    {
      query->atomNum=5;
      break;
    }
  case 'C':
    {
      query->atomNum=6;
      break;
    }
  case 'D':
    {
      query->atomNum=8;
      break;
    }
  case 'E':
    {
      query->atomNum=9;
      break;
    }
  case 'F':
    {
      query->atomNum=11;
      break;
    }
  case 'G':
    {
      query->atomNum=0;
      break;
    }
  case 'I':
    {
      query->atomNum=8;
      break;
    }
  case 'H':
    {
      query->atomNum=10;
      break;
    }
  case 'K':
    {
      query->atomNum=9;
      break;
    }
  case 'L':
    {
      query->atomNum=8;
      break;
    }
  case 'M':
    {
      query->atomNum=8;
      break;
    }
  case 'N':
    {
      query->atomNum=8;
      break;
    }
  case 'P':
    {
      query->atomNum=7;
      break;
    }
  case 'Q':
    {
      query->atomNum=9;
      break;
    }
  case 'R':
    {
      query->atomNum=11;
      break;
    }
  case 'S':
    {
      query->atomNum=6;
      break;
    }
  case 'T':
    {
      query->atomNum=7;
      break;
    }
  case 'V':
    {
      query->atomNum=7;
      break;
    }
  case 'W':
    {
      query->atomNum=14;
      break;
    }
  case 'Y':
    {
      query->atomNum=12;
      break;
    }
  default:
    query->atomNum=0;
    printf("Unknown residue %c", query->res);
    break;
  }

  if (query->atomNum)
  {
    query->atom= (ATOM_REC *)bti_malloc (query->atomNum*(sizeof(ATOM_REC)));
    convert (query->resName, query->res);

    for (counter=0;counter<query->atomNum;counter++)
    {
      strcpy (query->atom[counter].atomName,"");
      query->atom[counter].vec.x=0.0;
      query->atom[counter].vec.y=0.0;
      query->atom[counter].vec.z=0.0;
    }
  }
}
/*************************************************************\
| Precondition: generation of pdb file(s) hasn't been created |
| Postcondition: generation of pdb file(s) has been created   |
| Purpose: To create the pdb file(s).                         |
|          generation of pdb file(s) for pseudoproteins       |
\*************************************************************/

int genpdb(int no_pdbfile, char *pdb_dir, char * aa_seq, char *phipsiFilename,
           char *pdbtmp)
{

  char syscall_buffer[256];
  int counter=0;
  int format_no=0;    /* contains the number of elements converted probably */
  int db_res_no=0;       /* aa residue number of the database*/
  int pos=0,dbpos;          /* contains the position where the query sequence and the database sequence overlap */
  int line=0;         /* line counter in the new pdb file */
  
  int ident[10];
  int last_res_num=1;        /* last database number */
  int res_no=1;      /* the residue number for the query sequence */
  //long int offset;
  RES_REC query;

  float x=0, y=0, z=0;
  
  char aa_name[3];
  char db_aa_name[3];
  char* templatePDBFileName;  // contains the name of the pdbfile to be read in
  char* newPDBFileName;  // contains the name of the pdbfile to be created

  float t_x=0, t_y=0, t_z= 0;

  char temp4[256];
  //char temp6[256];
  char temp5[2];

  /* intialing variables */
#ifdef DEBUG

  printf ("\nResults Fname: %s \n",resultsFname );
#endif

  for (counter=0; counter<2; counter++)
  {
    aa_name[counter]='\0';
  }

  templatePDBFileName = bti_malloc(MAX_FNAME_LEN*sizeof(char));
  newPDBFileName = bti_malloc(MAX_FNAME_LEN*sizeof(char));

  pos=0;
  line=1;
  format_no=0;
  for(int i=0;i<10;i++)
  {
    order[i]=i;
    ident[i]=topAligns[i].ident;
  }

  /* Initialize query so we don't accidentally use uninitialized data */
  query.atom = NULL;
  query.atomNum = 0;
  query.res = 0;
  memset(query.resName, 0, 5);

  // sort the identities
  quicksort(0,9,order,ident);

  // copy the output file name (this is essentially substring)
  // e.g. test.fa.out -> test.fa.
  for(int i=0; i < (int)strlen(resultsFname)-3;i++)
  {
    temp4[i]=resultsFname[i];
  }
  temp4[ (int)strlen(resultsFname)-3 ]='\0';
  FILE* pdbfilein=fopen (pdbtmp,"w");

  if (pdbfilein==NULL)
  {
    printf ("\nCSIPDB: Could not create file \"%s\"! Program is going to terminate!\n",pdbtmp);
    exit(-1);
  }

  /* Write debug values to a file:
  <FILENAME>
  <VALUE OF NO_PDBFILE (1/0)>
  */
  fprintf (pdbfilein,"%s\n%d\n",temp4,no_pdbfile);
  fclose (pdbfilein);
  chi_struc=(CHI_REC**)bti_malloc(no_pdbfile*sizeof(CHI_REC*));
  for (counter=0;counter < no_pdbfile; counter++)
  {
    /* setting up variables */
    last_res_num=1;
    line=1;
    db_res_no=1;
    res_no=1;
    dbpos=0;


		printf("Alignment QP: %s\n", topAligns[order[counter]].qpRes );
		printf("Alignment DP: %s\n", topAligns[order[counter]].dpRes );

    /* use the new PDB database */
    strcpy (templatePDBFileName,pdb_dir);

#ifdef WIN32

    strcat (templatePDBFileName,"\\");
#else

    strcat (templatePDBFileName,"/");
#endif

    strcat (templatePDBFileName,topAligns[order[counter]].id);

    strcat (templatePDBFileName,".pdb");

    strcpy (newPDBFileName,temp4);
    sprintf (temp5,"%d",counter+1);
    strcat (newPDBFileName,temp5);
    strcat (newPDBFileName,".pdb");

    // If the file doesn't exist, go get it
    if ( ! fileExists( templatePDBFileName ) )
    {
      downloadPDBFile(templatePDBFileName);

      // make sure it worked
      if ( ! fileExists( templatePDBFileName ) )
      {
        show_msg("Downloading PDB file failed!\n");
        exit(-1);
      }
    }

    generate_chi_angle(templatePDBFileName,newPDBFileName,counter, phipsiFilename);

    
    // A new algorithm for copying the file
    int rv = copyPDBFileFromAlignment(templatePDBFileName, newPDBFileName, counter);
//    fclose (pdbfilein);
    return rv;

    /*************** THIS CODE IS NOT EXECUTED ANYMORE ***********/


    // contains the file pointer to the new PDB file
    FILE* pdbfileout=fopen (newPDBFileName,"w");

    for (pos=0; pos<topAligns[order[counter]].len; pos++)
    {
      /* setting up variables */
      int copy=TRUE;        /* copy=1 means continue to copy pdb file
                              copy=0 mean stop copying pdb file  */
      int first=TRUE;

      // move the variables for the alignment
      // i.e. if query starts after the database sequence, move the variables to accomodate that
      while(topAligns[order[counter]].dpRes[pos] == ' ')
      {
        res_no++;
        dbpos++;
        pos++;
      }

      if (topAligns[order[counter]].qpRes[pos] != ' ')
      {
        /* initializing variables */
        int res_num = 0;
        char temp[5];
        char temp2[5];
        char sLine[MAX_LINE_LEN];

        //printf("pos=%d, db_res_no=%d, dbpos=%d\n", pos, db_res_no, dbpos);
        while ( copy 
              && db_res_no <= pos+1-dbpos 
              && strncmp (temp,"TER",3)!=0 
              && fgets(sLine,MAX_LINE_LEN,pdbfilein)!= NULL )
        {
          sscanf (sLine,"%s",temp);
          if (strncmp (temp,"TER",3)==0)
          {
            break;
          }
          else if (strncmp(temp,"ATOM",4)!=0)
          {
            continue;
          }
          sscanf(&sLine[13],"%3s", temp2);
          sscanf(&sLine[17],"%3s",db_aa_name);
          sscanf(&sLine[22], "%d",&res_num);
          sscanf(&sLine[30],"%f%f%f",&x,&y,&z);

          if (res_num!=last_res_num)
          {
            // we've moved residues in the PDB file
            db_res_no++;
            last_res_num = res_num;
          }

          char trans_atoms[5];
          int N_left = 0;  // this stops it from duplicating atoms

          if (pos==(db_res_no-1+dbpos))
          {
            convert(aa_name,aa_seq[res_no-1]);
            query.res=aa_seq[res_no-1];


            if(N_left == 1)
            {
              // TODO N-left is never 1 !!
              fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d %11.3f %7.3f %7.3f  1.00  1.00    %s\n",
                      line++,trans_atoms,aa_name,res_no,t_x,t_y,t_z,temp4);

              //printf("ATOM5 %6d  %-3s %s %5d %11.3f %7.3f %7.3f  1.00  1.00    %s\n",
              //  line,trans_atoms,aa_name,res_no,t_x,t_y,t_z,temp4);

              N_left = 0;
            }

            if (strncmp(aa_name,db_aa_name,3)==0)
            {
              // if it's the same residue then print it

              if(sLine[16]==' ' || sLine[16]=='A')
              {
                if(temp2[0]!='H')
                {
                  fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",line++,temp2,aa_name,
                          res_no,x,y,z,temp4);

                  //printf("ATOM1 %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",line,temp2,aa_name,
                  //res_no,x,y,z,temp4);
                }
              }
            }
            else if (strcmp (temp2,"CB")==0)
            {
              copy = FALSE;
              switch (query.atomNum)
              {
              case 0:
                {
                  break;
                }
              case 5:
              default:
                {
                  if(sLine[16]==' ' || sLine[16]=='A')
                  {
                    if(temp2[0]!='H')
                    {
                      fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                              line++,temp2,aa_name, res_no,x,y,z, temp4);
                      //printf("ATOM2 %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                      //line,temp2,aa_name, res_no,x,y,z, temp4);
                    }
                  }
                  break;
                }
                free (query.atom);
                break;
              }
              res_no++;
            }
            else if (copy)
            {
              if(temp2[0]!='H')
              {
                if(sLine[16]==' ' || sLine[16]=='A')
                {
                  fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                          line++,temp2,aa_name,res_no,x,y,z,temp4);
                  //printf( "ATOM4 %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                  //line,temp2,aa_name,res_no,x,y,z,temp4);

                }
                if (first==1)
                {
                  convert_rec (&query);
                  first=0;
                }
                if (query.atomNum!=0)
                {
                  if (strcmp(temp2,"N")==0)
                  {
                    strcpy (query.atom[0].atomName,temp2);
                    query.atom[0].vec.x=x;
                    query.atom[0].vec.y=y;
                    query.atom[0].vec.z=z;
                  }
                  if (strcmp(temp2,"CA")==0)
                  {
                    strcpy (query.atom[1].atomName,temp2);
                    query.atom[1].vec.x=x;
                    query.atom[1].vec.y=y;
                    query.atom[1].vec.z=z;
                  }
                  if (strcmp(temp2,"C")==0)
                  {
                    strcpy (query.atom[2].atomName,temp2);
                    query.atom[2].vec.x=x;
                    query.atom[2].vec.y=y;
                    query.atom[2].vec.z=z;
                  }
                  if (strcmp(temp2,"O")==0)
                  {
                    strcpy (query.atom[3].atomName,temp2);
                    query.atom[3].vec.x=x;
                    query.atom[3].vec.y=y;
                    query.atom[3].vec.z=z;
                  }
                }
              }
            }
          }
          else if (pos<(db_res_no-1+dbpos) && res_no<mySeq.len)
          {
            /*
              In 102L this code is causing a wrong atom to be added to the PDB file
              The next residue is the same as the template (LYS), but it is
              not the correct residue and is not in the correct position.

              It seems to drop into here when we're transitioning from one residue to another.
              The problem is that it will also include the next residue if it the same residue name, 
              even after a gap.

              Temp Solution: make sure the next atom is a N, as that is the first atom always shown in PDB format
              Nope - need to keep track of when we're changing residues (by their number)
              - Scott 
            */

            //origion
            //if(topAligns[order[counter]].qpRes[pos+1]!= ' ')

            // If this residue is a gap, then don't add it
            if(topAligns[order[counter]].dpRes[pos+1]!= ' ' 
                && topAligns[order[counter]].qpRes[pos+1]!= ' '  )
            {
              convert(aa_name,aa_seq[res_no]);

              if(sLine[16]==' ' || sLine[16]=='A')
                fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d %11.3f %7.3f %7.3f  1.00  1.00    %s\n",line++,temp2,aa_name,
                        ++res_no,x,y,z,temp4);
              //printf("ATOM5 %6d  %-3s %s %5d %11.3f %7.3f %7.3f  1.00  1.00    %s\n",line,temp2,aa_name,
              //res_no,x,y,z,temp4);

            }
            else
            {
              // fall into here if the database sequence has a gap there
              // what purpose to these serve?
              res_no++;
              copy=FALSE;
              N_left = 1;
              t_x = x;
              t_y=y;
              t_z=z;
              sprintf(trans_atoms, "%s", temp2);
            }
          }

          fflush (pdbfileout);
        }
      }
    }
    fprintf (pdbfileout,"END\n");
    fflush (pdbfileout);


    fclose (pdbfilein);
    fclose (pdbfileout);
  }

  if ( strlen(newPDBFileName) > 120 )
  {
    printf( "Filename too long (max 120): %s!", newPDBFileName );
    exit(-1);
  }
  sprintf(syscall_buffer,"cp %s %s.genpdb.pdb\n",newPDBFileName,newPDBFileName);
  printf("system: %s\n", syscall_buffer);
  system(syscall_buffer);

  /* releases memory */
  free( newPDBFileName );
  free( templatePDBFileName );

  return 1;
}

/*************************************************************\
| Precondition: generation of pdb file(s) hasn't been created |
| Postcondition: generation of pdb file(s) has been created   |
| Purpose: To act as the interface between shifty and the     |
|          generation of pdb file(s) for pseudoproteins       |
\*************************************************************/

int csthread (char *aa_seq, int num, char *phipsiFilename, char *pdbtmp,
              char *pdbdir)
{
  int no_pdbfile=0;

  /* initialization and setting up of variables */
  switch (num)
  {
  case 1:
    {
      no_pdbfile=1;
      break;
    }
  case 2:
    {
      no_pdbfile=5;
      break;
    }
  }

  genpdb (no_pdbfile, pdbdir, aa_seq, phipsiFilename, pdbtmp);

  return 1;
}
void quicksort(int h,int t, int* order, int* ident)

{
  if(h<t)
  {
    partition(h,t, order,ident);

    h=righth;
    t=rightt;
    quicksort(lefth,leftt,order,ident);

    quicksort(h,t,order,ident);

  }
}
void partition(int h,int t,int *order, int *ident)

{
  int  pivot;
  int i,j;
  int tmp;
  i=h;
  j=t;
  pivot=ident[(i+j)/2];

  do
  {
    while(ident[i]>pivot)
      i++;
    while(ident[j]<pivot)
    {
      j--;
    }
    if (i<=j)
    {
      tmp=order[i];
      order[i]=order[j];
      order[j]=tmp;

      tmp=ident[i];
      ident[i]=ident[j];
      ident[j]=tmp;

      i++;
      j--;
    }
  }
  while(i<=j);
  lefth=h;
  leftt=j;
  righth=i;
  rightt=t;
}

/**
  Copies the template file, but re-numbers the residues from 1-N with no gaps in the number series

  @param inputFileName the template to copy
  @param outputFileName the new PDB we're going to create
**/
void cleanupTemplateFile(char* inputFileName, char* outputFileName)
{
  FILE* filein = open_file(inputFileName, "rb");
  FILE* fileout = open_file(outputFileName,"w");
  char sLine[MAX_LINE_LEN];
  
  char atom_name[MAX_NAME_LEN];
  char aa_name[MAX_NAME_LEN];
  float x, y, z;
  int res_num_from_file;
  int res_num = 0;
  int last_res_num = -999;
  int atom_num = 1;

  while ( fgets(sLine,MAX_LINE_LEN,filein) != NULL )
  {
    if(strncmp(sLine, "ATOM", 4) == 0){
      sscanf(&sLine[13],"%3s", atom_name);
      sscanf(&sLine[17],"%3s", aa_name);
      sscanf(&sLine[22], "%d", &res_num_from_file);
      sscanf(&sLine[30],"%f%f%f",&x,&y,&z);
  
      if ( res_num_from_file == 0 )
      {
        // ignore any prepended residues, ACE or FOR
        continue;
      }
      
      // if the residue number changed, increment ours
      if ( last_res_num != res_num_from_file )
        res_num++;
  
      last_res_num = res_num_from_file;
      
      fprintf(fileout, "ATOM %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                          atom_num++, atom_name, aa_name, res_num, x, y, z, "");
      #ifdef DEBUG
      fflush(fileout);
      #endif
    }
    if (strncmp (sLine,"TER",3)==0)
    {
      fprintf(fileout, "%s", sLine);
      fflush(fileout);
      break;
    }
  }
  fclose(filein);
  fclose(fileout);
}

/**
  Copies the template file, but adjusts for gaps and mismatched residues
  according to the alignment

  @param templatePDBFileName the template to copy
  @param newPDBFileName the new PDB we're going to create
  @param counter
**/
int copyPDBFileFromAlignment(char* templatePDBFileName, char* newPDBFileName, int counter) {
  int res_num  = 1;  // the residue number in the PDB file we're reading 
  int res_num_to_print = 1; // the residue index to print to the new file
  int dbpos = 0;    // position in the database
  int apos  = 0;    // position in the alignment
  FILE* pdbfilein;

  ALIGN_REC align = topAligns[order[counter]];

  char tmpPDBFile[1000];
  sprintf(tmpPDBFile, "%s%s", newPDBFileName, ".tmp");

  cleanupTemplateFile(templatePDBFileName, tmpPDBFile);

  // the tmp file is now the input into our copying algorithm
  pdbfilein=fopen (tmpPDBFile,"rb");

  if (pdbfilein==NULL)
  {
      printf ("\nCSIPDB: Could not open %s! Program is going to terminate!\n",templatePDBFileName);
      exit(-1);
  }
  
#ifdef DEBUG
  printf ("\nCSIPDB: Opened %s successfully!\n",templatePDBFileName);
#endif

  // Find the pointer to where the data begins
  int file_offset, first_res_num;
  while(TRUE)
  {
    char sLine[MAX_LINE_LEN];
    file_offset = ftell(pdbfilein);
    if(fgets(sLine,MAX_LINE_LEN,pdbfilein)!= NULL)
    {
      if(strncmp(sLine,"ATOM",4)==0)
      {
        sscanf(&sLine[22], "%d", &first_res_num);
        break;
      }
    }
  }

  while(align.dpRes[apos] == ' ')
  {
    //res_num++;
    //res_num_to_print++;
    //dbpos++;
    apos++;
    dbpos++;
  }

  // seek back to where the data begins
  fseek(pdbfilein,file_offset,SEEK_SET);

  // contains the file pointer to the new PDB file
  FILE* pdbfileout=open_file(newPDBFileName,"w"); 

  /* 
    for each residue we have 4 possible cases:
    Q = query residue
    D = database residue (from the database sequence for which we are copying the known PDB file)

    case 1: Q == D  && Q != ' ' && D != ' '
      => copy this residue
    case 2: Q != D && Q != ' ' && D != ' '
      => copy this residue, but convert it to the query's residue
    case 3: Q == ' ' && D != ' '
      => don't copy this residue, as the query is a gap
    case 4: D == ' ' && Q != ' '
      => there is nothing to copy, move on
  */
  
  
  int readNewLine = TRUE;
  int endOfFile = FALSE;
  int line = 1;
  while( apos++ < align.len && ! endOfFile )
  {
    char query_res = align.qpRes[apos-1];
    char db_res = align.dpRes[apos-1];
    
    int copy = FALSE;
    int convert_residue = FALSE;
    if ( query_res != ' ' ) {
      // excludes case 3

      if ( query_res == db_res ) {
        // case 1
        copy = TRUE;
      }
      else if ( query_res != db_res && db_res != ' ' )
      {
        // case 2
        copy = TRUE;
        convert_residue = TRUE;
      }
      else if ( db_res == ' ' ) {
        // case 4
        res_num_to_print++;  // skip it
        continue;
      }
    }

    // move the file pointer
    char sLine[MAX_LINE_LEN];

    /* Iterate through the file while we're on the same residue */
    while( TRUE )
    {
      if ( readNewLine ) {
        /* only read a line if we don't already have one
           We should only read a line if we've nulled it out, i.e. at the beginning
            Or when we've explicitly said we don't need it
        */
        if ( fgets(sLine,MAX_LINE_LEN,pdbfilein) == NULL )
        {
          //show_err(1, "CopyPDBFile", "Unexpected end of PDB FILE!");
          //return 0;
          endOfFile = TRUE;
          break;
        }
      }
      
      char temp[MAX_LINE_LEN];
      sscanf (sLine,"%s",temp);

      // if this is the end of the file, or not an atom line, 
      if (strncmp (temp,"TER",3)==0)
      {
        endOfFile = TRUE;
        break;
      }
      else if (strncmp(temp,"ATOM",4)!=0)
      {
        readNewLine = TRUE;
        continue;
      }

      // get the variables from this line
      
      char atom_name[MAX_NAME_LEN];
      char aa_name[MAX_NAME_LEN];
      float x, y, z;
      int res_num_from_file;
      char line_msg[MAX_LINE_LEN];
 
      sscanf(&sLine[13],"%3s", atom_name);
      sscanf(&sLine[17],"%3s",aa_name);
      sscanf(&sLine[22], "%d",&res_num_from_file);
      sscanf(&sLine[30],"%f%f%f",&x,&y,&z);

/*
      if ( strcmp(atom_name, "CB" ) == 0 ) 
      {
        For some reason, the original implementation thinks this should be the last atom, 
      so stop copying the rest of this residue record 
        copy = FALSE;
        
      }
*/
      if ( res_num_from_file == 0 )
      {
        // ignore any prepended residues, ACE or FOR
        continue;
      }
      
      if ( res_num == ( res_num_from_file - first_res_num + 1 )  ) {
        // this is the residue we want
        if ( copy ) {
          // we actually do want to copy this line
          if ( convert_residue )
            convert(aa_name,query_res);

          //sprintf(line_msg, "Copied from db");
          //sprintf(line_msg, "");
          line_msg[0] = 0;
          fprintf(pdbfileout, "ATOM %6d  %-3s %s %5d    %8.3f%8.3f%8.3f  1.00  1.00    %s\n",
                          line++, atom_name, aa_name, res_num_to_print, x, y, z, line_msg);
          fflush(pdbfileout);
        }

        // we're done with this line, read another
        readNewLine = TRUE;
      }
      else {
        // we've moved residues in the file, break
        readNewLine = FALSE;
        // we aren't done with this line, it belongs to the next residue, don't read another one

        res_num++;    // since we're moving residues

        if ( query_res !=  ' ' )
          res_num_to_print++; // the printed one is moving too, only if we're not on a gap
        break;
      }
    }
  }

  fprintf (pdbfileout,"END\n");
  fflush (pdbfileout);

  // close the new file
  fclose(pdbfileout);
  return 1;
}
