/*
	file: init.c
	set up variables, read in parameter file for shifty 
*/

#include "csipdb.h"

void read_parms_file(), read_similarity_mat();
extern void show_msg(), show_err();
extern int CHECKEXIST();
extern FILE *open_file();
FILE	*sbfp;
char	sMatFname[MAX_FNAME_LEN];
char 	shifty_lib[MAX_FNAME_LEN];
int	wtMat[WT_MAT_SIZE];
int	nearMatchScore;
int	seed;
int	sortAlgorithm;
double	jumStat[JUM_TESTS];
int	jumNum[JUM_TESTS];
int	updateFreq;
int	gapPenalty;
int	gapSizePenalty;
int	penCoil;
int	penMiddle;
int	penEnd;

FILE	*get_parms_file();


/*************************************************************/
/* obtain the parameter file and read it. 
*/
void init_parms(char *parmsFname, char *shifty_home, char *seqseedb)
{
	FILE *parmsfp;

	parmsfp = (FILE *)get_parms_file(parmsFname);
   if (parmsfp == NULL)
   	exit(1);
	
	read_parms_file(parmsfp, shifty_home, seqseedb);
	
} /* init_parms */


/*************************************************************/
/* Find the parameter file for this program. 
*/
FILE *get_parms_file(char *userFname)
{
	int 	option;
	char 	aStr[MAX_LINE_LEN];
	char	parmsFile[MAX_FNAME_LEN];
	char	default_parms[MAX_FNAME_LEN];
	FILE	*fp;

	fp = fopen(userFname, "r");
	
	if (fp == NULL) {
		sprintf(default_parms, "%s/shifty.parms", shifty_lib);
		fp = fopen(default_parms, "r");
	}
       
	while (fp == NULL) {
		show_msg("\n Program cannot find the parameter file.");
		show_msg("\n    1) I wish to enter a parameter file now.");
		show_msg("\n    2) I wish to exit this program.");
		show_msg("\n");
		show_msg("\n>> ");
		fscanf(stdin, "%s", aStr);
		option = atoi(aStr);
		switch (option) {
			case 1:		show_msg("\n Enter name of the parameters file: ");
						fscanf(stdin, "%s", parmsFile);
						fp = (FILE *)open_file(parmsFile, "r");
						break;
			case 2:		return(NULL);
			default:	show_msg("\n Error, illegal menu option.\n\n");
		}
	}

	return(fp);

}	/* get_parms_file */


/*************************************************************/
/* read parameter file until we come to the next prompt. 
*/
int read_prompt(FILE *fp)
{
	int 	ch1, ch2;

	ch2 = 0;
	while (ch2 != EOF) {
		ch1 = ch2;
		ch2 = getc(fp);

		if ((ch1 == PARMS_SCH) && (ch2 == PARMS_SCH))
			return(TRUE);
	}

	return(FALSE);

}	/* read_prompt */


/*------------------------
 * Read each parameter.
 *------------------------
 */
void read_parms_file(FILE *parmsfp, char *shifty_home, char *seqseedb)
{
	FILE	*fp;
	int	i;
	char	idStr[MAX_LINE_LEN];
  char realSMatFname[MAX_FNAME_LEN];

	do {
		if (read_prompt(parmsfp) == FALSE) {
			show_err(TRUE, MODULE, "Could not find parameter ID code.");
		}
		fscanf(parmsfp, "%s", idStr);
	} while ( strcmp(idStr, MODULE) );
	     
	if ((sbfp = fopen(seqseedb, "r")) == NULL)
		show_err(TRUE, MODULE, "Could not open seqbank database.");

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%s\n", sMatFname);

  sprintf(realSMatFname, "%s%c%s", shifty_home, '/', sMatFname);

	if ((fp = fopen(realSMatFname, "r")) == NULL)
		show_err(TRUE, MODULE, "Could not open scoring matrix file.");
	read_similarity_mat(fp, wtMat);			/* similarity scoring matrix */

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &nearMatchScore);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &seed);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &sortAlgorithm);

	for (i=0; i<JUM_TESTS; i++) {
		if (read_prompt(parmsfp) == FALSE)
			show_err(TRUE, MODULE, "Parameter expected but not found.");
		fscanf(parmsfp, "%d %lf", &jumNum[i], &jumStat[i]);
	}

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &updateFreq);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &gapPenalty);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &gapSizePenalty);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &penCoil);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &penEnd);

	if (read_prompt(parmsfp) == FALSE)
		show_err(TRUE, MODULE, "Parameter expected but not found.");
	fscanf(parmsfp, "%d", &penMiddle);

}	/* read_parms_file */


/*---------------------------
 * Reads similarity matrix.
 *---------------------------
 */
void read_similarity_mat(FILE *fp, int *wtMatrix)
{
	int	j, k;
	int	score;
	int	ixWt[WT_MAT_DIM], icWt[WT_MAT_DIM];
	char	label[MAX_LINE_LEN], comment[MAX_LINE_LEN];

	fgets(comment, MAX_LINE_LEN-1, fp);

	for (j=0; j<AA_NUM; j++) {
		fscanf(fp, "%s", label);
		icWt[j] = AA_IX(label[0]);
	}

	for (j=0; j<AA_NUM; j++) {
		fscanf(fp, "%s", label);
		ixWt[j] = AA_IX(label[0]);
		for (k=0; k<=j; k++) {
			fscanf(fp, "%d", &score);
			wtMatrix[(ixWt[j] * WT_MAT_DIM) + icWt[k]] = score;
			wtMatrix[(icWt[k] * WT_MAT_DIM) + ixWt[j]] = score;
		}
	}

} /* read_similarity_mat */



/*************************************************************/
/* set the library path.
*/
void set_lib_path(char *shifty_home)
{
  strcpy(shifty_lib, shifty_home);
}   /* set_lib_path */
