

/*--------------------------------------
 *	file: io.c
 *	common io routines used in shifty 
 *--------------------------------------
 */
#include "csipdb.h"
#include "loop.h"

void read_residues(FILE *fp, SB_REC *rSb, int flag);
void mask(char *pRes, char *sRes);
int gettype();
extern void show_msg(), show_err();
extern FILE *open_file();
/*----------------------------------------
 * ask user how many alignments/searches
 * to keep track of.
 *---------------------------------------- 
 */
int get_top_num(char *fStr)
{
  int	flag = FALSE, topNum;
  char	aStr[MAX_LINE_LEN];
  char	sVal[MAX_LINE_LEN];
	
  sprintf(sVal, "\n This program keeps track of the top 'x' %s.", fStr);
  show_msg(sVal);

  do {
    sprintf(sVal, "\n Enter a value for 'x' where 0 < x < %3d.",
	    MAX_RECORDS);
    show_msg(sVal);
    show_msg(KEYBD_PROMPT);
    fscanf(stdin, "%s", aStr);
    topNum = atoi(aStr);
    if ( (topNum > 0) && (topNum < MAX_RECORDS) )
      flag = TRUE;
    else
      show_msg("\n Error, illegal value entered.\n ");

  } while (flag == FALSE);

  return(topNum);

} /* get_top_num */


/*---------------------
 * setup output file 
 *---------------------
 */
void get_output_file(char *resultsFname)
{
  FILE *fp;

  fp = fopen(resultsFname, "w");
  while (fp == NULL) {
    printf("\n Can't write to %s.\n", resultsFname);
    show_msg("\n Enter output filename.\n");
    show_msg(KEYBD_PROMPT);
    fscanf(stdin, "%s", resultsFname);
    fp = (FILE *)open_file(resultsFname, "w");
  }

  fclose(fp);

} /* get_output_file */


/*-------------------------------------------
 * Reads in seqseebank database of proteins
 *-------------------------------------------
 */
int read_seq_sb(FILE *fp, SB_REC *rSb, int flag)
{
  int 	 len, fieldType, a;
  char	ch, sid[MAX_LINE_LEN], line[MAX_LINE_LEN];
  char	sVal[MAX_LINE_LEN];

  ch = a = getc(fp);
  while ((a != EOF) && (ch != SB_SCH)) {
    ch = a = getc(fp);
  }

  if (a == EOF) {
    rSb->len = 0;
    return(EOF);
  }
	
  fgets(rSb->title, MAX_TITLE_LEN-1, fp);

  fieldType = FLD_REF;

  fpos_t pos;
  while (fieldType != FLD_NONE) 
  {
    fgetpos(fp, &pos);  // store where we were before reading the line, in case we need to come back to it (stored in &pos)
    fgets(line, MAX_LINE_LEN-1, fp);
    fieldType = gettype(line);
    switch (fieldType)
    {
      case FLD_REF:
        break;
      case FLD_SBID:
        sscanf(line, "%*s%*s%s", rSb->sbid);
        break;
      case FLD_CLASS:
        sscanf(line, "%*s%*s%s", sid);
        if (strcmp(sid, B_CFORM) == 0)
          rSb->fold = B_IFORM;
        else if (strcmp(sid, M_CFORM) == 0)
          rSb->fold = M_IFORM;
        else if (strcmp(sid, A_CFORM) == 0)
          rSb->fold = A_IFORM;
        else if (strcmp(sid, AB_CFORM) == 0)
          rSb->fold = AB_IFORM;
        else if (strcmp(sid, CA_CFORM) == 0)
          rSb->fold = CA_IFORM;
        else if (strcmp(sid, CB_CFORM) == 0)
          rSb->fold = CB_IFORM;
        else if (strcmp(sid, CM_CFORM) == 0)
          rSb->fold = CM_IFORM;
        else
          {
//            sprintf(sVal, "\n Protein: %s", rSb->title);
//            show_msg(sVal);
            sprintf(sVal, "Error, illegal form in database.\n");
            show_err(TRUE, "read_seq_sb", sVal);
          }
        break;
      case FLD_BHID:
        sscanf(line, "%*s%*s%s", rSb->bhid);
        break;
      case FLD_PIRID:
        sscanf(line, "%*s%*s%s", rSb->pirid);
        break;
      case FLD_SPID:
        sscanf(line, "%*s%*s%s", rSb->spid);
        break;
      case FLD_RESOL:
        sscanf(line, "%*s%*s%lf", &rSb->resol);
        break;
      case FLD_RFACT:
        sscanf(line, "%*s%*s%lf", &rSb->rfact);
        break;
      case FLD_RESNUM:
        sscanf(line, "%*s%*s%d", &rSb->len);
        break;
      case FLD_NONE:
          // blank line
        break;
      default:
        fsetpos(fp, &pos);
//        sprintf(sVal, "\n Protein: %s", rSb->title);
//        show_msg(sVal);
        sprintf(sVal, "Error, illegal form in database.\n");
        show_msg(sVal);
        return 0;
    }
  }
 
  len = rSb->len;

  if (!(rSb->pRes = (char *)realloc(rSb->pRes,len+1)))
  {
    printf ("Couldn't realloc memory to rSb->pRes\n");
    exit(-1);
  }
  rSb->sRes = (char *)realloc(rSb->sRes,len+1);
  if (rSb->sRes==0)
  {
    printf ("Not enough memory to allocated to rSb->sRes\n");
    exit (-1);
  }
  if (!(rSb->aaRes = (char *)realloc(rSb->aaRes,len+1)))
  {
    printf ("Not enough memory to allocated to rSb->sRes\n");
    exit (-1);
  }

  read_residues(fp, rSb, flag);

  return(TRUE);

}	/* read_seq_sb */


/*---------------------------------------------------
 * Read sequence from seqseebank one line at a time 
 *---------------------------------------------------
 */
int get_seq(FILE *fp, char *seq)
{
  int ch, count = 0;

  while ( (ch = fgetc(fp)) != '\n' ) {
    if (ch != ' ') {
      *(seq++) = ch;
      ++count;
    }
  }
  
  *seq = '\0';

  return(count);  

} /* get_seq */



void mask(char *pRes, char *sRes)
{
  while (*sRes != '\0') {
    if (*sRes == 'B')
      *pRes = 'E';
    else
      *pRes = *sRes;

    pRes++;
    sRes++;
  }
  *pRes = '\0';

}/* mask */



/*----------------------------------
 * Reads residues from seqsee bank    
 *----------------------------------
 */
void read_residues(FILE *fp, SB_REC *rSb, int flag)
{
  int  lc = 1, wc = 0, ch;
  char ps[MAX_LINE_LEN+1], cs[MAX_LINE_LEN+1];

  while ((ch = fgetc(fp)) != PARMS_SCH && ch != '\n' && ch != EOF)  
    {
       
      ungetc(ch, fp);
       
      wc += get_seq(fp, ps);
      if (lc == 1)
	strncpy(rSb->pRes, ps, strlen(ps)+1);
      else
	strcat(rSb->pRes, ps);
       
      get_seq(fp, cs);
      if (lc == 1)
	strncpy(rSb->sRes, cs, strlen(cs)+1);		
      else
	strcat(rSb->sRes, cs);

      fgetc(fp);

      lc++;
    }

  ungetc(ch, fp);
  strcpy (rSb->aaRes,rSb->pRes);   
  if (flag == 1)
    mask(rSb->pRes, rSb->sRes);

#ifdef DEBUG
  if (wc != rSb->len)
    {
      printf("Error reading sequence %s: rSb->len = %d\trlen = %d\n", rSb->bhid, rSb->len, wc);
    }
#endif
  
} /* read_residues */



/*-----------------------------------------------
 * given an input line, to determine which type
 * of field is read. 
 *-----------------------------------------------
 */
int gettype(char *line)
{
  if ( !strncmp(line, SB_REF, FIELD_LEN_SB) ) 
    return(FLD_REF);
  else if ( !strncmp(line, SB_ID, FIELD_LEN_SB) )
    return(FLD_SBID);
  else if ( !strncmp(line, SB_BHID, FIELD_LEN_SB) )
    return(FLD_BHID);
  else if ( !strncmp(line, SB_CLASS, FIELD_LEN_SB) )
    return(FLD_CLASS);
  else if ( !strncmp(line, SB_PIRID, FIELD_LEN_SB) )
    return(FLD_PIRID);
  else if ( !strncmp(line, SB_SPID, FIELD_LEN_SB) )
    return(FLD_SPID);
  else if ( !strncmp(line, SB_RESOL, FIELD_LEN_SB) )
    return(FLD_RESOL);
  else if ( !strncmp(line, SB_RFACT, FIELD_LEN_SB) )
    return(FLD_RFACT);
  else if ( !strncmp(line, SB_RESNUM, FIELD_LEN_SB) )
    return(FLD_RESNUM);
  else if ( strncmp(line, "\n", 1) == 0 )
    return(FLD_NONE);
  else
    return(FLD_INVALID);
} /* gettype */
