/***************************************************************
 * File Name: loop.h
 * Purpose: This file contains routines required to perform
 *   csipdb gap finding and filling.
 *
 * Author: Philip S Luk ~ philipluk@bigfoot.com
 *
 * Date Written : February 01, 2001
 * Last Modified: July 05, 2001
 *
 * Written for Dr. D Wishart, Faculty orf Pharmacy &
 * Pharmaceutical Sciences at the University of Alberta ~ 2001
 * This file is provided as-is.
 **************************************************************/
#define LOOPPDB_IND   "looppdb.ind"   /* Name of the looppdb database index */
#define LOOPPDB_DB    "looppdb.new"   /* Name of the looppdb database */
#define ABCVALUES     "abcvalues.nfo"   /* abc file name for rotate function */
#define PQRVALUES     "pqrvalues.nfo"
#define PREROTCOORDS  "prerotcoords.nfo"
#define POSTROTCOORDS "postrotcoords.nfo"
#define ALBERTPDBTMP  "pdb.tmp"         /* Albert's tmp info store file */
#define PRE_POST_LOOP_PDB "pre_fist_loop.pdb" /*loop file*/
#define PRE_POST_PDB "pre_post.pdb" /*loop file*/
#define LOOP_ROTATED_PDB "loop_rotated.pdb" /*loop file*/
#define RESULT_FROM_SUP "result_from_sup"
#define SIMPLEX_OUTPUT "simplex_output"
#define SIZE_OF_LOOP_DISTMATRIX       3
#define MIN_CA_CA_DIST             3.00
#define COIL_DAT                     "lib/loop_coord.db"
#define COIL_IND                     "lib/loop_coord.ind" 
#define LOOP_TEMP_FILE	     "._loop_temp_file"
#define LOOP_PDB_LINE_READ_FORMAT         "%d %s %s %s %c %f %f %f"
#define LOOP_HEAD_READ_FORMAT_FOR_IND     ">%d:%d:%3s:%f:%ld\n"
/**TYPE*DEFINITIONS**/

typedef struct {
  double x;
  double y;
  double z;
} POINT_t;

typedef struct CAND{
  int loop_num;
  double score;
}CAND;

typedef struct {
  double Nx;  // N atom
  double Ny;
  double Nz;
  double Cx;  // C atom
  double Cy;
  double Cz;
  double Ax;	// CA atom
  double Ay;
  double Az;
  double Ox;	// O atom
  double Oy;
  double Oz;
  
  //added by xiaoli to record the coord for CB atom
  double Bx;
  double By;
  double Bz;
} SVECTOR;

typedef struct {
  char  atomName[4];
  int   resNum;
  float x, y, z;
} ATOM_INFO_REC;

/*Type Definitions added by Moustafa*/
typedef struct {
  unsigned int   loop_num;
  long int       file_pos;
  float          score;
  float          dist_matrix[SIZE_OF_LOOP_DISTMATRIX][SIZE_OF_LOOP_DISTMATRIX];
  float fist_dist, abs_diff_between_lengths;
  float rmsd_before_ga, rmsd_after_ga;
  float constraint_score_after_ga;
  char *ga_minimized_loop;
  float cn_dist, nc_dist;
}LOOP_PROP;

 typedef struct {
    int    res_no;
    char   res[5];
    char   atom[5];
    char   class[5];
    char   type;
    POINT  pt;
  } LOOP_b;	

typedef struct {
  int loop_num;
  long int file_pos;
  //float rmsd_before_ga;
  //float rmsd_after_ga;
  //float loop_score_after_ga;
  int no_of_conflicts;
  float dist_between_atoms[100];
}E_ARRAY;


typedef struct {
    int    num;
    int    len;
    char   type[4];
    float fist_dist;
    //double elbow_dist;
    //double fist_dist;
    long int file_pos;
  } LOOP_h_for_ind;		/* PDB loop index file header */
/**END*TYPE*DEFINITIONS**/

/*----------------------------------------------------
 * Find gaps in query sequence and data base sequence
 * from the alignment record given and stores gap
 * information in the gap info record pass in. match
 * rank is the ranking of the alignment in scoring.
 * Returns the number of gaps found in total.
 *---------------------------------------------------- */
int find_gaps(ALIGN_REC *align, char *pdbfile_name);


void set_loop_classification(ALIGN_REC * align, char before, char after, int gap);
int envelops_other_gap(ALIGN_REC * align, int gap);
void setup_query_gap(ALIGN_REC * align,int gapCount, int i);
void setup_database_gap(ALIGN_REC * align, int gapCount, int i);
void gap_continuation(ALIGN_REC * align, int gapcount, int i);
/*----------------------------------------------------
 * Reads the PDB file and calculates the fist and
 * elbow distances.
 *---------------------------------------------------- */
int calc_gaps_fist_elbow_dist(ALIGN_REC *align, char* pdbfile_name);

/*----------------------------------------------------
 * Dumps the gap information given from the alignment
 * passed in to the file name given 
 *---------------------------------------------------- */
int dump_gaps_to_file(ALIGN_REC *align, char* filename);

/*----------------------------------------------------------------
 * Get the closest previous secondary structure index and following
 * the gap for fist dist calcuations
 *---------------------------------------------------------------- */
int calc_prev_post_fist_struct_index(ALIGN_REC *align, int gap);

/*----------------------------------------------------------------
 * Get the closest previous secondary structure index and following
 * the gap for elbow dist calcuations
 *---------------------------------------------------------------- */
int calc_prev_post_elbow_struct_index(ALIGN_REC *align, int gap);

/*-----------------------------------------------
 * Parses looppdb database and finds appropriate
 * matches for all gaps in alignment sequence.
 * match * rank is the ranking of the alignment
 * in scoring.
 *----------------------------------------------- */
int fill_gaps(ALIGN_REC *align, char *pdbfile_name, char *shifty_home,
    char *pdbdir, char *simplexFilename);

/*-----------------------------------------------
 * Parses the looppdb database for loop ranking
 * information
 *----------------------------------------------- */
int get_candidates(FILE *fp, GAP_INFO_REC gapinfo);

/*-----------------------------------------------
 * Parses a data header line from the looppdb file
 * for the following fields.
 *----------------------------------------------- */
int parse_data_line(char *buffer, int *num, int *len, char *type, 
                    float *elbow, float *fist);

/*-----------------------------------------------
 * Ranks loops from looppdb to in order of 
 * suitability
 *----------------------------------------------- */
double rank_score(GAP_INFO_REC gapinfo, int len, char *type, float elbow, float fist);

/*-----------------------------------------------
 * Replaces the requested gap with the given
 * candidate based on the info found in the alignment
 * record.
 *----------------------------------------------- */
int insert_into_pdbfile_1(ALIGN_REC *align, int gap, int candidate,
    char *src_filename, char *tar_filename,
    char *pdbdir, char *shifty_home, char *simplexFilename);

/*-----------------------------------------------
 * Replaces line in PDB file.
 *----------------------------------------------- */
int pdb_line_replace(char *buffer, FILE *target);

/*----------------------------------------------------------------
 * Finds where the query string begins with respect to the data
 * base string. Important info for later PDB loop insertion.
 *---------------------------------------------------------------- */
int set_query_database_offset(ALIGN_REC *align);

/*-----------------------------------------------
 * Goes and fetches the (x,y,z) coordinates from
 * the given loop at the given residue of the
 * given atom from loop pdb database.
 *----------------------------------------------- */
int get_loop_coords(int loop_no, int res_no, char* atom, POINT_t* coordinate,
    char* res, char *shifty_home);

int get_pepmake_coords( int res_no, char* atom, POINT_t* coordinate);
/*----------------------------------------------------------------
 * Calculates the length between two points  based on the formula:
 * Distance = square_root((x2-x1)^2 + (y2-y1)^2 + (z2-z1)^2)
 *---------------------------------------------------------------- */
double calc_len(SVECTOR beg, SVECTOR end);
float calc_dist(SVECTOR beg, SVECTOR end);

/*----------------------------------------------------------------
 * Calculates the midpoint between two points based on the formula:
 * Midpoint :  x = (x1+x2)/2 y = (y1+y2)/2 z = (z1+z2)/2
 *---------------------------------------------------------------- */
void calc_midpoint(POINT_t a, POINT_t b, POINT_t *mid);

/*----------------------------------------------------------------
 * Reads one atom from the given pdb file, excluding H atoms.
 *---------------------------------------------------------------- */
int read_pdb(FILE *pdbfp, ATOM_INFO_REC *atom);

/*-----------------------------------------------
 * Gets the ABC/PQR coordinates for rotation.
 * Coordinates are based on fist-fist atoms.
 * PARAMS:
 * -src_filename is where to get current PDB file.
 * -candidate is the loop to insert from loopdb
 * -start/finish indices determine which piece from
 *  current PDB file is to be replaced by loop.
 * -POINT a,b,c,p,q,r are calculated based on the
 *  above information.
 *----------------------------------------------- */
void go_get_me_abc_pqr(char* src_filename, char* base_pdbfile,
                       int candidate,
                       int start_index, int finish_index,
                       int loop_len, int gap_size, 
                       int gap_type, int fist_index,
                       POINT_t* a, POINT_t* b, POINT_t* c,
                       POINT_t* p, POINT_t* q, POINT_t* r,
                       char *pdbdir);
void go_get_me_abc(char* src_filename, char* base_pdbfile,
                       int start_index, int finish_index,
                       int loop_len, int gap_size, 
                       int gap_type, int fist_index,
                       POINT_t* a, POINT_t* b, POINT_t* c);

/*----------------------------------------------------------------
 * Get the closest previous secondary structure index and following
 * the gap for fist dist calcuations. Also sets the class of the loop.
 *---------------------------------------------------------------- */
int set_prev_post_fist_struct_index(ALIGN_REC *align, int gap);

/*----------------------------------------------------------------
 * Get the closest previous secondary structure index and following
 * the gap for elbow dist calcuations
 *---------------------------------------------------------------- */
int set_prev_post_elbow_struct_index(ALIGN_REC *align, int gap);

/*----------------------------------------------------------------
 * Finds and fills gaps.
 *---------------------------------------------------------------- */
void do_gap_findin_and_fillin(ALIGN_REC *topAligns, int topAlignNum, PROGRAM_PARAMS params);

void refindgap(ALIGN_REC *align,int gap,char *pdb_filename);

float matrix_distance( float matrix1[4][4], float matrix2[4][4]);
int compare_arrays(const void *p, const void *q);
int compare_number(const void *p, const void *q);
int converttoint();
int insert_beg_into_pdbfile();
int insert_end_into_pdbfile();
int convert();
void get_pre_post_loop_pdbfile(int candidate,char *pdbdir, char *shifty_home);
  
int  get_pre_post_pdb(ALIGN_REC *align,int gap, char *src_filename);

void  insert_loop_pdb(char *src_filename, FILE *tarfp,char *superpose, int start_index, int finish_index, ALIGN_REC *align,int gap);

int get_coords(int count,int loop_begin, int residue_num, char *res, char *superpose_result, FILE *tmp, char *src_filename);

int simplex_needed(ALIGN_REC *align, int gap, char *loop_pdb, char *pre_post_pdb, double rmsd,double *cn, double *nc);

void do_GA_stuff (char *pdbtmp);


void calc_dists(SVECTOR beg, SVECTOR end, double dists[]);
double cal_coefficient(double array1[], double array2[]);
double rank_score1(GAP_INFO_REC gapinfo, int len, char *type,double dists[]);
void parse_data_line1(char *buffer, int *num, int *len, char *type, double dists[]);
double cal_sum_dif(double array1[], double array2[]);
double rank_score2(GAP_INFO_REC gapinfo, int len, char *type,double dists[]);
int get_candidates1(FILE *fp, GAP_INFO_REC gapinfo,CAND candidates[]);//original version
//CAND* get_candidates1(FILE* fp, GAP_INFO_REC gapinfo, int*); //added by Raj
int get_candidates2(FILE *fp, GAP_INFO_REC gapinfo);

int compare_double (const void *v1, const void *v2);
int insert_into_pdbfile(ALIGN_REC *align, int gap, int candidate,
                        char *src_filename, char *tar_filename,
                        char *pdbdir, char *shifty_home, double *cn_nc_dis_sum, char *simpexFilename);

/*Updated Function Declerations July 18, 2006*/
     
/***********************************************************
   * Searches the given loop file in PDB format for the
   * primary rotation reference point. Guarantee not to change
   * the contents of the file 'loop'.
   ***********************************************************/                   
int set_prim_loop_ref_pt (const char * loop, POINT * pt);

/***********************************************************
   * Searches the given loop file in PDB format for the
   * secondary rotation reference point. Guarantee not to change
   * the contents of the file 'loop'.
   ***********************************************************/
int set_sec_loop_ref_pt (const char * loop, POINT * pt) ;


/***********************************************************	
 * compares the scores of the distance matrices  
 ***********************************************************/ 
int compare_scores(const void *p, const void *q);

float compare_matrices(float loop_dist[][SIZE_OF_LOOP_DISTMATRIX], float oth_strands[][SIZE_OF_LOOP_DISTMATRIX]);

//*******************************************************************************************
// Check for distance violations when the loop has been inserted..
int evaluate_distance(char *pdbfile, E_ARRAY *e_arr );

/***********************************************  
   * Forms the CA_dmatrix of the loop that is to be inserted. 
   *  The pairwise distances between the 3 beginning CA atoms and the 
   *  last 3 CA atoms are calculated and stored in the distance matrix.
   **************************************************************/
void form_CA_dmatrix_of_loop(float dist_matrix[][SIZE_OF_LOOP_DISTMATRIX], SIX_CA_POINTS six_cas[], int seg_number);

/***********************************************************
 * Fetches the given loop from the database.
 ***********************************************************/
char *go_get_me_the_loop (long int file_position, int residues, POINT *first_pt, POINT *last_pt,int no_of_fist_resds);


/***********************************************************
   * Returns true if the given line matches the header line
   * pattern of the loop database.
   ***********************************************************/
boolean is_loop_header (char * line);

//***********************************************************
//* Parses loop database line and parses for PDB line info.
// ***********************************************************/

int read_loop_database_line (char * line, LOOP_b * l);

/***********************************************************
   * Searches the loop database and finds all suitable candidates.
   * Returns -1 if no loop cannot be found.
   ***********************************************************/
LOOP_PROP* find_loop_candidates(int residues,float desired_length,SIX_CA_POINTS six_cas[],int seg_number,
				int *no_of_loops, int *found_loop,char *loop_type);


/***********************************************************
   * Parses loop header and returns loop information.
   ***********************************************************/ 
int read_loop_header_for_ind (char * header, LOOP_h_for_ind * l);

void print_loop_properties(char *loop_information, LOOP_PROP *loop_arr, int no_of_loops,float loop_length, int residues);
/*TODO: Utility functions, should be placed in own Util.h*/
  
/***********************************************************
 * Read one line from the given pdb file and stores it in 
 * the given variable. Returns the number of items read or
 * EOF if end of file is encountered.
 ***********************************************************/
int read_pdb_line (FILE *fp, PDB_LINE *l);

/***********************************************************
 * Returns true if given atom is the Nitrogen atom.
 ***********************************************************/
boolean is_N_atom (char * atom);


/***********************************************************
 * Returns true if given atom is the Carbon atom.
 ***********************************************************/
boolean is_C_atom (char * atom);


/***********************************************************
 * Returns true if given atom is the CB atom.
 ***********************************************************/
boolean is_CB_atom (char * atom);

/***********************************************************
 * Returns true if given atom is the Oxygen atom.
 ***********************************************************/	
boolean is_O_atom (char * atom);

/***********************************************************
 * Returns true if given atom is the rotation reference
 * point atom.
 ***********************************************************/
boolean is_ref_atom (char * atom);

/***********************************************************
 * Returns true if extra is HELIX
 ***********************************************************/
boolean is_helix (char * extra);

/***********************************************************
 * Returns true if extra is COIL
 ***********************************************************/
boolean is_coil (char * extra);

/***********************************************************	
 * returns the distance between two points 
***********************************************************/ 
float get_distance(POINT *pt1, POINT *pt2);

int assign_pt (POINT * a, POINT b);

/***********************************************************
   * Takes in a line from the loop pdb database and creates
   * a pdb line data structure.
   ***********************************************************/
int create_pdb_line_from_loop_line (PDB_LINE * pdb, LOOP_b loop);

/***********************************************************
 * Write one pdb line to the given pdb using data from 
 * the given variable. Returns the number of items written.
 ***********************************************************/
int write_pdb_line(FILE * fp, PDB_LINE l);



/*TODO: create error.h for error handling*/
/***********************************************************
 * Prints an error message based on the error string passed
 * in. (See error.h for complete list of pre-defined error
 * strings.
 ***********************************************************/
void err_print (char * err_string, char * err_details);

void touch_up_pdbfile (char *file, char* reformatted_file);

void printPOINT(POINT *point);
