#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "btimem.h"
#include "csipdb.h"
#include "loop.h"
#include "misc.h"

#define DEBUG 1
#define         MAX_CHE_RES     1200     /* largest number of query sequence */

#define STOP_AFTER_ALIGNMENTS 1

char            resultsFname[MAX_FNAME_LEN];

/* #define shifty_command */

/*******************************************************************\
| Function: header                                                  |
| Precondition: the header hasn't been displayed                    |
| Postcondition: the header has been displayed                      |
| Purpose: To display the header for the program                    |
\*******************************************************************/
void
header ()
{
  int counter = 0;
	
  /* Creates the program header */

  for (counter = 0; counter < 60; counter++)
    {
      printf ("*");
    }
  printf ("\n%-59c*\n", '*');
  printf ("%-7c HOMODELLER Version 2.0 (November, 2004)%13c\n", '*', '*');
  printf ("%-59c*\n", '*');
  printf ("%-6c Faculty of Pharmacy and Pharmaceutical Sciences%6c\n", '*',
          '*');
  printf ("%-19c University of Alberta%19c\n", '*', '*');
  printf ("%-59c*\n", '*');
  printf ("* %-13s %-43s*\n", "Authors:",
          "Haiyan Zhang, Albert Leung and Wei Xia");
  printf ("* %-13s %-43s*\n", "Supervisor:", "David Wishart");
  printf ("* %-13s %-43s*\n", "Purpose:",
          "To generate a PDB file for a peptide");
  printf ("* %-13s %-43s*\n", "", "given its protein sequence");
  printf ("%-59c*\n", '*');
  for (counter = 0; counter < 60; counter++)
    {
      printf ("*");
    }
  printf ("\n\n");
}

/********************************************************\
| Purpose: to see if the file has the correct format     |
|********************************************************/

int
check (char *filename)
{
  FILE *infile;

  if ((infile = fopen (filename, "r")) == NULL)
    {
      printf ("The file %s doesn't exists!\n", filename);
      return (0);
    }
  else
    {
      fclose(infile);
      return 1;
    }
}

void clean(char *file_name)
{

  char filenames[256];
  char command[256];
  sprintf (filenames, "%s", file_name);
  strcat(filenames, "1.*");
  sprintf(command, "rm -rf %s", filenames);
  printf("%s\n", command);
//  system(command);

  remove
    (PRE_POST_LOOP_PDB);
  remove
    (PRE_POST_PDB);
  remove
    (PREROTCOORDS);
  remove
    (POSTROTCOORDS);

}

int main (int argc, char *argv[])
{
  FILE *inputp = 0;
  int input = 0, counter = 0;
  const int FILENAME_CNT = 3;
  char *filename[FILENAME_CNT],	/* array of filename.  0 - chemical shift data */
  temp[MAX_COMMAND_LEN], command[MAX_COMMAND_LEN];
  PROGRAM_PARAMS params;

  /* variables initialization */
  for (counter = 0; counter < FILENAME_CNT; counter++)
  {
    filename[counter] = bti_malloc (MAX_FNAME_LEN * sizeof (char));
  }
  sprintf(params.shifty_home, "data");
  sprintf(params.phipsiFilename, "./phipsi");
  sprintf(params.pdbtmp, "pdb.tmp");
  sprintf(params.seqseedb, "seqsee.db");
  sprintf(params.pdbdir, "pdb");
  sprintf(params.tmpdir, "tmp");
  sprintf(params.simplexFilename, ".");
  params.sequenceName[0] = '\0';

  // only one argument, use it as the file name to work with, and use defaults for the rest
  if ( argc == 2 ) 
  {
    sprintf (filename[0], "%s", argv[1]);
  }
  else 
  {
    // use named arguments
    int i = 1;
    while ( i < argc )
    {
      if ( strncmp(argv[i], "-f", 2) == 0  )
        sprintf (filename[0], "%s", argv[++i]);
      else if ( strncmp(argv[i], "-t", 2) == 0 )
        sprintf (params.sequenceName, "%s", argv[++i]);
      else if ( strncmp(argv[i], "-d", 2) == 0 )
        sprintf (params.seqseedb, "%s", argv[++i]);
      i++;
    }

    // not enough params
    if (argc < 8)
    {
      printf("Usage: %s <shiftyhome> <phipsi> <pdbtmp> <seqseedb> <pdbdir> <tmpdir> <sequence> <simplexdir>\n",argv[0]);
      printf("OR: %s <filename>\n",argv[0]);
      printf("OR: %s -f <filename> -t <template> [-d <seqsee.db>]\n",argv[0]);
      printf("Using defaults...\n");
  
      #ifdef DEBUG
        // Only want default filename when we're debugging and not specifying a filename
//        if ( strlen(filename[0]) == 0 )
//          filename[0] = "test/1AAZ_Part/1AAZ";
      #endif
    }
    else
    {
      sprintf(    params.shifty_home, argv[1]);
      sprintf(    params.phipsiFilename, argv[2]);
      sprintf(    params.pdbtmp, argv[3]);
      sprintf(    params.seqseedb, argv[4]);
      sprintf(    params.pdbdir, argv[5]);
      sprintf(    params.tmpdir, argv[6]);
      sprintf(    params.simplexFilename, argv[8]);
    }
  }

#ifdef DEBUG
  /* Creates the program header */
//  header();
#endif

  if (argc >= 8)
  {
    sprintf (filename[0], "%s", argv[7]);
  }

  /* Check to see if the chemical shift data files indeed exist */
  int tryCount = 0;
  while (check (filename[0]) == 0)
    {
      printf
      ("\n\tThe file containing fasta format protein sequence is not accessible to this program.\n");
      printf
      ("\nEnter a filename containing the fasta format protein sequence.\n\n>> ");
//      scanf ("%s", filename[0]);
      tryCount++;
      if ( tryCount > 0 )
        {
          exit(1);
        }
    }
  //sprintf (remote_pdb, "perl get_remote_pdb/get_remote_pdb.pl %s",filename[0]);
  //printf("%s\n",remote_pdb );
  //system(remote_pdb);

#ifdef DEBUG
  printf("\nRunning with options: \nFile = %s \nShifty = %s\nphipsi = %s", filename[0], params.shifty_home, params.phipsiFilename);
  printf("\nPDBtmp = %s", params.pdbtmp );
#endif

  if(csipdb(filename[0], params )==0)
    {
      exit(0);
    }

  /// WOAH ! TODO!  This looks like it's reading in variables from earlier in the program from a temp file!!!
  inputp = fopen ( params.pdbtmp, "r");
  fgets (temp, 132, inputp);
  strcpy (filename[1], temp);
  filename[1][strlen (temp) - 1] = '\0';

  fgets (temp, 132, inputp);
  sscanf (temp, "%d", &input);
  fclose (inputp);

  strcpy (filename[2], filename[1]);
  strcat (filename[2], "1.pdb.side");

#ifdef WIN32

  sprintf (command, "copy \"%s\" \"%s.pdb\"", filename[2], filename[0]);
#else

  sprintf (command, "cp \"%s\" \"%s.pdb\"", filename[2], filename[0]);
#endif
  
  sprintf (temp, "%s.pdb", filename[0]);
  printf("Output file: %s\n", temp);

#ifdef DEBUG

  printf ("command: %s\n", command);
#endif

  if (system (command) != 0)
  {
    printf ("\n\tSomething went wrong while copying %s to %s.\n",filename[2], filename[0]);
    exit (-2);
  }


  // cleanup temp files
  clean(filename[1]);

  // cleanup memory
  for (counter = 0; counter < FILENAME_CNT; counter++)
  {
    free( filename[counter] );
  }

  printf("Homodeller finished successfully.\n");
  fflush(stdout);

  closeDebug();

  return 0;
}

