
/*
	file: math.c
	generic mathematical routines
*/

#include <math.h>
#include "vector.h"
extern void show_err();
VECTOR diff(VECTOR m, VECTOR n)
{
	VECTOR o;

	o.x = m.x - n.x;
	o.y = m.y - n.y;
	o.z = m.z - n.z;

	return o;

}/* diff */


VECTOR add(VECTOR m, VECTOR n)
{
	VECTOR o;

	o.x = m.x + n.x;
	o.y = m.y + n.y;
	o.z = m.z + n.z;

	return o;

}/* add */


VECTOR multiply(VECTOR m, float f)
{
	VECTOR o;

	o.x = m.x * f;
	o.y = m.y * f;
	o.z = m.z * f;

	return o;

}/* multiply */


float norm(VECTOR m)
{
	return (float)sqrt(m.x*m.x + m.y*m.y + m.z*m.z);
}/* norm */


float dot(VECTOR m, VECTOR n)
{
	return (float)(m.x*n.x + m.y*n.y + m.z*n.z);
}/* dot */


VECTOR cross(VECTOR m, VECTOR n)
{
	VECTOR o;

	o.x = m.y*n.z - m.z*n.y;
	o.y = m.z*n.x - m.x*n.z;
	o.z = m.x*n.y - m.y*n.x;

	return o;

}/* cross */

/*************************************************************/
/* Calculate test stat given data array and mean. 
*/
double calc_test_stat(yData, nData, mean)
int		*yData, nData;
double	mean;
{
	register int	i, ySum, ySum2;
	double			yBar, sDev, tStat;

	if (nData < 2) {
		show_err("math.lib.c", "Error in calc_mean_stddev, nData < 2.");
	}

	ySum = 0;
	ySum2 = 0;
	for (i=0; i<nData; i++) {
		ySum += yData[i];
		ySum2 += yData[i] * yData[i];
	}

	yBar = ySum / (double)nData;
	sDev = sqrt((nData*ySum2 - ySum*ySum) / (double)(nData * (nData-1)));
	tStat = (mean - yBar) / sDev; 

	return(tStat);

}	/* calc_test_stat */
