
/*
	file: misc.c
	purpose: low level utility routines for shifty.
*/

#include "btimem.h"
#include "csipdb.h"
#include "misc.h"
#include <dirent.h>
#include <ctype.h>

// static char* PDBHTTPUrl = "http://www.rcsb.org/pdb/cgi/export.cgi/%s.pdb?job=download;pdbId=%s;format=PDB;pre=1&compression=None";

static char* PDBFTPUrl = "ftp://ftp.rcsb.org/pub/pdb/data/structures/all/pdb/";
FILE* DEBUG_FILE = NULL;

void toLowercase(char* s);
void toUppercase(char* s);

/*************************************************************/
/* write a message to the screen.
*/
void show_msg(char *msg)
{
#ifdef DEBUG
  fprintf(stdout, msg);
  fflush(stdout);
#endif
}	/* show_msg */


/*************************************************************/
/* generic error handler
*/
void show_err(dieFlag, sModule, msg)
int		dieFlag;
char	*sModule, *msg;
{

  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  if (dieFlag == TRUE)
    fprintf(stdout, " Fatal Error Detected!\n");
  else
    fprintf(stdout, " Warning, Error Detected!\n");
  fprintf(stdout, "\n");
  fprintf(stdout, " Module.: %s\n", sModule);
  fprintf(stdout, " Message: %s\n", msg);
  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  if (dieFlag == TRUE)
    exit(1);

}	/* show_err */


/*************************************************************/
/* out of memory error handler
*/
void show_err_mem(dieFlag, sModule, routine, id)
int		dieFlag;
char	*sModule, *routine, *id;
{

  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  fprintf(stdout, " Error: Out of memory!\n");
  fprintf(stdout, "\n");
  fprintf(stdout, " Module.: %s\n", sModule);
  fprintf(stdout, " Routine: %s\n", routine);
  fprintf(stdout, " Protein: %s\n", id);
  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  if (dieFlag == TRUE)
    exit(1);

}	/* show_err_mem */


/*************************************************************/
/* generic file opening routine
*/
FILE *open_file(char *fileName, char *fileOp)
{
  FILE 	*fp;
  char	sVal[MAX_LINE_LEN];

  fp = fopen(fileName, fileOp);
  if (fp == NULL)
  {
    sprintf(sVal, "\nError, cannot open file '%s'\n\n", fileName);
    show_msg(sVal);
  }

  return(fp);
}	/* open_file */


/*************************************************************/
/* malloc with error check
*/
char *emalloc(unsigned size, char *msg)
{
  static char *sp;

  if ((sp = (char *)bti_malloc(size)) != NULL)
    {
      memset(sp, 0, size);
      return(sp);
    }
  memory_error(msg);
  return NULL;
} /* emalloc */


/*************************************************************/
/* calloc with error check
*/
int *ecalloc(len, size, msg)
unsigned len, size;
char *msg;
{
  static int	*sp;

  if ((sp = (int *)bti_malloc(len * size)) != NULL)
    return(sp);
  memory_error(msg);
  return NULL;
}	/* ecalloc */


/*************************************************************/
/* prints out of memory error message and aborts.
*/
void memory_error(msg)
char *msg;
{

  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  fprintf(stdout, " Fatal Error: Out of memory...aborting\n");
  fprintf(stdout, " Module: %s\n", msg);
  fprintf(stdout, "\n");
  fprintf(stdout, "**************************************************\n");
  fprintf(stdout, "\n");
  exit(1);

}	/* memory_error */


/*************************************************************/
/* converts a string of characters 'inStr' to upper case letters.
*/
void to_upper(inStr, outStr)
char *inStr, *outStr;
{
  register int k, len;

  len = strlen(inStr);
  for (k=0; k<=len; k++)
    outStr[k] = AA_UC(inStr[k]);

}	/* to_upper */


/*************************************************************/
/* utility that subtracts 'A' from a string of characters.
*/
void chars_to_nums(len, inStr, outNums)
int len;
char *inStr;
int *outNums;
{
  register int k, ix;

  for (k=0; k<len; k++)
    {
      ix = AA_IX(inStr[k]);
      if ((ix >= 0) && (ix < CODES))
        outNums[k] = ix;
      else
        outNums[k] = CODES;
    }

}	/* chars_to_nums */


/*************************************************************/
/* utility that returns number of identical matches in 2 strings.
*/
int string_match_num(str1, str2, len1, len2)
char *str1, *str2;
int	len1, len2;
{
  register int k, minimum, ident;

  ident = 0;
  minimum = len1<len2 ? len1 : len2;

  for (k=0; k<minimum; k++)
    {
      if (*str1++ == *str2++)
        ident++;
      /*  else {
          if(*str1=='C' && *str2==' ')
             ident++;		      
       } */
    }


  return(ident);

}	/* string_match_num */


/*************************************************************/
/* Randomize the entries in an integer array.
*/
int jumble_array(arr, len, seed)
register int	*arr;
register int	len, seed;
{
  register int 	i, ix;
  register int 	temp;

  for (i=0; i<len; i++)
    {
      seed = RAND(seed);
      ix = seed % len;
      temp = arr[i];			/* do swap */
      arr[i] = arr[ix];
      arr[ix] = temp;
    }

  return(seed);

}	/* jumble_array */

int fileExists(char* fileName)
{
  FILE 	*fp;
  fp = fopen(fileName, "r");

  int rv = fp != NULL;
  if ( fp )
    fclose(fp);
  return(rv);
}

char* convertToPDBID(char* fileName) 
{
  char* pos;
  
  //remove "pdb/"
  if ( (pos = strstr(fileName, "pdb/" )) != NULL ) {
    fileName = pos + strlen("pdb/");
  }

  // remove ".pdb"
  if ( (pos = strstr(fileName, ".pdb" )) != NULL ) {
    *pos = 0;
  }
  
  // if its 5 letters long, cut it to 4
  if ( strlen( fileName ) > 4 )
    fileName[4] = 0;
   
  toLowercase( fileName );
  return  fileName;
}

void toLowercase(char* s) {
  for( int i=0; i<strlen(s); i++)
    s[i] = tolower(s[i]);
}
void toUppercase(char* s) {
  for( int i=0; i<strlen(s); i++)
    s[i] = toupper(s[i]);
}

void assert(int assertion, char* msg) {
  if ( ! assertion )
  {
    printf("Assertion Failed: %s\n", msg);
  }
}
/**
    Downloads a specified PDB ID from RCS PDB
**/
void downloadPDBFile(char* fileName)
{
    
  char tmpFileName[1024];
  char* mkdir = "mkdir pdb";
  
  // make sure pdb directory exists
  DIR* fd;
  if ( (fd = opendir("pdb")) == NULL)
    system(mkdir);
  else
    closedir(fd);
      
  // so we don't modify the original filename var
  strcpy( tmpFileName, fileName );

  char* pdbID = convertToPDBID( tmpFileName );
  
  // Download the file
  char tmp[1024];
  system("echo bin > cmd");
  system("echo lcd pdb > cmd");
  sprintf(tmp, "echo get pdb%s.ent.Z >> cmd", pdbID);
  system(tmp);
  system("echo quit >> cmd");
  
  printf("Downloading PDB file %s...\n", pdbID);
  sprintf(tmp, "ftp %s < cmd > /dev/null", PDBFTPUrl);
#ifdef DEBUG
  printf("%s", tmp);
#endif
  system(tmp);
  
  // uncompress it
  sprintf(tmp, "uncompress -f pdb/pdb%s.ent.Z", pdbID);
  int ret = system(tmp);
  if ( ret != 0 ) {
    sprintf(tmp, "gunzip -f pdb/pdb%s.ent.Z", pdbID);
    ret = system(tmp);
  }

  if ( ret != 0 )
  {
    // try wget
    sprintf(tmp, "wget %s/pdb%s.ent.Z", PDBFTPUrl, pdbID);
    system(tmp);
    sprintf(tmp, "mv pdb%s.ent.Z pdb/", pdbID);
    system(tmp);

    // uncompress it
    sprintf(tmp, "uncompress -f pdb/pdb%s.ent.Z", pdbID);
    int ret = system(tmp);
    if ( ret != 0 ) {
      sprintf(tmp, "gunzip -f pdb/pdb%s.ent.Z", pdbID);
      ret = system(tmp);
    }

  }
  
  char pdbID_uppercase[256]; 
  strcpy( pdbID_uppercase, pdbID);
  toUppercase(pdbID_uppercase);
  
  // move it
  sprintf(tmp, "mv pdb/pdb%s.ent pdb/%s.pdb", pdbID, pdbID_uppercase);
  system(tmp);

  // copy it to the original filename we were looking for
  sprintf(tmp, "cp pdb/%s.pdb %s", pdbID_uppercase, fileName);
  system(tmp);
  
  //sprintf(fileName, "pdb/%s.pdb", pdbID_uppercase);
}

void initDebug() 
{
  DEBUG_FILE = open_file("debug", "a");
  writeDebug( "Opening for writing");
}

void writeDebug(char* msg) 
{
  fprintf(DEBUG_FILE, "%s\n", msg);
  fflush(DEBUG_FILE);
}

void closeDebug() 
{
  if ( DEBUG_FILE )
    fclose(DEBUG_FILE);
}
