
/*
	file: calc.c
	contains calculation routines for phipsi
*/
#include <math.h>
#include "phipsi.h"


/* globals */
char	expatm[NUM_ATM_USED][MAX_LINE_LEN];

/* procedures */
float	torsion(), dot(), angle();
void	init_phipsi(), vecdif(), cross(), print_run(), print_angles();
int dischk();
int calc_phipsi(),read_atom();
void read_write_header();
/*************************************************************/
/* Main calculation loop
*/
void do_phipsi()
{

	/*print_run(outfp);
        */
	read_write_header();

#ifdef DEBUG
	fprintf(stdout, "   Calculating phipsi angles...\n");
#endif
	calc_phipsi();
#ifdef DEBUG
	fprintf(outfp, "\n");
#endif
	fclose(outfp);

#ifdef DEBUG
	fprintf(stdout, "   Done phipsi program.\n");
	fprintf(stdout, "  \n");
	fprintf(stdout, " ------------------------------------------------------------------\n");
#endif

}	/* do_phipsi */


/*************************************************************/
/* Calculate phipsi angles
*/
int calc_phipsi()
{
int 		type, i;
int			atm_ptr;						/* index for atom array */
int			angle_ptr;						/* index for phipsi array */
float		phipsi[SIZE_PHIPSI_ARRAY];
float		chi = 0.0;
ATOM_REC	atomArray[4];
ATOM_REC	atomCB, atomCOG;
int			read_into = 3;
int			resNum;

	init_phipsi();

    for (i=0; i<SIZE_PHIPSI_ARRAY; i++)
        phipsi[i] = 360.0;
	atomCOG.resNum = atomCB.resNum = VOID;

	/* start with the psi angle because phi cannot be calculated
	 * on the very first pass
	 */
	angle_ptr = 1;

	atm_ptr = 0;


	/* prime the rotating atom array with the first four atoms */
    type = read_atom(&atomArray[0], &atomCB, &atomCOG);
    type = read_atom(&atomArray[1], &atomCB, &atomCOG);
    type = read_atom(&atomArray[2], &atomCB, &atomCOG);
    type = read_atom(&atomArray[3], &atomCB, &atomCOG);
	resNum = atomArray[3].resNum;

	while ( (type != END) && (allChains || 
		(atomArray[read_into].resNum >= resNum)) ) {

	    /* calculate torsion angle */
   		phipsi[angle_ptr]=torsion(atomArray[atm_ptr], 
		atomArray[PLUS(atm_ptr,1)], atomArray[PLUS(atm_ptr,2)], 
		atomArray[PLUS(atm_ptr,3)]);

	  	if (0 == strcmp(atomArray[atm_ptr].atomName, "N")) {
			if ( (atomCOG.resNum != VOID) && (atomCB.resNum != VOID) )
				chi = torsion(atomArray[atm_ptr], 
				atomArray[PLUS(atm_ptr,1)], atomCB, atomCOG);
			else {
				chi = VOID;
			}	
			atomCOG.resNum = atomCB.resNum = VOID;
		}

	  	if (angle_ptr == OMEGA) { 
			print_angles(phipsi, chi, atomArray[atm_ptr].resName, 
				atomArray[atm_ptr].resNum);
			for (i=0; i<3; i++)
		   		phipsi[i] = 360.0;
		}

   	    /* rotate the pointers */
		angle_ptr = (angle_ptr + 1) % 3;
		atm_ptr = (atm_ptr + 1) % 4;

		/* plus(atm_ptr,3) is the same as atm_ptr - 1 */
		resNum = atomArray[read_into].resNum;
		read_into = PLUS(atm_ptr, 3);

    	type = read_atom(&atomArray[read_into], &atomCB, &atomCOG);

	}

	if ( (atomCOG.resNum != VOID) && (atomCB.resNum != VOID) )
		chi = torsion(atomArray[atm_ptr], 
		atomArray[PLUS(atm_ptr,1)], atomCB, atomCOG);
	else chi = VOID;

	print_angles(phipsi, chi, atomArray[atm_ptr].resName, 
	atomArray[atm_ptr].resNum);

	/* if there is a residue left which has not been printed */
	if ( atomArray[atm_ptr].resNum != atomArray[PLUS(atm_ptr,2)].resNum) {
		for (i=0; i<3; i++)
	   		phipsi[i] = 360.0;
		chi = 360.0;
		print_angles(phipsi, chi, atomArray[PLUS(atm_ptr,2)].resName, 
					 atomArray[PLUS(atm_ptr,2)].resNum);
	}	
	return 1;
}	/* calc_phipsi */


/*************************************************************/
/* initialisation 
*/
void init_phipsi()
{
	strcpy(expatm[0], "N");
	strcpy(expatm[1], "CA");
	strcpy(expatm[2], "C");

}	/* init_phipsi */


/*************************************************************/
/* Calculate torsion angle for four atoms
*/
float torsion(atm1, atm2, atm3, atm4)
ATOM_REC atm1, atm2, atm3, atm4;
{
int		ichk;
VECTOR	p, q, x, y;
float	ret, chk;

	vecdif(atm1, atm2, p);
	ichk = dischk(p);
	if (ichk > 0)
		return(360.0);

	vecdif(atm3, atm2, q);
	ichk = dischk(q);
	if (ichk > 0)
		return(360.0);

	cross(p,q,x);

	vecdif(atm2, atm3, p);

	vecdif(atm4, atm3, q);
	ichk = dischk(q);
	if (ichk > 0)
		return(360.0);

	cross(p,q,y);

	ret = angle(x, y);
	cross(x,y,q);
	chk = dot(p,q);
	if (chk > 0.0)
		ret = -ret;

	return(ret);

}	/* torsion */


/*************************************************************/
/* calculate the diffence of the vectors x and y. 
*/
void vecdif(vec1, vec2, result)
ATOM_REC	vec1, vec2; 
VECTOR		result;
{
	result[0] = vec1.x - vec2.x;
	result[1] = vec1.y - vec2.y;
	result[2] = vec1.z - vec2.z;

}	/* vecdif */


/*************************************************************/
/* checks the vector atm 
*/
int dischk(atm)
VECTOR	atm;
{
float	dssq;

	dssq = dot(atm, atm);
	if ( (dssq < 0.5) || (dssq > 9.0) )
		return(1);
	else return(0);

}	/* dischk */


/*************************************************************/
/* calculate the cross product of vectors x and y. 
*/
void cross(a, b, result)
VECTOR	a, b, result;
{
	result[X] = (a[Y]*b[Z]) - (b[Y]*a[Z]);
	result[Y] = (a[Z]*b[X]) - (b[Z]*a[X]);
	result[Z] = (a[X]*b[Y]) - (b[X]*a[Y]);	
}	/* cross */


/*************************************************************/
/* calculate the dot product of vectors x and y. 
*/
float dot(x, y)
VECTOR	x, y;
{
int		i;
float	dotprod;

	dotprod = 0.0;
	for (i=0; i<3; i++)
		dotprod = dotprod + (x[i]*y[i]);
	return(dotprod);

}	/* dot */


/*************************************************************/
/* calculate the angle between two vectors defined by the 
 * ends of three vectors.
 */
float angle(x, y)
VECTOR	x,y;
{
float	ang, q;
float	magx, magy;

	magx = (float) sqrt((double)dot(x,x));
	magy = (float) sqrt((double)dot(y,y));
	q = magx * magy; 

	if (q < 1.0e-7)
		q = (float)1.0e-7;
	ang = dot(x, y) / q;

	if (ang > 1.0)
		ang = 1.0;
	if (ang < -1.0)
		ang = -1.0;
	ang = (float)57.29577951 * (float) acos((double)ang);
	return(ang);

}	/* angle */
