
/*
	file:		main.c
	purpose:	contains main driver routines for phipsi program.

	author: leigh willard 
	date:	21 July 1992	
	place:	University of Alberta / Department of Biochemistry
*/

#include "phipsi.h"

FILE	*pdbfp, *outfp, *get_out_file();
int		res1, resn;

/* subroutines */
void show_version(), read_input(), setup_phipsi(),
	 show_err(),do_phipsi(), init_parms();
int checkExist();


/*************************************************************/
/* main program
*/
int main(argc, argv)
int argc;
char **argv;
{

	setup_phipsi(argc, argv);
	fprintf(stdout,  argv[0] );
	do_phipsi();
	return(0);

}	/* main */


/*************************************************************/
/* current version of the program 
*/
void show_version()
{

	fprintf(stdout,  "\n");
	fprintf(stdout,  " -----------------------------------------------------------------\n");
	fprintf(stdout,  "  \n");
	fprintf(stdout,  "   Program....: phipsi\n");
	fprintf(stdout,  "   Description: Calculate main-chain torsion angles\n");
	fprintf(stdout,  "\n"); 
	fprintf(stdout,  "   Authors....: David Wishart & Leigh Willard\n");
	fprintf(stdout,  "   Supervisor:  Brian Sykes\n");
	fprintf(stdout,  "   Location...: University of Alberta\n"); 
	fprintf(stdout,  "                Protein Engineering Network of\n");
	fprintf(stdout,  "                Centres of Excellence\n");
	fprintf(stdout,  "  \n");
	fprintf(stdout,  "   Version....: %s\n", VERS);
	fprintf(stdout,  "  \n");
	fprintf(stdout,  "  \n");

}	/* show_version */


/*************************************************************/
/* get flags, read parameters
*/
void setup_phipsi(argc, argv)
int argc;
char *argv[];
{
int     i;
char    pdbFname[MAX_FNAME_LEN],
		resultsFname[MAX_FNAME_LEN],
		parmsFname[MAX_FNAME_LEN];

		strcpy(parmsFname, USER_PARMS);
		for (i=1; i<argc; i++) {
			if (strcmp(argv[i], PARMS_FLAG) == 0) {
				strcpy(parmsFname, argv[i+1]);
				break;
			}
		}

	strcpy(pdbFname, "");
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], FILE_FLAG) == 0) {
			strcpy(pdbFname, argv[i+1]);
			break;
		}
	}
	if (strcmp(pdbFname, "") == 0)
		show_err(TRUE, "PHIPSI", 
				"Usage: phipsi -f infile [ -o outfile ]  [ -i parmsfile ]");
	if (checkExist(pdbFname) != TRUE) {
	  printf("File %s", pdbFname); fflush(stdout);
	  show_err(TRUE, "PhiPsi", "Input file does not exist");
	}
	pdbfp = fopen(pdbFname, "r");
	
	strcpy(resultsFname, "");
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], RESULTS_FLAG) == 0) {
			strcpy(resultsFname, argv[i+1]);
			break;
		}
	}

	init_parms( resultsFname);

}	/* setup_phipsi */
