
/*
	file: util.c
	contains utility routines for vadar 
*/

#include "stdvadar.h"
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#define SKIP -9

#define NAME_LOC 13
#define NAME_EXTRA_LOC 16
#define RES_LOC 17
#define CHAIN_LOC 21
#define RESNUM_LOC 22 /* renum the location so that it can read num > 1000, modified by haiyan zhang */
#define RESNUM_EXTRA_LOC 26
#define COORD_LOC 30

char	lastExtra = '-';
char	resNumExtra;
int		lastresNum = -999;
int		resNumAdd;
int skip_res;
char skip_resextra;
int get_res_num(),duplicate_res(),read_one_pdb();
/*************************************************************/
/* read one atom from pdb file, excluding H atoms
*/
char *read_pdb(pdbfp, atom, hflag) 
FILE 		*pdbfp;
ATOM_REC	*atom;
int hflag;
{
int ret;
char *line = NULL;

  line = (char *)malloc(MAX_LINE_LEN * sizeof(char));
  if (NULL == line)
  {
    return NULL;
  }
	ret = read_one_pdb(pdbfp, atom, line);
	if (hflag)
		while ((ret != 0) && (ret == SKIP) )
			ret = read_one_pdb(pdbfp, atom, line);

	else
		while ((ret != 0) && ((ret == SKIP) || (atom->atomName[0] == 'H')) )
			ret = read_one_pdb(pdbfp, atom, line);

	if (ret == 0)
  {
    if (line != NULL) {
      free(line);
    }
		return(NULL);
  } else {
    return(line);
  }

}	/* read_pdb */


/***************************************************************************
* PURPOSE:  to read one atom record (line) from the pdb file
*/
int read_one_pdb(pdbfp, atom, line)
FILE 		*pdbfp;
ATOM_REC	*atom;
char 		*line;
{
char *ret;
static float Nx, Ny, Nz;


	/* read one line in */

	ret = fgets(line, MAX_LINE_LEN-1, pdbfp);
	while ( 0 != strncmp(line, "ATOM", 4) ) {
		if (ret == NULL)
			return(0);
		ret = fgets(line, MAX_LINE_LEN-1, pdbfp);
	}
	if (ret == NULL)
		return(0);

	/* atom name */

	sscanf(line+NAME_LOC, "%3s", atom->atomName); 
        if (strcmp(atom->atomName, "OE")  == 0) strcpy(atom->atomName, "OE1");
	if (strcmp(atom->atomName, "AE1") == 0) strcpy(atom->atomName, "OE1");
	if (strcmp(atom->atomName, "AE2") == 0) strcpy(atom->atomName, "NE2");
	if (strcmp(atom->atomName, "AD1") == 0) strcpy(atom->atomName, "OD1");
	if (strcmp(atom->atomName, "AD2") == 0) strcpy(atom->atomName, "ND2");
	/* this is a check for an atom name of the form NNNX, where
	 * NNN is the atom name, and X is a character (A or B) only
	 * the A atoms should be read (the B's are duplicates)
     */
	if ( (line[NAME_EXTRA_LOC] != ' ') && (line[NAME_EXTRA_LOC] != 'A') )
		return(read_one_pdb(pdbfp, atom, line));
	/* also ignore hydrogen atom's - those starting with H */
	if (line[NAME_LOC-1] == 'H')
		return(read_one_pdb(pdbfp, atom, line));
       /* ignore the atom start with 'Q', add by Haiyan zhang */
          if (line[NAME_LOC] == 'Q'|| line[NAME_LOC]=='D')
                return(read_one_pdb(pdbfp, atom, line));

	/* residue name */
	sscanf(line+RES_LOC, "%3s", atom->resName); 


	/* chain indicator */
	sscanf(line+CHAIN_LOC, "%c", &atom->chain);


	/* x, y, and z coordinates */
	sscanf(line+COORD_LOC,  "%f%f%f", &(atom->x), &(atom->y), &(atom->z));


	if (SKIP == get_res_num(line, atom, Nx, Ny, Nz))
	{ 
        	return(SKIP);
        }
 
	if (strcmp(atom->atomName, "N") == 0) {
		Nx = atom->x;
		Ny = atom->y;
		Nz = atom->z;
	}

	return(1);

}	/* read_one_pdb */



int get_res_num(line, atom, Nx, Ny, Nz)
char		*line;
ATOM_REC	*atom;
float Nx, Ny, Nz;
{


	sscanf(line+RESNUM_LOC, "%d", &atom->origResNum); 
	atom->resNum = atom->origResNum;


	/* there may be a character here.  This happens when an extra res
	 * is added to the pdb file, but instead of renumbering the file,
	 * a character such as A is added to the res num.  But, we must
	 * renumber the pdb file if this happens, for the programs to run.
	 * It is also possible that this is a duplicate residue (alternate).
	 * If so, it will have the same coordinates as the previous residue,
	 * and then this residue should be discarded.
	 */
	resNumExtra = line[RESNUM_EXTRA_LOC];
	if ( (resNumExtra != lastExtra ) && (lastExtra != '-') &&
		(atom->resNum == lastresNum) ) {
		if (duplicate_res(atom, Nx, Ny, Nz)) {
			return(SKIP);
		}
		else resNumAdd++;
	}
	lastExtra = resNumExtra;
	lastresNum = atom->resNum;
	atom->resNum += resNumAdd;

	return(1);

}	/* get_res_num */


int duplicate_res(atom, Nx, Ny, Nz)
ATOM_REC *atom;
float Nx, Ny, Nz;
{

	if ( (atom->x == Nx) && (atom->y == Ny) && (atom->z == Nz) ) {
		skip_res = atom->resNum;
		skip_resextra = resNumExtra;
		return(TRUE);
	}

	if ( (atom->resNum == skip_res) && (resNumExtra == skip_resextra) ) {
		return(TRUE);
	}
	return(FALSE);

}	/* duplicate_atom */
