
/*
	file: print.c
	This file contains routines for printing results for shifty. 
*/

#include "csipdb.h"

void print_sb_align();
char match_res();
extern char *get_date();
extern void show_err();
/*************************************************************/
/* Print info about this run. 
*/
void print_run(fp, aSeq)
FILE *fp;
SEQ_REC *aSeq;
{

	fprintf(fp, "\n");
	fprintf(fp, "******************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, "   Program......: shifty (version %s)\n", VERSION);
	fprintf(fp, "   Description..: Find Best Alignments\n");
	
        char* date = get_date();
	fprintf(fp, "   Date...............: %s\n\n", date );
//        free(date);

	fprintf(fp, "\n");
	fprintf(fp, "   Sequence Name: %s", aSeq->id);
	fprintf(fp, "   Amino acids..: %-5d\n", aSeq->len);
	fprintf(fp, "\n");
	fprintf(fp, "   Scoring Mat..: %s\n", sMatFname);
	fprintf(fp, "   Gap Penalty..: %-2d\n", gapPenalty);
	fprintf(fp, "   Gap Size Pen.: %-2d\n", gapSizePenalty);
	fprintf(fp, "   Sort Method..: %-2d\n", sortAlgorithm);
	fprintf(fp, "\n");
	fprintf(fp, "******************************************************\n");
	fprintf(fp, "\n");

}	/* print_run */


/*----------------------------------------------
 * time to update user about the search so far 
 *----------------------------------------------
 */
void print_stats(FILE *fp, int pCount, double gScore)
{

	fprintf(fp, "Proteins Scanned: %5d  ", pCount);

	if (sortAlgorithm == 0) {
		fprintf(fp, "BestScore: %-5ld ", topAligns[0].oScor);
		fprintf(fp, "GroupScore: %-5d\n", (int)gScore);
	}
	else {
		fprintf(fp, "BestScore: %-5.2f ", topAligns[0].zVal);
		fprintf(fp, "GroupScore: %-5.2f\n", gScore);
	}

} /* print_stats */


/*************************************************************/
/* print alignment results to a file	
*/
void print_aligns(int pCount, int nAlign)
{
	register int 	i;
	FILE			*outfp;

	outfp = fopen(resultsFname, "w");
	if (outfp == NULL) 
		show_err(TRUE, MODULE, "Cannot open results file!");

	print_run(outfp, &mySeq);
	fprintf(outfp, "   Number of proteins tested.: %5d\n", pCount);
	fprintf(outfp, "   Number of alignments found: %5d\n", nAlign);

	for (i=0; i<nAlign; i++) {
		print_sb_align(outfp, &topAligns[i], i+1);
	}

	fclose(outfp);

} /* print_aligns */



/*************************************************************/
/* print out vital stats about this one particular alignment
*/
void print_sb_align(fp, align, count)
FILE	*fp;
ALIGN_REC *align;
int		count;
{
	register int	j, ix, qix, pNum, len;
	register int    numAminoSeq, numAminoData;
	register int    oldNumAS, oldNumAD;
	char			ch;

	fprintf(fp, "\n\n**********(%d)**********\n", count);
	fprintf(fp, "Title....: ");
	fputs(align->title, fp);
	fprintf(fp, "Id.......: %s\n", align->id);
	fprintf(fp, "Score....: %-6ld\n", align->oScor);
	if (sortAlgorithm != 0)
		fprintf(fp, "Test Stat: %-6.2f\n", align->zVal);
	fprintf(fp, "Matches..: %-6d", align->ident);


/*fprintf(fp, "\nID%:...: %4lf", align->perid);*/

	ix = 0;
	numAminoSeq = 0;
	numAminoData = 0;
	oldNumAS = 0;
	oldNumAD = 0;
	len = align->len;
	while (len > 0) {
		pNum = MIN(len, RES_PER_LINE);
		len -= pNum;
		qix = ix * RES_PER_LINE;

		fprintf(fp, "\n\nQuery Seq: ");
		for (j=0; j<pNum; j++) {
			putc(align->qpRes[qix + j], fp);
			if (align->qpRes[qix + j] != ' ')
				numAminoSeq++;
		}
		/* print amino acid count */
		for (j=pNum; j<RES_PER_LINE; j++)
			putc(' ', fp);
		if ( oldNumAS != numAminoSeq)
			fprintf(fp, "      %d", numAminoSeq);
		oldNumAS = numAminoSeq;

		fprintf(fp, "\nMatching.: ");
		for (j=0; j<pNum; j++) {
			ch = match_res(align->qpRes[qix + j], align->dpRes[qix + j]);
			if (ch == '=')
			    ch = '|';
			putc(ch, fp);
		}

		fprintf(fp, "\nDatabase.: ");
		for (j=0; j<pNum; j++) {
			putc(align->dpRes[qix + j], fp);
			if (align->dpRes[qix + j] != ' ')
			   numAminoData++;
		}
		/* print amino acid count */
		for (j=pNum; j<RES_PER_LINE; j++)
			putc(' ', fp);
		if ( oldNumAD != numAminoData)
			fprintf(fp, "      %d", numAminoData);
		oldNumAD = numAminoData;

		fprintf(fp, "\nStructure: ");
		for (j=0; j<pNum; j++)
			putc(align->dsRes[qix + j], fp);

		ix++;
	}

	fprintf(fp, "\n");

}	/* print_sb_align */


/*************************************************************/
/* determine the similarity between 2 residues according to 
** scoring matrix. Return appropriate mark character.
*/
char match_res(qch, dch)
char qch, dch;
{
	register int qNum, dNum;

	qNum = AA_IX(qch);
	dNum = AA_IX(dch);
	if ((qNum < CODES) && (qNum >= 0) && (dNum < CODES) && (dNum >= 0)) {
		if (qNum == dNum)
			return(AA_MATCH);
		else if (wtMat[qNum*WT_MAT_DIM + dNum] >= nearMatchScore)
			return(AA_NEAR_MATCH);
		else
			return(AA_NOT_MATCH);
	}
	else
		return(AA_NOT_MATCH);

}	/* match_res */
