/* MODULE: sidechain.h */
#ifndef __SIDECHAIN_H
#define __SIDECHAIN_H

#include "csipdb.h"

/***OVERVIEW************************************************
 * File Name     : sidechain.h
 * Purpose       : Side chain insertion module for thrifty.
 * Author        : Philip S Luk ~ philipluk@bigfoot.com
 * Date Written  : July 6, 2001
 * Last Modified : July 9, 2001
 ***END*OVERVIEW********************************************/

/* MODULE DEFINES */
  #define PROG_VERSION 	1.0
  #define MAX_RES_ATOMS 12		/* max num of atoms in residue   */
  #define MAX_RESIDUES 	1200		/* alloc enuf mem for this # res */
  #define ALBERTPDBTMP  "pdb.tmp"	/* Albert's tmp info store file  */
  #define SIDE_CHAIN_DB "side.db"	/* side chain database     */
  #define A_REF_ATOM	"N"		/* 'a' rotation ref atom   */	
  #define B_REF_ATOM	"CA"		/* 'b' rotation ref atom   */	
  #define C_REF_ATOM	"C"		/* 'c' rotation ref atom   */	
  #define ATOM_FLAG	"ATOM"		/* PDB file atom line flag */	
  #define END_FLAG	"END"		/* buffer/file end flag    */	
  #define TEMP_FILE	"sdechn.tmp"	/* temp file name          */	
/* END MODULE DEFINES */

/* TYPE DEFINTIONS */
/*  typedef enum {
    false, true
  } boolean;
*/
  typedef struct {
    double x; double y; double z;
  } POINT_t;				/* 3D Coordinate point type */

  typedef struct {
    char    name [5];			/* Residue id eg. MET */
    int     num_atoms;			/* Number of atoms in residue */
    POINT_t A, B, C;			/* Rotation reference points */
    float   chi;
    boolean need_sdechn;		/* Does residue need side chain? */
    boolean remove_extra_atoms;		/* Does residue need to remove extra atoms? */
  } RES_t;

  typedef struct {
    int num_atoms;
    char atoms[MAX_RES_ATOMS][5];            /* List of atoms this residue should have */
    boolean set;
  } amino_acid_t;
/* END TYPE DEFINTIONS */

/* GLOBAL VARIABLES */
  RES_t g_res_list[MAX_RESIDUES+1];
  int   g_res_count;

  amino_acid_t g_amino_acid_db[26];
/* END GLOBAL VARIABLES */

/* PROCEDURE DECLARATIONS */
/***********************************************************
 *
 ***********************************************************/
void do_side_chain_stuff (int topAlignNum, PROGRAM_PARAMS params);
/* END PROCEDURE DECLARATIONS */

#endif 
/* END MODULE: sidechain.h */
