#ifndef __PDBFILE_H__
#define __PDBFILE_H__


#include "vec3D.h"

#define UNKNOWN_TYPE   	 0
#define H_TYPE   			10
#define N_TYPE   			11
#define O_TYPE  			12
#define C_TYPE   			13
#define CA_TYPE   		14
#define CB_TYPE  			15
#define CG_TYPE  			16

#define BACKBONE			1
#define SIDECHAIN			2

typedef struct Atom {
  Vec3D point;
  int type;
  int resNum;
  char name[4];
  char resName[4];
  char *line;
} Atom;

typedef struct PDBFile {
  int numRes;
  int numAtoms;
  int capacity;
  Atom *atoms;
} PDBFile;

/* PROTOTYPES */
void freePDBFile(PDBFile *pdb);
int atomIsCA(Atom *a);
void getAtomPosition(Atom *a, double pos[3]);
int parsePDBFile(char *name, PDBFile *pdb, char chain);
int getNextAtomIndex(PDBFile *pdb, int n, char *tag);
int getNextAtomOfType(PDBFile *pdb, int n, int type);
char getResLetter(char *tag);
int getFirstAtomOfResidue(PDBFile *pdb, int resNum);

#endif
