/**************************************************************************

  pepmake.c
     Generate a PDB file from a file of internal  angles and coordinates.

  Date: 
     March, 2003

  Author:
     Aaron Davidson @ BTI; Adapted from code by Nelson Young

 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ga_util.h"
#include "vec3D.h"
#include "pdbfile.h"
#include "pepmake.h"
//#include "../util/memwatch.h"

char *atomNames[6] = {"CA ", "C  ", "O  ", "N  ", "CA ", "H  "};

void readTemplate(ResData *res, int ignoreBackbone);


/** options **/
static int sideChain = 1; 			// attach a side chain
static int printH = 1;				// print HN atoms
static int ignoreDBBackbone = 1; // ignore the database backbone atoms

/* default atom positions for a backbone. */
static Vec3D template[] = {
	{0.0, 0.0, 0.0},	  /* CA */
	{1.53, 0.0, 0.0},	  /* C  */
	{2.1686, 1.0629, 0.0},    /* 0  */
	{2.0669, -1.2059, 0.0},   /* N  */
	{3.5188, -1.4359, 0.0},   /* CA */
	{1.5223, -2.0446, 0.0},   /* H  */  
};


/* print the atoms in PDB format to the given file */
void printPDBAtoms(FILE *out, ResData *residues, int numRes) {

	int n,c=1;
	ResData *res = &residues[0];
	ResData *nextres = &residues[1];
	fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n",
			(c++), res->names[N], res->resName, (res->id),
			template[N][0], template[N][1], template[N][2]);

	for (n=0; n<numRes; n++) { 
		res = &residues[n];
		nextres = &residues[n+1];
		if ( (n != 0) && (n != (numRes-1)) ) {

	      fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG \n", (c++), res->names[CA], res->resName, (res->id), res->atoms[CA][0], res->atoms[CA][1], res->atoms[CA][2]);

  		   fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[C1], res->resName, (res->id), res->atoms[C1][0], res->atoms[C1][1], res->atoms[C1][2]);	
	
		   fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[O], res->resName, (res->id), res->atoms[O][0], res->atoms[O][1], res->atoms[O][2]);

		   if(strcmp(res->resName, "GLY")!=0)
		     fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CB], res->resName, (res->id), res->atoms[CB][0], res->atoms[CB][1], res->atoms[CB][2]);
	      
		   //if( (sideChain == 1) && strcmp(res->resName,"ALA")!=0 && strcmp(res->resName,"GLY") !=0 )
		   if( strcmp(res->resName,"ALA")!=0 && strcmp(res->resName,"GLY") !=0 ){
		     //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CG], res->resName, (res->id), res->atoms[CG][0], res->atoms[CG][1], res->atoms[CG][2]);
		   }

      	fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), nextres->names[N], nextres->resName, (nextres->id), res->atoms[N][0], res->atoms[N][1], res->atoms[N][2]);
	
	if ( strcmp(nextres->resName, "PRO")!=0 ) {
	  //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), nextres->names[H], nextres->resName, (nextres->id), res->atoms[H][0], res->atoms[H][1], res->atoms[H][2]);

			   }
  		} else {
		  if (n == 0) {
		    fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CA], res->resName, (res->id), res->atoms[CA][0], res->atoms[CA][1], res->atoms[CA][2]);
		    fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[C1], res->resName, (res->id), res->atoms[C1][0], res->atoms[C1][1], res->atoms[C1][2]);		
		    fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[O], res->resName, (res->id), res->atoms[O][0], res->atoms[O][1], res->atoms[O][2]);
		    //Due to a bug, the CB and the CG(if any)of the first residue is not printed..
		    //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CB], res->resName, (res->id), res->atoms[CB][0], res->atoms[CB][1], res->atoms[CB][2]);
		    /* if(sideChain == 1)
		       fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CG], res->resName, (res->id), res->atoms[CG][0], res->atoms[CG][1], res->atoms[CG][2]);*/
		    fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), nextres->names[N], nextres->resName, (nextres->id), res->atoms[N][0], res->atoms[N][1], res->atoms[N][2]);
		    
		    
		    if ( strcmp(nextres->resName, "PRO")!=0 ) {
		      //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), nextres->names[H], nextres->resName, (nextres->id), res->atoms[H][0], res->atoms[H][1], res->atoms[H][2]);
				   }
			} else {
		      fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CA], res->resName, (res->id), res->atoms[CA][0], res->atoms[CA][1], res->atoms[CA][2]);
  			   fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[C1], res->resName, (res->id), res->atoms[C1][0], res->atoms[C1][1], res->atoms[C1][2]);		
			   fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[O], res->resName, (res->id), res->atoms[O][0], res->atoms[O][1], res->atoms[O][2]);
			   if(strcmp(res->resName, "GLY")!=0)
			     fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CB], res->resName, (res->id), res->atoms[CB][0], res->atoms[CB][1], res->atoms[CB][2]);
			   //if( (sideChain == 1) && strcmp(res->resName,"ALA")!=0 && strcmp(res->resName,"GLY") !=0 )
			   if( strcmp(res->resName,"ALA")!=0 && strcmp(res->resName,"GLY") !=0 ){
			     //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00      TAG\n", (c++), res->names[CG], res->resName, (res->id), res->atoms[CG][0], res->atoms[CG][1], res->atoms[CG][2]);
			   }
			}
		}
   }		      
	//OLD CODE ...				
      //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00\n", (c++), res->names[N], res->resName, (res->id), res->atoms[N][0], res->atoms[N][1], res->atoms[N][2]);
	   //for (i=0; i<res->numAtoms; i++) {
	   	/* skips the CA2 and N, and the first residue's CB (to avoid a bug) */
	      //if ((i != CA2 && i != N) && !(n==0 && i==HN+1) && !( ( !printH || (strcmp(res->resName, "PRO") == 0) || (n == 0) ) && i==HN) )  {
	      //if (i != CA2) {
			//   fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00\n", (c++), res->names[i], res->resName, (res->id), res->atoms[i][0], res->atoms[i][1], res->atoms[i][2]);
			//}
	   //}
	   //fprintf(out, "ATOM %6d  %-3s %s %5d %11.3lf %7.3lf %7.3lf  1.00  1.00\n", (c++), res->names[N], res->resName, (res->id), res->atoms[N][0], res->atoms[N][1], res->atoms[N][2]);
	//}
}


/*************************************************************/
/* Apply the saved template of this residue's coordinates.   */
/* We rotate the residue into the canonical form, then swap  */
/* in the new coordinates, and then reverse all of the       */
/* rotations that were used to canonisize the coordinates    */
/*************************************************************/
static void applyTemplate(ResData *res) {
	int i;
	Vec3D xAxis = {1,0,0}; // a unit vector
   Vec3D T; // a temp vector
   double thetaZ, thetaY, thetaX;
	Vec3D transVec;

	copyVector(res->atoms[CA], transVec); // store for later
   for (i=0; i<res->numAtoms; i++) 
		difference(res->atoms[i], transVec, res->atoms[i]);  

   // compute angle from X axis
   setVector(T, res->atoms[C1][0], res->atoms[C1][1], 0);
   thetaZ = angle(T, xAxis);
   if (res->atoms[C1][1] > 0) thetaZ = 360 - thetaZ;
   // rotate atoms around Z axis
   for (i=0; i<res->numAtoms; i++) rotateZ(res->atoms[i], thetaZ);

   // compute angle from X axis
   setVector(T, res->atoms[C1][0], 0, res->atoms[C1][2]);
   thetaY = angle(T, xAxis);
   if (res->atoms[C1][2] < 0) thetaY = 360 - thetaY;
   // rotate atoms around Y-axis
   for (i=0; i<res->numAtoms; i++) rotateY(res->atoms[i], thetaY);

   // compute angle for N
   thetaX = atan(res->atoms[N][2] / res->atoms[N][1]);
   thetaX = 180 - (thetaX * 180.0/PI_GA);
   if (res->atoms[N][1] < 0) thetaX = 180+thetaX;
   for (i=0; i<res->numAtoms; i++) rotateX(res->atoms[i], thetaX);

	for (i=0; i<res->numAtoms; i++) {
		//printf("%d :  %lf %lf %lf\n", 
		//	i, res->atoms[i][0], res->atoms[i][1], res->atoms[i][2]);
		copyVector(res->template[i], res->atoms[i]);
	}

   for (i=0; i<res->numAtoms; i++) rotateX(res->atoms[i], -1*thetaX);
   for (i=0; i<res->numAtoms; i++) rotateY(res->atoms[i], -1*thetaY);
   for (i=0; i<res->numAtoms; i++) rotateZ(res->atoms[i], -1*thetaZ);

   for (i=0; i<res->numAtoms; i++) {
		summation(res->atoms[i], transVec, res->atoms[i]);  
	} 
}


/*************************************************************/
/* Rotate the current residue by phi & psi                   */
/*************************************************************/
static void rotatePhiPsi(int num, ResData *residues) {
	int i,j;
	Vec3D p1, p2, L, l, n;
	Vec3D q[MAX_ATOMS];
	ResData *res = &residues[num];

	difference(res->atoms[CA], res->atoms[C1], p1); 
	unitVector(p1, p1);
	difference(res->atoms[CA2], res->atoms[N], p2); 
	unitVector(p2, p2);	
	crossproduct(p2, p1, L);
	unitVector(L, L);
	copyVector(res->atoms[CA2], l);

	setVector(q[CA], 0, 0, 0);
	for (i=1; i<res->numAtoms; i++) {
		for (j=0; j<3; j++) {
			q[i][j] = res->atoms[i][j] - res->atoms[CA][j];
		}
	}
	for (i=0; i<res->numAtoms; i++) rotateCC(p1, q[i], 180);
	for (i=0; i<res->numAtoms; i++) rotateCC(L, q[i], -63.0);
	for (i=0; i<res->numAtoms; i++) rotateCC(p2, q[i], res->phi);
	copyVector(q[C1], n);
	unitVector(n, n);
	for (i=0; i<res->numAtoms; i++) rotateCC(n, q[i], res->psi);

	for (i=0; i<res->numAtoms; i++) {		
		for (j=0; j<3; j++) {
			res->atoms[i][j] = l[j] + q[i][j];
		}
	}
}

/*************************************************************/
/* Copy the position of src to be the position of dst        */
/*************************************************************/
void copyPosition(ResData *src, ResData *dst) {
	int i;
	for (i=0; i<src->numAtoms; i++) {
		copyVector(src->atoms[i], dst->atoms[i]);
	}
}

/*************************************************************/
/* Add the next residue to the peptide chain, and carry out  */
/* the appropriate rotations and translations.               */
/*************************************************************/
static void nextPeptide(ResData *res, int num, ResData *residues) {
	int i;
	Vec3D oldN, oldHN, CN;
 	double angle1, angle2,dist;
   if (num > 0) {
		copyPosition(&residues[num-1], res);
		applyTemplate(res);
	}

	/* cache old values for omega rotation */
   for(i=0; i<3; i++) oldN[i] = res->atoms[N][i];
   for(i=0; i<3; i++) oldHN[i] = res->atoms[H][i];
   for(i=0; i<3; i++) CN[i] = res->atoms[N][i] - res->atoms[C1][i];
	unitVector(CN, CN);

	rotatePhiPsi(num, residues);

  /* add by haiyan to add CB and CG correctly */
      
      if (strncmp(res->resName,"GLY",3)!=0)
                {
		 dist=1.53;angle1=109.5; angle2=109.5;
 		 OutofPlane( oldN, res->atoms[CA], res->atoms[C1], dist,angle1,angle2, res->atoms[CB] );
                }
                else
                {
                    dist=1;angle1=109.5; angle2=109.5;
                   OutofPlane( oldN, res->atoms[CA], res->atoms[C1], dist,
		   angle1,angle2, res->atoms[CB]);
       
                }
         /* if the alpha-hydrogen is also given, attach it as well. */
           /*     if( natoms > satoms )
                   OutofPlane( atoms.r[2], atoms.r[1], N_old, atoms.p[satoms+2][3],
                               atoms.p[satoms+2][2], atoms.p[satoms+2][1], atoms.r[satoms+2] );
 */
               /* attach CG atom*/
	       //printf("%s, chi=%f\n",res->resName,res->chi);
                if(strncmp(res->resName,"PRO",3)==0)
                 {
                  angle1=70;
                  angle2=150-(res->chi+50)/2;
                  }
                  else{
                 if(res->chi>-90 &&res->chi<=0)
                  {angle1=90;angle2=140;}
                 else if (res->chi>0 && res->chi<90){
                  angle1=90;angle2=90;}
                  else {
                   angle1=140;angle2=90;}
                   }
		   dist=2.55;
              OutofPlane(oldN, res->atoms[CA],  res->atoms[C1], dist,angle1,angle2,  res->atoms[CG] );
/* finish add by haiyan */
	/* add omega rotation */
   if (num > 0 && USE_OMEGA) {
      // Determine the actual angle of rotation
	   double omega = residues[num-1].omega;
		if (omega < 0) omega = omega + 180.0;
		else omega = omega - 180.0;
      //printf("rotating by omega = %lf\n", omega);
		for (i=0; i<3; i++) oldHN[i] -= oldN[i];
		for (i=0; i<res->numAtoms; i++) {
			difference(res->atoms[i], oldN, res->atoms[i]);
		}
		rotateCC(CN, oldHN, omega);
		for (i=0; i<res->numAtoms; i++) {
			rotateCC(CN, res->atoms[i], omega);
		}
		for (i=0; i<3; i++) oldHN[i] += oldN[i];
		for (i=0; i<res->numAtoms; i++) {
			summation(res->atoms[i], oldN, res->atoms[i]);
		}
	}

}
/********************************************
 to add atoms according to the angles 
*********************************************/
void OutofPlane( Vec3D a, Vec3D o, Vec3D b, double Wmag, double theta1, double theta2, Vec3D W )
{
        double   W1, W2, W3, CS1, CS2, dot12=0;
        int     i;
         Vec3D       v1, v2, v3;                                                                                                           
        for( i=0; i < 3; i++ ) v1[i] = a[i] - o[i];
        unitVector( v1,v1 );
        for( i=0; i < 3; i++ ) v2[i] = b[i] - o[i];
        unitVector( v2,v2 );
        crossproduct(  v1, v2, v3);
        unitVector( v3,v3 );
        if( theta1 == 180 || theta1 == -180 )   CS1 = -1.0;
        else    {
                theta1 *= M_PI/180;
                CS1 = cos(theta1);
        }
        if( theta2 == 180 || theta2 == -180 )   CS2 = -1.0;
        else    {
                theta2 *= M_PI/180;
                CS2 = cos(theta2);
        }
        for( i=0; i < 3; i++ ) dot12 += v1[i]*v2[i];
        /* Computing the non-orthogonal components in the local frame */
        W2 = Wmag*(CS2 - dot12*CS1)/(1.0 - dot12*dot12);
        W1 = Wmag*CS1 - dot12*W2;
        W3 = sqrt( Wmag*(Wmag - W1*CS1 - W2*CS2) );
        /* Transform back to the world frame */
        for( i=0; i < 3; i++ ) W[i] = W1*v1[i] + W2*v2[i] + W3*v3[i] + o[i];
}

/*************************************************************/
/* Initialize the residue atom coordinates to the default    */
/* template positions.                                       */
/*************************************************************/
void initResData(ResData *r) {
	int i;
	for (i=0; i<6; i++) { 
		copyVector(template[i], r->atoms[i]);
	}
}


/*************************************************************/
/* Read the next residue from the file. Returns false if the */
/* a residue could not be read (usually end of file)         */
/*************************************************************/
static int readResidue(ResData *res, FILE *fp) {
	int rc, num;

	/* read the residue & phi psi omega angles */
	rc = fscanf(fp, "%d%s%lf%lf%lf%lf%d", 
		&res->id, res->resName, &res->phi, &res->psi, 
		&res->omega, &res->chi, &num);
	if (rc != 7) return FALSE;
	return TRUE; 
}


/*************************************************************/
/* Opens the file and counts the number of residues entries  */
/* there are with dihedral angles, etc...                    */
/*************************************************************/
int countResidues(char *fname) {
	int n = 0;
	ResData res;
	FILE *fp = fopen(fname, "r");
	if (fp != NULL) {
		while (!feof(fp)) {
			if (readResidue(&res, fp)) {
				n++;
			} else {
				fclose(fp);
				return n;
			}
		}
		fclose(fp);
	}
	return n;
}



//********************************************************************************************
/* iteratively assembles the peptide */
void buildPeptide(ResData *residues, int numRes) {
  int i;
  for (i=0; i<numRes; i++) {
    
    ResData *res = &residues[i];
    //printf("before...(%.3f, %.3f,%.3f)omega=%.3f\n",res->atoms[CA][0],res->atoms[CA][1],res->atoms[CA][2],res->omega);
    nextPeptide(res, i, residues);
    //printf("after...(%.3f, %.3f,%.3f)omega=%.3f\n",res->atoms[CA][0],res->atoms[CA][1],res->atoms[CA][2],res->omega);
  }
}// END buildPeptide *************************************************************************



/* iteratively assembles the peptide */
void buildFinalPeptide(ResData *residues, int numRes) {
	int i;
	for (i=0; i<numRes; i++) {
	   ResData *res = &residues[i];
	   //fprintf( stderr, "%d: %s %lf %lf %lf\n\t", i, res->resName, res->phi, res->psi, res->omega);
		nextPeptide(res, i, residues);
	}
}


/*************************************************************/
/* Assuming dihedral angles have been set, this function     */
/* builds a PDB file from the representation.                */
/*************************************************************/
void readGAfolderAngles(ResData *residues, int n) {
   int i = 0;
	for (i=0; i<n; i++) {
		ResData *res = &residues[i];
		initResData(res);

		/* read atom coordinates for the residue template */
		readTemplate(res, ignoreDBBackbone); 
	//	printf("After reading Templates,%s -- phi=%4.3f psi=%4.3f and chi=%4.3f\n",res->resName,res->phi,res->psi,res->chi);
	        
		if(!sideChain) res->numAtoms = 7;
                if(sideChain) res->numAtoms = 8; //added by Raj..
	}
}


/*************************************************************/
/* PepMake reads a file of dihedral angles and distances and */
/* builds a PDB file from the representation.                */
/*************************************************************/
void readAngles(char *fname, ResData *residues, int n) {
	int i = 0;
	int rc = 0;
	FILE *fp = fopen(fname, "r");
	if (fp != NULL) {
		for (i=0; i<n; i++) {
			ResData *res = &residues[i];
			initResData(res);
			if (!readResidue(res, fp)) {
				fprintf(stderr, "Read residue failed\n");
			}
			/* read atom coordinates for the residue template */
			readTemplate(res, ignoreDBBackbone); 
			
			if (!sideChain) res->numAtoms = 7;
			if(sideChain) res->numAtoms = 8; //added by Raj..

			fprintf( stderr, "%d: %s %lf %lf %lf\n\t", i, res->resName, res->phi, res->psi, res->omega);
			for (rc=0; rc<res->numAtoms; rc++) {
				fprintf( stderr, "%s, ", res->names[rc]);
			}	
			fprintf( stderr, "\n");
		}
		fclose(fp);
	} else {
	  fprintf(stderr, "ERROR: Could not open file, or file does not contain dihedral angles.\n");exit(-1);
	}
}

/*************************************************************/
/* PepMake reads a file of dihedral angles and distances and */
/* builds a PDB file from the representation.                */
/*************************************************************/
static void pepmake(char *fname) {
	int n = countResidues(fname);
	fprintf(stderr, "Residues = %d\n", n);
	if (n > 0) {
		ResData *residues = SAFEMALLOC(sizeof(ResData)*n);
		readAngles(fname, residues, n);
		buildPeptide(residues, n);
		printPDBAtoms(stdout, residues, n);
		SAFEFREE(residues);
	}
}

/*************************************************************/
/* The main entry point                                      */
/*************************************************************/
int main_pepmake(int argc, char **argv) {
 	char *input;
	int i;
	
	fprintf(stderr, "**** BTI PepMake ****\n");
	if (argc < 2) {
		fprintf(stderr, "Usage:\n   pepmake [--printH] [--backbone_template] dihedral.angles.txt\n");
		fprintf(stderr, "Options:\n");
		fprintf(stderr, " --printH\n\t prints H atoms in the PDB file\n");
		fprintf(stderr, " --fixbb\n\t Uses single fixed template for all backbone atoms\n");
		return 0;
	}
	
	printH = 0;
	ignoreDBBackbone = 0;
	
	for (i=1; i<argc-1; i++) {
     	if (strcmp(argv[i], "--printH") == 0) {
     		printH = 1;
		} else if (strcmp(argv[i], "--fixbb") == 0) {
     		ignoreDBBackbone = 1;
		} else {
			fprintf(stderr, "Unrecognized Option: [%s]\n", argv[i]);			
			exit(1);	
		}
   }
	input = argv[argc-1];
	pepmake(input);	
	return 0;
}
