#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ga_util.h"
#include "vec3D.h"
#include "pdbfile.h"
#include "pepmake.h"
#include "template.h"
//#include "../util/memwatch.h"

/*************************************************************/
/* Reads the best matching template from the database, for   */
/* the given Residue.                                        */
/*************************************************************/
void readTemplate(ResData *res, int ignoreBackbone) {
	int n, i = 0, read;
	FILE *fp;
	double chi;
	char fname[200];
	char name[4];
	double x,y,z;
	int best = 1000;
	int debug = TRUE;

	//printf("res->resName = %s and strlen=%d",res->resName,strlen(res->resName));
	sprintf(fname, "%s%s%s", TEMPLATE_DATABASE, res->resName, ".db");
	
//	if (debug) printf("--%s-- phi = %4.3f psi = %4.3f CHI = %4.3f\n",res->resName,res->phi,res->psi,res->chi);
	if ((fp = fopen(fname, "r")) != NULL) {
		int r = 2;
		while (!feof(fp) && r >= 2) { 
			// hack to handle two formats
			r = fscanf(fp, "%lf %d", &chi, &n);
			if (r != 2) {
				char c;
				r = fscanf(fp, "%c%c%c %lf %d", &c, &c, &c, &chi, &n);	
			}
			if (r >= 2) {		
//				if (debug) printf("chi=%lf, n=%d\n", chi, n);
				read = FALSE;
				if (abs(chi - res->chi) < best) {
					best = abs(chi - res->chi);
					read = TRUE;
					res->numAtoms = n;		
					//if (debug) printf("Reading! %d\n", n);
				}else{
				  read = FALSE;
				}
				for (i=0; i<n; i++) {
					if (fscanf(fp, "%s%lf%lf%lf", name, &x, &y, &z) == 4) {
						if (read) {
							strncpy(res->names[i], name, 4);
							res->template[i][0] = x;
							res->template[i][1] = y;
							res->template[i][2] = z;
							if (debug) {
							  //printf("read: %lf %lf %lf\n", x, y, z);
							}
						}
					}
				}
			}
		}
		fclose(fp);
	} else {
	  fprintf(stderr, "Cannot open %s\n", fname);exit(-1);
	}

	if (ignoreBackbone) {
		/* use a generic template rather than the database for 
			the main chain atoms */
	  //printf("In template.c, ignoreBackbone = %d\n",ignoreBackbone);
	  setVector(res->template[CA], 0.0, 0.0, 0.0);
	  setVector(res->template[C1], 1.53, 0.0, 0.0);
	  setVector(res->template[O], 2.1686, 1.0629, 0.0);
	  setVector(res->template[N], 2.0669, -1.2059, 0.0);
	  setVector(res->template[CA2], 3.5188, -1.4359, 0.0);

	}

}
