/* MODULE: util.c */

/***OVERVIEW************************************************
 * File Name     : util/util.c
 * Purpose       : Error reporting module.
 * Author        : Philip S Luk ~ philipluk@bigfoot.com
 * Date Written  : July 9, 2001
 * Last Modified : July 9, 2001
 ***END*OVERVIEW********************************************/

/* STANDARD INCLUDES */
  #include <stdio.h>
  #include <string.h>
  #include <math.h>
  #include "util.h"
  
/* END STANDARD INCLUDES */

int append_file (char * appendor, char * appendee) {
/***********************************************************
 * Appends file 'appendee' to file 'appendor'. 
 ***********************************************************/

  FILE * tarfp;
  FILE * srcfp;
  char   buffer[MAX_LINE_LEN];

  tarfp = fopen (appendor, FILE_APPEND);
  srcfp = fopen (appendee, FILE_READ);

  while (fgets (buffer, MAX_LINE_LEN, srcfp) != NULL) {
    fputs (buffer, tarfp); fflush (tarfp);
  }

  fclose (srcfp);
  fclose (tarfp);

  /*remove (appendee);*/

  return 1;

} /* END append_file () */

int assign_pt (POINT * a, POINT b) {
/***********************************************************
 * Assigns point b to point a;
 ***********************************************************/
 
  a->x = b.x; a->y = b.y; a->z = b.z;

  return 1;

} /* END assign_pt () */


void printPOINT(POINT *point){
/***************************************************
 * Prints the co-ordinates of the point
 **************************************************/
  printf("(%6.2f, %6.2f, %6.2f)    ",point->x, point->y, point->z);
} /* END printPoint () */ 	




POINT  formvector(PTS *points){
/***************************************************
 * Forms a vector which points from the start to the
 *  end co-ordinates.
 **************************************************/
POINT vector;
  vector.x = points->end.x - points->begin.x;
  vector.y = points->end.y - points->begin.y; 
  vector.z = points->end.z - points->begin.z; 

  return (vector);
  }/* END formvector() */
	   

POINT make_neg_vector(POINT *some_vector){
/***************************************************
 * Forms the negative of a vector *
 **************************************************/
 POINT neg_vector;
  neg_vector.x = -(some_vector->x);
  neg_vector.y = -(some_vector->y);		
  neg_vector.z = -(some_vector->z);
  
  return(neg_vector);

 }/* END make_neg_vector() */
  
 
  
POINT form_unitvector(POINT *some_vector){
/**********************************************
 *	Forms the unit vector of some_vector and 
 returns the point co-ords of the unit vector
 **********************************************/
 POINT unitvector;
  unitvector.x = some_vector->x /sqrt(pow(some_vector->x,2)+pow(some_vector->y,2)+pow(some_vector->z,2) );
  unitvector.y = some_vector->y /sqrt(pow(some_vector->x,2)+pow(some_vector->y,2)+pow(some_vector->z,2) );
  unitvector.z = some_vector->z /sqrt(pow(some_vector->x,2)+pow(some_vector->y,2)+pow(some_vector->z,2) );
  
   return(unitvector);
  
 }/* END form_unitvector() */
 

 POINT add_twovectors(POINT *vector1, POINT *vector2){
/************************************************
*  Adds two vectors & returns the co-ords of the
   added vector
**************************************************/
POINT resultant;
resultant.x =  vector1->x + vector2->x;
resultant.y =  vector1->y + vector2->y;
resultant.z =  vector1->z + vector2->z;

return(resultant);
}/* END of add_twovectors */
	 


  POINT get_midpoint(POINT *pt1, POINT  *pt2){
/************************************************
*  gets the midpoint of two points 
**************************************************/
POINT midpoint;
midpoint.x =  (pt1->x + pt2->x)/2;
midpoint.y =  (pt1->y + pt2->y)/2;
midpoint.z =  (pt1->z + pt2->z)/2;

return(midpoint);
}/* END of get_midpoint */

POINT scale(POINT *some_vector, float factor){
/**********************************************
* scales some_vector by the factor
***********************************************/
POINT scaled_vector;
scaled_vector.x = some_vector->x * factor;
scaled_vector.y = some_vector->y * factor;
scaled_vector.z = some_vector->z * factor;

return(scaled_vector);
}/* END of scale() */	  



POINT formvector_betn_2pts(POINT *pt2, POINT  *pt1){
/***********************************************
 * forms the vector pointing from pt1 to pt2
 ***********************************************/
POINT vector;
vector.x = pt2->x - pt1->x;
vector.y = pt2->y - pt1->y;
vector.z = pt2->z - pt1->z;

return(vector);
}/*END of formvector_betn_2pts */



double get_dot_product(POINT *v1, POINT *v2){
/********************************************
 Calculates the dot_product of the vectors v1 and v2 
 dot-product = v1.x*v2.x + v1.y*v2.y + v1.z*v2.z 
 **********************************************/
double dot_product;
dot_product  = v1->x*v2->x + v1->y*v2->y + v1->z*v2->z ;
return(dot_product);
}


double get_norm (POINT *v){
/*******************************************
 Calculates the norm of a vector
 norm = sqrt(v.x^2 + v.y^2 + v.z^2)
 ******************************************/
double norm;
norm = sqrt( pow(v->x,2.00) + pow(v->y,2.00) + pow(v->z,2.00) );
return(norm);
}



POINT get_centroid(POINT *pt_1, POINT *pt_2, POINT *pt_3){
/**********************************************
 Calculates the centroid of three points 
 centroid.x = (pt_1.x + pt_2.x + pt_3.x)/3; and so on..
 *********************************************/
POINT centroid;
centroid.x = (pt_1->x + pt_2->x + pt_3->x)/3.00;
centroid.y = (pt_1->y + pt_2->y + pt_3->y)/3.00;
centroid.z = (pt_1->z + pt_2->z + pt_3->z)/3.00;

return(centroid);
}	




float get_distance(POINT *pt1, POINT *pt2)
{
/***********************************************************	
 * returns the distance between two points 
***********************************************************/ 
float l1, l2, l3;
	
	l1 = pow((pt1->x - pt2->x),2.00);
	l2 = pow((pt1->y - pt2->y),2.00);
	l3 = pow((pt1->z - pt2->z),2.00);

	return( sqrt (l1+l2+l3) );
} /* END float get_distance() */	


/*void form_distanceMatrix(POINT side1[], POINT side2[], MATRIX distance_matrix[][4]){*/
/************************************************
 Forms the distance matrix between the last 4 and first 4 
 residues of the two strands connecting the loop
************************************************/
/* int r, c;
 for(r=0; r<4; r++)
 {
	 for(c=0; c<4; c++)
	 {
		 distance_matrix[r][c] = get_distance(side1[r], side2[c]);
	 }
 }
	
}*/
	
void assign_first_res(EIGHTPTS_OF_RES *res, POINT **side){
	
	
	
   (*side)[0].x=(res)->first_N.x;  (*side)[0].y=(res)->first_N.y;  (*side)[0].z=(res)->first_N.z; 
   (*side)[1].x=(res)->first_CA.x; (*side)[1].y=(res)->first_CA.y; (*side)[1].z=(res)->first_CA.z; 
   (*side)[2].x=(res)->first_C.x;  (*side)[2].y=(res)->first_C.y;  (*side)[2].z=(res)->first_C.z; 
   (*side)[3].x=(res)->first_O.x;  (*side)[3].y=(res)->first_O.y;  (*side)[3].z=(res)->first_O.z; 
}	
	

void assign_last_res(EIGHTPTS_OF_RES *res, POINT **side){ 
	
	/*assign_pt( (*side)[0].N, res->last.N);
	assign_pt( (*side)[1].CA, res->last.CA);
	assign_pt( (*side)[2].C, res->last.C);
	assign_pt( (*side)[3].O, res->last.O);*/
	
			
			
   (*side)[0].x=(res)->last_N.x;  (*side)[0].y=(res)->last_N.y;  (*side)[0].z=(res)->last_N.z; 
   (*side)[1].x=(res)->last_CA.x; (*side)[1].y=(res)->last_CA.y; (*side)[1].z=(res)->last_CA.z; 
   (*side)[2].x=(res)->last_C.x;  (*side)[2].y=(res)->last_C.y;  (*side)[2].z=(res)->last_C.z; 
   (*side)[3].x=(res)->last_O.x;  (*side)[3].y=(res)->last_O.y;  (*side)[3].z=(res)->last_O.z; 
}		
	
	
	
	


int write_pdb_line (FILE * fp, PDB_LINE l) {
/***********************************************************
 * Write one pdb line to the given pdb using data from 
 * the given variable. Returns the number of items written.
 ***********************************************************/

  int ret;

  ret = fprintf (fp, PDB_WRITE_FORMAT,
                 l.flag,  l.atom_no, l.atom,  l.res,   l.res_no,
                 l.pt.x,  l.pt.y,    l.pt.z,  l.num1,  l.num2,
                 l.extra);

  return ret;

} /* END read_pdb_line () */

int read_pdb_line (FILE * fp, PDB_LINE * l) {
/***********************************************************
 * Read one line from the given pdb file and stores it in 
 * the given variable. Returns the number of items read or
 * EOF if end of file is encountered.
 ***********************************************************/
  
  int ret;

  ret = fscanf (fp, PDB_READ_FORMAT,
                l->flag,  &l->atom_no, l->atom,  l->res,   &l->res_no, 
                &l->pt.x, &l->pt.y,    &l->pt.z, &l->num1, &l->num2, 
                l->extra);

  return ret;

} /* END read_pdb_line () */

char res_str_to_char (char * res_name) {
/***********************************************************
 * Converts the residue's 3 character representation to it's
 * single character representation.
 ***********************************************************/

  if      (strcmp (res_name, "ALA") == 0) { return 'A'; }
  else if (strcmp (res_name, "ARG") == 0) { return 'R'; }
  else if (strcmp (res_name, "ASN") == 0) { return 'N'; }
  else if (strcmp (res_name, "ASP") == 0) { return 'D'; }
  else if (strcmp (res_name, "CYS") == 0) { return 'C'; }
  else if (strcmp (res_name, "GLN") == 0) { return 'Q'; }
  else if (strcmp (res_name, "GLU") == 0) { return 'E'; }
  else if (strcmp (res_name, "GLY") == 0) { return 'G'; }
  else if (strcmp (res_name, "HIS") == 0) { return 'H'; }
  else if (strcmp (res_name, "ILE") == 0) { return 'I'; }
  else if (strcmp (res_name, "LEU") == 0) { return 'L'; }
  else if (strcmp (res_name, "LYS") == 0) { return 'K'; }
  else if (strcmp (res_name, "MET") == 0) { return 'M'; }
  else if (strcmp (res_name, "PHE") == 0) { return 'F'; }
  else if (strcmp (res_name, "PRO") == 0) { return 'P'; }
  else if (strcmp (res_name, "SER") == 0) { return 'S'; }
  else if (strcmp (res_name, "THR") == 0) { return 'T'; }
  else if (strcmp (res_name, "TRP") == 0) { return 'W'; }
  else if (strcmp (res_name, "TYR") == 0) { return 'Y'; }
  else if (strcmp (res_name, "VAL") == 0) { return 'V'; }
  else                                    { return -1;  }

} /* END res_str_to_char () */

char * char_to_res_str (char res) {
/***********************************************************
 * Converts the residue's single character representation
 * to its 3 character representation.
 ***********************************************************/

  switch (res) {
  case 'A': return "ALA"; break;
  case 'R': return "ARG"; break;
  case 'N': return "ASN"; break;
  case 'D': return "ASP"; break;
  case 'C': return "CYS"; break;
  case 'Q': return "GLN"; break;
  case 'E': return "GLU"; break;
  case 'G': return "GLY"; break;
  case 'H': return "HIS"; break;
  case 'I': return "ILE"; break;
  case 'L': return "LEU"; break;
  case 'K': return "LYS"; break;
  case 'M': return "MET"; break;
  case 'F': return "PHE"; break;
  case 'P': return "PRO"; break;
  case 'S': return "SER"; break;
  case 'T': return "THR"; break;
  case 'W': return "TRP"; break;
  case 'Y': return "TYR"; break;
  case 'V': return "VAL"; break;
  default : return "FIL"; 
  } /* END _switch_ */

} /* END char_to_res_str () */

boolean is_helix (char * extra) {
/***********************************************************
 * Returns true if extra is HELIX
 ***********************************************************/

  if (strcmp (extra, helix) == 0) { return true;  }
  else                              { return false; }

} /* END is_helix () */

boolean is_coil (char * extra) {
/***********************************************************
 * Returns true if extra is COIL
 ***********************************************************/

  if (strcmp (extra, coil) == 0) { return true;  }
  else                              { return false; }

} /* END is_coil () */






boolean is_ref_atom (char * atom) {
/***********************************************************
 * Returns true if given atom is the rotation reference
 * point atom.
 ***********************************************************/

  if (strcmp (atom, REF_ATOM) == 0) { return true;  }
  else                              { return false; }

} /* END is_ref_atom () */


boolean is_N_atom (char * atom) {
/***********************************************************
 * Returns true if given atom is the Nitrogen atom.
 ***********************************************************/

  if (strcmp (atom, N_ATOM) == 0) { return true;  }
  else                              { return false; }

} /* END is_N_atom () */


boolean is_C_atom (char * atom) {
/***********************************************************
 * Returns true if given atom is the Carbon atom.
 ***********************************************************/

  if (strcmp (atom, C_ATOM) == 0) { return true;  }
  else                              { return false; }

} /* END is_C_atom () */


boolean is_CB_atom (char * atom){
/***********************************************************
 * Returns true if given atom is the CB atom.
 ***********************************************************/	
if( strcmp(atom,CB_ATOM) == 0) { return true; }
else                            {return false;}	
}/* END is_CB_atom () */


boolean is_O_atom (char * atom){
/***********************************************************
 * Returns true if given atom is the Oxygen atom.
 ***********************************************************/	
if (strcmp(atom,O_ATOM) == 0) { return true; }
else                            {return false;}	
}/* END is_O_atom () */



boolean is_GLY_residue (char * res){
/***********************************************************
 * Returns true if given res is GLYCINE.
 ***********************************************************/	
if( strcmp(res, GLYCINE) == 0) { return true; }
else                            {return false;}	
}/* END is_O_atom () */



/* END MODULE: util.c */
