#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>

/*******************************************************


C implementation of the k-nearest neighbour algorithm
for amino acid composition based classification
         
                                       Andrew Garrow

a.out   query_file   k_value   cross_detection_num   query_type


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.



*******************************************************/

/*---------------------------------------------------------

                   Functions 

---------------------------------------------------------*/

void query_seqs(char field);
void distance(float *training_seq);
void quick(float *items, char *types, int count);
void qs(float *items, char *types, int left, int right);




/*---------------------------------------------------------

                  Global variables 

---------------------------------------------------------*/


/* @AA types [19]*/

char aa_types[20]={'A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y'};




/* @Query sequence AA features [19]*/

int query_aa_composition[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
float query_aa_percentage[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
float aa_ga_weighting[20];


/* other variables */

float scores[100000]; 	/* actual scores*/
char score_type[10000]; /* instance classifications */
int score_num=0;	/* for where to place a score in score array*/
float sorted_items[100000];
char sorted_types[100000];

int current_query;	/* variable for cross-validation studies */
float w;
int num_queries;




/*---------------------------------------------------------

            MAIN

---------------------------------------------------------*/

int main(int argc, char *argv[])
{

FILE *fp, *zfp;
char line[5000];	/* File input */
char string[5000];	/* File input */
char seq[5000];		/* Sequence */
char *p;
char field[1];		/* b, a or g */
int total=0;		/* counter */
int n=0, q, i, ii, iii, count=0;	/* local counters */
float items[100000];	/* scores unsorted */
float perc;		/* % for aa composition */
char types[100000];	/* target functions unsorted */
float sorted_weights[1000];
float a_score=0.0, b_score=0.0, g_score=0.0;
int k;
float weighting_factor;
float sum_score;
float zscore[1];

int pos_sign;
int neg_sign;

float obs_neg_z;
float exp_neg_z;
float obs_pos_z;
float exp_pos_z;
float obs_neg_z_two;
float exp_neg_z_two;
float obs_neg_z_one;
float exp_neg_z_one;
float obs_pos_z_two;
float exp_pos_z_two;
float obs_pos_z_one;
float exp_pos_z_one;

float obs_neg_p;
float exp_neg_p;
float obs_pos_p;
float exp_pos_p;

float obs_s;
float obs_p;

float exp_s;
float exp_p;

/* numbers derived from calibration */
float neg_mean=-0.222613;
float pos_mean=0.060967;
float neg_stdev=0.110265;
float pos_stdev=0.189202;




k=atoi(argv[2]);
current_query=atoi(argv[3]);
w=atoi(argv[4]);
w=w/100;
num_queries=atoi(argv[5]);



for(i=0; i<5000; ++i)	/* clearing seq array */
	{
	seq[i]='\0';
	}




/* get GA vector */

if ((fp=fopen("ga_vector", "r"))==NULL) {
        fprintf(stderr, "error opening ga_vector input file\n");
        exit(1);
        }

while(1) {
        if(fgets(line, sizeof(line), fp) == NULL) {
                break;
                }
        sscanf (line, "%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f", &aa_ga_weighting[0], &aa_ga_weighting[1], &aa_ga_weighting[2], &aa_ga_weighting[3], &aa_ga_weighting[4],  &aa_ga_weighting[5], &aa_ga_weighting[6], &aa_ga_weighting[7], &aa_ga_weighting[8],  &aa_ga_weighting[9], &aa_ga_weighting[10], &aa_ga_weighting[11], &aa_ga_weighting[12], &aa_ga_weighting[13],  &aa_ga_weighting[14], &aa_ga_weighting[15], &aa_ga_weighting[16], &aa_ga_weighting[17],  &aa_ga_weighting[18], &aa_ga_weighting[19]);
        }



printf("%s", "GA vector:\n");
for(iii=0; iii<20; ++iii)
        {
        printf("%d%s%f%s",  iii, ": ", aa_ga_weighting[iii], "\t");
        }
printf("%s", "\n\n");




if ((fp=fopen(argv[1], "r"))==NULL) {
        fprintf(stderr, "error opening input file argv 1\n");
        exit(1);
        }



while(1) {

        if(fgets(line, sizeof(line), fp) == NULL) {
                break;
                }

	sscanf(line, "%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f", &query_aa_percentage[0], &query_aa_percentage[1], &query_aa_percentage[2], &query_aa_percentage[3], &query_aa_percentage[4], &query_aa_percentage[5], &query_aa_percentage[6], &query_aa_percentage[7], &query_aa_percentage[8], &query_aa_percentage[9], &query_aa_percentage[10], &query_aa_percentage[11], &query_aa_percentage[12], &query_aa_percentage[13],  &query_aa_percentage[14], &query_aa_percentage[15], &query_aa_percentage[16], &query_aa_percentage[17], &query_aa_percentage[18], &query_aa_percentage[19]);
	}






/* read in query sequence (avoid ^> line) */
/*
if ((fp=fopen(argv[1], "r"))==NULL) {
	fprintf(stderr, "error opening input file argv 1\n");
	exit(1);
	}

while(1) {

	if(fgets(line, sizeof(line), fp) == NULL) {
		break;
		}
	sscanf(line, "%s", string);
	
	p = strchr(line, '>');
	if(p==NULL)
		{ 
		if(n==1) {
			strncat(seq, line, 5000);
			}
		if(n==0) {
                        strcpy(seq, line);
                        n=1;
                        }
		}
	}



for(i=0; i<20; ++i)
	{
	for(ii=0; ii<5000; ++ii)
		{
		if(seq[ii]==aa_types[i]) 
			{
			query_aa_composition[i]++;
			total++;
			}
		}
	}

for(i=0; i<20; ++i)
	{
	perc = ((float) query_aa_composition[i]/total*100);
	query_aa_percentage[i]=perc;
	}
*/




/* query diff datasets */

field[0]='b';
query_seqs(*field);
field[0]='a';
query_seqs(*field);
field[0]='g';
query_seqs(*field);



/* sort neighbours */

for(i=0; i<score_num; ++i)	/* builds an array for sorting */
	{
	items[i]=scores[i];
	types[i]=score_type[i];
	}
count=score_num;
quick(items, types, count); 	/* sorts items in numerical order*/



/* print output */

printf("%s", "\n\nSorted items:\n\n");

for(i=0; i<score_num; ++i)
        {
        printf("%f%s%c%s", sorted_items[i], ": ", sorted_types[i], "\n");
        }




for(i=0; i<k; ++i)
	{
	q=0;
	if(sorted_items[i] < 0.00001) { q=1; /* sorted_items[i]=0.1; */  }
	sorted_weights[i]=1/sorted_items[i];

	if(q==0) {	/* just a check to ensure not self-detecting 
				- will alter !!				*/

		if(sorted_types[i]=='a') {
			a_score+=sorted_weights[i];
			}
		else if(sorted_types[i]=='g') {
       		        g_score+=sorted_weights[i];
               		}
		else if(sorted_types[i]=='b') {
        	        b_score+=sorted_weights[i];
                	}

		}
	}


printf ("%s%f%s%f%s%f%s", "\nk-nearest neighbour target function scores:\nb - ", b_score, "\na - ", a_score, "\ng - ", g_score, "\n\n");


sum_score=b_score-g_score;
sum_score=sum_score-a_score;
printf("%s%f%s", "Raw score: ", sum_score, "\n");




printf("%s", "\n\n-----------------------\n\nProbability based scoring:\n\n-----------------------\n\n");

/*sum_score=1.5;*//* testing stats*/

exp_neg_z=(sum_score-neg_mean)/neg_stdev;
exp_pos_z=(sum_score-pos_mean)/pos_stdev;

/* for reading z score table */
exp_neg_z=exp_neg_z*100;
exp_pos_z=exp_pos_z*100;

if(exp_pos_z>0) { pos_sign=1; } else { pos_sign=0; }
if(exp_neg_z>0) { neg_sign=1; } else { neg_sign=0; }

exp_pos_z=exp_pos_z*exp_pos_z;
exp_pos_z=sqrt(exp_pos_z);
exp_neg_z=exp_neg_z*exp_neg_z;
exp_neg_z=sqrt(exp_neg_z);

i=1;
if ((zfp=fopen("zscores_read.txt", "r"))==NULL) {
        fprintf(stderr, "error opening zscores input file\n");
        exit(1);
        }
while(1) {
        if(fgets(line, sizeof(line), zfp) == NULL) {
                break;
                }
        sscanf(line, "%f", &zscore[0]);

                if(pos_sign==0) {
                        if((i<=exp_pos_z+1))
                                {
                                exp_pos_p=zscore[0];
                                }
                        }
                else    {
                         if((i<=exp_pos_z+1))
                                {
                                exp_pos_p=1-zscore[0];
                                }
                        }

                if(neg_sign==0) {
                        if((i<=exp_neg_z+1))
                                {
                                exp_neg_p=1-zscore[0];
                                }
                        }
                else    {
                        if((i<=exp_neg_z+1))
                                {
                                exp_neg_p=zscore[0];
                                }
                        }
        i++;

        }

printf("%s", "Expected scores:-\n\n");

printf("%s%f%s", " Probability of a NON-BETA BARREL protein with this score: ", exp_neg_p, "\n");

exp_s=exp_pos_p-exp_neg_p;
printf("%s%f%s", " ExpDataBBTM_score: ",  exp_s, "\n");

exp_s=exp_pos_p/exp_neg_p;
exp_s=log(exp_s);
printf("%s%f%s", " ExpDataBBTM_ratio: ",  exp_s, "\n");

exp_s=exp_neg_p*num_queries;
printf("%s%f%s", " ExpDataE-value: ",  exp_s, "\n");






if(exp_s > 0)
        { printf("\nConclusion: BBMP"); }
else { printf("\nConclusion: Non BBMP"); }

printf("%s", "\n\n------------------------\n");


}













/* Reads in training instances and sends them to be 
   queried against query instance - query_seq_aa_percentage global 
*/

void query_seqs(char field)
{
FILE *fp;
FILE *ifp;
char file[50]; 
float training_seq[20];
float *tq;
char line[5000];
char linex[5000];
/*int query_num=1;*/
int query_num=0;
int line_num=1;
char leave_out_seqs_file[20];
int use_seq=1;
int leave_out_seqs[150];
int count;
char *tok;
char s1[]="\t";


if(field=='b')
	{
	strcpy(file, "bbtm_membrane_23_weighted.perc");
	}
else if (field=='a')
	{
	strcpy(file, "ahtm_membrane_23_weighted.perc");
	}
else if (field=='g')
        {
        strcpy(file, "non_membrane_23_weighted.perc");
	}




/* read in query sequence (avoid ^> line) */

if ((fp=fopen(file, "r"))==NULL) {
        fprintf(stderr, "error opening input file\n");
        exit(1);
        }


while(1) {

        if(fgets(line, sizeof(line), fp) == NULL) {
                break;
                }
	
	sscanf(line, "%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f", &training_seq[0], &training_seq[1], &training_seq[2], &training_seq[3], &training_seq[4],  &training_seq[5], &training_seq[6], &training_seq[7], &training_seq[8],  &training_seq[9], &training_seq[10], &training_seq[11], &training_seq[12], &training_seq[13],  &training_seq[14], &training_seq[15], &training_seq[16], &training_seq[17],  &training_seq[18], &training_seq[19]);
	

	/* This line determines whether homs included or not */
	
	count=0;
	use_seq=1;
	for(count=0; count<150; count++)
		{
                if(query_num==leave_out_seqs[count]) {
                        use_seq=0;
                        }
		}


	tq=training_seq;
	distance(tq);
	score_type[score_num]=field;

	if(field=='b') {
                scores[score_num]=scores[score_num]/w;
                }
        else if (field=='a') {
                scores[score_num]=scores[score_num]/w;
                }

	score_num++;
	query_num++;

        }

}









void distance(float *tq)
{
int i;
float training_aa_percentage[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
float aa_difs[20];
float sum_distances=0;

/* calcualate AA composition of this seq and euclidean distance */
/* parse that number to scores */

for(i=0; i<20; ++i)
	{
	aa_difs[i]=tq[i]-query_aa_percentage[i];
	aa_difs[i]=aa_difs[i]*aa_difs[i];
	aa_difs[i]=aa_difs[i]*aa_ga_weighting[i];
	sum_distances+=aa_difs[i];
	}

scores[score_num]=sqrt(sum_distances);

}







/* sorting algorithm functions */

void quick(float *items, char *types, int count)
{
qs(items, types, 0, count-1);
}






void qs(float *items, char *types, int left, int right)
{
int ii;
register int i, j;
float x, y;
char z;

i=left; j=right;
x=items[(left+right)/2];

do {
	while((items[i] < x) && (i < right)) i++;
	while((x < items[j]) && (j > left)) j--;

	if (i<=j) {
		y=items[i];
		z=types[i];
		items[i] = items[j];
		types[i] = types[j];
		items[j] = y;
		types[j] = z;
		i++; j--;
		}
	} while (i <= j);

if(left < j) qs(items, types, left, j );
if(i < right) qs(items, types, i, right);

else{
	for(ii=0; ii<score_num; ++ii)
	        {
       		sorted_items[ii]=items[ii];
        	sorted_types[ii]=types[ii];
		}
 	}
}
