
/*
	file: getinp.c
	user input routines
*/

#include "../include/seqsee.h"

static int read_menu_num();
static FILE *read_data_file();
extern void show_seq_file(), show_seq_file1(), show_seq_file2();
extern void show_prompt(), show_top_num(), show_err();
extern void show_mseq_files();
extern void show_msg(), exit();


/*************************************************************/
/* input several amino acid sequences from several files 
*/
int mget_seqs(dataFile, outfp, rSeq, fileType, maxSeqNum, aaMin, aaMax, mNum)
char *dataFile;
FILE *outfp;
SEQ_REC *rSeq;
int fileType, aaMin, aaMax, maxSeqNum, *mNum; 
{
	int num, okFlag;
	char sVal[MAX_LINE_LEN];
	FILE *datafp;

	num = 0;

	datafp = fopen(dataFile, "r");
	okFlag = mread_seq(datafp, outfp, rSeq, &num, maxSeqNum,
		fileType, aaMin, aaMax);

	if (okFlag == FALSE) {
		sprintf(sVal, "Cannot continue alignment until errors fixed...");
		show_err(TRUE, "mget_seqs", sVal);
	}

	fprintf(outfp, "\nNumber of sequences to align: %d", num);

	if (num <= 1) {
		sprintf(sVal, "Must have 2 or more sequences to align.");
		show_err(TRUE, "mget_seqs", sVal);
	}

	*mNum = num;
	return(TRUE);

}	/* mget_seqs */


/*************************************************************/
/* input a single amino acid sequence from one file 
*/
int get_seq(infp, outfp, rSeq, aaMin, aaMax, fileType)
FILE *infp, *outfp;
SEQ_REC *rSeq;
int aaMin, aaMax, fileType;
{
	FILE *datafp;
	int okFlag;
	char fname[MAX_FNAME_LEN];

	datafp = read_data_file(infp, fname, TRUE);
	okFlag = read_seq(datafp, outfp, rSeq, fileType, aaMin, aaMax);

	if (okFlag == TRUE) {
		fprintf(outfp, "\nProtein: %s\n", rSeq->title);
		fprintf(outfp, "Number of amino acids: %d\n", rSeq->len);
	}

	return(TRUE);

}	/* get_seq */


/*************************************************************/
/* Get number which determines top number of records to keep track of 
*/
int get_top_num(infp, outfp, msg, topMin, topMax)
FILE *infp, *outfp;
char *msg;
int topMin, topMax;
{	
	int	topNum;

	show_top_num(outfp, msg, topMin, topMax);
	topNum = read_menu_num(infp, topMin, topMax, TRUE);
	return(topNum);

}	/* get_top_num */


/*************************************************************/
/* read integer response to menu item, do range checking
*/
static int read_menu_num(infp, xMin, xMax, quitFlag)
FILE *infp;
int xMin, xMax, quitFlag;
{
	int select;
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	fscanf(infp, "%s", aStr);
	select = atoi(aStr);

	if ((select < xMin) || (select > xMax)) {
		select = ERROR_FLAG;
		sprintf(sVal, "Range error, Input=%d where Min=%d, Max=%d\n",
			select, xMin, xMax);
		show_err(quitFlag, "read_menu_num", sVal);
	}

	return(select);

}   /* read_menu_num */


/*************************************************************/
/* read a datafile name
*/
static FILE *read_data_file(fp, fname, quitFlag)
FILE *fp;
char *fname;
int quitFlag;
{
	FILE *datafp;
	char sVal[MAX_LINE_LEN];

	fscanf(fp, "%s", fname);
	datafp = fopen(fname, "r");

	if (datafp == NULL) {
		sprintf(sVal, "Cannot open data file: '%s'\n", fname);
		show_err(quitFlag, "read_data_file", sVal);
	}

	return(datafp);

}   /* read_data_file */
