
/*
	file: math.c
	purpose: mathematical utilites for SEQSEE.
*/

#include "../include/seqsee.h" 

/* procedure calls */
extern void show_err();
extern double sqrt(), pow();
extern int abs();


/*************************************************************/
/* Randomize the entries in an integer array. 
*/
int randomize_array(arr, len, seed)
register int *arr;
register int len, seed;
{
	register int i, ix;
	register int temp;

	for (i=0; i<len; i++) {
		seed = RAND(seed);
		ix = seed % len;
		temp = arr[i];			/* do swap */
		arr[i] = arr[ix];
		arr[ix] = temp;
	}

	return(seed);

}	/* randomize_array */


/*************************************************************/
/* Calculate test stat given data array and mean. 
*/
double calc_test_stat(yData, nData, mean)
int *yData, nData;
double mean;
{
	register int i, ySum, ySum2;
	double yBar, sDev, tStat;

	if (nData < 2) {
		show_err(TRUE, "calc_test_stat", "Number of data < 2.");
	}

	ySum = 0;
	ySum2 = 0;
	for (i=0; i<nData; i++) {
		ySum += yData[i];
		ySum2 += yData[i] * yData[i];
	}

	yBar = ySum / (double)nData;
	sDev = sqrt((nData*ySum2 - ySum*ySum) / (double)(nData * (nData-1)));
	tStat = (mean - yBar) / sDev; 

	return(tStat);

}	/* calc_test_stat */


/*************************************************************/
/* Calculate test stat given data array and mean. 
*/
double calc_fast_test_stat(mStr, dStr, nwScor, wtMean, wtVar)
char *mStr, *dStr;
int nwScor;
double wtMean, wtVar;
{
	int size;
	int mStart, mEnd, dStart, dEnd;
	int s1, s2, fit;
	double tStat;

	s1 = calc_align_strlen(mStr, &mStart, &mEnd);
	s2 = calc_align_strlen(dStr, &dStart, &dEnd);
	size = 6 + MIN(s1, s2);

	fit = abs(mStart - dStart);
	if (fit < 20)
		nwScor *= 1.0 + (0.0005 * (100 - (0.25 * fit*fit)));

	fit = abs(mEnd - dEnd);
	if (fit < 20)
		nwScor *= 1.0 + (0.0005 * (100 - (0.25 * fit*fit)));
	
	tStat = ((nwScor / (double)size) - wtMean) / sqrt((wtVar / (double)size));
	return(tStat);

}	/* calc_fast_test_stat */


/*************************************************************/
/* Calculate statistics for a similarity matrix
*/
void calc_wtmat_stats(wtMat, yBar, sDev)
int *wtMat;
double *yBar, *sDev;
{
	register int i, j, scor, nData;
	int ySum, ySum2;

	ySum = 0;
	ySum2 = 0;
	nData = 0;

	for (i=0; i<CODES; i++) {
		if AA_ICODE(i) {
			for (j=0; j<=i; j++) {
				if AA_ICODE(j) {
					scor = wtMat[i * WT_MAT_DIM + j];
					ySum += scor;
					ySum2 += scor * scor;
					nData++;
				}
			}
		}
	}

	*yBar = ySum / (double)nData;
	*sDev = sqrt((nData*ySum2 - ySum*ySum) / (double)(nData * (nData-1)));

}	/* calc_wtmat_stats */


/*************************************************************/
/* Calculate "true" alignment gap penalties from parameter values 
*/
int calc_gap_pen(wtMean, gapPen)
double wtMean;
int gapPen;
{
	double kTerm, kAve;
	int val;
	
	kTerm = 5.0;
	kAve = 50.0;

	val = (int)((kTerm * wtMean) * (gapPen / kAve));

	return(val);

}	/* calc_gap_pen */
