
/*
	file: misc.c
	purpose: This file contains low level utility routines for SEQSEE.
*/

#include "../include/seqsee.h" 
#include <stdlib.h>

/* procedure calls */
extern void show_err_mem(), show_err();
extern void exit();
/*extern int *calloc();
extern char *malloc();
*/

/*************************************************************/
/* calloc with error check
*/
int *ecalloc(len, size, msg)		
int len, size;
char *msg;
{
	int	*sp;

	if ((sp = (int *)calloc((unsigned)len, (unsigned)size)) == NULL) {
		show_err_mem(msg);
		exit(1);
	}

	return(sp);

}	/* ecalloc */


/*************************************************************/
/* malloc with error check
*/
char *emalloc(size, msg)		
int size;
char *msg;
{
	char *sp;

	if ((sp = (char *)malloc(size)) == NULL) {
		show_err_mem(msg);
		exit(1);
	}

	return(sp);

}	/* emalloc */


/*************************************************************/
/* converts a string of characters 'inStr' to upper case letters. 
*/
void to_upper(inStr, outStr)
char *inStr, *outStr;
{
	register int k, len;

	len = strlen(inStr);
	for (k=0; k<=len; k++)
		outStr[k] = AA_UC(inStr[k]);

}	/* to_upper */


/*************************************************************/
/* utility that subtracts 'A' from a string of characters.
*/
void chars_to_nums(len, inStr, outNums)
int len;
char *inStr;
int *outNums;
{
	register int k, ix;

	for (k=0; k<len; k++) {
		ix = AA_IX(inStr[k]);
		if ((ix >= 0) && (ix < CODES))
			outNums[k] = ix;
		else
			outNums[k] = CODES;
	}

}   /* chars_to_nums */


/*************************************************************/
/* utility that returns number of identical matches in 2 strings.
*/
int string_match_num(str1, str2, len1, len2)
char *str1, *str2;
int	len1, len2;
{
	register int k, minimum, ident;

	ident = 0;
	minimum = len1<len2 ? len1 : len2;

	for (k=0; k<minimum; k++)
		if (*str1++ == *str2++)
			ident++;

	return(ident);

}	/* string_match_num */


/*************************************************************/
/* utility that removes blanks, tabs, newlines from a string.
** adds end of string character and returns new string len.
*/
int remove_blanks(line, len, newLine)
char *line, *newLine;
int len;
{
	register int j, count, ch;

	count = 0;
	for (j=0; j<len; j++) {
		ch = line[j];
		if ((ch != ' ') && (ch != '\t') && (ch != '\n'))
			newLine[count++] = ch;
	}

	newLine[count] = '\0';
	return(count);

}   /* remove_blanks */


/*************************************************************/
/* insert the appropriate blanks into a string according to a
** template string. 
** eg, change "abccba" using "x xxx xx" to "a bcc ba"
*/
void str_match_blanks(inStr, iLen, tStr, tLen, tGapChar, outStr)
int iLen, tLen;
char tGapChar, *tStr, *inStr, *outStr;
{
	register int i, ix, bNum;
	char sVal[MAX_LINE_LEN];

	bNum = count_chars_in_string(tStr, tLen, ' ');
	if ((tLen - bNum) != iLen) {
		sprintf(sVal, "Program check: inStr = %s", inStr);
		show_err(TRUE, "str_match_blanks", sVal);
	}

	ix = 0;
	for (i=0; i<tLen; i++) {
		if (tStr[i] == tGapChar) {
			outStr[i] = tGapChar;
		}
		else {
			outStr[i] = inStr[ix];
			ix++;
		}
	}

}	/* str_match_blanks */


/*************************************************************/
/* change all occurrences of char 'x' to char 'y' in string 'z'.
*/
void str_scan(inStr, iLen, ich, och)
char *inStr;
int iLen, ich, och;
{
	register int i;

	for (i=0; i<iLen; i++) {
		if (inStr[i] == ich)
			inStr[i] = och;
	}

}	/* str_scan */


/*************************************************************/
/* given a string 'z' count the number of occurrences of char 'x'.
*/
int count_chars_in_string(aStr, len, ch)
char *aStr, ch;
int len;
{
	register int j, count;

	count = 0;
	for (j=0; j<len; j++) {
		if (aStr[j] == ch)
			count++;
	}

	return(count);

}	/* count_chars_in_string */


/*************************************************************/
/* return length of string minus the blanks at the beginning
** and end of the string.
*/
int calc_align_strlen(aStr, pStart, pEnd)
char *aStr;
int *pStart, *pEnd;
{
	register int j, count, len;

	count = 0;
	len = strlen(aStr);
	for (j=0; j<len; j++) {
		if (aStr[j] == ' ')
			count++;
		else
			break;
	}
	*pStart = count;

	count = strlen(aStr);
	for (j=len-1; j>=0; j--) {
		if (aStr[j] == ' ')
			count--;
		else
			break;
	}
	*pEnd = count;

	if (count < 0)
		return(0);

	return((*pEnd) - (*pStart));

}	/* calc_align_strlen */
