
/*
	file: readseq.c
	common routines for reading sequences for all format types
*/

#include "../include/seqsee.h"

/* procedure calls */
extern int read_seq_pir(), read_seq_sprot(), read_seq_pep();
extern int read_seq_pir_comp(), read_seq_rbo(), read_seq_keyboard();
extern void show_err();


/*************************************************************/
/* reads multiple sequences from one data file 
** exit when we have read enough sequences or EOF
*/
int mread_seq(infp, outfp, mySeq, seqIx, maxSeqCount, format, aaMin, aaMax)
FILE *infp, *outfp;
SEQ_REC	*mySeq;
int *seqIx, maxSeqCount;
int format, aaMin, aaMax;
{	
	int	ix, okFlag;
	SEQ_REC *rSeq;

	ix = *seqIx;
	okFlag = TRUE;

	while (okFlag == TRUE) {
			
		rSeq = &mySeq[ix];
		okFlag = read_seq(infp, outfp, rSeq, format, aaMin, aaMax);

		if (okFlag == TRUE) {
			ix++;
			if (ix >= maxSeqCount)
				break;
		}

	}

	*seqIx = ix;
	return(okFlag);

}	/* mread_seq */


/*************************************************************/
/* reads one amino acid sequence from a file. 
** Return FALSE if a problem occurred 
** Return EOF in no sequence read.
*/
int read_seq(infp, outfp, rSeq, format, aaMin, aaMax)
FILE *infp, *outfp;
SEQ_REC	*rSeq;
int format, aaMin, aaMax;
{
	int okFlag;
	char sVal[MAX_LINE_LEN];

	switch (format) {

		case SEQ_PIR:
			okFlag = read_seq_pir(infp, rSeq, aaMin, aaMax); break;

		case SEQ_SPROT:
			okFlag = read_seq_sprot(infp, rSeq); break;

		case SEQ_PEP:
			okFlag = read_seq_pep(infp, rSeq, aaMin, aaMax); break;

		case SEQ_PIRCOMP:
			okFlag = read_seq_pir_comp(infp, rSeq); 
			if (rSeq->len > aaMax)
				rSeq->len = aaMax;
			break;

		case SEQ_RBO:
			okFlag = read_seq_rbo(infp, rSeq, aaMin, aaMax); break;

		case SEQ_KBD:
			okFlag = read_seq_kbd(infp, outfp, rSeq, aaMin, aaMax); break;
				
		default:
			sprintf(sVal, "Illegal sequence format: %d\n", format);
			show_err(TRUE, "read_seq", sVal);

	}

	return(okFlag);
		 
}	/* read_seq */


/*************************************************************/
/* release storage associated with one sequence 
*/
void free_seq(dSeq)
SEQ_REC *dSeq;
{

	if (dSeq->title != NULL)
		free(dSeq->title);
	if (dSeq->acc != NULL)
		free(dSeq->acc);
	if (dSeq->organism != NULL)
		free(dSeq->organism);
	if (dSeq->ref != NULL)
		free(dSeq->ref);
	if (dSeq->pRes != NULL)
		free(dSeq->pRes);
	if (dSeq->sRes != NULL)
		free(dSeq->sRes);
	if (dSeq->uBuf != NULL)
		free(dSeq->uBuf);

}	/* free_seq */
