
/*
	file: rseq_kbd.c
	common routines for reading sequences from the keyboard
*/

#include "../include/seqsee.h"

/* procedure calls */
static void print_ruler();
extern void show_err();
extern char *emalloc();


/*************************************************************/
/* read user's query sequence from keyboard. 
*/
int read_seq_kbd(infp, outfp, rSeq, aaMin, aaMax)
FILE *infp, *outfp;
SEQ_REC	*rSeq;
int aaMin, aaMax;
{
	register int i, len, ch, doneFlag;
	char aaBuf[MAX_RES];
	char sVal[MAX_LINE_LEN], line[MAX_LINE_LEN];

	fprintf(outfp, "\n Enter title for the sequence. Use underscores instead ");
	fprintf(outfp, "\n of blanks to separate words (eg. thioredoxin_human).\n");
	fprintf(outfp, PROG_PROMPT);
	fscanf(infp, "%s", line);
	if (line[(strlen(line)) - 1] != '\n')
		strcat(line, "\n");

	rSeq->title = (char *)emalloc(strlen(line), "read_seq_kbd");
	strcpy(rSeq->title, line);

	strcpy(rSeq->acc, "000000");

	fprintf(outfp, "\n Enter each amino acid (one letter code). ");
	fprintf(outfp, "\n You may enter up to 50 amino acids on one line. ");
	fprintf(outfp, "\n Press <return> to get a new prompt line.");
	fprintf(outfp, "\n When you are done enter $ and press <return>.\n");
	print_ruler(outfp, 3);
	fprintf(outfp, PROG_PROMPT);
	fflush(outfp);

	doneFlag = FALSE;
	len = 0;
	ch = getc(infp);

	while (doneFlag == FALSE) {
		i = 0;
		while (i < 50) {
			ch = getc(infp);
			if (ch == '\n') {
				print_ruler(outfp, 3);
				fprintf(outfp, PROG_PROMPT);
				fflush(outfp);
				break;
			}
			if (ch == END_OF_SEQ) {
				doneFlag = TRUE;
				break;
			}
			if (AA_CVALID(ch) == FALSE) {
				sprintf(sVal, "Illegal amino acid '%c' in %s\n",
					ch, rSeq->title);
				show_err(FALSE, "read_seq_kbd", sVal);
				return(FALSE);
			}
			if (len < aaMax) {
				aaBuf[len++] = ch;
			}
			else {
				sprintf(sVal, "Too many amino acids (Max = %d) in %s\n",
					aaMax, rSeq->title);
				show_err(FALSE, "read_seq_kbd", sVal);
				return(FALSE);
			}
			i++;
		}
	}

	rSeq->pRes = (char *)emalloc(len + 1, "read_seq_kbd");
	strncpy(rSeq->pRes, aaBuf, len);
	rSeq->pRes[len] = '\0';
	to_upper(rSeq->pRes, rSeq->pRes);
	rSeq->len = strlen(rSeq->pRes);

	if (len < aaMin) {
		sprintf(sVal, "Too few amino acids (Min = %d) in %s\n",
			aaMin, rSeq->title);
		show_err(FALSE, "read_seq_kbd", sVal);
		return(FALSE);
	}

	return(TRUE);

}	/* read_seq_kbd */


/*************************************************************/
/* show sequence ruler 
*/
static void print_ruler(fp, numBlanks)
FILE *fp;
{
int i;

	fputc('\n', fp);
	for (i=0; i<numBlanks; i++)
		fputc(' ', fp);
	fprintf(fp, "         1         2         3         4         5");

	fputc('\n', fp);
	for (i=0; i<numBlanks; i++)
		fputc(' ', fp);
	fprintf(fp, "12345678901234567890123456789012345678901234567890");

	fputc('\n', fp);
	for (i=0; i<numBlanks; i++)
		fputc(' ', fp);
	fprintf(fp, "         |         |         |         |         |");

}	/* print_ruler */
