
/*
	file: rseq_pep.c
	routines for reading pepTools format protein sequences
*/

#include "../include/seqsee.h"

/* procedure calls */
extern char *emalloc();


/*************************************************************/
/* Reads one protein sequence (standard format)  
** Return number of amino acids in the sequence.
*/
int read_seq_pep(fp, rSeq, aaMin, aaMax)
FILE *fp;
SEQ_REC	*rSeq;
int aaMin, aaMax;
{
	char *rc;
	register int i, nd, quitFlag, len, rlen, slen, ulen;
	char line[MAX_RES];
	char rBuf[MAX_RES], sBuf[MAX_RES], uBuf[MAX_RES];
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	rSeq->len = 0;

/* look for start of sequence entry */

	while (TRUE) {

		rc = fgets(line, MAX_LINE_LEN-1, fp);
		if (rc == NULL) {
			return(EOF);
		}

		nd = sscanf(line, "%s%s", aStr, sVal);
		if (nd == 1) {
			if (0 == strcmp(aStr, "PSEQ"))
				break;
		}
	}

/* process fields of sequence */

	rlen = slen = ulen = 0;
	quitFlag = FALSE;
	rSeq->sFlag = 1;
	rSeq->uFlag = 1;
	while (quitFlag == FALSE) {

		rc = fgets(line, MAX_RES-1, fp);
		if (rc == NULL) {
			sprintf(sVal, "Unexpected EOF in sequence file: '%s'", rSeq->title);
			show_err(FALSE, "read_seq_pep", sVal);
			return(EOF);
		}

		len = strlen(&line[1]) - 1;

		switch (line[0]) {

			case 'n':   rSeq->title = (char *)emalloc((len+1),
							"read_seq_pep:title");
						strncpy(rSeq->title, &line[1], len);
						rSeq->title[len] = '\0';
						break;

			case 'A':	rSeq->acc = (char *)emalloc((len+1),
							"read_seq_pep:acc");
						strncpy(rSeq->acc, &line[1], len);
						rSeq->acc[len] = '\0';
						break;

			case 'o':	rSeq->organism = (char *)emalloc((len+1),
							"read_seq_pep:organism");
						strncpy(rSeq->organism, &line[1], len);
						rSeq->organism[len] = '\0';
						break;

			case 'i':	rSeq->ref = (char *)emalloc((len+1),
							"read_seq_pep:reference");
						strncpy(rSeq->ref, &line[1], len);
						rSeq->ref[len] = '\0';
						break;

			case 'r': 	strncpy(&rBuf[rlen], &line[1], len);
						rlen += len;
						break;

			case 'S': 	strncpy(&sBuf[slen], &line[1], len);
						slen += len;
						break;

			case 'G': 	strncpy(&uBuf[ulen], &line[1], len);
						ulen += len;
						break;

			case 'P':	break;
			case 'K':	break;
			case 'T':	break;

			case '\n':	quitFlag = TRUE; break;
			default:
				sprintf(sVal, "Cannot parse line in sequence file: '%s'", line);
				show_err(FALSE, "read_seq_pep", sVal);
				return(FALSE);
		}
	}

/* if no secondary structure info entered */

	if (slen == 0) {
		rSeq->sFlag = 0;
		slen = rlen;
		for (i=0; i<rlen; i++)
			sBuf[i] = CH_UNKNOWN;
	}

/* if no grouping info entered */

	if (ulen == 0) {
		rSeq->uFlag = 0;
		ulen = rlen;
		for (i=0; i<rlen; i++)
			uBuf[i] = GROUP_NOTHING;
	}

/* make sure things line up */

	if ((rlen != slen) && (rlen != ulen)) {
		sprintf(sVal, "Residues/Structure/Grouping align problems: %d %d %d", 
			rlen, slen, ulen);
		show_err(FALSE, "read_seq_pep", sVal);
		return(FALSE);
	}

/* save the information */

	rSeq->pRes = (char *)emalloc((rlen+1), "read_seq_pep:pRes");
	rSeq->sRes = (char *)emalloc((rlen+1), "read_seq_pep:sRes");
	rSeq->uBuf = (char *)emalloc((rlen+1), "read_seq_pep:uRes");

	rBuf[rlen] = '\0';
	sBuf[rlen] = '\0';
	uBuf[rlen] = '\0';

	to_upper(rBuf, rSeq->pRes);
	strncpy(rSeq->sRes, sBuf, rlen);
	strncpy(rSeq->uBuf, uBuf, rlen);

	rSeq->len = rlen;

	return(TRUE);

}	/* read_seq_pep */
