
/*
	file: rseq_pir.c
	read sequences formatted in pir public database 
*/

#include "../include/seqsee.h"

#define ID_KEY_PIR          "ENTRY"
#define TITLE_KEY_PIR       "TITLE"
#define SEQ_KEY_PIR         "SEQUENCE"


/* procedure calls */
extern void show_err();
extern char *emalloc();


/*************************************************************/
/* Reads one protein from PIR (NBRF form) database. 
** Return number of amino acids in the sequence.
*/
int read_seq_pir(fp, rSeq, aaMin, aaMax)
FILE *fp;
SEQ_REC	*rSeq;
int aaMin, aaMax;
{
	int	nd, okFlag, len;
	char ch, *aaBuf;
	char key[MAX_LINE_LEN], line[MAX_LINE_LEN];
	char titleBuf[MAX_LINE_LEN], sVal[MAX_LINE_LEN]; 

	rSeq->len = 0;

/* find accession key */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, ID_KEY_PIR)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

/* read accession */
	nd = fscanf(fp, "%s", line);
	if (nd != 1) {
		sprintf(sVal, "Unable to read accession for sequence\n");
		show_err(FALSE, "read_seq_pir", sVal);
		return(FALSE);
	}

	len = strlen(line);
	rSeq->acc = (char *)emalloc((len + 1), "read_seq_pir: acc");
	strcpy(rSeq->acc, line);
	rSeq->acc[len] = '\0';
	fgets(line, MAX_LINE_LEN-1, fp);

/* find title key */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, TITLE_KEY_PIR)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

/* read title */
	ch = getc(fp);
	len = 0;
	while (ch == ' ') {
		while ((ch == ' ') && (ch != EOF))
			ch = getc(fp);
		if (len > 0)
			titleBuf[len++] = ' ';
		while (ch != '\n') {
			titleBuf[len++] = ch;
			ch = getc(fp);
		}
		ch = getc(fp);
	}
	ungetc(ch, fp);

	rSeq->title = (char *)emalloc((len + 1), "read_seq_pir: title");
	strncpy(rSeq->title, titleBuf, len);
	rSeq->title[len] = '\0';

/* find sequence key */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, SEQ_KEY_PIR)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

	fgets(line, MAX_LINE_LEN-1, fp);
	fgets(line, MAX_LINE_LEN-1, fp);

/* read sequence */

	len = 0;
	okFlag = TRUE;
	aaBuf = (char *)emalloc((aaMax + 1), "read_seq_pir: aaBuf");
	while (TRUE) {

		fscanf(fp, "%s", sVal);
		if (strncmp(sVal, "///", 3) == 0)
			break;

		while (TRUE) {
			ch = getc(fp);
			if (ch == '\n')
				break;
			ch = getc(fp);
			if (ch == '\n')
				break;
			if (len >= aaMax) {
				okFlag = MAX_RES_ERR;
				fgets(line, MAX_LINE_LEN-1, fp);
				break;
			}
			aaBuf[len++] = AA_UC(ch);
			if (AA_CVALID(ch) != TRUE)
				okFlag = BAD_CHAR_ERR;
		}
	}
	if (len < aaMin)
		okFlag = MIN_RES_ERR;

	rSeq->pRes = (char *)emalloc((len + 1), "read_seq_pir: pRes");
	strncpy(rSeq->pRes, aaBuf, len);
	rSeq->pRes[len] = '\0';

	rSeq->len = len;
	rSeq->sFlag = 0;
	rSeq->uFlag = 0;

	rSeq->organism = NULL;
	rSeq->ref = NULL;
	rSeq->uBuf = NULL;
	rSeq->sRes = NULL;
	free(aaBuf);

	return(okFlag);

}	/* read_seq_pir */
