
/*
	reads sequences in the compressed pir database format
	This routine written by Charles Cruden with minor changes
	to fit into peptools.
*/

#include "../include/seqsee.h"
#include "../include/compact.h"
#include <stdlib.h>


static void rec_free (dbentry rec) {
  while (rec.no_refs) {
    rec.no_refs--;
/*    free (rec.refs[rec.no_refs].ref); */
    free (rec.refs[rec.no_refs].residues);
/*    free (rec.refs[rec.no_refs].accession); */
    free (rec.refs[rec.no_refs].journal);
/*    free (rec.refs[rec.no_refs].title); */
  }
  free (rec.refs);
  free (rec.entry);
  free (rec.title);
  free (rec.formal_name);
  free (rec.common_name);
  free (rec.sequence);
}    

static void ennewline (char *str, char *dest) {
  char *p, *q;

  p = str;
  q = dest;
  while (*p != '\n') {
    if (*p != 254)
      *q++ = *p++;
    else {
      p++;
      if (*p == 191)
	*q++ = '\n';
      else if (*p == 254)
	*q++ = 254;
      p++;
    }
  }
  *q = 0;
}


static void enrange (char *str, char *dest) {

  char *p, *q, range[20];
/* SCOTT * unsigned */  
  unsigned short i, j;

  p = str;
  q = dest;
  while (*p) {
    if (*p == ' ') 
    {
      i = (*(p+1))*256 + *(p+2);
      j = (*(p+3))*256 + *(p+4);
      sprintf (range, "%d-%d", i, j);
      strncpy (q, range, strlen(range));
      p += 5;
      q += strlen(range);
    } else
      *q++ = *p++;
  }
  *q = *p;
}


static void decompress (char *str, char *dest) {
  char *p, *q, *s;
  int c=1;


  p = str;
  q = dest;
  while (*p) {
    if (strchr ("@!$#", c)) {
      switch (c) {
      case '@':
	s = rep_words[*p - 1];
	break;
      case '#':
	s = journals[*p - 1];
	break;
      case '!':
	s = journals[*p + 255];
	break;
      case '$':
	s = journals[*p + 511];
	break;
      }
      strncpy (q-1, s, strlen(s));
      q += strlen(s)-1;
      p++;
      c = 1;
   } else if (*p > 128 && *(p-1) != ' ') {
      *q++ = ' ';
      c = *q++ = *p++ - 128;
    } else
      c = *q++ = *p++;
  }
  *q = *p;
}


int read_seq_pir_comp (input, rSeq)
FILE *input;
SEQ_REC *rSeq;
{
  char buf[10000], temp_buf[10000], *p, *q, r, repeat, offset=0;
  int c, comp_len;
  dbentry rec;
  short i, len, no_refs;
  
  while ((c = fgetc (input)) != '\n') {
    if (c == EOF) {
      rec.entry = NULL;
	  return(EOF);
    }
    ungetc (c, input);
    fscanf (input, "%*[^\n]\n");
  }
  if (!fread (buf, 1, 5, input)) {
    rec.entry = NULL;
	return(FALSE);
  }
  rec.class = buf[0];
  rec.length = buf[1]*256 + buf[2];
  comp_len = buf[3]*256 + buf[4];
  rec.sequence = (char *) malloc (rec.length+1);
  if (!fread (buf, comp_len, 1, input)) {
    rec.entry = NULL;
	return(FALSE);
  }

  memset (rec.sequence, 0, rec.length+1);
  q = rec.sequence;
  p = buf;
  repeat=0;
  while (q - rec.sequence < rec.length) {
    r = (*p >> offset) & 0x1F;
    if (offset > 3)
      r |= (*++p << (8-offset)) & 0x1F;
    if (r < 26) 
      if (repeat) {
	repeat++;
	for (; repeat; repeat--)
	  *q++ = 'A' + r;
      } else
	*q++ = 'A' + r;
    else
      repeat = 6*repeat + r - 26;
    offset = (offset+5)%8;
    if (!offset)
      p++;
  }
  if (offset)
    p++;

  fgets (buf, 9999, input);
  ungetc ('\n', input);

  ennewline (buf, temp_buf);
  enrange (temp_buf, buf);
  decompress (buf, temp_buf);

  p = temp_buf;
  sscanf (p, "%[^~]", buf);
  rec.title = strdup(buf);
  p += strlen(buf)+1;
  buf[0] = 0;
  sscanf (p, "%[^~]", buf);
  rec.entry = strdup (buf);
  p += strlen(buf)+1;
  buf[0] = 0;
  sscanf (p, "%[^~]", buf);
  rec.formal_name = strdup (buf);
  p += strlen(buf)+1;
  buf[0] = 0;
  sscanf (p, "%[^~]", buf);
  rec.common_name = strdup (buf);
  p += strlen(buf)+1;
  buf[0] = 0;
  rec.refs = (reference *) malloc (sizeof(reference) * (no_refs = *p++ - 1));
  for (i=0; i<no_refs; i++) {
/*    sscanf (p, "%[^~]", buf);
      rec.refs[i].ref = strdup (buf);
    p += strlen(buf);
    buf[0] = 0;  */
/*    sscanf (p, "%[^~]", buf);
    p += (strlen(buf)+1);
    if (!strcmp (buf, "R")) 
      strcpy (buf, rec.refs[i].ref);
    rec.refs[i].accession = strdup (buf);
    buf[0] = 0; */
    sscanf (p, "%[^~]", buf);
    p += (strlen(buf)+1);
    if (!strcmp (buf, "A")) 
      sprintf (buf, "1-%i", rec.length);
    rec.refs[i].residues = strdup(buf);
    buf[0] = 0;
    sscanf (p, "%[^~]", buf);
    rec.refs[i].journal = strdup (buf);
    p += strlen(buf)+1;
    buf[0] = 0;
/*    sscanf (p, "%[^~]", buf);
    rec.refs[i].title = strdup (buf);
    p += strlen(buf)+1; */
  }
  rec.no_refs = no_refs;
  rec.molecular_weight = atoi (p);

	rSeq->len = rec.length;
	rSeq->pRes = strdup(rec.sequence);
	rSeq->title = strdup(rec.title);
	rSeq->acc = strdup(rec.entry);
	rSeq->organism = NULL;
	rSeq->ref = NULL;
	rSeq->uBuf = NULL;
	rSeq->sRes = NULL;
	rec_free(rec);
  	return(TRUE);
}
