
/*
	file: rseq_sprot.c
	common routines for reading sequences in swiss-prot format
*/

#include "../include/seqsee.h"

#define ID_KEY_SP           "ID"
#define TITLE_KEY_SP        "DE"
#define SEQ_KEY_SP          "SQ"
#define END_SEQ             '/'


/* procedure calls */
extern void show_err();
extern char *emalloc();


/*************************************************************/
/* Reads one protein from SWISS-PROT (EMBL form) database. 
** Return number of amino acids in the sequence.
*/
int read_seq_sprot(fp, rSeq)
FILE *fp;
SEQ_REC	*rSeq;
{

	int	nd, okFlag, len, ch;
	char *rc, aaBuf[MAX_RES];
	char key[MAX_LINE_LEN]; 
	char sVal[MAX_LINE_LEN], line[MAX_LINE_LEN]; 

	rSeq->len = 0;

	sprintf(sVal, "Not supporting swiss-prot at this time\n");
	show_err(TRUE, "read_seq_sprot", sVal);

/* find ID code */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, ID_KEY_SP)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

	rc = fgets(line, MAX_LINE_LEN-1, fp);
	if (rc == NULL) {
		sprintf(sVal, "Unable to read accession for sequence\n");
		show_err(FALSE, "read_seq_sprot", sVal);
		return(FALSE);
	}
	strncpy(rSeq->acc, line, 6);
	rSeq->acc[6] = '\0';

/* find title */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, TITLE_KEY_SP)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

	ch = getc(fp);
	while ((ch == ' ') && (ch != EOF)) {
		ch = getc(fp);
	}
	ungetc(ch, fp);

	rc = fgets(line, MAX_LINE_LEN-1, fp);
	if (rc == NULL) {
		sprintf(sVal, "Unable to read title for sequence %s\n", rSeq->acc);
		show_err(FALSE, "read_seq_sprot", sVal);
		return(FALSE);
	}
	rSeq->title = (char *)emalloc(strlen(line), "read_seq_sprot");
	strcpy(rSeq->title, line);

/* find sequence key */
	nd = fscanf(fp, "%s", key);
	while (0 != strcmp(key, SEQ_KEY_SP)) {  
		if (nd == EOF)
			return(EOF);
		fgets(line, MAX_LINE_LEN-1, fp);
		nd = fscanf(fp, "%s", key);
	}

	fgets(line, MAX_LINE_LEN-1, fp);

/* read sequence */
	okFlag = TRUE;
	len = 0;
	ch = getc(fp);
	while ((ch != END_SEQ) && (ch != EOF)) {

		ch = AA_UC(ch);		/* convert to upper case */

		if (AA_CVALID(ch) == TRUE) {
			if (len < MAX_RES) {
				aaBuf[len++] = ch;
			}
			else {
				okFlag = MAX_RES_ERR;
				break;
			}
		}
		else {
			if ((ch != ' ') && (ch != '\n'))
				okFlag = BAD_CHAR_ERR;
		}

		ch = getc(fp);
	}

	rSeq->pRes = (char *)emalloc((len + 1), "read_seq_sprot: pRes");
	strncpy(rSeq->pRes, aaBuf, len);
	rSeq->pRes[len] = '\0';

	if (len < MIN_RES)
		okFlag = MIN_RES_ERR;

	rSeq->len = len;
	return(okFlag);

}	/* read_seq_sprot */
