
/*
	file: seqbank.c
    reads the SEQBANK database	
    20 may 1992
*/

#include "../include/seqsee.h"
#include "../include/seqbank.h"

extern char *emalloc();
extern void show_msg(), show_err();
static int gettype(), read_sb_db_line();


/*************************************************************/
/* Reads in SEQBANK database of proteins
*/
int read_seq_sb(fp, rSb)
FILE	*fp;
SB_REC	*rSb;
{

	register int ch;
	int	pCount, sCount, okFlag;
	int type, refCount;
	char sid[MAX_LINE_LEN], line[MAX_LINE_LEN];
	char pBuf[MAX_RES], sBuf[MAX_RES];
	char sVal[MAX_LINE_LEN];

	ch = getc(fp);
	while ((ch != EOF) && (ch != SB_SCH)) {
		ch = getc(fp);
	}

	if (ch == EOF) {
		rSb->len = 0;
		return(EOF);
	}

/* read title */
	fgets(rSb->title, MAX_TITLE_LEN-1, fp);

	type = 1;
	refCount = 0;

/* read in fields up until residue data */
    while (type != NONE) {
       	fgets(line, MAX_LINE_LEN-1, fp);
		type = gettype(line);
		switch (type) {
			case REF_FLD: strcpy(rSb->ref_id[refCount], &line[12]);
							refCount++; break;
			case SEQ_FLD: sscanf(line, "%*s%*s%s", rSb->seq_id); break;
			case BRK_FLD: sscanf(line, "%*s%*s%s", rSb->brk_id); break; 
			case PIR_FLD: sscanf(line, "%*s%*s%s", rSb->pir_id); break;
			case SWI_FLD: sscanf(line, "%*s%*s%s", rSb->swi_id); break;
			case RES_FLD: sscanf(line, "%*s%s", rSb->res); break;
			case FAC_FLD: sscanf(line, "%*s%*s%*s%s", rSb->r_fact); break;
			case OLD_FLD: sscanf(line, "%*s%*s%s", sid);
				if (strcmp(sid, B_CFORM) == 0)       
					rSb->fold = B_IFORM;
				else if (strcmp(sid, M_CFORM) == 0)  
					rSb->fold = M_IFORM;
				else if (strcmp(sid, A_CFORM) == 0)  
					rSb->fold = A_IFORM;
				else if (strcmp(sid, AB_CFORM) == 0) 
					rSb->fold = AB_IFORM;
				else if (strcmp(sid, CA_CFORM) == 0) 
					rSb->fold = CA_IFORM;
				else if (strcmp(sid, CB_CFORM) == 0) 
					rSb->fold = CB_IFORM;
				else if (strcmp(sid, CM_CFORM) == 0) 
					rSb->fold = CM_IFORM;
				else {
					sprintf(sVal, "\n Protein: %s", rSb->title);
					show_msg(sVal);
					sprintf(sVal, "Error, illegal form in SEQBANK.\n\n");
					show_err(TRUE, "sb.lib.c", sVal);
				}
				break;
			case NUM_FLD: sscanf(line, "%*s%*s%d", &rSb->len); break;
			case NONE: 
				if (line[0] == '#') {
					sprintf(sVal, "\n Protein: %s", rSb->title);
					show_msg(sVal);
					sprintf(sVal, "Error, illegal field in SEQBANK.\n\n");
					show_err(TRUE, "sb.lib.c", sVal);
				}
				/* else line is a residue */
				break;
		}
	}

/* read in residue data */
	okFlag = TRUE;
	pCount = 0;
	sCount = 0;
	while (okFlag == TRUE) {
		okFlag = read_sb_db_line(fp, pBuf, &pCount);
		okFlag = read_sb_db_line(fp, sBuf, &sCount);
	}

	if ((pCount != sCount) || (pCount != rSb->len)) {
		sprintf(sVal, "\n Protein: %s:\nLength inconsistency.", rSb->title);
		show_err(TRUE, "sb.lib.c", sVal);
	}

	rSb->pRes = (char *)emalloc((pCount + 1), "read_seq_sb: pRes");
	rSb->sRes = (char *)emalloc((pCount + 1), "read_seq_sb: sRes");
	strncpy(rSb->pRes, pBuf, pCount);
	strncpy(rSb->sRes, sBuf, pCount);
	rSb->pRes[pCount] = '\0';
	rSb->sRes[pCount] = '\0';

	return(TRUE);

}	/* read_seq_sb */


/*************************************************************/
/* Reads one line of residues from SEQBANK file
*/
static int read_sb_db_line(dbFile, sequence, ix)
FILE	*dbFile;
char	*sequence;
register int	*ix;
{
	register int ch;

	do {
		ch = getc(dbFile);
		if (ch == EOF)
			return(EOF);
		if (ch == SB_SCH) {
			ungetc(ch, dbFile);
			return(FALSE);
		}
	} while (AA_CVALID(ch) == FALSE);

	do {
		if (AA_CVALID(ch) == TRUE) {
			if ((*ix) >= MAX_RES)
				show_err(TRUE, "read_sb_db_line", "MAX_RES is not big enough.");
			else
				sequence[(*ix)++] = ch;
		}
		else if (ch == EOF)
			return(EOF);
		else if (ch == SB_SCH) {
			ungetc(ch, dbFile);
			return(FALSE);
		}
		ch = getc(dbFile);
	} while (ch != '\n');

	return(TRUE);

}	/* read_sb_db_line */


/*************************************************************/
/* given an input line, to determine which type of field is read. 
*/
static int gettype(line)
char *line;
{
 	if (strncmp(line, SB_REF, FIELD_LEN) == 0)
		return ( REF_FLD );
 	else if (strncmp(line, SB_SEQ, FIELD_LEN) == 0)
		return ( SEQ_FLD );
 	else if (strncmp(line, SB_BRK, FIELD_LEN) == 0)
		return ( BRK_FLD );
 	else if (strncmp(line, SB_PIR, FIELD_LEN) == 0)
		return ( PIR_FLD );
 	else if (strncmp(line, SB_SWI, FIELD_LEN) == 0)
		return ( SWI_FLD );
 	else if (strncmp(line, SB_RES, FIELD_LEN) == 0)
		return ( RES_FLD );
 	else if (strncmp(line, SB_FAC, FIELD_LEN) == 0)
		return ( FAC_FLD );
 	else if (strncmp(line, SB_OLD, FIELD_LEN) == 0) 
		return ( OLD_FLD );
	else if (strncmp(line, SB_NUM, FIELD_LEN) == 0)
		return ( NUM_FLD );
	else return ( NONE );

}	/* gettype */
