
/*
	file: show.c
	routines for printing messages to the screen
*/

#include "../include/seqsee.h" 

extern void exit();


/*************************************************************/
/* generic error handler 
*/
void show_err(dieFlag, sModule, msg)
int dieFlag;
char *sModule, *msg;
{

	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");
	if (dieFlag == TRUE)
		fprintf(stdout, " Fatal Error Detected!\n");
	else
		fprintf(stdout, " Warning, Error Detected!\n");
	fprintf(stdout, "\n");
	fprintf(stdout, " Module.: %s\n", sModule);
	fprintf(stdout, " Message: %s\n", msg);
	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");

	if (dieFlag == TRUE)
		exit(1);

}	/* show_err */


/*************************************************************/
/* prints out of memory error
*/
void show_err_mem(msg)		
char *msg;
{

	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");
	fprintf(stdout, " Fatal Error: Out of memory...aborting\n");
	fprintf(stdout, " Module: %s\n", msg);
	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");

}	/* show_err_mem */


/*************************************************************/
/* write a message to the screen. 
*/
void show_msg(fp, msg)
FILE *fp;
char *msg;
{

	fprintf(fp, msg);

}	/* show_msg */


/*************************************************************/
/* put a message and expect a user response 
*/
void show_prompt(fp, msg)
FILE *fp;
char *msg;
{

	fprintf(fp, "%s%s", msg, PROG_PROMPT);

}	/* show_prompt */


/*************************************************************/
/* prompt to enter one amino acid sequence 
*/
void show_seq_file(fp)
FILE *fp;
{

	fprintf(fp, "\n");
	fprintf(fp, "\n Your amino acid sequence is now required:\n");
	fprintf(fp, "\n    %d) Read sequence using IG format.",
		SEQ_RBO);
	fprintf(fp, "\n    %d) Read sequence using PIR format.",
		SEQ_PIR);
	fprintf(fp, "\n    %d) Read sequence using Swiss-prot format.",
		SEQ_SPROT);
	fprintf(fp, "\n    %d) Read sequence using Peptool format.",
		SEQ_PEP);
	fprintf(fp, "\n    %d) Read sequence using PIR compressed format.",
		SEQ_PIRCOMP);
	fprintf(fp, "\n    %d) Sequence entered from the keyboard.", SEQ_KBD);
	fprintf(fp, "\n    7) I am not ready, exit the program.");
	fprintf(fp, "\n");
	fprintf(fp, "\n Enter a number (then press return).\n ");
	fprintf(fp, PROG_PROMPT);

}	/* show_seq_file */


/*************************************************************/
/* prompt to enter one amino acid sequence 
*/
void show_seq_file1(fp)
FILE *fp;
{

	fprintf(fp, "\n");
	fprintf(fp, "\n Your first amino acid sequence is now required:\n");
	fprintf(fp, "\n    %d) Read sequence from input file (IG format).",
		SEQ_RBO);
	fprintf(fp, "\n    %d) Read sequence from input file (PIR format).",
		SEQ_PIR);
	fprintf(fp, "\n    %d) Read sequence from input file (Swiss-prot format).",
		SEQ_SPROT);
	fprintf(fp, "\n    %d) Read sequence from input file (Peptool format).",
		SEQ_PEP);
	fprintf(fp, "\n    %d) Sequence entered from the keyboard.", SEQ_KBD);
	fprintf(fp, "\n    6) I am not ready, exit the program.");
	fprintf(fp, "\n");
	fprintf(fp, "\n Enter a number (then press return).\n ");
	fprintf(fp, PROG_PROMPT);

}	/* show_seq_file1 */


/*************************************************************/
/* prompt to enter one amino acid sequence 
*/
void show_seq_file2(fp)
FILE *fp;
{

	fprintf(fp, "\n");
	fprintf(fp, "\n Your second amino acid sequence is now required:\n");
	fprintf(fp, "\n    %d) Read sequence from input file (IG format).",
		SEQ_RBO);
	fprintf(fp, "\n    %d) Read sequence from input file (PIR format).",
		SEQ_PIR);
	fprintf(fp, "\n    %d) Read sequence from input file (Swiss-prot format).",
		SEQ_SPROT);
	fprintf(fp, "\n    %d) Read sequence from input file (Peptool format).",
		SEQ_PEP);
	fprintf(fp, "\n    %d) Sequence entered from the keyboard.", SEQ_KBD);
	fprintf(fp, "\n    6) I am not ready, exit the program.");
	fprintf(fp, "\n");
	fprintf(fp, "\n Enter a number (then press return).\n ");
	fprintf(fp, PROG_PROMPT);

}	/* show_seq_file2 */


/*************************************************************/
/* ask user how many alignments/searches/whatever to keep track of. 
*/
void show_top_num(fp, fStr, topMin, topMax)
FILE *fp;
char *fStr;
int topMin, topMax;
{
	char sVal[MAX_LINE_LEN];

	sprintf(sVal, "\n This program keeps track of the top 'x' %s.", fStr);
	fprintf(fp, sVal);

	sprintf(sVal, "\n Enter a value for 'x' where %d < x < %3d.\n",
		topMin, topMax);
	fprintf(fp, sVal);
	fprintf(fp, PROG_PROMPT);

}	/* show_top_num */


/*************************************************************/
/* select the appropriate database 
*/
void show_dbase_select(fp)
FILE *fp;
{

	fprintf(fp, "\n");
	fprintf(fp, "\n Which database do you wish to select?\n");
	fprintf(fp, "\n    1) PIR/SWISS-PROT");
	fprintf(fp, "\n    2) SEQBANK");
	fprintf(fp, "\n    0) Exit");
	fprintf(fp, "\n");
	fprintf(fp, "\n Enter a number (then press return).\n ");
	fprintf(fp, PROG_PROMPT);

}	/* show_dbase_select */


/*************************************************************/
/* prompt to enter files of amino acid sequences 
*/
void show_mseq_files(fp)
FILE *fp;
{

	fprintf(fp, "\n");
	fprintf(fp, "\n Which option do you wish to choose?\n");
	fprintf(fp, "\n    %d) I wish to enter my sequences (IG format).",
		SEQ_RBO);
	fprintf(fp, "\n    %d) I wish to enter my sequences (PIR format).",
		SEQ_PIR);
	fprintf(fp, "\n    %d) I wish to enter my sequences (Swissprot format).",
		SEQ_SPROT);
	fprintf(fp, "\n    %d) I wish to enter my sequences (Peptool format).",
		SEQ_PEP);
	fprintf(fp, "\n    5) I am done entering all my sequences.");
	fprintf(fp, "\n    6) Exit this program.");
	fprintf(fp, "\n");
	fprintf(fp, "\n");
	fprintf(fp, "\n Enter a number (then press return).\n ");
	fprintf(fp, PROG_PROMPT);

}	/* show_mseq_files */
