
/*
	file...: main.c
	purpose: main program for xalign 

	author.: Robert Boyko
	date...: Nov 30, 1995	
	place..: University of Alberta / Department of Biochemistry
*/

#include "include/seqsee.h"
#include "include/xalign.h"
#include "driver.h"
#include <stdlib.h>

/* static vars */
static char parmsFname[MAX_FNAME_LEN];
static char resultsFname[MAX_FNAME_LEN];
static char dataFname[MAX_FNAME_LEN];
static int printFormat; /* set TRUE to print peptool format output */

/* procedure calls */
extern void xalign_main(), get_output_file();
extern FILE *open_data_file(), *check_parms_file();
extern int read_seq_select(), get_xalign_format();
static void show_version(), get_args(), setup_parms_file();


/*************************************************************/
/* main program
*/
int main(argc, argv)
int argc;
char *argv[];
{
	int select, format;
	FILE *datafp;

	show_version(stdout);
	get_args(argc, argv);
	check_parms_file(parmsFname);

	format = get_xalign_format();

	if (format != 0) {
   		datafp = open_data_file(dataFname, "xalign sequence datafile");
		//select = read_seq_select(datafp, format);
                select = 1;
		get_output_file(resultsFname);

		setup_parms_file(parmsFname, NEW_PARMS, format, dataFname, select);
		xalign_main(NEW_PARMS, resultsFname);
	}

	exit(0);

}	/* main */


/*************************************************************/
/* read command line arguments 
*/
static void get_args(argc, argv)
int argc;
char *argv[];
{
	int i;

	strcpy(parmsFname, DEFAULT_PARMS);
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], PARMS_FLAG) == 0) {
			strcpy(parmsFname, argv[i+1]);
			break;
		}
	}

	printFormat = FALSE;
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], PRINT_FORMAT_FLAG) == 0) {
			printFormat = TRUE;
			break;
		}
	}

	strcpy(resultsFname, "");
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], RESULTS_FLAG) == 0) {
			strcpy(resultsFname, argv[i+1]);
			break;
		}
	}

	strcpy(dataFname, "");
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], DATA_FLAG) == 0) {
			strcpy(dataFname, argv[i+1]);
			break;
		}
	}

}	/* get_args */


/*************************************************************/
/* current version of the program 
*/
static void show_version(fp)
FILE *fp;
{

	fprintf(fp, "\n\n");
	fprintf(fp, "**************************************************\n");
	fprintf(fp, "\n");
	fprintf(fp, " Program.: xalign_x - Version %s (c)\n", VERSION);
	fprintf(fp, "\n");
	fprintf(fp, " Authors.: Robert Boyko\n");
	fprintf(fp, "           David Wishart\n");
	fprintf(fp, "           Brian Sykes\n");
	fprintf(fp, " Location: University of Alberta\n");
	fprintf(fp, "           Department of Biochemistry\n");
	fprintf(fp, "\n");
	fprintf(fp, " Funding for this project was provided by the\n");
	fprintf(fp, " Protein Engineering Network Centres of Excellence\n");
	fprintf(fp, " and the Medical Research Council of Canada.\n");
	fprintf(fp, "\n");
	fprintf(fp, "**************************************************\n");
	fprintf(fp,  "\n");

}	/* show_version */


/*************************************************************/
/* combine input data and old parms file into new parms format 
*/
static void setup_parms_file(parms, newParms, format, dataFile, select)
char *parms, *newParms, *dataFile;
int format, select;
{
	int ch;
	FILE *outfp, *parmsfp;

	parmsfp = fopen(parms, "r");
	outfp = fopen(newParms, "w");

	while ((ch = fgetc(parmsfp)) != EOF)
		fputc(ch, outfp);

	/* if you comment out these parms, you will simulate running 
	   gui version of the program */

	fprintf(outfp, "\n");
	fprintf(outfp, "%s %d\n", KEY_ASCII_FLAG, PARM_ASCII_FLAG);
	fprintf(outfp, "%s %d\n", KEY_RES_LINE_LEN, PARM_RES_LINE_LEN);
	fprintf(outfp, "%s %d\n", KEY_PRINT_FORMAT, printFormat);
	/*
	fprintf(outfp, "%s %d\n", KEY_PRINT_PAIRWISE, PARM_PRINT_PAIRWISE);
	*/

	/* these parms are commented out but having them here
	   makes it alot easier to run tests of the gui xalign */

	fprintf(outfp, "%s %d\n", KEY_DATA_FORMAT, format);
	fprintf(outfp, "\n");
	fprintf(outfp, "%s %s\n", KEY_DATA_FILE, dataFile);
	fprintf(outfp, "%s %d\n", KEY_SEQ_SELECT, select);

	fclose(parmsfp);
	fclose(outfp);

}	/* setup_parms_file */
